@@ -66,6 +66,13 @@
 #include <VBox/com/defs.h>
 #include <VBox/com/assert.h>
 
+#define LOGREF(prefix, pObj, cRefs) com::LogRef("%s {%p} cRefs=%d\n", (prefix), (pObj), (cRefs))
+
+namespace com
+{
+    void LogRef(const char *pcszFormat, ...);
+}
+
 /**
  *  Strong referencing operators. Used as a second argument to ComPtr<>/ComObjPtr<>.
  */
@@ -74,8 +81,14 @@ class ComStrongRef
 {
 protected:
 
-    static void addref (C *p) { p->AddRef(); }
-    static void release (C *p) { p->Release(); }
+    static void addref(C *p)
+    {
+        p->AddRef();
+    }
+    static void release(C *p)
+    {
+        p->Release();
+    }
 };
 
 /**
@@ -86,8 +99,8 @@ class ComWeakRef
 {
 protected:
 
-    static void addref  (C * /* p */) {}
-    static void release (C * /* p */) {}
+    static void addref(C * /* p */) {}
+    static void release(C * /* p */) {}
 };
 
 /**
@@ -104,14 +117,14 @@ protected:
  *              IUnknown).
  */
 template <class I1, class I2>
-inline bool ComPtrEquals (I1 *aThis, I2 *aThat)
+inline bool ComPtrEquals(I1 *aThis, I2 *aThat)
 {
     IUnknown *thatUnk = NULL, *thisUnk = NULL;
     if (aThat)
-        aThat->QueryInterface (COM_IIDOF (IUnknown), (void **) &thatUnk);
+        aThat->QueryInterface(COM_IIDOF(IUnknown), (void**)&thatUnk);
     if (aThis)
-        aThis->QueryInterface (COM_IIDOF (IUnknown), (void **) &thisUnk);
-    bool equal = thisUnk == thatUnk;
+        aThis->QueryInterface(COM_IIDOF(IUnknown), (void**)&thisUnk);
+    bool equal = (thisUnk == thatUnk);
     if (thisUnk)
         thisUnk->Release();
     if (thatUnk)
@@ -121,12 +134,12 @@ inline bool ComPtrEquals (I1 *aThis, I2 *aThat)
 
 /* specialization for <Any, IUnknown> */
 template <class I1>
-inline bool ComPtrEquals (I1 *aThis, IUnknown *aThat)
+inline bool ComPtrEquals(I1 *aThis, IUnknown *aThat)
 {
     IUnknown *thisUnk = NULL;
     if (aThis)
-        aThis->QueryInterface (COM_IIDOF (IUnknown), (void **) &thisUnk);
-    bool equal = thisUnk == aThat;
+        aThis->QueryInterface(COM_IIDOF(IUnknown), (void**)&thisUnk);
+    bool equal = (thisUnk == aThat);
     if (thisUnk)
         thisUnk->Release();
     return equal;
@@ -134,12 +147,12 @@ inline bool ComPtrEquals (I1 *aThis, IUnknown *aThat)
 
 /** Specialization for <IUnknown, Any> */
 template <class I2>
-inline bool ComPtrEquals (IUnknown *aThis, I2 *aThat)
+inline bool ComPtrEquals(IUnknown *aThis, I2 *aThat)
 {
     IUnknown *thatUnk = NULL;
     if (aThat)
-        aThat->QueryInterface (COM_IIDOF (IUnknown), (void **) &thatUnk);
-    bool equal = aThis == thatUnk;
+        aThat->QueryInterface(COM_IIDOF(IUnknown), (void**)&thatUnk);
+    bool equal = (aThis == thatUnk);
     if (thatUnk)
         thatUnk->Release();
     return equal;
@@ -147,7 +160,7 @@ inline bool ComPtrEquals (IUnknown *aThis, I2 *aThat)
 
 /* specialization for IUnknown */
 template<>
-inline bool ComPtrEquals <IUnknown, IUnknown> (IUnknown *aThis, IUnknown *aThat)
+inline bool ComPtrEquals<IUnknown, IUnknown>(IUnknown *aThis, IUnknown *aThat)
 {
     return aThis == aThat;
 }
