@@ -1,5 +1,5 @@
 /** @file
- * HWACCM - VMX Structures and Definitions.
+ * HWACCM - VMX Structures and Definitions. (VMM)
  */
 
 /*
@@ -377,10 +377,43 @@ typedef enum
     /** 32bit hackishness. */
     VMX_FLUSH_32BIT_HACK                        = 0x7fffffff
 } VMX_FLUSH;
+/** @} */
+
+/** @name MSR load/store elements
+ * @{
+ */
+#pragma pack(1)
+typedef struct
+{
+    uint32_t    u32IndexMSR;
+    uint32_t    u32Reserved;
+    uint64_t    u64Value;
+} VMXMSR;
+#pragma pack()
+/** Pointer to an MSR load/store element. */
+typedef VMXMSR *PVMXMSR;
+/** Pointer to a const MSR load/store element. */
+typedef const VMXMSR *PCVMXMSR;
 
 /** @} */
 
 
+/** @name VT-x capability qword
+ * @{
+ */
+#pragma pack(1)
+typedef union
+{
+    struct
+    {
+        uint32_t        disallowed0;
+        uint32_t        allowed1;
+    } n;
+    uint64_t            u;
+} VMX_CAPABILITY;
+#pragma pack()
+/** @} */
+
 /** @name VMX Basic Exit Reasons.
  * @{
  */
@@ -393,6 +426,8 @@ typedef enum
 /** @name VMX Basic Exit Reasons.
  * @{
  */
+/** -1 Invalid exit code */
+#define VMX_EXIT_INVALID            -1
 /** 0 Exception or non-maskable interrupt (NMI). */
 #define VMX_EXIT_EXCEPTION          0
 /** 1 External interrupt. */
@@ -842,12 +877,18 @@ typedef enum
 #define VMX_VMCS_CTRL_PROC_EXEC2_EPT                            RT_BIT(1)
 /** Descriptor table instructions cause VM-exits. */
 #define VMX_VMCS_CTRL_PROC_EXEC2_DESCRIPTOR_INSTR_EXIT          RT_BIT(2)
+/** RDTSCP causes a VM-exit. */
+#define VMX_VMCS_CTRL_PROC_EXEC2_RDTSCP_EXIT                    RT_BIT(3)
 /** Virtualize x2APIC mode. */
 #define VMX_VMCS_CTRL_PROC_EXEC2_X2APIC                         RT_BIT(4)
 /** VPID supported/enabled. */
 #define VMX_VMCS_CTRL_PROC_EXEC2_VPID                           RT_BIT(5)
 /** VM Exit when executing the WBINVD instruction. */
 #define VMX_VMCS_CTRL_PROC_EXEC2_WBINVD_EXIT                    RT_BIT(6)
+/** Unrestricted guest execution. */
+#define VMX_VMCS_CTRL_PROC_EXEC2_REAL_MODE                      RT_BIT(7)
+/** A specified nr of pause loops cause a VM-exit. */
+#define VMX_VMCS_CTRL_PROC_EXEC2_PAUSE_LOOP_EXIT                RT_BIT(10)
 /** @} */
 
 
@@ -1072,6 +1113,22 @@ typedef enum
 #define VMX_EXIT_QUALIFICATION_CRX_ACCESS_LMSW         3
 /** @} */
 
+/** @name VMX_EXIT_QUALIFICATION_TASK_SWITCH
+ * @{
+ */
+#define VMX_EXIT_QUALIFICATION_TASK_SWITCH_SELECTOR(a)  (a & 0xffff)
+#define VMX_EXIT_QUALIFICATION_TASK_SWITCH_TYPE(a)      ((a >> 30)& 0x3)
+/** Task switch caused by a call instruction. */
+#define VMX_EXIT_QUALIFICATION_TASK_SWITCH_TYPE_CALL    0
+/** Task switch caused by an iret instruction. */
+#define VMX_EXIT_QUALIFICATION_TASK_SWITCH_TYPE_IRET    1
+/** Task switch caused by a jmp instruction. */
+#define VMX_EXIT_QUALIFICATION_TASK_SWITCH_TYPE_JMP     2
+/** Task switch caused by an interrupt gate. */
+#define VMX_EXIT_QUALIFICATION_TASK_SWITCH_TYPE_IDT     3
+
+/** @} */
+
 
 /** @name VMX_EXIT_EPT_VIOLATION
  * @{
@@ -1131,6 +1188,34 @@ typedef enum
 #define VMX_EXIT_QUALIFICATION_IO_ENCODING_IMM         1
 /** @} */
 
+/** @name VMX_EXIT_APIC_ACCESS
+ * @{
+ */
+/** 0-11:   If the APIC-access VM exit is due to a linear access, the offset of access within the APIC page. */
+#define VMX_EXIT_QUALIFICATION_APIC_ACCESS_OFFSET(a)    (a & 0xfff)
+/** 12-15:  Access type. */
+#define VMX_EXIT_QUALIFICATION_APIC_ACCESS_TYPE(a)      ((a >> 12) & 0xf)
+/* Rest reserved. */
+/** @} */
+
+
+/** @name VMX_EXIT_QUALIFICATION_APIC_ACCESS_TYPE; access types
+ * @{
+ */
+/** Linear read access. */
+#define VMX_APIC_ACCESS_TYPE_LINEAR_READ                0
+/** Linear write access. */
+#define VMX_APIC_ACCESS_TYPE_LINEAR_WRITE               1
+/** Linear instruction fetch access. */
+#define VMX_APIC_ACCESS_TYPE_LINEAR_INSTR_FETCH         2
+/** Linear read/write access during event delivery. */
+#define VMX_APIC_ACCESS_TYPE_LINEAR_EVENT_DELIVERY      3
+/** Physical read/write access during event delivery. */
+#define VMX_APIC_ACCESS_TYPE_PHYSICAL_EVENT_DELIVERY    10
+/** Physical access for an instruction fetch or during instruction execution. */
+#define VMX_APIC_ACCESS_TYPE_PHYSICAL_INSTR             15
+/** @} */
+
 /** @} */
 
 /** @name VMCS field encoding - Natural width guest state fields
