@@ -1,5 +1,5 @@
 /** @file
- * MM - The Memory Manager.
+ * MM - The Memory Manager. (VMM)
  */
 
 /*
@@ -37,91 +37,12 @@
 #include <VBox/sup.h>
 
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 /** @defgroup grp_mm       The Memory Manager API
  * @{
  */
 
-/** @name RAM Page Flags
- * Since internal ranges have a byte granularity it's possible for a
- * page be flagged for several uses. The access virtualization in PGM
- * will choose the most restricted one and use EM to emulate access to
- * the less restricted areas of the page.
- *
- * Bits 0-11 only since they are fitted into the offset part of a physical memory address.
- * @{
- */
-#if 1
-/** Reserved - Not RAM, ROM nor MMIO2.
- * If this bit is cleared the memory is assumed to be some kind of RAM.
- * Normal MMIO may set it but that depends on whether the RAM range was
- * created specially for the MMIO or not.
- *
- * @remarks The current implementation will always reserve backing
- *          memory for reserved  ranges to simplify things.
- */
-#define MM_RAM_FLAGS_RESERVED           RT_BIT(0)
-/** ROM - Read Only Memory.
- * The page have a HC physical address which contains the BIOS code. All write
- * access is trapped and ignored.
- *
- * HACK: Writable shadow ROM is indicated by both ROM and MMIO2 being
- *       set. (We're out of bits.)
- */
-#define MM_RAM_FLAGS_ROM                RT_BIT(1)
-/** MMIO - Memory Mapped I/O.
- * All access is trapped and emulated. No physical backing is required, but
- * might for various reasons be present.
- */
-#define MM_RAM_FLAGS_MMIO               RT_BIT(2)
-/** MMIO2 - Memory Mapped I/O, variation 2.
- * The virtualization is performed using real memory and only catching
- * a few accesses for like keeping track for dirty pages.
- * @remark Involved in the shadow ROM hack.
- */
-#define MM_RAM_FLAGS_MMIO2              RT_BIT(3)
-#endif
-
-#ifndef VBOX_WITH_NEW_PHYS_CODE
-/** Physical backing memory is allocated dynamically. Not set implies a one time static allocation. */
-#define MM_RAM_FLAGS_DYNAMIC_ALLOC      RT_BIT(11)
-#endif /* !VBOX_WITH_NEW_PHYS_CODE */
-
-/** The shift used to get the reference count. */
-#define MM_RAM_FLAGS_CREFS_SHIFT        62
-/** The mask applied to the the page pool idx after using MM_RAM_FLAGS_CREFS_SHIFT to shift it down. */
-#define MM_RAM_FLAGS_CREFS_MASK         0x3
-/** The (shifted) cRef value used to indiciate that the idx is the head of a
- * physical cross reference extent list. */
-#define MM_RAM_FLAGS_CREFS_PHYSEXT      MM_RAM_FLAGS_CREFS_MASK
-/** The shift used to get the page pool idx. (Apply MM_RAM_FLAGS_IDX_MASK to the result when shifting down). */
-#define MM_RAM_FLAGS_IDX_SHIFT          48
-/** The mask applied to the the page pool idx after using MM_RAM_FLAGS_IDX_SHIFT to shift it down. */
-#define MM_RAM_FLAGS_IDX_MASK           0x3fff
-/** The idx value when we're out of of extents or there are simply too many mappings of this page. */
-#define MM_RAM_FLAGS_IDX_OVERFLOWED     MM_RAM_FLAGS_IDX_MASK
-
-/** Mask for masking off any references to the page. */
-#define MM_RAM_FLAGS_NO_REFS_MASK       UINT64_C(0x0000ffffffffffff)
-/** @} */
-
-#ifndef VBOX_WITH_NEW_PHYS_CODE
-/** @name MMR3PhysRegisterEx registration type
- * @{
- */
-typedef enum
-{
-    /** Normal physical region (flags specify exact page type) */
-    MM_PHYS_TYPE_NORMAL               = 0,
-    /** Allocate part of a dynamically allocated physical region */
-    MM_PHYS_TYPE_DYNALLOC_CHUNK,
-
-    MM_PHYS_TYPE_32BIT_HACK = 0x7fffffff
-} MMPHYSREG;
-/** @} */
-#endif
-
 /**
  * Memory Allocation Tags.
  * For use with MMHyperAlloc(), MMR3HeapAlloc(), MMR3HeapAllocEx(),
@@ -144,6 +65,7 @@ typedef enum MMTAG
     MM_TAG_CPUM_CTX,
 
     MM_TAG_DBGF,
+    MM_TAG_DBGF_AS,
     MM_TAG_DBGF_INFO,
     MM_TAG_DBGF_LINE,
     MM_TAG_DBGF_LINE_DUP,
@@ -172,10 +94,13 @@ typedef enum MMTAG
     MM_TAG_PDM,
     MM_TAG_PDM_ASYNC_COMPLETION,
     MM_TAG_PDM_DEVICE,
+    MM_TAG_PDM_DEVICE_DESC,
     MM_TAG_PDM_DEVICE_USER,
     MM_TAG_PDM_DRIVER,
+    MM_TAG_PDM_DRIVER_DESC,
     MM_TAG_PDM_DRIVER_USER,
     MM_TAG_PDM_USB,
+    MM_TAG_PDM_USB_DESC,
     MM_TAG_PDM_USB_USER,
     MM_TAG_PDM_LUN,
     MM_TAG_PDM_QUEUE,
@@ -184,6 +109,7 @@ typedef enum MMTAG
     MM_TAG_PGM,
     MM_TAG_PGM_CHUNK_MAPPING,
     MM_TAG_PGM_HANDLERS,
+    MM_TAG_PGM_MAPPINGS,
     MM_TAG_PGM_PHYS,
     MM_TAG_PGM_POOL,
 
@@ -281,6 +207,7 @@ DECLINLINE(RTRCPTR) MMHyperCCToRC(PVM pVM, void *pv)
 VMMDECL(int)        MMHyperAlloc(PVM pVM, size_t cb, uint32_t uAlignment, MMTAG enmTag, void **ppv);
 VMMDECL(int)        MMHyperFree(PVM pVM, void *pv);
 VMMDECL(void)       MMHyperHeapCheck(PVM pVM);
+VMMDECL(int)        MMR3LockCall(PVM pVM);
 #ifdef DEBUG
 VMMDECL(void)       MMHyperHeapDump(PVM pVM);
 #endif
@@ -323,19 +250,26 @@ VMMR3DECL(int)      MMR3HyperInitFinalize(PVM pVM);
 VMMR3DECL(int)      MMR3Term(PVM pVM);
 VMMR3DECL(void)     MMR3TermUVM(PUVM pUVM);
 VMMR3DECL(void)     MMR3Reset(PVM pVM);
+VMMR3DECL(int)      MMR3ReserveHandyPages(PVM pVM, uint32_t cHandyPages);
 VMMR3DECL(int)      MMR3IncreaseBaseReservation(PVM pVM, uint64_t cAddBasePages);
 VMMR3DECL(int)      MMR3AdjustFixedReservation(PVM pVM, int32_t cDeltaFixedPages, const char *pszDesc);
 VMMR3DECL(int)      MMR3UpdateShadowReservation(PVM pVM, uint32_t cShadowPages);
 
 VMMR3DECL(int)      MMR3HCPhys2HCVirt(PVM pVM, RTHCPHYS HCPhys, void **ppv);
-VMMR3DECL(int)      MMR3ReadGCVirt(PVM pVM, void *pvDst, RTGCPTR GCPtr, size_t cb);
-VMMR3DECL(int)      MMR3WriteGCVirt(PVM pVM, RTGCPTR GCPtrDst, const void *pvSrc, size_t cb);
-
+VMMR3DECL(void)     MMR3ReleaseOwnedLocks(PVM pVM);
 
 /** @defgroup grp_mm_r3_hyper  Hypervisor Memory Manager (HC R3 Portion)
  * @ingroup grp_mm_r3
  * @{ */
-VMMDECL(int)        MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, uint32_t uAlignment, MMTAG enmTag, void **ppv);
+VMMR3DECL(int)      MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, uint32_t uAlignment, MMTAG enmTag, void **ppv);
+VMMR3DECL(int)      MMR3HyperAllocOnceNoRelEx(PVM pVM, size_t cb, uint32_t uAlignment, MMTAG enmTag, uint32_t fFlags, void **ppv);
+/** @name  MMR3HyperAllocOnceNoRelEx flags
+ * @{ */
+/** Must have kernel mapping.
+ * If not specified, the R0 pointer may point to the user process mapping. */
+#define MMHYPER_AONR_FLAGS_KERNEL_MAPPING   RT_BIT(0)
+/** @} */
+VMMR3DECL(int)      MMR3HyperSetGuard(PVM pVM, void *pvStart, size_t cb, bool fSet);
 VMMR3DECL(int)      MMR3HyperMapHCPhys(PVM pVM, void *pvR3, RTR0PTR pvR0, RTHCPHYS HCPhys, size_t cb, const char *pszDesc, PRTGCPTR pGCPtr);
 VMMR3DECL(int)      MMR3HyperMapGCPhys(PVM pVM, RTGCPHYS GCPhys, size_t cb, const char *pszDesc, PRTGCPTR pGCPtr);
 VMMR3DECL(int)      MMR3HyperMapMMIO2(PVM pVM, PPDMDEVINS pDevIns, uint32_t iRegion, RTGCPHYS off, RTGCPHYS cb, const char *pszDesc, PRTRCPTR pRCPtr);
@@ -350,15 +284,9 @@ VMMR3DECL(int)      MMR3HyperReadGCVirt(PVM pVM, void *pvDst, RTGCPTR GCPtr, siz
 
 
 /** @defgroup grp_mm_phys   Guest Physical Memory Manager
+ * @todo retire this group, elimintating or moving MMR3PhysGetRamSize to PGMPhys.
  * @ingroup grp_mm_r3
  * @{ */
-VMMR3DECL(int)      MMR3PhysRegister(PVM pVM, void *pvRam, RTGCPHYS GCPhys, unsigned cb, unsigned fFlags, const char *pszDesc);
-#ifndef VBOX_WITH_NEW_PHYS_CODE
-VMMR3DECL(int)      MMR3PhysRegisterEx(PVM pVM, void *pvRam, RTGCPHYS GCPhys, unsigned cb, unsigned fFlags, MMPHYSREG enmType, const char *pszDesc);
-#endif
-VMMR3DECL(int)      MMR3PhysRomRegister(PVM pVM, PPDMDEVINS pDevIns, RTGCPHYS GCPhys, RTUINT cbRange, const void *pvBinary, bool fShadow, const char *pszDesc);
-VMMR3DECL(int)      MMR3PhysRomProtect(PVM pVM, RTGCPHYS GCPhys, RTUINT cbRange);
-VMMR3DECL(int)      MMR3PhysReserve(PVM pVM, RTGCPHYS GCPhys, RTUINT cbRange, const char *pszDesc);
 VMMR3DECL(uint64_t) MMR3PhysGetRamSize(PVM pVM);
 /** @} */
 
@@ -398,6 +326,19 @@ VMMR3DECL(char *)   MMR3HeapAPrintfVU(PUVM pUVM, MMTAG enmTag, const char *pszFo
 VMMR3DECL(void)     MMR3HeapFree(void *pv);
 /** @} */
 
+/** @defgroup grp_mm_heap   User-kernel Heap Manager.
+ * @ingroup grp_mm_r3
+ *
+ * The memory is safely accessible from kernel context as well as user land.
+ *
+ * @{ */
+VMMR3DECL(void *)   MMR3UkHeapAlloc(PVM pVM, MMTAG enmTag, size_t cbSize, PRTR0PTR pR0Ptr);
+VMMR3DECL(int)      MMR3UkHeapAllocEx(PVM pVM, MMTAG enmTag, size_t cbSize, void **ppv, PRTR0PTR pR0Ptr);
+VMMR3DECL(void *)   MMR3UkHeapAllocZ(PVM pVM, MMTAG enmTag, size_t cbSize, PRTR0PTR pR0Ptr);
+VMMR3DECL(int)      MMR3UkHeapAllocZEx(PVM pVM, MMTAG enmTag, size_t cbSize, void **ppv, PRTR0PTR pR0Ptr);
+VMMR3DECL(void)     MMR3UkHeapFree(PVM pVM, void *pv, MMTAG enmTag);
+/** @} */
+
 /** @} */
 #endif /* IN_RING3 */
 
@@ -420,7 +361,7 @@ VMMRCDECL(int)      MMGCRamWrite(PVM pVM, void *pDst, void *pSrc, size_t cb);
 #endif /* IN_RC */
 
 /** @} */
-__END_DECLS
+RT_C_DECLS_END
 
 
 #endif
