@@ -1,5 +1,5 @@
 /** @file
- * PDM - Pluggable Device Manager, Critical Sections.
+ * PDM - Pluggable Device Manager, Critical Sections. (VMM)
  */
 
 /*
@@ -63,7 +63,9 @@ VMMDECL(bool)       PDMCritSectIsOwner(PCPDMCRITSECT pCritSect);
 VMMDECL(bool)       PDMCritSectIsOwnerEx(PCPDMCRITSECT pCritSect, VMCPUID idCpu);
 VMMDECL(bool)       PDMCritSectIsOwned(PCPDMCRITSECT pCritSect);
 VMMDECL(bool)       PDMCritSectIsInitialized(PCPDMCRITSECT pCritSect);
+VMMDECL(bool)       PDMCritSectHasWaiters(PCPDMCRITSECT pCritSect);
 VMMDECL(uint32_t)   PDMCritSectGetRecursion(PCPDMCRITSECT pCritSect);
+VMMR3DECL(bool)     PDMR3CritSectYield(PPDMCRITSECT pCritSect);
 VMMR3DECL(const char *) PDMR3CritSectName(PCPDMCRITSECT pCritSect);
 VMMR3DECL(int)      PDMR3CritSectScheduleExitEvent(PPDMCRITSECT pCritSect, RTSEMEVENT EventToSignal);
 VMMR3DECL(int)      PDMR3CritSectDelete(PPDMCRITSECT pCritSect);
