@@ -1,5 +1,5 @@
 /** @file
- * STAM - Statistics Manager.
+ * STAM - Statistics Manager. (VMM)
  */
 
 /*
@@ -58,7 +58,7 @@ RT_C_DECLS_BEGIN
 #ifdef __GNUC__
 # if defined(RT_ARCH_X86)
    /* This produces optimal assembler code for x86 but does not work for AMD64 ('A' means 'either rax or rdx') */
-#  define STAM_GET_TS(u64) __asm__ __volatile__ ("rdtsc\n\t" : "=A" (u64));
+#  define STAM_GET_TS(u64) __asm__ __volatile__ ("rdtsc\n\t" : "=A" (u64))
 # elif defined(RT_ARCH_AMD64)
 #  define STAM_GET_TS(u64) \
     do { uint64_t low; uint64_t high; \
@@ -66,21 +66,23 @@ RT_C_DECLS_BEGIN
          (u64) = ((high << 32) | low); \
     } while (0)
 # endif
-#elif _MSC_VER >= 1400
-# pragma intrinsic(__rdtsc)
-# define STAM_GET_TS(u64)    \
-    do { (u64) = __rdtsc(); } while (0)
 #else
-# define STAM_GET_TS(u64)    \
-    do {                               \
-        uint64_t u64Tmp;               \
-        __asm {                        \
-            __asm rdtsc                \
-            __asm mov dword ptr [u64Tmp],     eax   \
-            __asm mov dword ptr [u64Tmp + 4], edx   \
-        }                              \
-        (u64) = u64Tmp; \
-    } while (0)
+# if _MSC_VER >= 1400
+#  pragma intrinsic(__rdtsc)
+#  define STAM_GET_TS(u64)    \
+     do { (u64) = __rdtsc(); } while (0)
+# else
+#  define STAM_GET_TS(u64)    \
+     do {                               \
+         uint64_t u64Tmp;               \
+         __asm {                        \
+             __asm rdtsc                \
+             __asm mov dword ptr [u64Tmp],     eax   \
+             __asm mov dword ptr [u64Tmp + 4], edx   \
+         }                              \
+         (u64) = u64Tmp; \
+     } while (0)
+# endif
 #endif
 
 
