@@ -1,5 +1,5 @@
 /** @file
- * SUP - Support Library.
+ * SUP - Support Library. (HDrv)
  */
 
 /*
@@ -36,7 +36,7 @@
 #include <iprt/stdarg.h>
 #include <iprt/asm.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 /** @defgroup   grp_sup     The Support Library API
  * @{
@@ -67,7 +67,7 @@ typedef const SUPPAGE *PCSUPPAGE;
 typedef enum SUPPAGINGMODE
 {
     /** The usual invalid entry.
-     * This is returned by SUPGetPagingMode()  */
+     * This is returned by SUPR3GetPagingMode()  */
     SUPPAGINGMODE_INVALID = 0,
     /** Normal 32-bit paging, no global pages */
     SUPPAGINGMODE_32_BIT,
@@ -127,8 +127,10 @@ typedef struct SUPGIPCPU
      * This history is used to calculate u32UpdateIntervalTSC.
      */
     volatile uint32_t   au32TSCHistory[8];
+    /** The interval between the last two NanoTS updates. (experiment for now) */
+    volatile uint32_t   u32UpdateIntervalNS;
     /** Reserved for future per processor data. */
-    volatile uint32_t   au32Reserved[6];
+    volatile uint32_t   au32Reserved[5];
 } SUPGIPCPU;
 AssertCompileSize(SUPGIPCPU, 96);
 /*AssertCompileMemberAlignment(SUPGIPCPU, u64TSC, 8); -fixme */
@@ -290,6 +292,13 @@ typedef SUPVMMR0REQHDR *PSUPVMMR0REQHDR;
 #define SUP_VMMR0_DO_NOP        2
 /** @} */
 
+/** SUPR3QueryVTCaps capability flags
+ * @{
+ */
+#define SUPVTCAPS_AMD_V             RT_BIT(0)
+#define SUPVTCAPS_VT_X              RT_BIT(1)
+#define SUPVTCAPS_NESTED_PAGING     RT_BIT(2)
+/** @} */
 
 /**
  * Request for generic FNSUPR0SERVICEREQHANDLER calls.
@@ -307,6 +316,138 @@ typedef SUPR0SERVICEREQHDR *PSUPR0SERVICEREQHDR;
 #define SUPR0SERVICEREQHDR_MAGIC    UINT32_C(0x19640416)
 
 
+/** Event semaphore handle. Ring-0 / ring-3. */
+typedef R0PTRTYPE(struct SUPSEMEVENTHANDLE *) SUPSEMEVENT;
+/** Pointer to an event semaphore handle. */
+typedef SUPSEMEVENT *PSUPSEMEVENT;
+/** Nil event semaphore handle. */
+#define NIL_SUPSEMEVENT         ((SUPSEMEVENT)0)
+
+/**
+ * Creates a single release event semaphore.
+ *
+ * @returns VBox status code.
+ * @param   pSession        The session handle of the caller.
+ * @param   phEvent         Where to return the handle to the event semaphore.
+ */
+SUPDECL(int) SUPSemEventCreate(PSUPDRVSESSION pSession, PSUPSEMEVENT phEvent);
+
+/**
+ * Closes a single release event semaphore handle.
+ *
+ * @returns VBox status code.
+ * @retval  VINF_OBJECT_DESTROYED if the semaphore was destroyed.
+ * @retval  VINF_SUCCESS if the handle was successfully closed but the sempahore
+ *          object remained alive because of other references.
+ *
+ * @param   pSession            The session handle of the caller.
+ * @param   hEvent              The handle. Nil is quietly ignored.
+ */
+SUPDECL(int) SUPSemEventClose(PSUPDRVSESSION pSession, SUPSEMEVENT hEvent);
+
+/**
+ * Signals a single release event semaphore.
+ *
+ * @returns VBox status code.
+ * @param   pSession            The session handle of the caller.
+ * @param   hEvent              The semaphore handle.
+ */
+SUPDECL(int) SUPSemEventSignal(PSUPDRVSESSION pSession, SUPSEMEVENT hEvent);
+
+#ifdef IN_RING0
+/**
+ * Waits on a single release event semaphore, not interruptible.
+ *
+ * @returns VBox status code.
+ * @param   pSession            The session handle of the caller.
+ * @param   hEvent              The semaphore handle.
+ * @param   cMillies            The number of milliseconds to wait.
+ * @remarks Not available in ring-3.
+ */
+SUPDECL(int) SUPSemEventWait(PSUPDRVSESSION pSession, SUPSEMEVENT hEvent, uint32_t cMillies);
+#endif
+
+/**
+ * Waits on a single release event semaphore, interruptible.
+ *
+ * @returns VBox status code.
+ * @param   pSession            The session handle of the caller.
+ * @param   hEvent              The semaphore handle.
+ * @param   cMillies            The number of milliseconds to wait.
+ */
+SUPDECL(int) SUPSemEventWaitNoResume(PSUPDRVSESSION pSession, SUPSEMEVENT hEvent, uint32_t cMillies);
+
+
+/** Multiple release event semaphore handle. Ring-0 / ring-3. */
+typedef R0PTRTYPE(struct SUPSEMEVENTMULTIHANDLE *)  SUPSEMEVENTMULTI;
+/** Pointer to an multiple release event semaphore handle. */
+typedef SUPSEMEVENTMULTI                           *PSUPSEMEVENTMULTI;
+/** Nil multiple release event semaphore handle. */
+#define NIL_SUPSEMEVENTMULTI                        ((SUPSEMEVENTMULTI)0)
+
+/**
+ * Creates a multiple release event semaphore.
+ *
+ * @returns VBox status code.
+ * @param   pSession        The session handle of the caller.
+ * @param   phEventMulti    Where to return the handle to the event semaphore.
+ */
+SUPDECL(int) SUPSemEventMultiCreate(PSUPDRVSESSION pSession, PSUPSEMEVENTMULTI phEventMulti);
+
+/**
+ * Closes a multiple release event semaphore handle.
+ *
+ * @returns VBox status code.
+ * @retval  VINF_OBJECT_DESTROYED if the semaphore was destroyed.
+ * @retval  VINF_SUCCESS if the handle was successfully closed but the sempahore
+ *          object remained alive because of other references.
+ *
+ * @param   pSession            The session handle of the caller.
+ * @param   hEventMulti         The handle. Nil is quietly ignored.
+ */
+SUPDECL(int) SUPSemEventMultiClose(PSUPDRVSESSION pSession, SUPSEMEVENTMULTI hEventMulti);
+
+/**
+ * Signals a multiple release event semaphore.
+ *
+ * @returns VBox status code.
+ * @param   pSession            The session handle of the caller.
+ * @param   hEventMulti         The semaphore handle.
+ */
+SUPDECL(int) SUPSemEventMultiSignal(PSUPDRVSESSION pSession, SUPSEMEVENTMULTI hEventMulti);
+
+/**
+ * Resets a multiple release event semaphore.
+ *
+ * @returns VBox status code.
+ * @param   pSession            The session handle of the caller.
+ * @param   hEventMulti         The semaphore handle.
+ */
+SUPDECL(int) SUPSemEventMultiReset(PSUPDRVSESSION pSession, SUPSEMEVENTMULTI hEventMulti);
+
+#ifdef IN_RING0
+/**
+ * Waits on a multiple release event semaphore, not interruptible.
+ *
+ * @returns VBox status code.
+ * @param   pSession            The session handle of the caller.
+ * @param   hEventMulti         The semaphore handle.
+ * @param   cMillies            The number of milliseconds to wait.
+ * @remarks Not available in ring-3.
+ */
+SUPDECL(int) SUPSemEventMultiWait(PSUPDRVSESSION pSession, SUPSEMEVENTMULTI hEventMulti, uint32_t cMillies);
+#endif
+
+/**
+ * Waits on a multiple release event semaphore, interruptible.
+ *
+ * @returns VBox status code.
+ * @param   pSession            The session handle of the caller.
+ * @param   hEventMulti         The semaphore handle.
+ * @param   cMillies            The number of milliseconds to wait.
+ */
+SUPDECL(int) SUPSemEventMultiWaitNoResume(PSUPDRVSESSION pSession, SUPSEMEVENTMULTI hEventMulti, uint32_t cMillies);
+
 
 #ifdef IN_RING3
 
@@ -320,14 +461,14 @@ typedef SUPR0SERVICEREQHDR *PSUPR0SERVICEREQHDR;
  *
  * @returns VBox status code.
  */
-SUPR3DECL(int) SUPInstall(void);
+SUPR3DECL(int) SUPR3Install(void);
 
 /**
  * Uninstalls the support library.
  *
  * @returns VBox status code.
  */
-SUPR3DECL(int) SUPUninstall(void);
+SUPR3DECL(int) SUPR3Uninstall(void);
 
 /**
  * Trusted main entry point.
@@ -410,7 +551,7 @@ DECLHIDDEN(int) SUPR3HardenedMain(const char *pszProgName, uint32_t fFlags, int
 /**
  * Initializes the support library.
  * Each succesful call to SUPR3Init() must be countered by a
- * call to SUPTerm(false).
+ * call to SUPR3Term(false).
  *
  * @returns VBox status code.
  * @param   ppSession       Where to store the session handle. Defaults to NULL.
@@ -425,9 +566,9 @@ SUPR3DECL(int) SUPR3Init(PSUPDRVSESSION *ppSession);
  *                      init call count and just terminated.
  */
 #ifdef __cplusplus
-SUPR3DECL(int) SUPTerm(bool fForced = false);
+SUPR3DECL(int) SUPR3Term(bool fForced = false);
 #else
-SUPR3DECL(int) SUPTerm(int fForced);
+SUPR3DECL(int) SUPR3Term(int fForced);
 #endif
 
 /**
@@ -438,7 +579,7 @@ SUPR3DECL(int) SUPTerm(int fForced);
  *                      NIL_RTR0PTR can be used to unset the handle when the
  *                      VM is about to be destroyed.
  */
-SUPR3DECL(int) SUPSetVMForFastIOCtl(PVMR0 pVMR0);
+SUPR3DECL(int) SUPR3SetVMForFastIOCtl(PVMR0 pVMR0);
 
 /**
  * Calls the HC R0 VMM entry point.
@@ -446,38 +587,40 @@ SUPR3DECL(int) SUPSetVMForFastIOCtl(PVMR0 pVMR0);
  *
  * @returns error code specific to uFunction.
  * @param   pVMR0       Pointer to the Ring-0 (Host Context) mapping of the VM structure.
+ * @param   idCpu       The virtual CPU ID.
  * @param   uOperation  Operation to execute.
  * @param   pvArg       Argument.
  */
-SUPR3DECL(int) SUPCallVMMR0(PVMR0 pVMR0, unsigned uOperation, void *pvArg);
+SUPR3DECL(int) SUPR3CallVMMR0(PVMR0 pVMR0, VMCPUID idCpu, unsigned uOperation, void *pvArg);
 
 /**
- * Variant of SUPCallVMMR0, except that this takes the fast ioclt path
+ * Variant of SUPR3CallVMMR0, except that this takes the fast ioclt path
  * regardsless of compile-time defaults.
  *
  * @returns VBox status code.
  * @param   pVMR0       The ring-0 VM handle.
  * @param   uOperation  The operation; only the SUP_VMMR0_DO_* ones are valid.
- * @param   idCPU       VMCPU id.
+ * @param   idCpu       The virtual CPU ID.
  */
-SUPR3DECL(int) SUPCallVMMR0Fast(PVMR0 pVMR0, unsigned uOperation, unsigned idCPU);
+SUPR3DECL(int) SUPR3CallVMMR0Fast(PVMR0 pVMR0, unsigned uOperation, VMCPUID idCpu);
 
 /**
- * Calls the HC R0 VMM entry point, in a safer but slower manner than SUPCallVMMR0.
- * When entering using this call the R0 components can call into the host kernel
- * (i.e. use the SUPR0 and RT APIs).
+ * Calls the HC R0 VMM entry point, in a safer but slower manner than
+ * SUPR3CallVMMR0. When entering using this call the R0 components can call
+ * into the host kernel (i.e. use the SUPR0 and RT APIs).
  *
  * See VMMR0Entry() for more details.
  *
  * @returns error code specific to uFunction.
  * @param   pVMR0       Pointer to the Ring-0 (Host Context) mapping of the VM structure.
+ * @param   idCpu       The virtual CPU ID.
  * @param   uOperation  Operation to execute.
  * @param   u64Arg      Constant argument.
  * @param   pReqHdr     Pointer to a request header. Optional.
  *                      This will be copied in and out of kernel space. There currently is a size
  *                      limit on this, just below 4KB.
  */
-SUPR3DECL(int) SUPCallVMMR0Ex(PVMR0 pVMR0, unsigned uOperation, uint64_t u64Arg, PSUPVMMR0REQHDR pReqHdr);
+SUPR3DECL(int) SUPR3CallVMMR0Ex(PVMR0 pVMR0, VMCPUID idCpu, unsigned uOperation, uint64_t u64Arg, PSUPVMMR0REQHDR pReqHdr);
 
 /**
  * Calls a ring-0 service.
@@ -495,56 +638,70 @@ SUPR3DECL(int) SUPCallVMMR0Ex(PVMR0 pVMR0, unsigned uOperation, uint64_t u64Arg,
  */
 SUPR3DECL(int) SUPR3CallR0Service(const char *pszService, size_t cchService, uint32_t uOperation, uint64_t u64Arg, PSUPR0SERVICEREQHDR pReqHdr);
 
+/** Which logger. */
+typedef enum SUPLOGGER
+{
+    SUPLOGGER_DEBUG = 1,
+    SUPLOGGER_RELEASE
+} SUPLOGGER;
+
+/**
+ * Changes the settings of the specified ring-0 logger.
+ *
+ * @returns VBox status code.
+ * @param   enmWhich    Which logger.
+ * @param   pszFlags    The flags settings.
+ * @param   pszGroups   The groups settings.
+ * @param   pszDest     The destionation specificier.
+ */
+SUPR3DECL(int) SUPR3LoggerSettings(SUPLOGGER enmWhich, const char *pszFlags, const char *pszGroups, const char *pszDest);
+
+/**
+ * Creates a ring-0 logger instance.
+ *
+ * @returns VBox status code.
+ * @param   enmWhich    Which logger to create.
+ * @param   pszFlags    The flags settings.
+ * @param   pszGroups   The groups settings.
+ * @param   pszDest     The destionation specificier.
+ */
+SUPR3DECL(int) SUPR3LoggerCreate(SUPLOGGER enmWhich, const char *pszFlags, const char *pszGroups, const char *pszDest);
+
+/**
+ * Destroys a ring-0 logger instance.
+ *
+ * @returns VBox status code.
+ * @param   enmWhich    Which logger.
+ */
+SUPR3DECL(int) SUPR3LoggerDestroy(SUPLOGGER enmWhich);
+
 /**
  * Queries the paging mode of the host OS.
  *
  * @returns The paging mode.
  */
-SUPR3DECL(SUPPAGINGMODE) SUPGetPagingMode(void);
+SUPR3DECL(SUPPAGINGMODE) SUPR3GetPagingMode(void);
 
 /**
  * Allocate zero-filled pages.
  *
- * Use this to allocate a number of pages rather than using RTMem*() and mess with
- * alignment. The returned address is of course page aligned. Call SUPPageFree()
- * to free the pages once done with them.
+ * Use this to allocate a number of pages suitable for seeding / locking.
+ * Call SUPR3PageFree() to free the pages once done with them.
  *
  * @returns VBox status.
  * @param   cPages          Number of pages to allocate.
  * @param   ppvPages        Where to store the base pointer to the allocated pages.
  */
-SUPR3DECL(int) SUPPageAlloc(size_t cPages, void **ppvPages);
+SUPR3DECL(int) SUPR3PageAlloc(size_t cPages, void **ppvPages);
 
 /**
- * Frees pages allocated with SUPPageAlloc().
+ * Frees pages allocated with SUPR3PageAlloc().
  *
  * @returns VBox status.
- * @param   pvPages         Pointer returned by SUPPageAlloc().
+ * @param   pvPages         Pointer returned by SUPR3PageAlloc().
  * @param   cPages          Number of pages that was allocated.
  */
-SUPR3DECL(int) SUPPageFree(void *pvPages, size_t cPages);
-
-/**
- * Locks down the physical memory backing a virtual memory
- * range in the current process.
- *
- * @returns VBox status code.
- * @param   pvStart         Start of virtual memory range.
- *                          Must be page aligned.
- * @param   cPages          Number of pages.
- * @param   paPages         Where to store the physical page addresses returned.
- *                          On entry this will point to an array of with cbMemory >> PAGE_SHIFT entries.
- */
-SUPR3DECL(int) SUPPageLock(void *pvStart, size_t cPages, PSUPPAGE paPages);
-
-/**
- * Releases locked down pages.
- *
- * @returns VBox status code.
- * @param   pvStart         Start of virtual memory range previously locked
- *                          down by SUPPageLock().
- */
-SUPR3DECL(int) SUPPageUnlock(void *pvStart);
+SUPR3DECL(int) SUPR3PageFree(void *pvPages, size_t cPages);
 
 /**
  * Allocate non-zeroed, locked, pages with user and, optionally, kernel
@@ -552,10 +709,6 @@ SUPR3DECL(int) SUPPageUnlock(void *pvStart);
  *
  * Use SUPR3PageFreeEx() to free memory allocated with this function.
  *
- * This SUPR3PageAllocEx and SUPR3PageFreeEx replaces SUPPageAllocLocked,
- * SUPPageAllocLockedEx, SUPPageFreeLocked, SUPPageAlloc, SUPPageLock,
- * SUPPageUnlock and SUPPageFree.
- *
  * @returns VBox status code.
  * @param   cPages          The number of pages to allocate.
  * @param   fFlags          Flags, reserved. Must be zero.
@@ -570,17 +723,39 @@ SUPR3DECL(int) SUPR3PageAllocEx(size_t cPages, uint32_t fFlags, void **ppvPages,
 /**
  * Maps a portion of a ring-3 only allocation into kernel space.
  *
- * @return VBox status code.
+ * @returns VBox status code.
  *
- * @param  pvR3             The address SUPR3PageAllocEx return.
- * @param  off              Offset to start mapping at. Must be page aligned.
- * @param  cb               Number of bytes to map. Must be page aligned.
- * @param  fFlags           Flags, must be zero.
- * @param  pR0Ptr           Where to store the address on success.
+ * @param   pvR3            The address SUPR3PageAllocEx return.
+ * @param   off             Offset to start mapping at. Must be page aligned.
+ * @param   cb              Number of bytes to map. Must be page aligned.
+ * @param   fFlags          Flags, must be zero.
+ * @param   pR0Ptr          Where to store the address on success.
  *
  */
 SUPR3DECL(int) SUPR3PageMapKernel(void *pvR3, uint32_t off, uint32_t cb, uint32_t fFlags, PRTR0PTR pR0Ptr);
 
+/**
+ * Changes the protection of
+ *
+ * @returns VBox status code.
+ * @retval  VERR_NOT_SUPPORTED if the OS doesn't allow us to change page level
+ *          protection. See also RTR0MemObjProtect.
+ *
+ * @param   pvR3            The ring-3 address SUPR3PageAllocEx returned.
+ * @param   R0Ptr           The ring-0 address SUPR3PageAllocEx returned if it
+ *                          is desired that the corresponding ring-0 page
+ *                          mappings should change protection as well. Pass
+ *                          NIL_RTR0PTR if the ring-0 pages should remain
+ *                          unaffected.
+ * @param   off             Offset to start at which to start chagning the page
+ *                          level protection. Must be page aligned.
+ * @param   cb              Number of bytes to change. Must be page aligned.
+ * @param   fProt           The new page level protection, either a combination
+ *                          of RTMEM_PROT_READ, RTMEM_PROT_WRITE and
+ *                          RTMEM_PROT_EXEC, or just RTMEM_PROT_NONE.
+ */
+SUPR3DECL(int) SUPR3PageProtect(void *pvR3, RTR0PTR R0Ptr, uint32_t off, uint32_t cb, uint32_t fProt);
+
 /**
  * Free pages allocated by SUPR3PageAllocEx.
  *
@@ -590,44 +765,6 @@ SUPR3DECL(int) SUPR3PageMapKernel(void *pvR3, uint32_t off, uint32_t cb, uint32_
  */
 SUPR3DECL(int) SUPR3PageFreeEx(void *pvPages, size_t cPages);
 
-/**
- * Allocate non-zeroed locked pages.
- *
- * Use this to allocate a number of pages rather than using RTMem*() and mess with
- * alignment. The returned address is of course page aligned. Call SUPPageFreeLocked()
- * to free the pages once done with them.
- *
- * @returns VBox status code.
- * @param   cPages          Number of pages to allocate.
- * @param   ppvPages        Where to store the base pointer to the allocated pages.
- * @param   paPages         Where to store the physical page addresses returned.
- *                          On entry this will point to an array of with cbMemory >> PAGE_SHIFT entries.
- *                          NULL is allowed.
- */
-SUPR3DECL(int) SUPPageAllocLockedEx(size_t cPages, void **ppvPages, PSUPPAGE paPages);
-
-/**
- * Frees locked pages allocated with SUPPageAllocLocked().
- *
- * @returns VBox status.
- * @param   pvPages         Pointer returned by SUPPageAlloc().
- * @param   cPages          Number of pages that was allocated.
- */
-SUPR3DECL(int) SUPPageFreeLocked(void *pvPages, size_t cPages);
-
-/**
- * Allocated memory with page aligned memory with a contiguous and locked physical
- * memory backing below 4GB.
- *
- * @returns Pointer to the allocated memory (virtual address).
- *          *pHCPhys is set to the physical address of the memory.
- *          The returned memory must be freed using SUPContFree().
- * @returns NULL on failure.
- * @param   cPages      Number of pages to allocate.
- * @param   pHCPhys     Where to store the physical address of the memory block.
- */
-SUPR3DECL(void *) SUPContAlloc(size_t cPages, PRTHCPHYS pHCPhys);
-
 /**
  * Allocated memory with page aligned memory with a contiguous and locked physical
  * memory backing below 4GB.
@@ -635,7 +772,7 @@ SUPR3DECL(void *) SUPContAlloc(size_t cPages, PRTHCPHYS pHCPhys);
  * @returns Pointer to the allocated memory (virtual address).
  *          *pHCPhys is set to the physical address of the memory.
  *          If ppvR0 isn't NULL, *ppvR0 is set to the ring-0 mapping.
- *          The returned memory must be freed using SUPContFree().
+ *          The returned memory must be freed using SUPR3ContFree().
  * @returns NULL on failure.
  * @param   cPages      Number of pages to allocate.
  * @param   pR0Ptr      Where to store the ring-0 mapping of the allocation. (optional)
@@ -645,16 +782,16 @@ SUPR3DECL(void *) SUPContAlloc(size_t cPages, PRTHCPHYS pHCPhys);
  *          ring-3 mapping executable on WIN64. This is a serious problem in regard to
  *          the world switchers.
  */
-SUPR3DECL(void *) SUPContAlloc2(size_t cPages, PRTR0PTR pR0Ptr, PRTHCPHYS pHCPhys);
+SUPR3DECL(void *) SUPR3ContAlloc(size_t cPages, PRTR0PTR pR0Ptr, PRTHCPHYS pHCPhys);
 
 /**
- * Frees memory allocated with SUPContAlloc().
+ * Frees memory allocated with SUPR3ContAlloc().
  *
  * @returns VBox status code.
  * @param   pv          Pointer to the memory block which should be freed.
  * @param   cPages      Number of pages to be freed.
  */
-SUPR3DECL(int) SUPContFree(void *pv, size_t cPages);
+SUPR3DECL(int) SUPR3ContFree(void *pv, size_t cPages);
 
 /**
  * Allocated non contiguous physical memory below 4GB.
@@ -665,21 +802,21 @@ SUPR3DECL(int) SUPContFree(void *pv, size_t cPages);
  * @returns NULL on failure.
  * @param   cPages      Number of pages to allocate.
  * @param   ppvPages    Where to store the pointer to the allocated memory.
- *                      The pointer stored here on success must be passed to SUPLowFree when
- *                      the memory should be released.
+ *                      The pointer stored here on success must be passed to
+ *                      SUPR3LowFree when the memory should be released.
  * @param   ppvPagesR0  Where to store the ring-0 pointer to the allocated memory. optional.
  * @param   paPages     Where to store the physical addresses of the individual pages.
  */
-SUPR3DECL(int) SUPLowAlloc(size_t cPages, void **ppvPages, PRTR0PTR ppvPagesR0, PSUPPAGE paPages);
+SUPR3DECL(int) SUPR3LowAlloc(size_t cPages, void **ppvPages, PRTR0PTR ppvPagesR0, PSUPPAGE paPages);
 
 /**
- * Frees memory allocated with SUPLowAlloc().
+ * Frees memory allocated with SUPR3LowAlloc().
  *
  * @returns VBox status code.
  * @param   pv          Pointer to the memory block which should be freed.
  * @param   cPages      Number of pages that was allocated.
  */
-SUPR3DECL(int) SUPLowFree(void *pv, size_t cPages);
+SUPR3DECL(int) SUPR3LowFree(void *pv, size_t cPages);
 
 /**
  * Load a module into R0 HC.
@@ -692,7 +829,7 @@ SUPR3DECL(int) SUPLowFree(void *pv, size_t cPages);
  * @param   pszModule       The module name. Max 32 bytes.
  * @param   ppvImageBase        Where to store the image address.
  */
-SUPR3DECL(int) SUPLoadModule(const char *pszFilename, const char *pszModule, void **ppvImageBase);
+SUPR3DECL(int) SUPR3LoadModule(const char *pszFilename, const char *pszModule, void **ppvImageBase);
 
 /**
  * Load a module into R0 HC.
@@ -717,7 +854,7 @@ SUPR3DECL(int) SUPR3LoadServiceModule(const char *pszFilename, const char *pszMo
  * @param   pszModule       The module to free.
  * @remark  This will not actually 'free' the module, there are of course usage counting.
  */
-SUPR3DECL(int) SUPFreeModule(void *pvImageBase);
+SUPR3DECL(int) SUPR3FreeModule(void *pvImageBase);
 
 /**
  * Get the address of a symbol in a ring-0 module.
@@ -728,23 +865,23 @@ SUPR3DECL(int) SUPFreeModule(void *pvImageBase);
  *                          ordinal value rather than a string pointer.
  * @param   ppvValue        Where to store the symbol value.
  */
-SUPR3DECL(int) SUPGetSymbolR0(void *pvImageBase, const char *pszSymbol, void **ppvValue);
+SUPR3DECL(int) SUPR3GetSymbolR0(void *pvImageBase, const char *pszSymbol, void **ppvValue);
 
 /**
  * Load R0 HC VMM code.
  *
  * @returns VBox status code.
- * @deprecated  Use SUPLoadModule(pszFilename, "VMMR0.r0", &pvImageBase)
+ * @deprecated  Use SUPR3LoadModule(pszFilename, "VMMR0.r0", &pvImageBase)
  */
-SUPR3DECL(int) SUPLoadVMM(const char *pszFilename);
+SUPR3DECL(int) SUPR3LoadVMM(const char *pszFilename);
 
 /**
  * Unloads R0 HC VMM code.
  *
  * @returns VBox status code.
- * @deprecated  Use SUPFreeModule().
+ * @deprecated  Use SUPR3FreeModule().
  */
-SUPR3DECL(int) SUPUnloadVMM(void);
+SUPR3DECL(int) SUPR3UnloadVMM(void);
 
 /**
  * Get the physical address of the GIP.
@@ -752,7 +889,7 @@ SUPR3DECL(int) SUPUnloadVMM(void);
  * @returns VBox status code.
  * @param   pHCPhys     Where to store the physical address of the GIP.
  */
-SUPR3DECL(int) SUPGipGetPhys(PRTHCPHYS pHCPhys);
+SUPR3DECL(int) SUPR3GipGetPhys(PRTHCPHYS pHCPhys);
 
 /**
  * Verifies the integrity of a file, and optionally opens it.
@@ -795,6 +932,23 @@ SUPR3DECL(int) SUPR3HardenedLdrLoad(const char *pszFilename, PRTLDRMOD phLdrMod)
  */
 SUPR3DECL(int) SUPR3HardenedLdrLoadAppPriv(const char *pszFilename, PRTLDRMOD phLdrMod);
 
+
+/**
+ * Check if the host kernel can run in VMX root mode.
+ *
+ * @returns VINF_SUCCESS if supported, error code indicating why if not.
+ */
+SUPR3DECL(int) SUPR3QueryVTxSupported(void);
+
+
+/**
+ * Return VT-x/AMD-V capabilities.
+ *
+ * @returns VINF_SUCCESS if supported, error code indicating why if not.
+ * @param   pfCaps      Pointer to capability dword (out).
+ */
+SUPR3DECL(int) SUPR3QueryVTCaps(uint32_t *pfCaps);
+
 /** @} */
 #endif /* IN_RING3 */
 
@@ -818,6 +972,10 @@ typedef enum SUPDRVOBJTYPE
     SUPDRVOBJTYPE_INTERNAL_NETWORK,
     /** Internal network interface. */
     SUPDRVOBJTYPE_INTERNAL_NETWORK_INTERFACE,
+    /** Single release event semaphore. */
+    SUPDRVOBJTYPE_SEM_EVENT,
+    /** Multiple release event semaphore. */
+    SUPDRVOBJTYPE_SEM_EVENT_MULTI,
     /** The first invalid object type in this end. */
     SUPDRVOBJTYPE_END,
     /** The usual 32-bit type size hack. */
@@ -851,11 +1009,12 @@ SUPR0DECL(int) SUPR0LowFree(PSUPDRVSESSION pSession, RTHCUINTPTR uPtr);
 SUPR0DECL(int) SUPR0MemAlloc(PSUPDRVSESSION pSession, uint32_t cb, PRTR0PTR ppvR0, PRTR3PTR ppvR3);
 SUPR0DECL(int) SUPR0MemGetPhys(PSUPDRVSESSION pSession, RTHCUINTPTR uPtr, PSUPPAGE paPages);
 SUPR0DECL(int) SUPR0MemFree(PSUPDRVSESSION pSession, RTHCUINTPTR uPtr);
-SUPR0DECL(int) SUPR0PageAlloc(PSUPDRVSESSION pSession, uint32_t cPages, PRTR3PTR ppvR3, PRTHCPHYS paPages);
 SUPR0DECL(int) SUPR0PageAllocEx(PSUPDRVSESSION pSession, uint32_t cPages, uint32_t fFlags, PRTR3PTR ppvR3, PRTR0PTR ppvR0, PRTHCPHYS paPages);
 SUPR0DECL(int) SUPR0PageMapKernel(PSUPDRVSESSION pSession, RTR3PTR pvR3, uint32_t offSub, uint32_t cbSub, uint32_t fFlags, PRTR0PTR ppvR0);
+SUPR0DECL(int) SUPR0PageProtect(PSUPDRVSESSION pSession, RTR3PTR pvR3, RTR0PTR pvR0, uint32_t offSub, uint32_t cbSub, uint32_t fProt);
 SUPR0DECL(int) SUPR0PageFree(PSUPDRVSESSION pSession, RTR3PTR pvR3);
 SUPR0DECL(int) SUPR0GipMap(PSUPDRVSESSION pSession, PRTR3PTR ppGipR3, PRTHCPHYS pHCPhysGip);
+SUPR0DECL(int) SUPR0QueryVTCaps(PSUPDRVSESSION pSession, uint32_t *pfCaps);
 SUPR0DECL(int) SUPR0GipUnmap(PSUPDRVSESSION pSession);
 SUPR0DECL(int) SUPR0Printf(const char *pszFormat, ...);
 SUPR0DECL(SUPPAGINGMODE) SUPR0GetPagingMode(void);
@@ -970,7 +1129,7 @@ SUPR0DECL(int) SUPR0IdcComponentDeregisterFactory(PSUPDRVIDCHANDLE pHandle, PCSU
 
 /** @} */
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
