@@ -297,6 +297,25 @@ RT_C_DECLS_END
     AssertCompile(RT_OFFSETOF(type, member) == (off))
 #endif
 
+/** @def AssertCompile2MemberOffsets
+ * Asserts that two (sub-structure) members in union have the same offset.
+ * @param   type    The type.
+ * @param   member1 The first member.
+ * @param   member2 The second member.
+ */
+#if defined(__GNUC__) && defined(__cplusplus)
+# if __GNUC__ >= 4
+#  define AssertCompile2MemberOffsets(type, member1, member2) \
+    AssertCompile(__builtin_offsetof(type, member1) == __builtin_offsetof(type, member2))
+# else
+#  define AssertCompile2MemberOffsets(type, member1, member2) \
+    AssertCompile(RT_OFFSETOF(type, member1) == RT_OFFSETOF(type, member2))
+# endif
+#else
+# define AssertCompile2MemberOffsets(type, member1, member2) \
+    AssertCompile(RT_OFFSETOF(type, member1) == RT_OFFSETOF(type, member2))
+#endif
+
 /** @} */
 
 
@@ -359,6 +378,33 @@ RT_C_DECLS_END
 #endif
 
 
+/** @def AssertStmt
+ * Assert that an expression is true. If false, hit breakpoint and execute the
+ * statement.
+ * @param   expr    Expression which should be true.
+ * @param   stmt    Statement to execute on failure.
+ */
+#ifdef RT_STRICT
+# define AssertStmt(expr, stmt)  \
+    do { \
+        if (RT_UNLIKELY(!(expr))) \
+        { \
+            AssertMsg1(#expr, __LINE__, __FILE__, __PRETTY_FUNCTION__); \
+            RTAssertPanic(); \
+            stmt; \
+        } \
+    } while (0)
+#else
+# define AssertStmt(expr, stmt)  \
+    do { \
+        if (RT_UNLIKELY(!(expr))) \
+        { \
+            stmt; \
+        } \
+    } while (0)
+#endif
+
+
 /** @def AssertReturn
  * Assert that an expression is true and returns if it isn't.
  * In RT_STRICT mode it will hit a breakpoint before returning.
@@ -1670,7 +1716,7 @@ RT_C_DECLS_END
  * On failure it will print info about the rc and hit a breakpoint.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRC(rc)                AssertMsgRC(rc, ("%Rra\n", (rc)))
 
@@ -1679,7 +1725,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCReturn(rc, rcRet)   AssertMsgRCReturn(rc, ("%Rra\n", (rc)), rcRet)
 
@@ -1687,7 +1733,7 @@ RT_C_DECLS_END
  * Asserts a iprt status code successful, bitch (RT_STRICT mode only) and return if it isn't.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCReturnVoid(rc)      AssertMsgRCReturnVoid(rc, ("%Rra\n", (rc)))
 
@@ -1695,7 +1741,7 @@ RT_C_DECLS_END
  * Asserts a iprt status code successful, bitch (RT_STRICT mode only) and break if it isn't.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCBreak(rc)           AssertMsgRCBreak(rc, ("%Rra\n", (rc)))
 
@@ -1704,7 +1750,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCBreakStmt(rc, stmt) AssertMsgRCBreakStmt(rc, ("%Rra\n", (rc)), stmt)
 
@@ -1715,7 +1761,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertMsgRC(rc, msg) \
     do { AssertMsg(RT_SUCCESS_NP(rc), msg); NOREF(rc); } while (0)
@@ -1728,7 +1774,7 @@ RT_C_DECLS_END
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertMsgRCReturn(rc, msg, rcRet) \
     do { AssertMsgReturn(RT_SUCCESS_NP(rc), msg, rcRet); NOREF(rc); } while (0)
@@ -1740,7 +1786,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertMsgRCReturnVoid(rc, msg) \
     do { AssertMsgReturnVoid(RT_SUCCESS_NP(rc), msg); NOREF(rc); } while (0)
@@ -1752,7 +1798,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertMsgRCBreak(rc, msg) \
     if (1) { AssertMsgBreak(RT_SUCCESS(rc), msg); NOREF(rc); } else do {} while (0)
@@ -1765,7 +1811,7 @@ RT_C_DECLS_END
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertMsgRCBreakStmt(rc, msg, stmt) \
     if (1) { AssertMsgBreakStmt(RT_SUCCESS_NP(rc), msg, stmt); NOREF(rc); } else do {} while (0)
@@ -1776,7 +1822,7 @@ RT_C_DECLS_END
  * On failure it will print info about the rc and hit a breakpoint.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCSuccess(rc)                 AssertMsg((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -1785,7 +1831,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCSuccessReturn(rc, rcRet)    AssertMsgReturn((rc) == VINF_SUCCESS, ("%Rra\n", (rc)), rcRet)
 
@@ -1793,7 +1839,7 @@ RT_C_DECLS_END
  * Asserts that an iprt status code equals VINF_SUCCESS, bitch (RT_STRICT mode only) and return if it isn't.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCSuccessReturnVoid(rc)       AssertMsgReturnVoid((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -1801,7 +1847,7 @@ RT_C_DECLS_END
  * Asserts that an iprt status code equals VINF_SUCCESS, bitch (RT_STRICT mode only) and break if it isn't.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCSuccessBreak(rc)            AssertMsgBreak((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -1810,7 +1856,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times. In release mode is NOREF()'ed.
+ * @remark  rc is referenced multiple times. In release mode is NOREF()'ed.
  */
 #define AssertRCSuccessBreakStmt(rc, stmt)  AssertMsgBreakStmt((rc) == VINF_SUCCESS, ("%Rra\n", (rc)), stmt)
 
@@ -1819,7 +1865,7 @@ RT_C_DECLS_END
  * Asserts a iprt status code successful.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRC(rc)                      AssertLogRelMsgRC(rc, ("%Rra\n", (rc)))
 
@@ -1828,7 +1874,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCReturn(rc, rcRet)         AssertLogRelMsgRCReturn(rc, ("%Rra\n", (rc)), rcRet)
 
@@ -1836,7 +1882,7 @@ RT_C_DECLS_END
  * Asserts a iprt status code successful, returning (void) if it isn't.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCReturnVoid(rc)            AssertLogRelMsgRCReturnVoid(rc, ("%Rra\n", (rc)))
 
@@ -1844,7 +1890,7 @@ RT_C_DECLS_END
  * Asserts a iprt status code successful, breaking if it isn't.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCBreak(rc)                 AssertLogRelMsgRCBreak(rc, ("%Rra\n", (rc)))
 
@@ -1853,7 +1899,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCBreakStmt(rc, stmt)       AssertLogRelMsgRCBreakStmt(rc, ("%Rra\n", (rc)), stmt)
 
@@ -1862,7 +1908,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelMsgRC(rc, msg)              AssertLogRelMsg(RT_SUCCESS_NP(rc), msg)
 
@@ -1872,7 +1918,7 @@ RT_C_DECLS_END
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelMsgRCReturn(rc, msg, rcRet) AssertLogRelMsgReturn(RT_SUCCESS_NP(rc), msg, rcRet)
 
@@ -1881,7 +1927,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelMsgRCReturnVoid(rc, msg)    AssertLogRelMsgReturnVoid(RT_SUCCESS_NP(rc), msg)
 
@@ -1890,7 +1936,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelMsgRCBreak(rc, msg)         AssertLogRelMsgBreak(RT_SUCCESS(rc), msg)
 
@@ -1900,7 +1946,7 @@ RT_C_DECLS_END
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelMsgRCBreakStmt(rc, msg, stmt) AssertLogRelMsgBreakStmt(RT_SUCCESS_NP(rc), msg, stmt)
 
@@ -1908,7 +1954,7 @@ RT_C_DECLS_END
  * Asserts that an iprt status code equals VINF_SUCCESS.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCSuccess(rc)               AssertLogRelMsg((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -1917,7 +1963,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCSuccessReturn(rc, rcRet)  AssertLogRelMsgReturn((rc) == VINF_SUCCESS, ("%Rra\n", (rc)), rcRet)
 
@@ -1925,7 +1971,7 @@ RT_C_DECLS_END
  * Asserts that an iprt status code equals VINF_SUCCESS.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCSuccessReturnVoid(rc)     AssertLogRelMsgReturnVoid((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -1933,7 +1979,7 @@ RT_C_DECLS_END
  * Asserts that an iprt status code equals VINF_SUCCESS.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCSuccessBreak(rc)          AssertLogRelMsgBreak((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -1942,7 +1988,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertLogRelRCSuccessBreakStmt(rc, stmt) AssertLogRelMsgBreakStmt((rc) == VINF_SUCCESS, ("%Rra\n", (rc)), stmt)
 
@@ -1953,7 +1999,7 @@ RT_C_DECLS_END
  * On failure information about the error will be printed and a breakpoint hit.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRC(rc)                 AssertReleaseMsgRC(rc, ("%Rra\n", (rc)))
 
@@ -1965,7 +2011,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCReturn(rc, rcRet)    AssertReleaseMsgRCReturn(rc, ("%Rra\n", (rc)), rcRet)
 
@@ -1976,7 +2022,7 @@ RT_C_DECLS_END
  * and finally returning from the function if the breakpoint is somehow ignored.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCReturnVoid(rc)       AssertReleaseMsgRCReturnVoid(rc, ("%Rra\n", (rc)))
 
@@ -1987,7 +2033,7 @@ RT_C_DECLS_END
  * and finally breaking the current statement if the breakpoint is somehow ignored.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCBreak(rc)            AssertReleaseMsgRCBreak(rc, ("%Rra\n", (rc)))
 
@@ -1999,7 +2045,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCBreakStmt(rc, stmt)  AssertReleaseMsgRCBreakStmt(rc, ("%Rra\n", (rc)), stmt)
 
@@ -2010,7 +2056,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseMsgRC(rc, msg)         AssertReleaseMsg(RT_SUCCESS_NP(rc), msg)
 
@@ -2023,7 +2069,7 @@ RT_C_DECLS_END
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseMsgRCReturn(rc, msg, rcRet)    AssertReleaseMsgReturn(RT_SUCCESS_NP(rc), msg, rcRet)
 
@@ -2035,7 +2081,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseMsgRCReturnVoid(rc, msg)    AssertReleaseMsgReturnVoid(RT_SUCCESS_NP(rc), msg)
 
@@ -2047,7 +2093,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseMsgRCBreak(rc, msg)        AssertReleaseMsgBreak(RT_SUCCESS(rc), msg)
 
@@ -2060,7 +2106,7 @@ RT_C_DECLS_END
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseMsgRCBreakStmt(rc, msg, stmt)  AssertReleaseMsgBreakStmt(RT_SUCCESS_NP(rc), msg, stmt)
 
@@ -2070,7 +2116,7 @@ RT_C_DECLS_END
  * On failure information about the error will be printed and a breakpoint hit.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCSuccess(rc)                  AssertReleaseMsg((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -2082,7 +2128,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   rcRet   What is to be presented to return.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCSuccessReturn(rc, rcRet)     AssertReleaseMsgReturn((rc) == VINF_SUCCESS, ("%Rra\n", (rc)), rcRet)
 
@@ -2093,7 +2139,7 @@ RT_C_DECLS_END
  * and finally returning from the function if the breakpoint is somehow ignored.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCSuccessReturnVoid(rc)     AssertReleaseMsgReturnVoid((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -2104,7 +2150,7 @@ RT_C_DECLS_END
  * and finally breaking the current statement if the breakpoint is somehow ignored.
  *
  * @param   rc      iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCSuccessBreak(rc)         AssertReleaseMsgBreak((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
@@ -2116,7 +2162,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   stmt    Statement to execute before break in case of a failed assertion.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertReleaseRCSuccessBreakStmt(rc, stmt)   AssertReleaseMsgBreakStmt((rc) == VINF_SUCCESS, ("%Rra\n", (rc)), stmt)
 
@@ -2127,7 +2173,7 @@ RT_C_DECLS_END
  * On failure information about the error will be printed and a breakpoint hit.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertFatalRC(rc)               AssertFatalMsgRC(rc, ("%Rra\n", (rc)))
 
@@ -2138,7 +2184,7 @@ RT_C_DECLS_END
  *
  * @param   rc      iprt status code.
  * @param   msg     printf argument list (in parenthesis).
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertFatalMsgRC(rc, msg)       AssertFatalMsg(RT_SUCCESS_NP(rc), msg)
 
@@ -2148,7 +2194,7 @@ RT_C_DECLS_END
  * On failure information about the error will be printed and a breakpoint hit.
  *
  * @param   rc  iprt status code.
- * @remark  rc is references multiple times.
+ * @remark  rc is referenced multiple times.
  */
 #define AssertFatalRCSuccess(rc)        AssertFatalMsg((rc) == VINF_SUCCESS, ("%Rra\n", (rc)))
 
