@@ -45,23 +45,23 @@
 # include <sys/cdefs.h>
 #else
 
- /** @def __BEGIN_DECLS
-  * Used to start a block of function declarations which are shared
-  * between C and C++ program.
-  */
+/** @def RT_C_DECLS_BEGIN
+ * Used to start a block of function declarations which are shared
+ * between C and C++ program.
+ */
 
- /** @def __END_DECLS
-  * Used to end a block of function declarations which are shared
-  * between C and C++ program.
-  */
+/** @def RT_C_DECLS_END
+ * Used to end a block of function declarations which are shared
+ * between C and C++ program.
+ */
 
- #if defined(__cplusplus)
- # define __BEGIN_DECLS extern "C" {
- # define __END_DECLS   }
- #else
- # define __BEGIN_DECLS
- # define __END_DECLS
- #endif
+# if defined(__cplusplus)
+#  define RT_C_DECLS_BEGIN extern "C" {
+#  define RT_C_DECLS_END   }
+# else
+#  define RT_C_DECLS_BEGIN
+#  define RT_C_DECLS_END
+# endif
 
 #endif
 
@@ -85,6 +85,7 @@
 #define RT_STRICT
 #define Breakpoint
 #define RT_NO_DEPRECATED_MACROS
+#define RT_EXCEPTIONS_ENABLED
 #endif /* DOXYGEN_RUNNING */
 
 /** @def RT_ARCH_X86
@@ -473,6 +474,62 @@
 #endif
 
 
+/** @def RT_NOTHING
+ * A macro that expands to nothing.
+ * This is primarily intended as a dummy argument for macros to avoid the
+ * undefined behavior passing empty arguments to an macro (ISO C90 and C++98,
+ * gcc v4.4 warns about it).
+ */
+#define RT_NOTHING
+
+/** @def RT_EXCEPTIONS_ENABLED
+ * Defined when C++ exceptions are enabled.
+ */
+#if !defined(RT_EXCEPTIONS_ENABLED) \
+ &&  defined(__cplusplus) \
+ && (   (defined(_MSC_VER) && defined(_CPPUNWIND)) \
+     || (defined(__GNUC__) && defined(__EXCEPTIONS)))
+# define RT_EXCEPTIONS_ENABLED
+#endif
+
+/** @def RT_NO_THROW
+ * How to express that a function doesn't throw C++ exceptions
+ * and the compiler can thus save itself the bother of trying
+ * to catch any of them. Put this between the closing parenthesis
+ * and the semicolon in function prototypes (and implementation if C++).
+ */
+#ifdef RT_EXCEPTIONS_ENABLED
+# define RT_NO_THROW            throw()
+#else
+# define RT_NO_THROW
+#endif
+
+/** @def RT_THROW
+ * How to express that a method or function throws a type of exceptions. Some
+ * compilers does not want this kind of information and will warning about it.
+ *
+ * @param   type    The type exception.
+ *
+ * @remarks If the actual throwing is done from the header, enclose it by
+ *          \#ifdef RT_EXCEPTIONS_ENABLED ... \#else ... \#endif so the header
+ *          compiles cleanly without exceptions enabled.
+ *
+ *          Do NOT use this for the actual throwing of exceptions!
+ */
+#ifdef RT_EXCEPTIONS_ENABLED
+# ifdef _MSC_VER
+#  if _MSC_VER >= 1400
+#   define RT_THROW(type)
+#  else
+#   define RT_THROW(type)       throw(type)
+#  endif
+# else
+#  define RT_THROW(type)        throw(type)
+# endif
+#else
+# define RT_THROW(type)
+#endif
+
 /** @def RTCALL
  * The standard calling convention for the Runtime interfaces.
  */
@@ -484,20 +541,6 @@
 # define RTCALL
 #endif
 
-/** @def RT_NO_THROW
- * How to express that a function doesn't throw C++ exceptions
- * and the compiler can thus save itself the bother of trying
- * to catch any of them. Put this between the closing parenthesis
- * and the semicolon in function prototypes (and implementation if C++).
- */
-#if defined(__cplusplus) \
- && (   (defined(_MSC_VER) && defined(_CPPUNWIND)) \
-     || (defined(__GNUC__) && defined(__EXCEPTIONS)))
-# define RT_NO_THROW    throw()
-#else
-# define RT_NO_THROW
-#endif
-
 /** @def DECLEXPORT
  * How to declare an exported function.
  * @param   type    The return type of the function declaration.
@@ -530,6 +573,20 @@
 # define DECLHIDDEN(type)       __attribute__((visibility("hidden"))) type
 #endif
 
+/** @def DECL_INVALID
+ * How to declare a function not available for linking in the current context.
+ * The purpose is to create compile or like time errors when used.  This isn't
+ * possible on all platforms.
+ * @param   type    The return type of the function.
+ */
+#if defined(_MSC_VER)
+# define DECL_INVALID(type)     __declspec(dllimport) type __stdcall
+#elif defined(__GNUC__) && defined(__cplusplus)
+# define DECL_INVALID(type)     extern "C++" type
+#else
+# define DECL_INVALID(type)     type
+#endif
+
 /** @def DECLASM
  * How to declare an internal assembly function.
  * @param   type    The return type of the function declaration.
@@ -647,6 +704,36 @@
 #endif
 
 
+/** @def DECL_FORCE_INLINE
+ * How to declare a function as inline and try convince the compiler to always
+ * inline it regardless of optimization switches.
+ * @param   type    The return type of the function declaration.
+ * @remarks Use sparsely and with care. Don't use this macro on C++ methods.
+ */
+#ifdef __GNUC__
+# define DECL_FORCE_INLINE(type)    __attribute__((always_inline)) DECLINLINE(type)
+#elif defined(_MSC_VER)
+# define DECL_FORCE_INLINE(type)    __forceinline type
+#else
+# define DECL_FORCE_INLINE(type)    DECLINLINE(type)
+#endif
+
+
+/** @def DECL_NO_INLINE
+ * How to declare a function telling the compiler not to inline it.
+ * @param   scope   The function scope, static or RT_NOTHING.
+ * @param   type    The return type of the function declaration.
+ * @remarks Don't use this macro on C++ methods.
+ */
+#ifdef __GNUC__
+# define DECL_NO_INLINE(scope,type) __attribute__((noinline)) scope type
+#elif defined(_MSC_VER)
+# define DECL_NO_INLINE(scope,type) __declspec(noline) scope type
+#else
+# define DECL_NO_INLINE(scope,type) scope type
+#endif
+
+
 /** @def IN_RT_STATIC
  * Used to indicate whether we're linking against a static IPRT
  * or not. The IPRT symbols will be declared as hidden (if
@@ -738,6 +825,19 @@
 # define RTDATADECL(type)   DECLIMPORT(type)
 #endif
 
+/** @def RT_DECL_CLASS
+ * Declares an class living in the runtime.
+ */
+#if defined(IN_RT_R3) || defined(IN_RT_GC) || defined(IN_RT_R0)
+# ifdef IN_RT_STATIC
+#  define RT_DECL_CLASS
+# else
+#  define RT_DECL_CLASS     DECLEXPORT_CLASS
+# endif
+#else
+# define RT_DECL_CLASS      DECLIMPORT_CLASS
+#endif
+
 
 /** @def RT_NOCRT
  * Symbol name wrapper for the No-CRT bits.
@@ -823,23 +923,33 @@
 #endif
 
 
+/** @def RT_STR
+ * Returns the argument as a string constant.
+ * @param   str     Argument to stringify.  */
+#define RT_STR(str)             #str
+/** @def RT_XSTR
+ * Returns the expanded argument as a string.
+ * @param   str     Argument to expand and stringy. */
+#define RT_XSTR(str)            RT_STR(str)
+
+
 /** @def RT_BIT
- * Make a bitmask for one integer sized bit.
- * @param   bit     Bit number.
+ * Convert a bit number into an integer bitmask (unsigned).
+ * @param   bit     The bit number.
  */
-#define RT_BIT(bit)                     (1U << (bit))
+#define RT_BIT(bit)                             ( 1U << (bit) )
 
 /** @def RT_BIT_32
- * Make a 32-bit bitmask for one bit.
- * @param   bit     Bit number.
+ * Convert a bit number into a 32-bit bitmask (unsigned).
+ * @param   bit     The bit number.
  */
-#define RT_BIT_32(bit)                  (UINT32_C(1) << (bit))
+#define RT_BIT_32(bit)                          ( UINT32_C(1) << (bit) )
 
 /** @def RT_BIT_64
- * Make a 64-bit bitmask for one bit.
- * @param   bit     Bit number.
+ * Convert a bit number into a 64-bit bitmask (unsigned).
+ * @param   bit     The bit number.
  */
-#define RT_BIT_64(bit)                  (UINT64_C(1) << (bit))
+#define RT_BIT_64(bit)                          ( UINT64_C(1) << (bit) )
 
 /** @def RT_ALIGN
  * Align macro.
@@ -855,7 +965,7 @@
  *
  *          In short: Don't use this macro. Use RT_ALIGN_T() instead.
  */
-#define RT_ALIGN(u, uAlignment)         ( ((u) + ((uAlignment) - 1)) & ~((uAlignment) - 1) )
+#define RT_ALIGN(u, uAlignment)                 ( ((u) + ((uAlignment) - 1)) & ~((uAlignment) - 1) )
 
 /** @def RT_ALIGN_T
  * Align macro.
@@ -864,7 +974,7 @@
  * @param   type        Integer type to use while aligning.
  * @remark  This macro is the preferred alignment macro, it doesn't have any of the pitfalls RT_ALIGN has.
  */
-#define RT_ALIGN_T(u, uAlignment, type) ( ((type)(u) + ((uAlignment) - 1)) & ~(type)((uAlignment) - 1) )
+#define RT_ALIGN_T(u, uAlignment, type)         ( ((type)(u) + ((uAlignment) - 1)) & ~(type)((uAlignment) - 1) )
 
 /** @def RT_ALIGN_32
  * Align macro for a 32-bit value.
@@ -900,7 +1010,7 @@
  * @param   uAlignment  The alignment. Power of two!
  * @param   CastType    The type to cast the result to.
  */
-#define RT_ALIGN_PT(u, uAlignment, CastType)    ((CastType)RT_ALIGN_T(u, uAlignment, uintptr_t))
+#define RT_ALIGN_PT(u, uAlignment, CastType)    ( (CastType)RT_ALIGN_T(u, uAlignment, uintptr_t) )
 
 /** @def RT_ALIGN_R3PT
  * Align macro for ring-3 pointers with type cast.
@@ -908,7 +1018,7 @@
  * @param   uAlignment  The alignment. Power of two!
  * @param   CastType    The type to cast the result to.
  */
-#define RT_ALIGN_R3PT(u, uAlignment, CastType)  ((CastType)RT_ALIGN_T(u, uAlignment, RTR3UINTPTR))
+#define RT_ALIGN_R3PT(u, uAlignment, CastType)  ( (CastType)RT_ALIGN_T(u, uAlignment, RTR3UINTPTR) )
 
 /** @def RT_ALIGN_R0PT
  * Align macro for ring-0 pointers with type cast.
@@ -916,7 +1026,7 @@
  * @param   uAlignment  The alignment. Power of two!
  * @param   CastType    The type to cast the result to.
  */
-#define RT_ALIGN_R0PT(u, uAlignment, CastType)  ((CastType)RT_ALIGN_T(u, uAlignment, RTR0UINTPTR))
+#define RT_ALIGN_R0PT(u, uAlignment, CastType)  ( (CastType)RT_ALIGN_T(u, uAlignment, RTR0UINTPTR) )
 
 /** @def RT_ALIGN_GCPT
  * Align macro for GC pointers with type cast.
@@ -924,7 +1034,7 @@
  * @param   uAlignment  The alignment. Power of two!
  * @param   CastType        The type to cast the result to.
  */
-#define RT_ALIGN_GCPT(u, uAlignment, CastType)  ((CastType)RT_ALIGN_T(u, uAlignment, RTGCUINTPTR))
+#define RT_ALIGN_GCPT(u, uAlignment, CastType)  ( (CastType)RT_ALIGN_T(u, uAlignment, RTGCUINTPTR) )
 
 
 /** @def RT_OFFSETOF
@@ -939,7 +1049,7 @@
  * @param   type    Structure type.
  * @param   member  Member.
  */
-#define RT_OFFSETOF(type, member)   ( (int)(uintptr_t)&( ((type *)(void *)0)->member) )
+#define RT_OFFSETOF(type, member)               ( (int)(uintptr_t)&( ((type *)(void *)0)->member) )
 
 /** @def RT_UOFFSETOF
  * Our own special offsetof() variant, returns an unsigned result.
@@ -953,7 +1063,27 @@
  * @param   type    Structure type.
  * @param   member  Member.
  */
-#define RT_UOFFSETOF(type, member)   ( (uintptr_t)&( ((type *)(void *)0)->member) )
+#define RT_UOFFSETOF(type, member)              ( (uintptr_t)&( ((type *)(void *)0)->member) )
+
+/** @def RT_OFFSETOF_ADD
+ * RT_OFFSETOF with an addend.
+ *
+ * @returns offset into the structure of the specified member. signed.
+ * @param   type    Structure type.
+ * @param   member  Member.
+ * @param   addend  The addend to add to the offset.
+ */
+#define RT_OFFSETOF_ADD(type, member, addend)   ( (int)RT_UOFFSETOF_ADD(type, member, addend) )
+
+/** @def RT_UOFFSETOF_ADD
+ * RT_UOFFSETOF with an addend.
+ *
+ * @returns offset into the structure of the specified member. signed.
+ * @param   type    Structure type.
+ * @param   member  Member.
+ * @param   addend  The addend to add to the offset.
+ */
+#define RT_UOFFSETOF_ADD(type, member, addend)  ( (uintptr_t)&( ((type *)(void *)(uintptr_t)(addend))->member) )
 
 /** @def RT_SIZEOFMEMB
  * Get the size of a structure member.
@@ -962,14 +1092,23 @@
  * @param   type    Structure type.
  * @param   member  Member.
  */
-#define RT_SIZEOFMEMB(type, member) ( sizeof(((type *)(void *)0)->member) )
+#define RT_SIZEOFMEMB(type, member)             ( sizeof(((type *)(void *)0)->member) )
+
+/** @def RT_FROM_MEMBER
+ * Convert a pointer to a structure member into a pointer to the structure.
+ * @returns pointer to the structure.
+ * @param   pMember Pointer to the member.
+ * @param   Type    Strucutre type.
+ * @param   Member  Member name.
+ */
+#define RT_FROM_MEMBER(pMem, Type, Member)      ( (Type *) ((uint8_t *)(void *)(pMem) - RT_UOFFSETOF(Type, Member)) )
 
 /** @def RT_ELEMENTS
- * Calcs the number of elements in an array.
+ * Calculates the number of elements in a statically sized array.
  * @returns Element count.
  * @param   aArray      Array in question.
  */
-#define RT_ELEMENTS(aArray)         ( sizeof(aArray) / sizeof((aArray)[0]) )
+#define RT_ELEMENTS(aArray)                     ( sizeof(aArray) / sizeof((aArray)[0]) )
 
 #ifdef RT_OS_OS2
 /* Undefine RT_MAX since there is an unfortunate clash with the max
@@ -983,7 +1122,7 @@
  * @param   Value1      Value 1
  * @param   Value2      Value 2
  */
-#define RT_MAX(Value1, Value2)  ((Value1) >= (Value2) ? (Value1) : (Value2))
+#define RT_MAX(Value1, Value2)                  ( (Value1) >= (Value2) ? (Value1) : (Value2) )
 
 /** @def RT_MIN
  * Finds the minimum value.
@@ -991,60 +1130,69 @@
  * @param   Value1      Value 1
  * @param   Value2      Value 2
  */
-#define RT_MIN(Value1, Value2)  ((Value1) <= (Value2) ? (Value1) : (Value2))
+#define RT_MIN(Value1, Value2)                  ( (Value1) <= (Value2) ? (Value1) : (Value2) )
+
+/** @def RT_CLAMP
+ * Clamps the value to minimum and maximum values.
+ * @returns The clamped value.
+ * @param   Value       The value to check.
+ * @param   Min         Minimum value.
+ * @param   Max         Maximum value.
+ */
+#define RT_CLAMP(Value, Min, Max)               ( ((Value) > (Max)) ? (Max) : (((Value) < (Min)) ? (Min) : (Value)) )
 
 /** @def RT_ABS
  * Get the absolute (non-negative) value.
  * @returns The absolute value of Value.
  * @param   Value       The value.
  */
-#define RT_ABS(Value)           ((Value) >= 0 ? (Value) : -(Value))
+#define RT_ABS(Value)                           ( (Value) >= 0 ? (Value) : -(Value) )
 
 /** @def RT_LODWORD
  * Gets the low dword (=uint32_t) of something. */
-#define RT_LODWORD(a)           ( (uint32_t)(a) )
+#define RT_LODWORD(a)                           ( (uint32_t)(a) )
 
 /** @def RT_HIDWORD
  * Gets the high dword (=uint32_t) of a 64-bit of something. */
-#define RT_HIDWORD(a)           ( (uint32_t)((a) >> 32) )
+#define RT_HIDWORD(a)                           ( (uint32_t)((a) >> 32) )
 
 /** @def RT_LOWORD
  * Gets the low word (=uint16_t) of something. */
-#define RT_LOWORD(a)            ((a) & 0xffff)
+#define RT_LOWORD(a)                            ( (a) & 0xffff )
 
 /** @def RT_HIWORD
  * Gets the high word (=uint16_t) of a 32-bit something. */
-#define RT_HIWORD(a)            ((a) >> 16)
+#define RT_HIWORD(a)                            ( (a) >> 16 )
 
 /** @def RT_LOBYTE
  * Gets the low byte of something. */
-#define RT_LOBYTE(a)            ((a) & 0xff)
+#define RT_LOBYTE(a)                            ( (a) & 0xff )
 
 /** @def RT_HIBYTE
  * Gets the low byte of a 16-bit something. */
-#define RT_HIBYTE(a)            ((a) >> 8)
+#define RT_HIBYTE(a)                            ( (a) >> 8 )
 
 /** @def RT_BYTE1
  * Gets first byte of something. */
-#define RT_BYTE1(a)             ((a) & 0xff)
+#define RT_BYTE1(a)                             ( (a) & 0xff )
 
 /** @def RT_BYTE2
  * Gets second byte of something. */
-#define RT_BYTE2(a)             (((a) >> 8) & 0xff)
+#define RT_BYTE2(a)                             ( ((a) >> 8) & 0xff )
 
 /** @def RT_BYTE3
  * Gets second byte of something. */
-#define RT_BYTE3(a)             (((a) >> 16) & 0xff)
+#define RT_BYTE3(a)                             ( ((a) >> 16) & 0xff )
 
 /** @def RT_BYTE4
  * Gets fourth byte of something. */
-#define RT_BYTE4(a)             (((a) >> 24) & 0xff)
+#define RT_BYTE4(a)                             ( ((a) >> 24) & 0xff )
 
 
 /** @def RT_MAKE_U64
  * Constructs a uint64_t value from two uint32_t values.
  */
-#define RT_MAKE_U64(Lo, Hi) ( (uint64_t)((uint32_t)(Hi)) << 32 | (uint32_t)(Lo) )
+#define RT_MAKE_U64(Lo, Hi)                     ( (uint64_t)((uint32_t)(Hi)) << 32 | (uint32_t)(Lo) )
 
 /** @def RT_MAKE_U64_FROM_U16
  * Constructs a uint64_t value from four uint16_t values.
@@ -1438,7 +1586,7 @@
 #define NIL_OFFSET   (~0U)
 
 /** @def NOREF
- * Keeps the compiler from bitching about an unused parameters.
+ * Keeps the compiler from bitching about an unused parameter.
  */
 #define NOREF(var)               (void)(var)
 
@@ -1462,7 +1610,7 @@
 #endif
 
 
-/** Size Constants
+/** @defgroup grp_rt_cdefs_size     Size Constants
  * (Of course, these are binary computer terms, not SI.)
  * @{
  */
@@ -1504,34 +1652,95 @@
 #define _2E             0x2000000000000000ULL
 /** @} */
 
+
+/** @defgroup grp_rt_cdefs_dbgtype  Debug Info Types
+ * @{ */
+/** Other format. */
+#define RT_DBGTYPE_OTHER        RT_BIT_32(0)
+/** Stabs. */
+#define RT_DBGTYPE_STABS        RT_BIT_32(1)
+/** Debug With Arbitrary Record Format (DWARF). */
+#define RT_DBGTYPE_DWARF        RT_BIT_32(2)
+/** Microsoft Codeview debug info. */
+#define RT_DBGTYPE_CODEVIEW     RT_BIT_32(3)
+/** Watcom debug info. */
+#define RT_DBGTYPE_WATCOM       RT_BIT_32(4)
+/** IBM High Level Language debug info. */
+#define RT_DBGTYPE_HLL          RT_BIT_32(5)
+/** Old OS/2 and Windows symbol file. */
+#define RT_DBGTYPE_SYM          RT_BIT_32(6)
+/** Map file. */
+#define RT_DBGTYPE_MAP          RT_BIT_32(7)
+/** @} */
+
+
+/** @defgroup grp_rt_cdefs_exetype  Executable Image Types
+ * @{ */
+/** Some other format. */
+#define RT_EXETYPE_OTHER        RT_BIT_32(0)
+/** Portable Executable. */
+#define RT_EXETYPE_PE           RT_BIT_32(1)
+/** Linear eXecutable. */
+#define RT_EXETYPE_LX           RT_BIT_32(2)
+/** Linear Executable. */
+#define RT_EXETYPE_LE           RT_BIT_32(3)
+/** New Executable. */
+#define RT_EXETYPE_NE           RT_BIT_32(4)
+/** DOS Executable (Mark Zbikowski). */
+#define RT_EXETYPE_MZ           RT_BIT_32(5)
+/** COM Executable. */
+#define RT_EXETYPE_COM          RT_BIT_32(6)
+/** a.out Executable. */
+#define RT_EXETYPE_AOUT         RT_BIT_32(7)
+/** Executable and Linkable Format. */
+#define RT_EXETYPE_ELF          RT_BIT_32(8)
+/** Mach-O Executable (including FAT ones). */
+#define RT_EXETYPE_MACHO        RT_BIT_32(9)
+/** TE from UEFI. */
+#define RT_EXETYPE_TE           RT_BIT_32(9)
+/** @} */
+
+
 /** @def VALID_PTR
  * Pointer validation macro.
- * @param   ptr
+ * @param   ptr         The pointer.
  */
 #if defined(RT_ARCH_AMD64)
 # ifdef IN_RING3
 #  if defined(RT_OS_DARWIN) /* first 4GB is reserved for legacy kernel. */
-#   define VALID_PTR(ptr)   (   (uintptr_t)(ptr) >= _4G \
-                             && !((uintptr_t)(ptr) & 0xffff800000000000ULL) )
+#   define RT_VALID_PTR(ptr)    (   (uintptr_t)(ptr) >= _4G \
+                                 && !((uintptr_t)(ptr) & 0xffff800000000000ULL) )
 #  elif defined(RT_OS_SOLARIS) /* The kernel only used the top 2TB, but keep it simple. */
-#   define VALID_PTR(ptr)   (   (uintptr_t)(ptr) + 0x1000U >= 0x2000U \
-                             && (   ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0xffff800000000000ULL \
-                                 || ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0) )
+#   define RT_VALID_PTR(ptr)    (   (uintptr_t)(ptr) + 0x1000U >= 0x2000U \
+                                 && (   ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0xffff800000000000ULL \
+                                     || ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0) )
 #  else
-#   define VALID_PTR(ptr)   (   (uintptr_t)(ptr) + 0x1000U >= 0x2000U \
-                             && !((uintptr_t)(ptr) & 0xffff800000000000ULL) )
+#   define RT_VALID_PTR(ptr)    (   (uintptr_t)(ptr) + 0x1000U >= 0x2000U \
+                                 && !((uintptr_t)(ptr) & 0xffff800000000000ULL) )
 #  endif
 # else /* !IN_RING3 */
-#  define VALID_PTR(ptr)    (   (uintptr_t)(ptr) + 0x1000U >= 0x2000U \
-                             && (   ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0xffff800000000000ULL \
-                                 || ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0) )
+#  define RT_VALID_PTR(ptr)     (   (uintptr_t)(ptr) + 0x1000U >= 0x2000U \
+                                 && (   ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0xffff800000000000ULL \
+                                     || ((uintptr_t)(ptr) & 0xffff800000000000ULL) == 0) )
 # endif /* !IN_RING3 */
 #elif defined(RT_ARCH_X86)
-# define VALID_PTR(ptr)     ( (uintptr_t)(ptr) + 0x1000U >= 0x2000U )
+# define RT_VALID_PTR(ptr)      ( (uintptr_t)(ptr) + 0x1000U >= 0x2000U )
 #else
 # error "Architecture identifier missing / not implemented."
 #endif
 
+/** Old name for RT_VALID_PTR.  */
+#define VALID_PTR(ptr)      RT_VALID_PTR(ptr)
+
+/** @def RT_VALID_ALIGNED_PTR
+ * Pointer validation macro that also checks the alignment.
+ * @param   ptr         The pointer.
+ * @param   align       The alignment, must be a power of two.
+ */
+#define RT_VALID_ALIGNED_PTR(ptr, align)   \
+    (   !((uintptr_t)(ptr) & (uintptr_t)((align) - 1)) \
+     && VALID_PTR(ptr) )
+
 
 /** @def VALID_PHYS32
  * 32 bits physical address validation macro.
@@ -1540,26 +1749,27 @@
 #define VALID_PHYS32(Phys)  ( (uint64_t)(Phys) < (uint64_t)_4G )
 
 /** @def N_
- * The \#define N_ is used mark a string for translation. This is usable in
+ * The \#define N_ is used to mark a string for translation. This is usable in
  * any part of the code, as it is only used by the tools that create message
  * catalogs. This macro is a no-op as far as the compiler and code generation
  * is concerned.
  *
- * If you want to both mark a string for translation and translate it, use _.
+ * If you want to both mark a string for translation and translate it, use _().
  */
 #define N_(s) (s)
 
 /** @def _
- * The \#define _ is used mark a string for translation and to translate it in
- * one step.
+ * The \#define _ is used to mark a string for translation and to translate it
+ * in one step.
  *
- * If you want to only mark a string for translation, use N_.
+ * If you want to only mark a string for translation, use N_().
  */
 #define _(s) gettext(s)
 
 
 /** @def __PRETTY_FUNCTION__
- *  With GNU C we'd like to use the builtin __PRETTY_FUNCTION__, so define that for the other compilers.
+ *  With GNU C we'd like to use the builtin __PRETTY_FUNCTION__, so define that
+ *  for the other compilers.
  */
 #if !defined(__GNUC__) && !defined(__PRETTY_FUNCTION__)
 # define __PRETTY_FUNCTION__    __FUNCTION__
@@ -1567,10 +1777,10 @@
 
 
 /** @def RT_STRICT
- * The \#define RT_STRICT controls whether or not assertions and other runtime checks
- * should be compiled in or not.
+ * The \#define RT_STRICT controls whether or not assertions and other runtime
+ * checks should be compiled in or not.
  *
- * If you want assertions which are not a subject to compile time options use
+ * If you want assertions which are not subject to compile time options use
  * the AssertRelease*() flavors.
  */
 #if !defined(RT_STRICT) && defined(DEBUG)
