@@ -34,7 +34,7 @@
 #include <iprt/types.h>
 #include <iprt/stdarg.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 /** @defgroup grp_rt_stream     RTStrm - File Streams
  * @ingroup grp_rt
@@ -65,6 +65,30 @@ extern RTDATADECL(PRTSTREAM)    g_pStdOut;
  */
 RTR3DECL(int) RTStrmOpen(const char *pszFilename, const char *pszMode, PRTSTREAM *ppStream);
 
+/**
+ * Opens a file stream.
+ *
+ * @returns iprt status code.
+ * @param   pszMode         The open mode. See fopen() standard.
+ *                          Format: <a|r|w>[+][b|t]
+ * @param   ppStream        Where to store the opened stream.
+ * @param   pszFilenameFmt  Filename path format string.
+ * @param   args            Arguments to the format string.
+ */
+RTR3DECL(int) RTStrmOpenFV(const char *pszMode, PRTSTREAM *ppStream, const char *pszFilenameFmt, va_list args);
+
+/**
+ * Opens a file stream.
+ *
+ * @returns iprt status code.
+ * @param   pszMode         The open mode. See fopen() standard.
+ *                          Format: <a|r|w>[+][b|t]
+ * @param   ppStream        Where to store the opened stream.
+ * @param   pszFilenameFmt  Filename path format string.
+ * @param   ...             Arguments to the format string.
+ */
+RTR3DECL(int) RTStrmOpenF(const char *pszMode, PRTSTREAM *ppStream, const char *pszFilenameFmt, ...);
+
 /**
  * Closes the specified stream.
  *
@@ -73,6 +97,39 @@ RTR3DECL(int) RTStrmOpen(const char *pszFilename, const char *pszMode, PRTSTREAM
  */
 RTR3DECL(int) RTStrmClose(PRTSTREAM pStream);
 
+/**
+ * Get the pending error of the stream.
+ *
+ * @returns iprt status code. of the stream.
+ * @param   pStream         The stream.
+ */
+RTR3DECL(int) RTStrmError(PRTSTREAM pStream);
+
+/**
+ * Clears stream error condition.
+ *
+ * All stream operations save RTStrmClose and this will fail
+ * while an error is asserted on the stream
+ *
+ * @returns iprt status code.
+ * @param   pStream         The stream.
+ */
+RTR3DECL(int) RTStrmClearError(PRTSTREAM pStream);
+
+/**
+ * Rewinds the stream.
+ *
+ * Stream errors will be reset on success.
+ *
+ * @returns IPRT status code.
+ *
+ * @param   pStream         The stream.
+ *
+ * @remarks Not all streams are rewindable and that behavior is currently
+ *          undefined for those.
+ */
+RTR3DECL(int) RTStrmRewind(PRTSTREAM pStream);
+
 /**
  * Reads from a file stream.
  *
@@ -215,7 +272,7 @@ RTR3DECL(int) RTPrintfV(const char *pszFormat, va_list args);
 
 /** @} */
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
