@@ -33,6 +33,7 @@
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
 #include <iprt/thread.h>
+#include <iprt/net.h>
 
 #ifdef IN_RING0
 # error "There are no RTFile APIs available Ring-0 Host Context!"
@@ -131,6 +132,14 @@ RTR3DECL(int) RTTcpServerListen(PRTTCPSERVER pServer, PFNRTTCPSERVE pfnServe, vo
  */
 RTR3DECL(int) RTTcpServerDisconnectClient(PRTTCPSERVER pServer);
 
+/**
+ * Shuts down the server, leaving client connections open.
+ *
+ * @returns IPRT status code.
+ * @param   pServer         Handle to the server.
+ */
+RTR3DECL(int) RTTcpServerShutdown(PRTTCPSERVER pServer);
+
 /**
  * Connect (as a client) to a TCP Server.
  *
@@ -215,6 +224,23 @@ RTR3DECL(int)  RTTcpAccept(RTSOCKET Sock, unsigned uPort, PRTSOCKET pSockAccepte
 
 #endif
 
+/**
+ * Gets the address of the local side.
+ *
+ * @returns IPRT status code.
+ * @param   Sock            Socket descriptor.
+ * @param   pAddr           Where to store the local address on success.
+ */
+RTR3DECL(int) RTTcpGetLocalAddress(RTSOCKET Sock, PRTNETADDR pAddr);
+
+/**
+ * Gets the address of the other party.
+ *
+ * @returns IPRT status code.
+ * @param   Sock            Socket descriptor.
+ * @param   pAddr           Where to store the peer address on success.
+ */
+RTR3DECL(int) RTTcpGetPeerAddress(RTSOCKET Sock, PRTNETADDR pAddr);
 
 /** @} */
 RT_C_DECLS_END
