@@ -33,7 +33,7 @@
 #include <iprt/cdefs.h>
 #include <iprt/types.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 /** @defgroup grp_rt_time   RTTime - Time
  * @ingroup grp_rt
@@ -354,7 +354,7 @@ DECLINLINE(struct timeval *) RTTimeSpecGetTimeval(PCRTTIMESPEC pTime, struct tim
     if (i32Micro < 0)
     {
         i32Micro += 1000000;
-        i64++;
+        i64--;
     }
     pTimeval->tv_sec = (time_t)i64;
     pTimeval->tv_usec = i32Micro;
@@ -705,6 +705,15 @@ RTDECL(PRTTIME) RTTimeLocalNormalize(PRTTIME pTime);
  */
 RTDECL(char *) RTTimeToString(PCRTTIME pTime, char *psz, size_t cb);
 
+/**
+ * Checks if a year is a leap year or not.
+ *
+ * @returns true if it's a leap year.
+ * @returns false if it's a common year.
+ * @param   i32Year     The year in question.
+ */
+RTDECL(bool) RTTimeIsLeapYear(int32_t i32Year);
+
 /**
  * Gets the current nanosecond timestamp.
  *
@@ -923,7 +932,7 @@ RTDECL(uint64_t) RTTimeProgramStartNanoTS(void);
 
 /** @} */
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
