@@ -58,7 +58,7 @@
 #  ifndef _STDDEF_H_
 #   undef offsetof
 #  endif
-#  include <stddef.h>
+#  include <sys/stddef.h>
 #  ifndef _SYS_TYPES_H_
 #   undef offsetof
 #  endif
@@ -67,6 +67,19 @@
 #   error "offsetof is not defined..."
 #  endif
 
+# elif defined(RT_OS_FREEBSD) && HC_ARCH_BITS == 64 && defined(RT_ARCH_X86)
+    /*
+     * Kludge for compiling 32-bit code on a 64-bit FreeBSD:
+     *  FreeBSD declares uint64_t and int64_t wrong (long unsigned and long int
+     *  though they need to be long long unsigned and long long int). These
+     *  defines conflict with our decleration in stdint.h. Adding the defines
+     *  below omits the definitions in the system header.
+     */
+#  include <stddef.h>
+#  define _UINT64_T_DECLARED
+#  define _INT64_T_DECLARED
+#  include <sys/types.h>
+
 # elif defined(RT_OS_LINUX) && defined(__KERNEL__)
     /*
      * Kludge for the linux kernel:
@@ -84,7 +97,9 @@
 #  define true linux_true
 #  define false linux_false
 #  define uintptr_t linux_uintptr_t
-#  include <linux/autoconf.h>
+#  ifndef AUTOCONF_INCLUDED
+#   include <linux/autoconf.h>
+#  endif
 #  include <linux/types.h>
 #  include <linux/stddef.h>
 #  undef uintptr_t
@@ -103,7 +118,7 @@
    typedef intptr_t ssize_t;
 # endif
 
-#else /* no crt */
+#else  /* no crt */
 # include <iprt/nocrt/compiler/compiler.h>
 #endif /* no crt */
 
@@ -187,12 +202,12 @@ typedef union RTUINT16U
     /** 16-bit view. */
     uint16_t    au16[1];
     /** 8-bit view. */
-    uint8_t     au8[4];
+    uint8_t     au8[2];
     /** 16-bit hi/lo view. */
     struct
     {
-        uint16_t    Lo;
-        uint16_t    Hi;
+        uint8_t    Lo;
+        uint8_t    Hi;
     } s;
 } RTUINT16U;
 /** Pointer to a 16-bit unsigned integer union. */
@@ -387,6 +402,14 @@ typedef uint64_t        RTUINTPTR;
 typedef RTUINTPTR      *PRTUINTPTR;
 /** Pointer const to unsigned integer which can contain both GC and HC pointers. */
 typedef const RTUINTPTR *PCRTUINTPTR;
+/** The maximum value the RTUINTPTR type can hold. */
+#if (HC_ARCH_BITS == 32 && GC_ARCH_BITS == 32)
+# define RTUINTPTR_MAX  UINT32_MAX
+#elif (HC_ARCH_BITS == 64 || GC_ARCH_BITS == 64)
+# define RTUINTPTR_MAX  UINT64_MAX
+#else
+#  error Unsupported HC_ARCH_BITS and/or GC_ARCH_BITS values.
+#endif
 
 /** Signed integer. */
 typedef int32_t         RTINT;
@@ -507,163 +530,226 @@ typedef const RTFAR64 *PCRTFAR64;
 
 /** HC Natural signed integer.
  * @deprecated silly type. */
-typedef int32_t         RTHCINT;
+typedef int32_t             RTHCINT;
 /** Pointer to HC Natural signed integer.
  * @deprecated silly type. */
-typedef RTHCINT        *PRTHCINT;
+typedef RTHCINT            *PRTHCINT;
 /** Pointer to const HC Natural signed integer.
  * @deprecated silly type. */
-typedef const RTHCINT  *PCRTHCINT;
+typedef const RTHCINT      *PCRTHCINT;
 
 /** HC Natural unsigned integer.
  * @deprecated silly type. */
-typedef uint32_t        RTHCUINT;
+typedef uint32_t            RTHCUINT;
 /** Pointer to HC Natural unsigned integer.
  * @deprecated silly type. */
-typedef RTHCUINT       *PRTHCUINT;
+typedef RTHCUINT           *PRTHCUINT;
 /** Pointer to const HC Natural unsigned integer.
  * @deprecated silly type. */
-typedef const RTHCUINT *PCRTHCUINT;
+typedef const RTHCUINT     *PCRTHCUINT;
 
 
 /** Signed integer which can contain a HC pointer. */
 #if HC_ARCH_BITS == 32
-typedef int32_t         RTHCINTPTR;
+typedef int32_t             RTHCINTPTR;
 #elif HC_ARCH_BITS == 64
-typedef int64_t         RTHCINTPTR;
+typedef int64_t             RTHCINTPTR;
 #else
-#  error Unsupported HC_ARCH_BITS value.
+# error Unsupported HC_ARCH_BITS value.
 #endif
 /** Pointer to signed integer which can contain a HC pointer. */
-typedef RTHCINTPTR     *PRTHCINTPTR;
+typedef RTHCINTPTR         *PRTHCINTPTR;
 /** Pointer to const signed integer which can contain a HC pointer. */
-typedef const RTHCINTPTR *PCRTHCINTPTR;
+typedef const RTHCINTPTR   *PCRTHCINTPTR;
+/** Max RTHCINTPTR value. */
+#if HC_ARCH_BITS == 32
+# define RTHCINTPTR_MAX     INT32_MAX
+#else
+# define RTHCINTPTR_MAX     INT64_MAX
+#endif
+/** Min RTHCINTPTR value. */
+#if HC_ARCH_BITS == 32
+# define RTHCINTPTR_MIN     INT32_MIN
+#else
+# define RTHCINTPTR_MIN     INT64_MIN
+#endif
 
 /** Signed integer which can contain a HC ring-3 pointer. */
 #if R3_ARCH_BITS == 32
-typedef int32_t         RTR3INTPTR;
+typedef int32_t             RTR3INTPTR;
 #elif R3_ARCH_BITS == 64
-typedef int64_t         RTR3INTPTR;
+typedef int64_t             RTR3INTPTR;
 #else
 #  error Unsupported R3_ARCH_BITS value.
 #endif
 /** Pointer to signed integer which can contain a HC ring-3 pointer. */
-typedef RTR3INTPTR     *PRTR3INTPTR;
+typedef RTR3INTPTR         *PRTR3INTPTR;
 /** Pointer to const signed integer which can contain a HC ring-3 pointer. */
-typedef const RTR3INTPTR *PCRTR3INTPTR;
+typedef const RTR3INTPTR   *PCRTR3INTPTR;
+/** Max RTR3INTPTR value. */
+#if R3_ARCH_BITS == 32
+# define RTR3INTPTR_MAX     INT32_MAX
+#else
+# define RTR3INTPTR_MAX     INT64_MAX
+#endif
+/** Min RTR3INTPTR value. */
+#if R3_ARCH_BITS == 32
+# define RTR3INTPTR_MIN     INT32_MIN
+#else
+# define RTR3INTPTR_MIN     INT64_MIN
+#endif
 
 /** Signed integer which can contain a HC ring-0 pointer. */
 #if R0_ARCH_BITS == 32
-typedef int32_t         RTR0INTPTR;
+typedef int32_t             RTR0INTPTR;
 #elif R0_ARCH_BITS == 64
-typedef int64_t         RTR0INTPTR;
+typedef int64_t             RTR0INTPTR;
 #else
-#  error Unsupported R0_ARCH_BITS value.
+# error Unsupported R0_ARCH_BITS value.
 #endif
 /** Pointer to signed integer which can contain a HC ring-0 pointer. */
-typedef RTR0INTPTR     *PRTR0INTPTR;
+typedef RTR0INTPTR         *PRTR0INTPTR;
 /** Pointer to const signed integer which can contain a HC ring-0 pointer. */
-typedef const RTR0INTPTR *PCRTR0INTPTR;
+typedef const RTR0INTPTR   *PCRTR0INTPTR;
+/** Max RTR0INTPTR value. */
+#if R0_ARCH_BITS == 32
+# define RTR0INTPTR_MAX     INT32_MAX
+#else
+# define RTR0INTPTR_MAX     INT64_MAX
+#endif
+/** Min RTHCINTPTR value. */
+#if R0_ARCH_BITS == 32
+# define RTR0INTPTR_MIN     INT32_MIN
+#else
+# define RTR0INTPTR_MIN     INT64_MIN
+#endif
 
 
 /** Unsigned integer which can contain a HC pointer. */
 #if HC_ARCH_BITS == 32
-typedef uint32_t        RTHCUINTPTR;
+typedef uint32_t            RTHCUINTPTR;
 #elif HC_ARCH_BITS == 64
-typedef uint64_t        RTHCUINTPTR;
+typedef uint64_t            RTHCUINTPTR;
 #else
-#  error Unsupported HC_ARCH_BITS value.
+# error Unsupported HC_ARCH_BITS value.
 #endif
 /** Pointer to unsigned integer which can contain a HC pointer. */
-typedef RTHCUINTPTR    *PRTHCUINTPTR;
+typedef RTHCUINTPTR        *PRTHCUINTPTR;
 /** Pointer to unsigned integer which can contain a HC pointer. */
-typedef const RTHCUINTPTR *PCRTHCUINTPTR;
+typedef const RTHCUINTPTR  *PCRTHCUINTPTR;
+/** Max RTHCUINTTPR value. */
+#if HC_ARCH_BITS == 32
+# define RTHCUINTPTR_MAX    UINT32_MAX
+#else
+# define RTHCUINTPTR_MAX    UINT64_MAX
+#endif
 
 /** Unsigned integer which can contain a HC ring-3 pointer. */
 #if R3_ARCH_BITS == 32
-typedef uint32_t        RTR3UINTPTR;
+typedef uint32_t            RTR3UINTPTR;
 #elif R3_ARCH_BITS == 64
-typedef uint64_t        RTR3UINTPTR;
+typedef uint64_t            RTR3UINTPTR;
 #else
-#  error Unsupported R3_ARCH_BITS value.
+# error Unsupported R3_ARCH_BITS value.
 #endif
 /** Pointer to unsigned integer which can contain a HC ring-3 pointer. */
-typedef RTR3UINTPTR    *PRTR3UINTPTR;
+typedef RTR3UINTPTR        *PRTR3UINTPTR;
 /** Pointer to unsigned integer which can contain a HC ring-3 pointer. */
-typedef const RTR3UINTPTR *PCRTR3UINTPTR;
+typedef const RTR3UINTPTR  *PCRTR3UINTPTR;
+/** Max RTHCUINTTPR value. */
+#if R3_ARCH_BITS == 32
+# define RTR3UINTPTR_MAX    UINT32_MAX
+#else
+# define RTR3UINTPTR_MAX    UINT64_MAX
+#endif
 
 /** Unsigned integer which can contain a HC ring-0 pointer. */
 #if R0_ARCH_BITS == 32
-typedef uint32_t        RTR0UINTPTR;
+typedef uint32_t            RTR0UINTPTR;
 #elif R0_ARCH_BITS == 64
-typedef uint64_t        RTR0UINTPTR;
+typedef uint64_t            RTR0UINTPTR;
 #else
-#  error Unsupported R0_ARCH_BITS value.
+# error Unsupported R0_ARCH_BITS value.
 #endif
 /** Pointer to unsigned integer which can contain a HC ring-0 pointer. */
-typedef RTR0UINTPTR    *PRTR0UINTPTR;
+typedef RTR0UINTPTR        *PRTR0UINTPTR;
 /** Pointer to unsigned integer which can contain a HC ring-0 pointer. */
-typedef const RTR0UINTPTR *PCRTR0UINTPTR;
+typedef const RTR0UINTPTR  *PCRTR0UINTPTR;
+/** Max RTR0UINTTPR value. */
+#if HC_ARCH_BITS == 32
+# define RTR0UINTPTR_MAX    UINT32_MAX
+#else
+# define RTR0UINTPTR_MAX    UINT64_MAX
+#endif
 
 
 /** Host Physical Memory Address. */
-typedef uint64_t        RTHCPHYS;
+typedef uint64_t            RTHCPHYS;
 /** Pointer to Host Physical Memory Address. */
-typedef RTHCPHYS       *PRTHCPHYS;
+typedef RTHCPHYS           *PRTHCPHYS;
 /** Pointer to const Host Physical Memory Address. */
-typedef const RTHCPHYS *PCRTHCPHYS;
+typedef const RTHCPHYS     *PCRTHCPHYS;
 /** @def NIL_RTHCPHYS
  * NIL HC Physical Address.
  * NIL_RTHCPHYS is used to signal an invalid physical address, similar
  * to the NULL pointer.
  */
-#define NIL_RTHCPHYS     ((RTHCPHYS)~0U) /** @todo change this to (~(VBOXHCPHYS)0) */
+#define NIL_RTHCPHYS        (~(RTHCPHYS)0)
+/** Max RTHCPHYS value. */
+#define RTHCPHYS_MAX        UINT64_MAX
 
 
 /** HC pointer. */
 #ifndef IN_RC
-typedef void *          RTHCPTR;
+typedef void *              RTHCPTR;
 #else
-typedef RTHCUINTPTR     RTHCPTR;
+typedef RTHCUINTPTR         RTHCPTR;
 #endif
 /** Pointer to HC pointer. */
-typedef RTHCPTR        *PRTHCPTR;
+typedef RTHCPTR            *PRTHCPTR;
 /** Pointer to const HC pointer. */
-typedef const RTHCPTR  *PCRTHCPTR;
+typedef const RTHCPTR      *PCRTHCPTR;
 /** @def NIL_RTHCPTR
  * NIL HC pointer.
  */
-#define NIL_RTHCPTR     ((RTHCPTR)0)
+#define NIL_RTHCPTR         ((RTHCPTR)0)
+/** Max RTHCPTR value. */
+#define RTHCPTR_MAX         ((RTHCPTR)RTHCUINTPTR_MAX)
+
 
 /** HC ring-3 pointer. */
 #ifdef  IN_RING3
-typedef void *          RTR3PTR;
+typedef void *              RTR3PTR;
 #else
-typedef RTR3UINTPTR     RTR3PTR;
+typedef RTR3UINTPTR         RTR3PTR;
 #endif
 /** Pointer to HC ring-3 pointer. */
-typedef RTR3PTR        *PRTR3PTR;
+typedef RTR3PTR            *PRTR3PTR;
 /** Pointer to const HC ring-3 pointer. */
-typedef const RTR3PTR  *PCRTR3PTR;
+typedef const RTR3PTR      *PCRTR3PTR;
 /** @def NIL_RTR3PTR
  * NIL HC ring-3 pointer.
  */
-#define NIL_RTR3PTR     ((RTR3PTR)0)
+#define NIL_RTR3PTR         ((RTR3PTR)0)
+/** Max RTR3PTR value. */
+#define RTR3PTR_MAX         ((RTR3PTR)RTR3UINTPTR_MAX)
 
 /** HC ring-0 pointer. */
 #ifdef  IN_RING0
-typedef void *          RTR0PTR;
+typedef void *              RTR0PTR;
 #else
-typedef RTR0UINTPTR     RTR0PTR;
+typedef RTR0UINTPTR         RTR0PTR;
 #endif
 /** Pointer to HC ring-0 pointer. */
-typedef RTR0PTR        *PRTR0PTR;
+typedef RTR0PTR            *PRTR0PTR;
 /** Pointer to const HC ring-0 pointer. */
-typedef const RTR0PTR  *PCRTR0PTR;
+typedef const RTR0PTR      *PCRTR0PTR;
 /** @def NIL_RTR0PTR
  * NIL HC ring-0 pointer.
  */
-#define NIL_RTR0PTR     ((RTR0PTR)0)
+#define NIL_RTR0PTR         ((RTR0PTR)0)
+/** Max RTR3PTR value. */
+#define RTR0PTR_MAX         ((RTR0PTR)RTR0UINTPTR_MAX)
 
 
 /** Unsigned integer register in the host context. */
@@ -780,26 +866,28 @@ typedef RTGCUINTPTR64   *PRTGCUINTPTR64;
 typedef const RTGCUINTPTR64 *PCRTGCUINTPTR64;
 
 /** Guest Physical Memory Address.*/
-typedef uint64_t        RTGCPHYS;
+typedef uint64_t            RTGCPHYS;
 /** Pointer to Guest Physical Memory Address. */
-typedef RTGCPHYS       *PRTGCPHYS;
+typedef RTGCPHYS           *PRTGCPHYS;
 /** Pointer to const Guest Physical Memory Address. */
-typedef const RTGCPHYS *PCRTGCPHYS;
+typedef const RTGCPHYS     *PCRTGCPHYS;
 /** @def NIL_RTGCPHYS
  * NIL GC Physical Address.
  * NIL_RTGCPHYS is used to signal an invalid physical address, similar
  * to the NULL pointer. Note that this value may actually be valid in
  * some contexts.
  */
-#define NIL_RTGCPHYS     (~(RTGCPHYS)0U)
+#define NIL_RTGCPHYS        (~(RTGCPHYS)0U)
+/** Max guest physical memory address value. */
+#define RTGCPHYS_MAX        UINT64_MAX
 
 
 /** Guest Physical Memory Address; limited to 32 bits.*/
-typedef uint32_t        RTGCPHYS32;
+typedef uint32_t            RTGCPHYS32;
 /** Pointer to Guest Physical Memory Address. */
-typedef RTGCPHYS32     *PRTGCPHYS32;
+typedef RTGCPHYS32         *PRTGCPHYS32;
 /** Pointer to const Guest Physical Memory Address. */
-typedef const RTGCPHYS32 *PCRTGCPHYS32;
+typedef const RTGCPHYS32    *PCRTGCPHYS32;
 /** @def NIL_RTGCPHYS32
  * NIL GC Physical Address.
  * NIL_RTGCPHYS32 is used to signal an invalid physical address, similar
@@ -858,21 +946,25 @@ typedef RTGCPTR64       RTGCPTR;
 /** Pointer to a guest context pointer. */
 typedef PRTGCPTR64      PRTGCPTR;
 /** Pointer to a const guest context pointer. */
-typedef PCRTGCPTR64    PCRTGCPTR;
+typedef PCRTGCPTR64     PCRTGCPTR;
 /** @def NIL_RTGCPTR
  * NIL GC pointer.
  */
-#define NIL_RTGCPTR     NIL_RTGCPTR64
+# define NIL_RTGCPTR    NIL_RTGCPTR64
+/** Max RTGCPTR value. */
+# define RTGCPTR_MAX    UINT64_MAX
 #elif GC_ARCH_BITS == 32
 typedef RTGCPTR32       RTGCPTR;
 /** Pointer to a guest context pointer. */
 typedef PRTGCPTR32      PRTGCPTR;
 /** Pointer to a const guest context pointer. */
-typedef PCRTGCPTR32    PCRTGCPTR;
+typedef PCRTGCPTR32     PCRTGCPTR;
 /** @def NIL_RTGCPTR
  * NIL GC pointer.
  */
-#define NIL_RTGCPTR     NIL_RTGCPTR32
+# define NIL_RTGCPTR     NIL_RTGCPTR32
+/** Max RTGCPTR value. */
+# define RTGCPTR_MAX    UINT32_MAX
 #else
 # error "Unsupported GC_ARCH_BITS!"
 #endif
@@ -891,16 +983,16 @@ typedef RTGCUINTREG64        *PRTGCUINTREG64;
 typedef const RTGCUINTREG64  *PCRTGCUINTREG64;
 
 #if GC_ARCH_BITS == 64
-typedef RTGCUINTREG64       RTGCUINTREG;
+typedef RTGCUINTREG64           RTGCUINTREG;
 #elif GC_ARCH_BITS == 32
-typedef RTGCUINTREG32       RTGCUINTREG;
+typedef RTGCUINTREG32           RTGCUINTREG;
 #else
 # error "Unsupported GC_ARCH_BITS!"
 #endif
 /** Pointer to an unsigned integer register in the guest context. */
-typedef RTGCUINTREG        *PRTGCUINTREG;
+typedef RTGCUINTREG            *PRTGCUINTREG;
 /** Pointer to a const unsigned integer register in the guest context. */
-typedef const RTGCUINTREG  *PCRTGCUINTREG;
+typedef const RTGCUINTREG      *PCRTGCUINTREG;
 
 /** @} */
 
@@ -1012,6 +1104,20 @@ typedef RTFILE                                     *PRTFILE;
 /** Nil file handle. */
 #define NIL_RTFILE                                  (~(RTFILE)0)
 
+/** Async I/O request handle. */
+typedef R3PTRTYPE(struct RTFILEAIOREQINTERNAL *)    RTFILEAIOREQ;
+/** Pointer to a async I/O request handle. */
+typedef RTFILEAIOREQ                               *PRTFILEAIOREQ;
+/** Nil request handle. */
+#define NIL_RTFILEAIOREQ                            0
+
+/** Async I/O completion context handle. */
+typedef R3PTRTYPE(struct RTFILEAIOCTXINTERNAL *)    RTFILEAIOCTX;
+/** Pointer to a async I/O completion context handle. */
+typedef RTFILEAIOCTX                               *PRTFILEAIOCTX;
+/** Nil context handle. */
+#define NIL_RTFILEAIOCTX                            0
+
 /** Loader module handle. */
 typedef R3PTRTYPE(struct RTLDRMODINTERNAL *)        RTLDRMOD;
 /** Pointer to a loader module handle. */
@@ -1066,7 +1172,7 @@ typedef RTSEMEVENTMULTI                            *PRTSEMEVENTMULTI;
 /** @typedef RTSEMFASTMUTEX
  * Fast mutex Semaphore handle. */
 typedef R3R0PTRTYPE(struct RTSEMFASTMUTEXINTERNAL *) RTSEMFASTMUTEX;
-/** Pointer to a mutex semaphore handle. */
+/** Pointer to a fast mutex semaphore handle. */
 typedef RTSEMFASTMUTEX                             *PRTSEMFASTMUTEX;
 /** Nil fast mutex semaphore handle. */
 #define NIL_RTSEMFASTMUTEX                          0
@@ -1079,6 +1185,14 @@ typedef RTSEMMUTEX                                 *PRTSEMMUTEX;
 /** Nil mutex semaphore handle. */
 #define NIL_RTSEMMUTEX                              0
 
+/** @typedef RTSEMSPINMUTEX
+ * Spinning mutex Semaphore handle. */
+typedef R3R0PTRTYPE(struct RTSEMSPINMUTEXINTERNAL *) RTSEMSPINMUTEX;
+/** Pointer to a spinning mutex semaphore handle. */
+typedef RTSEMSPINMUTEX                             *PRTSEMSPINMUTEX;
+/** Nil spinning mutex semaphore handle. */
+#define NIL_RTSEMSPINMUTEX                          0
+
 /** @typedef RTSEMRW
  * Read/Write Semaphore handle. */
 typedef R3R0PTRTYPE(struct RTSEMRWINTERNAL *)       RTSEMRW;
@@ -1109,7 +1223,7 @@ typedef RTTHREAD                                   *PRTTHREAD;
 #define NIL_RTTHREAD                                0
 
 /** A TLS index. */
-typedef int                                         RTTLS;
+typedef RTHCINTPTR                                  RTTLS;
 /** Pointer to a TLS index. */
 typedef RTTLS                                      *PRTTLS;
 /** Pointer to a const TLS index. */
@@ -1124,6 +1238,13 @@ typedef RTHEAPSIMPLE                               *PRTHEAPSIMPLE;
 /** NIL simple heap handle. */
 #define NIL_RTHEAPSIMPLE                            ((RTHEAPSIMPLE)0)
 
+/** Handle to a offset based heap. */
+typedef R3R0PTRTYPE(struct RTHEAPOFFSETINTERNAL *)  RTHEAPOFFSET;
+/** Pointer to a handle to a offset based heap. */
+typedef RTHEAPOFFSET                               *PRTHEAPOFFSET;
+/** NIL offset based heap handle. */
+#define NIL_RTHEAPOFFSET                            ((RTHEAPOFFSET)0)
+
 /** Handle to an environment block. */
 typedef R3PTRTYPE(struct RTENVINTERNAL *)           RTENV;
 /** Pointer to a handle to an environment block. */
@@ -1135,7 +1256,7 @@ typedef RTENV                                      *PRTENV;
  * @remarks This doesn't have to correspond to the APIC ID (intel/amd). Nor
  *          does it have to correspond to the bits in the affinity mask, at
  *          least not until we've sorted out Windows NT. */
-typedef RTHCUINTPTR                                 RTCPUID;
+typedef uint32_t                                    RTCPUID;
 /** Pointer to a CPU identifier. */
 typedef RTCPUID                                    *PRTCPUID;
 /** Pointer to a const CPU identifier. */
@@ -1144,7 +1265,8 @@ typedef RTCPUID const                              *PCRTCPUID;
 #define NIL_RTCPUID                                 ((RTCPUID)~0)
 
 /** A CPU set.
- * Treat this as an opaque type and always use RTCpuSet* for manupulating it. */
+ * Treat this as an opaque type and always use RTCpuSet* for manupulating it.
+ * @remarks Subject to change. */
 typedef uint64_t                                    RTCPUSET;
 /** Pointer to a CPU set. */
 typedef RTCPUSET                                   *PRTCPUSET;
@@ -1152,7 +1274,7 @@ typedef RTCPUSET                                   *PRTCPUSET;
 typedef RTCPUSET const                             *PCRTCPUSET;
 
 /** A handle table handle. */
-typedef struct RTHANDLETABLEINT                    *RTHANDLETABLE;
+typedef R3R0PTRTYPE(struct RTHANDLETABLEINT *)      RTHANDLETABLE;
 /** A pointer to a handle table handle. */
 typedef RTHANDLETABLE                              *PRTHANDLETABLE;
 /** @def NIL_RTHANDLETABLE
@@ -1160,7 +1282,7 @@ typedef RTHANDLETABLE                              *PRTHANDLETABLE;
 #define NIL_RTHANDLETABLE                           ((RTHANDLETABLE)0)
 
 /** A handle to a low resolution timer. */
-typedef struct RTTIMERLRINT                        *RTTIMERLR;
+typedef R3R0PTRTYPE(struct RTTIMERLRINT *)          RTTIMERLR;
 /** A pointer to a low resolution timer handle. */
 typedef RTTIMERLR                                  *PRTTIMERLR;
 /** @def NIL_RTTIMERLR
@@ -1168,12 +1290,44 @@ typedef RTTIMERLR                                  *PRTTIMERLR;
 #define NIL_RTTIMERLR                               ((RTTIMERLR)0)
 
 /** Handle to a random number generator. */
-typedef struct RTRANDINT                           *RTRAND;
+typedef R3R0PTRTYPE(struct RTRANDINT *)             RTRAND;
 /** Pointer to a random number generator handle. */
 typedef RTRAND                                     *PRTRAND;
 /** NIL random number genrator handle value. */
 #define NIL_RTRAND                                  ((RTRAND)0)
 
+/** Debug address space handle. */
+typedef R3R0PTRTYPE(struct RTDBGASINT *)            RTDBGAS;
+/** Pointer to a debug address space handle. */
+typedef RTDBGAS                                    *PRTDBGAS;
+/** NIL debug address space handle. */
+#define NIL_RTDBGAS                                 ((RTDBGAS)0)
+
+/** Debug module handle. */
+typedef R3R0PTRTYPE(struct RTDBGMODINT *)           RTDBGMOD;
+/** Pointer to a debug module handle. */
+typedef RTDBGMOD                                   *PRTDBGMOD;
+/** NIL debug module handle. */
+#define NIL_RTDBGMOD                                ((RTDBGMOD)0)
+
+/** Memory pool handle. */
+typedef R3R0PTRTYPE(struct RTMEMPOOLINT *)          RTMEMPOOL;
+/** Pointer to a memory pool handle. */
+typedef RTMEMPOOL                                  *PRTMEMPOOL;
+/** NIL memory pool handle. */
+#define NIL_RTMEMPOOL                               ((RTMEMPOOL)0)
+/** The default memory pool handle. */
+#define RTMEMPOOL_DEFAULT                           ((RTMEMPOOL)-2)
+
+/** String cache handle. */
+typedef R3R0PTRTYPE(struct RTSTRCACHEINT *)         RTSTRCACHE;
+/** Pointer to a string cache handle. */
+typedef RTSTRCACHE                                 *PRTSTRCACHE;
+/** NIL string cache handle. */
+#define NIL_RTSTRCACHE                              ((RTSTRCACHE)0)
+/** The default string cache handle. */
+#define RTSTRCACHE_DEFAULT                          ((RTSTRCACHE)-2)
+
 
 /**
  * UUID data type.
@@ -1306,6 +1460,38 @@ typedef RTMAC *PRTMAC;
 /** Pointer to a readonly MAC address. */
 typedef const RTMAC *PCRTMAC;
 
+
+#ifdef __cplusplus
+/**
+ * Strict type validation helper class.
+ *
+ * See RTErrStrictType and RT_SUCCESS_NP.
+ */
+class RTErrStrictType2
+{
+protected:
+    /** The status code.  */
+    int32_t m_rc;
+
+public:
+    /**
+     * Constructor.
+     * @param   rc      IPRT style status code.
+     */
+    RTErrStrictType2(int32_t rc) : m_rc(rc)
+    {
+    }
+
+    /**
+     * Get the status code.
+     * @returns IPRT style status code.
+     */
+    int32_t getValue() const
+    {
+        return m_rc;
+    }
+};
+#endif /* __cplusplus */
 /** @} */
 
 #endif
