@@ -1,6 +1,6 @@
-/** $Id: VBoxControl.cpp $ */
+/* $Id: VBoxControl.cpp $ */
 /** @file
- * VBoxControl - Guest Additions Command Line Management Interface
+ * VBoxControl - Guest Additions Command Line Management Interface.
  */
 
 /*
@@ -19,28 +19,26 @@
  * additional information or have any questions.
  */
 
-
-
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
+#include <iprt/alloca.h>
+#include <iprt/autores.h>
+#include <iprt/buildconfig.h>
+#include <iprt/initterm.h>
 #include <iprt/mem.h>
+#include <iprt/path.h>
 #include <iprt/string.h>
 #include <iprt/stream.h>
-#include <iprt/path.h>
-#include <iprt/initterm.h>
-#include <iprt/autores.h>
 #include <VBox/log.h>
-#include <VBox/VBoxGuest.h>
 #include <VBox/version.h>
+#include <VBox/VBoxGuestLib.h>
 #ifdef RT_OS_WINDOWS
-# include <windows.h>
-# include <malloc.h>  /* for alloca */
+# include <Windows.h>
 #endif
 #ifdef VBOX_WITH_GUEST_PROPS
 # include <VBox/HostServices/GuestPropertySvc.h>
 #endif
-#include "VBoxControl.h"
 
 /*******************************************************************************
 *   Global Variables                                                           *
@@ -652,7 +650,7 @@ void getCustomModes(HKEY hkeyVideo)
     int curMode = 0;
 
     /* null out the table */
-    memset(customModes, 0, sizeof(customModes));
+    RT_ZERO(customModes);
 
     do
     {
@@ -859,7 +857,7 @@ int handleRemoveCustomMode(int argc, char *argv[])
                 && (customModes[i].bpp  == bpp))
             {
                 RTPrintf("found mode at index %d\n", i);
-                memset(&customModes[i], 0, sizeof(customModes[i]));
+                RT_ZERO(customModes[i]);
                 break;
             }
         }
@@ -1313,7 +1311,7 @@ int main(int argc, char **argv)
            )
             {
                 /* Print version number, and do nothing else. */
-                RTPrintf("%sr%d\n", VBOX_VERSION_STRING, VBoxSVNRev ());
+                RTPrintf("%sr%u\n", VBOX_VERSION_STRING, RTBldCfgRevision());
                 onlyinfo = true;
                 showlogo = false;
                 done = true;
@@ -1434,3 +1432,4 @@ int main(int argc, char **argv)
 
     return rc;
 }
+
