@@ -1,7 +1,9 @@
+/* $Revision: 57546 $ */
 /** @file
+ * VBoxGuestLib - Host-Guest Communication Manager.
  *
- * VBoxGuestLib - A support library for VirtualBox guest additions:
- * Host-Guest Communication Manager
+ * These public functions can be only used by other drivers. They all
+ * do an IOCTL to VBoxGuest via IDC.
  */
 
 /*
@@ -15,19 +17,23 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
  * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
  * Clara, CA 95054 USA or visit http://www.sun.com if you need
  * additional information or have any questions.
  */
 
-/* These public functions can be only used by other drivers.
- * They all do an IOCTL to VBoxGuest.
- */
-
 /* Entire file is ifdef'ed with !VBGL_VBOXGUEST */
 #ifndef VBGL_VBOXGUEST
 
-#include <VBox/VBoxGuestLib.h>
 #include "VBGLInternal.h"
 
 #include <iprt/assert.h>
@@ -36,16 +42,25 @@
 
 #define VBGL_HGCM_ASSERTMsg AssertReleaseMsg
 
-int vbglHGCMInit (void)
+/**
+ * Initializes the HGCM VBGL bits.
+ *
+ * @return VBox status code.
+ */
+int vbglR0HGCMInit (void)
 {
-    RTSemFastMutexCreate(&g_vbgldata.mutexHGCMHandle);
-
-    return VINF_SUCCESS;
+    return RTSemFastMutexCreate(&g_vbgldata.mutexHGCMHandle);
 }
 
-int vbglHGCMTerminate (void)
+/**
+ * Initializes the HGCM VBGL bits.
+ *
+ * @return VBox status code.
+ */
+int vbglR0HGCMTerminate (void)
 {
     RTSemFastMutexDestroy(g_vbgldata.mutexHGCMHandle);
+    g_vbgldata.mutexHGCMHandle = NIL_RTSEMFASTMUTEX;
 
     return VINF_SUCCESS;
 }
@@ -54,8 +69,8 @@ DECLINLINE(int) vbglHandleHeapEnter (void)
 {
     int rc = RTSemFastMutexRequest(g_vbgldata.mutexHGCMHandle);
 
-    VBGL_HGCM_ASSERTMsg(VBOX_SUCCESS(rc),
-                        ("Failed to request handle heap mutex, rc = %Vrc\n", rc));
+    VBGL_HGCM_ASSERTMsg(RT_SUCCESS(rc),
+                        ("Failed to request handle heap mutex, rc = %Rrc\n", rc));
 
     return rc;
 }
@@ -71,7 +86,7 @@ struct VBGLHGCMHANDLEDATA *vbglHGCMHandleAlloc (void)
     int rc = vbglHandleHeapEnter ();
     uint32_t i;
 
-    if (VBOX_FAILURE (rc))
+    if (RT_FAILURE (rc))
         return NULL;
 
     p = NULL;
@@ -80,7 +95,7 @@ struct VBGLHGCMHANDLEDATA *vbglHGCMHandleAlloc (void)
      * @todo bitmap for faster search and other obvious optimizations.
      */
 
-    for (i = 0; i < ELEMENTS(g_vbgldata.aHGCMHandleData); i++)
+    for (i = 0; i < RT_ELEMENTS(g_vbgldata.aHGCMHandleData); i++)
     {
         if (!g_vbgldata.aHGCMHandleData[i].fAllocated)
         {
@@ -107,7 +122,7 @@ void vbglHGCMHandleFree (struct VBGLHGCMHANDLEDATA *pHandle)
 
     rc = vbglHandleHeapEnter ();
 
-    if (VBOX_FAILURE (rc))
+    if (RT_FAILURE (rc))
         return;
 
     VBGL_HGCM_ASSERTMsg(pHandle->fAllocated,
@@ -138,11 +153,11 @@ DECLVBGL(int) VbglHGCMConnect (VBGLHGCMHANDLE *pHandle, VBoxGuestHGCMConnectInfo
     {
         rc = vbglDriverOpen (&pHandleData->driver);
 
-        if (VBOX_SUCCESS(rc))
+        if (RT_SUCCESS(rc))
         {
-            rc = vbglDriverIOCtl (&pHandleData->driver, IOCTL_VBOXGUEST_HGCM_CONNECT, pData, sizeof (*pData));
+            rc = vbglDriverIOCtl (&pHandleData->driver, VBOXGUEST_IOCTL_HGCM_CONNECT, pData, sizeof (*pData));
 
-            if (VBOX_SUCCESS(rc))
+            if (RT_SUCCESS(rc))
             {
                 *pHandle = pHandleData;
             }
@@ -152,7 +167,7 @@ DECLVBGL(int) VbglHGCMConnect (VBGLHGCMHANDLE *pHandle, VBoxGuestHGCMConnectInfo
             }
         }
 
-        if (VBOX_FAILURE(rc))
+        if (RT_FAILURE(rc))
         {
             vbglHGCMHandleFree (pHandleData);
         }
@@ -165,7 +180,7 @@ DECLVBGL(int) VbglHGCMDisconnect (VBGLHGCMHANDLE handle, VBoxGuestHGCMDisconnect
 {
     int rc = VINF_SUCCESS;
 
-    rc = vbglDriverIOCtl (&handle->driver, IOCTL_VBOXGUEST_HGCM_DISCONNECT, pData, sizeof (*pData));
+    rc = vbglDriverIOCtl (&handle->driver, VBOXGUEST_IOCTL_HGCM_DISCONNECT, pData, sizeof (*pData));
 
     vbglDriverClose (&handle->driver);
 
@@ -181,9 +196,27 @@ DECLVBGL(int) VbglHGCMCall (VBGLHGCMHANDLE handle, VBoxGuestHGCMCallInfo *pData,
     VBGL_HGCM_ASSERTMsg(cbData >= sizeof (VBoxGuestHGCMCallInfo) + pData->cParms * sizeof (HGCMFunctionParameter),
                         ("cbData = %d, cParms = %d (calculated size %d)\n", cbData, pData->cParms, sizeof (VBoxGuestHGCMCallInfo) + pData->cParms * sizeof (VBoxGuestHGCMCallInfo)));
 
-    rc = vbglDriverIOCtl (&handle->driver, IOCTL_VBOXGUEST_HGCM_CALL, pData, cbData);
+    rc = vbglDriverIOCtl (&handle->driver, VBOXGUEST_IOCTL_HGCM_CALL(cbData), pData, cbData);
 
     return rc;
 }
 
-#endif /* VBGL_VBOXGUEST */
+DECLVBGL(int) VbglHGCMCallTimed (VBGLHGCMHANDLE handle,
+                                 VBoxGuestHGCMCallInfoTimed *pData, uint32_t cbData)
+{
+    int rc = VINF_SUCCESS;
+
+    uint32_t cbExpected =   sizeof (VBoxGuestHGCMCallInfoTimed)
+                          + pData->info.cParms * sizeof (HGCMFunctionParameter);
+    VBGL_HGCM_ASSERTMsg(cbData >= cbExpected,
+                        ("cbData = %d, cParms = %d (calculated size %d)\n",
+                        cbData, pData->info.cParms, cbExpected));
+
+    rc = vbglDriverIOCtl (&handle->driver, VBOXGUEST_IOCTL_HGCM_CALL_TIMED(cbData),
+                          pData, cbData);
+
+    return rc;
+}
+
+#endif /* !VBGL_VBOXGUEST */
+
