@@ -1,7 +1,6 @@
+/* $Revision: 57546 $ */
 /** @file
- *
- * VBoxGuestLib - A support library for VirtualBox guest additions:
- * Library initialization
+ * VBoxGuestLibR0 - Library initialization.
  */
 
 /*
@@ -15,28 +14,72 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
  * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
  * Clara, CA 95054 USA or visit http://www.sun.com if you need
  * additional information or have any questions.
  */
 
-
-#include <VBox/VBoxGuestLib.h>
-
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
 #define VBGL_DECL_DATA
 #include "VBGLInternal.h"
 
 #include <iprt/string.h>
 #include <iprt/assert.h>
 
+/*******************************************************************************
+*   Global Variables                                                           *
+*******************************************************************************/
+/** The global VBGL instance data.  */
 VBGLDATA g_vbgldata;
 
+/**
+ * Used by vbglQueryVMMDevPort and VbglInit to try get the host feature mask and
+ * version information (g_vbgldata::hostVersion).
+ *
+ * This was first implemented by the host in 3.1 and we quietly ignore failures
+ * for that reason.
+ */
+static void vbglR0QueryHostVersion (void)
+{
+    VMMDevReqHostVersion *pReq;
+
+    int rc = VbglGRAlloc ((VMMDevRequestHeader **) &pReq, sizeof (*pReq), VMMDevReq_GetHostVersion);
+
+    if (RT_SUCCESS (rc))
+    {
+        rc = VbglGRPerform (&pReq->header);
+
+        if (RT_SUCCESS (rc))
+        {
+            g_vbgldata.hostVersion = *pReq;
+            Log (("vbglR0QueryHostVersion: %u.%u.%ur%u %#x\n",
+                  pReq->major, pReq->minor, pReq->build, pReq->revision, pReq->features));
+        }
+
+        VbglGRFree (&pReq->header);
+    }
+}
+
 #ifndef VBGL_VBOXGUEST
-/* The guest library uses lazy initialization for VMMDev port and memory,
+/**
+ * The guest library uses lazy initialization for VMMDev port and memory,
  * because these values are provided by the VBoxGuest driver and it might
  * be loaded later than other drivers.
+ *
  * The VbglEnter checks the current library status, tries to retrive these
  * values and fails if they are unavailable.
+ *
  */
 static void vbglQueryVMMDevPort (void)
 {
@@ -46,20 +89,25 @@ static void vbglQueryVMMDevPort (void)
 
     rc = vbglDriverOpen (&driver);
 
-    if (VBOX_SUCCESS(rc))
+    if (RT_SUCCESS(rc))
     {
+        /*
+         * Try query the port info.
+         */
         VBoxGuestPortInfo port;
 
-        rc = vbglDriverIOCtl (&driver, IOCTL_VBOXGUEST_GETVMMDEVPORT, &port, sizeof (port));
+        rc = vbglDriverIOCtl (&driver, VBOXGUEST_IOCTL_GETVMMDEVPORT, &port, sizeof (port));
 
-        if (VBOX_SUCCESS (rc))
+        if (RT_SUCCESS (rc))
         {
             dprintf (("port = 0x%04X, mem = %p\n", port.portAddress, port.pVMMDevMemory));
-            
+
             g_vbgldata.portVMMDev = port.portAddress;
             g_vbgldata.pVMMDevMemory = port.pVMMDevMemory;
 
             g_vbgldata.status = VbglStatusReady;
+
+            vbglR0QueryHostVersion();
         }
 
         vbglDriverClose (&driver);
@@ -67,9 +115,16 @@ static void vbglQueryVMMDevPort (void)
 
     dprintf (("vbglQueryVMMDevPort rc = %d\n", rc));
 }
-#endif
+#endif /* !VBGL_VBOXGUEST */
 
-int VbglEnter (void)
+/**
+ * Checks if VBGL has been initialized.
+ *
+ * The the client library, this will lazily complete the initialization.
+ *
+ * @return VINF_SUCCESS or VERR_VBGL_NOT_INITIALIZED.
+ */
+int vbglR0Enter (void)
 {
     int rc;
 
@@ -81,9 +136,9 @@ int VbglEnter (void)
 #endif
 
     rc = g_vbgldata.status == VbglStatusReady? VINF_SUCCESS: VERR_VBGL_NOT_INITIALIZED;
-    
+
     // dprintf(("VbglEnter: rc = %d\n", rc));
-    
+
     return rc;
 }
 
@@ -91,20 +146,20 @@ int vbglInitCommon (void)
 {
     int rc = VINF_SUCCESS;
 
-    memset(&g_vbgldata, 0, sizeof (VBGLDATA));
+    RT_ZERO(g_vbgldata);
 
     g_vbgldata.status = VbglStatusInitializing;
 
     rc = VbglPhysHeapInit ();
 
-    if (VBOX_SUCCESS(rc))
+    if (RT_SUCCESS(rc))
     {
         /* other subsystems, none yet */
         ;
     }
 
     dprintf(("vbglInitCommon: rc = %d\n", rc));
-    
+
     return rc;
 }
 
@@ -112,7 +167,7 @@ DECLVBGL(void) vbglTerminateCommon (void)
 {
     VbglPhysHeapTerminate ();
 
-    memset(&g_vbgldata, 0, sizeof (VBGLDATA));
+    RT_ZERO(g_vbgldata);
 
     return;
 }
@@ -123,23 +178,29 @@ DECLVBGL(int) VbglInit (VBGLIOPORT portVMMDev, VMMDevMemory *pVMMDevMemory)
 {
     int rc = VINF_SUCCESS;
 
+# ifdef RT_OS_WINDOWS /** @todo r=bird: this doesn't make sense. Is there something special going on on windows? */
     dprintf(("vbglInit: starts g_vbgldata.status %d\n", g_vbgldata.status));
-    
+
     if (g_vbgldata.status == VbglStatusInitializing
         || g_vbgldata.status == VbglStatusReady)
     {
         /* Initialization is already in process. */
         return rc;
     }
+# else
+    dprintf(("vbglInit: starts\n"));
+# endif
 
     rc = vbglInitCommon ();
 
-    if (VBOX_SUCCESS(rc))
+    if (RT_SUCCESS(rc))
     {
         g_vbgldata.portVMMDev = portVMMDev;
         g_vbgldata.pVMMDevMemory = pVMMDevMemory;
 
         g_vbgldata.status = VbglStatusReady;
+
+        vbglR0QueryHostVersion();
     }
     else
     {
@@ -172,16 +233,16 @@ DECLVBGL(int) VbglInit (void)
 
     rc = vbglInitCommon ();
 
-    if (VBOX_SUCCESS(rc))
+    if (RT_SUCCESS(rc))
     {
         /* Try to obtain VMMDev port via IOCTL to VBoxGuest main driver. */
         vbglQueryVMMDevPort ();
 
-#ifdef VBOX_HGCM
-        rc = vbglHGCMInit ();
-#endif /* VBOX_HGCM */
+# ifdef VBOX_WITH_HGCM
+        rc = vbglR0HGCMInit ();
+# endif /* VBOX_WITH_HGCM */
 
-        if (VBOX_FAILURE(rc))
+        if (RT_FAILURE(rc))
         {
             vbglTerminateCommon ();
         }
@@ -194,9 +255,9 @@ DECLVBGL(void) VbglTerminate (void)
 {
     vbglTerminateCommon ();
 
-#ifdef VBOX_HGCM
-    vbglHGCMTerminate ();
-#endif
+# ifdef VBOX_WITH_HGCM
+    vbglR0HGCMTerminate ();
+# endif
 
     return;
 }
