@@ -1,7 +1,6 @@
+/* $Revision: 57546 $ */
 /** @file
- *
- * VBoxGuestLib - A support library for VirtualBox guest additions:
- * Physical memory heap
+ * VBoxGuestLibR0 - Physical memory heap.
  */
 
 /*
@@ -15,12 +14,20 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
  * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
  * Clara, CA 95054 USA or visit http://www.sun.com if you need
  * additional information or have any questions.
  */
 
-#include <VBox/VBoxGuestLib.h>
 #include "VBGLInternal.h"
 
 #include <iprt/assert.h>
@@ -627,3 +634,4 @@ DECLVBGL(void) VbglPhysHeapTerminate (void)
 
     RTSemFastMutexDestroy(g_vbgldata.mutexHeap);
 }
+
