@@ -1,7 +1,6 @@
+/* $Revision: 57546 $ */
 /** @file
- *
- * VBoxGuestLib - A support library for VirtualBox guest additions:
- * Central calls
+ * VBoxGuestLibR0 - Central calls.
  */
 
 /*
@@ -15,6 +14,15 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
  * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
  * Clara, CA 95054 USA or visit http://www.sun.com if you need
  * additional information or have any questions.
@@ -30,6 +38,7 @@
 # include "VBoxCalls.h"
 #endif
 #include <iprt/time.h>
+#include <iprt/mem.h>
 #include <iprt/path.h>
 #include <iprt/string.h>
 
@@ -78,12 +87,12 @@ DECLVBGL(int) vboxConnect (PVBSFCLIENT pClient)
 /*    Log(("VBOXSF: VBoxSF::vboxConnect: VbglHGCMConnect rc = %#x, result = %#x\n",
          rc, data.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.result;
     }
 
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         pClient->ulClientID = data.u32ClientID;
     }
@@ -128,7 +137,7 @@ DECLVBGL(int) vboxCallQueryMappings (PVBSFCLIENT pClient, SHFLMAPPING paMappings
 
     data.mappings.type                   = VMMDevHGCMParmType_LinAddr;
     data.mappings.u.Pointer.size         = sizeof (SHFLMAPPING) * *pcMappings;
-    data.mappings.u.Pointer.u.linearAddr = (RTGCPTR)&paMappings[0];
+    data.mappings.u.Pointer.u.linearAddr = (uintptr_t)&paMappings[0];
 
 /*    Log(("VBOXSF: in ifs difference %d\n",
          (char *)&data.flags.type - (char *)&data.callInfo.cParms));
@@ -139,12 +148,12 @@ DECLVBGL(int) vboxCallQueryMappings (PVBSFCLIENT pClient, SHFLMAPPING paMappings
     Log(("VBOXSF: VBoxSF::vboxCallQueryMappings: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
 
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         *pcMappings = data.numberOfMappings.u.value32;
     }
@@ -165,14 +174,14 @@ DECLVBGL(int) vboxCallQueryMapName (PVBSFCLIENT pClient, SHFLROOT root, SHFLSTRI
 
     data.name.type                   = VMMDevHGCMParmType_LinAddr;
     data.name.u.Pointer.size         = size;
-    data.name.u.Pointer.u.linearAddr = (RTGCPTR)pString;
+    data.name.u.Pointer.u.linearAddr = (uintptr_t)pString;
 
     rc = VbglHGCMCall (pClient->handle, &data.callInfo, sizeof (data));
 
 /*    Log(("VBOXSF: VBoxSF::vboxCallQueryMapName: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -191,7 +200,7 @@ DECLVBGL(int) vboxCallMapFolder(PVBSFCLIENT pClient, PSHFLSTRING szFolderName,
 
     data.path.type                    = VMMDevHGCMParmType_LinAddr;
     data.path.u.Pointer.size          = ShflStringSizeOfBuffer (szFolderName);
-    data.path.u.Pointer.u.linearAddr  = (RTGCPTR)szFolderName;
+    data.path.u.Pointer.u.linearAddr  = (uintptr_t)szFolderName;
 
     data.root.type                    = VMMDevHGCMParmType_32bit;
     data.root.u.value32               = 0;
@@ -211,7 +220,7 @@ DECLVBGL(int) vboxCallMapFolder(PVBSFCLIENT pClient, PSHFLSTRING szFolderName,
 /*    Log(("VBOXSF: VBoxSF::vboxCallMapFolder: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         pMap->root = data.root.u.value32;
         rc         = data.callInfo.result;
@@ -226,7 +235,7 @@ DECLVBGL(int) vboxCallMapFolder(PVBSFCLIENT pClient, PSHFLSTRING szFolderName,
 
         data.path.type                    = VMMDevHGCMParmType_LinAddr;
         data.path.u.Pointer.size          = ShflStringSizeOfBuffer (szFolderName);
-        data.path.u.Pointer.u.linearAddr  = (RTGCPTR)szFolderName;
+        data.path.u.Pointer.u.linearAddr  = (uintptr_t)szFolderName;
 
         data.root.type                    = VMMDevHGCMParmType_32bit;
         data.root.u.value32               = 0;
@@ -236,7 +245,7 @@ DECLVBGL(int) vboxCallMapFolder(PVBSFCLIENT pClient, PSHFLSTRING szFolderName,
 
         rc = VbglHGCMCall (pClient->handle, &data.callInfo, sizeof (data));
 
-        if (VBOX_SUCCESS (rc))
+        if (RT_SUCCESS (rc))
         {
             pMap->root = data.root.u.value32;
             rc         = data.callInfo.result;
@@ -261,7 +270,7 @@ DECLVBGL(int) vboxCallUnmapFolder(PVBSFCLIENT pClient, PVBSFMAP pMap)
 /*    Log(("VBOXSF: VBoxSF::vboxCallUnmapFolder: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc         = data.callInfo.result;
     }
@@ -283,18 +292,18 @@ DECLVBGL(int) vboxCallCreate (PVBSFCLIENT pClient, PVBSFMAP pMap,
 
     data.path.type                    = VMMDevHGCMParmType_LinAddr;
     data.path.u.Pointer.size          = ShflStringSizeOfBuffer (pParsedPath);
-    data.path.u.Pointer.u.linearAddr  = (RTGCPTR)pParsedPath;
+    data.path.u.Pointer.u.linearAddr  = (uintptr_t)pParsedPath;
 
     data.parms.type                   = VMMDevHGCMParmType_LinAddr;
     data.parms.u.Pointer.size         = sizeof (SHFLCREATEPARMS);
-    data.parms.u.Pointer.u.linearAddr = (RTGCPTR)pCreateParms;
+    data.parms.u.Pointer.u.linearAddr = (uintptr_t)pCreateParms;
 
     rc = VbglHGCMCall (pClient->handle, &data.callInfo, sizeof (data));
 
 /*    Log(("VBOXSF: VBoxSF::vboxCallCreate: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -320,7 +329,7 @@ DECLVBGL(int) vboxCallClose (PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE Hand
 /*    Log(("VBOXSF: VBoxSF::vboxCallClose: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -342,7 +351,7 @@ DECLVBGL(int) vboxCallRemove (PVBSFCLIENT pClient, PVBSFMAP pMap,
 
     data.path.type                      = VMMDevHGCMParmType_LinAddr_In;
     data.path.u.Pointer.size            = ShflStringSizeOfBuffer (pParsedPath);
-    data.path.u.Pointer.u.linearAddr    = (RTGCPTR)pParsedPath;
+    data.path.u.Pointer.u.linearAddr    = (uintptr_t)pParsedPath;
 
     data.flags.type                     = VMMDevHGCMParmType_32bit;
     data.flags.u.value32                = flags;
@@ -352,7 +361,7 @@ DECLVBGL(int) vboxCallRemove (PVBSFCLIENT pClient, PVBSFMAP pMap,
 /*    Log(("VBOXSF: VBoxSF::vboxCallRemove: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -374,11 +383,11 @@ DECLVBGL(int) vboxCallRename (PVBSFCLIENT pClient, PVBSFMAP pMap,
 
     data.src.type                       = VMMDevHGCMParmType_LinAddr_In;
     data.src.u.Pointer.size             = ShflStringSizeOfBuffer (pSrcPath);
-    data.src.u.Pointer.u.linearAddr     = (RTGCPTR)pSrcPath;
+    data.src.u.Pointer.u.linearAddr     = (uintptr_t)pSrcPath;
 
     data.dest.type                      = VMMDevHGCMParmType_LinAddr_In;
     data.dest.u.Pointer.size            = ShflStringSizeOfBuffer (pDestPath);
-    data.dest.u.Pointer.u.linearAddr    = (RTGCPTR)pDestPath;
+    data.dest.u.Pointer.u.linearAddr    = (uintptr_t)pDestPath;
 
     data.flags.type                     = VMMDevHGCMParmType_32bit;
     data.flags.u.value32                = flags;
@@ -388,7 +397,7 @@ DECLVBGL(int) vboxCallRename (PVBSFCLIENT pClient, PVBSFMAP pMap,
 /*    Log(("VBOXSF: VBoxSF::vboxCallRename: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -415,14 +424,14 @@ DECLVBGL(int) vboxCallRead(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFile,
     data.cb.u.value32                   = *pcbBuffer;
     data.buffer.type                    = (fLocked) ? VMMDevHGCMParmType_LinAddr_Locked_Out : VMMDevHGCMParmType_LinAddr_Out;
     data.buffer.u.Pointer.size          = *pcbBuffer;
-    data.buffer.u.Pointer.u.linearAddr  = (RTGCPTR)pBuffer;
+    data.buffer.u.Pointer.u.linearAddr  = (uintptr_t)pBuffer;
 
     rc = VbglHGCMCall (pClient->handle, &data.callInfo, sizeof (data));
 
 /*    Log(("VBOXSF: VBoxSF::vboxCallRead: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
         *pcbBuffer = data.cb.u.value32;
@@ -450,14 +459,14 @@ DECLVBGL(int) vboxCallWrite(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFile
     data.cb.u.value32                   = *pcbBuffer;
     data.buffer.type                    = (fLocked) ? VMMDevHGCMParmType_LinAddr_Locked_In : VMMDevHGCMParmType_LinAddr_In;
     data.buffer.u.Pointer.size          = *pcbBuffer;
-    data.buffer.u.Pointer.u.linearAddr  = (RTGCPTR)pBuffer;
+    data.buffer.u.Pointer.u.linearAddr  = (uintptr_t)pBuffer;
 
     rc = VbglHGCMCall (pClient->handle, &data.callInfo, sizeof (data));
 
 /*    Log(("VBOXSF: VBoxSF::vboxCallWrite: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
         *pcbBuffer = data.cb.u.value32;
@@ -465,6 +474,53 @@ DECLVBGL(int) vboxCallWrite(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFile
     return rc;
 }
 
+DECLVBGL(int) VbglR0SfWritePhysCont(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFile, uint64_t offset, uint32_t *pcbBuffer, RTCCPHYS PhysBuffer)
+{
+    uint32_t            cbToWrite = *pcbBuffer;
+    uint32_t            cPages    = RT_ALIGN_32((PhysBuffer & PAGE_OFFSET_MASK) + cbToWrite, PAGE_SIZE) >> PAGE_SHIFT;
+    uint32_t            cbData    = sizeof(VBoxSFWrite) + RT_UOFFSETOF(HGCMPageListInfo, aPages[cPages]);
+    VBoxSFWrite        *pData     = (VBoxSFWrite *)RTMemTmpAlloc(cbData);
+    HGCMPageListInfo   *pPgLst    = (HGCMPageListInfo *)(pData + 1);
+    uint32_t            iPage;
+    int                 rc;
+
+    if (RT_UNLIKELY(!pData))
+        return VERR_NO_TMP_MEMORY;
+
+    VBOX_INIT_CALL(&pData->callInfo, WRITE, pClient);
+
+    pData->root.type                      = VMMDevHGCMParmType_32bit;
+    pData->root.u.value32                 = pMap->root;
+
+    pData->handle.type                    = VMMDevHGCMParmType_64bit;
+    pData->handle.u.value64               = hFile;
+    pData->offset.type                    = VMMDevHGCMParmType_64bit;
+    pData->offset.u.value64               = offset;
+    pData->cb.type                        = VMMDevHGCMParmType_32bit;
+    pData->cb.u.value32                   = cbToWrite;
+    pData->buffer.type                    = VMMDevHGCMParmType_PageList;
+    pData->buffer.u.PageList.size         = cbToWrite;
+    pData->buffer.u.PageList.offset       = sizeof(VBoxSFWrite);
+
+    pPgLst->flags = VBOX_HGCM_F_PARM_DIRECTION_TO_HOST;
+    pPgLst->offFirstPage = PhysBuffer & PAGE_OFFSET_MASK;
+    pPgLst->cPages = cPages;
+    PhysBuffer &= ~(RTCCPHYS)PAGE_OFFSET_MASK;
+    for (iPage = 0; iPage < cPages; iPage++, PhysBuffer += PAGE_SIZE)
+        pPgLst->aPages[iPage] = PhysBuffer;
+
+    rc = VbglHGCMCall (pClient->handle, &pData->callInfo, cbData);
+    if (RT_SUCCESS (rc))
+    {
+        rc = pData->callInfo.result;
+        *pcbBuffer = pData->cb.u.value32;
+    }
+
+    RTMemTmpFree(pData);
+    return rc;
+
+}
+
 DECLVBGL(int) vboxCallFlush(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFile)
 {
     int rc = VINF_SUCCESS;
@@ -484,7 +540,7 @@ DECLVBGL(int) vboxCallFlush(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFile
 /*    Log(("VBOXSF: VBoxSF::vboxCallFlush: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -520,11 +576,11 @@ DECLVBGL(int) vboxCallDirInfo (
     data.path.type                      = VMMDevHGCMParmType_LinAddr_In;
     data.path.u.Pointer.size            =
         (ParsedPath) ? ShflStringSizeOfBuffer(ParsedPath) : 0;
-    data.path.u.Pointer.u.linearAddr    = (RTGCPTR) ParsedPath;
+    data.path.u.Pointer.u.linearAddr    = (uintptr_t) ParsedPath;
 
     data.buffer.type                    = VMMDevHGCMParmType_LinAddr_Out;
     data.buffer.u.Pointer.size          = *pcbBuffer;
-    data.buffer.u.Pointer.u.linearAddr  = (RTGCPTR)pBuffer;
+    data.buffer.u.Pointer.u.linearAddr  = (uintptr_t)pBuffer;
 
     data.resumePoint.type               = VMMDevHGCMParmType_32bit;
     data.resumePoint.u.value32          = index;
@@ -536,7 +592,7 @@ DECLVBGL(int) vboxCallDirInfo (
 /*    Log(("VBOXSF: VBoxSF::vboxCallDirInfo: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -565,14 +621,14 @@ DECLVBGL(int) vboxCallFSInfo(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFil
     data.cb.u.value32                   = *pcbBuffer;
     data.info.type                      = VMMDevHGCMParmType_LinAddr;
     data.info.u.Pointer.size            = *pcbBuffer;
-    data.info.u.Pointer.u.linearAddr    = (RTGCPTR)pBuffer;
+    data.info.u.Pointer.u.linearAddr    = (uintptr_t)pBuffer;
 
     rc = VbglHGCMCall (pClient->handle, &data.callInfo, sizeof (data));
 
 /*    Log(("VBOXSF: VBoxSF::vboxCallFileInfo: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
         *pcbBuffer = data.cb.u.value32;
@@ -607,7 +663,7 @@ DECLVBGL(int) vboxCallLock(PVBSFCLIENT pClient, PVBSFMAP pMap, SHFLHANDLE hFile,
 /*    Log(("VBOXSF: VBoxSF::vboxCallLock: "
          "VbglHGCMCall rc = %#x, result = %#x\n", rc, data.callInfo.result));
 */
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = data.callInfo.result;
     }
@@ -622,7 +678,7 @@ DECLVBGL(int) vboxCallSetUtf8 (PVBSFCLIENT pClient)
 
     VBOX_INIT_CALL (&callInfo, SET_UTF8, pClient);
     rc = VbglHGCMCall (pClient->handle, &callInfo, sizeof (callInfo));
-    if (VBOX_SUCCESS (rc))
+    if (RT_SUCCESS (rc))
     {
         rc = callInfo.result;
     }
