@@ -14,6 +14,15 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
  * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
  * Clara, CA 95054 USA or visit http://www.sun.com if you need
  * additional information or have any questions.
@@ -31,20 +40,24 @@
 # define INCL_ERRORS
 # include <os2.h>
 
-#elif defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
+#elif defined(RT_OS_FREEBSD) \
+   || defined(RT_OS_LINUX) \
+   || defined(RT_OS_SOLARIS)
 # include <sys/types.h>
 # include <sys/stat.h>
+# if defined(RT_OS_LINUX) /** @todo check this on solaris+freebsd as well. */
+#  include <sys/ioctl.h>
+# endif
 # include <errno.h>
 # include <unistd.h>
 #endif
 
-#include <iprt/time.h>
-#include <iprt/asm.h>
-#include <iprt/string.h>
-#include <iprt/file.h>
 #include <iprt/assert.h>
+#include <iprt/asm.h>
+#include <iprt/file.h>
+#include <iprt/time.h>
+#include <iprt/string.h>
 #include <iprt/thread.h>
-#include <VBox/VBoxGuest.h>
 #include <VBox/log.h>
 #include "VBGLR3Internal.h"
 
@@ -53,11 +66,12 @@
    prototype what we need here. */
 # define XF86_O_RDWR  0x0002
 typedef void *pointer;
-extern "C" int xf86open(const char*, int,...);
+extern "C" int xf86open(const char *, int, ...);
 extern "C" int xf86close(int);
 extern "C" int xf86ioctl(int, unsigned long, pointer);
 #endif
 
+
 /*******************************************************************************
 *   Global Variables                                                           *
 *******************************************************************************/
@@ -77,7 +91,11 @@ static RTFILE g_File = NIL_RTFILE;
 static uint32_t volatile g_cInits = 0;
 
 
-VBGLR3DECL(int) VbglR3Init(void)
+
+/**
+ * Implementation of VbglR3Init and VbglR3InitUser
+ */
+static int vbglR3Init(const char *pszDeviceName)
 {
     uint32_t cInits = ASMAtomicIncU32(&g_cInits);
 #ifndef VBOX_VBGLR3_XFREE86
@@ -114,7 +132,7 @@ VBGLR3DECL(int) VbglR3Init(void)
      * Have to use CreateFile here as we want to specify FILE_FLAG_OVERLAPPED
      * and possible some other bits not availble thru iprt/file.h.
      */
-    HANDLE hFile = CreateFile(VBOXGUEST_DEVICE_NAME,
+    HANDLE hFile = CreateFile(pszDeviceName,
                               GENERIC_READ | GENERIC_WRITE,
                               FILE_SHARE_READ | FILE_SHARE_WRITE,
                               NULL,
@@ -134,7 +152,7 @@ VBGLR3DECL(int) VbglR3Init(void)
      */
     HFILE hf = NULLHANDLE;
     ULONG ulAction = 0;
-    APIRET rc = DosOpen((PCSZ)VBOXGUEST_DEVICE_NAME, &hf, &ulAction, 0, FILE_NORMAL,
+    APIRET rc = DosOpen((PCSZ)pszDeviceName, &hf, &ulAction, 0, FILE_NORMAL,
                         OPEN_ACTION_OPEN_IF_EXISTS,
                         OPEN_FLAGS_FAIL_ON_ERROR | OPEN_FLAGS_NOINHERIT | OPEN_SHARE_DENYNONE | OPEN_ACCESS_READWRITE,
                         NULL);
@@ -177,52 +195,17 @@ VBGLR3DECL(int) VbglR3Init(void)
     }
     g_File = hf;
 
-#elif defined(RT_OS_FREEBSD)
-    /*
-     * Try open the BSD device. The device cloning makes this a bit of work.
-     */
-# if defined(VBOX_VBGLR3_XFREE86)
-    int File = 0;
-# else
-    RTFILE File = 0;
-# endif
-    int rc;
-    char szDevice[sizeof(VBOXGUEST_DEVICE_NAME) + 16];
-    for (unsigned iUnit = 0; iUnit < 1024; iUnit++)
-    {
-        RTStrPrintf(szDevice, sizeof(szDevice), VBOXGUEST_DEVICE_NAME "%d", iUnit);
-# if defined(VBOX_VBGLR3_XFREE86)
-        File = xf86open(szDevice, XF86_O_RDWR);
-        if (File >= 0)
-            break;
-# else
-        rc = RTFileOpen(&File, szDevice, RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
-        if (RT_SUCCESS(rc))
-            break;
-# endif
-    }
-
-# if defined(VBOX_VBGLR3_XFREE86)
-    if (File == -1)
-        return VERR_OPEN_FAILED;
-# else
-    if (RT_FAILURE(rc))
-        return rc;
-# endif
-
-    g_File = File;
-
-#elif defined(VBOX_VBGLR3_XFREE86) && !defined(RT_OS_FREEBSD)
-    int File = xf86open(VBOXGUEST_DEVICE_NAME, XF86_O_RDWR);
+#elif defined(VBOX_VBGLR3_XFREE86)
+    int File = xf86open(pszDeviceName, XF86_O_RDWR);
     if (File == -1)
         return VERR_OPEN_FAILED;
     g_File = File;
 
 #else
 
-    /* The default implemenation. (linux, solaris) */
+    /* The default implemenation. (linux, solaris, freebsd) */
     RTFILE File;
-    int rc = RTFileOpen(&File, VBOXGUEST_DEVICE_NAME, RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
+    int rc = RTFileOpen(&File, pszDeviceName, RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
     if (RT_FAILURE(rc))
         return rc;
     g_File = File;
@@ -245,6 +228,26 @@ VBGLR3DECL(int) VbglR3Init(void)
 }
 
 
+/**
+ * Open the VBox R3 Guest Library.  This should be called by system daemons
+ * and processes.
+ */
+VBGLR3DECL(int) VbglR3Init(void)
+{
+    return vbglR3Init(VBOXGUEST_DEVICE_NAME);
+}
+
+
+/**
+ * Open the VBox R3 Guest Library.  Equivalent to VbglR3Init, but for user
+ * session processes.
+ */
+VBGLR3DECL(int) VbglR3InitUser(void)
+{
+    return vbglR3Init(VBOXGUEST_USER_DEVICE_NAME);
+}
+
+
 VBGLR3DECL(void) VbglR3Term(void)
 {
     /*
@@ -307,7 +310,7 @@ int vbglR3DoIOCtl(unsigned iFunction, void *pvData, size_t cbData)
 {
 #if defined(RT_OS_WINDOWS)
     DWORD cbReturned = 0;
-    if (!DeviceIoControl(g_hFile, iFunction, pvData, cbData, pvData, cbData, &cbReturned, NULL))
+    if (!DeviceIoControl(g_hFile, iFunction, pvData, (DWORD)cbData, pvData, (DWORD)cbData, &cbReturned, NULL))
     {
 /** @todo The passing of error codes needs to be tested and fixed (as does *all* the other hosts except for
  * OS/2).  The idea is that the VBox status codes in ring-0 should be transfered without loss down to
@@ -336,8 +339,15 @@ int vbglR3DoIOCtl(unsigned iFunction, void *pvData, size_t cbData)
     Hdr.u32Magic = VBGLBIGREQ_MAGIC;
     Hdr.cbData = cbData;
     Hdr.pvDataR3 = pvData;
+# if HC_ARCH_BITS == 32
+    Hdr.u32Padding = 0;
+# endif
 
-/** @todo test status code passing! */
+/** @todo test status code passing! Check that the kernel doesn't do any
+ *        error checks using specific errno values, and just pass an VBox
+ *        error instead of an errno.h one. Alternatively, extend/redefine the
+ *        header with an error code return field (much better alternative
+ *        actually). */
     int rc = ioctl((int)g_File, iFunction, &Hdr);
     if (rc == -1)
     {
@@ -346,6 +356,27 @@ int vbglR3DoIOCtl(unsigned iFunction, void *pvData, size_t cbData)
     }
     return VINF_SUCCESS;
 
+#elif defined(RT_OS_LINUX)
+# ifdef VBOX_VBGLR3_XFREE86
+    int rc = xf86ioctl((int)g_File, iFunction, pvData);
+# else
+    int rc = ioctl((int)g_File, iFunction, pvData);
+# endif
+    if (RT_LIKELY(rc == 0))
+        return VINF_SUCCESS;
+
+    /* Positive values are negated VBox error status codes. */
+    if (rc > 0)
+        rc = -rc;
+    else
+# ifdef VBOX_VBGLR3_XFREE86
+        rc = VERR_FILE_IO_ERROR;
+#  else
+        rc = RTErrConvertFromErrno(errno);
+# endif
+    NOREF(cbData);
+    return rc;
+
 #elif defined(VBOX_VBGLR3_XFREE86)
     /* PORTME - This is preferred over the RTFileIOCtl variant below, just be careful with the (int). */
 /** @todo test status code passing! */
