@@ -14,6 +14,15 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
  * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
  * Clara, CA 95054 USA or visit http://www.sun.com if you need
  * additional information or have any questions.
@@ -48,10 +57,10 @@
 # include <errno.h>
 #endif
 
-#include <iprt/string.h>
 #include <iprt/file.h>
 #include <iprt/process.h>
-#include <VBox/VBoxGuest.h>
+#include <iprt/string.h>
+#include "VBGLR3Internal.h"
 
 
 /**
@@ -146,7 +155,7 @@ VBGLR3DECL(int) VbglR3Daemonize(bool fNoChDir, bool fNoClose)
 
     struct sigaction OldSigAct;
     struct sigaction SigAct;
-    memset(&SigAct, 0, sizeof(SigAct));
+    RT_ZERO(SigAct);
     SigAct.sa_handler = SIG_IGN;
     int rcSigAct = sigaction(SIGHUP, &SigAct, &OldSigAct);
 
