@@ -1,4 +1,4 @@
-/** $Id: VBoxServiceClipboard-os2.cpp 30496 2008-05-05 13:56:33Z bird $ */
+/** $Id: VBoxServiceClipboard-os2.cpp $ */
 /** @file
  * VBoxService - Guest Additions Clipboard Service, OS/2.
  */
@@ -36,7 +36,7 @@
 #include <iprt/param.h>
 #include <iprt/assert.h>
 #include <iprt/asm.h>
-#include <VBox/VBoxGuest.h>
+#include <VBox/VBoxGuestLib.h>
 #include <VBox/HostServices/VBoxClipboardSvc.h>
 #include "VBoxServiceInternal.h"
 
