@@ -35,7 +35,7 @@
 #include <iprt/semaphore.h>
 #include <iprt/thread.h>
 #include <VBox/version.h>
-#include <VBox/VBoxGuest.h>
+#include <VBox/VBoxGuestLib.h>
 #include "VBoxServiceInternal.h"
 #include "VBoxServiceUtils.h"
 
@@ -133,46 +133,11 @@ static int VBoxServiceExecValidateFlags(const char *pszFlags)
  */
 static int VBoxServiceExecReadHostProp(const char *pszPropName, char **ppszValue, uint64_t *puTimestamp)
 {
-    size_t  cbBuf = _1K;
-    void   *pvBuf = NULL;
-    int     rc;
-
-    *ppszValue = NULL;
-
-    for (unsigned cTries = 0; cTries < 10; cTries++)
+    char *pszFlags;
+    uint64_t uTimestamp;
+    int rc = VBoxServiceReadProp(g_uExecGuestPropSvcClientID, pszPropName, ppszValue, &pszFlags, &uTimestamp);
+    if (RT_SUCCESS(rc))
     {
-        /*
-         * (Re-)Allocate the buffer and try read the property.
-         */
-        RTMemFree(pvBuf);
-        pvBuf = RTMemAlloc(cbBuf);
-        if (!pvBuf)
-        {
-            VBoxServiceError("Exec: Failed to allocate %zu bytes\n", cbBuf);
-            rc = VERR_NO_MEMORY;
-            break;
-        }
-        char    *pszValue;
-        char    *pszFlags;
-        uint64_t uTimestamp;
-        rc = VbglR3GuestPropRead(g_uExecGuestPropSvcClientID, pszPropName,
-                                 pvBuf, cbBuf,
-                                 &pszValue, &uTimestamp, &pszFlags, NULL);
-        if (RT_FAILURE(rc))
-        {
-            if (rc == VERR_BUFFER_OVERFLOW)
-            {
-                /* try again with a bigger buffer. */
-                cbBuf *= 2;
-                continue;
-            }
-            if (rc == VERR_NOT_FOUND)
-                VBoxServiceVerbose(2, "Exec: %s not found\n", pszPropName);
-            else
-                VBoxServiceError("Exec: Failed to query \"%s\": %Rrc\n", pszPropName, rc);
-            break;
-        }
-
         /*
          * Validate it and set return values on success.
          */
@@ -183,24 +148,16 @@ static int VBoxServiceExecReadHostProp(const char *pszPropName, char **ppszValue
             if (++s_cBitched < 10)
                 VBoxServiceError("Exec: Flag validation failed for \"%s\": %Rrc; flags=\"%s\"\n",
                                  pszPropName, rc, pszFlags);
-            break;
         }
-        VBoxServiceVerbose(2, "Exec: Read \"%s\" = \"%s\", timestamp %RU64n\n",
-                           pszPropName, pszValue, uTimestamp);
-        *ppszValue = RTStrDup(pszValue);
-        if (!*ppszValue)
+        else
         {
-            VBoxServiceError("Exec: RTStrDup failed for \"%s\"\n", pszValue);
-            rc = VERR_NO_MEMORY;
-            break;
+            VBoxServiceVerbose(2, "Exec: Read \"%s\" = \"%s\", timestamp %RU64n\n",
+                               pszPropName, *ppszValue, uTimestamp);
+            if (puTimestamp)
+                *puTimestamp = uTimestamp;
         }
-
-        if (puTimestamp)
-            *puTimestamp = uTimestamp;
-        break; /* done */
+        RTStrFree(pszFlags);
     }
-
-    RTMemFree(pvBuf);
     return rc;
 }
 
@@ -346,7 +303,7 @@ DECLCALLBACK(int) VBoxServiceExecWorker(bool volatile *pfShutdown)
             char szSysprepCmd[RTPATH_MAX] = "C:\\sysprep\\sysprep.exe";
             OSVERSIONINFOEX OSInfoEx;
             RT_ZERO(OSInfoEx);
-            OSInfoEx.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+            OSInfoEx.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
             if (    GetVersionEx((LPOSVERSIONINFO) &OSInfoEx)
                 &&  OSInfoEx.dwPlatformId == VER_PLATFORM_WIN32_NT
                 &&  OSInfoEx.dwMajorVersion >= 6 /* Vista or later */)
@@ -398,11 +355,9 @@ DECLCALLBACK(int) VBoxServiceExecWorker(bool volatile *pfShutdown)
                                     /*
                                      * Store the result in Set return value so the host knows what happend.
                                      */
-                                    rc = VbglR3GuestPropWriteValueF(g_uExecGuestPropSvcClientID,
-                                                                    "/VirtualBox/HostGuest/SysprepRet",
-                                                                    "%d", Status.iStatus);
-                                    if (RT_FAILURE(rc))
-                                        VBoxServiceError("Exec: Failed to write SysprepRet: rc=%Rrc\n", rc);
+                                    VBoxServiceWritePropF(g_uExecGuestPropSvcClientID,
+                                                          "/VirtualBox/HostGuest/SysprepRet",
+                                                          "%d", Status.iStatus);
                                 }
                                 else
                                     VBoxServiceError("Exec: RTProcWait failed for sysprep: %Rrc\n", rc);
@@ -436,11 +391,15 @@ DECLCALLBACK(int) VBoxServiceExecWorker(bool volatile *pfShutdown)
                 &&  rc != VERR_FILE_NOT_FOUND)
             {
                 VBoxServiceVerbose(1, "Exec: Stopping sysprep processing (rc=%Rrc)\n", rc);
-                rc = VbglR3GuestPropWriteValueF(g_uExecGuestPropSvcClientID, "/VirtualBox/HostGuest/SysprepVBoxRC", "%d", rc);
-                if (RT_FAILURE(rc))
-                    VBoxServiceError("Exec: Failed to write SysprepVBoxRC: rc=%Rrc\n", rc);
                 fSysprepDone = true;
             }
+
+            /*
+             * Always let the host know what happend, except when the guest property
+             * value is empty/missing.
+             */
+            if (rc != VERR_NOT_FOUND)
+                VBoxServiceWritePropF(g_uExecGuestPropSvcClientID, "/VirtualBox/HostGuest/SysprepVBoxRC", "%d", rc);
         }
 
 #ifdef FULL_FEATURED_EXEC
@@ -502,8 +461,11 @@ static DECLCALLBACK(void) VBoxServiceExecTerm(void)
     VbglR3GuestPropDisconnect(g_uExecGuestPropSvcClientID);
     g_uExecGuestPropSvcClientID = 0;
 
-    RTSemEventMultiDestroy(g_hExecEvent);
-    g_hExecEvent = NIL_RTSEMEVENTMULTI;
+    if (g_hExecEvent != NIL_RTSEMEVENTMULTI)
+    {
+        RTSemEventMultiDestroy(g_hExecEvent);
+        g_hExecEvent = NIL_RTSEMEVENTMULTI;
+    }
 }
 
 
