@@ -79,18 +79,10 @@ static SwapBuffersFunc_t origSwapBuffers;
 static DrawBufferFunc_t origDrawBuffer;
 static ScissorFunc_t origScissor;
 
-static void stubCheckWindowState(void)
+static void stubCheckWindowState(WindowInfo *window)
 {
-    WindowInfo *window;
     bool bForceUpdate = false;
 
-    CRASSERT(stub.trackWindowSize || stub.trackWindowPos);
-
-    if (!stub.currentContext)
-        return;
-
-    window = stub.currentContext->currentDrawable;
-
 #ifdef WINDOWS
     /* @todo install hook and track for WM_DISPLAYCHANGE */
     {
@@ -128,6 +120,64 @@ static void stubCheckWindowState(void)
     }
 }
 
+static bool stubSystemWindowExist(WindowInfo *pWindow)
+{
+#ifdef WINDOWS
+    if (!WindowFromDC(pWindow->drawable))
+    {
+        return false;
+    }
+#else
+    Window root;
+    int x, y;
+    unsigned int border, depth, w, h;
+
+    if (!XGetGeometry(pWindow->dpy, pWindow->drawable, &root, &x, &y, &w, &h, &border, &depth))
+    {
+        return false;
+    }
+#endif
+
+    return true;
+}
+
+static void stubCheckWindowsCB(unsigned long key, void *data1, void *data2)
+{
+    WindowInfo *pWindow = (WindowInfo *) data1;
+    ContextInfo *pCtx = (ContextInfo *) data2;
+
+    if (pWindow == pCtx->currentDrawable
+        || pWindow->type!=CHROMIUM
+        || pWindow->pOwner!=pCtx)
+    {
+        return;
+    }
+
+    if (!stubSystemWindowExist(pWindow))
+    {
+#ifdef WINDOWS
+        crWindowDestroy((GLint)pWindow->hWnd);
+#else
+        crWindowDestroy((GLint)pWindow->drawable);
+#endif
+        return;
+    }
+
+    stubCheckWindowState(pWindow);
+}
+
+static void stubCheckWindowsState(void)
+{
+    CRASSERT(stub.trackWindowSize || stub.trackWindowPos);
+
+    if (!stub.currentContext)
+        return;
+
+    stubCheckWindowState(stub.currentContext->currentDrawable);
+
+    crHashtableWalk(stub.windowTable, stubCheckWindowsCB, stub.currentContext);
+}
+
 
 /**
  * Override the head SPU's glClear function.
@@ -136,7 +186,7 @@ static void stubCheckWindowState(void)
  */
 static void SPU_APIENTRY trapClear(GLbitfield mask)
 {
-    stubCheckWindowState();
+    stubCheckWindowsState();
     /* call the original SPU glClear function */
     origClear(mask);
 }
@@ -147,7 +197,7 @@ static void SPU_APIENTRY trapClear(GLbitfield mask)
  */
 static void SPU_APIENTRY trapViewport(GLint x, GLint y, GLsizei w, GLsizei h)
 {
-    stubCheckWindowState();
+    stubCheckWindowsState();
     /* call the original SPU glViewport function */
     if (!stub.viewportHack)
     {
@@ -166,13 +216,13 @@ static void SPU_APIENTRY trapViewport(GLint x, GLint y, GLsizei w, GLsizei h)
 
 static void SPU_APIENTRY trapSwapBuffers(GLint window, GLint flags)
 {
-    stubCheckWindowState();
+    stubCheckWindowsState();
     origSwapBuffers(window, flags);
 }
 
 static void SPU_APIENTRY trapDrawBuffer(GLenum buf)
 {
-    stubCheckWindowState();
+    stubCheckWindowsState();
     origDrawBuffer(buf);
 }
 
@@ -324,7 +374,7 @@ static void stubInitVars(void)
     stub.matchChromiumWindowCount = 0;
     stub.matchChromiumWindowID = NULL;
     stub.matchWindowTitle = NULL;
-    stub.ignoreFreeglutMenus = 1;
+    stub.ignoreFreeglutMenus = 0;
     stub.threadSafe = GL_FALSE;
     stub.trackWindowSize = 0;
     stub.trackWindowPos = 0;
@@ -504,7 +554,7 @@ void stubSetDefaultConfigurationOptions(void)
     stub.matchChromiumWindowID = NULL;
     stub.numIgnoreWindowID = 0;
     stub.matchWindowTitle = NULL;
-    stub.ignoreFreeglutMenus = 1;
+    stub.ignoreFreeglutMenus = 0;
     stub.trackWindowSize = 1;
     stub.trackWindowPos = 1;
     stub.trackWindowVisibility = 1;
@@ -523,13 +573,17 @@ void stubSetDefaultConfigurationOptions(void)
         char name[1000];
         int i;
 
-        crGetProcName(name, 1000);
-        for (i=0; gsViewportHackApps[i]; ++i)
+        /* Apply viewport hack only if we're running under wine */
+        if (NULL!=GetModuleHandle("wined3d.dll"))
         {
-            if (!stricmp(name, gsViewportHackApps[i]))
+            crGetProcName(name, 1000);
+            for (i=0; gsViewportHackApps[i]; ++i)
             {
-                stub.viewportHack = 1;
-                break;
+                if (!stricmp(name, gsViewportHackApps[i]))
+                {
+                    stub.viewportHack = 1;
+                    break;
+                }
             }
         }
     }
@@ -588,7 +642,7 @@ stubInit(void)
     }
 #endif
 
-    strcpy(response, "3 0 array 1 feedback 2 pack");
+    strcpy(response, "2 0 feedback 1 pack");
     spuchain = crStrSplit( response, " " );
     num_spus = crStrToInt( spuchain[0] );
     spu_ids = (int *) crAlloc( num_spus * sizeof( *spu_ids ) );
