@@ -1,6 +1,8 @@
 #! /bin/sh
 # Sun VirtualBox
-# Linux Additions kernel module init script
+# Linux Additions kernel module init script ($Revision: 58884 $)
+#
+
 #
 # Copyright (C) 2006-2009 Sun Microsystems, Inc.
 #
@@ -18,7 +20,7 @@
 #
 
 
-# chkconfig: 35 30 70
+# chkconfig: 357 30 70
 # description: VirtualBox Linux Additions kernel modules
 #
 ### BEGIN INIT INFO
@@ -31,10 +33,33 @@
 ### END INIT INFO
 
 PATH=$PATH:/bin:/sbin:/usr/sbin
-BUILDVBOXADD=`/bin/ls /usr/src/vboxadd*/build_in_tmp 2>/dev/null|cut -d' ' -f1`
+PACKAGE=VBoxGuestAdditions
+BUILDVBOXGUEST=`/bin/ls /usr/src/vboxguest*/build_in_tmp 2>/dev/null|cut -d' ' -f1`
 BUILDVBOXVFS=`/bin/ls /usr/src/vboxvfs*/build_in_tmp 2>/dev/null|cut -d' ' -f1`
 BUILDVBOXVIDEO=`/bin/ls /usr/src/vboxvideo*/build_in_tmp 2>/dev/null|cut -d' ' -f1`
 LOG="/var/log/vboxadd-install.log"
+MODPROBE=/sbin/modprobe
+
+if $MODPROBE -c | grep -q '^allow_unsupported_modules  *0'; then
+  MODPROBE="$MODPROBE --allow-unsupported-modules"
+fi
+
+# Check architecture
+cpu=`uname -m`;
+case "$cpu" in
+  i[3456789]86|x86)
+    cpu="x86"
+    lib_path="/usr/lib"
+    ;;
+  x86_64|amd64)
+    cpu="amd64"
+    if test -d "/usr/lib64"; then
+      lib_path="/usr/lib64"
+    else
+      lib_path="/usr/lib"
+    fi
+    ;;
+esac
 
 if [ -f /etc/arch-release ]; then
     system=arch
@@ -141,7 +166,7 @@ if [ "$system" = "other" ]; then
     }
 fi
 
-dev=/dev/vboxadd
+dev=/dev/vboxguest
 userdev=/dev/vboxuser
 owner=vboxadd
 group=1
@@ -157,6 +182,11 @@ fail()
     exit 1
 }
 
+running_vboxguest()
+{
+    lsmod | grep -q "vboxguest[^_-]"
+}
+
 running_vboxadd()
 {
     lsmod | grep -q "vboxadd[^_-]"
@@ -169,8 +199,8 @@ running_vboxvfs()
 
 start()
 {
-    begin "Starting VirtualBox Additions ";
-    running_vboxadd || {
+    begin "Starting the VirtualBox Guest Additions ";
+    running_vboxguest || {
         rm -f $dev || {
             fail "Cannot remove $dev"
         }
@@ -179,35 +209,35 @@ start()
             fail "Cannot remove $userdev"
         }
 
-        modprobe vboxadd >/dev/null 2>&1 || {
-            fail "modprobe vboxadd failed"
+        $MODPROBE vboxguest >/dev/null 2>&1 || {
+            fail "modprobe vboxguest failed"
         }
         sleep .5
     }
     if [ ! -c $dev ]; then
-        maj=`sed -n 's;\([0-9]\+\) vboxadd;\1;p' /proc/devices`
+        maj=`sed -n 's;\([0-9]\+\) vboxguest;\1;p' /proc/devices`
         if [ ! -z "$maj" ]; then
             min=0
         else
-            min=`sed -n 's;\([0-9]\+\) vboxadd;\1;p' /proc/misc`
+            min=`sed -n 's;\([0-9]\+\) vboxguest;\1;p' /proc/misc`
             if [ ! -z "$min" ]; then
                 maj=10
             fi
         fi
         test -z "$maj" && {
-            rmmod vboxadd 2>/dev/null
+            rmmod vboxguest 2>/dev/null
             fail "Cannot locate the VirtualBox device"
         }
 
         mknod -m 0664 $dev c $maj $min || {
-            rmmod vboxadd 2>/dev/null
+            rmmod vboxguest 2>/dev/null
             fail "Cannot create device $dev with major $maj and minor $min"
         }
     fi
     chown $owner:$group $dev 2>/dev/null || {
         rm -f $dev 2>/dev/null
         rm -f $userdev 2>/dev/null
-        rmmod vboxadd 2>/dev/null
+        rmmod vboxguest 2>/dev/null
         fail "Cannot change owner $owner:$group for device $dev"
     }
 
@@ -217,13 +247,13 @@ start()
         if [ ! -z "$min" ]; then
             mknod -m 0666 $userdev c $maj $min || {
                 rm -f $dev 2>/dev/null
-                rmmod vboxadd 2>/dev/null
+                rmmod vboxguest 2>/dev/null
                 fail "Cannot create device $userdev with major $maj and minor $min"
             }
             chown $owner:$group $userdev 2>/dev/null || {
                 rm -f $dev 2>/dev/null
                 rm -f $userdev 2>/dev/null
-                rmmod vboxadd 2>/dev/null
+                rmmod vboxguest 2>/dev/null
                 fail "Cannot change owner $owner:$group for device $userdev"
             }
         fi
@@ -231,7 +261,7 @@ start()
 
     if [ -n "$BUILDVBOXVFS" ]; then
         running_vboxvfs || {
-            modprobe vboxvfs > /dev/null 2>&1 || {
+            $MODPROBE vboxvfs > /dev/null 2>&1 || {
                 if dmesg | grep "vboxConnect failed" > /dev/null 2>&1; then
                     fail_msg
                     echo "Unable to start shared folders support.  Make sure that your VirtualBox build"
@@ -254,7 +284,7 @@ start()
 stop()
 {
     begin "Stopping VirtualBox Additions ";
-    if !umount -a -t vboxsf 2>/dev/null; then
+    if ! umount -a -t vboxsf 2>/dev/null; then
         fail "Cannot unmount vboxsf folders"
     fi
     if [ -n "$BUILDVBOXVFS" ]; then
@@ -262,8 +292,8 @@ stop()
             rmmod vboxvfs 2>/dev/null || fail "Cannot unload module vboxvfs"
         fi
     fi
-    if running_vboxadd; then
-        rmmod vboxadd 2>/dev/null || fail "Cannot unload module vboxadd"
+    if running_vboxguest; then
+        rmmod vboxguest 2>/dev/null || fail "Cannot unload module vboxguest"
         rm -f $userdev || fail "Cannot unlink $userdev"
         rm -f $dev || fail "Cannot unlink $dev"
     fi
@@ -277,48 +307,150 @@ restart()
     return 0
 }
 
+# setup_script
 setup()
 {
     # don't stop the old modules here -- they might be in use
+    if find /lib/modules/`uname -r` -name "vboxvideo\.*" 2>/dev/null|grep -q vboxvideo; then
+        begin "Removing old VirtualBox vboxvideo kernel module"
+        find /lib/modules/`uname -r` -name "vboxvideo\.*" 2>/dev/null|xargs rm -f 2>/dev/null
+        succ_msg
+    fi
     if find /lib/modules/`uname -r` -name "vboxvfs\.*" 2>/dev/null|grep -q vboxvfs; then
         begin "Removing old VirtualBox vboxvfs kernel module"
         find /lib/modules/`uname -r` -name "vboxvfs\.*" 2>/dev/null|xargs rm -f 2>/dev/null
         succ_msg
     fi
-    if find /lib/modules/`uname -r` -name "vboxadd\.*" 2>/dev/null|grep -q vboxadd; then
-        begin "Removing old VirtualBox vboxadd kernel module"
-        find /lib/modules/`uname -r` -name "vboxadd\.*" 2>/dev/null|xargs rm -f 2>/dev/null
+    if find /lib/modules/`uname -r` -name "vboxguest\.*" 2>/dev/null|grep -q vboxguest; then
+        begin "Removing old VirtualBox vboxguest kernel module"
+        find /lib/modules/`uname -r` -name "vboxguest\.*" 2>/dev/null|xargs rm -f 2>/dev/null
         succ_msg
     fi
-    begin "Recompiling VirtualBox kernel modules"
-    if ! $BUILDVBOXADD \
-        --save-module-symvers /tmp/vboxadd-Module.symvers \
-        --no-print-directory install > $LOG 2>&1; then
+    begin "Building the VirtualBox Guest Additions kernel modules"
+    if ! sh /usr/share/$PACKAGE/test/build_in_tmp \
+        --no-dkms --no-print-directory > $LOG 2>&1; then
+        fail "`printf "Your system does not seem to be set up to build kernel modules.\nLook at $LOG to find out what went wrong"`"
+    fi
+    echo
+    if ! sh /usr/share/$PACKAGE/test_drm/build_in_tmp \
+        --no-dkms --no-print-directory >> $LOG 2>&1; then
+        printf "\nYour guest system does not seem to have sufficient OpenGL support to enable\naccelerated 3D effects (this requires Linux 2.6.27 or later in the guest\nsystem).  This Guest Additions feature will be disabled.\n\n"
+        BUILDVBOXVIDEO=""
+    fi
+    begin "Building the main Guest Additions module"
+    if ! $BUILDVBOXGUEST \
+        --save-module-symvers /tmp/vboxguest-Module.symvers \
+        --no-print-directory install >> $LOG 2>&1; then
         fail "Look at $LOG to find out what went wrong"
     fi
+    succ_msg
     if [ -n "$BUILDVBOXVFS" ]; then
+        begin "Building the shared folder support module"
         if ! $BUILDVBOXVFS \
-            --use-module-symvers /tmp/vboxadd-Module.symvers \
+            --use-module-symvers /tmp/vboxguest-Module.symvers \
             --no-print-directory install >> $LOG 2>&1; then
             fail "Look at $LOG to find out what went wrong"
         fi
+        succ_msg
     fi
     if [ -n "$BUILDVBOXVIDEO" ]; then
+        begin "Building the OpenGL support module"
         if ! $BUILDVBOXVIDEO \
-            --use-module-symvers /tmp/vboxadd-Module.symvers \
+            --use-module-symvers /tmp/vboxguest-Module.symvers \
             --no-print-directory install >> $LOG 2>&1; then
             fail "Look at $LOG to find out what went wrong"
         fi
+        succ_msg
     fi
+    depmod
+
+    begin "Doing non-kernel setup of the Guest Additions"
+    echo "Creating user for the Guest Additions." >> $LOG
+    # This is the LSB version of useradd and should work on recent
+    # distributions
+    useradd -d /var/run/vboxadd -g 1 -r -s /bin/false vboxadd >/dev/null 2>&1
+    # And for the others, we choose a UID ourselves
+    useradd -d /var/run/vboxadd -g 1 -u 501 -o -s /bin/false vboxadd >/dev/null 2>&1
+
+    # Create udev description file
+    if [ -d /etc/udev/rules.d ]; then
+        echo "Creating udev rule for the Guest Additions kernel module." >> $LOG
+        udev_call=""
+        udev_app=`which udevadm 2> /dev/null`
+        if [ $? -eq 0 ]; then
+            udev_call="${udev_app} version 2> /dev/null"
+        else
+            udev_app=`which udevinfo 2> /dev/null`
+            if [ $? -eq 0 ]; then
+                udev_call="${udev_app} -V 2> /dev/null"
+            fi
+        fi
+        udev_fix="="
+        if [ "${udev_call}" != "" ]; then
+            udev_out=`${udev_call}`
+            udev_ver=`expr "$udev_out" : '[^0-9]*\([0-9]*\)'`
+            if [ "$udev_ver" = "" -o "$udev_ver" -lt 55 ]; then
+               udev_fix=""
+            fi
+        fi
+        ## @todo 60-vboxadd.rules -> 60-vboxguest.rules ?
+        echo "KERNEL=${udev_fix}\"vboxguest\", NAME=\"vboxguest\", OWNER=\"vboxadd\", MODE=\"0660\"" > /etc/udev/rules.d/60-vboxadd.rules
+        echo "KERNEL=${udev_fix}\"vboxuser\", NAME=\"vboxuser\", OWNER=\"vboxadd\", MODE=\"0666\"" >> /etc/udev/rules.d/60-vboxadd.rules
+    fi
+
+    # Put mount.vboxsf in the right place
+    ln -sf "$lib_path/$PACKAGE/mount.vboxsf" /sbin
+    # At least Fedora 11 and Fedora 12 demand on the correct security context when
+    # executing this command from service scripts. Shouldn't hurt for other distributions.
+    chcon -u system_u -t mount_exec_t "$lib_path/$PACKAGE/mount.vboxsf" > /dev/null 2>&1
+
     succ_msg
-    start
-    echo
-    echo "You should reboot your guest to make sure the new modules are actually used"
+    if running_vboxguest || running_vboxadd; then
+        printf "You should restart your guest to make sure the new modules are actually used\n\n"
+    else
+        start
+    fi
+}
+
+# cleanup_script
+cleanup()
+{
+    # Delete old versions of VBox modules.
+    DKMS=`which dkms 2>/dev/null`
+    if [ -n "$DKMS" ]; then
+      echo "Attempt to remove old DKMS modules..."
+      for mod in vboxadd vboxguest vboxvfs vboxvideo; do
+        $DKMS status -m $mod | while read line; do
+          if echo "$line" | grep -q added > /dev/null ||
+             echo "$line" | grep -q built > /dev/null ||
+             echo "$line" | grep -q installed > /dev/null; then
+            version=`echo "$line" | sed "s/$mod,\([^,]*\)[,:].*/\1/;t;d"`
+            echo "  removing module $mod version $version"
+            $DKMS remove -m $mod -v $version --all 1>&2
+          fi
+        done
+      done
+      echo "Done."
+    fi
+
+    # Remove old installed modules
+    find /lib/modules -name vboxadd\* | xargs rm 2>/dev/null
+    find /lib/modules -name vboxguest\* | xargs rm 2>/dev/null
+    find /lib/modules -name vboxvfs\* | xargs rm 2>/dev/null
+    find /lib/modules -name vboxvideo\* | xargs rm 2>/dev/null
+    depmod
+
+    # Remove old module sources
+    rm -rf /usr/src/vboxadd-* /usr/src/vboxguest-* /usr/src/vboxvfs-* /usr/src/vboxvideo-*
+
+    # Remove other files
+    rm /sbin/mount.vboxsf 2>/dev/null
+    rm /etc/udev/rules.d/60-vboxadd.rules 2>/dev/null
 }
 
 dmnstatus()
 {
-    if running_vboxadd; then
+    if running_vboxguest; then
         echo "The VirtualBox Additions are currently running."
     else
         echo "The VirtualBox Additions are not currently running."
@@ -338,6 +470,9 @@ restart)
 setup)
     setup
     ;;
+cleanup)
+    cleanup
+    ;;
 status)
     dmnstatus
     ;;
