@@ -46,14 +46,11 @@
 
 // #define wchar_t linux_wchar_t
 
-#define xstr(s) str(s)
-#define str(s) #s
-
 MODULE_DESCRIPTION ("Host file system access VFS for VirtualBox");
 MODULE_AUTHOR ("Sun Microsystems, Inc.");
 MODULE_LICENSE ("GPL");
 #ifdef MODULE_VERSION
-MODULE_VERSION(VBOX_VERSION_STRING " (interface " xstr(VMMDEV_VERSION) ")");
+MODULE_VERSION(VBOX_VERSION_STRING " (interface " RT_XSTR(VMMDEV_VERSION) ")");
 #endif
 
 /* globals */
@@ -83,7 +80,7 @@ sf_glob_alloc (struct vbsf_mount_info_new *info, struct sf_glob_info **sf_gp)
                 goto fail0;
         }
 
-        memset(sf_g, 0, sizeof(*sf_g));
+        RT_ZERO(*sf_g);
 
         if (   info->nullchar     != '\0'
             || info->signature[0] != VBSF_MOUNT_SIGNATURE_BYTE_0
@@ -263,7 +260,18 @@ sf_read_super_aux (struct super_block *sb, void *data, int flags)
         sb->s_magic = 0xface;
         sb->s_blocksize = 1024;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 4, 3)
-        sb->s_maxbytes = ~0ULL; /* seek */
+        /* Required for seek/sendfile.
+         *
+         * Must by less than or equal to INT64_MAX despite the fact that the
+         * declaration of this variable is unsigned long long. See determination
+         * of 'loff_t max' in fs/read_write.c / do_sendfile(). I don't know the
+         * correct limit but MAX_LFS_FILESIZE (8TB-1 on 32-bit boxes) takes the
+         * page cache into account and is the suggested limit. */
+# if defined MAX_LFS_FILESIZE
+        sb->s_maxbytes = MAX_LFS_FILESIZE;
+# else
+        sb->s_maxbytes = 0x7fffffffffffffffULL;
+# endif
 #endif
         sb->s_op = &sf_super_ops;
 
@@ -278,6 +286,12 @@ sf_read_super_aux (struct super_block *sb, void *data, int flags)
                 goto fail3;
         }
 
+        if (sf_init_backing_dev(sf_g, info->name)) {
+                err = -EINVAL;
+                LogFunc(("could not init bdi\n"));
+                goto fail4;
+        }
+
         sf_init_inode (sf_g, iroot, &fsinfo);
         SET_INODE_INFO (iroot, sf_i);
 
@@ -289,13 +303,15 @@ sf_read_super_aux (struct super_block *sb, void *data, int flags)
         if (!droot) {
                 err = -ENOMEM;  /* XXX */
                 LogFunc(("d_alloc_root failed\n"));
-                goto fail4;
+                goto fail5;
         }
 
         sb->s_root = droot;
         SET_GLOB_INFO (sb, sf_g);
         return 0;
 
+ fail5:
+        sf_done_backing_dev(sf_g);
  fail4:
         iput (iroot);
  fail3:
@@ -363,6 +379,7 @@ sf_put_super (struct super_block *sb)
 
         sf_g = GET_GLOB_INFO (sb);
         BUG_ON (!sf_g);
+        sf_done_backing_dev(sf_g);
         sf_glob_free (sf_g);
 }
 
@@ -440,9 +457,6 @@ static struct file_system_type vboxsf_fs_type = {
 };
 #endif
 
-extern int CMC_API
-vboxadd_cmc_ctl_guest_filter_mask (uint32_t or_mask, uint32_t not_mask);
-
 /* Module initialization/finalization handlers */
 static int __init
 init (void)
@@ -458,7 +472,7 @@ init (void)
                         "Mount information structure is too large %lu\n"
                         "Must be less than or equal to %lu\n",
                         (unsigned long)sizeof (struct vbsf_mount_info_new),
-                        PAGE_SIZE);
+                        (unsigned long)PAGE_SIZE);
                 return -EINVAL;
         }
 
@@ -468,11 +482,6 @@ init (void)
                 return err;
         }
 
-        if (vboxadd_cmc_ctl_guest_filter_mask (VMMDEV_EVENT_HGCM, 0)) {
-                rcRet = -EINVAL;
-                goto fail0;
-        }
-
         rcVBox = vboxInit ();
         if (RT_FAILURE (rcVBox)) {
                 LogRelFunc (("vboxInit failed, rc=%d\n", rcVBox));
@@ -496,7 +505,7 @@ init (void)
 
         printk(KERN_DEBUG
                "vboxvfs: Successfully loaded version " VBOX_VERSION_STRING
-               " (interface " xstr(VMMDEV_VERSION) ")\n");
+               " (interface " RT_XSTR(VMMDEV_VERSION) ")\n");
 
         return 0;
 
@@ -505,7 +514,6 @@ init (void)
  fail1:
         vboxUninit ();
  fail0:
-        vboxadd_cmc_ctl_guest_filter_mask (0, VMMDEV_EVENT_HGCM);
         unregister_filesystem (&vboxsf_fs_type);
         return rcRet;
 }
@@ -517,7 +525,6 @@ fini (void)
 
         vboxDisconnect (&client_handle);
         vboxUninit ();
-        vboxadd_cmc_ctl_guest_filter_mask (0, VMMDEV_EVENT_HGCM);
         unregister_filesystem (&vboxsf_fs_type);
 }
 
