@@ -438,7 +438,7 @@ init_vga_card:
   ret
 
 msg_vga_init:
-.ascii "Sun xVM VirtualBox Version "
+.ascii "Sun VirtualBox Version "
 .ascii VBOX_VERSION_STRING
 .ascii " VGA BIOS"
 .byte 0x0d,0x0a,0x00
@@ -669,6 +669,10 @@ static void int10_func(DI, SI, BP, SP, BX, DX, CX, AX, DS, ES, FLAGS)
    case 0x0E:
      // Ralf Brown Interrupt list is WRONG on bh(page)
      // We do output only on the current page !
+#ifdef DEBUG
+     printf("write_teletype %02x\n", GET_AL());
+#endif
+
      biosfn_write_teletype(GET_AL(),0xff,GET_BL(),NO_ATTR);
      break;
    case 0x10:
@@ -2051,7 +2055,9 @@ Bit8u car;Bit8u page;Bit8u attr;Bit8u flag;
   {
    if(vga_modes[line].class==TEXT)
     {
-     biosfn_scroll(0x01,0x07,0,0,nbrows-1,nbcols-1,page,SCROLL_UP);
+     address=SCREEN_MEM_START(nbcols,nbrows,page)+(xcurs+(ycurs-1)*nbcols)*2;
+     attr=read_byte(vga_modes[line].sstart,address+1);
+     biosfn_scroll(0x01,attr,0,0,nbrows-1,nbcols-1,page,SCROLL_UP);
     }
    else
     {
@@ -3208,7 +3214,7 @@ Bit16u BX;Bit16u ES;Bit16u DI;
 }
 
 // --------------------------------------------------------------------------------------------
-static Bit16u biosfn_read_video_state_size2 (CX) 
+static Bit16u biosfn_read_video_state_size2 (CX)
      Bit16u CX;
 {
     Bit16u size;
@@ -3224,13 +3230,13 @@ static Bit16u biosfn_read_video_state_size2 (CX)
     }
     return size;
 }
-static void biosfn_read_video_state_size (CX, BX) 
+static void biosfn_read_video_state_size (CX, BX)
      Bit16u CX; Bit16u *BX;
 {
     Bit16u ss=get_SS();
     write_word(ss, BX, biosfn_read_video_state_size2(CX));
 }
-static Bit16u biosfn_save_video_state (CX,ES,BX) 
+static Bit16u biosfn_save_video_state (CX,ES,BX)
      Bit16u CX;Bit16u ES;Bit16u BX;
 {
     Bit16u i, v, crtc_addr, ar_index;
@@ -3315,7 +3321,7 @@ static Bit16u biosfn_save_video_state (CX,ES,BX)
     return BX;
 }
 
-static Bit16u biosfn_restore_video_state (CX,ES,BX) 
+static Bit16u biosfn_restore_video_state (CX,ES,BX)
      Bit16u CX;Bit16u ES;Bit16u BX;
 {
     Bit16u i, crtc_addr, v, addr1, ar_index;
