@@ -40,11 +40,6 @@
  */
 #undef USE_LOWCPU
 
-/* Define this if your compiler doesn't like prototypes */
-#ifndef __STDC__
-#define NO_PROTOTYPES
-#endif
-
 /*********************************************************/
 /*
  * Autoconf defined configuration options
@@ -126,9 +121,6 @@
 /* Define to whatever your compiler thinks const should be */
 #define const const
 
-/* Define if your compiler doesn't like prototypes */
-#undef NO_PROTOTYPES
-
 /* Define if you don't have u_int32_t etc. typedef'd */
 #undef NEED_TYPEDEFS
 #ifdef __sun__
