@@ -53,9 +53,9 @@ static bool     g_fPingReply = false;
 static uint32_t g_cOtherPkts = 0;
 static uint32_t g_cArpPkts = 0;
 static uint32_t g_cIpv4Pkts = 0;
-static uint32_t     g_cUdpPkts = 0;
-static uint32_t         g_cDhcpPkts = 0;
-static uint32_t     g_cTcpPkts = 0;
+static uint32_t g_cUdpPkts = 0;
+static uint32_t g_cDhcpPkts = 0;
+static uint32_t g_cTcpPkts = 0;
 
 
 /**
@@ -294,7 +294,7 @@ static void doXmitFrame(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNETBUF
      * Don't bother with dealing with overflows like DrvIntNet does, because
      * it's not supposed to happen here in this testcase.
      */
-    int rc = tstIntNetWriteFrame(pBuf, &pBuf->Send, pvFrame, cbFrame);
+    int rc = tstIntNetWriteFrame(pBuf, &pBuf->Send, pvFrame, (uint32_t)cbFrame);
     if (RT_SUCCESS(rc))
     {
         if (pFileRaw)
@@ -311,10 +311,10 @@ static void doXmitFrame(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNETBUF
     SendReq.Hdr.cbReq = sizeof(SendReq);
     SendReq.pSession = pSession;
     SendReq.hIf = hIf;
-    rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_SEND, 0, &SendReq.Hdr);
+    rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_SEND, 0, &SendReq.Hdr);
     if (RT_FAILURE(rc))
     {
-        RTPrintf("tstIntNet-1: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_IF_SEND,) failed, rc=%Rrc\n", rc);
+        RTPrintf("tstIntNet-1: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_IF_SEND,) failed, rc=%Rrc\n", rc);
         g_cErrors++;
     }
 
@@ -420,9 +420,9 @@ static void doXmitTest(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNETBUF p
 }
 
 
-static uint16_t icmpChecksum(PRTNETICMPV4HDR pHdr, int cbHdr)
+static uint16_t icmpChecksum(PRTNETICMPV4HDR pHdr, size_t cbHdr)
 {
-    int cbLeft = cbHdr;
+    size_t cbLeft = cbHdr;
     uint16_t *pbSrc = (uint16_t *)pHdr;
     uint16_t oddByte = 0;
     int cSum = 0;
@@ -480,7 +480,7 @@ static void doPingTest(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNETBUF p
     pIpHdr->ip_v = 4;
     pIpHdr->ip_hl = sizeof(*pIpHdr) / sizeof(uint32_t);
     pIpHdr->ip_tos = 0;
-    pIpHdr->ip_len = RT_H2BE_U16(sizeof(*pIcmpEcho) + cbPad + sizeof(*pIpHdr));
+    pIpHdr->ip_len = RT_H2BE_U16((uint16_t)(sizeof(*pIcmpEcho) + cbPad + sizeof(*pIpHdr)));
     pIpHdr->ip_id = (uint16_t)RTRandU32();
     pIpHdr->ip_off = 0;
     pIpHdr->ip_ttl = 255;
@@ -541,7 +541,7 @@ static void doPacketSniffing(INTNETIFHANDLE hIf, PSUPDRVSESSION pSession, PINTNE
         WaitReq.pSession = pSession;
         WaitReq.hIf = hIf;
         WaitReq.cMillies = cMillies - (uint32_t)cElapsedMillies;
-        int rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_WAIT, 0, &WaitReq.Hdr);
+        int rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_WAIT, 0, &WaitReq.Hdr);
         if (rc == VERR_TIMEOUT)
             break;
         if (RT_FAILURE(rc))
@@ -669,7 +669,7 @@ int main(int argc, char **argv)
      */
     RTR3Init();
 
-    static RTOPTIONDEF const s_aOptions[] =
+    static RTGETOPTDEF const s_aOptions[] =
     {
         { "--duration",     'd', RTGETOPT_REQ_UINT32 },
         { "--file",         'f', RTGETOPT_REQ_STRING },
@@ -716,8 +716,10 @@ int main(int argc, char **argv)
     int rc;
     int ch;
     int iArg = 1;
-    RTOPTIONUNION Value;
-    while ((ch = RTGetOpt(argc,argv, &s_aOptions[0], RT_ELEMENTS(s_aOptions), &iArg, &Value)))
+    RTGETOPTUNION Value;
+    RTGETOPTSTATE GetState;
+    RTGetOptInit(&GetState, argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), 1, 0 /* fFlags */);
+    while ((ch = RTGetOpt(&GetState, &Value)))
         switch (ch)
         {
             case 'd':
@@ -804,7 +806,19 @@ int main(int argc, char **argv)
 
             case '?':
             case 'h':
-                RTPrintf("syntax: tstIntNet-1 [-pStx-] [-d <secs>] [-f <file>] [-r <size>] [-s <size>]\n");
+                RTPrintf("syntax: tstIntNet-1 <options>\n"
+                         "\n"
+                         "Options:\n");
+                for (size_t i = 0; i < RT_ELEMENTS(s_aOptions); i++)
+                    RTPrintf("    -%c,%s\n", s_aOptions[i].iShort, s_aOptions[i].pszLong);
+                RTPrintf("\n"
+                         "Examples:\n"
+                         "    tstIntNet-1 -r 8192 -s 4096 -xS\n"
+                         "    tstIntNet-1 -n VBoxNetDhcp -r 4096 -s 4096 -i \"\" -xS\n");
+                return 1;
+
+            case VINF_GETOPT_NOT_OPTION:
+                RTPrintf("tstIntNetR0: invalid argument: %s\n", Value.psz);
                 return 1;
 
             default:
@@ -816,12 +830,6 @@ int main(int argc, char **argv)
                     RTPrintf("tstIntNetR0: invalid argument: %Rrc - %s\n", ch, argv[iArg]);
                 return 1;
         }
-    if (iArg < argc)
-    {
-        RTPrintf("tstIntNetR0: invalid argument: %s\n", argv[iArg]);
-        return 1;
-    }
-
 
     RTPrintf("tstIntNet-1: TESTING...\n");
 
@@ -837,17 +845,17 @@ int main(int argc, char **argv)
     }
 
     char szPath[RTPATH_MAX];
-    rc = RTPathProgram(szPath, sizeof(szPath) - sizeof("/../VMMR0.r0"));
+    rc = RTPathExecDir(szPath, sizeof(szPath) - sizeof("/../VMMR0.r0"));
     if (RT_FAILURE(rc))
     {
-        RTPrintf("tstIntNet-1: RTPathProgram -> %Rrc\n", rc);
+        RTPrintf("tstIntNet-1: RTPathExecDir -> %Rrc\n", rc);
         return 1;
     }
 
-    rc = SUPLoadVMM(strcat(szPath, "/../VMMR0.r0"));
+    rc = SUPR3LoadVMM(strcat(szPath, "/../VMMR0.r0"));
     if (RT_FAILURE(rc))
     {
-        RTPrintf("tstIntNet-1: SUPLoadVMM(\"%s\") -> %Rrc\n", szPath, rc);
+        RTPrintf("tstIntNet-1: SUPR3LoadVMM(\"%s\") -> %Rrc\n", szPath, rc);
         return 1;
     }
 
@@ -861,7 +869,7 @@ int main(int argc, char **argv)
     OpenReq.pSession = pSession;
     strncpy(OpenReq.szNetwork, pszNetwork, sizeof(OpenReq.szNetwork));
     strncpy(OpenReq.szTrunk, pszIf, sizeof(OpenReq.szTrunk));
-    OpenReq.enmTrunkType = kIntNetTrunkType_NetFlt;
+    OpenReq.enmTrunkType = *pszIf ? kIntNetTrunkType_NetFlt : kIntNetTrunkType_WhateverNone;
     OpenReq.fFlags = fMacSharing ? INTNET_OPEN_FLAGS_SHARED_MAC_ON_WIRE : 0;
     OpenReq.cbSend = cbSend;
     OpenReq.cbRecv = cbRecv;
@@ -873,7 +881,7 @@ int main(int argc, char **argv)
     RTPrintf("tstIntNet-1: attempting to open/create network \"%s\" with NetFlt trunk \"%s\"...\n",
              OpenReq.szNetwork, OpenReq.szTrunk);
     RTStrmFlush(g_pStdOut);
-    rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_OPEN, 0, &OpenReq.Hdr);
+    rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_OPEN, 0, &OpenReq.Hdr);
     if (RT_SUCCESS(rc))
     {
         RTPrintf("tstIntNet-1: successfully opened/created \"%s\" with NetFlt trunk \"%s\" - hIf=%#x\n",
@@ -889,7 +897,7 @@ int main(int argc, char **argv)
         GetRing3BufferReq.pSession = pSession;
         GetRing3BufferReq.hIf = OpenReq.hIf;
         GetRing3BufferReq.pRing3Buf = NULL;
-        rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_GET_RING3_BUFFER, 0, &GetRing3BufferReq.Hdr);
+        rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_GET_RING3_BUFFER, 0, &GetRing3BufferReq.Hdr);
         if (RT_SUCCESS(rc))
         {
             PINTNETBUF pBuf = GetRing3BufferReq.pRing3Buf;
@@ -904,7 +912,7 @@ int main(int argc, char **argv)
                 PromiscReq.pSession     = pSession;
                 PromiscReq.hIf          = OpenReq.hIf;
                 PromiscReq.fPromiscuous = true;
-                rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE, 0, &PromiscReq.Hdr);
+                rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE, 0, &PromiscReq.Hdr);
                 if (RT_SUCCESS(rc))
                     RTPrintf("tstIntNet-1: interface in promiscuous mode\n");
             }
@@ -919,7 +927,7 @@ int main(int argc, char **argv)
                 ActiveReq.pSession = pSession;
                 ActiveReq.hIf = OpenReq.hIf;
                 ActiveReq.fActive = true;
-                rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_SET_ACTIVE, 0, &ActiveReq.Hdr);
+                rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_SET_ACTIVE, 0, &ActiveReq.Hdr);
                 if (RT_SUCCESS(rc))
                 {
                     /*
@@ -963,29 +971,29 @@ int main(int argc, char **argv)
                 }
                 else
                 {
-                    RTPrintf("tstIntNet-1: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE,) failed, rc=%Rrc\n", rc);
+                    RTPrintf("tstIntNet-1: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE,) failed, rc=%Rrc\n", rc);
                     g_cErrors++;
                 }
             }
             else
             {
-                RTPrintf("tstIntNet-1: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE,) failed, rc=%Rrc\n", rc);
+                RTPrintf("tstIntNet-1: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE,) failed, rc=%Rrc\n", rc);
                 g_cErrors++;
             }
         }
         else
         {
-            RTPrintf("tstIntNet-1: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_IF_GET_RING3_BUFFER,) failed, rc=%Rrc\n", rc);
+            RTPrintf("tstIntNet-1: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_IF_GET_RING3_BUFFER,) failed, rc=%Rrc\n", rc);
             g_cErrors++;
         }
     }
     else
     {
-        RTPrintf("tstIntNet-1: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_OPEN,) failed, rc=%Rrc\n", rc);
+        RTPrintf("tstIntNet-1: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_OPEN,) failed, rc=%Rrc\n", rc);
         g_cErrors++;
     }
 
-    SUPTerm(false /* not forced */);
+    SUPR3Term(false /*fForced*/);
 
     /* close open files  */
     if (pFileRaw)
