@@ -55,7 +55,6 @@
 #define LOG_GROUP LOG_GROUP_DEFAULT ///@todo LOG_GROUP_DEV_DMA
 #include <VBox/log.h>
 #include <iprt/assert.h>
-#include <iprt/uuid.h>
 #include <iprt/string.h>
 
 #include <stdio.h>
@@ -845,14 +844,14 @@ static uint8_t get_mode_wrapper (PPDMDEVINS pDevIns, unsigned nchan)
     return DMA_get_channel_mode (s, nchan);
 }
 
-static void DMAReset (PPDMDEVINS pDevIns)
+static void dmaReset (PPDMDEVINS pDevIns)
 {
     DMAState *s = PDMINS_2_DATA (pDevIns, DMAState *);
     dma_reset (&s->dma_controllers[0]);
     dma_reset (&s->dma_controllers[1]);
 }
 
-static DECLCALLBACK(int) SaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) dmaSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
 {
     DMAState *s = PDMINS_2_DATA (pDevIns, DMAState *);
     dma_save (pSSMHandle, &s->dma_controllers[0]);
@@ -860,19 +859,18 @@ static DECLCALLBACK(int) SaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(int) LoadExec (PPDMDEVINS pDevIns,
-                                   PSSMHANDLE pSSMHandle,
-                                   uint32_t u32Version)
+static DECLCALLBACK(int) dmaLoadExec (PPDMDEVINS pDevIns,
+                                      PSSMHANDLE pSSMHandle,
+                                      uint32_t uVersion,
+                                      uint32_t uPass)
 {
     DMAState *s = PDMINS_2_DATA (pDevIns, DMAState *);
 
-    if (u32Version != 1) {
-        AssertFailed ();
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
-    }
+    AssertMsgReturn (uVersion == 1, ("%d\n", uVersion), VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION);
+    Assert (uPass == SSM_PASS_FINAL); NOREF(uPass);
 
-    dma_load (pSSMHandle, &s->dma_controllers[0], u32Version);
-    return dma_load (pSSMHandle, &s->dma_controllers[1], u32Version);
+    dma_load (pSSMHandle, &s->dma_controllers[0], uVersion);
+    return dma_load (pSSMHandle, &s->dma_controllers[1], uVersion);
 }
 
 /**
@@ -888,7 +886,7 @@ static DECLCALLBACK(int) LoadExec (PPDMDEVINS pDevIns,
  *                      of the device instance. It's also found in pDevIns->pCfgHandle, but like
  *                      iInstance it's expected to be used a bit in this function.
  */
-static DECLCALLBACK(int) DMAConstruct(PPDMDEVINS pDevIns,
+static DECLCALLBACK(int) dmaConstruct(PPDMDEVINS pDevIns,
                                       int iInstance,
                                       PCFGMNODE pCfgHandle)
 {
@@ -931,8 +929,7 @@ static DECLCALLBACK(int) DMAConstruct(PPDMDEVINS pDevIns,
         return rc;
     }
 
-    rc = PDMDevHlpSSMRegister (pDevIns, pDevIns->pDevReg->szDeviceName, iInstance, 1, sizeof (*s),
-                               NULL, SaveExec, NULL, NULL, LoadExec, NULL);
+    rc = PDMDevHlpSSMRegister (pDevIns, 1 /*uVersion*/, sizeof (*s), dmaSaveExec, dmaLoadExec);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -963,7 +960,7 @@ const PDMDEVREG g_DeviceDMA =
     /* cbInstance */
     sizeof(DMAState),
     /* pfnConstruct */
-    DMAConstruct,
+    dmaConstruct,
     /* pfnDestruct */
     NULL,
     /* pfnRelocate */
@@ -973,7 +970,7 @@ const PDMDEVREG g_DeviceDMA =
     /* pfnPowerOn */
     NULL,
     /* pfnReset */
-    DMAReset,
+    dmaReset,
     /* pfnSuspend */
     NULL,
     /* pfnResume */
