@@ -64,7 +64,6 @@
 #elif defined(RT_OS_SOLARIS)
 # include <stropts.h>
 # include <fcntl.h>
-# include <ctype.h>
 # include <errno.h>
 # include <pwd.h>
 # include <unistd.h>
@@ -341,7 +340,7 @@ DECLCALLBACK(int) drvHostDvdPoll(PDRVHOSTBASE pThis)
     {
         fMediaPresent = false;
         fMediaChanged = true;
-        /** @todo check this media chance stuff on Darwin. */
+        /** @todo check this media change stuff on Darwin. */
     }
 
 #elif defined(RT_OS_LINUX)
@@ -738,14 +737,9 @@ DECLCALLBACK(void) drvHostDvdDestruct(PPDMDRVINS pDrvIns)
 /**
  * Construct a host dvd drive driver instance.
  *
- * @returns VBox status.
- * @param   pDrvIns     The driver instance data.
- *                      If the registration structure is needed, pDrvIns->pDrvReg points to it.
- * @param   pCfgHandle  Configuration node handle for the driver. Use this to obtain the configuration
- *                      of the driver instance. It's also found in pDrvIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @copydoc FNPDMDRVCONSTRUCT
  */
-static DECLCALLBACK(int) drvHostDvdConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
+static DECLCALLBACK(int) drvHostDvdConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags)
 {
     PDRVHOSTBASE pThis = PDMINS_2_DATA(pDrvIns, PDRVHOSTBASE);
     LogFlow(("drvHostDvdConstruct: iInstance=%d\n", pDrvIns->iInstance));
@@ -859,7 +853,15 @@ const PDMDRVREG g_DrvHostDVD =
     NULL,
     /* pfnResume */
     NULL,
+    /* pfnAttach */
+    NULL,
     /* pfnDetach */
-    NULL
+    NULL,
+    /* pfnPowerOff */
+    NULL,
+    /* pfnSoftReset */
+    NULL,
+    /* u32EndVersion */
+    PDM_DRVREG_VERSION
 };
 
