@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -28,7 +28,7 @@
 #ifndef VBOX_VDICORE_VD
 #include <VBox/VBoxHDD.h>
 #else /* VBOX_VDICORE_VD */
-#include <VBox/VBoxHDD-new.h>
+#include <VBox/VBoxHDD.h>
 #endif /* VBOX_VDICORE_VD */
 #include <VBox/pdm.h>
 #include <VBox/mm.h>
@@ -50,7 +50,7 @@
 /** Image info, not handled anyhow.
  *  Must be less than 64 bytes in length, including the trailing 0.
  */
-#define VDI_IMAGE_FILE_INFO   "<<< Sun xVM VirtualBox Disk Image >>>\n"
+#define VDI_IMAGE_FILE_INFO   "<<< Sun VirtualBox Disk Image >>>\n"
 
 /** The Sector size.
  * Currently we support only 512 bytes sectors.
@@ -527,8 +527,13 @@ typedef struct VDIIMAGEDESC
     /** Link to child image descriptor, if any. */
     struct VDIIMAGEDESC    *pNext;
 #endif /* !VBOX_VDICORE_VD */
+#ifndef VBOX_WITH_NEW_IO_CODE
     /** File handle. */
     RTFILE                  File;
+#else
+    /** Opaque storage handle. */
+    void                   *pvStorage;
+#endif
 #ifndef VBOX_VDICORE_VD
     /** True if the image is operating in readonly mode. */
     bool                    fReadOnly;
@@ -584,6 +589,12 @@ typedef struct VDIIMAGEDESC
     PVDINTERFACE            pInterfaceError;
     /** Error interface callback table. */
     PVDINTERFACEERROR       pInterfaceErrorCallbacks;
+# ifdef VBOX_WITH_NEW_IO_CODE
+    /** Async I/O interface. */
+    PVDINTERFACE        pInterfaceAsyncIO;
+    /** Async I/O interface callbacks. */
+    PVDINTERFACEASYNCIO pInterfaceAsyncIOCallbacks;
+# endif
 #endif /* VBOX_VDICORE_VD */
 } VDIIMAGEDESC, *PVDIIMAGEDESC;
 
@@ -642,7 +653,7 @@ struct VDIDISK
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 #ifndef VBOX_VDICORE_VD
 VBOXDDU_DECL(void) vdiInitVDIDisk(PVDIDISK pDisk);
@@ -650,6 +661,6 @@ VBOXDDU_DECL(void) VDIFlushImage(PVDIIMAGEDESC pImage);
 VBOXDDU_DECL(int)  vdiChangeImageMode(PVDIIMAGEDESC pImage, bool fReadOnly);
 #endif /* !VBOX_VDICORE_VD */
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
