@@ -57,7 +57,7 @@ int main(int argc, char **argv)
             cpu.mode = CPUMODE_32BIT;
             if (RT_SUCCESS(DISInstr(&cpu, pInstr, 0, &cb, szOutput)))
             {
-                printf(szOutput);
+                printf("%s", szOutput);
             }
             else
             {
@@ -81,7 +81,7 @@ int main(int argc, char **argv)
             memset(&cpu, 0, sizeof(cpu));
             cpu.mode = CPUMODE_64BIT;
             if (RT_SUCCESS(DISInstr(&cpu, pInstr, 0, &cb, szOutput)))
-                printf(szOutput);
+                printf("%s", szOutput);
             else
             {
                 printf("DISOne failed!\n");
