@@ -47,8 +47,7 @@ public:
     virtual ~Keyboard();
 
     STDMETHOD(PutScancode)(LONG scancode);
-    STDMETHOD(PutScancodes)(LONG *scancodes,
-                            ULONG count,
+    STDMETHOD(PutScancodes)(ComSafeArrayIn (LONG, scancodes),
                             ULONG *codesStored);
     STDMETHOD(PutCAD)();
 
@@ -57,7 +56,7 @@ public:
 private:
 
     static DECLCALLBACK(void *) drvQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface);
-    static DECLCALLBACK(int)    drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle);
+    static DECLCALLBACK(int)    drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags);
     static DECLCALLBACK(void)   drvDestruct(PPDMDRVINS pDrvIns);
 
     /** Pointer to the associated keyboard driver. */
