@@ -135,16 +135,8 @@ STDMETHODIMP MachineDebugger::COMSETTER(RecompileUser)(BOOL enable)
         return E_FAIL;
     }
 
-    PVMREQ pReq;
     EMRAWMODE rawModeFlag = enable ? EMRAW_RING3_DISABLE : EMRAW_RING3_ENABLE;
-    int rcVBox = VMR3ReqCall(pVM, VMCPUID_ANY, &pReq, RT_INDEFINITE_WAIT,
-                             (PFNRT)EMR3RawSetMode, 2, pVM, rawModeFlag);
-    if (RT_SUCCESS(rcVBox))
-    {
-        rcVBox = pReq->iStatus;
-        VMR3ReqFree(pReq);
-    }
-
+    int rcVBox = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)EMR3RawSetMode, 2, pVM, rawModeFlag);
     if (RT_SUCCESS(rcVBox))
         return S_OK;
 
@@ -195,16 +187,8 @@ STDMETHODIMP MachineDebugger::COMSETTER(RecompileSupervisor)(BOOL enable)
         return E_FAIL;
     }
 
-    PVMREQ pReq;
     EMRAWMODE rawModeFlag = enable ? EMRAW_RING0_DISABLE : EMRAW_RING0_ENABLE;
-    int rcVBox = VMR3ReqCall(pVM, VMCPUID_ANY, &pReq, RT_INDEFINITE_WAIT,
-                             (PFNRT)EMR3RawSetMode, 2, pVM, rawModeFlag);
-    if (RT_SUCCESS(rcVBox))
-    {
-        rcVBox = pReq->iStatus;
-        VMR3ReqFree(pReq);
-    }
-
+    int rcVBox = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)EMR3RawSetMode, 2, pVM, rawModeFlag);
     if (RT_SUCCESS(rcVBox))
         return S_OK;
 
