@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -857,7 +857,7 @@ uint8_t SDLConsole::keyEventToKeyCode(const SDL_KeyboardEvent *ev)
             case SDLK_RSUPER:           keycode = 0x5c | 0x80; break;
             case SDLK_LMETA:
             case SDLK_LSUPER:           keycode = 0x5b | 0x80; break;
-            /* Sssumes normal key. */
+            /* Assumes normal key. */
             default:                    keycode = s_aMacToSet1[keycode]; break;
         }
     }
@@ -1071,7 +1071,7 @@ void SDLConsole::processKey(SDL_KeyboardEvent *ev)
 
 #ifdef RT_OS_DARWIN
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 /* Private interface in 10.3 and later. */
 typedef int CGSConnection;
 typedef enum
@@ -1083,7 +1083,7 @@ typedef enum
 extern CGSConnection _CGSDefaultConnection(void);
 extern CGError CGSGetGlobalHotKeyOperatingMode(CGSConnection Connection, CGSGlobalHotKeyOperatingMode *enmMode);
 extern CGError CGSSetGlobalHotKeyOperatingMode(CGSConnection Connection, CGSGlobalHotKeyOperatingMode enmMode);
-__END_DECLS
+RT_C_DECLS_END
 
 /** Keeping track of whether we disabled the hotkeys or not. */
 static bool g_fHotKeysDisabled = false;
@@ -1268,7 +1268,7 @@ void SDLConsole::updateTitlebar()
     char pszTitle[1024];
 
     RTStrPrintf(pszTitle, sizeof(pszTitle),
-                "Sun xVM VirtualBox%s%s",
+                "Sun VirtualBox%s%s",
                 g_uProgressPercent == ~0U && machineState == VMSTATE_SUSPENDED ? " - [Paused]" : "",
                 mfInputGrab                                                    ? " - [Input captured]": "");
 
@@ -1311,8 +1311,8 @@ void SDLConsole::updateTitlebarProgress(const char *pszStr, int iPercent)
 {
     char szTitle[256];
     AssertMsg(iPercent >= 0 && iPercent <= 100, ("%d\n", iPercent));
-    RTStrPrintf(szTitle, sizeof(szTitle), "Sun xVM VirtualBox - %s %d%%...", pszStr, iPercent);
-    SDL_WM_SetCaption(szTitle, "Sun xVM VirtualBox");
+    RTStrPrintf(szTitle, sizeof(szTitle), "Sun VirtualBox - %s %d%%...", pszStr, iPercent);
+    SDL_WM_SetCaption(szTitle, "Sun VirtualBox");
 }
 
 /**
