@@ -5,15 +5,19 @@
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2007 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
  */
 
 #ifndef ____H_USBPROXYSERVICE
@@ -135,14 +139,6 @@ protected:
     virtual PUSBDEVICE getDevices (void);
 
 public:
-    /**
-     * Free all the members of a USB interface returned by getDevice().
-     *
-     * @param   pIf         Pointer to the interface.
-     * @param   cIfs        Number of consecutive interfaces pIf points to
-     */
-    static void freeInterfaceMembers (PUSBINTERFACE pIf, unsigned cIfs);
-
     /**
      * Free one USB device returned by getDevice().
      *
