@@ -28,8 +28,7 @@
 # include <VBox/com/defs.h>
 #endif
 #include <VBox/pdm.h>
-#include <VBox/VBoxDev.h>
-#include <VBox/VBoxGuest.h>
+#include <VBox/VMMDev.h>
 #include <VBox/cfgm.h>
 #include <VBox/err.h>
 #include <iprt/assert.h>
@@ -41,7 +40,7 @@
 #include "MouseImpl.h"
 #include "DisplayImpl.h"
 #include "ConsoleImpl.h"
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
 #include "HGCM.h"
 #endif
 
@@ -68,7 +67,7 @@ typedef struct DRVMAINVMMDEV
     /** Our VMM device connector interface. */
     PDMIVMMDEVCONNECTOR         Connector;
 
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
     /** Pointer to the HGCM port interface of the driver/device above us. */
     PPDMIHGCMPORT               pHGCMPort;
     /** Our HGCM connector interface. */
@@ -79,14 +78,14 @@ typedef struct DRVMAINVMMDEV
 /** Converts PDMIVMMDEVCONNECTOR pointer to a DRVMAINVMMDEV pointer. */
 #define PDMIVMMDEVCONNECTOR_2_MAINVMMDEV(pInterface) ( (PDRVMAINVMMDEV) ((uintptr_t)pInterface - RT_OFFSETOF(DRVMAINVMMDEV, Connector)) )
 
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
 /** Converts PDMIHGCMCONNECTOR pointer to a DRVMAINVMMDEV pointer. */
 #define PDMIHGCMCONNECTOR_2_MAINVMMDEV(pInterface) ( (PDRVMAINVMMDEV) ((uintptr_t)pInterface - RT_OFFSETOF(DRVMAINVMMDEV, HGCMConnector)) )
 #endif
 
 VMMDev::VMMDev() : mpDrv(NULL)
 {
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
     int rc = VINF_SUCCESS;
     if (fActivateHGCM())
         rc = HGCMHostInit();
@@ -96,10 +95,10 @@ VMMDev::VMMDev() : mpDrv(NULL)
 
 VMMDev::~VMMDev()
 {
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
     if (fActivateHGCM())
         HGCMHostShutdown ();
-#endif /* VBOX_HGCM */
+#endif /* VBOX_WITH_HGCM */
 }
 
 
@@ -172,8 +171,8 @@ DECLCALLBACK(void) VMMDev::UpdateMouseCapabilities(PPDMIVMMDEVCONNECTOR pInterfa
 
     if (gMouse)
     {
-        gMouse->setAbsoluteCoordinates(!!(newCapabilities & VMMDEV_MOUSEGUESTWANTSABS));
-        gMouse->setNeedsHostCursor(!!(newCapabilities & VMMDEV_MOUSEGUESTNEEDSHOSTCUR));
+        gMouse->setAbsoluteCoordinates(!!(newCapabilities & VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE));
+        gMouse->setNeedsHostCursor(!!(newCapabilities & VMMDEV_MOUSE_GUEST_NEEDS_HOST_CURSOR));
     }
     if (gConsole)
     {
@@ -258,7 +257,7 @@ DECLCALLBACK(int) VMMDev::GetHeightReduction(PPDMIVMMDEVCONNECTOR pInterface, ui
     return VINF_SUCCESS;
 }
 
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
 
 /* HGCM connector interface */
 
@@ -267,7 +266,7 @@ static DECLCALLBACK(int) iface_hgcmConnect (PPDMIHGCMCONNECTOR pInterface, PVBOX
     LogSunlover(("Enter\n"));
 
     PDRVMAINVMMDEV pDrv = PDMIHGCMCONNECTOR_2_MAINVMMDEV(pInterface);
-    
+
     if (    !pServiceLocation
         || (   pServiceLocation->type != VMMDevHGCMLoc_LocalHost
             && pServiceLocation->type != VMMDevHGCMLoc_LocalHost_Existing))
@@ -317,14 +316,16 @@ static DECLCALLBACK(int) iface_hgcmSave(PPDMDRVINS pDrvIns, PSSMHANDLE pSSM)
  * @returns VBox status code.
  * @param   pDrvIns         Driver instance of the driver which registered the data unit.
  * @param   pSSM            SSM operation handle.
- * @param   u32Version      Data layout version.
+ * @param   uVersion        Data layout version.
+ * @param   uPass           The data pass.
  */
-static DECLCALLBACK(int) iface_hgcmLoad(PPDMDRVINS pDrvIns, PSSMHANDLE pSSM, uint32_t u32Version)
+static DECLCALLBACK(int) iface_hgcmLoad(PPDMDRVINS pDrvIns, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
 {
     LogFlowFunc(("Enter\n"));
 
-    if (u32Version != HGCM_SSM_VERSION)
+    if (uVersion != HGCM_SSM_VERSION)
         return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
+    Assert(uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     return HGCMHostLoadState (pSSM);
 }
@@ -352,14 +353,14 @@ int VMMDev::hgcmHostCall (const char *pszServiceName, uint32_t u32Function,
 DECLCALLBACK(void *) VMMDev::drvQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface)
 {
     PPDMDRVINS pDrvIns = PDMIBASE_2_PDMDRV(pInterface);
-    PDRVMAINVMMDEV pDrv = PDMINS2DATA(pDrvIns, PDRVMAINVMMDEV);
+    PDRVMAINVMMDEV pDrv = PDMINS_2_DATA(pDrvIns, PDRVMAINVMMDEV);
     switch (enmInterface)
     {
         case PDMINTERFACE_BASE:
             return &pDrvIns->IBase;
         case PDMINTERFACE_VMMDEV_CONNECTOR:
             return &pDrv->Connector;
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
         case PDMINTERFACE_HGCM_CONNECTOR:
             if (fActivateHGCM())
                 return &pDrv->HGCMConnector;
@@ -380,23 +381,18 @@ DECLCALLBACK(void *) VMMDev::drvQueryInterface(PPDMIBASE pInterface, PDMINTERFAC
  */
 DECLCALLBACK(void) VMMDev::drvDestruct(PPDMDRVINS pDrvIns)
 {
-    /*PDRVMAINVMMDEV pData = PDMINS2DATA(pDrvIns, PDRVMAINVMMDEV); - unused variables makes gcc bitch. */
+    /*PDRVMAINVMMDEV pData = PDMINS_2_DATA(pDrvIns, PDRVMAINVMMDEV); - unused variables makes gcc bitch. */
 }
 
 
 /**
  * Construct a VMMDev driver instance.
  *
- * @returns VBox status.
- * @param   pDrvIns     The driver instance data.
- *                      If the registration structure is needed, pDrvIns->pDrvReg points to it.
- * @param   pCfgHandle  Configuration node handle for the driver. Use this to obtain the configuration
- *                      of the driver instance. It's also found in pDrvIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @copydoc FNPDMDRVCONSTRUCT
  */
-DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
+DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags)
 {
-    PDRVMAINVMMDEV pData = PDMINS2DATA(pDrvIns, PDRVMAINVMMDEV);
+    PDRVMAINVMMDEV pData = PDMINS_2_DATA(pDrvIns, PDRVMAINVMMDEV);
     LogFlow(("Keyboard::drvConstruct: iInstance=%d\n", pDrvIns->iInstance));
 
     /*
@@ -404,14 +400,9 @@ DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
      */
     if (!CFGMR3AreValuesValid(pCfgHandle, "Object\0"))
         return VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
-
-    PPDMIBASE pBaseIgnore;
-    int rc = pDrvIns->pDrvHlp->pfnAttach(pDrvIns, &pBaseIgnore);
-    if (rc != VERR_PDM_NO_ATTACHED_DRIVER)
-    {
-        AssertMsgFailed(("Configuration error: Not possible to attach anything to this driver!\n"));
-        return VERR_PDM_DRVINS_NO_ATTACH;
-    }
+    AssertMsgReturn(PDMDrvHlpNoAttach(pDrvIns) == VERR_PDM_NO_ATTACHED_DRIVER,
+                    ("Configuration error: Not possible to attach anything to this driver!\n"),
+                    VERR_PDM_DRVINS_NO_ATTACH);
 
     /*
      * IBase.
@@ -429,7 +420,7 @@ DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
     pData->Connector.pfnSetVisibleRegion        = iface_SetVisibleRegion;
     pData->Connector.pfnQueryVisibleRegion      = iface_QueryVisibleRegion;
 
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
     if (fActivateHGCM())
     {
         pData->HGCMConnector.pfnConnect         = iface_hgcmConnect;
@@ -448,7 +439,7 @@ DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
         return VERR_PDM_MISSING_INTERFACE_ABOVE;
     }
 
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
     if (fActivateHGCM())
     {
         pData->pHGCMPort = (PPDMIHGCMPORT)pDrvIns->pUpBase->pfnQueryInterface(pDrvIns->pUpBase, PDMINTERFACE_HGCM_PORT);
@@ -464,30 +455,37 @@ DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
      * Get the VMMDev object pointer and update the mpDrv member.
      */
     void *pv;
-    rc = CFGMR3QueryPtr(pCfgHandle, "Object", &pv);
-    if (VBOX_FAILURE(rc))
+    int rc = CFGMR3QueryPtr(pCfgHandle, "Object", &pv);
+    if (RT_FAILURE(rc))
     {
-        AssertMsgFailed(("Configuration error: No/bad \"Object\" value! rc=%Vrc\n", rc));
+        AssertMsgFailed(("Configuration error: No/bad \"Object\" value! rc=%Rrc\n", rc));
         return rc;
     }
 
     pData->pVMMDev = (VMMDev*)pv;        /** @todo Check this cast! */
     pData->pVMMDev->mpDrv = pData;
 
-#ifdef VBOX_HGCM
+#ifdef VBOX_WITH_HGCM
     if (fActivateHGCM())
     {
         rc = pData->pVMMDev->hgcmLoadService (VBOXSHAREDFOLDERS_DLL, "VBoxSharedFolders");
-        pData->pVMMDev->fSharedFolderActive = VBOX_SUCCESS(rc);
-        if (VBOX_SUCCESS(rc))
+        pData->pVMMDev->fSharedFolderActive = RT_SUCCESS(rc);
+        if (RT_SUCCESS(rc))
             LogRel(("Shared Folders service loaded.\n"));
         else
-            LogRel(("Failed to load Shared Folders service %Vrc\n", rc));
+            LogRel(("Failed to load Shared Folders service %Rrc\n", rc));
+
+
+        rc = PDMDrvHlpSSMRegisterEx(pDrvIns, HGCM_SSM_VERSION, 4096 /* bad guess */,
+                                    NULL, NULL, NULL,
+                                    NULL, iface_hgcmSave, NULL,
+                                    NULL, iface_hgcmLoad, NULL);
+        if (RT_FAILURE(rc))
+            return rc;
 
-        pDrvIns->pDrvHlp->pfnSSMRegister(pDrvIns, "HGCM", 0, HGCM_SSM_VERSION, 4096/* bad guess */, NULL, iface_hgcmSave, NULL, NULL, iface_hgcmLoad, NULL);
     }
-#endif /* VBOX_HGCM */
-    
+#endif /* VBOX_WITH_HGCM */
+
     return VINF_SUCCESS;
 }
 
@@ -500,7 +498,7 @@ const PDMDRVREG VMMDev::DrvReg =
     /* u32Version */
     PDM_DRVREG_VERSION,
     /* szDriverName */
-    "MainVMMDev",
+    "HGCM",
     /* pszDescription */
     "Main VMMDev driver (Main as in the API).",
     /* fFlags */
@@ -525,6 +523,15 @@ const PDMDRVREG VMMDev::DrvReg =
     NULL,
     /* pfnResume */
     NULL,
+    /* pfnAttach */
+    NULL,
     /* pfnDetach */
-    NULL
+    NULL,
+    /* pfnPowerOff */
+    NULL,
+    /* pfnSoftReset */
+    NULL,
+    /* u32EndVersion */
+    PDM_DRVREG_VERSION
 };
+
