@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -33,21 +33,29 @@ public:
 
     NS_DECL_ISUPPORTS
 
-    NS_IMETHOD GetWidth(uint32 *width);
-    NS_IMETHOD GetHeight(uint32_t *height);
+    NS_IMETHOD GetWidth(PRUint32 *width);
+    NS_IMETHOD GetHeight(PRUint32 *height);
     NS_IMETHOD Lock();
     NS_IMETHOD Unlock();
-    NS_IMETHOD GetAddress(uint32_t *address);
-    NS_IMETHOD GetBitsPerPixel(uint32_t *bitsPerPixel);
-    NS_IMETHOD GetBytesPerLine(uint32_t *bytesPerLine);
-    NS_IMETHOD GetPixelFormat(ULONG *pixelFormat);
-    NS_IMETHOD GetUsesGuestVRAM(BOOL *usesGuestVRAM);
-    NS_IMETHOD NotifyUpdate(uint32_t x, uint32_t y,
-                            uint32_t w, uint32_t h);
-    NS_IMETHOD RequestResize(ULONG aScreenId, ULONG pixelFormat, uint32_t vram,
-                             uint32_t bitsPerPixel, uint32_t bytesPerLine,
-                             uint32_t w, uint32_t h,
+    NS_IMETHOD GetAddress(PRUint8 **address);
+    NS_IMETHOD GetBitsPerPixel(PRUint32 *bitsPerPixel);
+    NS_IMETHOD GetBytesPerLine(PRUint32 *bytesPerLine);
+    NS_IMETHOD GetPixelFormat(PRUint32 *pixelFormat);
+    NS_IMETHOD GetUsesGuestVRAM(PRBool *usesGuestVRAM);
+    NS_IMETHOD GetHeightReduction(PRUint32 *heightReduction);
+    NS_IMETHOD GetOverlay(IFramebufferOverlay **aOverlay);
+    NS_IMETHOD GetWinId(PRUint64 *winId);
+    NS_IMETHOD NotifyUpdate(PRUint32 x, PRUint32 y, PRUint32 w, PRUint32 h);
+    NS_IMETHOD RequestResize(PRUint32 aScreenId, PRUint32 pixelFormat, PRUint8 *vram,
+                             PRUint32 bitsPerPixel, PRUint32 bytesPerLine,
+                             PRUint32 w, PRUint32 h,
                              PRBool *finished);
+    NS_IMETHOD VideoModeSupported(PRUint32 width, PRUint32 height, PRUint32 bpp, PRBool *supported);
+    NS_IMETHOD GetVisibleRegion(PRUint8 *aRectangles, PRUint32 aCount, PRUint32 *aCountCopied);
+    NS_IMETHOD SetVisibleRegion(PRUint8 *aRectangles, PRUint32 aCount);
+
+    NS_IMETHOD ProcessVHWACommand(PRUint8 *pCommand);
+
 private:
     int createSurface(uint32_t w, uint32_t h);
 
