@@ -157,6 +157,11 @@ void printUsageInternal(USAGECATEGORY u64Cmd)
                 "       of a container can be registered.\n"
                 "\n"
                 : "",
+            (u64Cmd & USAGE_DUMPHDINFO) ?
+                "  dumphdinfo <filepath>\n"
+                "       Prints information about the image at the given location.\n"
+                "\n"
+                : "",
             (u64Cmd & USAGE_LISTPARTITIONS) ?
                 "  listpartitions -rawdisk <diskname>\n"
                 "       Lists all partitions on <diskname>.\n"
@@ -437,8 +442,8 @@ static int CmdLoadSyms(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, C
     /* offDelta */
     if (argc >= 3)
     {
-        int rc = RTStrToInt64Ex(argv[2], NULL, 0, &offDelta);
-        if (RT_FAILURE(rc))
+        int irc = RTStrToInt64Ex(argv[2], NULL, 0, &offDelta);
+        if (RT_FAILURE(irc))
             return errorArgument(argv[0], "Failed to read delta '%s', rc=%Rrc\n", argv[2], rc);
     }
 
@@ -449,16 +454,16 @@ static int CmdLoadSyms(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, C
     /* ModuleAddress */
     if (argc >= 5)
     {
-        int rc = RTStrToUInt64Ex(argv[4], NULL, 0, &ModuleAddress);
-        if (RT_FAILURE(rc))
+        int irc = RTStrToUInt64Ex(argv[4], NULL, 0, &ModuleAddress);
+        if (RT_FAILURE(irc))
             return errorArgument(argv[0], "Failed to read module address '%s', rc=%Rrc\n", argv[4], rc);
     }
 
     /* ModuleSize */
     if (argc >= 6)
     {
-        int rc = RTStrToUInt64Ex(argv[5], NULL, 0, &ModuleSize);
-        if (RT_FAILURE(rc))
+        int irc = RTStrToUInt64Ex(argv[5], NULL, 0, &ModuleSize);
+        if (RT_FAILURE(irc))
             return errorArgument(argv[0], "Failed to read module size '%s', rc=%Rrc\n", argv[5], rc);
     }
 
@@ -468,15 +473,15 @@ static int CmdLoadSyms(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, C
     Utf8Str KeyStr;
     HRESULT hrc = NewUniqueKey(machine, "VBoxInternal/DBGF/loadsyms", KeyStr);
     if (SUCCEEDED(hrc))
-        hrc = SetString(machine, "VBoxInternal/DBGF/loadsyms", KeyStr, "Filename", pszFilename);
+        hrc = SetString(machine, "VBoxInternal/DBGF/loadsyms", KeyStr.c_str(), "Filename", pszFilename);
     if (SUCCEEDED(hrc) && argc >= 3)
-        hrc = SetInt64(machine, "VBoxInternal/DBGF/loadsyms", KeyStr, "Delta", offDelta);
+        hrc = SetInt64(machine, "VBoxInternal/DBGF/loadsyms", KeyStr.c_str(), "Delta", offDelta);
     if (SUCCEEDED(hrc) && argc >= 4)
-        hrc = SetString(machine, "VBoxInternal/DBGF/loadsyms", KeyStr, "Module", pszModule);
+        hrc = SetString(machine, "VBoxInternal/DBGF/loadsyms", KeyStr.c_str(), "Module", pszModule);
     if (SUCCEEDED(hrc) && argc >= 5)
-        hrc = SetUInt64(machine, "VBoxInternal/DBGF/loadsyms", KeyStr, "ModuleAddress", ModuleAddress);
+        hrc = SetUInt64(machine, "VBoxInternal/DBGF/loadsyms", KeyStr.c_str(), "ModuleAddress", ModuleAddress);
     if (SUCCEEDED(hrc) && argc >= 6)
-        hrc = SetUInt64(machine, "VBoxInternal/DBGF/loadsyms", KeyStr, "ModuleSize", ModuleSize);
+        hrc = SetUInt64(machine, "VBoxInternal/DBGF/loadsyms", KeyStr.c_str(), "ModuleSize", ModuleSize);
 
     return FAILED(hrc);
 }
@@ -490,7 +495,7 @@ static DECLCALLBACK(void) handleVDError(void *pvUser, int rc, RT_SRC_POS_DECL, c
     RTPrintf("Error code %Rrc at %s(%u) in function %s\n", rc, RT_SRC_POS_ARGS);
 }
 
-static int handleSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
+static int CmdSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
 {
     /* we need exactly one parameter: the image file */
     if (argc != 1)
@@ -504,7 +509,7 @@ static int handleSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBo
 
     /* just try it */
     char *pszFormat = NULL;
-    int rc = VDGetFormat(argv[0], &pszFormat);
+    int rc = VDGetFormat(NULL, argv[0], &pszFormat);
     if (RT_FAILURE(rc))
     {
         RTPrintf("Format autodetect failed: %Rrc\n", rc);
@@ -519,6 +524,7 @@ static int handleSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBo
     vdInterfaceErrorCallbacks.cbSize       = sizeof(VDINTERFACEERROR);
     vdInterfaceErrorCallbacks.enmInterface = VDINTERFACETYPE_ERROR;
     vdInterfaceErrorCallbacks.pfnError     = handleVDError;
+    vdInterfaceErrorCallbacks.pfnMessage   = NULL;
 
     rc = VDInterfaceAdd(&vdInterfaceError, "VBoxManage_IError", VDINTERFACETYPE_ERROR,
                         &vdInterfaceErrorCallbacks, NULL, &pVDIfs);
@@ -550,6 +556,70 @@ static int handleSetHDUUID(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBo
     return RT_FAILURE(rc);
 }
 
+
+static int handleVDMessage(void *pvUser, const char *pszFormat, ...)
+{
+    NOREF(pvUser);
+    va_list args;
+    va_start(args, pszFormat);
+    int rc = RTPrintfV(pszFormat, args);
+    va_end(args);
+    return rc;
+}
+
+static int CmdDumpHDInfo(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox, ComPtr<ISession> aSession)
+{
+    /* we need exactly one parameter: the image file */
+    if (argc != 1)
+    {
+        return errorSyntax(USAGE_SETHDUUID, "Not enough parameters");
+    }
+
+    /* just try it */
+    char *pszFormat = NULL;
+    int rc = VDGetFormat(NULL, argv[0], &pszFormat);
+    if (RT_FAILURE(rc))
+    {
+        RTPrintf("Format autodetect failed: %Rrc\n", rc);
+        return 1;
+    }
+
+    PVBOXHDD pDisk = NULL;
+
+    PVDINTERFACE     pVDIfs = NULL;
+    VDINTERFACE      vdInterfaceError;
+    VDINTERFACEERROR vdInterfaceErrorCallbacks;
+    vdInterfaceErrorCallbacks.cbSize       = sizeof(VDINTERFACEERROR);
+    vdInterfaceErrorCallbacks.enmInterface = VDINTERFACETYPE_ERROR;
+    vdInterfaceErrorCallbacks.pfnError     = handleVDError;
+    vdInterfaceErrorCallbacks.pfnMessage   = handleVDMessage;
+
+    rc = VDInterfaceAdd(&vdInterfaceError, "VBoxManage_IError", VDINTERFACETYPE_ERROR,
+                        &vdInterfaceErrorCallbacks, NULL, &pVDIfs);
+    AssertRC(rc);
+
+    rc = VDCreate(pVDIfs, &pDisk);
+    if (RT_FAILURE(rc))
+    {
+        RTPrintf("Error while creating the virtual disk container: %Rrc\n", rc);
+        return 1;
+    }
+
+    /* Open the image */
+    rc = VDOpen(pDisk, pszFormat, argv[0], VD_OPEN_FLAGS_NORMAL, NULL);
+    if (RT_FAILURE(rc))
+    {
+        RTPrintf("Error while opening the image: %Rrc\n", rc);
+        return 1;
+    }
+
+    VDDumpImages(pDisk);
+
+    VDCloseAll(pDisk);
+
+    return RT_FAILURE(rc);
+}
+
 static int partRead(RTFILE File, PHOSTPARTITIONS pPart)
 {
     uint8_t aBuffer[512];
@@ -753,7 +823,7 @@ static int CmdListPartitions(int argc, char **argv, ComPtr<IVirtualBox> aVirtual
         return errorSyntax(USAGE_LISTPARTITIONS, "Mandatory parameter -rawdisk missing");
 
     RTFILE RawFile;
-    int vrc = RTFileOpen(&RawFile, rawdisk.raw(), RTFILE_O_OPEN | RTFILE_O_READ | RTFILE_O_DENY_WRITE);
+    int vrc = RTFileOpen(&RawFile, rawdisk.raw(), RTFILE_O_READ | RTFILE_O_OPEN | RTFILE_O_DENY_WRITE);
     if (RT_FAILURE(vrc))
     {
         RTPrintf("Error opening the raw disk: %Rrc\n", vrc);
@@ -873,7 +943,7 @@ static int CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirtualB
     fRelative = true;
 #endif /* RT_OS_DARWIN */
     RTFILE RawFile;
-    int vrc = RTFileOpen(&RawFile, rawdisk.raw(), RTFILE_O_OPEN | RTFILE_O_READ | RTFILE_O_DENY_WRITE);
+    int vrc = RTFileOpen(&RawFile, rawdisk.raw(), RTFILE_O_READ | RTFILE_O_OPEN | RTFILE_O_DENY_WRITE);
     if (RT_FAILURE(vrc))
     {
         RTPrintf("Error opening the raw disk '%s': %Rrc\n", rawdisk.raw(), vrc);
@@ -1200,7 +1270,7 @@ static int CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirtualB
                     &&  pszMBRFilename)
                 {
                     RTFILE MBRFile;
-                    vrc = RTFileOpen(&MBRFile, pszMBRFilename, RTFILE_O_OPEN | RTFILE_O_READ | RTFILE_O_DENY_WRITE);
+                    vrc = RTFileOpen(&MBRFile, pszMBRFilename, RTFILE_O_READ | RTFILE_O_OPEN | RTFILE_O_DENY_WRITE);
                     if (RT_FAILURE(vrc))
                     {
                         RTPrintf("Cannot open replacement MBR file '%s' specified with -mbr: %Rrc\n", pszMBRFilename, vrc);
@@ -1226,6 +1296,7 @@ static int CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirtualB
     vdInterfaceErrorCallbacks.cbSize       = sizeof(VDINTERFACEERROR);
     vdInterfaceErrorCallbacks.enmInterface = VDINTERFACETYPE_ERROR;
     vdInterfaceErrorCallbacks.pfnError     = handleVDError;
+    vdInterfaceErrorCallbacks.pfnMessage   = NULL;
 
     vrc = VDInterfaceAdd(&vdInterfaceError, "VBoxManage_IError", VDINTERFACETYPE_ERROR,
                          &vdInterfaceErrorCallbacks, NULL, &pVDIfs);
@@ -1280,7 +1351,7 @@ static int CmdCreateRawVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirtualB
 
     if (fRegister)
     {
-        ComPtr<IHardDisk> hardDisk;
+        ComPtr<IMedium> hardDisk;
         CHECK_ERROR(aVirtualBox, OpenHardDisk(filename, AccessMode_ReadWrite, false, Bstr(""), false, Bstr(""), hardDisk.asOutParam()));
     }
 
@@ -1335,6 +1406,7 @@ static int CmdRenameVMDK(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBox,
     vdInterfaceErrorCallbacks.cbSize       = sizeof(VDINTERFACEERROR);
     vdInterfaceErrorCallbacks.enmInterface = VDINTERFACETYPE_ERROR;
     vdInterfaceErrorCallbacks.pfnError     = handleVDError;
+    vdInterfaceErrorCallbacks.pfnMessage   = NULL;
 
     int vrc = VDInterfaceAdd(&vdInterfaceError, "VBoxManage_IError", VDINTERFACETYPE_ERROR,
                              &vdInterfaceErrorCallbacks, NULL, &pVDIfs);
@@ -1416,6 +1488,7 @@ static int CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBo
     vdInterfaceErrorCallbacks.cbSize       = sizeof(VDINTERFACEERROR);
     vdInterfaceErrorCallbacks.enmInterface = VDINTERFACETYPE_ERROR;
     vdInterfaceErrorCallbacks.pfnError     = handleVDError;
+    vdInterfaceErrorCallbacks.pfnMessage   = NULL;
 
     int vrc = VDInterfaceAdd(&vdInterfaceError, "VBoxManage_IError", VDINTERFACETYPE_ERROR,
                              &vdInterfaceErrorCallbacks, NULL, &pVDIfs);
@@ -1434,7 +1507,7 @@ static int CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBo
     if (fWriteToStdOut)
         outFile = 1;
     else
-        vrc = RTFileOpen(&outFile, Utf8Str(dst).raw(), RTFILE_O_OPEN | RTFILE_O_CREATE | RTFILE_O_WRITE | RTFILE_O_DENY_ALL);
+        vrc = RTFileOpen(&outFile, Utf8Str(dst).raw(), RTFILE_O_WRITE | RTFILE_O_CREATE | RTFILE_O_DENY_ALL);
     if (RT_FAILURE(vrc))
     {
         VDCloseAll(pDisk);
@@ -1445,7 +1518,7 @@ static int CmdConvertToRaw(int argc, char **argv, ComPtr<IVirtualBox> aVirtualBo
     if (srcformat.isEmpty())
     {
         char *pszFormat = NULL;
-        vrc = VDGetFormat(Utf8Str(src).raw(), &pszFormat);
+        vrc = VDGetFormat(NULL, Utf8Str(src).raw(), &pszFormat);
         if (RT_FAILURE(vrc))
         {
             VDCloseAll(pDisk);
@@ -1580,6 +1653,7 @@ static int CmdConvertHardDisk(int argc, char **argv, ComPtr<IVirtualBox> aVirtua
     vdInterfaceErrorCallbacks.cbSize       = sizeof(VDINTERFACEERROR);
     vdInterfaceErrorCallbacks.enmInterface = VDINTERFACETYPE_ERROR;
     vdInterfaceErrorCallbacks.pfnError     = handleVDError;
+    vdInterfaceErrorCallbacks.pfnMessage   = NULL;
 
     vrc = VDInterfaceAdd(&vdInterfaceError, "VBoxManage_IError", VDINTERFACETYPE_ERROR,
                          &vdInterfaceErrorCallbacks, NULL, &pVDIfs);
@@ -1591,7 +1665,7 @@ static int CmdConvertHardDisk(int argc, char **argv, ComPtr<IVirtualBox> aVirtua
         if (srcformat.isEmpty())
         {
             char *pszFormat = NULL;
-            vrc = VDGetFormat(Utf8Str(src).raw(), &pszFormat);
+            vrc = VDGetFormat(NULL, Utf8Str(src).raw(), &pszFormat);
             if (RT_FAILURE(vrc))
             {
                 RTPrintf("No file format specified and autodetect failed - please specify format: %Rrc\n", vrc);
@@ -1702,7 +1776,9 @@ int handleInternalCommands(HandlerArg *a)
     //if (!strcmp(pszCmd, "unloadsyms"))
     //    return CmdUnloadSyms(argc - 1 , &a->argv[1]);
     if (!strcmp(pszCmd, "sethduuid") || !strcmp(pszCmd, "setvdiuuid"))
-        return handleSetHDUUID(a->argc - 1, &a->argv[1], a->virtualBox, a->session);
+        return CmdSetHDUUID(a->argc - 1, &a->argv[1], a->virtualBox, a->session);
+    if (!strcmp(pszCmd, "dumphdinfo"))
+        return CmdDumpHDInfo(a->argc - 1, &a->argv[1], a->virtualBox, a->session);
     if (!strcmp(pszCmd, "listpartitions"))
         return CmdListPartitions(a->argc - 1, &a->argv[1], a->virtualBox, a->session);
     if (!strcmp(pszCmd, "createrawvmdk"))
