@@ -46,10 +46,14 @@ using namespace com;
 #include "Framebuffer.h"
 #include "Ico64x01.h"
 
+#if defined(RT_OS_WINDOWS) || defined(RT_OS_LINUX)
+#include <SDL_syswm.h>           /* for SDL_GetWMInfo() */
+#endif
+
 #if defined(VBOX_WITH_XPCOM)
-NS_IMPL_ISUPPORTS1_CI(VBoxSDLFB, IFramebuffer)
+NS_IMPL_THREADSAFE_ISUPPORTS1_CI(VBoxSDLFB, IFramebuffer)
 NS_DECL_CLASSINFO(VBoxSDLFB)
-NS_IMPL_ISUPPORTS1_CI(VBoxSDLFBOverlay, IFramebufferOverlay)
+NS_IMPL_THREADSAFE_ISUPPORTS1_CI(VBoxSDLFBOverlay, IFramebufferOverlay)
 NS_DECL_CLASSINFO(VBoxSDLFBOverlay)
 #endif
 
@@ -66,6 +70,9 @@ DECLSPEC void (SDLCALL *pTTF_Quit)(void);
 }
 #endif /* VBOX_SECURELABEL */
 
+static SDL_Surface *gWMIcon;                    /**< the application icon */
+static RTNATIVETHREAD gSdlNativeThread;         /**< the SDL thread */
+
 //
 // Constructor / destructor
 //
@@ -82,7 +89,8 @@ DECLSPEC void (SDLCALL *pTTF_Quit)(void);
  * @param iFixedWidth    fixed SDL width (-1 means not set)
  * @param iFixedHeight   fixed SDL height (-1 means not set)
  */
-VBoxSDLFB::VBoxSDLFB(bool fFullscreen, bool fResizable, bool fShowSDLConfig,
+VBoxSDLFB::VBoxSDLFB(uint32_t uScreenId,
+                     bool fFullscreen, bool fResizable, bool fShowSDLConfig,
                      bool fKeepHostRes, uint32_t u32FixedWidth,
                      uint32_t u32FixedHeight, uint32_t u32FixedBPP)
 {
@@ -93,7 +101,12 @@ VBoxSDLFB::VBoxSDLFB(bool fFullscreen, bool fResizable, bool fShowSDLConfig,
     refcnt = 0;
 #endif
 
+    mScreenId       = uScreenId;
     mScreen         = NULL;
+#ifdef VBOX_WITH_SDL13
+    mWindow         = 0;
+    mTexture        = 0;
+#endif
     mSurfVRAM       = NULL;
     mfInitialized   = false;
     mfFullscreen    = fFullscreen;
@@ -104,7 +117,6 @@ VBoxSDLFB::VBoxSDLFB(bool fFullscreen, bool fResizable, bool fShowSDLConfig,
     mFixedSDLWidth  = u32FixedWidth;
     mFixedSDLHeight = u32FixedHeight;
     mFixedSDLBPP    = u32FixedBPP;
-    mDefaultSDLBPP  = 32;
     mCenterXOffset  = 0;
     mCenterYOffset  = 0;
     /* Start with standard screen dimensions. */
@@ -115,19 +127,48 @@ VBoxSDLFB::VBoxSDLFB(bool fFullscreen, bool fResizable, bool fShowSDLConfig,
     mPtrVRAM        = NULL;
     mBitsPerPixel   = 0;
     mBytesPerLine   = 0;
+    mfSameSizeRequested = false;
 #ifdef VBOX_SECURELABEL
     mLabelFont      = NULL;
     mLabelHeight    = 0;
     mLabelOffs      = 0;
 #endif
-    mWMIcon         = NULL;
-
-    /* memorize the thread that inited us, that's the SDL thread */
-    mSdlNativeThread = RTThreadNativeSelf();
 
     rc = RTCritSectInit(&mUpdateLock);
     AssertMsg(rc == VINF_SUCCESS, ("Error from RTCritSectInit!\n"));
 
+    resizeGuest();
+    Assert(mScreen);
+    mfInitialized = true;
+}
+
+VBoxSDLFB::~VBoxSDLFB()
+{
+    LogFlow(("VBoxSDLFB::~VBoxSDLFB\n"));
+    if (mSurfVRAM)
+    {
+        SDL_FreeSurface(mSurfVRAM);
+        mSurfVRAM = NULL;
+    }
+    mScreen = NULL;
+
+#ifdef VBOX_SECURELABEL
+    if (mLabelFont)
+        pTTF_CloseFont(mLabelFont);
+    if (pTTF_Quit)
+        pTTF_Quit();
+#endif
+
+    RTCritSectDelete(&mUpdateLock);
+}
+
+bool VBoxSDLFB::init(bool fShowSDLConfig)
+{
+    LogFlow(("VBoxSDLFB::init\n"));
+
+    /* memorize the thread that inited us, that's the SDL thread */
+    gSdlNativeThread = RTThreadNativeSelf();
+
 #ifdef RT_OS_WINDOWS
     /* default to DirectX if nothing else set */
     if (!RTEnvGet("SDL_VIDEODRIVER"))
@@ -141,27 +182,19 @@ VBoxSDLFB::VBoxSDLFB(bool fFullscreen, bool fResizable, bool fShowSDLConfig,
      * See http://wiki.clug.org.za/wiki/QEMU_mouse_not_working */
     RTEnvSet("SDL_VIDEO_X11_DGAMOUSE", "0");
 #endif
-    rc = SDL_InitSubSystem(SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_NOPARACHUTE);
+    int rc = SDL_InitSubSystem(SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_NOPARACHUTE);
     if (rc != 0)
     {
         RTPrintf("SDL Error: '%s'\n", SDL_GetError());
-        return;
+        return false;
     }
 
     const SDL_VideoInfo *videoInfo = SDL_GetVideoInfo();
     Assert(videoInfo);
     if (videoInfo)
     {
-        switch (videoInfo->vfmt->BitsPerPixel)
-        {
-            case 16: mDefaultSDLBPP = 16; break;
-            case 24: mDefaultSDLBPP = 24; break;
-            default:
-            case 32: mDefaultSDLBPP = 32; break;
-        }
-
         /* output what SDL is capable of */
-        if (mfShowSDLConfig)
+        if (fShowSDLConfig)
             RTPrintf("SDL capabilities:\n"
                      "  Hardware surface support:                    %s\n"
                      "  Window manager available:                    %s\n"
@@ -191,27 +224,34 @@ VBoxSDLFB::VBoxSDLFB(bool fFullscreen, bool fResizable, bool fShowSDLConfig,
 
     if (12320 == g_cbIco64x01)
     {
-        mWMIcon = SDL_AllocSurface(SDL_SWSURFACE, 64, 64, 24, 0xff, 0xff00, 0xff0000, 0);
+        gWMIcon = SDL_AllocSurface(SDL_SWSURFACE, 64, 64, 24, 0xff, 0xff00, 0xff0000, 0);
         /** @todo make it as simple as possible. No PNM interpreter here... */
-        if (mWMIcon)
+        if (gWMIcon)
         {
-            memcpy(mWMIcon->pixels, g_abIco64x01+32, g_cbIco64x01-32);
-            SDL_WM_SetIcon(mWMIcon, NULL);
+            memcpy(gWMIcon->pixels, g_abIco64x01+32, g_cbIco64x01-32);
+            SDL_WM_SetIcon(gWMIcon, NULL);
         }
     }
 
-    resizeGuest();
-    Assert(mScreen);
-    mfInitialized = true;
+    return true;
 }
 
-VBoxSDLFB::~VBoxSDLFB()
+/**
+ * Terminate SDL
+ *
+ * @remarks must be called from the SDL thread!
+ */
+void VBoxSDLFB::uninit()
 {
-    LogFlow(("VBoxSDLFB::~VBoxSDLFB\n"));
-    RTCritSectDelete(&mUpdateLock);
+    AssertMsg(gSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
+    SDL_QuitSubSystem(SDL_INIT_VIDEO);
+    if (gWMIcon)
+    {
+        SDL_FreeSurface(gWMIcon);
+        gWMIcon = NULL;
+    }
 }
 
-
 /**
  * Returns the current framebuffer width in pixels.
  *
@@ -278,16 +318,14 @@ STDMETHODIMP VBoxSDLFB::COMGETTER(Address)(BYTE **address)
     if (!address)
         return E_INVALIDARG;
 
-    if (mSurfVRAM)
-    {
-        *address = (BYTE *) mSurfVRAM->pixels;
-    }
-    else
+    if (!mSurfVRAM)
     {
         /* That's actually rather bad. */
         AssertMsgFailed(("mSurfVRAM is NULL!\n"));
         return E_FAIL;
     }
+
+    *address = (BYTE *) mSurfVRAM->pixels;
     LogFlow(("VBoxSDL::GetAddress returning %p\n", *address));
     return S_OK;
 }
@@ -377,6 +415,20 @@ STDMETHODIMP VBoxSDLFB::COMGETTER(Overlay)(IFramebufferOverlay **aOverlay)
     return S_OK;
 }
 
+/**
+ * Returns handle of window where framebuffer context is being drawn
+ *
+ * @returns COM status code.
+ * @param   winId Handle of associated window.
+ */
+STDMETHODIMP VBoxSDLFB::COMGETTER(WinId)(uint64_t *winId)
+{
+    if (!winId)
+        return E_POINTER;
+    *winId = mWinId;
+    return S_OK;
+}
+
 /**
  * Notify framebuffer of an update.
  *
@@ -392,7 +444,7 @@ STDMETHODIMP VBoxSDLFB::COMGETTER(Overlay)(IFramebufferOverlay **aOverlay)
  *                   continuing with display updates.
  */
 STDMETHODIMP VBoxSDLFB::NotifyUpdate(ULONG x, ULONG y,
-                                     ULONG w, ULONG h, BOOL *finished)
+                                     ULONG w, ULONG h)
 {
     /*
      * The input values are in guest screen coordinates.
@@ -409,6 +461,7 @@ STDMETHODIMP VBoxSDLFB::NotifyUpdate(ULONG x, ULONG y,
      */
     SDL_Event event;
     event.type       = SDL_USEREVENT;
+    event.user.code  = mScreenId;
     event.user.type  = SDL_USER_EVENT_UPDATERECT;
     // 16 bit is enough for coordinates
     event.user.data1 = (void*)(x << 16 | y);
@@ -418,12 +471,6 @@ STDMETHODIMP VBoxSDLFB::NotifyUpdate(ULONG x, ULONG y,
     update(x, y, w, h, true /* fGuestRelative */);
 #endif /* !VBOXSDL_WITH_X11 */
 
-    /*
-     * The Display thread can continue as we will lock the framebuffer
-     * from the SDL thread when we get to actually doing the update.
-     */
-    if (finished)
-        *finished = TRUE;
     return S_OK;
 }
 
@@ -462,6 +509,25 @@ STDMETHODIMP VBoxSDLFB::RequestResize(ULONG aScreenId, ULONG pixelFormat, BYTE *
         return E_FAIL;
     }
 
+    /*
+     * Optimize the case when the guest has changed only the VRAM ptr
+     * and the framebuffer uses the guest VRAM as the source bitmap.
+     */
+    if (   mGuestXRes    == w
+        && mGuestYRes    == h
+        && mPixelFormat  == pixelFormat
+        && mBitsPerPixel == bitsPerPixel
+        && mBytesPerLine == bytesPerLine
+        && mUsesGuestVRAM
+       )
+    {
+        mfSameSizeRequested = true;
+    }
+    else
+    {
+        mfSameSizeRequested = false;
+    }
+
     mGuestXRes   = w;
     mGuestYRes   = h;
     mPixelFormat = pixelFormat;
@@ -473,6 +539,7 @@ STDMETHODIMP VBoxSDLFB::RequestResize(ULONG aScreenId, ULONG pixelFormat, BYTE *
     SDL_Event event;
     event.type       = SDL_USEREVENT;
     event.user.type  = SDL_USER_EVENT_RESIZE;
+    event.user.code  = mScreenId;
 
     /* Try multiple times if necessary */
     PushSDLEventForSure(&event);
@@ -485,36 +552,6 @@ STDMETHODIMP VBoxSDLFB::RequestResize(ULONG aScreenId, ULONG pixelFormat, BYTE *
     return S_OK;
 }
 
-/**
- * Returns which acceleration operations are supported
- *
- * @returns   COM status code
- * @param     operation acceleration operation code
- * @supported result
- */
-STDMETHODIMP VBoxSDLFB::OperationSupported(FramebufferAccelerationOperation_T operation, BOOL *supported)
-{
-    if (!supported)
-        return E_POINTER;
-
-    // SDL gives us software surfaces, futile
-    *supported = false;
-#if 0
-    switch (operation)
-    {
-        case FramebufferAccelerationOperation_SolidFillAcceleration:
-            *supported = true;
-             break;
-        case FramebufferAccelerationOperation_ScreenCopyAcceleration:
-            *supported = true;
-             break;
-        default:
-            *supported = false;
-    }
-#endif
-    return S_OK;
-}
-
 /**
  * Returns whether we like the given video mode.
  *
@@ -549,41 +586,6 @@ STDMETHODIMP VBoxSDLFB::VideoModeSupported(ULONG width, ULONG height, ULONG bpp,
     return S_OK;
 }
 
-STDMETHODIMP VBoxSDLFB::SolidFill(ULONG x, ULONG y, ULONG width, ULONG height,
-                                  ULONG color, BOOL *handled)
-{
-    if (!handled)
-        return E_POINTER;
-    // SDL gives us software surfaces, futile
-#if 0
-    printf("SolidFill: x: %d, y: %d, w: %d, h: %d, color: %d\n", x, y, width, height, color);
-    SDL_Rect rect = { (Sint16)x, (Sint16)y, (Sint16)width, (Sint16)height };
-    SDL_FillRect(mScreen, &rect, color);
-    //SDL_UpdateRect(mScreen, x, y, width, height);
-    *handled = true;
-#else
-    *handled = false;
-#endif
-    return S_OK;
-}
-
-STDMETHODIMP VBoxSDLFB::CopyScreenBits(ULONG xDst, ULONG yDst, ULONG xSrc, ULONG ySrc,
-                                       ULONG width, ULONG height, BOOL *handled)
-{
-    if (!handled)
-        return E_POINTER;
-    // SDL gives us software surfaces, futile
-#if 0
-    SDL_Rect srcRect = { (Sint16)xSrc, (Sint16)ySrc, (Sint16)width, (Sint16)height };
-    SDL_Rect dstRect = { (Sint16)xDst, (Sint16)yDst, (Sint16)width, (Sint16)height };
-    SDL_BlitSurface(mScreen, &srcRect, mScreen, &dstRect);
-    *handled = true;
-#else
-    *handled = false;
-#endif
-    return S_OK;
-}
-
 STDMETHODIMP VBoxSDLFB::GetVisibleRegion(BYTE *aRectangles, ULONG aCount,
                                          ULONG *aCountCopied)
 {
@@ -614,6 +616,10 @@ STDMETHODIMP VBoxSDLFB::SetVisibleRegion(BYTE *aRectangles, ULONG aCount)
     return S_OK;
 }
 
+STDMETHODIMP VBoxSDLFB::ProcessVHWACommand(BYTE *pCommand)
+{
+    return E_NOTIMPL;
+}
 //
 // Internal public methods
 //
@@ -625,7 +631,7 @@ STDMETHODIMP VBoxSDLFB::SetVisibleRegion(BYTE *aRectangles, ULONG aCount)
 void VBoxSDLFB::resizeGuest()
 {
     LogFlowFunc (("mGuestXRes: %d, mGuestYRes: %d\n", mGuestXRes, mGuestYRes));
-    AssertMsg(mSdlNativeThread == RTThreadNativeSelf(),
+    AssertMsg(gSdlNativeThread == RTThreadNativeSelf(),
               ("Wrong thread! SDL is not threadsafe!\n"));
 
     uint32_t Rmask, Gmask, Bmask, Amask = 0;
@@ -675,19 +681,31 @@ void VBoxSDLFB::resizeGuest()
     /* is the guest in a linear framebuffer mode we support? */
     if (mUsesGuestVRAM)
     {
-
         /* Create a source surface from guest VRAM. */
         mSurfVRAM = SDL_CreateRGBSurfaceFrom(mPtrVRAM, mGuestXRes, mGuestYRes, mBitsPerPixel,
                                              mBytesPerLine, Rmask, Gmask, Bmask, Amask);
+        LogRel(("mSurfVRAM from guest %d x %d\n", mGuestXRes,  mGuestYRes));
     }
     else
     {
         /* Create a software surface for which SDL allocates the RAM */
         mSurfVRAM = SDL_CreateRGBSurface(SDL_SWSURFACE, mGuestXRes, mGuestYRes, mBitsPerPixel,
                                          Rmask, Gmask, Bmask, Amask);
+        LogRel(("mSurfVRAM from SDL %d x %d\n", mGuestXRes,  mGuestYRes));
     }
     LogFlow(("VBoxSDL:: created VRAM surface %p\n", mSurfVRAM));
 
+    if (mfSameSizeRequested && mUsesGuestVRAM)
+    {
+        /*
+         * Same size has been requested and the framebuffer still uses the guest VRAM.
+         * Reset the condition and return.
+         */
+        mfSameSizeRequested = false;
+        LogFlow(("VBoxSDL:: the same resolution requested, skipping the resize.\n"));
+        return;
+    }
+
     /* now adjust the SDL resolution */
     resizeSDL();
 }
@@ -772,11 +790,103 @@ void VBoxSDLFB::resizeSDL(void)
     /* we don't have any extra space by default */
     mTopOffset = 0;
 
+#if defined(VBOX_WITH_SDL13)
+    int sdlWindowFlags = SDL_WINDOW_SHOWN;
+    if (mfResizable)
+        sdlWindowFlags |= SDL_WINDOW_RESIZABLE;
+    if (!mWindow)
+    {
+        SDL_DisplayMode desktop_mode;
+        int x = 40 + mScreenId * 20;
+        int y = 40 + mScreenId * 15;
+
+        SDL_GetDesktopDisplayMode(&desktop_mode);
+        /* create new window */
+
+        char szTitle[64];
+        RTStrPrintf(szTitle, sizeof(szTitle), "SDL window %d", mScreenId);
+        mWindow = SDL_CreateWindow(szTitle, x, y,
+                                   newWidth, newHeight, sdlWindowFlags);
+        if (SDL_CreateRenderer(mWindow, -1,
+                               SDL_RENDERER_SINGLEBUFFER | SDL_RENDERER_PRESENTDISCARD) < 0)
+            AssertReleaseFailed();
+
+        SDL_GetRendererInfo(&mRenderInfo);
+
+        mTexture = SDL_CreateTexture(desktop_mode.format,
+                                     SDL_TEXTUREACCESS_STREAMING, newWidth, newHeight);
+        if (!mTexture)
+            AssertReleaseFailed();
+    }
+    else
+    {
+        int w, h;
+        uint32_t format;
+        int access;
+
+        /* resize current window */
+        SDL_GetWindowSize(mWindow, &w, &h);
+
+        if (w != (int)newWidth || h != (int)newHeight)
+            SDL_SetWindowSize(mWindow, newWidth, newHeight);
+
+        SDL_QueryTexture(mTexture, &format, &access, &w, &h);
+        SDL_SelectRenderer(mWindow);
+        SDL_DestroyTexture(mTexture);
+        mTexture = SDL_CreateTexture(format, access, newWidth, newHeight);
+        if (!mTexture)
+            AssertReleaseFailed();
+    }
+
+    void *pixels;
+    int pitch;
+    int w, h, bpp;
+    uint32_t Rmask, Gmask, Bmask, Amask;
+    uint32_t format;
+
+    if (SDL_QueryTexture(mTexture, &format, NULL, &w, &h) < 0)
+        AssertReleaseFailed();
+
+    if (!SDL_PixelFormatEnumToMasks(format, &bpp, &Rmask, &Gmask, &Bmask, &Amask))
+        AssertReleaseFailed();
+
+    if (SDL_QueryTexturePixels(mTexture, &pixels, &pitch) == 0)
+    {
+        mScreen = SDL_CreateRGBSurfaceFrom(pixels, w, h, bpp, pitch,
+                                           Rmask, Gmask, Bmask, Amask);
+    }
+    else
+    {
+        mScreen = SDL_CreateRGBSurface(0, w, h, bpp, Rmask, Gmask, Bmask, Amask);
+        AssertReleaseFailed();
+    }
+
+    SDL_SetClipRect(mScreen, NULL);
+
+#else
     /*
      * Now set the screen resolution and get the surface pointer
      * @todo BPP is not supported!
      */
     mScreen = SDL_SetVideoMode(newWidth, newHeight, 0, sdlFlags);
+
+    /*
+     * Set the Window ID. Currently used for OpenGL accelerated guests.
+     */
+# if defined (RT_OS_WINDOWS)
+    SDL_SysWMinfo info;
+    SDL_VERSION(&info.version);
+    if (SDL_GetWMInfo(&info))
+        mWinId = (ULONG64) info.window;
+# elif defined (RT_OS_LINUX)
+    SDL_SysWMinfo info;
+    SDL_VERSION(&info.version);
+    if (SDL_GetWMInfo(&info))
+        mWinId = (ULONG64) info.info.x11.wmwindow;
+# else
+    /* XXX ignore this for other architectures */
+# endif
+#endif
 #ifdef VBOX_SECURELABEL
     /*
      * For non fixed SDL resolution, the above call tried to add the label height
@@ -837,7 +947,7 @@ void VBoxSDLFB::resizeSDL(void)
 void VBoxSDLFB::update(int x, int y, int w, int h, bool fGuestRelative)
 {
 #ifdef VBOXSDL_WITH_X11
-    AssertMsg(mSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
+    AssertMsg(gSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
 #endif
     Assert(mScreen);
     Assert(mSurfVRAM);
@@ -901,16 +1011,22 @@ void VBoxSDLFB::update(int x, int y, int w, int h, bool fGuestRelative)
     dstRect.w = w;
     dstRect.h = RT_MAX(0, h - yCutoffGuest);
 
-    //RTPrintf("y = %d h = %d mapped to srcY %d srcH %d mapped to dstY = %d dstH %d (guestrel: %d, mLabelHeight: %d, mTopOffset: %d)\n",
-    //         y, h, srcRect.y, srcRect.h, dstRect.y, dstRect.h, fGuestRelative, mLabelHeight, mTopOffset);
-
     /*
      * Now we just blit
      */
     SDL_BlitSurface(mSurfVRAM, &srcRect, mScreen, &dstRect);
     /* hardware surfaces don't need update notifications */
+#if defined(VBOX_WITH_SDL13)
+    AssertRelease(mScreen->flags & SDL_PREALLOC);
+    SDL_SelectRenderer(mWindow);
+    SDL_DirtyTexture(mTexture, 1, &dstRect);
+    AssertRelease(mRenderInfo.flags & SDL_RENDERER_PRESENTCOPY);
+    SDL_RenderCopy(mTexture, &dstRect, &dstRect);
+    SDL_RenderPresent();
+#else
     if ((mScreen->flags & SDL_HWSURFACE) == 0)
         SDL_UpdateRect(mScreen, dstRect.x, dstRect.y, dstRect.w, dstRect.h);
+#endif
 
 #ifdef VBOX_SECURELABEL
     if (fPaintLabel)
@@ -925,17 +1041,11 @@ void VBoxSDLFB::update(int x, int y, int w, int h, bool fGuestRelative)
  */
 void VBoxSDLFB::repaint()
 {
-    AssertMsg(mSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
+    AssertMsg(gSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
     LogFlow(("VBoxSDLFB::repaint\n"));
     update(0, 0, mScreen->w, mScreen->h, false /* fGuestRelative */);
 }
 
-bool VBoxSDLFB::getFullscreen()
-{
-    LogFlow(("VBoxSDLFB::getFullscreen\n"));
-    return mfFullscreen;
-}
-
 /**
  * Toggle fullscreen mode
  *
@@ -943,7 +1053,7 @@ bool VBoxSDLFB::getFullscreen()
  */
 void VBoxSDLFB::setFullscreen(bool fFullscreen)
 {
-    AssertMsg(mSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
+    AssertMsg(gSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
     LogFlow(("VBoxSDLFB::SetFullscreen: fullscreen: %d\n", fFullscreen));
     mfFullscreen = fFullscreen;
     /* only change the SDL resolution, do not touch the guest framebuffer */
@@ -986,28 +1096,6 @@ void VBoxSDLFB::getFullscreenGeometry(uint32_t *width, uint32_t *height)
     }
 }
 
-/**
- * Returns the current x offset of the start of the guest screen
- *
- * @returns current x offset in pixels
- */
-int VBoxSDLFB::getXOffset()
-{
-    /* there can only be an offset for centering */
-    return mCenterXOffset;
-}
-
-/**
- * Returns the current y offset of the start of the guest screen
- *
- * @returns current y offset in pixels
- */
-int VBoxSDLFB::getYOffset()
-{
-    /* we might have a top offset and a center offset */
-    return mTopOffset + mCenterYOffset;
-}
-
 #ifdef VBOX_SECURELABEL
 /**
  * Setup the secure labeling parameters
@@ -1072,7 +1160,7 @@ void VBoxSDLFB::setSecureLabelColor(uint32_t colorFG, uint32_t colorBG)
 void VBoxSDLFB::paintSecureLabel(int x, int y, int w, int h, bool fForce)
 {
 #ifdef VBOXSDL_WITH_X11
-    AssertMsg(mSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
+    AssertMsg(gSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
 #endif
     /* only when the function is present */
     if (!pTTF_RenderUTF8_Solid)
@@ -1090,7 +1178,9 @@ void VBoxSDLFB::paintSecureLabel(int x, int y, int w, int h, bool fForce)
                                             mSecureLabelColorBG & 0x000000FF)); /* blue  */
 
     /* now the text */
-    if (mLabelFont != NULL && mSecureLabelText)
+    if (    mLabelFont != NULL
+         && !mSecureLabelText.isEmpty()
+       )
     {
         SDL_Color clrFg = {(mSecureLabelColorFG & 0x00FF0000) >> 16,
                            (mSecureLabelColorFG & 0x0000FF00) >> 8,
@@ -1108,34 +1198,6 @@ void VBoxSDLFB::paintSecureLabel(int x, int y, int w, int h, bool fForce)
 }
 #endif /* VBOX_SECURELABEL */
 
-/**
- * Terminate SDL
- *
- * @remarks must be called from the SDL thread!
- */
-void VBoxSDLFB::uninit()
-{
-    AssertMsg(mSdlNativeThread == RTThreadNativeSelf(), ("Wrong thread! SDL is not threadsafe!\n"));
-    if (mSurfVRAM)
-    {
-        SDL_FreeSurface(mSurfVRAM);
-        mSurfVRAM = NULL;
-    }
-    SDL_QuitSubSystem(SDL_INIT_VIDEO);
-#ifdef VBOX_SECURELABEL
-    if (mLabelFont)
-        pTTF_CloseFont(mLabelFont);
-    if (pTTF_Quit)
-        pTTF_Quit();
-#endif
-    mScreen = NULL;
-    if (mWMIcon)
-    {
-        SDL_FreeSurface(mWMIcon);
-        mWMIcon = NULL;
-    }
-}
-
 // IFramebufferOverlay
 ///////////////////////////////////////////////////////////////////////////////////
 
@@ -1401,6 +1463,22 @@ STDMETHODIMP VBoxSDLFBOverlay::COMGETTER(Overlay)(IFramebufferOverlay **aOverlay
     return S_OK;
 }
 
+/**
+ * Returns associated window handle. We return NULL here.
+ *
+ * @returns COM status code
+ * @param   winId Address of result buffer.
+ */
+STDMETHODIMP VBoxSDLFBOverlay::COMGETTER(WinId)(ULONG64 *winId)
+{
+    LogFlow(("VBoxSDLFBOverlay::GetWinId\n"));
+    if (!winId)
+        return E_INVALIDARG;
+    *winId = 0;
+    return S_OK;
+}
+
+
 /**
  * Lock the overlay.  This should not be used - lock the parent IFramebuffer instead.
  *
@@ -1457,9 +1535,9 @@ STDMETHODIMP VBoxSDLFBOverlay::Move(ULONG x, ULONG y)
  * checking to the parent.
  */
 STDMETHODIMP VBoxSDLFBOverlay::NotifyUpdate(ULONG x, ULONG y,
-                            ULONG w, ULONG h, BOOL *finished)
+                            ULONG w, ULONG h)
 {
-    return mParent->NotifyUpdate(x + mOverlayX, y + mOverlayY, w, h, finished);
+    return mParent->NotifyUpdate(x + mOverlayX, y + mOverlayY, w, h);
 }
 
 /**
@@ -1495,25 +1573,6 @@ STDMETHODIMP VBoxSDLFBOverlay::RequestResize(ULONG aScreenId, ULONG pixelFormat,
     return S_OK;
 }
 
-/**
- * Queries whether we support a given accelerated opperation.  Since we do not currently
- * support any accelerated operations, we always return false in supported.
- *
- * @returns          COM status code
- * @param  operation The operation being queried
- * @retval supported Whether or not we support that operation
- */
-STDMETHODIMP VBoxSDLFBOverlay::OperationSupported(FramebufferAccelerationOperation_T
-                                                  operation, BOOL *supported)
-{
-    if (!supported)
-        return E_POINTER;
-    /* We currently do not support any acceleration here, and will probably not in
-       the forseeable future. */
-    *supported = false;
-    return S_OK;
-}
-
 /**
  * Returns whether we like the given video mode.
  *
@@ -1536,43 +1595,3 @@ STDMETHODIMP VBoxSDLFBOverlay::VideoModeSupported(ULONG width, ULONG height, ULO
         *supported = false;
     return S_OK;
 }
-
-/**
- * Fill an area of the framebuffer with solid colour
- *
- * @returns COM status code
- * @param   x       X co-ordinate of the area to fill, top-left corner
- * @param   y       Y co-ordinate of the area to fill, top-left corner
- * @param   width   width of the area to fill
- * @param   height  height of the area to fill
- * @param   color   colour with which to fill the area
- * @retval  handled whether we support this operation or not
- *
- * Since we currently do not have any way of doing this faster than
- * the VGA device, we simply false in handled.
- */
-STDMETHODIMP VBoxSDLFBOverlay::SolidFill(ULONG x, ULONG y, ULONG width,
-                                         ULONG height, ULONG color, BOOL *handled)
-{
-    LogFlow(("VBoxSDLFBOverlay::SolidFill called\n"));
-    if (!handled)
-        return E_POINTER;
-    *handled = false;
-    return S_OK;
-}
-
-/**
- * Since we currently do not have any way of doing this faster than
- * the VGA device, we simply false in handled.
- */
-STDMETHODIMP VBoxSDLFBOverlay::CopyScreenBits(ULONG xDst, ULONG yDst, ULONG xSrc,
-                                              ULONG ySrc, ULONG width,
-                                              ULONG height, BOOL *handled)
-{
-    LogFlow(("VBoxSDLFBOverlay::CopyScreenBits called.\n"));
-    if (!handled)
-        return E_POINTER;
-    *handled = false;
-    return S_OK;
-}
-
