@@ -27,6 +27,9 @@
 #include "VBoxConsoleView.h"
 
 #include <qapplication.h>
+//Added by qt3to4:
+#include <QMoveEvent>
+#include <QPaintEvent>
 
 #include <iprt/param.h>
 #include <iprt/alloc.h>
@@ -225,22 +228,20 @@ void VBoxDDRAWFrameBuffer::releaseObjects()
 
 /** @note This method is called on EMT from under this object's lock */
 STDMETHODIMP VBoxDDRAWFrameBuffer::NotifyUpdate (ULONG aX, ULONG aY,
-                                                 ULONG aW, ULONG aH,
-                                                 BOOL *aFinished)
+                                                 ULONG aW, ULONG aH)
 {
     LOGDDRAW(("DDRAW: NotifyUpdate %d,%d %dx%d\n", aX, aY, aW, aH));
 
     if (mSynchronousUpdates)
     {
-        mView->updateContents (aX, aY, aW, aH);
+//#warning check me!
+        mView->viewport()->update (aX, aY, aW, aH);
     }
     else
     {
         drawRect (aX, aY, aW, aH);
     }
 
-    *aFinished = TRUE;
-
     return S_OK;
 }
 
@@ -276,7 +277,8 @@ void VBoxDDRAWFrameBuffer::resizeEvent (VBoxResizeEvent *re)
 
     getWindowPosition();
 
-    mView->setBackgroundMode (Qt::NoBackground);
+//#warning: port me
+//    mView->setBackgroundMode (Qt::NoBackground);
 }
 
 void VBoxDDRAWFrameBuffer::moveEvent (QMoveEvent *me)
