@@ -140,7 +140,7 @@ extern "C" {
 #define WGL_ACCUM_ALPHA_BITS_EXT        0x2021
 #endif
 #ifndef WGL_DEPTH_BITS_EXT
-#define WGL_DEPTH_BITS_EXT          0x2022
+#define WGL_DEPTH_BITS_EXT              0x2022
 #endif
 #ifndef WGL_STENCIL_BITS_EXT
 #define WGL_STENCIL_BITS_EXT            0x2023
@@ -149,7 +149,139 @@ extern "C" {
 #define WGL_SAMPLE_BUFFERS_EXT          0x2041
 #endif
 #ifndef WGL_SAMPLES_EXT
-#define WGL_SAMPLES_EXT             0x2042
+#define WGL_SAMPLES_EXT                 0x2042
+#endif
+#ifndef WGL_SUPPORT_OPENGL_ARB
+#define WGL_SUPPORT_OPENGL_ARB          0x2010
+#endif
+#ifndef WGL_NUMBER_PIXEL_FORMATS_ARB
+#define WGL_NUMBER_PIXEL_FORMATS_ARB    0x2000
+#endif
+#ifndef WGL_FULL_ACCELERATION_ARB
+#define WGL_FULL_ACCELERATION_ARB       0x2027
+#endif
+#ifndef WGL_SWAP_UNDEFINED_ARB
+#define WGL_SWAP_UNDEFINED_ARB          0x202A
+#endif
+#ifndef WGL_TYPE_RGBA_ARB
+#define WGL_TYPE_RGBA_ARB               0x202B
+#endif
+#ifndef WGL_DRAW_TO_WINDOW_ARB
+#define WGL_DRAW_TO_WINDOW_ARB                  0x2001
+#endif
+#ifndef WGL_DRAW_TO_BITMAP_ARB
+#define WGL_DRAW_TO_BITMAP_ARB                  0x2002
+#endif
+#ifndef WGL_DOUBLE_BUFFER_ARB
+#define WGL_DOUBLE_BUFFER_ARB                   0x2011
+#endif
+#ifndef WGL_NEED_PALETTE_ARB
+#define WGL_NEED_PALETTE_ARB                    0x2004
+#endif
+#ifndef WGL_NEED_SYSTEM_PALETTE_ARB
+#define WGL_NEED_SYSTEM_PALETTE_ARB             0x2005
+#endif
+#ifndef WGL_SWAP_LAYER_BUFFERS_ARB
+#define WGL_SWAP_LAYER_BUFFERS_ARB              0x2006
+#endif
+#ifndef WGL_NUMBER_OVERLAYS_ARB
+#define WGL_NUMBER_OVERLAYS_ARB                 0x2008
+#endif
+#ifndef WGL_NUMBER_UNDERLAYS_ARB
+#define WGL_NUMBER_UNDERLAYS_ARB                0x2009
+#endif
+#ifndef WGL_TRANSPARENT_ARB
+#define WGL_TRANSPARENT_ARB                     0x200A
+#endif
+#ifndef WGL_TRANSPARENT_RED_VALUE_ARB
+#define WGL_TRANSPARENT_RED_VALUE_ARB           0x2037
+#endif
+#ifndef WGL_TRANSPARENT_GREEN_VALUE_ARB
+#define WGL_TRANSPARENT_GREEN_VALUE_ARB         0x2038
+#endif
+#ifndef WGL_TRANSPARENT_BLUE_VALUE_ARB
+#define WGL_TRANSPARENT_BLUE_VALUE_ARB          0x2039
+#endif
+#ifndef WGL_TRANSPARENT_ALPHA_VALUE_ARB
+#define WGL_TRANSPARENT_ALPHA_VALUE_ARB         0x203A
+#endif
+#ifndef WGL_TRANSPARENT_INDEX_VALUE_ARB
+#define WGL_TRANSPARENT_INDEX_VALUE_ARB         0x203B
+#endif
+#ifndef WGL_SHARE_STENCIL_ARB
+#define WGL_SHARE_STENCIL_ARB                   0x200D
+#endif
+#ifndef WGL_SHARE_ACCUM_ARB                    
+#define WGL_SHARE_ACCUM_ARB                     0x200E
+#endif
+#ifndef WGL_SUPPORT_GDI_ARB                    
+#define WGL_SUPPORT_GDI_ARB                     0x200F
+#endif
+#ifndef WGL_RED_BITS_ARB                       
+#define WGL_RED_BITS_ARB                        0x2015
+#endif
+#ifndef WGL_RED_SHIFT_ARB                      
+#define WGL_RED_SHIFT_ARB                       0x2016
+#endif
+#ifndef WGL_GREEN_BITS_ARB                     
+#define WGL_GREEN_BITS_ARB                      0x2017
+#endif
+#ifndef WGL_GREEN_SHIFT_ARB                    
+#define WGL_GREEN_SHIFT_ARB                     0x2018
+#endif
+#ifndef WGL_BLUE_BITS_ARB                      
+#define WGL_BLUE_BITS_ARB                       0x2019
+#endif
+#ifndef WGL_BLUE_SHIFT_ARB                     
+#define WGL_BLUE_SHIFT_ARB                      0x201A
+#endif
+#ifndef WGL_ALPHA_BITS_ARB                     
+#define WGL_ALPHA_BITS_ARB                      0x201B
+#endif
+#ifndef WGL_ALPHA_SHIFT_ARB                    
+#define WGL_ALPHA_SHIFT_ARB                     0x201C
+#endif
+#ifndef WGL_ACCUM_BITS_ARB                     
+#define WGL_ACCUM_BITS_ARB                      0x201D
+#endif
+#ifndef WGL_ACCUM_RED_BITS_ARB                 
+#define WGL_ACCUM_RED_BITS_ARB                  0x201E
+#endif
+#ifndef WGL_ACCUM_GREEN_BITS_ARB               
+#define WGL_ACCUM_GREEN_BITS_ARB                0x201F
+#endif
+#ifndef WGL_ACCUM_BLUE_BITS_ARB                
+#define WGL_ACCUM_BLUE_BITS_ARB                 0x2020
+#endif
+#ifndef WGL_ACCUM_ALPHA_BITS_ARB               
+#define WGL_ACCUM_ALPHA_BITS_ARB                0x2021
+#endif
+#ifndef WGL_DEPTH_BITS_ARB                     
+#define WGL_DEPTH_BITS_ARB                      0x2022
+#endif
+#ifndef WGL_STENCIL_BITS_ARB                   
+#define WGL_STENCIL_BITS_ARB                    0x2023
+#endif
+#ifndef WGL_AUX_BUFFERS_ARB                    
+#define WGL_AUX_BUFFERS_ARB                     0x2024
+#endif
+#ifndef WGL_STEREO_ARB                         
+#define WGL_STEREO_ARB                          0x2012
+#endif
+#ifndef WGL_ACCELERATION_ARB                   
+#define WGL_ACCELERATION_ARB                    0x2003
+#endif
+#ifndef WGL_SHARE_DEPTH_ARB                    
+#define WGL_SHARE_DEPTH_ARB                     0x200C
+#endif
+#ifndef WGL_PIXEL_TYPE_ARB                     
+#define WGL_PIXEL_TYPE_ARB                      0x2013
+#endif
+#ifndef WGL_COLOR_BITS_ARB                     
+#define WGL_COLOR_BITS_ARB                      0x2014
+#endif
+#ifndef WGL_SWAP_METHOD_ARB                    
+#define WGL_SWAP_METHOD_ARB                     0x2007
 #endif
 
 /*
@@ -544,6 +676,14 @@ extern void APIENTRY glZPixCR(GLsizei width, GLsizei height, GLenum format,
 
 #endif /* GL_CR_zpix */
 
+/*Allow to use glGetString to query real host GPU info*/
+#ifndef GL_CR_real_vendor_strings
+#define GL_CR_real_vendor_strings 1
+#define GL_REAL_VENDOR   0x8B23
+#define GL_REAL_VERSION  0x8B24
+#define GL_REAL_RENDERER 0x8B25
+#endif
+
 
 /**********************************************************************/
 /*****                Chromium-specific API                       *****/
@@ -617,7 +757,6 @@ CR_PROC APIENTRY crGetProcAddress( const char *name );
 #define GET_PROC(NAME) NULL
 #endif
 
-
 #ifdef __cplusplus
 }
 #endif
