@@ -36,9 +36,9 @@ void crStateBufferObjectInit (CRContext *ctx)
     RESET(bb->elementsBinding, ctx->bitid);
 
 #ifdef IN_GUEST
-    b->retainBufferData = GL_FALSE;
-#else
     b->retainBufferData = GL_TRUE;
+#else
+    b->retainBufferData = GL_FALSE;
 #endif
 
     b->nullBuffer = AllocBufferObject(0);
@@ -47,17 +47,33 @@ void crStateBufferObjectInit (CRContext *ctx)
     b->nullBuffer->refCount = 3;
 
     b->buffers = crAllocHashtable();
+
+    b->bResyncNeeded = GL_FALSE;
 }
 
 
+static void crStateFreeBufferObject(void *data)
+{
+    CRBufferObject *pObj = (CRBufferObject *)data;
+    if (pObj->data) crFree(pObj->data);
+
+#ifndef IN_GUEST
+    if (diff_api.DeleteBuffersARB)
+    {
+        diff_api.DeleteBuffersARB(1, &pObj->name);
+    }
+#endif
+
+    crFree(pObj);
+}
+
 void crStateBufferObjectDestroy (CRContext *ctx)
 {
     CRBufferObjectState *b = &ctx->bufferobject;
-    crFreeHashtable(b->buffers, crFree);
+    crFreeHashtable(b->buffers, crStateFreeBufferObject);
     crFree(b->nullBuffer);
 }
 
-
 void STATE_APIENTRY
 crStateBindBufferARB (GLenum target, GLuint buffer)
 {
@@ -118,7 +134,9 @@ crStateBindBufferARB (GLenum target, GLuint buffer)
     }
 
     if (oldObj->refCount <= 0) {
-        crHashtableDelete(b->buffers, (unsigned long) oldObj->name, crFree);
+        /*we shouldn't reach this point*/
+        CRASSERT(false);
+        crHashtableDelete(b->buffers, (unsigned long) oldObj->name, crStateFreeBufferObject);
     }
 }
 
@@ -162,9 +180,9 @@ crStateDeleteBuffersARB(GLsizei n, const GLuint *buffers)
                     DIRTY(bb->dirty, g->neg_bitid);
                     DIRTY(bb->elementsBinding, g->neg_bitid);
                 }
-                /* XXX check bindings with the vertex arrays */
+                /* @todo check bindings with the vertex arrays */
 
-                crHashtableDelete(b->buffers, buffers[i], crFree);
+                crHashtableDelete(b->buffers, buffers[i], crStateFreeBufferObject);
             }
         }
     }
@@ -732,6 +750,23 @@ void crStateBufferObjectDiff(CRBufferObjectBits *bb, CRbitvalue *bitID,
     }
 }
 
+static void crStateBufferObjectSyncCB(unsigned long key, void *data1, void *data2)
+{
+    CRBufferObject *pBufferObj = (CRBufferObject *) data1;
+    CRBufferObjectState *pState = (CRBufferObjectState *) data2;
+
+    if (pBufferObj->data)
+    {
+        diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, pBufferObj->name);
+        diff_api.BufferDataARB(GL_ARRAY_BUFFER_ARB, pBufferObj->size, pBufferObj->data, pBufferObj->usage);
+
+        if (!pState->retainBufferData)
+        {
+            crFree(pBufferObj->data);
+            pBufferObj->data = NULL;
+        }
+    }
+}
 
 /*
  * XXX this function might need some testing/fixing.
@@ -740,11 +775,100 @@ void crStateBufferObjectSwitch(CRBufferObjectBits *bb, CRbitvalue *bitID,
                                                              CRContext *fromCtx, CRContext *toCtx)
 {
     const CRBufferObjectState *from = &(fromCtx->bufferobject);
-    const CRBufferObjectState *to = &(toCtx->bufferobject);
+    CRBufferObjectState *to = &(toCtx->bufferobject);
+    int i;
 
     if (!HaveBufferObjectExtension())
         return;
 
+    if (to->bResyncNeeded)
+    {
+        CRClientPointer *cp;
+
+        crHashtableWalk(to->buffers, crStateBufferObjectSyncCB, to);
+        to->bResyncNeeded = GL_FALSE;
+
+        /*@todo, move to state_client.c*/
+
+        cp = &toCtx->client.array.v;
+        if (cp->buffer->name)
+        {
+            diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+            diff_api.VertexPointer(cp->size, cp->type, cp->stride, cp->p);
+        }
+
+        cp = &toCtx->client.array.c;
+        if (cp->buffer->name)
+        {
+            diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+            diff_api.ColorPointer(cp->size, cp->type, cp->stride, cp->p);
+        }
+
+        cp = &toCtx->client.array.f;
+        if (cp->buffer->name)
+        {
+            diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+            diff_api.FogCoordPointerEXT(cp->type, cp->stride, cp->p);
+        }
+
+        cp = &toCtx->client.array.s;
+        if (cp->buffer->name)
+        {
+            diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+            diff_api.SecondaryColorPointerEXT(cp->size, cp->type, cp->stride, cp->p);
+        }
+
+        cp = &toCtx->client.array.e;
+        if (cp->buffer->name)
+        {
+            diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+            diff_api.EdgeFlagPointer(cp->stride, cp->p);
+        }
+
+        cp = &toCtx->client.array.i;
+        if (cp->buffer->name)
+        {
+            diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+            diff_api.IndexPointer(cp->type, cp->stride, cp->p);
+        }
+
+        cp = &toCtx->client.array.n;
+        if (cp->buffer->name)
+        {
+            diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+            diff_api.NormalPointer(cp->type, cp->stride, cp->p);
+        }
+
+        for (i = 0; i < CR_MAX_TEXTURE_UNITS; i++)
+        {
+            cp = &toCtx->client.array.t[i];
+            if (cp->buffer->name)
+            {
+                if (diff_api.ActiveTextureARB)
+                    diff_api.ActiveTextureARB(i+GL_TEXTURE0_ARB);
+                diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+                diff_api.TexCoordPointer(cp->size, cp->type, cp->stride, cp->p);
+            }
+        }
+
+        if (diff_api.ActiveTextureARB)
+            diff_api.ActiveTextureARB(toCtx->client.curClientTextureUnit+GL_TEXTURE0_ARB);
+
+#ifdef CR_NV_vertex_program
+        for (i = 0; i < CR_MAX_VERTEX_ATTRIBS; i++)
+        {
+            cp = &toCtx->client.array.a[i];
+            if (cp->buffer->name)
+            {
+                diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, cp->buffer->name);
+                diff_api.VertexAttribPointerARB(i, cp->size, cp->type, cp->normalized, cp->stride, cp->p);
+            }
+        }
+#endif
+        diff_api.BindBufferARB(GL_ARRAY_BUFFER_ARB, to->arrayBuffer->name);
+        diff_api.BindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, to->elementsBuffer->name);
+    }
+
     /* ARRAY_BUFFER binding */
     if (CHECKDIRTY(bb->arrayBinding, bitID)) {
         if (from->arrayBuffer != to->arrayBuffer) {
