@@ -40,7 +40,11 @@
 #include "cr_threads.h"
 #include "net_internals.h"
 
-#include <VBox/VBoxGuest.h>
+#if 1 /** @todo Try use the Vbgl interface instead of talking directly to the driver? */
+# include <VBox/VBoxGuest.h>
+#else
+# include <VBox/VBoxGuestLib.h>
+#endif
 #include <VBox/HostServices/VBoxCrOpenGLSvc.h>
 
 typedef struct {
@@ -94,6 +98,7 @@ typedef struct CRVBOXHGCMBUFFER {
 /* Some forward declarations */
 static void crVBoxHGCMReceiveMessage(CRConnection *conn);
 
+#ifndef IN_GUEST
 static bool _crVBoxHGCMReadBytes(CRConnection *conn, void *buf, uint32_t len)
 {
     CRASSERT(conn && buf);
@@ -108,6 +113,7 @@ static bool _crVBoxHGCMReadBytes(CRConnection *conn, void *buf, uint32_t len)
 
     return TRUE;
 }
+#endif
 
 /*@todo get rid of it*/
 static bool _crVBoxHGCMWriteBytes(CRConnection *conn, const void *buf, uint32_t len)
@@ -144,12 +150,12 @@ static bool _crVBoxHGCMWriteBytes(CRConnection *conn, const void *buf, uint32_t
  * @param   pvData      Data pointer
  * @param   cbData      Data size
  */
-/*@todo use vbglR3DoIOCtl here instead */
+/** @todo use vbglR3DoIOCtl here instead */
 static int crVBoxHGCMCall(void *pvData, unsigned cbData)
 {
 #ifdef IN_GUEST
 
-#ifdef RT_OS_WINDOWS
+# ifdef RT_OS_WINDOWS
     DWORD cbReturned;
 
     if (DeviceIoControl (g_crvboxhgcm.hGuestDrv,
@@ -163,14 +169,36 @@ static int crVBoxHGCMCall(void *pvData, unsigned cbData)
     }
     crDebug("vboxCall failed with %x\n", GetLastError());
     return VERR_NOT_SUPPORTED;
-#else
-    if (ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CALL(cbData), pvData) >= 0)
+# else
+    int rc;
+#  ifdef RT_OS_SOLARIS
+    VBGLBIGREQ Hdr;
+    Hdr.u32Magic = VBGLBIGREQ_MAGIC;
+    Hdr.cbData = cbData;
+    Hdr.pvDataR3 = pvData;
+#   if HC_ARCH_BITS == 32
+    Hdr.u32Padding = 0;
+#   endif
+    rc = ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CALL(cbData), &Hdr);
+#  else
+    rc = ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CALL(cbData), pvData);
+#  endif
+#  ifdef RT_OS_LINUX
+    if (rc == 0)
+#  else
+    if (rc >= 0)
+#  endif
     {
         return VINF_SUCCESS;
     }
-    crDebug("vboxCall failed with %x\n", errno);
+#  ifdef RT_OS_LINUX
+    if (rc >= 0) /* positive values are negated VBox error status codes. */
+        crWarning("vboxCall failed with VBox status code %d\n", -rc);
+    else
+#  endif
+        crWarning("vboxCall failed with %x\n", errno);
     return VERR_NOT_SUPPORTED;
-#endif /*#ifdef RT_OS_WINDOWS*/
+# endif /*#ifdef RT_OS_WINDOWS*/
 
 #else /*#ifdef IN_GUEST*/
     crError("crVBoxHGCMCall called on host side!");
@@ -309,20 +337,20 @@ static void crVBoxHGCMWriteExact(CRConnection *conn, const void *buf, unsigned i
     CRVBOXHGCMWRITE parms;
     int rc;
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_WRITE;
     parms.hdr.cParms      = SHCRGL_CPARMS_WRITE;
 
     parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_In;
     parms.pBuffer.u.Pointer.size         = len;
-    parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) buf;
+    parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) buf;
 
     rc = crVBoxHGCMCall(&parms, sizeof(parms));
 
     if (RT_FAILURE(rc) || RT_FAILURE(parms.hdr.result))
     {
-        crDebug("SHCRGL_GUEST_FN_WRITE failed with %x %x\n", rc, parms.hdr.result);
+        crWarning("SHCRGL_GUEST_FN_WRITE failed with %x %x\n", rc, parms.hdr.result);
     }
 }
 
@@ -331,7 +359,7 @@ static void crVBoxHGCMReadExact( CRConnection *conn, const void *buf, unsigned i
     CRVBOXHGCMREAD parms;
     int rc;
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_READ;
     parms.hdr.cParms      = SHCRGL_CPARMS_READ;
@@ -339,7 +367,7 @@ static void crVBoxHGCMReadExact( CRConnection *conn, const void *buf, unsigned i
     CRASSERT(!conn->pBuffer); //make sure there's no data to process
     parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_Out;
     parms.pBuffer.u.Pointer.size         = conn->cbHostBufferAllocated;
-    parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) conn->pHostBuffer;
+    parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) conn->pHostBuffer;
 
     parms.cbBuffer.type      = VMMDevHGCMParmType_32bit;
     parms.cbBuffer.u.value32 = 0;
@@ -348,13 +376,14 @@ static void crVBoxHGCMReadExact( CRConnection *conn, const void *buf, unsigned i
 
     if (RT_FAILURE(rc) || RT_FAILURE(parms.hdr.result))
     {
-        crDebug("SHCRGL_GUEST_FN_WRITE_READ failed with %x %x\n", rc, parms.hdr.result);
+        crWarning("SHCRGL_GUEST_FN_READ failed with %x %x\n", rc, parms.hdr.result);
         return;
     }
 
     if (parms.cbBuffer.u.value32)
     {
-        conn->pBuffer  = (uint8_t*) parms.pBuffer.u.Pointer.u.linearAddr;
+        //conn->pBuffer  = (uint8_t*) parms.pBuffer.u.Pointer.u.linearAddr;
+        conn->pBuffer  = conn->pHostBuffer;
         conn->cbBuffer = parms.cbBuffer.u.value32;
     }
 
@@ -373,7 +402,7 @@ crVBoxHGCMWriteReadExact(CRConnection *conn, const void *buf, unsigned int len,
     CRVBOXHGCMWRITEREAD parms;
     int rc;
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_WRITE_READ;
     parms.hdr.cParms      = SHCRGL_CPARMS_WRITE_READ;
@@ -382,19 +411,19 @@ crVBoxHGCMWriteReadExact(CRConnection *conn, const void *buf, unsigned int len,
     {
         parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_In;
         parms.pBuffer.u.Pointer.size         = len;
-        parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) buf;
+        parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) buf;
     }
-    /*else //@todo it fails badly, have to check why
+    /*else ///@todo it fails badly, have to check why. bird: This fails because buf isn't a physical address?
     {
         parms.pBuffer.type                 = VMMDevHGCMParmType_PhysAddr;
         parms.pBuffer.u.Pointer.size       = len;
-        parms.pBuffer.u.Pointer.u.physAddr = (VMMDEVHYPPHYS32) buf;
+        parms.pBuffer.u.Pointer.u.physAddr = (uintptr_t) buf;
     }*/
 
     CRASSERT(!conn->pBuffer); //make sure there's no data to process
     parms.pWriteback.type                   = VMMDevHGCMParmType_LinAddr_Out;
     parms.pWriteback.u.Pointer.size         = conn->cbHostBufferAllocated;
-    parms.pWriteback.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) conn->pHostBuffer;
+    parms.pWriteback.u.Pointer.u.linearAddr = (uintptr_t) conn->pHostBuffer;
 
     parms.cbWriteback.type      = VMMDevHGCMParmType_32bit;
     parms.cbWriteback.u.value32 = 0;
@@ -422,14 +451,15 @@ crVBoxHGCMWriteReadExact(CRConnection *conn, const void *buf, unsigned int len,
         }
         else
         {
-            crDebug("SHCRGL_GUEST_FN_WRITE_READ failed with %x %x\n", rc, parms.hdr.result);
+            crWarning("SHCRGL_GUEST_FN_WRITE_READ (%i) failed with %x %x\n", len, rc, parms.hdr.result);
             return;
         }
     }
 
     if (parms.cbWriteback.u.value32)
     {
-        conn->pBuffer  = (uint8_t*) parms.pWriteback.u.Pointer.u.linearAddr;
+        //conn->pBuffer  = (uint8_t*) parms.pWriteback.u.Pointer.u.linearAddr;
+        conn->pBuffer  = conn->pHostBuffer;
         conn->cbBuffer = parms.cbWriteback.u.value32;
     }
 
@@ -491,14 +521,14 @@ static void crVBoxHGCMPollHost(CRConnection *conn)
 
     CRASSERT(!conn->pBuffer);
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_READ;
     parms.hdr.cParms      = SHCRGL_CPARMS_READ;
 
     parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_Out;
     parms.pBuffer.u.Pointer.size         = conn->cbHostBufferAllocated;
-    parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) conn->pHostBuffer;
+    parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) conn->pHostBuffer;
 
     parms.cbBuffer.type      = VMMDevHGCMParmType_32bit;
     parms.cbBuffer.u.value32 = 0;
@@ -610,9 +640,9 @@ static void crVBoxHGCMReceiveMessage(CRConnection *conn)
             hgcm_buffer->magic     = CR_VBOXHGCM_BUFFER_MAGIC;
             hgcm_buffer->kind      = CR_VBOXHGCM_MEMORY_BIG;
             hgcm_buffer->allocated = sizeof(CRVBOXHGCMBUFFER) + len;
-#ifdef RT_OS_WINDOWS
+# ifdef RT_OS_WINDOWS
             hgcm_buffer->pDDS      = NULL;
-#endif
+# endif
         }
 
         hgcm_buffer->len = len;
@@ -651,6 +681,36 @@ static void crVBoxHGCMAccept( CRConnection *conn, const char *hostname, unsigned
 #endif
 }
 
+static int crVBoxHGCMSetVersion(CRConnection *conn, unsigned int vMajor, unsigned int vMinor)
+{
+    CRVBOXHGCMSETVERSION parms;
+    int rc;
+
+    parms.hdr.result      = VERR_WRONG_ORDER;
+    parms.hdr.u32ClientID = conn->u32ClientID;
+    parms.hdr.u32Function = SHCRGL_GUEST_FN_SET_VERSION;
+    parms.hdr.cParms      = SHCRGL_CPARMS_SET_VERSION;
+
+    parms.vMajor.type      = VMMDevHGCMParmType_32bit;
+    parms.vMajor.u.value32 = CR_PROTOCOL_VERSION_MAJOR;
+    parms.vMinor.type      = VMMDevHGCMParmType_32bit;
+    parms.vMinor.u.value32 = CR_PROTOCOL_VERSION_MINOR;
+
+    rc = crVBoxHGCMCall(&parms, sizeof(parms));
+
+    if (RT_FAILURE(rc) || RT_FAILURE(parms.hdr.result))
+    {
+        crWarning("Host doesn't accept our version %d.%d. Make sure you have appropriate additions installed!",
+                  parms.vMajor.u.value32, parms.vMinor.u.value32);
+        return FALSE;
+    }
+
+    conn->vMajor = CR_PROTOCOL_VERSION_MAJOR;
+    conn->vMinor = CR_PROTOCOL_VERSION_MINOR;
+
+    return TRUE;
+}
+
 /**
  * The function that actually connects.  This should only be called by clients,
  * guests in vbox case.
@@ -686,7 +746,7 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
 #else
     if (g_crvboxhgcm.iGuestDrv == INVALID_HANDLE_VALUE)
     {
-        g_crvboxhgcm.iGuestDrv = open(VBOXGUEST_DEVICE_NAME, O_RDWR, 0);
+        g_crvboxhgcm.iGuestDrv = open(VBOXGUEST_USER_DEVICE_NAME, O_RDWR, 0);
         if (g_crvboxhgcm.iGuestDrv == INVALID_HANDLE_VALUE)
         {
             crDebug("could not open Guest Additions kernel module! rc = %d\n", errno);
@@ -706,6 +766,15 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
                         &info, sizeof (info),
                         &cbReturned,
                         NULL))
+#elif defined(RT_OS_SOLARIS)
+    VBGLBIGREQ Hdr;
+    Hdr.u32Magic = VBGLBIGREQ_MAGIC;
+    Hdr.cbData = sizeof(info);
+    Hdr.pvDataR3 = &info;
+# if HC_ARCH_BITS == 32
+    Hdr.u32Padding = 0;
+# endif
+    if (ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CONNECT, &Hdr) >= 0)
 #else
     /*@todo it'd fail */
     if (ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CONNECT, &info, sizeof (info)) >= 0)
@@ -714,20 +783,22 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
         if (info.result == VINF_SUCCESS)
         {
             conn->u32ClientID = info.u32ClientID;
-            crDebug("HGCM connect was successful: client id =%x\n", conn->u32ClientID);
+            crDebug("HGCM connect was successful: client id =0x%x\n", conn->u32ClientID);
+
+            return crVBoxHGCMSetVersion(conn, CR_PROTOCOL_VERSION_MAJOR, CR_PROTOCOL_VERSION_MINOR);
         }
         else
         {
-            crDebug("HGCM connect failed with rc=%x\n", info.result);
+            crDebug("HGCM connect failed with rc=0x%x\n", info.result);
             return FALSE;
         }
     }
     else
     {
 #ifdef RT_OS_WINDOWS
-        crDebug("HGCM connect failed with rc=%x\n", GetLastError());
+        crDebug("IOCTL for HGCM connect failed with rc=0x%x\n", GetLastError());
 #else
-        crDebug("HGCM connect failed with rc=%x\n", errno);
+        crDebug("IOCTL for HGCM connect failed with rc=0x%x\n", errno);
 #endif
         return FALSE;
     }
@@ -744,11 +815,13 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
 /*@todo same, replace DeviceIoControl with vbglR3DoIOCtl */
 static void crVBoxHGCMDoDisconnect( CRConnection *conn )
 {
+#ifdef IN_GUEST
     VBoxGuestHGCMDisconnectInfo info;
-#ifdef RT_OS_WINDOWS
+# ifdef RT_OS_WINDOWS
     DWORD cbReturned;
-#endif
+# endif
     int i;
+#endif
 
     if (conn->pHostBuffer)
     {
@@ -777,13 +850,13 @@ static void crVBoxHGCMDoDisconnect( CRConnection *conn )
     }
 
 #ifndef IN_GUEST
-#else
+#else /* IN_GUEST */
     if (conn->u32ClientID)
     {
         memset (&info, 0, sizeof (info));
         info.u32ClientID = conn->u32ClientID;
 
-#ifdef RT_OS_WINDOWS
+# ifdef RT_OS_WINDOWS
         if ( !DeviceIoControl(g_crvboxhgcm.hGuestDrv,
                                VBOXGUEST_IOCTL_HGCM_DISCONNECT,
                                &info, sizeof (info),
@@ -793,12 +866,21 @@ static void crVBoxHGCMDoDisconnect( CRConnection *conn )
         {
             crDebug("Disconnect failed with %x\n", GetLastError());
         }
-#else
+# elif defined(RT_OS_SOLARIS)
+        VBGLBIGREQ Hdr;
+        Hdr.u32Magic = VBGLBIGREQ_MAGIC;
+        Hdr.cbData = sizeof(info);
+        Hdr.pvDataR3 = &info;
+#  if HC_ARCH_BITS == 32
+        Hdr.u32Padding = 0;
+#  endif
+        if (ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_DISCONNECT, &Hdr) >= 0)
+# else
         if (ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_DISCONNECT, &info, sizeof (info)) < 0)
         {
             crDebug("Disconnect failed with %x\n", errno);
         }
-#endif
+# endif
 
         conn->u32ClientID = 0;
     }
@@ -811,15 +893,15 @@ static void crVBoxHGCMDoDisconnect( CRConnection *conn )
     /* close guest additions driver*/
     if (i>=g_crvboxhgcm.num_conns)
     {
-#ifdef RT_OS_WINDOWS
+# ifdef RT_OS_WINDOWS
         CloseHandle(g_crvboxhgcm.hGuestDrv);
         g_crvboxhgcm.hGuestDrv = INVALID_HANDLE_VALUE;
-#else
+# else
         close(g_crvboxhgcm.iGuestDrv);
         g_crvboxhgcm.iGuestDrv = INVALID_HANDLE_VALUE;
-#endif
+# endif
     }
-#endif
+#endif /* IN_GUEST */
 }
 
 static void crVBoxHGCMInstantReclaim(CRConnection *conn, CRMessage *mess)
@@ -971,7 +1053,7 @@ void crVBoxHGCMConnection(CRConnection *conn)
     conn->allow_redir_ptr = 1;
 
     //@todo remove this crap at all later
-    conn->cbHostBufferAllocated = 1*1024;
+    conn->cbHostBufferAllocated = 2*1024;
     conn->pHostBuffer = (uint8_t*) crAlloc(conn->cbHostBufferAllocated);
     CRASSERT(conn->pHostBuffer);
     conn->cbHostBuffer = 0;
