@@ -1,4 +1,4 @@
-/* $Revision: 49134 $ */
+/* $Revision: 56351 $ */
 /** @file
  * VirtualBox Support Driver - Internal header.
  */
@@ -46,8 +46,10 @@
 #include <iprt/string.h>
 #include <iprt/err.h>
 
+#ifdef SUPDRV_AGNOSTIC
+/* do nothing */
 
-#if defined(RT_OS_WINDOWS)
+#elif defined(RT_OS_WINDOWS)
     RT_C_DECLS_BEGIN
 #   if (_MSC_VER >= 1400) && !defined(VBOX_WITH_PATCHED_DDK)
 #       define _InterlockedExchange           _InterlockedExchange_StupidDDKVsCompilerCrap
@@ -78,7 +80,9 @@
     RT_C_DECLS_END
 
 #elif defined(RT_OS_LINUX)
-#   include <linux/autoconf.h>
+#   ifndef AUTOCONF_INCLUDED
+#    include <linux/autoconf.h>
+#   endif
 #   include <linux/version.h>
 #   if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
 #       define MODVERSIONS
@@ -122,7 +126,6 @@
 #   define memcmp  libkern_memcmp
 #   define strchr  libkern_strchr
 #   define strrchr libkern_strrchr
-#   define ffs     libkern_ffs
 #   define ffsl    libkern_ffsl
 #   define fls     libkern_fls
 #   define flsl    libkern_flsl
@@ -186,9 +189,11 @@
 #elif defined(RT_OS_LINUX)
 
 /* check kernel version */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
-# error Unsupported kernel version!
-#endif
+# ifndef SUPDRV_AGNOSTIC
+#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#   error Unsupported kernel version!
+#  endif
+# endif
 
 RT_C_DECLS_BEGIN
 int  linux_dprintf(const char *format, ...);
@@ -319,6 +324,17 @@ RT_C_DECLS_END
 /** @} */
 
 
+/**
+ * Validates a session pointer.
+ *
+ * @returns true/false accordingly.
+ * @param   pSession    The session.
+ */
+#define SUP_IS_SESSION_VALID(pSession)  \
+    (   VALID_PTR(pSession) \
+     && pSession->u32Cookie == BIRD_INV)
+
+
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
 *******************************************************************************/
@@ -531,21 +547,23 @@ typedef struct SUPDRVSESSION
     /** Which process this session is associated with.
      * This is NIL_RTR0PROCESS for kernel sessions and valid for user ones. */
     RTR0PROCESS                     R0Process;
-#if defined(RT_OS_DARWIN)
+#ifndef SUPDRV_AGNOSTIC
+# if defined(RT_OS_DARWIN)
     /** Pointer to the associated org_virtualbox_SupDrvClient object. */
     void                           *pvSupDrvClient;
     /** Whether this session has been opened or not. */
     bool                            fOpened;
-#endif
-#if defined(RT_OS_OS2)
+# endif
+# if defined(RT_OS_OS2)
     /** The system file number of this session. */
     uint16_t                        sfn;
     uint16_t                        Alignment; /**< Alignment */
-#endif
-#if defined(RT_OS_DARWIN) || defined(RT_OS_OS2) || defined(RT_OS_SOLARIS)
+# endif
+# if defined(RT_OS_DARWIN) || defined(RT_OS_OS2) || defined(RT_OS_SOLARIS)
     /** Pointer to the next session with the same hash. */
     PSUPDRVSESSION                  pNextHash;
-#endif
+# endif
+#endif /* !SUPDRV_AGNOSTIC */
 } SUPDRVSESSION;
 
 
@@ -607,18 +625,20 @@ typedef struct SUPDRVDEVEXT
      * This CPU is responsible for the updating the common GIP data. */
     RTCPUID volatile                idGipMaster;
 
-#ifdef RT_OS_WINDOWS
-    /* Callback object returned by ExCreateCallback. */
-    PCALLBACK_OBJECT                pObjPowerCallback;
-    /* Callback handle returned by ExRegisterCallback. */
-    PVOID                           hPowerCallback;
-#endif
-
     /** Component factory mutex.
      * This protects pComponentFactoryHead and component factory querying. */
     RTSEMFASTMUTEX                  mtxComponentFactory;
     /** The head of the list of registered component factories. */
     PSUPDRVFACTORYREG               pComponentFactoryHead;
+
+#ifndef SUPDRV_AGNOSTIC
+# ifdef RT_OS_WINDOWS
+    /* Callback object returned by ExCreateCallback. */
+    PCALLBACK_OBJECT                pObjPowerCallback;
+    /* Callback handle returned by ExRegisterCallback. */
+    PVOID                           hPowerCallback;
+# endif
+#endif
 } SUPDRVDEVEXT;
 
 
@@ -632,9 +652,11 @@ bool VBOXCALL   supdrvOSObjCanAccess(PSUPDRVOBJ pObj, PSUPDRVSESSION pSession, c
 bool VBOXCALL   supdrvOSGetForcedAsyncTscMode(PSUPDRVDEVEXT pDevExt);
 int  VBOXCALL   supdrvOSEnableVTx(bool fEnabled);
 
+
 /*******************************************************************************
 *   Shared Functions                                                           *
 *******************************************************************************/
+/* SUPDrv.c */
 int  VBOXCALL   supdrvIOCtl(uintptr_t uIOCtl, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPREQHDR pReqHdr);
 int  VBOXCALL   supdrvIOCtlFast(uintptr_t uIOCtl, VMCPUID idCpu, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession);
 int  VBOXCALL   supdrvIDC(uintptr_t uIOCtl, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPDRVIDCREQHDR pReqHdr);
@@ -645,10 +667,13 @@ void VBOXCALL   supdrvCloseSession(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSessio
 void VBOXCALL   supdrvCleanupSession(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession);
 int  VBOXCALL   supdrvGipInit(PSUPDRVDEVEXT pDevExt, PSUPGLOBALINFOPAGE pGip, RTHCPHYS HCPhys, uint64_t u64NanoTS, unsigned uUpdateHz);
 void VBOXCALL   supdrvGipTerm(PSUPGLOBALINFOPAGE pGip);
-void VBOXCALL   supdrvGipUpdate(PSUPGLOBALINFOPAGE pGip, uint64_t u64NanoTS);
-void VBOXCALL   supdrvGipUpdatePerCpu(PSUPGLOBALINFOPAGE pGip, uint64_t u64NanoTS, unsigned iCpu);
+void VBOXCALL   supdrvGipUpdate(PSUPGLOBALINFOPAGE pGip, uint64_t u64NanoTS, uint64_t u64TSC);
+void VBOXCALL   supdrvGipUpdatePerCpu(PSUPGLOBALINFOPAGE pGip, uint64_t u64NanoTS, uint64_t u64TSC, unsigned iCpu);
 bool VBOXCALL   supdrvDetermineAsyncTsc(uint64_t *pu64DiffCores);
 
+/* SUPDrvAgnostic.c */
+SUPGIPMODE VBOXCALL supdrvGipDeterminTscMode(PSUPDRVDEVEXT pDevExt);
+
 RT_C_DECLS_END
 
 #endif
