@@ -99,7 +99,7 @@ static bool                     g_fPreInited = false;
 /** The SUPLib instance data.
  * Well, at least parts of it, specificly the parts that are being handed over
  * via the pre-init mechanism from the hardened executable stub.  */
-static SUPLIBDATA               g_supLibData =
+SUPLIBDATA                      g_supLibData =
 {
     NIL_RTFILE
 #if   defined(RT_OS_DARWIN)
@@ -124,20 +124,20 @@ static PSUPGLOBALINFOPAGE       g_pSUPGlobalInfoPageR0;
 static RTHCPHYS                 g_HCPhysSUPGlobalInfoPage = NIL_RTHCPHYS;
 
 /** The negotiated cookie. */
-uint32_t            g_u32Cookie = 0;
+uint32_t                        g_u32Cookie = 0;
 /** The negotiated session cookie. */
-uint32_t            g_u32SessionCookie;
+uint32_t                        g_u32SessionCookie;
 /** Session handle. */
-PSUPDRVSESSION      g_pSession;
+PSUPDRVSESSION                  g_pSession;
 /** R0 SUP Functions used for resolving referenced to the SUPR0 module. */
-static PSUPQUERYFUNCS g_pFunctions;
+static PSUPQUERYFUNCS           g_pFunctions;
 
 /** VMMR0 Load Address. */
-static RTR0PTR      g_pvVMMR0 = NIL_RTR0PTR;
+static RTR0PTR                  g_pvVMMR0 = NIL_RTR0PTR;
 /** PAGE_ALLOC_EX sans kernel mapping support indicator. */
-static bool         g_fSupportsPageAllocNoKernel = true;
+static bool                     g_fSupportsPageAllocNoKernel = true;
 /** Fake mode indicator. (~0 at first, 0 or 1 after first test) */
-static uint32_t     g_u32FakeMode = ~0;
+static uint32_t                 g_u32FakeMode = ~0;
 
 
 /*******************************************************************************
@@ -148,13 +148,25 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
 static DECLCALLBACK(int) supLoadModuleResolveImport(RTLDRMOD hLdrMod, const char *pszModule, const char *pszSymbol, unsigned uSymbol, RTUINTPTR *pValue, void *pvUser);
 
 
-SUPR3DECL(int) SUPInstall(void)
+/** Touch a range of pages. */
+DECLINLINE(void) supR3TouchPages(void *pv, size_t cPages)
+{
+    uint32_t volatile *pu32 = (uint32_t volatile *)pv;
+    while (cPages-- > 0)
+    {
+        ASMAtomicCmpXchgU32(pu32, 0, 0);
+        pu32 += PAGE_SIZE / sizeof(uint32_t);
+    }
+}
+
+
+SUPR3DECL(int) SUPR3Install(void)
 {
     return suplibOsInstall();
 }
 
 
-SUPR3DECL(int) SUPUninstall(void)
+SUPR3DECL(int) SUPR3Uninstall(void)
 {
     return suplibOsUninstall();
 }
@@ -259,16 +271,16 @@ SUPR3DECL(int) SUPR3Init(PSUPDRVSESSION *ppSession)
         CookieReq.Hdr.rc = VERR_INTERNAL_ERROR;
         strcpy(CookieReq.u.In.szMagic, SUPCOOKIE_MAGIC);
         CookieReq.u.In.u32ReqVersion = SUPDRV_IOC_VERSION;
-        const uint32_t MinVersion = (SUPDRV_IOC_VERSION & 0xffff0000) == 0x000a0000
-                                  ? 0x000a0009
-                                  :  SUPDRV_IOC_VERSION & 0xffff0000;
-        CookieReq.u.In.u32MinVersion = MinVersion;
+        const uint32_t uMinVersion = (SUPDRV_IOC_VERSION & 0xffff0000) == 0x00100001
+                                   ?  0x00100001
+                                   :  SUPDRV_IOC_VERSION & 0xffff0000;
+        CookieReq.u.In.u32MinVersion = uMinVersion;
         rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_COOKIE, &CookieReq, SUP_IOCTL_COOKIE_SIZE);
         if (    RT_SUCCESS(rc)
             &&  RT_SUCCESS(CookieReq.Hdr.rc))
         {
             if (    (CookieReq.u.Out.u32SessionVersion & 0xffff0000) == (SUPDRV_IOC_VERSION & 0xffff0000)
-                &&  CookieReq.u.Out.u32SessionVersion >= MinVersion)
+                &&  CookieReq.u.Out.u32SessionVersion >= uMinVersion)
             {
                 /*
                  * Query the functions.
@@ -287,42 +299,43 @@ SUPR3DECL(int) SUPR3Init(PSUPDRVSESSION *ppSession)
                         rc = pFuncsReq->Hdr.rc;
                     if (RT_SUCCESS(rc))
                     {
-                        g_u32Cookie         = CookieReq.u.Out.u32Cookie;
-                        g_u32SessionCookie  = CookieReq.u.Out.u32SessionCookie;
-                        g_pSession          = CookieReq.u.Out.pSession;
-                        g_pFunctions        = pFuncsReq;
-                        if (ppSession)
-                            *ppSession = CookieReq.u.Out.pSession;
-
                         /*
                          * Map the GIP into userspace.
-                         * This is an optional feature, so we will ignore any failures here.
                          */
-                        if (!g_pSUPGlobalInfoPage)
+                        Assert(!g_pSUPGlobalInfoPage);
+                        SUPGIPMAP GipMapReq;
+                        GipMapReq.Hdr.u32Cookie         = CookieReq.u.Out.u32Cookie;
+                        GipMapReq.Hdr.u32SessionCookie  = CookieReq.u.Out.u32SessionCookie;
+                        GipMapReq.Hdr.cbIn              = SUP_IOCTL_GIP_MAP_SIZE_IN;
+                        GipMapReq.Hdr.cbOut             = SUP_IOCTL_GIP_MAP_SIZE_OUT;
+                        GipMapReq.Hdr.fFlags            = SUPREQHDR_FLAGS_DEFAULT;
+                        GipMapReq.Hdr.rc                = VERR_INTERNAL_ERROR;
+                        GipMapReq.u.Out.HCPhysGip       = NIL_RTHCPHYS;
+                        GipMapReq.u.Out.pGipR0          = NIL_RTR0PTR;
+                        GipMapReq.u.Out.pGipR3          = NULL;
+                        rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_GIP_MAP, &GipMapReq, SUP_IOCTL_GIP_MAP_SIZE);
+                        if (RT_SUCCESS(rc))
+                            rc = GipMapReq.Hdr.rc;
+                        if (RT_SUCCESS(rc))
                         {
-                            SUPGIPMAP GipMapReq;
-                            GipMapReq.Hdr.u32Cookie = g_u32Cookie;
-                            GipMapReq.Hdr.u32SessionCookie = g_u32SessionCookie;
-                            GipMapReq.Hdr.cbIn = SUP_IOCTL_GIP_MAP_SIZE_IN;
-                            GipMapReq.Hdr.cbOut = SUP_IOCTL_GIP_MAP_SIZE_OUT;
-                            GipMapReq.Hdr.fFlags = SUPREQHDR_FLAGS_DEFAULT;
-                            GipMapReq.Hdr.rc = VERR_INTERNAL_ERROR;
-                            GipMapReq.u.Out.HCPhysGip = NIL_RTHCPHYS;
-                            GipMapReq.u.Out.pGipR0 = NIL_RTR0PTR;
-                            GipMapReq.u.Out.pGipR3 = NULL;
-                            rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_GIP_MAP, &GipMapReq, SUP_IOCTL_GIP_MAP_SIZE);
-                            if (RT_SUCCESS(rc))
-                                rc = GipMapReq.Hdr.rc;
-                            if (RT_SUCCESS(rc))
-                            {
-                                AssertRelease(GipMapReq.u.Out.pGipR3->u32Magic == SUPGLOBALINFOPAGE_MAGIC);
-                                AssertRelease(GipMapReq.u.Out.pGipR3->u32Version >= SUPGLOBALINFOPAGE_VERSION);
-                                ASMAtomicXchgSize(&g_HCPhysSUPGlobalInfoPage, GipMapReq.u.Out.HCPhysGip);
-                                ASMAtomicCmpXchgPtr((void * volatile *)&g_pSUPGlobalInfoPage, GipMapReq.u.Out.pGipR3, NULL);
-                                ASMAtomicCmpXchgPtr((void * volatile *)&g_pSUPGlobalInfoPageR0, (void *)GipMapReq.u.Out.pGipR0, NULL);
-                            }
+                            AssertRelease(GipMapReq.u.Out.pGipR3->u32Magic == SUPGLOBALINFOPAGE_MAGIC);
+                            AssertRelease(GipMapReq.u.Out.pGipR3->u32Version >= SUPGLOBALINFOPAGE_VERSION);
+
+                            /*
+                             * Set the globals and return success.
+                             */
+                            ASMAtomicXchgSize(&g_HCPhysSUPGlobalInfoPage, GipMapReq.u.Out.HCPhysGip);
+                            ASMAtomicCmpXchgPtr((void * volatile *)&g_pSUPGlobalInfoPage, GipMapReq.u.Out.pGipR3, NULL);
+                            ASMAtomicCmpXchgPtr((void * volatile *)&g_pSUPGlobalInfoPageR0, (void *)GipMapReq.u.Out.pGipR0, NULL);
+
+                            g_u32Cookie         = CookieReq.u.Out.u32Cookie;
+                            g_u32SessionCookie  = CookieReq.u.Out.u32SessionCookie;
+                            g_pSession          = CookieReq.u.Out.pSession;
+                            g_pFunctions        = pFuncsReq;
+                            if (ppSession)
+                                *ppSession = CookieReq.u.Out.pSession;
+                            return VINF_SUCCESS;
                         }
-                        return VINF_SUCCESS;
                     }
 
                     /* bailout */
@@ -334,7 +347,7 @@ SUPR3DECL(int) SUPR3Init(PSUPDRVSESSION *ppSession)
             else
             {
                 LogRel(("Support driver version mismatch: SessionVersion=%#x DriverVersion=%#x ClientVersion=%#x MinVersion=%#x\n",
-                        CookieReq.u.Out.u32SessionVersion, CookieReq.u.Out.u32DriverVersion, SUPDRV_IOC_VERSION, MinVersion));
+                        CookieReq.u.Out.u32SessionVersion, CookieReq.u.Out.u32DriverVersion, SUPDRV_IOC_VERSION, uMinVersion));
                 rc = VERR_VM_DRIVER_VERSION_MISMATCH;
             }
         }
@@ -358,7 +371,6 @@ SUPR3DECL(int) SUPR3Init(PSUPDRVSESSION *ppSession)
 
         suplibOsTerm(&g_supLibData);
     }
-    AssertMsgFailed(("SUPR3Init() failed rc=%Rrc\n", rc));
     g_cInits--;
 
     return rc;
@@ -400,62 +412,63 @@ static int supInitFake(PSUPDRVSESSION *ppSession)
         { "SUPR0MemFree",                           0xefef000a },
         { "SUPR0Printf",                            0xefef000b },
         { "SUPR0GetPagingMode",                     0xefef000c },
-        { "SUPR0EnableVTx",                         0xefef000c },
-        { "RTMemAlloc",                             0xefef000d },
-        { "RTMemAllocZ",                            0xefef000e },
-        { "RTMemFree",                              0xefef000f },
-        { "RTR0MemObjAddress",                      0xefef0010 },
-        { "RTR0MemObjAddressR3",                    0xefef0011 },
-        { "RTR0MemObjAllocPage",                    0xefef0012 },
-        { "RTR0MemObjAllocPhysNC",                  0xefef0013 },
-        { "RTR0MemObjAllocLow",                     0xefef0014 },
-        { "RTR0MemObjEnterPhys",                    0xefef0014 },
-        { "RTR0MemObjFree",                         0xefef0015 },
-        { "RTR0MemObjGetPagePhysAddr",              0xefef0016 },
-        { "RTR0MemObjMapUser",                      0xefef0017 },
-        { "RTR0MemObjMapKernel",                    0xefef0017 },
-        { "RTR0MemObjMapKernelEx",                  0xefef0017 },
-        { "RTProcSelf",                             0xefef0038 },
-        { "RTR0ProcHandleSelf",                     0xefef0039 },
-        { "RTSemEventCreate",                       0xefef0018 },
-        { "RTSemEventSignal",                       0xefef0019 },
-        { "RTSemEventWait",                         0xefef001a },
-        { "RTSemEventWaitNoResume",                 0xefef001b },
-        { "RTSemEventDestroy",                      0xefef001c },
-        { "RTSemEventMultiCreate",                  0xefef001d },
-        { "RTSemEventMultiSignal",                  0xefef001e },
-        { "RTSemEventMultiReset",                   0xefef001f },
-        { "RTSemEventMultiWait",                    0xefef0020 },
-        { "RTSemEventMultiWaitNoResume",            0xefef0021 },
-        { "RTSemEventMultiDestroy",                 0xefef0022 },
-        { "RTSemFastMutexCreate",                   0xefef0023 },
-        { "RTSemFastMutexDestroy",                  0xefef0024 },
-        { "RTSemFastMutexRequest",                  0xefef0025 },
-        { "RTSemFastMutexRelease",                  0xefef0026 },
-        { "RTSpinlockCreate",                       0xefef0027 },
-        { "RTSpinlockDestroy",                      0xefef0028 },
-        { "RTSpinlockAcquire",                      0xefef0029 },
-        { "RTSpinlockRelease",                      0xefef002a },
-        { "RTSpinlockAcquireNoInts",                0xefef002b },
-        { "RTSpinlockReleaseNoInts",                0xefef002c },
-        { "RTTimeNanoTS",                           0xefef002d },
-        { "RTTimeMillieTS",                         0xefef002e },
-        { "RTTimeSystemNanoTS",                     0xefef002f },
-        { "RTTimeSystemMillieTS",                   0xefef0030 },
-        { "RTThreadNativeSelf",                     0xefef0031 },
-        { "RTThreadSleep",                          0xefef0032 },
-        { "RTThreadYield",                          0xefef0033 },
-        { "RTLogDefaultInstance",                   0xefef0034 },
-        { "RTLogRelDefaultInstance",                0xefef0035 },
-        { "RTLogSetDefaultInstanceThread",          0xefef0036 },
-        { "RTLogLogger",                            0xefef0037 },
-        { "RTLogLoggerEx",                          0xefef0038 },
-        { "RTLogLoggerExV",                         0xefef0039 },
-        { "AssertMsg1",                             0xefef003a },
-        { "AssertMsg2",                             0xefef003b },
-        { "RTAssertMsg1",                           0xefef003c },
-        { "RTAssertMsg2",                           0xefef003d },
-        { "RTAssertMsg2V",                          0xefef003e },
+        { "SUPR0EnableVTx",                         0xefef000e },
+        { "RTMemAlloc",                             0xefef000f },
+        { "RTMemAllocZ",                            0xefef0010 },
+        { "RTMemFree",                              0xefef0011 },
+        { "RTR0MemObjAddress",                      0xefef0012 },
+        { "RTR0MemObjAddressR3",                    0xefef0013 },
+        { "RTR0MemObjAllocPage",                    0xefef0014 },
+        { "RTR0MemObjAllocPhysNC",                  0xefef0015 },
+        { "RTR0MemObjAllocLow",                     0xefef0016 },
+        { "RTR0MemObjEnterPhys",                    0xefef0017 },
+        { "RTR0MemObjFree",                         0xefef0018 },
+        { "RTR0MemObjGetPagePhysAddr",              0xefef0019 },
+        { "RTR0MemObjMapUser",                      0xefef001a },
+        { "RTR0MemObjMapKernel",                    0xefef001b },
+        { "RTR0MemObjMapKernelEx",                  0xefef001c },
+        { "RTProcSelf",                             0xefef001d },
+        { "RTR0ProcHandleSelf",                     0xefef001e },
+        { "RTSemEventCreate",                       0xefef001f },
+        { "RTSemEventSignal",                       0xefef0020 },
+        { "RTSemEventWait",                         0xefef0021 },
+        { "RTSemEventWaitNoResume",                 0xefef0022 },
+        { "RTSemEventDestroy",                      0xefef0023 },
+        { "RTSemEventMultiCreate",                  0xefef0024 },
+        { "RTSemEventMultiSignal",                  0xefef0025 },
+        { "RTSemEventMultiReset",                   0xefef0026 },
+        { "RTSemEventMultiWait",                    0xefef0027 },
+        { "RTSemEventMultiWaitNoResume",            0xefef0028 },
+        { "RTSemEventMultiDestroy",                 0xefef0029 },
+        { "RTSemFastMutexCreate",                   0xefef002a },
+        { "RTSemFastMutexDestroy",                  0xefef002b },
+        { "RTSemFastMutexRequest",                  0xefef002c },
+        { "RTSemFastMutexRelease",                  0xefef002d },
+        { "RTSpinlockCreate",                       0xefef002e },
+        { "RTSpinlockDestroy",                      0xefef002f },
+        { "RTSpinlockAcquire",                      0xefef0030 },
+        { "RTSpinlockRelease",                      0xefef0031 },
+        { "RTSpinlockAcquireNoInts",                0xefef0032 },
+        { "RTSpinlockReleaseNoInts",                0xefef0033 },
+        { "RTTimeNanoTS",                           0xefef0034 },
+        { "RTTimeMillieTS",                         0xefef0035 },
+        { "RTTimeSystemNanoTS",                     0xefef0036 },
+        { "RTTimeSystemMillieTS",                   0xefef0037 },
+        { "RTThreadNativeSelf",                     0xefef0038 },
+        { "RTThreadSleep",                          0xefef0039 },
+        { "RTThreadYield",                          0xefef003a },
+        { "RTLogDefaultInstance",                   0xefef003b },
+        { "RTLogRelDefaultInstance",                0xefef003c },
+        { "RTLogSetDefaultInstanceThread",          0xefef003d },
+        { "RTLogLogger",                            0xefef003e },
+        { "RTLogLoggerEx",                          0xefef003f },
+        { "RTLogLoggerExV",                         0xefef0040 },
+        { "AssertMsg1",                             0xefef0041 },
+        { "AssertMsg2",                             0xefef0042 },
+        { "RTAssertMsg1",                           0xefef0043 },
+        { "RTAssertMsg2",                           0xefef0044 },
+        { "RTAssertMsg2V",                          0xefef0045 },
+        { "SUPR0QueryVTCaps",                       0xefef0046 },
     };
 
     /* fake r0 functions. */
@@ -485,12 +498,12 @@ static int supInitFake(PSUPDRVSESSION *ppSession)
 }
 
 
-SUPR3DECL(int) SUPTerm(bool fForced)
+SUPR3DECL(int) SUPR3Term(bool fForced)
 {
     /*
      * Verify state.
      */
-    AssertMsg(g_cInits > 0, ("SUPTerm() is called before SUPR3Init()!\n"));
+    AssertMsg(g_cInits > 0, ("SUPR3Term() is called before SUPR3Init()!\n"));
     if (g_cInits == 0)
         return VERR_WRONG_ORDER;
     if (g_cInits == 1 || fForced)
@@ -525,7 +538,7 @@ SUPR3DECL(int) SUPTerm(bool fForced)
 }
 
 
-SUPR3DECL(SUPPAGINGMODE) SUPGetPagingMode(void)
+SUPR3DECL(SUPPAGINGMODE) SUPR3GetPagingMode(void)
 {
     /* fake */
     if (RT_UNLIKELY(g_u32FakeMode))
@@ -549,7 +562,7 @@ SUPR3DECL(SUPPAGINGMODE) SUPGetPagingMode(void)
     if (    RT_FAILURE(rc)
         ||  RT_FAILURE(Req.Hdr.rc))
     {
-        LogRel(("SUPGetPagingMode: %Rrc %Rrc\n", rc, Req.Hdr.rc));
+        LogRel(("SUPR3GetPagingMode: %Rrc %Rrc\n", rc, Req.Hdr.rc));
         Req.u.Out.enmMode = SUPPAGINGMODE_INVALID;
     }
 
@@ -567,7 +580,7 @@ static int supCallVMMR0ExFake(PVMR0 pVMR0, unsigned uOperation, uint64_t u64Arg,
 }
 
 
-SUPR3DECL(int) SUPCallVMMR0Fast(PVMR0 pVMR0, unsigned uOperation, unsigned idCpu)
+SUPR3DECL(int) SUPR3CallVMMR0Fast(PVMR0 pVMR0, unsigned uOperation, VMCPUID idCpu)
 {
     if (RT_LIKELY(uOperation == SUP_VMMR0_DO_RAW_RUN))
         return suplibOsIOCtlFast(&g_supLibData, SUP_IOCTL_FAST_DO_RAW_RUN, idCpu);
@@ -581,7 +594,7 @@ SUPR3DECL(int) SUPCallVMMR0Fast(PVMR0 pVMR0, unsigned uOperation, unsigned idCpu
 }
 
 
-SUPR3DECL(int) SUPCallVMMR0Ex(PVMR0 pVMR0, unsigned uOperation, uint64_t u64Arg, PSUPVMMR0REQHDR pReqHdr)
+SUPR3DECL(int) SUPR3CallVMMR0Ex(PVMR0 pVMR0, VMCPUID idCpu, unsigned uOperation, uint64_t u64Arg, PSUPVMMR0REQHDR pReqHdr)
 {
     /*
      * The following operations don't belong here.
@@ -608,6 +621,7 @@ SUPR3DECL(int) SUPCallVMMR0Ex(PVMR0 pVMR0, unsigned uOperation, uint64_t u64Arg,
         Req.Hdr.fFlags = SUPREQHDR_FLAGS_DEFAULT;
         Req.Hdr.rc = VERR_INTERNAL_ERROR;
         Req.u.In.pVMR0 = pVMR0;
+        Req.u.In.idCpu = idCpu;
         Req.u.In.uOperation = uOperation;
         Req.u.In.u64Arg = u64Arg;
         rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_CALL_VMMR0(0), &Req, SUP_IOCTL_CALL_VMMR0_SIZE(0));
@@ -628,6 +642,7 @@ SUPR3DECL(int) SUPCallVMMR0Ex(PVMR0 pVMR0, unsigned uOperation, uint64_t u64Arg,
         pReq->Hdr.fFlags = SUPREQHDR_FLAGS_DEFAULT;
         pReq->Hdr.rc = VERR_INTERNAL_ERROR;
         pReq->u.In.pVMR0 = pVMR0;
+        pReq->u.In.idCpu = idCpu;
         pReq->u.In.uOperation = uOperation;
         pReq->u.In.u64Arg = u64Arg;
         memcpy(&pReq->abReqPkt[0], pReqHdr, cbReq);
@@ -642,7 +657,7 @@ SUPR3DECL(int) SUPCallVMMR0Ex(PVMR0 pVMR0, unsigned uOperation, uint64_t u64Arg,
 }
 
 
-SUPR3DECL(int) SUPCallVMMR0(PVMR0 pVMR0, unsigned uOperation, void *pvArg)
+SUPR3DECL(int) SUPR3CallVMMR0(PVMR0 pVMR0, VMCPUID idCpu, unsigned uOperation, void *pvArg)
 {
     /*
      * The following operations don't belong here.
@@ -652,11 +667,11 @@ SUPR3DECL(int) SUPCallVMMR0(PVMR0 pVMR0, unsigned uOperation, void *pvArg)
                     &&  uOperation != SUP_VMMR0_DO_NOP,
                     ("%#x\n", uOperation),
                     VERR_INTERNAL_ERROR);
-    return SUPCallVMMR0Ex(pVMR0, uOperation, (uintptr_t)pvArg, NULL);
+    return SUPR3CallVMMR0Ex(pVMR0, idCpu, uOperation, (uintptr_t)pvArg, NULL);
 }
 
 
-SUPR3DECL(int) SUPSetVMForFastIOCtl(PVMR0 pVMR0)
+SUPR3DECL(int) SUPR3SetVMForFastIOCtl(PVMR0 pVMR0)
 {
     if (RT_UNLIKELY(g_u32FakeMode))
         return VINF_SUCCESS;
@@ -733,7 +748,105 @@ SUPR3DECL(int) SUPR3CallR0Service(const char *pszService, size_t cchService, uin
 }
 
 
-SUPR3DECL(int) SUPPageAlloc(size_t cPages, void **ppvPages)
+/**
+ * Worker for the SUPR3Logger* APIs.
+ *
+ * @returns VBox status code.
+ * @param   enmWhich    Which logger.
+ * @param   fWhat       What to do with the logger.
+ * @param   pszFlags    The flags settings.
+ * @param   pszGroups   The groups settings.
+ * @param   pszDest     The destionation specificier.
+ */
+static int supR3LoggerSettings(SUPLOGGER enmWhich, uint32_t fWhat, const char *pszFlags, const char *pszGroups, const char *pszDest)
+{
+    uint32_t const cchFlags  = pszFlags  ? (uint32_t)strlen(pszFlags)  : 0;
+    uint32_t const cchGroups = pszGroups ? (uint32_t)strlen(pszGroups) : 0;
+    uint32_t const cchDest   = pszDest   ? (uint32_t)strlen(pszDest)   : 0;
+    uint32_t const cbStrTab  = cchFlags  + !!cchFlags
+                             + cchGroups + !!cchGroups
+                             + cchDest   + !!cchDest
+                             + (!cchFlags && !cchGroups && !cchDest);
+
+    PSUPLOGGERSETTINGS pReq  = (PSUPLOGGERSETTINGS)alloca(SUP_IOCTL_LOGGER_SETTINGS_SIZE(cbStrTab));
+    pReq->Hdr.u32Cookie = g_u32Cookie;
+    pReq->Hdr.u32SessionCookie = g_u32SessionCookie;
+    pReq->Hdr.cbIn  = SUP_IOCTL_LOGGER_SETTINGS_SIZE_IN(cbStrTab);
+    pReq->Hdr.cbOut = SUP_IOCTL_LOGGER_SETTINGS_SIZE_OUT;
+    pReq->Hdr.fFlags= SUPREQHDR_FLAGS_DEFAULT;
+    pReq->Hdr.rc    = VERR_INTERNAL_ERROR;
+    switch (enmWhich)
+    {
+        case SUPLOGGER_DEBUG:   pReq->u.In.fWhich = SUPLOGGERSETTINGS_WHICH_DEBUG; break;
+        case SUPLOGGER_RELEASE: pReq->u.In.fWhich = SUPLOGGERSETTINGS_WHICH_RELEASE; break;
+        default:
+            return VERR_INVALID_PARAMETER;
+    }
+    pReq->u.In.fWhat = fWhat;
+
+    uint32_t off = 0;
+    if (cchFlags)
+    {
+        pReq->u.In.offFlags = off;
+        memcpy(&pReq->u.In.szStrings[off], pszFlags, cchFlags + 1);
+        off += cchFlags + 1;
+    }
+    else
+        pReq->u.In.offFlags = cbStrTab - 1;
+
+    if (cchGroups)
+    {
+        pReq->u.In.offGroups = off;
+        memcpy(&pReq->u.In.szStrings[off], pszGroups, cchGroups + 1);
+        off += cchGroups + 1;
+    }
+    else
+        pReq->u.In.offGroups = cbStrTab - 1;
+
+    if (cchDest)
+    {
+        pReq->u.In.offDestination = off;
+        memcpy(&pReq->u.In.szStrings[off], pszDest, cchDest + 1);
+        off += cchDest + 1;
+    }
+    else
+        pReq->u.In.offDestination = cbStrTab - 1;
+
+    if (!off)
+    {
+        pReq->u.In.szStrings[0] = '\0';
+        off++;
+    }
+    Assert(off == cbStrTab);
+    Assert(pReq->u.In.szStrings[cbStrTab - 1] == '\0');
+
+
+    int rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_LOGGER_SETTINGS(cbStrTab), pReq, SUP_IOCTL_LOGGER_SETTINGS_SIZE(cbStrTab));
+    if (RT_SUCCESS(rc))
+        rc = pReq->Hdr.rc;
+    return rc;
+}
+
+
+SUPR3DECL(int) SUPR3LoggerSettings(SUPLOGGER enmWhich, const char *pszFlags, const char *pszGroups, const char *pszDest)
+{
+    return supR3LoggerSettings(enmWhich, SUPLOGGERSETTINGS_WHAT_SETTINGS, pszFlags, pszGroups, pszDest);
+}
+
+
+SUPR3DECL(int) SUPR3LoggerCreate(SUPLOGGER enmWhich, const char *pszFlags, const char *pszGroups, const char *pszDest)
+{
+    return supR3LoggerSettings(enmWhich, SUPLOGGERSETTINGS_WHAT_CREATE, pszFlags, pszGroups, pszDest);
+}
+
+
+SUPR3DECL(int) SUPR3LoggerDestroy(SUPLOGGER enmWhich)
+{
+    return supR3LoggerSettings(enmWhich, SUPLOGGERSETTINGS_WHAT_DESTROY, NULL, NULL, NULL);
+}
+
+
+SUPR3DECL(int) SUPR3PageAlloc(size_t cPages, void **ppvPages)
 {
     /*
      * Validate.
@@ -742,22 +855,14 @@ SUPR3DECL(int) SUPPageAlloc(size_t cPages, void **ppvPages)
     *ppvPages = NULL;
     AssertReturn(cPages > 0, VERR_PAGE_COUNT_OUT_OF_RANGE);
 
-#ifdef RT_OS_WINDOWS
-    /*
-     * Temporary hack for windows until we've sorted out the
-     * locked memory that doesn't need to be accessible from kernel space.
-     */
-    return SUPPageAllocLockedEx(cPages, ppvPages, NULL);
-#else
     /*
      * Call OS specific worker.
      */
     return suplibOsPageAlloc(&g_supLibData, cPages, ppvPages);
-#endif
 }
 
 
-SUPR3DECL(int) SUPPageFree(void *pvPages, size_t cPages)
+SUPR3DECL(int) SUPR3PageFree(void *pvPages, size_t cPages)
 {
     /*
      * Validate.
@@ -765,21 +870,25 @@ SUPR3DECL(int) SUPPageFree(void *pvPages, size_t cPages)
     AssertPtrReturn(pvPages, VERR_INVALID_POINTER);
     AssertReturn(cPages > 0, VERR_PAGE_COUNT_OUT_OF_RANGE);
 
-#ifdef RT_OS_WINDOWS
-    /*
-     * Temporary hack for windows, see above.
-     */
-    return SUPPageFreeLocked(pvPages, cPages);
-#else
     /*
      * Call OS specific worker.
      */
     return suplibOsPageFree(&g_supLibData, pvPages, cPages);
-#endif
 }
 
 
-SUPR3DECL(int) SUPPageLock(void *pvStart, size_t cPages, PSUPPAGE paPages)
+/**
+ * Locks down the physical memory backing a virtual memory
+ * range in the current process.
+ *
+ * @returns VBox status code.
+ * @param   pvStart         Start of virtual memory range.
+ *                          Must be page aligned.
+ * @param   cPages          Number of pages.
+ * @param   paPages         Where to store the physical page addresses returned.
+ *                          On entry this will point to an array of with cbMemory >> PAGE_SHIFT entries.
+ */
+SUPR3DECL(int) supR3PageLock(void *pvStart, size_t cPages, PSUPPAGE paPages)
 {
     /*
      * Validate.
@@ -834,7 +943,14 @@ SUPR3DECL(int) SUPPageLock(void *pvStart, size_t cPages, PSUPPAGE paPages)
 }
 
 
-SUPR3DECL(int) SUPPageUnlock(void *pvStart)
+/**
+ * Releases locked down pages.
+ *
+ * @returns VBox status code.
+ * @param   pvStart         Start of virtual memory range previously locked
+ *                          down by SUPPageLock().
+ */
+SUPR3DECL(int) supR3PageUnlock(void *pvStart)
 {
     /*
      * Validate.
@@ -864,40 +980,9 @@ SUPR3DECL(int) SUPPageUnlock(void *pvStart)
 }
 
 
-SUPR3DECL(int) SUPPageAllocLockedEx(size_t cPages, void **ppvPages, PSUPPAGE paPages)
-{
-    return SUPR3PageAllocEx(cPages, 0 /*fFlags*/, ppvPages, NULL /*pR0Ptr*/, paPages);
-}
-
-
-SUPR3DECL(int) SUPPageFreeLocked(void *pvPages, size_t cPages)
-{
-    /*
-     * Validate.
-     */
-    AssertPtrReturn(pvPages, VERR_INVALID_POINTER);
-    AssertReturn(cPages > 0, VERR_PAGE_COUNT_OUT_OF_RANGE);
-
-    /*
-     * Check if we're employing the fallback or not to avoid the
-     * fuzzy handling of this in SUPR3PageFreeEx.
-     */
-    int rc;
-    if (g_fSupportsPageAllocNoKernel)
-        rc = SUPR3PageFreeEx(pvPages, cPages);
-    else
-    {
-        /* fallback */
-        rc = SUPPageUnlock(pvPages);
-        if (RT_SUCCESS(rc))
-            rc = suplibOsPageFree(&g_supLibData, pvPages, cPages);
-    }
-    return rc;
-}
-
-
 /**
- * Fallback for SUPPageAllocLockedEx on systems where RTR0MemObjPhysAllocNC isn't supported.
+ * Fallback for SUPR3PageAllocEx on systems where RTR0MemObjPhysAllocNC isn't
+ * supported.
  */
 static int supPagePageAllocNoKernelFallback(size_t cPages, void **ppvPages, PSUPPAGE paPages)
 {
@@ -906,7 +991,7 @@ static int supPagePageAllocNoKernelFallback(size_t cPages, void **ppvPages, PSUP
     {
         if (!paPages)
             paPages = (PSUPPAGE)alloca(sizeof(paPages[0]) * cPages);
-        rc = SUPPageLock(*ppvPages, cPages, paPages);
+        rc = supR3PageLock(*ppvPages, cPages, paPages);
         if (RT_FAILURE(rc))
             suplibOsPageFree(&g_supLibData, *ppvPages, cPages);
     }
@@ -987,6 +1072,9 @@ SUPR3DECL(int) SUPR3PageAllocEx(size_t cPages, uint32_t fFlags, void **ppvPages,
                         paPages[iPage].Phys = pReq->u.Out.aPages[iPage];
                         Assert(!(paPages[iPage].Phys & ~X86_PTE_PAE_PG_MASK));
                     }
+#ifdef RT_OS_DARWIN /* HACK ALERT! */
+                supR3TouchPages(pReq->u.Out.pvR3, cPages);
+#endif
             }
             else if (   rc == VERR_NOT_SUPPORTED
                      && !pR0Ptr)
@@ -1044,6 +1132,48 @@ SUPR3DECL(int) SUPR3PageMapKernel(void *pvR3, uint32_t off, uint32_t cb, uint32_
 }
 
 
+SUPR3DECL(int) SUPR3PageProtect(void *pvR3, RTR0PTR R0Ptr, uint32_t off, uint32_t cb, uint32_t fProt)
+{
+    /*
+     * Validate.
+     */
+    AssertPtrReturn(pvR3, VERR_INVALID_POINTER);
+    Assert(!(off & PAGE_OFFSET_MASK));
+    Assert(!(cb & PAGE_OFFSET_MASK) && cb);
+    AssertReturn(!(fProt & ~(RTMEM_PROT_NONE | RTMEM_PROT_READ | RTMEM_PROT_WRITE | RTMEM_PROT_EXEC)), VERR_INVALID_PARAMETER);
+
+    /* fake */
+    if (RT_UNLIKELY(g_u32FakeMode))
+        return RTMemProtect((uint8_t *)pvR3 + off, cb, fProt);
+
+    /*
+     * Some OSes can do this from ring-3, so try that before we
+     * issue the IOCtl to the SUPDRV kernel module.
+     * (Yea, this isn't very nice, but just try get the job done for now.)
+     */
+#if !defined(RT_OS_SOLARIS)
+    RTMemProtect((uint8_t *)pvR3 + off, cb, fProt);
+#endif
+
+    SUPPAGEPROTECT Req;
+    Req.Hdr.u32Cookie = g_u32Cookie;
+    Req.Hdr.u32SessionCookie = g_u32SessionCookie;
+    Req.Hdr.cbIn = SUP_IOCTL_PAGE_PROTECT_SIZE_IN;
+    Req.Hdr.cbOut = SUP_IOCTL_PAGE_PROTECT_SIZE_OUT;
+    Req.Hdr.fFlags = SUPREQHDR_FLAGS_DEFAULT;
+    Req.Hdr.rc = VERR_INTERNAL_ERROR;
+    Req.u.In.pvR3 = pvR3;
+    Req.u.In.pvR0 = R0Ptr;
+    Req.u.In.offSub = off;
+    Req.u.In.cbSub = cb;
+    Req.u.In.fProt = fProt;
+    int rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_PAGE_PROTECT, &Req, SUP_IOCTL_PAGE_PROTECT_SIZE);
+    if (RT_SUCCESS(rc))
+        rc = Req.Hdr.rc;
+    return rc;
+}
+
+
 SUPR3DECL(int) SUPR3PageFreeEx(void *pvPages, size_t cPages)
 {
     /*
@@ -1079,7 +1209,7 @@ SUPR3DECL(int) SUPR3PageFreeEx(void *pvPages, size_t cPages)
         if (    rc == VERR_INVALID_PARAMETER
             &&  !g_fSupportsPageAllocNoKernel)
         {
-            int rc2 = SUPPageUnlock(pvPages);
+            int rc2 = supR3PageUnlock(pvPages);
             if (RT_SUCCESS(rc2))
                 rc = suplibOsPageFree(&g_supLibData, pvPages, cPages);
         }
@@ -1088,13 +1218,7 @@ SUPR3DECL(int) SUPR3PageFreeEx(void *pvPages, size_t cPages)
 }
 
 
-SUPR3DECL(void *) SUPContAlloc(size_t cPages, PRTHCPHYS pHCPhys)
-{
-    return SUPContAlloc2(cPages, NIL_RTR0PTR, pHCPhys);
-}
-
-
-SUPR3DECL(void *) SUPContAlloc2(size_t cPages, PRTR0PTR pR0Ptr, PRTHCPHYS pHCPhys)
+SUPR3DECL(void *) SUPR3ContAlloc(size_t cPages, PRTR0PTR pR0Ptr, PRTHCPHYS pHCPhys)
 {
     /*
      * Validate.
@@ -1136,6 +1260,9 @@ SUPR3DECL(void *) SUPContAlloc2(size_t cPages, PRTR0PTR pR0Ptr, PRTHCPHYS pHCPhy
         *pHCPhys = Req.u.Out.HCPhys;
         if (pR0Ptr)
             *pR0Ptr = Req.u.Out.pvR0;
+#ifdef RT_OS_DARWIN /* HACK ALERT! */
+        supR3TouchPages(Req.u.Out.pvR3, cPages);
+#endif
         return Req.u.Out.pvR3;
     }
 
@@ -1143,7 +1270,7 @@ SUPR3DECL(void *) SUPContAlloc2(size_t cPages, PRTR0PTR pR0Ptr, PRTHCPHYS pHCPhy
 }
 
 
-SUPR3DECL(int) SUPContFree(void *pv, size_t cPages)
+SUPR3DECL(int) SUPR3ContFree(void *pv, size_t cPages)
 {
     /*
      * Validate.
@@ -1178,7 +1305,7 @@ SUPR3DECL(int) SUPContFree(void *pv, size_t cPages)
 }
 
 
-SUPR3DECL(int) SUPLowAlloc(size_t cPages, void **ppvPages, PRTR0PTR ppvPagesR0, PSUPPAGE paPages)
+SUPR3DECL(int) SUPR3LowAlloc(size_t cPages, void **ppvPages, PRTR0PTR ppvPagesR0, PSUPPAGE paPages)
 {
     /*
      * Validate.
@@ -1233,6 +1360,9 @@ SUPR3DECL(int) SUPLowAlloc(size_t cPages, void **ppvPages, PRTR0PTR ppvPagesR0,
                     Assert(!(paPages[iPage].Phys & ~X86_PTE_PAE_PG_MASK));
                     Assert(paPages[iPage].Phys <= UINT32_C(0xfffff000));
                 }
+#ifdef RT_OS_DARWIN /* HACK ALERT! */
+            supR3TouchPages(pReq->u.Out.pvR3, cPages);
+#endif
         }
         RTMemTmpFree(pReq);
     }
@@ -1243,7 +1373,7 @@ SUPR3DECL(int) SUPLowAlloc(size_t cPages, void **ppvPages, PRTR0PTR ppvPagesR0,
 }
 
 
-SUPR3DECL(int) SUPLowFree(void *pv, size_t cPages)
+SUPR3DECL(int) SUPR3LowFree(void *pv, size_t cPages)
 {
     /*
      * Validate.
@@ -1302,7 +1432,7 @@ SUPR3DECL(int) SUPR3HardenedVerifyFile(const char *pszFilename, const char *pszM
 }
 
 
-SUPR3DECL(int) SUPLoadModule(const char *pszFilename, const char *pszModule, void **ppvImageBase)
+SUPR3DECL(int) SUPR3LoadModule(const char *pszFilename, const char *pszModule, void **ppvImageBase)
 {
     int rc = VINF_SUCCESS;
 #ifdef VBOX_WITH_HARDENING
@@ -1314,7 +1444,7 @@ SUPR3DECL(int) SUPLoadModule(const char *pszFilename, const char *pszModule, voi
     if (RT_SUCCESS(rc))
         rc = supLoadModule(pszFilename, pszModule, NULL, ppvImageBase);
     else
-        LogRel(("SUPLoadModule: Verification of \"%s\" failed, rc=%Rrc\n", rc));
+        LogRel(("SUPR3LoadModule: Verification of \"%s\" failed, rc=%Rrc\n", rc));
     return rc;
 }
 
@@ -1387,12 +1517,12 @@ static DECLCALLBACK(int) supLoadModuleResolveImport(RTLDRMOD hLdrMod, const char
     /*
      * Check the VMMR0.r0 module if loaded.
      */
-    /** @todo call the SUPLoadModule caller.... */
+    /** @todo call the SUPR3LoadModule caller.... */
     /** @todo proper reference counting and such. */
     if (g_pvVMMR0 != NIL_RTR0PTR)
     {
         void *pvValue;
-        if (!SUPGetSymbolR0((void *)g_pvVMMR0, pszSymbol, &pvValue))
+        if (!SUPR3GetSymbolR0((void *)g_pvVMMR0, pszSymbol, &pvValue))
         {
             *pValue = (uintptr_t)pvValue;
             return VINF_SUCCESS;
@@ -1506,7 +1636,7 @@ static DECLCALLBACK(int) supLoadModuleCreateTabsCB(RTLDRMOD hLdrMod, const char
 
 
 /**
- * Worker for SUPLoadModule().
+ * Worker for SUPR3LoadModule().
  *
  * @returns VBox status code.
  * @param   pszFilename     Name of the VMMR0 image file
@@ -1529,7 +1659,7 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
      * Open image file and figure its size.
      */
     RTLDRMOD hLdrMod;
-    int rc = RTLdrOpen(pszFilename, &hLdrMod);
+    int rc = RTLdrOpen(pszFilename, 0, RTLDRARCH_HOST, &hLdrMod);
     if (!RT_SUCCESS(rc))
         return rc;
 
@@ -1722,7 +1852,7 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
 }
 
 
-SUPR3DECL(int) SUPFreeModule(void *pvImageBase)
+SUPR3DECL(int) SUPR3FreeModule(void *pvImageBase)
 {
     /* fake */
     if (RT_UNLIKELY(g_u32FakeMode))
@@ -1752,7 +1882,7 @@ SUPR3DECL(int) SUPFreeModule(void *pvImageBase)
 }
 
 
-SUPR3DECL(int) SUPGetSymbolR0(void *pvImageBase, const char *pszSymbol, void **ppvValue)
+SUPR3DECL(int) SUPR3GetSymbolR0(void *pvImageBase, const char *pszSymbol, void **ppvValue)
 {
     *ppvValue = NULL;
 
@@ -1787,20 +1917,20 @@ SUPR3DECL(int) SUPGetSymbolR0(void *pvImageBase, const char *pszSymbol, void **p
 }
 
 
-SUPR3DECL(int) SUPLoadVMM(const char *pszFilename)
+SUPR3DECL(int) SUPR3LoadVMM(const char *pszFilename)
 {
     void *pvImageBase;
-    return SUPLoadModule(pszFilename, "VMMR0.r0", &pvImageBase);
+    return SUPR3LoadModule(pszFilename, "VMMR0.r0", &pvImageBase);
 }
 
 
-SUPR3DECL(int) SUPUnloadVMM(void)
+SUPR3DECL(int) SUPR3UnloadVMM(void)
 {
-    return SUPFreeModule((void*)g_pvVMMR0);
+    return SUPR3FreeModule((void*)g_pvVMMR0);
 }
 
 
-SUPR3DECL(int) SUPGipGetPhys(PRTHCPHYS pHCPhys)
+SUPR3DECL(int) SUPR3GipGetPhys(PRTHCPHYS pHCPhys)
 {
     if (g_pSUPGlobalInfoPage)
     {
@@ -1926,3 +2056,45 @@ SUPR3DECL(int) SUPR3HardenedLdrLoadAppPriv(const char *pszFilename, PRTLDRMOD ph
     return rc;
 }
 
+
+SUPR3DECL(int) SUPR3QueryVTxSupported(void)
+{
+#ifdef RT_OS_LINUX
+    return suplibOsQueryVTxSupported();
+#else
+    return VINF_SUCCESS;
+#endif
+}
+
+
+SUPR3DECL(int) SUPR3QueryVTCaps(uint32_t *pfCaps)
+{
+    AssertPtrReturn(pfCaps, VERR_INVALID_POINTER);
+
+    *pfCaps = 0;
+
+    /* fake */
+    if (RT_UNLIKELY(g_u32FakeMode))
+        return VINF_SUCCESS;
+
+    /*
+     * Issue IOCtl to the SUPDRV kernel module.
+     */
+    SUPVTCAPS Req;
+    Req.Hdr.u32Cookie = g_u32Cookie;
+    Req.Hdr.u32SessionCookie = g_u32SessionCookie;
+    Req.Hdr.cbIn = SUP_IOCTL_VT_CAPS_SIZE_IN;
+    Req.Hdr.cbOut = SUP_IOCTL_VT_CAPS_SIZE_OUT;
+    Req.Hdr.fFlags = SUPREQHDR_FLAGS_DEFAULT;
+    Req.Hdr.rc = VERR_INTERNAL_ERROR;
+    Req.u.Out.Caps = 0;
+    int rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_VT_CAPS, &Req, SUP_IOCTL_VT_CAPS_SIZE);
+    if (RT_SUCCESS(rc))
+    {
+        rc = Req.Hdr.rc;
+        if (RT_SUCCESS(rc))
+            *pfCaps = Req.u.Out.Caps;
+    }
+    return rc;
+}
+
