@@ -1,4 +1,4 @@
-/* $Rev: 52819 $ */
+/* $Rev: 58763 $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - Linux specifics.
  */
@@ -43,7 +43,7 @@
 #include <iprt/semaphore.h>
 #include <iprt/initterm.h>
 #include <iprt/process.h>
-#include <iprt/err.h>
+#include <VBox/err.h>
 #include <iprt/mem.h>
 #include <VBox/log.h>
 #include <iprt/mp.h>
@@ -125,9 +125,6 @@ extern int nmi_active;
 
 #endif /* CONFIG_X86_LOCAL_APIC */
 
-#define xstr(s) str(s)
-#define str(s) #s
-
 
 /*******************************************************************************
 *   Global Variables                                                           *
@@ -224,8 +221,10 @@ static struct miscdevice gMiscDevice =
 # if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 30)
 static struct dev_pm_ops gPlatformPMOps =
 {
-    .suspend = VBoxDrvSuspend,
-    .resume = VBoxDrvResume,
+    .suspend = VBoxDrvSuspend,  /* before entering deep sleep */
+    .resume  = VBoxDrvResume,   /* after wakeup from deep sleep */
+    .freeze  = VBoxDrvSuspend,  /* before creating hibernation image */
+    .restore = VBoxDrvResume,   /* after wakeing up from hibernation */
 };
 # endif
 
@@ -234,7 +233,7 @@ static struct platform_driver gPlatformDriver =
     .probe = VBoxDrvProbe,
 # if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 30)
     .suspend = VBoxDrvSuspend,
-    .resume = VBoxDrvResume,
+    .resume  = VBoxDrvResume,
 # endif
     /** @todo .shutdown? */
     .driver =
@@ -490,14 +489,16 @@ static int __init VBoxDrvLinuxInit(void)
             /* performance counter generates NMI and is not masked? */
             if ((GET_APIC_DELIVERY_MODE(v) == APIC_MODE_NMI) && !(v & APIC_LVT_MASKED))
             {
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31) && defined(CONFIG_PERF_COUNTERS)
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31) \
+     && (defined(CONFIG_PERF_COUNTERS) || defined(CONFIG_PERF_EVENTS))
                 /* 2.6.31+: The performance counter framework will initialize the LVTPC
                  * vector as NMI. We can't disable the framework but the kernel loader
                  * script will do 'echo 2 > /proc/sys/kernel/perf_counter_paranoid'
                  * which hopefilly prevents any usage of hardware performance counters
-                 * and therefore triggering of NMIs. */
+                 * and therefore triggering of NMIs.
+                 * 2.6.32+: CONFIG_PERF_COUNTERS => CONFIG_PERF_EVENTS */
                 printk(KERN_ERR DEVICE_NAME
-                       ": Warning: 2.6.31+ kernel detected. Most likely the hwardware performance\n"
+                       ": Warning: 2.6.31+ kernel detected. Most likely the hardware performance\n"
                                 DEVICE_NAME
                        ": counter framework which can generate NMIs is active. You have to prevent\n"
                                 DEVICE_NAME
@@ -529,7 +530,8 @@ nmi_activated:
     }
 # if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
     printk(KERN_DEBUG DEVICE_NAME ": Successfully done.\n");
-#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31) && defined(CONFIG_PERF_COUNTERS)
+#  if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31) \
+      && (defined(CONFIG_PERF_COUNTERS) || defined(CONFIG_PERF_EVENTS))
 no_error:
 #  endif
 # endif /* >= 2.6.19 */
@@ -616,7 +618,7 @@ no_error:
                                g_DevExt.pGip->u32Mode == SUPGIPMODE_SYNC_TSC ? "'synchronous'" : "'asynchronous'");
                         LogFlow(("VBoxDrv::ModuleInit returning %#x\n", rc));
                         printk(KERN_DEBUG DEVICE_NAME ": Successfully loaded version "
-                                VBOX_VERSION_STRING " (interface " xstr(SUPDRV_IOC_VERSION) ").\n");
+                                VBOX_VERSION_STRING " (interface " RT_XSTR(SUPDRV_IOC_VERSION) ").\n");
                         return rc;
                     }
 #ifdef VBOX_WITH_SUSPEND_NOTIFICATION
@@ -627,7 +629,7 @@ no_error:
             }
 
             rc = -EINVAL;
-            RTR0Term();
+            RTR0TermForced();
         }
         else
             rc = -EINVAL;
@@ -688,7 +690,7 @@ static void __exit VBoxDrvLinuxUnload(void)
      * Destroy GIP, delete the device extension and terminate IPRT.
      */
     supdrvDeleteDevExt(&g_DevExt);
-    RTR0Term();
+    RTR0TermForced();
 }
 
 
@@ -1064,7 +1066,7 @@ MODULE_AUTHOR("Sun Microsystems, Inc.");
 MODULE_DESCRIPTION("VirtualBox Support Driver");
 MODULE_LICENSE("GPL");
 #ifdef MODULE_VERSION
-MODULE_VERSION(VBOX_VERSION_STRING " (" xstr(SUPDRV_IOC_VERSION) ")");
+MODULE_VERSION(VBOX_VERSION_STRING " (" RT_XSTR(SUPDRV_IOC_VERSION) ")");
 #endif
 
 module_param(force_async_tsc, int, 0444);
