@@ -66,10 +66,6 @@
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
-/** @todo this quoting macros probably should be moved to a common place.
-  * The indirection is for expanding macros passed to the first macro. */
-#define VBOXSOLQUOTE2(x)         #x
-#define VBOXSOLQUOTE(x)          VBOXSOLQUOTE2(x)
 /** The module name. */
 #define DEVICE_NAME              "vboxdrv"
 /** The module description as seen in 'modinfo'. */
@@ -144,7 +140,7 @@ static struct dev_ops g_VBoxDrvSolarisDevOps =
 static struct modldrv g_VBoxDrvSolarisModule =
 {
     &mod_driverops,         /* extern from kernel */
-    DEVICE_DESC " " VBOX_VERSION_STRING "r" VBOXSOLQUOTE(VBOX_SVN_REV),
+    DEVICE_DESC " " VBOX_VERSION_STRING "r" RT_XSTR(VBOX_SVN_REV),
     &g_VBoxDrvSolarisDevOps
 };
 
@@ -245,7 +241,7 @@ int _init(void)
         }
         else
             LogRel((DEVICE_NAME ":VBoxDrvSolarisAttach: supdrvInitDevExt failed\n"));
-        RTR0Term();
+        RTR0TermForced();
     }
     else
         LogRel((DEVICE_NAME ":VBoxDrvSolarisAttach: failed to init R0Drv\n"));
@@ -272,7 +268,7 @@ int _fini(void)
     AssertRC(rc);
     g_Spinlock = NIL_RTSPINLOCK;
 
-    RTR0Term();
+    RTR0TermForced();
 
     memset(&g_DevExt, 0, sizeof(g_DevExt));
 
@@ -711,7 +707,7 @@ static int VBoxDrvSolarisIOCtlSlow(PSUPDRVSESSION pSession, int iCmd, int Mode,
 {
     int         rc;
     uint32_t    cbBuf = 0;
-    union 
+    union
     {
         SUPREQHDR   Hdr;
         uint8_t     abBuf[64];
@@ -774,7 +770,7 @@ static int VBoxDrvSolarisIOCtlSlow(PSUPDRVSESSION pSession, int iCmd, int Mode,
      * Process the IOCtl.
      */
     rc = supdrvIOCtl(iCmd, &g_DevExt, pSession, pHdr);
-    
+
     /*
      * Copy ioctl data and output buffer back to user space.
      */
