@@ -1,7 +1,6 @@
+/* $Id: SUPUninstall.cpp $ */
 /** @file
- *
- * VBox host drivers - Ring-0 support drivers - Testcases:
- * Testcase for driver uninstall
+ * SUPUninstall - Driver Uninstall.
  */
 
 /*
@@ -36,18 +35,19 @@
 #include <VBox/sup.h>
 #include <VBox/err.h>
 #include <iprt/initterm.h>
-#include <stdio.h>
+#include <iprt/stream.h>
+
 
 int main(int argc, char **argv)
 {
     RTR3Init();
-    int rc = SUPUninstall();
+    int rc = SUPR3Uninstall();
     if (RT_SUCCESS(rc))
     {
-        printf("uninstalled successfully\n");
+        RTPrintf("uninstalled successfully\n");
         return 0;
     }
-    printf("uninstallation failed. rc=%d\n", rc);
-
+    RTPrintf("uninstallation failed. rc=%Rrc\n", rc);
     return 1;
 }
+
