@@ -27,6 +27,8 @@
 #include <linux/netdevice.h>
 #include <linux/etherdevice.h>
 #include <linux/rtnetlink.h>
+#include <linux/miscdevice.h>
+#include <linux/ip.h>
 
 #define LOG_GROUP LOG_GROUP_NET_FLT_DRV
 #include <VBox/log.h>
@@ -98,7 +100,7 @@ unsigned dev_get_flags(const struct net_device *dev)
 
         flags = (dev->flags & ~(IFF_PROMISC |
                                 IFF_ALLMULTI |
-                                IFF_RUNNING)) | 
+                                IFF_RUNNING)) |
                 (dev->gflags & (IFF_PROMISC |
                                 IFF_ALLMULTI));
 
@@ -122,21 +124,6 @@ static void     VBoxNetFltLinuxUnload(void);
 /**
  * The (common) global data.
  */
-#ifdef RT_ARCH_AMD64
-/**
- * Memory for the executable memory heap (in IPRT).
- */
-extern uint8_t g_abExecMemory[4096]; /* cannot donate less than one page */
-__asm__(".section execmemory, \"awx\", @progbits\n\t"
-        ".align 32\n\t"
-        ".globl g_abExecMemory\n"
-        "g_abExecMemory:\n\t"
-        ".zero 4096\n\t"
-        ".type g_abExecMemory, @object\n\t"
-        ".size g_abExecMemory, 4096\n\t"
-        ".text\n\t");
-#endif
-
 static VBOXNETFLTGLOBALS g_VBoxNetFltGlobals;
 
 module_init(VBoxNetFltLinuxInit);
@@ -146,16 +133,9 @@ MODULE_AUTHOR("Sun Microsystems, Inc.");
 MODULE_DESCRIPTION("VirtualBox Network Filter Driver");
 MODULE_LICENSE("GPL");
 #ifdef MODULE_VERSION
-# define xstr(s) str(s)
-# define str(s)  #s
-MODULE_VERSION(VBOX_VERSION_STRING " (" xstr(INTNETTRUNKIFPORT_VERSION) ")");
+MODULE_VERSION(VBOX_VERSION_STRING " (" RT_XSTR(INTNETTRUNKIFPORT_VERSION) ")");
 #endif
 
-/**
- * The (common) global data.
- */
-static VBOXNETFLTGLOBALS g_VBoxNetFltGlobals;
-
 
 /**
  * Initialize module.
@@ -171,14 +151,6 @@ static int __init VBoxNetFltLinuxInit(void)
     rc = RTR0Init(0);
     if (RT_SUCCESS(rc))
     {
-#ifdef RT_ARCH_AMD64
-        rc = RTR0MemExecDonate(&g_abExecMemory[0], sizeof(g_abExecMemory));
-        printk("VBoxNetFlt: dbg - g_abExecMemory=%p\n", (void *)&g_abExecMemory[0]);
-        if (RT_FAILURE(rc))
-        {
-            printk("VBoxNetFlt: failed to donate exec memory, no logging will be available.\n");
-        }
-#endif
         Log(("VBoxNetFltLinuxInit\n"));
 
         /*
@@ -188,7 +160,7 @@ static int __init VBoxNetFltLinuxInit(void)
          * for establishing the connect to the support driver.
          */
         memset(&g_VBoxNetFltGlobals, 0, sizeof(g_VBoxNetFltGlobals));
-        rc = vboxNetFltInitGlobals(&g_VBoxNetFltGlobals);
+        rc = vboxNetFltInitGlobalsAndIdc(&g_VBoxNetFltGlobals);
         if (RT_SUCCESS(rc))
         {
             LogRel(("VBoxNetFlt: Successfully started.\n"));
@@ -220,7 +192,7 @@ static void __exit VBoxNetFltLinuxUnload(void)
     /*
      * Undo the work done during start (in reverse order).
      */
-    rc = vboxNetFltTryDeleteGlobals(&g_VBoxNetFltGlobals);
+    rc = vboxNetFltTryDeleteIdcAndGlobals(&g_VBoxNetFltGlobals);
     AssertRC(rc); NOREF(rc);
 
     RTR0Term();
@@ -476,12 +448,19 @@ static int vboxNetFltLinuxPacketHandler(struct sk_buff *pBuf,
     struct net_device *pDev;
     LogFlow(("vboxNetFltLinuxPacketHandler: pBuf=%p pSkbDev=%p pPacketType=%p\n",
              pBuf, pSkbDev, pPacketType));
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
+    Log3(("vboxNetFltLinuxPacketHandler: skb len=%u data_len=%u truesize=%u next=%p nr_frags=%u gso_size=%u gso_seqs=%u gso_type=%x frag_list=%p pkt_type=%x\n",
+          pBuf->len, pBuf->data_len, pBuf->truesize, pBuf->next, skb_shinfo(pBuf)->nr_frags, skb_shinfo(pBuf)->gso_size, skb_shinfo(pBuf)->gso_segs, skb_shinfo(pBuf)->gso_type, skb_shinfo(pBuf)->frag_list, pBuf->pkt_type));
+#else
+    Log3(("vboxNetFltLinuxPacketHandler: skb len=%u data_len=%u truesize=%u next=%p nr_frags=%u tso_size=%u tso_seqs=%u frag_list=%p pkt_type=%x\n",
+          pBuf->len, pBuf->data_len, pBuf->truesize, pBuf->next, skb_shinfo(pBuf)->nr_frags, skb_shinfo(pBuf)->tso_size, skb_shinfo(pBuf)->tso_segs, skb_shinfo(pBuf)->frag_list, pBuf->pkt_type));
+#endif
     /*
      * Drop it immediately?
      */
     if (!pBuf)
         return 0;
-    
+
     pThis = VBOX_FLT_PT_TO_INST(pPacketType);
     pDev = (struct net_device *)ASMAtomicUoReadPtr((void * volatile *)&pThis->u.s.pDev);
     if (pThis->u.s.pDev != pSkbDev)
@@ -511,6 +490,13 @@ static int vboxNetFltLinuxPacketHandler(struct sk_buff *pBuf,
             return 0;
         }
         pBuf = pCopy;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
+        Log3(("vboxNetFltLinuxPacketHandler: skb copy len=%u data_len=%u truesize=%u next=%p nr_frags=%u gso_size=%u gso_seqs=%u gso_type=%x frag_list=%p pkt_type=%x\n",
+              pBuf->len, pBuf->data_len, pBuf->truesize, pBuf->next, skb_shinfo(pBuf)->nr_frags, skb_shinfo(pBuf)->gso_size, skb_shinfo(pBuf)->gso_segs, skb_shinfo(pBuf)->gso_type, skb_shinfo(pBuf)->frag_list, pBuf->pkt_type));
+#else
+        Log3(("vboxNetFltLinuxPacketHandler: skb copy len=%u data_len=%u truesize=%u next=%p nr_frags=%u tso_size=%u tso_seqs=%u frag_list=%p pkt_type=%x\n",
+              pBuf->len, pBuf->data_len, pBuf->truesize, pBuf->next, skb_shinfo(pBuf)->nr_frags, skb_shinfo(pBuf)->tso_size, skb_shinfo(pBuf)->tso_segs, skb_shinfo(pBuf)->frag_list, pBuf->pkt_type));
+#endif
     }
 #endif
 
@@ -552,7 +538,7 @@ static void  vboxNetFltLinuxFreeSkBuff(struct sk_buff *pBuf, PINTNETSG pSG)
         kunmap(pSG->aSegs[i+1].pv);
     }
 #endif
-            
+
     dev_kfree_skb(pBuf);
 }
 
@@ -616,11 +602,24 @@ static void vboxNetFltLinuxForwardToIntNet(PVBOXNETFLTINS pThis, struct sk_buff
     {
         /* Need to segment the packet */
         struct sk_buff *pNext, *pSegment;
-        //Log2(("vboxNetFltLinuxForwardToIntNet: cb=%u gso_size=%u gso_segs=%u gso_type=%u\n",
-        //      pBuf->len, skb_shinfo(pBuf)->gso_size, skb_shinfo(pBuf)->gso_segs, skb_shinfo(pBuf)->gso_type));
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
+    Log3(("vboxNetFltLinuxForwardToIntNet: skb len=%u data_len=%u truesize=%u next=%p nr_frags=%u gso_size=%u gso_seqs=%u gso_type=%x frag_list=%p pkt_type=%x ip_summed=%d\n",
+          pBuf->len, pBuf->data_len, pBuf->truesize, pBuf->next, skb_shinfo(pBuf)->nr_frags, skb_shinfo(pBuf)->gso_size, skb_shinfo(pBuf)->gso_segs, skb_shinfo(pBuf)->gso_type, skb_shinfo(pBuf)->frag_list, pBuf->pkt_type, pBuf->ip_summed));
+#endif
 
-        for (pSegment = VBOX_SKB_GSO_SEGMENT(pBuf); pSegment; pSegment = pNext)
+        pSegment = VBOX_SKB_GSO_SEGMENT(pBuf);
+        if (IS_ERR(pSegment))
         {
+            dev_kfree_skb(pBuf);
+            LogRel(("VBoxNetFlt: Failed to segment a packet (%d).\n", PTR_ERR(pBuf)));
+            return;
+        }
+        for (; pSegment; pSegment = pNext)
+        {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
+            Log3(("vboxNetFltLinuxForwardToIntNet: segment len=%u data_len=%u truesize=%u next=%p nr_frags=%u gso_size=%u gso_seqs=%u gso_type=%x frag_list=%p pkt_type=%x\n",
+                  pSegment->len, pSegment->data_len, pSegment->truesize, pSegment->next, skb_shinfo(pSegment)->nr_frags, skb_shinfo(pSegment)->gso_size, skb_shinfo(pSegment)->gso_segs, skb_shinfo(pSegment)->gso_type, skb_shinfo(pSegment)->frag_list, pSegment->pkt_type));
+#endif
             pNext = pSegment->next;
             pSegment->next = 0;
             vboxNetFltLinuxForwardSegment(pThis, pSegment, fSrc);
@@ -629,13 +628,30 @@ static void vboxNetFltLinuxForwardToIntNet(PVBOXNETFLTINS pThis, struct sk_buff
     }
     else
     {
-        if (pBuf->ip_summed == CHECKSUM_PARTIAL)
+        if (pBuf->ip_summed == CHECKSUM_PARTIAL && pBuf->pkt_type == PACKET_OUTGOING)
+        {
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 18)
+            /*
+             * Try to work around the problem with CentOS 4.7 and 5.2 (2.6.9
+             * and 2.6.18 kernels), they pass wrong 'h' pointer down. We take IP
+             * header length from the header itself and reconstruct 'h' pointer
+             * to TCP (or whatever) header.
+             */
+            unsigned char *tmp = pBuf->h.raw;
+            if (pBuf->h.raw == pBuf->nh.raw && pBuf->protocol == htons(ETH_P_IP))
+                pBuf->h.raw = pBuf->nh.raw + pBuf->nh.iph->ihl * 4;
+#endif /* LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 18) */
             if (VBOX_SKB_CHECKSUM_HELP(pBuf))
             {
                 LogRel(("VBoxNetFlt: Failed to compute checksum, dropping the packet.\n"));
                 dev_kfree_skb(pBuf);
                 return;
             }
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 18)
+            /* Restore the original (wrong) pointer. */
+            pBuf->h.raw = tmp;
+#endif /* LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 18) */
+        }
         vboxNetFltLinuxForwardSegment(pThis, pBuf, fSrc);
     }
     /*
@@ -938,8 +954,12 @@ void vboxNetFltPortOsSetActive(PVBOXNETFLTINS pThis, bool fActive)
 {
     struct net_device * pDev;
 
-    LogFlow(("vboxNetFltPortOsSetActive: pThis=%p (%s), fActive=%s\n",
-             pThis, pThis->szName, fActive?"true":"false"));
+    LogFlow(("vboxNetFltPortOsSetActive: pThis=%p (%s), fActive=%s, fDisablePromiscuous=%s\n",
+             pThis, pThis->szName, fActive?"true":"false",
+             pThis->fDisablePromiscuous?"true":"false"));
+
+    if (pThis->fDisablePromiscuous)
+        return;
 
     pDev = vboxNetFltLinuxRetainNetDev(pThis);
     if (pDev)
@@ -950,64 +970,24 @@ void vboxNetFltPortOsSetActive(PVBOXNETFLTINS pThis, bool fActive)
          * Also, we have a bit or race conditions wrt the maintance of
          * host the interface promiscuity for vboxNetFltPortOsIsPromiscuous.
          */
-        u_int16_t fIf;
 #ifdef LOG_ENABLED
+        u_int16_t fIf;
         unsigned const cPromiscBefore = VBOX_GET_PCOUNT(pDev);
 #endif
         if (fActive)
         {
             Assert(!pThis->u.s.fPromiscuousSet);
 
-#if 0
-            /*
-             * Try bring the interface up and running if it's down.
-             */
-            fIf = dev_get_flags(pDev);
-            if ((fIf & (IFF_UP | IFF_RUNNING)) != (IFF_UP | IFF_RUNNING))
-            {
-                rtnl_lock();
-                int err = dev_change_flags(pDev, fIf | IFF_UP);
-                rtnl_unlock();
-                fIf = dev_get_flags(pDev);
-            }
-
-            /*
-             * Is it already up?  If it isn't, leave it to the link event or
-             * we'll upset if_pcount (as stated above, ifnet_set_promiscuous is weird).
-             */
-            if ((fIf & (IFF_UP | IFF_RUNNING)) == (IFF_UP | IFF_RUNNING)
-                && !ASMAtomicReadBool(&pThis->u.s.fPromiscuousSet))
-            {
-#endif
-                rtnl_lock();
-                dev_set_promiscuity(pDev, 1);
-                rtnl_unlock();
-                pThis->u.s.fPromiscuousSet = true;
-                Log(("vboxNetFltPortOsSetActive: enabled promiscuous mode on %s (%d)\n", pThis->szName, VBOX_GET_PCOUNT(pDev)));
-#if 0
-                /* check if it actually worked, this stuff is not always behaving well. */
-                if (!(dev_get_flags(pDev) & IFF_PROMISC))
-                {
-                    err = dev_change_flags(pDev, fIf | IFF_PROMISC);
-                    if (!err)
-                        Log(("vboxNetFlt: fixed IFF_PROMISC on %s (%d->%d)\n", pThis->szName, cPromiscBefore, VBOX_GET_PCOUNT(pDev)));
-                    else
-                        Log(("VBoxNetFlt: failed to fix IFF_PROMISC on %s, err=%d (%d->%d)\n",
-                             pThis->szName, err, cPromiscBefore, VBOX_GET_PCOUNT(pDev)));
-                }
-#endif
-#if 0
-            }
-            else if (!err)
-                Log(("VBoxNetFlt: Waiting for the link to come up... (%d->%d)\n", cPromiscBefore, VBOX_GET_PCOUNT(pDev)));
-            if (err)
-                LogRel(("VBoxNetFlt: Failed to put '%s' into promiscuous mode, err=%d (%d->%d)\n", pThis->szName, err, cPromiscBefore, VBOX_GET_PCOUNT(pDev)));
-#endif
+            rtnl_lock();
+            dev_set_promiscuity(pDev, 1);
+            rtnl_unlock();
+            pThis->u.s.fPromiscuousSet = true;
+            Log(("vboxNetFltPortOsSetActive: enabled promiscuous mode on %s (%d)\n", pThis->szName, VBOX_GET_PCOUNT(pDev)));
         }
         else
         {
             if (pThis->u.s.fPromiscuousSet)
-            {    
+            {
                 rtnl_lock();
                 dev_set_promiscuity(pDev, -1);
                 rtnl_unlock();
@@ -1015,8 +995,10 @@ void vboxNetFltPortOsSetActive(PVBOXNETFLTINS pThis, bool fActive)
             }
             pThis->u.s.fPromiscuousSet = false;
 
+#ifdef LOG_ENABLED
             fIf = dev_get_flags(pDev);
             Log(("VBoxNetFlt: fIf=%#x; %d->%d\n", fIf, cPromiscBefore, VBOX_GET_PCOUNT(pDev)));
+#endif
         }
 
         vboxNetFltLinuxReleaseNetDev(pThis, pDev);
@@ -1058,12 +1040,15 @@ void vboxNetFltOsDeleteInstance(PVBOXNETFLTINS pThis)
     }
     Log(("vboxNetFltOsDeleteInstance: this=%p: Notifier removed.\n", pThis));
     unregister_netdevice_notifier(&pThis->u.s.Notifier);
+    module_put(THIS_MODULE);
 }
 
 
-int  vboxNetFltOsInitInstance(PVBOXNETFLTINS pThis)
+int  vboxNetFltOsInitInstance(PVBOXNETFLTINS pThis, void *pvContext)
 {
     int err;
+    NOREF(pvContext);
+
     pThis->u.s.Notifier.notifier_call = vboxNetFltLinuxNotifierCallback;
     err = register_netdevice_notifier(&pThis->u.s.Notifier);
     if (err)
@@ -1074,8 +1059,13 @@ int  vboxNetFltOsInitInstance(PVBOXNETFLTINS pThis)
         LogRel(("VBoxNetFlt: failed to find %s.\n", pThis->szName));
         return VERR_INTNET_FLT_IF_NOT_FOUND;
     }
+
     Log(("vboxNetFltOsInitInstance: this=%p: Notifier installed.\n", pThis));
-    return pThis->fDisconnectedFromHost ? VERR_INTNET_FLT_IF_FAILED : VINF_SUCCESS;
+    if (   pThis->fDisconnectedFromHost
+        || !try_module_get(THIS_MODULE))
+        return VERR_INTNET_FLT_IF_FAILED;
+
+    return VINF_SUCCESS;
 }
 
 int  vboxNetFltOsPreInitInstance(PVBOXNETFLTINS pThis)
