@@ -20,6 +20,60 @@
  * additional information or have any questions.
  */
 
+
+/** @page pg_hostclip       The Shared Clipboard Host Service
+ *
+ * The shared clipboard host service provides a proxy between the host's
+ * clipboard and a similar proxy running on a guest.  The service is split
+ * into a platform-independant core and platform-specific backends.  The
+ * service defines two communication protocols - one to communicate with the
+ * clipboard service running on the guest, and one to communicate with the
+ * backend.  These will be described in a very skeletal fashion here.
+ *
+ * @section sec_hostclip_guest_proto  The guest communication protocol
+ *
+ * The guest clipboard service communicates with the host service via HGCM
+ * (the host service runs as an HGCM service).  The guest clipboard must
+ * connect to the host service before all else (Windows hosts currently only
+ * support one simultaneous connection).  Once it has connected, it can send
+ * HGCM messages to the host services, some of which will receive replies from
+ * the host.  The host can only reply to a guest message, it cannot initiate
+ * any communication.  The guest can in theory send any number of messages in
+ * parallel (see the descriptions of the messages for the practice), and the
+ * host will receive these in sequence, and may reply to them at once
+ * (releasing the caller in the guest) or defer the reply until later.
+ *
+ * There are currently four messages defined.  The first is
+ * VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG, which waits for a message from the
+ * host.  Host messages currently defined are
+ * VBOX_SHARED_CLIPBOARD_HOST_MSG_QUIT (unused),
+ * VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA (request that the guest send the
+ * contents of its clipboard to the host) and
+ * VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS (to notify the guest that new
+ * clipboard data is available).  If a host message is sent while the guest is
+ * not waiting, it will be queued until the guest requests it.  At most one
+ * host message of each type will be kept in the queue.  The host code only
+ * supports a single simultaneous VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG call
+ * from the guest.
+ *
+ * The second guest message is VBOX_SHARED_CLIPBOARD_FN_FORMATS, which tells
+ * the host that the guest has new clipboard data available.  The third is
+ * VBOX_SHARED_CLIPBOARD_FN_READ_DATA, which asks the host to send its
+ * clipboard data and waits until it arrives.  The host supports at most one
+ * simultaneous VBOX_SHARED_CLIPBOARD_FN_READ_DATA call from the guest - if a
+ * second call is made before the first has returned, the first will be
+ * aborted.
+ *
+ * The last guest message is VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA, which is
+ * used to send the contents of the guest clipboard to the host.  This call
+ * should be used after the host has requested data from the guest.
+ *
+ * @section sec_hostclip_backend_proto  The communication protocol with the
+ *                                      platform-specific backend
+ *
+ * This section may be written in the future :)
+ */
+
 #include <VBox/HostServices/VBoxClipboardSvc.h>
 #include <VBox/HostServices/VBoxClipboardExt.h>
 
@@ -123,21 +177,21 @@ static bool vboxSvcClipboardReturnMsg (VBOXCLIPBOARDCLIENTDATA *pClient, VBOXHGC
     /* Message priority is taken into account. */
     if (pClient->fMsgQuit)
     {
-        LogFlow(("vboxSvcClipboardReturnMsg: Quit\n"));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: Quit\n"));
         VBoxHGCMParmUInt32Set (&paParms[0], VBOX_SHARED_CLIPBOARD_HOST_MSG_QUIT);
         VBoxHGCMParmUInt32Set (&paParms[1], 0);
         pClient->fMsgQuit = false;
     }
     else if (pClient->fMsgReadData)
     {
-        LogFlow(("vboxSvcClipboardReturnMsg: ReadData %02X\n", pClient->u32RequestedFormat));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: ReadData %02X\n", pClient->u32RequestedFormat));
         VBoxHGCMParmUInt32Set (&paParms[0], VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA);
         VBoxHGCMParmUInt32Set (&paParms[1], pClient->u32RequestedFormat);
         pClient->fMsgReadData = false;
     }
     else if (pClient->fMsgFormats)
     {
-        LogFlow(("vboxSvcClipboardReturnMsg: Formats %02X\n", pClient->u32AvailableFormats));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: Formats %02X\n", pClient->u32AvailableFormats));
         VBoxHGCMParmUInt32Set (&paParms[0], VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS);
         VBoxHGCMParmUInt32Set (&paParms[1], pClient->u32AvailableFormats);
         pClient->fMsgFormats = false;
@@ -145,7 +199,7 @@ static bool vboxSvcClipboardReturnMsg (VBOXCLIPBOARDCLIENTDATA *pClient, VBOXHGC
     else
     {
         /* No pending messages. */
-        LogFlow(("vboxSvcClipboardReturnMsg: no message\n"));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: no message\n"));
         return false;
     }
 
@@ -161,7 +215,7 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
         {
             case VBOX_SHARED_CLIPBOARD_HOST_MSG_QUIT:
             {
-                LogFlow(("vboxSvcClipboardReportMsg: Quit\n"));
+                LogRelFlow(("vboxSvcClipboardReportMsg: Quit\n"));
                 pClient->fMsgQuit = true;
             } break;
             case VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA:
@@ -173,7 +227,7 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
                     break;
                 }
 
-                LogFlow(("vboxSvcClipboardReportMsg: ReadData %02X\n", u32Formats));
+                LogRelFlow(("vboxSvcClipboardReportMsg: ReadData %02X\n", u32Formats));
                 pClient->u32RequestedFormat = u32Formats;
                 pClient->fMsgReadData = true;
             } break;
@@ -186,14 +240,14 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
                     break;
                 }
 
-                LogFlow(("vboxSvcClipboardReportMsg: Formats %02X\n", u32Formats));
+                LogRelFlow(("vboxSvcClipboardReportMsg: Formats %02X\n", u32Formats));
                 pClient->u32AvailableFormats = u32Formats;
                 pClient->fMsgFormats = true;
             } break;
             default:
             {
                 /* Invalid message. */
-                LogFlow(("vboxSvcClipboardReportMsg: invalid message %d\n", u32Msg));
+                LogRelFlow(("vboxSvcClipboardReportMsg: invalid message %d\n", u32Msg));
             } break;
         }
 
@@ -215,7 +269,7 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
 
             if (fMessageReturned)
             {
-                LogFlow(("vboxSvcClipboardReportMsg: CallComplete\n"));
+                LogRelFlow(("vboxSvcClipboardReportMsg: CallComplete\n"));
                 g_pHelpers->pfnCallComplete (callHandle, VINF_SUCCESS);
             }
         }
@@ -302,7 +356,7 @@ static DECLCALLBACK(int) svcConnect (void *, uint32_t u32ClientID, void *pvClien
         g_pClient = pClient;
     }
 
-    Log(("vboxClipboardConnect: rc = %Rrc\n", rc));
+    LogRel2(("vboxClipboardConnect: rc = %Rrc\n", rc));
 
     return rc;
 }
@@ -317,8 +371,8 @@ static DECLCALLBACK(void) svcCall (void *,
 {
     int rc = VINF_SUCCESS;
 
-    Log(("svcCall: u32ClientID = %d, fn = %d, cParms = %d, pparms = %d\n",
-         u32ClientID, u32Function, cParms, paParms));
+    LogRel2(("svcCall: u32ClientID = %d, fn = %d, cParms = %d, pparms = %d\n",
+             u32ClientID, u32Function, cParms, paParms));
 
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
@@ -330,7 +384,7 @@ static DECLCALLBACK(void) svcCall (void *,
     for (i = 0; i < cParms; i++)
     {
         /** @todo parameters other than 32 bit */
-        Log(("    pparms[%d]: type %d value %d\n", i, paParms[i].type, paParms[i].u.uint32));
+        LogRel2(("    pparms[%d]: type %d value %d\n", i, paParms[i].type, paParms[i].u.uint32));
     }
 #endif
 
@@ -339,7 +393,7 @@ static DECLCALLBACK(void) svcCall (void *,
         case VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG:
         {
             /* The quest requests a host message. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG\n"));
 
             if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_GET_HOST_MSG)
             {
@@ -372,7 +426,7 @@ static DECLCALLBACK(void) svcCall (void *,
                         pClient->async.callHandle = callHandle;
                         pClient->async.paParms    = paParms;
 
-                        Log(("svcCall: async.\n"));
+                        LogRel2(("svcCall: async.\n"));
                     }
 
                     vboxSvcClipboardUnlock ();
@@ -387,7 +441,7 @@ static DECLCALLBACK(void) svcCall (void *,
         case VBOX_SHARED_CLIPBOARD_FN_FORMATS:
         {
             /* The guest reports that some formats are available. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_FORMATS\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_FORMATS\n"));
 
             if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_FORMATS)
             {
@@ -432,7 +486,7 @@ static DECLCALLBACK(void) svcCall (void *,
         case VBOX_SHARED_CLIPBOARD_FN_READ_DATA:
         {
             /* The guest wants to read data in the given format. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_READ_DATA\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_READ_DATA\n"));
 
             if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_READ_DATA)
             {
@@ -478,7 +532,7 @@ static DECLCALLBACK(void) svcCall (void *,
 
                             g_fReadingData = true;
                             rc = g_pfnExtension (g_pvExtension, VBOX_CLIPBOARD_EXT_FN_DATA_READ, &parms, sizeof (parms));
-                            LogFlow(("DATA: g_fDelayedAnnouncement = %d, g_u32DelayedFormats = 0x%x\n", g_fDelayedAnnouncement, g_u32DelayedFormats));
+                            LogRelFlow(("DATA: g_fDelayedAnnouncement = %d, g_u32DelayedFormats = 0x%x\n", g_fDelayedAnnouncement, g_u32DelayedFormats));
                             if (g_fDelayedAnnouncement)
                             {
                                 vboxSvcClipboardReportMsg (g_pClient, VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS, g_u32DelayedFormats);
@@ -494,10 +548,29 @@ static DECLCALLBACK(void) svcCall (void *,
                         }
                         else
                         {
+                            /* Release any other pending read, as we only
+                             * support one pending read at one time. */
+                            vboxSvcClipboardCompleteReadData(pClient, VERR_NO_DATA, 0);
                             rc = vboxClipboardReadData (pClient, u32Format, pv, cb, &cbActual);
                         }
 
-                        if (RT_SUCCESS (rc))
+                        /* Remember our read request until it is completed.
+                         * See the protocol description above for more
+                         * information. */
+                        if (rc == VINF_HGCM_ASYNC_EXECUTE)
+                        {
+                            if (vboxSvcClipboardLock())
+                            {
+                                pClient->asyncRead.callHandle = callHandle;
+                                pClient->asyncRead.paParms    = paParms;
+                                pClient->fReadPending         = true;
+                                fAsynchronousProcessing = true;
+                                vboxSvcClipboardUnlock();
+                            }
+                            else
+                                rc = VERR_NOT_SUPPORTED;
+                        }
+                        else if (RT_SUCCESS (rc))
                         {
                             VBoxHGCMParmUInt32Set (&paParms[2], cbActual);
                         }
@@ -509,7 +582,7 @@ static DECLCALLBACK(void) svcCall (void *,
         case VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA:
         {
             /* The guest writes the requested data. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA\n"));
 
             if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_WRITE_DATA)
             {
@@ -567,7 +640,7 @@ static DECLCALLBACK(void) svcCall (void *,
         }
     }
 
-    LogFlow(("svcCall: rc = %Rrc\n", rc));
+    LogRelFlow(("svcCall: rc = %Rrc\n", rc));
 
     if (!fAsynchronousProcessing)
     {
@@ -575,6 +648,29 @@ static DECLCALLBACK(void) svcCall (void *,
     }
 }
 
+/** If the client in the guest is waiting for a read operation to complete
+ * then complete it, otherwise return.  See the protocol description in the
+ * shared clipboard module description. */
+void vboxSvcClipboardCompleteReadData(VBOXCLIPBOARDCLIENTDATA *pClient, int rc, uint32_t cbActual)
+{
+    VBOXHGCMCALLHANDLE callHandle = NULL;
+    VBOXHGCMSVCPARM *paParms = NULL;
+    bool fReadPending = false;
+    if (vboxSvcClipboardLock())  /* if not can we do anything useful? */
+    {
+        callHandle   = pClient->asyncRead.callHandle;
+        paParms      = pClient->asyncRead.paParms;
+        fReadPending = pClient->fReadPending;
+        pClient->fReadPending = false;
+        vboxSvcClipboardUnlock();
+    }
+    if (fReadPending)
+    {
+        VBoxHGCMParmUInt32Set (&paParms[2], cbActual);
+        g_pHelpers->pfnCallComplete (callHandle, rc);
+    }
+}
+
 /*
  * We differentiate between a function handler for the guest and one for the host.
  */
@@ -585,14 +681,14 @@ static DECLCALLBACK(int) svcHostCall (void *,
 {
     int rc = VINF_SUCCESS;
 
-    Log(("svcHostCall: fn = %d, cParms = %d, pparms = %d\n",
+    LogRel2(("svcHostCall: fn = %d, cParms = %d, pparms = %d\n",
          u32Function, cParms, paParms));
 
     switch (u32Function)
     {
         case VBOX_SHARED_CLIPBOARD_HOST_FN_SET_MODE:
         {
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_HOST_FN_SET_MODE\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_HOST_FN_SET_MODE\n"));
 
             if (cParms != 1)
             {
@@ -618,10 +714,23 @@ static DECLCALLBACK(int) svcHostCall (void *,
             break;
     }
 
-    LogFlow(("svcHostCall: rc = %Rrc\n", rc));
+    LogRelFlow(("svcHostCall: rc = %Rrc\n", rc));
     return rc;
 }
 
+/**
+ * SSM descriptor table for the VBOXCLIPBOARDCLIENTDATA structure.
+ */
+static SSMFIELD const g_aClipboardClientDataFields[] =
+{
+    SSMFIELD_ENTRY(VBOXCLIPBOARDCLIENTDATA, u32ClientID),  /* for validation purposes */
+    SSMFIELD_ENTRY(VBOXCLIPBOARDCLIENTDATA, fMsgQuit),
+    SSMFIELD_ENTRY(VBOXCLIPBOARDCLIENTDATA, fMsgReadData),
+    SSMFIELD_ENTRY(VBOXCLIPBOARDCLIENTDATA, fMsgFormats),
+    SSMFIELD_ENTRY(VBOXCLIPBOARDCLIENTDATA, u32RequestedFormat),
+    SSMFIELD_ENTRY_TERM()
+};
+
 static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
 {
     /* If there are any pending requests, they must be completed here. Since
@@ -635,16 +744,15 @@ static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClie
      * by VMMDev. The service therefore must save state as if there were no
      * pending request.
      */
-    Log(("svcSaveState: u32ClientID = %d\n", u32ClientID));
+    LogRel2 (("svcSaveState: u32ClientID = %d\n", u32ClientID));
 
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
-    /* Save client structure length & contents */
-    int rc = SSMR3PutU32(pSSM, sizeof(*pClient));
-    AssertRCReturn(rc, rc);
-
-    rc = SSMR3PutMem(pSSM, pClient, sizeof(*pClient));
-    AssertRCReturn(rc, rc);
+    /* This field used to be the length. We're using it as a version field
+       with the high bit set. */
+    SSMR3PutU32 (pSSM, UINT32_C (0x80000002));
+    int rc = SSMR3PutStructEx (pSSM, pClient, sizeof(*pClient), 0 /*fFlags*/, &g_aClipboardClientDataFields[0], NULL);
+    AssertRCReturn (rc, rc);
 
     if (pClient->fAsync)
     {
@@ -652,45 +760,119 @@ static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClie
         pClient->fAsync = false;
     }
 
+    vboxSvcClipboardCompleteReadData (pClient, VINF_SUCCESS, 0);
+
     return VINF_SUCCESS;
 }
 
 static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
 {
-    Log(("svcLoadState: u32ClientID = %d\n", u32ClientID));
+    LogRel2 (("svcLoadState: u32ClientID = %d\n", u32ClientID));
 
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
     /* Existing client can not be in async state yet. */
-    Assert(!pClient->fAsync);
+    Assert (!pClient->fAsync);
+
+    /* Save the client ID for data validation. */
+    /** @todo isn't this the same as u32ClientID? Playing safe for now... */
+    uint32_t const u32ClientIDOld = pClient->u32ClientID;
 
     /* Restore the client data. */
-    uint32_t len;
-    int rc = SSMR3GetU32(pSSM, &len);
-    AssertRCReturn(rc, rc);
-
-    if (len != sizeof(VBOXCLIPBOARDCLIENTDATA))
+    uint32_t lenOrVer;
+    int rc = SSMR3GetU32 (pSSM, &lenOrVer);
+    AssertRCReturn (rc, rc);
+    if (lenOrVer == UINT32_C (0x80000002))
     {
-        Log(("Client len mismatch: %d %d\n", len, sizeof (VBOXCLIPBOARDCLIENTDATA)));
+        rc = SSMR3GetStructEx (pSSM, pClient, sizeof(*pClient), 0 /*fFlags*/, &g_aClipboardClientDataFields[0], NULL);
+        AssertRCReturn (rc, rc);
+    }
+    else if (lenOrVer == (SSMR3HandleHostBits (pSSM) == 64 ? 72 : 48))
+    {
+        /**
+         * This structure corresponds to the original layout of the
+         * VBOXCLIPBOARDCLIENTDATA structure.  As the structure was saved as a whole
+         * when saving state, we need to remember it forever in order to preserve
+         * compatibility.
+         *
+         * (Starting with 3.1 this is no longer used.)
+         */
+        typedef struct _CLIPSAVEDSTATEDATA
+        {
+            struct _CLIPSAVEDSTATEDATA *pNext;
+            struct _CLIPSAVEDSTATEDATA *pPrev;
+
+            VBOXCLIPBOARDCONTEXT *pCtx;
+
+            uint32_t u32ClientID;
+
+            bool fAsync: 1; /* Guest is waiting for a message. */
+
+            bool fMsgQuit: 1;
+            bool fMsgReadData: 1;
+            bool fMsgFormats: 1;
+
+            struct {
+                VBOXHGCMCALLHANDLE callHandle;
+                VBOXHGCMSVCPARM *paParms;
+            } async;
+
+            struct {
+                 void *pv;
+                 uint32_t cb;
+                 uint32_t u32Format;
+            } data;
+
+            uint32_t u32AvailableFormats;
+            uint32_t u32RequestedFormat;
+
+        } CLIPSAVEDSTATEDATA;
+
+        /**
+         * SSM descriptor table for the CLIPSAVEDSTATEDATA structure.
+         */
+        static SSMFIELD const s_aClipSavedStateDataFields30[] =
+        {
+            SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, pNext),
+            SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, pPrev),
+            SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, pCtx),
+            SSMFIELD_ENTRY(                 CLIPSAVEDSTATEDATA, u32ClientID),
+            SSMFIELD_ENTRY_CUSTOM(fMsgQuit+fMsgReadData+fMsgFormats, RT_OFFSETOF(CLIPSAVEDSTATEDATA, u32ClientID) + 4, 4),
+            SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, async.callHandle),
+            SSMFIELD_ENTRY_IGN_HCPTR(       CLIPSAVEDSTATEDATA, async.paParms),
+            SSMFIELD_ENTRY_IGNORE(          CLIPSAVEDSTATEDATA, data.pv),
+            SSMFIELD_ENTRY_IGNORE(          CLIPSAVEDSTATEDATA, data.cb),
+            SSMFIELD_ENTRY_IGNORE(          CLIPSAVEDSTATEDATA, data.u32Format),
+            SSMFIELD_ENTRY_IGNORE(          CLIPSAVEDSTATEDATA, u32AvailableFormats),
+            SSMFIELD_ENTRY(                 CLIPSAVEDSTATEDATA, u32RequestedFormat),
+            SSMFIELD_ENTRY_TERM()
+        };
+
+        CLIPSAVEDSTATEDATA savedState;
+        RT_ZERO (savedState);
+        rc = SSMR3GetStructEx (pSSM, &savedState, sizeof(savedState), SSMSTRUCT_FLAGS_MEM_BAND_AID,
+                               &s_aClipSavedStateDataFields30[0], NULL);
+        AssertRCReturn (rc, rc);
+
+        pClient->fMsgQuit           = savedState.fMsgQuit;
+        pClient->fMsgReadData       = savedState.fMsgReadData;
+        pClient->fMsgFormats        = savedState.fMsgFormats;
+        pClient->u32RequestedFormat = savedState.u32RequestedFormat;
+    }
+    else
+    {
+        LogRel (("Client data size mismatch: got %#x\n", lenOrVer));
         return VERR_SSM_DATA_UNIT_FORMAT_CHANGED;
     }
 
-    VBOXCLIPBOARDCLIENTDATA client;
-    rc = SSMR3GetMem(pSSM, &client, sizeof(client));
-    AssertRCReturn(rc, rc);
-
-    /* Verify the loaded clients data and update the pClient. */
-    if (pClient->u32ClientID != client.u32ClientID)
+    /* Verify the client ID. */
+    if (pClient->u32ClientID != u32ClientIDOld)
     {
-        Log(("Client ID mismatch: %d %d\n", pClient->u32ClientID, client.u32ClientID));
+        LogRel (("Client ID mismatch: expected %d, got %d\n", u32ClientIDOld, pClient->u32ClientID));
+        pClient->u32ClientID = u32ClientIDOld;
         return VERR_SSM_DATA_UNIT_FORMAT_CHANGED;
     }
 
-    pClient->fMsgQuit     = client.fMsgQuit;
-    pClient->fMsgReadData = client.fMsgReadData;
-    pClient->fMsgFormats  = client.fMsgFormats;
-    pClient->u32RequestedFormat = client.u32RequestedFormat;
-
     /* Actual host data are to be reported to guest (SYNC). */
     vboxClipboardSync (pClient);
 
@@ -705,7 +887,7 @@ static DECLCALLBACK(int) extCallback (uint32_t u32Function, uint32_t u32Format,
         {
             case VBOX_CLIPBOARD_EXT_FN_FORMAT_ANNOUNCE:
             {
-                LogFlow(("ANNOUNCE: g_fReadingData = %d\n", g_fReadingData));
+                LogRelFlow(("ANNOUNCE: g_fReadingData = %d\n", g_fReadingData));
                 if (g_fReadingData)
                 {
                     g_fDelayedAnnouncement = true;
@@ -732,7 +914,7 @@ static DECLCALLBACK(int) extCallback (uint32_t u32Function, uint32_t u32Format,
 
 static DECLCALLBACK(int) svcRegisterExtension(void *, PFNHGCMSVCEXT pfnExtension, void *pvExtension)
 {
-    LogFlowFunc(("pfnExtension = %p\n", pfnExtension));
+    LogRelFlowFunc(("pfnExtension = %p\n", pfnExtension));
 
     VBOXCLIPBOARDEXTPARMS parms;
 
@@ -765,7 +947,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
 {
     int rc = VINF_SUCCESS;
 
-    LogFlowFunc(("ptable = %p\n", ptable));
+    LogRelFlowFunc(("ptable = %p\n", ptable));
 
     if (!ptable)
     {
@@ -773,7 +955,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
     }
     else
     {
-        Log(("VBoxHGCMSvcLoad: ptable->cbSize = %d, ptable->u32Version = 0x%08X\n", ptable->cbSize, ptable->u32Version));
+        LogRel2(("VBoxHGCMSvcLoad: ptable->cbSize = %d, ptable->u32Version = 0x%08X\n", ptable->cbSize, ptable->u32Version));
 
         if (   ptable->cbSize != sizeof (VBOXHGCMSVCFNTABLE)
             || ptable->u32Version != VBOX_HGCM_SVC_VERSION)
