@@ -135,34 +135,37 @@ static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClie
     /* Save all the active mappings. */
     for (int i=0;i<SHFL_MAX_MAPPINGS;i++)
     {
-        rc = SSMR3PutU32(pSSM, FolderMapping[i].cMappings);
+        /* Mapping are saved in the order of increasing root handle values. */
+        MAPPING *pFolderMapping = vbsfMappingGetByRoot(i);
+
+        rc = SSMR3PutU32(pSSM, pFolderMapping? pFolderMapping->cMappings: 0);
         AssertRCReturn(rc, rc);
 
-        rc = SSMR3PutBool(pSSM, FolderMapping[i].fValid);
+        rc = SSMR3PutBool(pSSM, pFolderMapping? pFolderMapping->fValid: false);
         AssertRCReturn(rc, rc);
 
-        if (FolderMapping[i].fValid)
+        if (pFolderMapping && pFolderMapping->fValid)
         {
             uint32_t len;
 
-            len = ShflStringSizeOfBuffer(FolderMapping[i].pFolderName);
+            len = ShflStringSizeOfBuffer(pFolderMapping->pFolderName);
             rc = SSMR3PutU32(pSSM, len);
             AssertRCReturn(rc, rc);
 
-            rc = SSMR3PutMem(pSSM, FolderMapping[i].pFolderName, len);
+            rc = SSMR3PutMem(pSSM, pFolderMapping->pFolderName, len);
             AssertRCReturn(rc, rc);
 
-            len = ShflStringSizeOfBuffer(FolderMapping[i].pMapName);
+            len = ShflStringSizeOfBuffer(pFolderMapping->pMapName);
             rc = SSMR3PutU32(pSSM, len);
             AssertRCReturn(rc, rc);
 
-            rc = SSMR3PutMem(pSSM, FolderMapping[i].pMapName, len);
+            rc = SSMR3PutMem(pSSM, pFolderMapping->pMapName, len);
             AssertRCReturn(rc, rc);
 
-            rc = SSMR3PutBool(pSSM, FolderMapping[i].fHostCaseSensitive);
+            rc = SSMR3PutBool(pSSM, pFolderMapping->fHostCaseSensitive);
             AssertRCReturn(rc, rc);
 
-            rc = SSMR3PutBool(pSSM, FolderMapping[i].fGuestCaseSensitive);
+            rc = SSMR3PutBool(pSSM, pFolderMapping->fGuestCaseSensitive);
             AssertRCReturn(rc, rc);
         }
     }
@@ -202,74 +205,60 @@ static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClie
     /* We don't actually (fully) restore the state; we simply check if the current state is as we it expect it to be. */
     for (int i=0;i<SHFL_MAX_MAPPINGS;i++)
     {
-        bool fValid;
+        /* Load the saved mapping description and try to find it in the mappings. */
+        MAPPING mapping;
+        memset (&mapping, 0, sizeof (mapping));
 
         /* restore the folder mapping counter. */
-        rc = SSMR3GetU32(pSSM, &FolderMapping[i].cMappings);
+        rc = SSMR3GetU32(pSSM, &mapping.cMappings);
         AssertRCReturn(rc, rc);
 
-        rc = SSMR3GetBool(pSSM, &fValid);
+        rc = SSMR3GetBool(pSSM, &mapping.fValid);
         AssertRCReturn(rc, rc);
 
-        if (fValid != FolderMapping[i].fValid)
-            return VERR_SSM_UNEXPECTED_DATA;
-
-        if (FolderMapping[i].fValid)
+        if (mapping.fValid)
         {
-            PSHFLSTRING pName;
+            uint32_t cbFolderName;
+            PSHFLSTRING pFolderName;
 
-            /* Check the host path name. */
-            rc = SSMR3GetU32(pSSM, &len);
+            uint32_t cbMapName;
+            PSHFLSTRING pMapName;
+
+            /* Load the host path name. */
+            rc = SSMR3GetU32(pSSM, &cbFolderName);
             AssertRCReturn(rc, rc);
 
-            if (len != ShflStringSizeOfBuffer(FolderMapping[i].pFolderName))
-                return VERR_SSM_UNEXPECTED_DATA;
+            pFolderName = (PSHFLSTRING)RTMemAlloc(cbFolderName);
+            AssertReturn(pFolderName != NULL, VERR_NO_MEMORY);
 
-            pName = (PSHFLSTRING)RTMemAlloc(len);
-            Assert(pName);
-            if (pName == NULL)
-                return VERR_NO_MEMORY;
-
-            rc = SSMR3GetMem(pSSM, pName, len);
+            rc = SSMR3GetMem(pSSM, pFolderName, cbFolderName);
             AssertRCReturn(rc, rc);
 
-            if (memcmp(FolderMapping[i].pFolderName, pName, len))
-            {
-                RTMemFree(pName);
-                return VERR_SSM_UNEXPECTED_DATA;
-            }
-            RTMemFree(pName);
-
-            /* Check the map name. */
-            rc = SSMR3GetU32(pSSM, &len);
+            /* Load the map name. */
+            rc = SSMR3GetU32(pSSM, &cbMapName);
             AssertRCReturn(rc, rc);
 
-            if (len != ShflStringSizeOfBuffer(FolderMapping[i].pMapName))
-                return VERR_SSM_UNEXPECTED_DATA;
+            pMapName = (PSHFLSTRING)RTMemAlloc(cbMapName);
+            AssertReturn(pMapName != NULL, VERR_NO_MEMORY);
 
-            pName = (PSHFLSTRING)RTMemAlloc(len);
-            Assert(pName);
-            if (pName == NULL)
-                return VERR_NO_MEMORY;
-
-            rc = SSMR3GetMem(pSSM, pName, len);
+            rc = SSMR3GetMem(pSSM, pMapName, cbMapName);
             AssertRCReturn(rc, rc);
 
-            if (memcmp(FolderMapping[i].pMapName, pName, len))
-            {
-                RTMemFree(pName);
-                return VERR_SSM_UNEXPECTED_DATA;
-            }
-            RTMemFree(pName);
-
-            bool fCaseSensitive;
-
-            rc = SSMR3GetBool(pSSM, &fCaseSensitive);
+            rc = SSMR3GetBool(pSSM, &mapping.fHostCaseSensitive);
             AssertRCReturn(rc, rc);
-            if (FolderMapping[i].fHostCaseSensitive != fCaseSensitive)
-                return VERR_SSM_UNEXPECTED_DATA;
 
-            rc = SSMR3GetBool(pSSM, &FolderMapping[i].fGuestCaseSensitive);
+            rc = SSMR3GetBool(pSSM, &mapping.fGuestCaseSensitive);
+            AssertRCReturn(rc, rc);
+
+            mapping.pFolderName = pFolderName;
+            mapping.pMapName = pMapName;
+
+            /* 'i' is the root handle of the saved mapping. */
+            rc = vbsfMappingLoaded (&mapping, i);
+
+            RTMemFree(pMapName);
+            RTMemFree(pFolderName);
+
             AssertRCReturn(rc, rc);
         }
     }
@@ -325,7 +314,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
 
                 /* Verify parameters values. */
                 if (   (fu32Flags & ~SHFL_MF_UTF8) != 0
-                    || (cbMappings < sizeof (SHFLMAPPING) * cMappings)
+                    || cbMappings / sizeof (SHFLMAPPING) < cMappings
                    )
                 {
                     rc = VERR_INVALID_PARAMETER;
@@ -340,7 +329,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
 
                     rc = vbsfMappingsQuery (pClient, pMappings, &cMappings);
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         paParms[1].u.uint32 = cMappings;
@@ -371,12 +360,9 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 /* Fetch parameters. */
                 SHFLROOT  root         = (SHFLROOT)paParms[0].u.uint32;
                 SHFLSTRING *pString    = (SHFLSTRING *)paParms[1].u.pointer.addr;
-                uint32_t cbString      = paParms[1].u.pointer.size;
 
                 /* Verify parameters values. */
-                if (   (cbString < sizeof (SHFLSTRING))
-                    || (cbString < pString->u16Size)
-                   )
+                if (!ShflStringIsValid(pString, paParms[1].u.pointer.size))
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -385,7 +371,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                     /* Execute the function. */
                     rc = vbsfMappingsQueryName (pClient, root, pString);
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         ; /* none */
@@ -422,7 +408,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t cbParms        = paParms[2].u.pointer.size;
 
                 /* Verify parameters values. */
-                if (   (cbPath < sizeof (SHFLSTRING))
+                if (   !ShflStringIsValid(pPath, cbPath)
                     || (cbParms != sizeof (SHFLCREATEPARMS))
                    )
                 {
@@ -436,7 +422,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
 
                     rc = vbsfCreate (pClient, root, pPath, cbPath, pParms);
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         ; /* none */
@@ -484,7 +470,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
 
                     rc = vbsfClose (pClient, root, Handle);
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         ; /* none */
@@ -524,7 +510,9 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint8_t   *pBuffer = (uint8_t *)paParms[4].u.pointer.addr;
 
                 /* Verify parameters values. */
-                if (Handle == SHFL_HANDLE_ROOT)
+                if (   Handle == SHFL_HANDLE_ROOT
+                    || count > paParms[4].u.pointer.size
+                   )
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -547,7 +535,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                     if (pStatusLed)
                         pStatusLed->Actual.s.fReading = 0;
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         paParms[3].u.uint32 = count;
@@ -589,7 +577,9 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint8_t   *pBuffer = (uint8_t *)paParms[4].u.pointer.addr;
 
                 /* Verify parameters values. */
-                if (Handle == SHFL_HANDLE_ROOT)
+                if (   Handle == SHFL_HANDLE_ROOT
+                    || count > paParms[4].u.pointer.size
+                   )
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -612,7 +602,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                     if (pStatusLed)
                         pStatusLed->Actual.s.fWriting = 0;
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         paParms[3].u.uint32 = count;
@@ -672,13 +662,27 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                      *       completed, the another thread must call
                      *
                      *           g_pHelpers->pfnCallComplete (callHandle, rc);
+                     *
+                     * The operation is async.
+                     * fAsynchronousProcessing = true;
                      */
 
-                    /* Operation is async. */
-                    fAsynchronousProcessing = true;
+                    /* Here the operation must be posted to another thread. At the moment it is not implemented.
+                     * Until it is implemented, try to perform the operation without waiting.
+                     */
+                    flags &= ~SHFL_LOCK_WAIT;
 
-                    /* Here the operation must be posted to another thread. At the moment it is not implemented. */
-                    rc = VERR_NOT_SUPPORTED;
+                    /* Execute the function. */
+                    if ((flags & SHFL_LOCK_MODE_MASK) == SHFL_LOCK_CANCEL)
+                        rc = vbsfUnlock(pClient, root, Handle, offset, length, flags);
+                    else
+                        rc = vbsfLock(pClient, root, Handle, offset, length, flags);
+
+                    if (VBOX_SUCCESS(rc))
+                    {
+                        /* Update parameters.*/
+                        /* none */
+                    }
                 }
                 else
                 {
@@ -688,7 +692,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                     else
                         rc = vbsfLock(pClient, root, Handle, offset, length, flags);
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         /* none */
@@ -734,6 +738,8 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
 
                 /* Verify parameters values. */
                 if (   (length < sizeof (SHFLDIRINFO))
+                    ||  length > paParms[5].u.pointer.size
+                    ||  !ShflStringIsValidOrNull(pPath, paParms[4].u.pointer.size)
                    )
                 {
                     rc = VERR_INVALID_PARAMETER;
@@ -755,7 +761,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                     if (rc == VERR_NO_MORE_FILES && cFiles != 0)
                         rc = VINF_SUCCESS; /* Successfully return these files. */
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         paParms[3].u.uint32 = length;
@@ -797,13 +803,21 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 SHFLROOT    root       = (SHFLROOT)paParms[1].u.uint32;
                 RTUTF16     delimiter  = (RTUTF16)paParms[2].u.uint32;
 
-                /* Execute the function. */
-                rc = vbsfMapFolder (pClient, pszMapName, delimiter, false,  &root);
-
-                if (VBOX_SUCCESS(rc))
+                /* Verify parameters values. */
+                if (!ShflStringIsValid(pszMapName, paParms[0].u.pointer.size))
                 {
-                    /* Update parameters.*/
-                    paParms[1].u.uint32 = root;
+                    rc = VERR_INVALID_PARAMETER;
+                }
+                else
+                {
+                    /* Execute the function. */
+                    rc = vbsfMapFolder (pClient, pszMapName, delimiter, false,  &root);
+
+                    if (RT_SUCCESS(rc))
+                    {
+                        /* Update parameters.*/
+                        paParms[1].u.uint32 = root;
+                    }
                 }
             }
             break;
@@ -840,13 +854,22 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 RTUTF16     delimiter  = (RTUTF16)paParms[2].u.uint32;
                 bool        fCaseSensitive = !!paParms[3].u.uint32;
 
-                /* Execute the function. */
-                rc = vbsfMapFolder (pClient, pszMapName, delimiter, fCaseSensitive, &root);
-
-                if (VBOX_SUCCESS(rc))
+                /* Verify parameters values. */
+                if (!ShflStringIsValid(pszMapName, paParms[0].u.pointer.size))
                 {
-                    /* Update parameters.*/
-                    paParms[1].u.uint32 = root;
+                    rc = VERR_INVALID_PARAMETER;
+                }
+                else
+                {
+
+                    /* Execute the function. */
+                    rc = vbsfMapFolder (pClient, pszMapName, delimiter, fCaseSensitive, &root);
+
+                    if (RT_SUCCESS(rc))
+                    {
+                        /* Update parameters.*/
+                        paParms[1].u.uint32 = root;
+                    }
                 }
             }
             LogRel(("SharedFolders host service: map operation result %Rrc.\n", rc));
@@ -879,7 +902,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 /* Execute the function. */
                 rc = vbsfUnmapFolder (pClient, root);
 
-                if (VBOX_SUCCESS(rc))
+                if (RT_SUCCESS(rc))
                 {
                     /* Update parameters.*/
                     /* nothing */
@@ -918,20 +941,28 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t   length  = paParms[3].u.uint32;
                 uint8_t   *pBuffer = (uint8_t *)paParms[4].u.pointer.addr;
 
-                /* Execute the function. */
-                if (flags & SHFL_INFO_SET)
-                    rc = vbsfSetFSInfo (pClient, root, Handle, flags, &length, pBuffer);
-                else /* SHFL_INFO_GET */
-                    rc = vbsfQueryFSInfo (pClient, root, Handle, flags, &length, pBuffer);
-
-                if (VBOX_SUCCESS(rc))
+                /* Verify parameters values. */
+                if (length > paParms[4].u.pointer.size)
                 {
-                    /* Update parameters.*/
-                    paParms[3].u.uint32 = length;
+                    rc = VERR_INVALID_PARAMETER;
                 }
                 else
                 {
-                    paParms[3].u.uint32 = 0;  /* nothing read */
+                    /* Execute the function. */
+                    if (flags & SHFL_INFO_SET)
+                        rc = vbsfSetFSInfo (pClient, root, Handle, flags, &length, pBuffer);
+                    else /* SHFL_INFO_GET */
+                        rc = vbsfQueryFSInfo (pClient, root, Handle, flags, &length, pBuffer);
+
+                    if (RT_SUCCESS(rc))
+                    {
+                        /* Update parameters.*/
+                        paParms[3].u.uint32 = length;
+                    }
+                    else
+                    {
+                        paParms[3].u.uint32 = 0;  /* nothing read */
+                    }
                 }
             }
             break;
@@ -963,8 +994,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 uint32_t flags          = paParms[2].u.uint32;
 
                 /* Verify parameters values. */
-                if (   (cbPath < sizeof (SHFLSTRING))
-                   )
+                if (!ShflStringIsValid(pPath, cbPath))
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
@@ -973,7 +1003,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                     /* Execute the function. */
 
                     rc = vbsfRemove (pClient, root, pPath, cbPath, flags);
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         ; /* none */
@@ -1005,14 +1035,12 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 /* Fetch parameters. */
                 SHFLROOT    root        = (SHFLROOT)paParms[0].u.uint32;
                 SHFLSTRING *pSrc        = (SHFLSTRING *)paParms[1].u.pointer.addr;
-                uint32_t    cbSrc       = paParms[1].u.pointer.size;
                 SHFLSTRING *pDest       = (SHFLSTRING *)paParms[2].u.pointer.addr;
-                uint32_t    cbDest      = paParms[2].u.pointer.size;
                 uint32_t    flags       = paParms[3].u.uint32;
 
                 /* Verify parameters values. */
-                if (   (cbSrc < sizeof (SHFLSTRING))
-                    || (cbDest < sizeof (SHFLSTRING))
+                if (    !ShflStringIsValid(pSrc, paParms[1].u.pointer.size)
+                    ||  !ShflStringIsValid(pDest, paParms[2].u.pointer.size)
                    )
                 {
                     rc = VERR_INVALID_PARAMETER;
@@ -1021,7 +1049,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 {
                     /* Execute the function. */
                     rc = vbsfRename (pClient, root, pSrc, pDest, flags);
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Update parameters.*/
                         ; /* none */
@@ -1070,7 +1098,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
 
                     rc = vbsfFlush (pClient, root, Handle);
 
-                    if (VBOX_SUCCESS(rc))
+                    if (RT_SUCCESS(rc))
                     {
                         /* Nothing to do */
                     }
@@ -1092,12 +1120,12 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
         }
     }
 
-    LogFlow(("svcCall: rc = %Vrc\n", rc));
+    LogFlow(("svcCall: rc = %Rrc\n", rc));
 
     if (   !fAsynchronousProcessing
-        || VBOX_FAILURE (rc))
+        || RT_FAILURE (rc))
     {
-        /* Complete the operation if it was unsuccessful or 
+        /* Complete the operation if it was unsuccessful or
          * it was processed synchronously.
          */
         g_pHelpers->pfnCallComplete (callHandle, rc);
@@ -1152,16 +1180,12 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
         {
             /* Fetch parameters. */
             SHFLSTRING *pFolderName = (SHFLSTRING *)paParms[0].u.pointer.addr;
-            uint32_t cbStringFolder = paParms[0].u.pointer.size;
             SHFLSTRING *pMapName    = (SHFLSTRING *)paParms[1].u.pointer.addr;
-            uint32_t cbStringMap    = paParms[1].u.pointer.size;
             uint32_t fWritable      = paParms[2].u.uint32;
 
             /* Verify parameters values. */
-            if (   (cbStringFolder < sizeof (SHFLSTRING))
-                || (cbStringFolder < pFolderName->u16Size)
-                || (cbStringMap < sizeof (SHFLSTRING))
-                || (cbStringMap < pMapName->u16Size)
+            if (    !ShflStringIsValid(pFolderName, paParms[0].u.pointer.size)
+                ||  !ShflStringIsValid(pMapName, paParms[1].u.pointer.size)
                )
             {
                 rc = VERR_INVALID_PARAMETER;
@@ -1171,7 +1195,7 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
                 /* Execute the function. */
                 rc = vbsfMappingsAdd (pFolderName, pMapName, fWritable);
 
-                if (VBOX_SUCCESS(rc))
+                if (RT_SUCCESS(rc))
                 {
                     /* Update parameters.*/
                     ; /* none */
@@ -1201,13 +1225,10 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
         else
         {
             /* Fetch parameters. */
-            SHFLSTRING *pString    = (SHFLSTRING *)paParms[0].u.pointer.addr;
-            uint32_t cbString      = paParms[0].u.pointer.size;
+            SHFLSTRING *pString = (SHFLSTRING *)paParms[0].u.pointer.addr;
 
             /* Verify parameters values. */
-            if (   (cbString < sizeof (SHFLSTRING))
-                || (cbString < pString->u16Size)
-               )
+            if (!ShflStringIsValid(pString, paParms[0].u.pointer.size))
             {
                 rc = VERR_INVALID_PARAMETER;
             }
@@ -1216,7 +1237,7 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
                 /* Execute the function. */
                 rc = vbsfMappingsRemove (pString);
 
-                if (VBOX_SUCCESS(rc))
+                if (RT_SUCCESS(rc))
                 {
                     /* Update parameters.*/
                     ; /* none */
@@ -1268,7 +1289,7 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
         break;
     }
 
-    LogFlow(("svcHostCall: rc = %Vrc\n", rc));
+    LogFlow(("svcHostCall: rc = %Rrc\n", rc));
     return rc;
 }
 
@@ -1312,6 +1333,8 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
         /* Init handle table */
         rc = vbsfInitHandleTable();
         AssertRC(rc);
+
+        vbsfMappingInit();
     }
 
     return rc;
