@@ -1,23 +1,25 @@
 /** @file
- *
- * Shared Folders:
- * Main header. Common data and function prototypes definitions.
+ * Shared Folders: Main header - Common data and function prototypes definitions.
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2007 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
  */
 
-#ifndef __SHFL__H
-#define __SHFL__H
+#ifndef ___SHFL_H
+#define ___SHFL_H
 
 #include <VBox/err.h>
 #include <VBox/hgcmsvc.h>
@@ -48,8 +50,9 @@ typedef struct _SHFLCLIENTDATA
     /** Client flags */
     uint32_t fu32Flags;
 
-    RTUCS2   PathDelimiter;
+    RTUTF16  PathDelimiter;
 } SHFLCLIENTDATA;
 
 
-#endif /* __SHFL__H */
+#endif /* !___SHFL_H */
+
