@@ -32,6 +32,7 @@
 #include <iprt/path.h>
 #include <iprt/string.h>
 #include <iprt/uni.h>
+#include <iprt/stream.h>
 #ifdef RT_OS_DARWIN
 #include <Carbon/Carbon.h>
 #endif
@@ -39,6 +40,11 @@
 #undef LogFlow
 #define LogFlow Log
 
+/**
+ * @todo find a better solution for supporting the execute bit for non-windows
+ * guests on windows host. Search for "0111" to find all the relevant places.
+ */
+
 void vbsfStripLastComponent (char *pszFullPath, uint32_t cbFullPathRoot)
 {
     RTUNICP cp;
@@ -136,7 +142,7 @@ static int vbsfCorrectCasing(char *pszFullPath, char *pszStartComponent)
     {
         size_t cbDirEntrySize = cbDirEntry;
 
-        rc = RTDirReadEx(hSearch, pDirEntry, &cbDirEntrySize, RTFSOBJATTRADD_NOTHING);
+        rc = RTDirReadEx(hSearch, pDirEntry, &cbDirEntrySize, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
         if (rc == VERR_NO_MORE_FILES)
             break;
 
@@ -172,8 +178,72 @@ end:
     return rc;
 }
 
-static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pPath,
-                              uint32_t cbPath, char **ppszFullPath, uint32_t *pcbFullPathRoot, bool fWildCard = false)
+static int vbsfPathCheck(const char *pUtf8Path, size_t cbPath)
+{
+    int rc = VINF_SUCCESS;
+
+    /* The pUtf8Path is what the guest sent. Verify that the path is within the root.
+     * Count '..' and other path components and check that we do not go over the root.
+     */
+
+    size_t i = 0;
+    int cComponents = 0; /* How many normal path components. */
+    int cParentDirs = 0; /* How many '..' components. */
+
+    for (;;)
+    {
+        /* Skip leading path delimiters. */
+        while (   i < cbPath
+               && (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'))
+            i++;
+
+        if (i >= cbPath)
+            break;
+
+        /* Check if that is a dot component. */
+        int cDots = 0;
+        while (i < cbPath && pUtf8Path[i] == '.')
+        {
+            cDots++;
+            i++;
+        }
+
+        if (   cDots >= 2 /* Consider all multidots sequences as a 'parent dir'. */
+            && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
+        {
+            cParentDirs++;
+        }
+        else if (   cDots == 1
+                 && (i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/')))
+        {
+            /* Single dot, nothing changes. */
+        }
+        else
+        {
+            /* Skip this component. */
+            while (   i < cbPath
+                   && (pUtf8Path[i] != '\\' && pUtf8Path[i] != '/'))
+                i++;
+
+            cComponents++;
+        }
+
+        Assert(i >= cbPath || (pUtf8Path[i] == '\\' || pUtf8Path[i] == '/'));
+
+        /* Verify counters for every component. */
+        if (cParentDirs > cComponents)
+        {
+            rc = VERR_INVALID_NAME;
+            break;
+        }
+    }
+
+    return rc;
+}
+
+static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, PSHFLSTRING pPath,
+                              uint32_t cbPath, char **ppszFullPath, uint32_t *pcbFullPathRoot,
+                              bool fWildCard = false, bool fPreserveLastComponent = false)
 {
     int rc = VINF_SUCCESS;
 
@@ -191,10 +261,16 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
 
     if (BIT_FLAG(pClient->fu32Flags, SHFL_CF_UTF8))
     {
-        int rc;
         char *utf8Root;
 
-        rc = RTUtf16ToUtf8 (pwszRoot, &utf8Root);
+        /* Verify that the path is under the root directory. */
+        rc = vbsfPathCheck((const char *)&pPath->String.utf8[0], pPath->u16Length);
+
+        if (RT_SUCCESS (rc))
+        {
+            rc = RTUtf16ToUtf8 (pwszRoot, &utf8Root);
+        }
+
         if (RT_SUCCESS (rc))
         {
             size_t cbUtf8Root, cbUtf8FullPath;
@@ -339,14 +415,26 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
                 {
                     AssertFailed();
 #ifdef RT_OS_DARWIN
-                	RTMemFree(pPath);
-                	pPath = pPathParameter;
+                    RTMemFree(pPath);
+                    pPath = pPathParameter;
 #endif
                     return rc;
                 }
 
                 uint32_t l = (uint32_t)strlen (dst);
 
+                /* Verify that the path is under the root directory. */
+                rc = vbsfPathCheck(dst, l);
+
+                if (RT_FAILURE(rc))
+                {
+#ifdef RT_OS_DARWIN
+                    RTMemFree(pPath);
+                    pPath = pPathParameter;
+#endif
+                    return rc;
+                }
+
                 cb -= l;
                 dst += l;
 
@@ -369,11 +457,11 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
             &&  !vbsfIsGuestMappingCaseSensitive(root))
         {
             RTFSOBJINFO info;
-            char *pszWildCardComponent = NULL;
+            char *pszLastComponent = NULL;
 
-            if (fWildCard)
+            if (fWildCard || fPreserveLastComponent)
             {
-                /* strip off the last path component, that contains the wildcard(s) */
+                /* strip off the last path component, that has to be preserved: contains the wildcard(s) or a 'rename' target. */
                 uint32_t len = (uint32_t)strlen(pszFullPath);
                 char    *src = pszFullPath + len - 1;
 
@@ -399,16 +487,16 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
                         temp++;
                     }
 
-                    if (fHaveWildcards)
+                    if (fHaveWildcards || fPreserveLastComponent)
                     {
-                        pszWildCardComponent = src;
-                        *pszWildCardComponent = 0;
+                        pszLastComponent = src;
+                        *pszLastComponent = 0;
                     }
                 }
             }
 
             /** @todo don't check when creating files or directories; waste of time */
-            rc = RTPathQueryInfo(pszFullPath, &info, RTFSOBJATTRADD_NOTHING);
+            rc = RTPathQueryInfoEx(pszFullPath, &info, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
             if (rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND)
             {
                 uint32_t len = (uint32_t)strlen(pszFullPath);
@@ -422,7 +510,7 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
                     if (*src == RTPATH_DELIMITER)
                     {
                         *src = 0;
-                        rc = RTPathQueryInfo (pszFullPath, &info, RTFSOBJATTRADD_NOTHING);
+                        rc = RTPathQueryInfoEx(pszFullPath, &info, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
                         *src = RTPATH_DELIMITER;
                         if (rc == VINF_SUCCESS)
                         {
@@ -458,7 +546,7 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
                         {
                             fEndOfString = false;
                             *end = 0;
-                            rc = RTPathQueryInfo(src, &info, RTFSOBJATTRADD_NOTHING);
+                            rc = RTPathQueryInfoEx(src, &info, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
                             Assert(rc == VINF_SUCCESS || rc == VERR_FILE_NOT_FOUND || rc == VERR_PATH_NOT_FOUND);
                         }
                         else
@@ -492,8 +580,8 @@ static int vbsfBuildFullPath (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING
                     rc = VERR_FILE_NOT_FOUND;
 
             }
-            if (pszWildCardComponent)
-                *pszWildCardComponent = RTPATH_DELIMITER;
+            if (pszLastComponent)
+                *pszLastComponent = RTPATH_DELIMITER;
 
             /* might be a new file so don't fail here! */
             rc = VINF_SUCCESS;
@@ -516,13 +604,53 @@ static void vbsfFreeFullPath (char *pszFullPath)
  *
  * @returns iprt status code
  * @param  fShflFlags shared folder create flags
+ * @param  fMode      file attibutes
  * @retval pfOpen     iprt create flags
  */
-static int vbsfConvertFileOpenFlags(unsigned fShflFlags, unsigned *pfOpen)
+static int vbsfConvertFileOpenFlags(unsigned fShflFlags, RTFMODE fMode, SHFLHANDLE handleInitial, uint32_t *pfOpen)
 {
-    unsigned fOpen = 0;
+    uint32_t fOpen = 0;
     int rc = VINF_SUCCESS;
 
+    if (   (fMode & RTFS_DOS_MASK) != 0
+        && (fMode & RTFS_UNIX_MASK) == 0)
+    {
+        /* A DOS/Windows guest, make RTFS_UNIX_* from RTFS_DOS_*.
+         * @todo this is based on rtFsModeNormalize/rtFsModeFromDos.
+         *       May be better to use RTFsModeNormalize here.
+         */
+        fMode |= RTFS_UNIX_IRUSR | RTFS_UNIX_IRGRP | RTFS_UNIX_IROTH;
+        /* x for directories. */
+        if (fMode & RTFS_DOS_DIRECTORY)
+            fMode |= RTFS_TYPE_DIRECTORY | RTFS_UNIX_IXUSR | RTFS_UNIX_IXGRP | RTFS_UNIX_IXOTH;
+        /* writable? */
+        if (!(fMode & RTFS_DOS_READONLY))
+            fMode |= RTFS_UNIX_IWUSR | RTFS_UNIX_IWGRP | RTFS_UNIX_IWOTH;
+
+        /* Set the requested mode using only allowed bits. */
+        fOpen |= ((fMode & RTFS_UNIX_MASK) << RTFILE_O_CREATE_MODE_SHIFT) & RTFILE_O_CREATE_MODE_MASK;
+    }
+    else
+    {
+        /* Old linux and solaris additions did not initialize the Info.Attr.fMode field
+         * and it contained random bits from stack. Detect this using the handle field value
+         * passed from the guest: old additions set it (incorrectly) to 0, new additions
+         * set it to SHFL_HANDLE_NIL(~0).
+         */
+        if (handleInitial == 0)
+        {
+            /* Old additions. Do nothing, use default mode. */
+        }
+        else
+        {
+            /* New additions or Windows additions. Set the requested mode using only allowed bits.
+             * Note: Windows guest set RTFS_UNIX_MASK bits to 0, which means a default mode
+             *       will be set in fOpen.
+             */
+            fOpen |= ((fMode & RTFS_UNIX_MASK) << RTFILE_O_CREATE_MODE_SHIFT) & RTFILE_O_CREATE_MODE_MASK;
+        }
+    }
+
     switch (BIT_FLAG(fShflFlags, SHFL_CF_ACCESS_MASK_RW))
     {
         default:
@@ -556,6 +684,43 @@ static int vbsfConvertFileOpenFlags(unsigned fShflFlags, unsigned *pfOpen)
         }
     }
 
+    if (fShflFlags & SHFL_CF_ACCESS_APPEND)
+    {
+        fOpen |= RTFILE_O_APPEND;
+    }
+
+    switch (BIT_FLAG(fShflFlags, SHFL_CF_ACCESS_MASK_ATTR))
+    {
+        default:
+        case SHFL_CF_ACCESS_ATTR_NONE:
+        {
+            fOpen |= RTFILE_O_ACCESS_ATTR_DEFAULT;
+            Log(("FLAG: SHFL_CF_ACCESS_ATTR_NONE\n"));
+            break;
+        }
+
+        case SHFL_CF_ACCESS_ATTR_READ:
+        {
+            fOpen |= RTFILE_O_ACCESS_ATTR_READ;
+            Log(("FLAG: SHFL_CF_ACCESS_ATTR_READ\n"));
+            break;
+        }
+
+        case SHFL_CF_ACCESS_ATTR_WRITE:
+        {
+            fOpen |= RTFILE_O_ACCESS_ATTR_WRITE;
+            Log(("FLAG: SHFL_CF_ACCESS_ATTR_WRITE\n"));
+            break;
+        }
+
+        case SHFL_CF_ACCESS_ATTR_READWRITE:
+        {
+            fOpen |= RTFILE_O_ACCESS_ATTR_READWRITE;
+            Log(("FLAG: SHFL_CF_ACCESS_ATTR_READWRITE\n"));
+            break;
+        }
+    }
+
     /* Sharing mask */
     switch (BIT_FLAG(fShflFlags, SHFL_CF_ACCESS_MASK_DENY))
     {
@@ -681,11 +846,11 @@ static int vbsfOpenFile (const char *pszPath, SHFLCREATEPARMS *pParms)
     SHFLHANDLE      handle = SHFL_HANDLE_NIL;
     SHFLFILEHANDLE *pHandle = 0;
     /* Open or create a file. */
-    unsigned fOpen = 0;
+    uint32_t fOpen = 0;
     bool fNoError = false;
     static int cErrors;
 
-    int rc = vbsfConvertFileOpenFlags(pParms->CreateFlags, &fOpen);
+    int rc = vbsfConvertFileOpenFlags(pParms->CreateFlags, pParms->Info.Attr.fMode, pParms->Handle, &fOpen);
     if (RT_SUCCESS(rc))
     {
         handle  = vbsfAllocFileHandle();
@@ -721,8 +886,11 @@ static int vbsfOpenFile (const char *pszPath, SHFLCREATEPARMS *pParms)
             RTFSOBJINFO info;
 
             /** @todo Possible race left here. */
-            if (RT_SUCCESS(RTPathQueryInfo (pszPath, &info, RTFSOBJATTRADD_NOTHING)))
+            if (RT_SUCCESS(RTPathQueryInfoEx(pszPath, &info, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK)))
             {
+#ifdef RT_OS_WINDOWS
+                info.Attr.fMode |= 0111;
+#endif
                 pParms->Info = info;
             }
             pParms->Result = SHFL_FILE_EXISTS;
@@ -796,6 +964,9 @@ static int vbsfOpenFile (const char *pszPath, SHFLCREATEPARMS *pParms)
         rc = RTFileQueryInfo (pHandle->file.Handle, &info, RTFSOBJATTRADD_NOTHING);
         if (RT_SUCCESS(rc))
         {
+#ifdef RT_OS_WINDOWS
+            info.Attr.fMode |= 0111;
+#endif
             pParms->Info = info;
         }
     }
@@ -812,6 +983,7 @@ static int vbsfOpenFile (const char *pszPath, SHFLCREATEPARMS *pParms)
         {
             vbsfFreeFileHandle (handle);
         }
+        pParms->Handle = SHFL_HANDLE_NIL;
     }
     else
     {
@@ -926,6 +1098,7 @@ static int vbsfOpenDir (const char *pszPath, SHFLCREATEPARMS *pParms)
         {
             vbsfFreeFileHandle (handle);
         }
+        pParms->Handle = SHFL_HANDLE_NIL;
     }
     else
     {
@@ -986,13 +1159,16 @@ static int vbsfLookupFile(char *pszPath, SHFLCREATEPARMS *pParms)
     RTFSOBJINFO info;
     int rc;
 
-    rc = RTPathQueryInfo (pszPath, &info, RTFSOBJATTRADD_NOTHING);
+    rc = RTPathQueryInfoEx(pszPath, &info, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
     LogFlow(("SHFL_CF_LOOKUP\n"));
     /* Client just wants to know if the object exists. */
     switch (rc)
     {
         case VINF_SUCCESS:
         {
+#ifdef RT_OS_WINDOWS
+            info.Attr.fMode |= 0111;
+#endif
             pParms->Info = info;
             pParms->Result = SHFL_FILE_EXISTS;
             break;
@@ -1012,6 +1188,7 @@ static int vbsfLookupFile(char *pszPath, SHFLCREATEPARMS *pParms)
             break;
         }
     }
+    pParms->Handle = SHFL_HANDLE_NIL;
     return rc;
 }
 
@@ -1059,9 +1236,11 @@ int vbsfCreate (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pPath, uint3
 
     if (RT_SUCCESS (rc))
     {
-        /* Reset return values in case client forgot to do so. */
+        /* Reset return value in case client forgot to do so.
+         * pParms->Handle must not be reset here, as it is used
+         * in vbsfOpenFile to detect old additions.
+         */
         pParms->Result = SHFL_NO_RESULT;
-        pParms->Handle = SHFL_HANDLE_NIL;
 
         if (BIT_FLAG(pParms->CreateFlags, SHFL_CF_LOOKUP))
         {
@@ -1072,8 +1251,8 @@ int vbsfCreate (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pPath, uint3
             /* Query path information. */
             RTFSOBJINFO info;
 
-            rc = RTPathQueryInfo (pszFullPath, &info, RTFSOBJATTRADD_NOTHING);
-            LogFlow(("RTPathQueryInfo returned %Rrc\n", rc));
+            rc = RTPathQueryInfoEx(pszFullPath, &info, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
+            LogFlow(("RTPathQueryInfoEx returned %Rrc\n", rc));
 
             if (RT_SUCCESS(rc))
             {
@@ -1105,11 +1284,16 @@ int vbsfCreate (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pPath, uint3
 
             rc = VINF_SUCCESS;
 
-            /* write access requested? */
-            if (pParms->CreateFlags & (  SHFL_CF_ACT_REPLACE_IF_EXISTS
-                                       | SHFL_CF_ACT_OVERWRITE_IF_EXISTS
-                                       | SHFL_CF_ACT_CREATE_IF_NEW
-                                       | SHFL_CF_ACCESS_WRITE))
+            /* Note: do not check the SHFL_CF_ACCESS_WRITE here, only check if the open operation
+             * will cause changes.
+             *
+             * Actual operations (write, set attr, etc), which can write to a shared folder, have
+             * the check and will return VERR_WRITE_PROTECT if the folder is not writable.
+             */
+            if (   (pParms->CreateFlags & SHFL_CF_ACT_MASK_IF_EXISTS) == SHFL_CF_ACT_REPLACE_IF_EXISTS
+                || (pParms->CreateFlags & SHFL_CF_ACT_MASK_IF_EXISTS) == SHFL_CF_ACT_OVERWRITE_IF_EXISTS
+                || (pParms->CreateFlags & SHFL_CF_ACT_MASK_IF_NEW) == SHFL_CF_ACT_CREATE_IF_NEW
+               )
             {
                 /* is the guest allowed to write to this share? */
                 bool fWritable;
@@ -1129,6 +1313,10 @@ int vbsfCreate (SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pPath, uint3
                     rc = vbsfOpenFile (pszFullPath, pParms);
                 }
             }
+            else
+            {
+                pParms->Handle = SHFL_HANDLE_NIL;
+            }
         }
 
         /* free the path string */
@@ -1261,8 +1449,8 @@ int vbsfFlush(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle)
     return rc;
 }
 
-int vbsfDirList(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle, SHFLSTRING *pPath, uint32_t flags, uint32_t *pcbBuffer, uint8_t *pBuffer,
-                uint32_t *pIndex, uint32_t *pcFiles)
+int vbsfDirList(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle, SHFLSTRING *pPath, uint32_t flags,
+                uint32_t *pcbBuffer, uint8_t *pBuffer, uint32_t *pIndex, uint32_t *pcFiles)
 {
     SHFLFILEHANDLE *pHandle = (SHFLFILEHANDLE *)vbsfQueryHandle(Handle, SHFL_HF_TYPE_DIR);
     PRTDIRENTRYEX  pDirEntry = 0, pDirEntryOrg;
@@ -1342,7 +1530,7 @@ int vbsfDirList(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle, SHFLS
         {
             pDirEntry = pDirEntryOrg;
 
-            rc = RTDirReadEx(DirHandle, pDirEntry, &cbDirEntrySize, RTFSOBJATTRADD_NOTHING);
+            rc = RTDirReadEx(DirHandle, pDirEntry, &cbDirEntrySize, RTFSOBJATTRADD_NOTHING, RTPATH_F_FOLLOW_LINK);
             if (rc == VERR_NO_MORE_FILES)
             {
                 *pIndex = 0; /* listing completed */
@@ -1379,6 +1567,9 @@ int vbsfDirList(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle, SHFLS
             return VINF_SUCCESS;    /* Return directly and don't free pDirEntry */
         }
 
+#ifdef RT_OS_WINDOWS
+        pDirEntry->Info.Attr.fMode |= 0111;
+#endif
         pSFDEntry->Info = pDirEntry->Info;
         pSFDEntry->cucShortName = 0;
 
@@ -1486,6 +1677,10 @@ int vbsfQueryFileInfo(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Handle,
     else
     {
         rc = RTFileQueryInfo(pHandle->file.Handle, pObjInfo, RTFSOBJATTRADD_NOTHING);
+#ifdef RT_OS_WINDOWS
+        if (RT_SUCCESS(rc) && RTFS_IS_FILE(pObjInfo->Attr.fMode))
+            pObjInfo->Attr.fMode |= 0111;
+#endif
     }
     if (rc == VINF_SUCCESS)
     {
@@ -1549,15 +1744,25 @@ static int vbsfSetFileInfo(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE Ha
         /* Change file attributes if necessary */
         if (pSFDEntry->Attr.fMode)
         {
-            rc = RTFileSetMode((RTFILE)pHandle->file.Handle, pSFDEntry->Attr.fMode);
+            RTFMODE fMode = pSFDEntry->Attr.fMode;
+
+#ifndef RT_OS_WINDOWS
+            /* Don't allow the guest to clear the own bit, otherwise the guest wouldn't be
+             * able to access this file anymore. Only for guests, which set the UNIX mode. */
+            if (fMode & RTFS_UNIX_MASK)
+                fMode |= RTFS_UNIX_IRUSR;
+#endif
+
+            rc = RTFileSetMode((RTFILE)pHandle->file.Handle, fMode);
             if (rc != VINF_SUCCESS)
             {
-                Log(("RTFileSetMode %x failed with %Rrc\n", pSFDEntry->Attr.fMode, rc));
+                Log(("RTFileSetMode %x failed with %Rrc\n", fMode, rc));
                 /* silent failure, because this tends to fail with e.g. windows guest & linux host */
                 rc = VINF_SUCCESS;
             }
         }
     }
+    /* TODO: mode for directories */
 
     if (rc == VINF_SUCCESS)
     {
@@ -1608,6 +1813,9 @@ static int vbsfSetEndOfFile(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLHANDLE H
         rc = RTFileQueryInfo(pHandle->file.Handle, &fileinfo, RTFSOBJATTRADD_NOTHING);
         if (rc == VINF_SUCCESS)
         {
+#ifdef RT_OS_WINDOWS
+            fileinfo.Attr.fMode |= 0111;
+#endif
             *pSFDEntry = fileinfo;
             *pcbBuffer = sizeof(RTFSOBJINFO);
         }
@@ -1865,7 +2073,7 @@ int vbsfRename(SHFLCLIENTDATA *pClient, SHFLROOT root, SHFLSTRING *pSrc, SHFLSTR
     if (rc != VINF_SUCCESS)
         return rc;
 
-    rc = vbsfBuildFullPath (pClient, root, pDest, pDest->u16Size, &pszFullPathDest, NULL);
+    rc = vbsfBuildFullPath (pClient, root, pDest, pDest->u16Size, &pszFullPathDest, NULL, false, true);
     if (RT_SUCCESS (rc))
     {
         Log(("Rename %s to %s\n", pszFullPathSrc, pszFullPathDest));
