@@ -13,6 +13,7 @@
 #include "cr_mem.h"
 #include "cr_hash.h"
 #include "server_dispatch.h"
+#include "state/cr_texture.h"
 #include <signal.h>
 #include <stdlib.h>
 #define DEBUG_FP_EXCEPTIONS 0
@@ -117,8 +118,6 @@ static void crServerTearDown( void )
     crSPUUnloadChain(cr_server.head_spu);
     cr_server.head_spu = NULL;
 #endif
-
-    crUnloadOpenGL();
 }
 
 static void crServerClose( unsigned int id )
@@ -319,14 +318,21 @@ GLboolean crVBoxServerInit(void)
     return GL_TRUE;
 }
 
-void crVBoxServerAddClient(uint32_t u32ClientID)
+int32_t crVBoxServerAddClient(uint32_t u32ClientID)
 {
-    CRClient *newClient = (CRClient *) crCalloc(sizeof(CRClient));
-    
+    CRClient *newClient;
+
+    if (cr_server.numClients>=CR_MAX_CLIENTS)
+    {
+        return VERR_MAX_THRDS_REACHED;
+    }
+
+    newClient = (CRClient *) crCalloc(sizeof(CRClient));    
     crDebug("crServer: AddClient u32ClientID=%d", u32ClientID);
 
     newClient->spu_id = 0;
     newClient->currentCtx = cr_server.DummyContext;
+    newClient->currentContextNumber = -1;
     newClient->conn = crNetAcceptClient(cr_server.protocol, NULL,
                                         cr_server.tcpip_port,
                                         cr_server.mtu, 0);
@@ -335,6 +341,8 @@ void crVBoxServerAddClient(uint32_t u32ClientID)
     cr_server.clients[cr_server.numClients++] = newClient;
 
     crServerAddToRunQueue(newClient);
+
+    return VINF_SUCCESS;
 }
 
 void crVBoxServerRemoveClient(uint32_t u32ClientID)
@@ -359,10 +367,10 @@ void crVBoxServerRemoveClient(uint32_t u32ClientID)
     pClient->conn->Disconnect(pClient->conn);
 
     /* Let server clear client from the queue */
-    crServerServiceClients();
+    crServerDeleteClient(pClient);
 }
 
-void crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cbBuffer)
+int32_t crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cbBuffer)
 {
     CRClient *pClient;
     int32_t i;
@@ -380,6 +388,8 @@ void crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cb
     pClient = cr_server.clients[i];
     CRASSERT(pClient);
 
+    if (!pClient->conn->vMajor) return VERR_NOT_SUPPORTED;
+
     CRASSERT(pBuffer);
 
     /* This should never fire unless we start to multithread */
@@ -435,6 +445,8 @@ void crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cb
     crStateResetCurrentPointers(&cr_server.current);
 
     CRASSERT(!pClient->conn->allow_redir_ptr || crNetNumMessages(pClient->conn)==0);
+
+    return VINF_SUCCESS;
 }
 
 int32_t crVBoxServerClientRead(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t *pcbBuffer)
@@ -455,9 +467,11 @@ int32_t crVBoxServerClientRead(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t
     pClient = cr_server.clients[i];
     CRASSERT(pClient);
 
+    if (!pClient->conn->vMajor) return VERR_NOT_SUPPORTED;
+
     if (pClient->conn->cbHostBuffer > *pcbBuffer)
     {
-        crWarning("crServer: [%lx] ClientRead u32ClientID=%d FAIL, host buffer too small %d of %d",
+        crDebug("crServer: [%lx] ClientRead u32ClientID=%d FAIL, host buffer too small %d of %d",
                   crThreadID(), u32ClientID, *pcbBuffer, pClient->conn->cbHostBuffer);
 
         /* Return the size of needed buffer */
@@ -479,6 +493,33 @@ int32_t crVBoxServerClientRead(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t
     return VINF_SUCCESS;
 }
 
+int32_t crVBoxServerClientSetVersion(uint32_t u32ClientID, uint32_t vMajor, uint32_t vMinor)
+{
+    CRClient *pClient;
+    int32_t i;
+
+    for (i = 0; i < cr_server.numClients; i++)
+    {
+        if (cr_server.clients[i] && cr_server.clients[i]->conn 
+            && cr_server.clients[i]->conn->u32ClientID==u32ClientID) 
+        {
+            break;
+        }
+    }
+    pClient = cr_server.clients[i];
+    CRASSERT(pClient);
+
+    pClient->conn->vMajor = vMajor;
+    pClient->conn->vMinor = vMinor;
+
+    if (vMajor != CR_PROTOCOL_VERSION_MAJOR
+        || vMinor != CR_PROTOCOL_VERSION_MINOR)
+    {
+        return VERR_NOT_SUPPORTED;
+    }
+    else return VINF_SUCCESS;
+}
+
 int
 CRServerMain(int argc, char *argv[])
 {
@@ -556,6 +597,21 @@ static void crVBoxServerSaveContextStateCB(unsigned long key, void *data1, void
     rc = SSMR3PutMem(pSSM, &key, sizeof(key));
     CRASSERT(rc == VINF_SUCCESS);
     
+#ifdef CR_STATE_NO_TEXTURE_IMAGE_STORE
+    if (cr_server.curClient)
+    {
+        unsigned long id;
+        if (!crHashtableGetDataKey(cr_server.contextTable, pContext, &id))
+        {
+            crWarning("No client id for server ctx %d", pContext->id);
+        }
+        else
+        {
+            crServerDispatchMakeCurrent(cr_server.curClient->currentWindow, 0, id);
+        }
+    }
+#endif
+
     rc = crStateSaveContext(pContext, pSSM);
     CRASSERT(rc == VINF_SUCCESS);
 }
@@ -568,6 +624,9 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
     uint32_t ui32;
     GLboolean b;
     unsigned long key;
+#ifdef CR_STATE_NO_TEXTURE_IMAGE_STORE
+    unsigned long ctxID=-1, winID=-1;
+#endif
 
     /* We shouldn't be called if there's no clients at all*/
     CRASSERT(cr_server.numClients>0);
@@ -601,6 +660,15 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
     AssertRCReturn(rc, rc);
     crHashtableWalk(cr_server.pContextCreateInfoTable, crVBoxServerSaveCreateInfoCB, pSSM);
 
+#ifdef CR_STATE_NO_TEXTURE_IMAGE_STORE
+    /* Save current win and ctx IDs, as we'd rebind contexts when saving textures */
+    if (cr_server.curClient)
+    {
+        ctxID = cr_server.curClient->currentContextNumber;
+        winID = cr_server.curClient->currentWindow;
+    }
+#endif
+
     /* Save contexts state tracker data */
     /* @todo For now just some blind data dumps, 
      * but I've a feeling those should be saved/restored in a very strict sequence to
@@ -609,6 +677,14 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
      */
     crHashtableWalk(cr_server.contextTable, crVBoxServerSaveContextStateCB, pSSM);
 
+#ifdef CR_STATE_NO_TEXTURE_IMAGE_STORE
+    /* Restore original win and ctx IDs*/
+    if (cr_server.curClient)
+    {
+        crServerDispatchMakeCurrent(winID, 0, ctxID);
+    }
+#endif
+
     /* Save windows creation info */
     ui32 = crHashtableNumElements(cr_server.pWindowCreateInfoTable);
     rc = SSMR3PutU32(pSSM, (uint32_t) ui32);
@@ -640,6 +716,12 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
             rc = SSMR3PutU32(pSSM, pClient->conn->u32ClientID);
             AssertRCReturn(rc, rc);
 
+            rc = SSMR3PutU32(pSSM, pClient->conn->vMajor);
+            AssertRCReturn(rc, rc);
+
+            rc = SSMR3PutU32(pSSM, pClient->conn->vMinor);
+            AssertRCReturn(rc, rc);
+
             rc = SSMR3PutMem(pSSM, pClient, sizeof(*pClient));
             AssertRCReturn(rc, rc);
 
@@ -666,7 +748,7 @@ DECLEXPORT(int32_t) crVBoxServerSaveState(PSSMHANDLE pSSM)
     return VINF_SUCCESS;
 }
 
-DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM)
+DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM, uint32_t version)
 {
     int32_t  rc, i;
     uint32_t ui, uiNumElems;
@@ -791,6 +873,15 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM)
             /* If this assert fires, then we should search correct client in the list first*/
             CRASSERT(ui == pClient->conn->u32ClientID);
 
+            if (version>=4)
+            {
+                rc = SSMR3GetU32(pSSM, &pClient->conn->vMajor);
+                AssertRCReturn(rc, rc);
+
+                rc = SSMR3GetU32(pSSM, &pClient->conn->vMinor);
+                AssertRCReturn(rc, rc);
+            }
+
             rc = SSMR3GetMem(pSSM, &client, sizeof(client));
             CRASSERT(rc == VINF_SUCCESS);
 
@@ -828,8 +919,8 @@ DECLEXPORT(int32_t) crVBoxServerLoadState(PSSMHANDLE pSSM)
                 //pClient->currentWindow = winID;
             }
 
-                /* Restore client active context and window */
-                crServerDispatchMakeCurrent(winID, 0, ctxID);
+            /* Restore client active context and window */
+            crServerDispatchMakeCurrent(winID, 0, ctxID);
 
             if (0)
             {
