@@ -110,14 +110,14 @@ void crServerAddToRunQueue( CRClient *client )
 
 
 
-static void
+void
 crServerDeleteClient( CRClient *client )
 {
     int i, j;
     int32_t pos;
+    CRClient *oldclient = cr_server.curClient;
 
-    crDebug("Deleting client %p (%d msgs left)", client,
-                    crNetNumMessages(client->conn));
+    crDebug("Deleting client %p (%d msgs left)", client, crNetNumMessages(client->conn));
 
 #if 0
     if (crNetNumMessages(client->conn) > 0) {
@@ -142,6 +142,22 @@ crServerDeleteClient( CRClient *client )
         }
     }
 
+    cr_server.curClient = client;
+
+    /* Destroy any windows created by the client */
+    for (pos = 0; pos<CR_MAX_WINDOWS && client->windowList[pos]; pos++) 
+    {
+        cr_server.dispatch.WindowDestroy(client->windowList[pos]);
+    }
+
+    /* Check if we have context(s) made by this client left, could happen if client side code is lazy */
+    for (pos = 0; pos<CR_MAX_CONTEXTS && client->contextList[pos]; pos++) 
+    {
+        cr_server.dispatch.DestroyContext(client->contextList[pos]);
+    }
+
+    cr_server.curClient = oldclient;
+
     /* remove from the run queue */
     if (cr_server.run_queue)
     {
@@ -179,18 +195,6 @@ crServerDeleteClient( CRClient *client )
 
     crNetFreeConnection(client->conn);
 
-    /* Destroy any windows created by the client */
-    for (pos = 0; pos<CR_MAX_WINDOWS && client->windowList[pos]; pos++) 
-    {
-        cr_server.dispatch.WindowDestroy(client->windowList[pos]);
-    }
-
-    /* Check if we have context(s) made by this client left, could happen if client side code is lazy */
-    for (pos = 0; pos<CR_MAX_CONTEXTS && client->contextList[pos]; pos++) 
-    {
-        cr_server.dispatch.DestroyContext(client->contextList[pos]);
-    }
-
     crFree(client);
 }
 
