@@ -242,7 +242,7 @@ renderspuMakeCurrent(GLint crWindow, GLint nativeWindow, GLint ctx)
              * If the mapPending flag is set, then we should now make the window
              * visible.
              */
-            renderspu_SystemShowWindow( window, GL_TRUE );
+            /*renderspu_SystemShowWindow( window, GL_TRUE );*/
             window->mapPending = GL_FALSE;
         }
         window->everCurrent = GL_TRUE;
@@ -337,11 +337,24 @@ renderspuWindowCreate( const char *dpyName, GLint visBits )
     return window->id;
 }
 
+static void renderspuCheckCurrentCtxWindowCB(unsigned long key, void *data1, void *data2)
+{
+    ContextInfo *pCtx = (ContextInfo *) data1;
+    WindowInfo *pWindow = data2;
+    (void) key;
+
+    if (pCtx->currentWindow==pWindow)
+    {
+        renderspuMakeCurrent(0, 0, pCtx->id);
+    }    
+}
 
 static void
 RENDER_APIENTRY renderspuWindowDestroy( GLint win )
 {
     WindowInfo *window;
+    GET_CONTEXT(pOldCtx);
+
     CRASSERT(win >= 0);
     window = (WindowInfo *) crHashtableSearch(render_spu.windowTable, win);
     if (window) {
@@ -349,6 +362,19 @@ RENDER_APIENTRY renderspuWindowDestroy( GLint win )
         renderspu_SystemDestroyWindow( window );
         /* remove window info from hash table, and free it */
         crHashtableDelete(render_spu.windowTable, win, crFree);
+
+        /* check if this window is bound to some ctx. Note: window pointer is already freed here */
+        crHashtableWalk(render_spu.contextTable, renderspuCheckCurrentCtxWindowCB, window);
+
+        /* restore current context */
+        {
+            GET_CONTEXT(pNewCtx);
+            if (pNewCtx!=pOldCtx)
+            {
+                renderspuMakeCurrent(pOldCtx&&pOldCtx->currentWindow ? pOldCtx->currentWindow->id:0, 0, 
+                                     pOldCtx ? pOldCtx->id:0);
+            }
+        }
     }
     else {
         crDebug("Render SPU: Attempt to destroy invalid window (%d)", win);
@@ -1014,15 +1040,22 @@ renderspuGetString(GLenum pname)
 
 
 #if defined(DARWIN)
+# ifdef VBOX_WITH_COCOA_QT
 void renderspuFlush()
 {
-    glFlush();
+    renderspu_SystemFlush();
 }
 
 void renderspuFinish()
 {
-    glFinish();
+    renderspu_SystemFinish();
 }
+
+void renderspuBindFramebufferEXT(GLenum target, GLuint framebuffer)
+{
+    renderspu_SystemBindFramebufferEXT(target, framebuffer);
+}
+# endif
 #endif
 
 #define FILLIN( NAME, FUNC ) \
@@ -1062,8 +1095,11 @@ renderspuCreateFunctions(SPUNamedFunctionTable table[])
     FILLIN( "GetChromiumParametervCR", renderspuGetChromiumParametervCR );
     FILLIN( "GetString", renderspuGetString );
 #if defined(DARWIN)
-    FILLIN( "Finish", renderspuFinish );
+# ifdef VBOX_WITH_COCOA_QT
     FILLIN( "Flush", renderspuFlush );
+    FILLIN( "Finish", renderspuFinish );
+    FILLIN( "BindFramebufferEXT", renderspuBindFramebufferEXT );
+# endif
 #endif
     return i;
 }
