@@ -40,7 +40,8 @@
 #include <VBox/VRDPOrders.h>
 #endif /* VBOX_WITH_VRDP */
 
-class VRDPConsoleCallback : public IConsoleCallback
+class VRDPConsoleCallback :
+    VBOX_SCRIPTABLE_IMPL(IConsoleCallback)
 {
 public:
     VRDPConsoleCallback (ConsoleVRDPServer *server) :
@@ -115,12 +116,7 @@ public:
         return S_OK;
     }
 
-    STDMETHOD(OnDVDDriveChange)()
-    {
-        return S_OK;
-    }
-
-    STDMETHOD(OnFloppyDriveChange)()
+    STDMETHOD(OnMediumChange)(IMediumAttachment *aAttachment)
     {
         return S_OK;
     }
@@ -140,11 +136,21 @@ public:
         return S_OK;
     }
 
+    STDMETHOD(OnStorageControllerChange) ()
+    {
+        return S_OK;
+    }
+
     STDMETHOD(OnVRDPServerChange)()
     {
         return S_OK;
     }
 
+    STDMETHOD(OnRemoteDisplayInfoChange)()
+    {
+        return S_OK;
+    }
+
     STDMETHOD(OnUSBControllerChange)()
     {
         return S_OK;
@@ -620,12 +626,8 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
     {
         case VRDP_QP_NETWORK_PORT:
         {
+            /* This is obsolete, the VRDP server uses VRDP_QP_NETWORK_PORT_RANGE instead. */
             ULONG port = 0;
-            server->mConsole->getVRDPServer ()->COMGETTER(Port) (&port);
-            if (port == 0)
-            {
-                port = VRDP_DEFAULT_PORT;
-            }
 
             if (cbBuffer >= sizeof (uint32_t))
             {
@@ -701,6 +703,80 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
             *pcbOut = sizeof (uint32_t);
         } break;
 
+        case VRDP_QP_NETWORK_PORT_RANGE:
+        {
+            com::Bstr bstr;
+            HRESULT hrc = server->mConsole->getVRDPServer ()->COMGETTER(Ports) (bstr.asOutParam());
+
+            if (hrc != S_OK)
+            {
+                bstr = "";
+            }
+
+            if (bstr == "0")
+            {
+                bstr = "3389";
+            }
+
+            /* The server expects UTF8. */
+            com::Utf8Str portRange = bstr;
+
+            size_t cbPortRange = portRange.length () + 1;
+
+            if (cbPortRange >= 0x10000)
+            {
+                /* More than 64K seems to be an  invalid port range string. */
+                rc = VERR_TOO_MUCH_DATA;
+                break;
+            }
+
+            if ((size_t)cbBuffer >= cbPortRange)
+            {
+                if (cbPortRange > 0)
+                {
+                    if (portRange.raw())
+                    {
+                        memcpy (pvBuffer, portRange.raw(), cbPortRange);
+                    }
+                    else
+                    {
+                        /* The value is an empty string. */
+                        *(uint8_t *)pvBuffer = 0;
+                    }
+                }
+
+                rc = VINF_SUCCESS;
+            }
+            else
+            {
+                rc = VINF_BUFFER_OVERFLOW;
+            }
+
+            *pcbOut = (uint32_t)cbPortRange;
+        } break;
+
+        case VRDP_SP_NETWORK_BIND_PORT:
+        {
+            if (cbBuffer != sizeof (uint32_t))
+            {
+                rc = VERR_INVALID_PARAMETER;
+                break;
+            }
+
+            ULONG port = *(uint32_t *)pvBuffer;
+            
+            server->mVRDPBindPort = port;
+
+            rc = VINF_SUCCESS;
+
+            if (pcbOut)
+            {
+                *pcbOut = sizeof (uint32_t);
+            }
+
+            server->mConsole->onRemoteDisplayInfoChange ();
+        } break;
+
         default:
             break;
     }
@@ -951,12 +1027,12 @@ DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackInput (void *pvCallback, int t
 
                 if (server->m_fGuestWantsAbsolute)
                 {
-                    pConsole->getMouse()->PutMouseEventAbsolute (pInputPoint->x + 1, pInputPoint->y + 1, iWheel, mouseButtons);
+                    pConsole->getMouse()->PutMouseEventAbsolute (pInputPoint->x + 1, pInputPoint->y + 1, iWheel, 0 /* Horizontal wheel */, mouseButtons);
                 } else
                 {
                     pConsole->getMouse()->PutMouseEvent (pInputPoint->x - server->m_mousex,
                                                          pInputPoint->y - server->m_mousey,
-                                                         iWheel, mouseButtons);
+                                                         iWheel, 0 /* Horizontal wheel */, mouseButtons);
                     server->m_mousex = pInputPoint->x;
                     server->m_mousey = pInputPoint->y;
                 }
@@ -1058,6 +1134,8 @@ ConsoleVRDPServer::ConsoleVRDPServer (Console *console)
     mConsoleCallback = new VRDPConsoleCallback(this);
     mConsoleCallback->AddRef();
     console->RegisterCallback(mConsoleCallback);
+
+    mVRDPBindPort = -1;
 #endif /* VBOX_WITH_VRDP */
 
     mAuthLibrary = 0;
@@ -1105,7 +1183,7 @@ int ConsoleVRDPServer::Launch (void)
     HRESULT rc2 = vrdpserver->COMGETTER(Enabled) (&vrdpEnabled);
     AssertComRC(rc2);
 
-    if (SUCCEEDED (rc2) && vrdpEnabled)
+    if (SUCCEEDED(rc2) && vrdpEnabled)
     {
         if (loadVRDPLibrary ())
         {
@@ -1272,19 +1350,19 @@ void ConsoleVRDPServer::remoteUSBThreadStart (void)
 {
     int rc = RTSemEventCreate (&mUSBBackends.event);
 
-    if (RT_FAILURE (rc))
+    if (RT_FAILURE(rc))
     {
         AssertFailed ();
         mUSBBackends.event = 0;
     }
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         rc = RTThreadCreate (&mUSBBackends.thread, threadRemoteUSB, this, 65536,
                              RTTHREADTYPE_VRDP_IO, RTTHREADFLAGS_WAITABLE, "remote usb");
     }
 
-    if (RT_FAILURE (rc))
+    if (RT_FAILURE(rc))
     {
         LogRel(("Warning: could not start the remote USB thread, rc = %Rrc!!!\n", rc));
         mUSBBackends.thread = NIL_RTTHREAD;
@@ -1294,7 +1372,7 @@ void ConsoleVRDPServer::remoteUSBThreadStart (void)
         /* Wait until the thread is ready. */
         rc = RTThreadUserWait (mUSBBackends.thread, 60000);
         AssertRC (rc);
-        Assert (mUSBBackends.fThreadRunning || RT_FAILURE (rc));
+        Assert (mUSBBackends.fThreadRunning || RT_FAILURE(rc));
     }
 }
 
@@ -1358,26 +1436,32 @@ VRDPAuthResult ConsoleVRDPServer::Authenticate (const Guid &uuid, VRDPAuthGuestJ
         LogRel(("VRDPAUTH: ConsoleVRDPServer::Authenticate: loading external authentication library '%ls'\n", authLibrary.raw()));
 
         int rc = RTLdrLoad (filename.raw(), &mAuthLibrary);
-        if (RT_FAILURE (rc))
+        if (RT_FAILURE(rc))
             LogRel(("VRDPAUTH: Failed to load external authentication library. Error code: %Rrc\n", rc));
 
-        if (RT_SUCCESS (rc))
+        if (RT_SUCCESS(rc))
         {
             /* Get the entry point. */
             mpfnAuthEntry2 = NULL;
             int rc2 = RTLdrGetSymbol(mAuthLibrary, "VRDPAuth2", (void**)&mpfnAuthEntry2);
-            if (RT_FAILURE (rc2))
+            if (RT_FAILURE(rc2))
             {
-                LogRel(("VRDPAUTH: Could not resolve import '%s'. Error code: %Rrc\n", "VRDPAuth2", rc2));
+                if (rc2 != VERR_SYMBOL_NOT_FOUND)
+                {
+                    LogRel(("VRDPAUTH: Could not resolve import '%s'. Error code: %Rrc\n", "VRDPAuth2", rc2));
+                }
                 rc = rc2;
             }
 
             /* Get the entry point. */
             mpfnAuthEntry = NULL;
             rc2 = RTLdrGetSymbol(mAuthLibrary, "VRDPAuth", (void**)&mpfnAuthEntry);
-            if (RT_FAILURE (rc2))
+            if (RT_FAILURE(rc2))
             {
-                LogRel(("VRDPAUTH: Could not resolve import '%s'. Error code: %Rrc\n", "VRDPAuth", rc2));
+                if (rc2 != VERR_SYMBOL_NOT_FOUND)
+                {
+                    LogRel(("VRDPAUTH: Could not resolve import '%s'. Error code: %Rrc\n", "VRDPAuth", rc2));
+                }
                 rc = rc2;
             }
 
@@ -1388,7 +1472,7 @@ VRDPAuthResult ConsoleVRDPServer::Authenticate (const Guid &uuid, VRDPAuthGuestJ
             }
         }
 
-        if (RT_FAILURE (rc))
+        if (RT_FAILURE(rc))
         {
             mConsole->reportAuthLibraryError (filename.raw(), rc);
 
@@ -1584,13 +1668,13 @@ void ConsoleVRDPServer::ClipboardCreate (uint32_t u32ClientId)
 {
     int rc = lockConsoleVRDPServer ();
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         if (mcClipboardRefs == 0)
         {
             rc = HGCMHostRegisterServiceExtension (&mhClipboard, "VBoxSharedClipboard", ClipboardServiceExtension, this);
 
-            if (RT_SUCCESS (rc))
+            if (RT_SUCCESS(rc))
             {
                 mcClipboardRefs++;
             }
@@ -1604,7 +1688,7 @@ void ConsoleVRDPServer::ClipboardDelete (uint32_t u32ClientId)
 {
     int rc = lockConsoleVRDPServer ();
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         mcClipboardRefs--;
 
@@ -1635,7 +1719,7 @@ void ConsoleVRDPServer::USBBackendCreate (uint32_t u32ClientId, void **ppvInterc
         /* Append the new instance in the list. */
         int rc = lockConsoleVRDPServer ();
 
-        if (RT_SUCCESS (rc))
+        if (RT_SUCCESS(rc))
         {
             pRemoteUSBBackend->pNext = mUSBBackends.pHead;
             if (mUSBBackends.pHead)
@@ -1657,7 +1741,7 @@ void ConsoleVRDPServer::USBBackendCreate (uint32_t u32ClientId, void **ppvInterc
             }
         }
 
-        if (RT_FAILURE (rc))
+        if (RT_FAILURE(rc))
         {
             pRemoteUSBBackend->Release ();
         }
@@ -1675,7 +1759,7 @@ void ConsoleVRDPServer::USBBackendDelete (uint32_t u32ClientId)
     /* Find the instance. */
     int rc = lockConsoleVRDPServer ();
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         pRemoteUSBBackend = usbBackendFind (u32ClientId);
 
@@ -1704,7 +1788,7 @@ void *ConsoleVRDPServer::USBBackendRequestPointer (uint32_t u32ClientId, const G
     /* Find the instance. */
     int rc = lockConsoleVRDPServer ();
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         pRemoteUSBBackend = usbBackendFind (u32ClientId);
 
@@ -1744,7 +1828,7 @@ void ConsoleVRDPServer::USBBackendReleasePointer (const Guid *pGuid)
     /* Find the instance. */
     int rc = lockConsoleVRDPServer ();
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         pRemoteUSBBackend = usbBackendFindByUUID (pGuid);
 
@@ -1772,7 +1856,7 @@ RemoteUSBBackend *ConsoleVRDPServer::usbBackendGetNext (RemoteUSBBackend *pRemot
 
     int rc = lockConsoleVRDPServer ();
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         if (pRemoteUSBBackend == NULL)
         {
@@ -1941,7 +2025,17 @@ void ConsoleVRDPServer::SendUSBRequest (uint32_t u32ClientId, void *pvParms, uin
 void ConsoleVRDPServer::QueryInfo (uint32_t index, void *pvBuffer, uint32_t cbBuffer, uint32_t *pcbOut) const
 {
 #ifdef VBOX_WITH_VRDP
-    if (mpEntryPoints && mhServer)
+    if (index == VRDP_QI_PORT)
+    {
+        uint32_t cbOut = sizeof (int32_t);
+
+        if (cbBuffer >= cbOut)
+        {
+            *pcbOut = cbOut;
+            *(int32_t *)pvBuffer = (int32_t)mVRDPBindPort;
+        }
+    }
+    else if (mpEntryPoints && mhServer)
     {
         mpEntryPoints->VRDPQueryInfo (mhServer, index, pvBuffer, cbBuffer, pcbOut);
     }
@@ -2036,15 +2130,15 @@ void RemoteDisplayInfo::FinalRelease()
  */
 HRESULT RemoteDisplayInfo::init (Console *aParent)
 {
-    LogFlowThisFunc (("aParent=%p\n", aParent));
+    LogFlowThisFunc(("aParent=%p\n", aParent));
 
     ComAssertRet (aParent, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst (mParent) = aParent;
+    unconst(mParent) = aParent;
 
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
@@ -2058,14 +2152,14 @@ HRESULT RemoteDisplayInfo::init (Console *aParent)
  */
 void RemoteDisplayInfo::uninit()
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
     /* Enclose the state transition Ready->InUninit->NotReady */
-    AutoUninitSpan autoUninitSpan (this);
+    AutoUninitSpan autoUninitSpan(this);
     if (autoUninitSpan.uninitDone())
         return;
 
-    unconst (mParent).setNull();
+    unconst(mParent).setNull();
 }
 
 // IRemoteDisplayInfo properties
@@ -2077,11 +2171,11 @@ void RemoteDisplayInfo::uninit()
         if (!a##_aName)                                                   \
             return E_POINTER;                                             \
                                                                           \
-        AutoCaller autoCaller (this);                                     \
-        CheckComRCReturnRC (autoCaller.rc());                             \
+        AutoCaller autoCaller(this);                                     \
+        CheckComRCReturnRC(autoCaller.rc());                             \
                                                                           \
         /* todo: Not sure if a AutoReadLock would be sufficient. */       \
-        AutoWriteLock alock (this);                                       \
+        AutoWriteLock alock(this);                                       \
                                                                           \
         uint32_t value;                                                   \
         uint32_t cbOut = 0;                                               \
@@ -2100,11 +2194,11 @@ void RemoteDisplayInfo::uninit()
         if (!a##_aName)                                                   \
             return E_POINTER;                                             \
                                                                           \
-        AutoCaller autoCaller (this);                                     \
-        CheckComRCReturnRC (autoCaller.rc());                             \
+        AutoCaller autoCaller(this);                                     \
+        CheckComRCReturnRC(autoCaller.rc());                             \
                                                                           \
         /* todo: Not sure if a AutoReadLock would be sufficient. */       \
-        AutoWriteLock alock (this);                                       \
+        AutoWriteLock alock(this);                                       \
                                                                           \
         _aType value;                                                     \
         uint32_t cbOut = 0;                                               \
@@ -2123,11 +2217,11 @@ void RemoteDisplayInfo::uninit()
         if (!a##_aName)                                                   \
             return E_POINTER;                                             \
                                                                           \
-        AutoCaller autoCaller (this);                                     \
-        CheckComRCReturnRC (autoCaller.rc());                             \
+        AutoCaller autoCaller(this);                                     \
+        CheckComRCReturnRC(autoCaller.rc());                             \
                                                                           \
         /* todo: Not sure if a AutoReadLock would be sufficient. */       \
-        AutoWriteLock alock (this);                                       \
+        AutoWriteLock alock(this);                                       \
                                                                           \
         uint32_t cbOut = 0;                                               \
                                                                           \
@@ -2137,7 +2231,7 @@ void RemoteDisplayInfo::uninit()
         if (cbOut == 0)                                                   \
         {                                                                 \
             Bstr str("");                                                 \
-            str.cloneTo (a##_aName);                                      \
+            str.cloneTo(a##_aName);                                      \
             return S_OK;                                                  \
         }                                                                 \
                                                                           \
@@ -2156,7 +2250,7 @@ void RemoteDisplayInfo::uninit()
                                                                           \
         Bstr str(pchBuffer);                                              \
                                                                           \
-        str.cloneTo (a##_aName);                                          \
+        str.cloneTo(a##_aName);                                          \
                                                                           \
         RTMemTmpFree (pchBuffer);                                         \
                                                                           \
@@ -2164,6 +2258,7 @@ void RemoteDisplayInfo::uninit()
     }
 
 IMPL_GETTER_BOOL   (BOOL,    Active,             VRDP_QI_ACTIVE);
+IMPL_GETTER_SCALAR (LONG,    Port,               VRDP_QI_PORT);
 IMPL_GETTER_SCALAR (ULONG,   NumberOfClients,    VRDP_QI_NUMBER_OF_CLIENTS);
 IMPL_GETTER_SCALAR (LONG64,  BeginTime,          VRDP_QI_BEGIN_TIME);
 IMPL_GETTER_SCALAR (LONG64,  EndTime,            VRDP_QI_END_TIME);
