@@ -54,15 +54,15 @@ void ParallelPort::FinalRelease()
  */
 HRESULT ParallelPort::init (Machine *aParent, ULONG aSlot)
 {
-    LogFlowThisFunc (("aParent=%p, aSlot=%d\n", aParent, aSlot));
+    LogFlowThisFunc(("aParent=%p, aSlot=%d\n", aParent, aSlot));
 
     ComAssertRet (aParent, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst (mParent) = aParent;
+    unconst(mParent) = aParent;
     /* mPeer is left null */
 
     mData.allocate();
@@ -88,19 +88,19 @@ HRESULT ParallelPort::init (Machine *aParent, ULONG aSlot)
  */
 HRESULT ParallelPort::init (Machine *aParent, ParallelPort *aThat)
 {
-    LogFlowThisFunc (("aParent=%p, aThat=%p\n", aParent, aThat));
+    LogFlowThisFunc(("aParent=%p, aThat=%p\n", aParent, aThat));
 
     ComAssertRet (aParent && aThat, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst (mParent) = aParent;
-    unconst (mPeer) = aThat;
+    unconst(mParent) = aParent;
+    unconst(mPeer) = aThat;
 
     AutoCaller thatCaller (aThat);
-    AssertComRCReturnRC (thatCaller.rc());
+    AssertComRCReturnRC(thatCaller.rc());
 
     AutoReadLock thatLock (aThat);
     mData.share (aThat->mData);
@@ -120,19 +120,19 @@ HRESULT ParallelPort::init (Machine *aParent, ParallelPort *aThat)
  */
 HRESULT ParallelPort::initCopy (Machine *aParent, ParallelPort *aThat)
 {
-    LogFlowThisFunc (("aParent=%p, aThat=%p\n", aParent, aThat));
+    LogFlowThisFunc(("aParent=%p, aThat=%p\n", aParent, aThat));
 
     ComAssertRet (aParent && aThat, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst (mParent) = aParent;
+    unconst(mParent) = aParent;
     /* mPeer is left null */
 
     AutoCaller thatCaller (aThat);
-    AssertComRCReturnRC (thatCaller.rc());
+    AssertComRCReturnRC(thatCaller.rc());
 
     AutoReadLock thatLock (aThat);
     mData.attachCopy (aThat->mData);
@@ -149,17 +149,17 @@ HRESULT ParallelPort::initCopy (Machine *aParent, ParallelPort *aThat)
  */
 void ParallelPort::uninit()
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
     /* Enclose the state transition Ready->InUninit->NotReady */
-    AutoUninitSpan autoUninitSpan (this);
+    AutoUninitSpan autoUninitSpan(this);
     if (autoUninitSpan.uninitDone())
         return;
 
     mData.free();
 
-    unconst (mPeer).setNull();
-    unconst (mParent).setNull();
+    unconst(mPeer).setNull();
+    unconst(mParent).setNull();
 }
 
 // public methods only for internal purposes
@@ -173,16 +173,12 @@ void ParallelPort::uninit()
  *
  *  @note Locks this object for writing.
  */
-HRESULT ParallelPort::loadSettings (const settings::Key &aPortNode)
+HRESULT ParallelPort::loadSettings(const settings::ParallelPort &data)
 {
-    using namespace settings;
+    AutoCaller autoCaller(this);
+    AssertComRCReturnRC(autoCaller.rc());
 
-    AssertReturn (!aPortNode.isNull(), E_FAIL);
-
-    AutoCaller autoCaller (this);
-    AssertComRCReturnRC (autoCaller.rc());
-
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     /* Note: we assume that the default values for attributes of optional
      * nodes are assigned in the Data::Data() constructor and don't do it
@@ -196,16 +192,15 @@ HRESULT ParallelPort::loadSettings (const settings::Key &aPortNode)
      * default to B. */
 
     /* enabled (required) */
-    mData->mEnabled = aPortNode.value <bool> ("enabled");
+    mData->mEnabled = data.fEnabled;
     /* I/O base (required) */
-    mData->mIOBase = aPortNode.value <ULONG> ("IOBase");
+    mData->mIOBase = data.ulIOBase;
     /* IRQ (required) */
-    mData->mIRQ = aPortNode.value <ULONG> ("IRQ");
+    mData->mIRQ = data.ulIRQ;
     /* device path (optional, defaults to null) */
-    Bstr path = aPortNode.stringValue ("path");
-
+    Bstr path(data.strPath);
     HRESULT rc = checkSetPath (path);
-    CheckComRCReturnRC (rc);
+    CheckComRCReturnRC(rc);
     mData->mPath = path;
 
     return S_OK;
@@ -220,24 +215,17 @@ HRESULT ParallelPort::loadSettings (const settings::Key &aPortNode)
  *
  *  @note Locks this object for reading.
  */
-HRESULT ParallelPort::saveSettings (settings::Key &aPortNode)
+HRESULT ParallelPort::saveSettings(settings::ParallelPort &data)
 {
-    using namespace settings;
+    AutoCaller autoCaller(this);
+    AssertComRCReturnRC(autoCaller.rc());
 
-    AssertReturn (!aPortNode.isNull(), E_FAIL);
+    AutoReadLock alock(this);
 
-    AutoCaller autoCaller (this);
-    AssertComRCReturnRC (autoCaller.rc());
-
-    AutoReadLock alock (this);
-
-    aPortNode.setValue <bool> ("enabled", !!mData->mEnabled);
-    aPortNode.setValue <ULONG> ("IOBase", mData->mIOBase, 16);
-    aPortNode.setValue <ULONG> ("IRQ", mData->mIRQ);
-
-    /* 'path' is optional in XML */
-    if (!mData->mPath.isEmpty())
-        aPortNode.setValue <Bstr> ("path", mData->mPath);
+    data.fEnabled = !!mData->mEnabled;
+    data.ulIOBase = mData->mIOBase;
+    data.ulIRQ = mData->mIRQ;
+    data.strPath = mData->mPath;
 
     return S_OK;
 }
@@ -248,10 +236,10 @@ HRESULT ParallelPort::saveSettings (settings::Key &aPortNode)
 bool ParallelPort::rollback()
 {
     /* sanity */
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), false);
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     bool changed = false;
 
@@ -273,7 +261,7 @@ bool ParallelPort::rollback()
 void ParallelPort::commit()
 {
     /* sanity */
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturnVoid (autoCaller.rc());
 
     /* sanity too */
@@ -304,7 +292,7 @@ void ParallelPort::copyFrom (ParallelPort *aThat)
     AssertReturnVoid (aThat != NULL);
 
     /* sanity */
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturnVoid (autoCaller.rc());
 
     /* sanity too */
@@ -326,10 +314,10 @@ STDMETHODIMP ParallelPort::COMGETTER(Enabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     *aEnabled = mData->mEnabled;
 
@@ -338,16 +326,16 @@ STDMETHODIMP ParallelPort::COMGETTER(Enabled) (BOOL *aEnabled)
 
 STDMETHODIMP ParallelPort::COMSETTER(Enabled) (BOOL aEnabled)
 {
-    LogFlowThisFunc (("aEnabled=%RTbool\n", aEnabled));
+    LogFlowThisFunc(("aEnabled=%RTbool\n", aEnabled));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
     /* the machine needs to be mutable */
     Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC (adep.rc());
+    CheckComRCReturnRC(adep.rc());
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     if (mData->mEnabled != aEnabled)
     {
@@ -374,10 +362,10 @@ STDMETHODIMP ParallelPort::COMGETTER(Slot) (ULONG *aSlot)
 {
     CheckComArgOutPointerValid(aSlot);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     *aSlot = mData->mSlot;
 
@@ -388,10 +376,10 @@ STDMETHODIMP ParallelPort::COMGETTER(IRQ) (ULONG *aIRQ)
 {
     CheckComArgOutPointerValid(aIRQ);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     *aIRQ = mData->mIRQ;
 
@@ -408,14 +396,14 @@ STDMETHODIMP ParallelPort::COMSETTER(IRQ)(ULONG aIRQ)
                 "%lu (must be in range [0, %lu])"),
             mData->mSlot, aIRQ, 255);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
     /* the machine needs to be mutable */
     Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC (adep.rc());
+    CheckComRCReturnRC(adep.rc());
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     HRESULT rc = S_OK;
     bool emitChangeEvent = false;
@@ -442,10 +430,10 @@ STDMETHODIMP ParallelPort::COMGETTER(IOBase) (ULONG *aIOBase)
 {
     CheckComArgOutPointerValid(aIOBase);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     *aIOBase = mData->mIOBase;
 
@@ -462,14 +450,14 @@ STDMETHODIMP ParallelPort::COMSETTER(IOBase)(ULONG aIOBase)
                 "%lu (must be in range [0, 0x%X])"),
             mData->mSlot, aIOBase, 0, 0xFFFF);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
     /* the machine needs to be mutable */
     Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC (adep.rc());
+    CheckComRCReturnRC(adep.rc());
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     HRESULT rc = S_OK;
     bool emitChangeEvent = false;
@@ -496,12 +484,12 @@ STDMETHODIMP ParallelPort::COMGETTER(Path) (BSTR *aPath)
 {
     CheckComArgOutPointerValid(aPath);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
-    mData->mPath.cloneTo (aPath);
+    mData->mPath.cloneTo(aPath);
 
     return S_OK;
 }
@@ -511,7 +499,7 @@ STDMETHODIMP ParallelPort::COMGETTER(Path) (BSTR *aPath)
  */
 HRESULT ParallelPort::checkSetPath (CBSTR aPath)
 {
-    AssertReturn (isWriteLockOnCurrentThread(), E_FAIL);
+    AssertReturn(isWriteLockOnCurrentThread(), E_FAIL);
 
     if (mData->mEnabled &&
         (aPath == NULL || *aPath == '\0'))
@@ -525,14 +513,14 @@ HRESULT ParallelPort::checkSetPath (CBSTR aPath)
 
 STDMETHODIMP ParallelPort::COMSETTER(Path) (IN_BSTR aPath)
 {
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
     /* the machine needs to be mutable */
     Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC (adep.rc());
+    CheckComRCReturnRC(adep.rc());
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     /* we treat empty as null when e.g. saving to XML, do the same here */
     if (aPath && *aPath == '\0')
@@ -541,7 +529,7 @@ STDMETHODIMP ParallelPort::COMSETTER(Path) (IN_BSTR aPath)
     if (mData->mPath != aPath)
     {
         HRESULT rc = checkSetPath (aPath);
-        CheckComRCReturnRC (rc);
+        CheckComRCReturnRC(rc);
 
         mData.backup();
         mData->mPath = aPath;
