@@ -29,6 +29,7 @@
 
 #include "SessionImpl.h"
 #include "ConsoleImpl.h"
+#include "Global.h"
 
 #include "Logging.h"
 
@@ -49,7 +50,8 @@ static DECLCALLBACK(int) IPCMutexHolderThread (RTTHREAD Thread, void *pvUser);
     do { \
         if (mState != SessionState_Open) \
             return setError (E_UNEXPECTED, \
-                tr ("The session is not open")); \
+                tr ("The session is not open (session state: %s)"), \
+                Global::stringifySessionState(mState)); \
     } while (0)
 
 // constructor / destructor
@@ -57,14 +59,14 @@ static DECLCALLBACK(int) IPCMutexHolderThread (RTTHREAD Thread, void *pvUser);
 
 HRESULT Session::FinalConstruct()
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
     return init();
 }
 
 void Session::FinalRelease()
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
     uninit (true /* aFinalRelease */);
 }
@@ -78,8 +80,8 @@ void Session::FinalRelease()
 HRESULT Session::init()
 {
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
     LogFlowThisFuncEnter();
 
@@ -114,19 +116,19 @@ HRESULT Session::init()
 void Session::uninit (bool aFinalRelease)
 {
     LogFlowThisFuncEnter();
-    LogFlowThisFunc (("aFinalRelease=%d\n", aFinalRelease));
+    LogFlowThisFunc(("aFinalRelease=%d\n", aFinalRelease));
 
     /* Enclose the state transition Ready->InUninit->NotReady */
-    AutoUninitSpan autoUninitSpan (this);
+    AutoUninitSpan autoUninitSpan(this);
     if (autoUninitSpan.uninitDone())
     {
-        LogFlowThisFunc (("Already uninitialized.\n"));
+        LogFlowThisFunc(("Already uninitialized.\n"));
         LogFlowThisFuncLeave();
         return;
     }
 
     /* close() needs write lock */
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     if (mState != SessionState_Closed)
     {
@@ -147,10 +149,10 @@ STDMETHODIMP Session::COMGETTER(State) (SessionState_T *aState)
 {
     CheckComArgOutPointerValid(aState);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     *aState = mState;
 
@@ -161,10 +163,10 @@ STDMETHODIMP Session::COMGETTER(Type) (SessionType_T *aType)
 {
     CheckComArgOutPointerValid(aType);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     CHECK_OPEN();
 
@@ -176,19 +178,19 @@ STDMETHODIMP Session::COMGETTER(Machine) (IMachine **aMachine)
 {
     CheckComArgOutPointerValid(aMachine);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     CHECK_OPEN();
 
     HRESULT rc = E_FAIL;
 
     if (mConsole)
-        rc = mConsole->machine().queryInterfaceTo (aMachine);
+        rc = mConsole->machine().queryInterfaceTo(aMachine);
     else
-        rc = mRemoteMachine.queryInterfaceTo (aMachine);
+        rc = mRemoteMachine.queryInterfaceTo(aMachine);
     ComAssertComRC (rc);
 
     return rc;
@@ -198,19 +200,19 @@ STDMETHODIMP Session::COMGETTER(Console) (IConsole **aConsole)
 {
     CheckComArgOutPointerValid(aConsole);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     CHECK_OPEN();
 
     HRESULT rc = E_FAIL;
 
     if (mConsole)
-        rc = mConsole.queryInterfaceTo (aConsole);
+        rc = mConsole.queryInterfaceTo(aConsole);
     else
-        rc = mRemoteConsole.queryInterfaceTo (aConsole);
+        rc = mRemoteConsole.queryInterfaceTo(aConsole);
     ComAssertComRC (rc);
 
     return rc;
@@ -221,13 +223,13 @@ STDMETHODIMP Session::COMGETTER(Console) (IConsole **aConsole)
 
 STDMETHODIMP Session::Close()
 {
-    LogFlowThisFunc (("mState=%d, mType=%d\n", mState, mType));
+    LogFlowThisFunc(("mState=%d, mType=%d\n", mState, mType));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    CheckComRCReturnRC(autoCaller.rc());
 
     /* close() needs write lock */
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
     CHECK_OPEN();
 
@@ -239,12 +241,12 @@ STDMETHODIMP Session::Close()
 
 STDMETHODIMP Session::GetPID (ULONG *aPid)
 {
-    AssertReturn (aPid, E_POINTER);
+    AssertReturn(aPid, E_POINTER);
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     *aPid = (ULONG) RTProcSelf();
     AssertCompile (sizeof (*aPid) == sizeof (RTPROCESS));
@@ -255,14 +257,14 @@ STDMETHODIMP Session::GetPID (ULONG *aPid)
 STDMETHODIMP Session::GetRemoteConsole (IConsole **aConsole)
 {
     LogFlowThisFuncEnter();
-    AssertReturn (aConsole, E_POINTER);
+    AssertReturn(aConsole, E_POINTER);
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
-    AssertReturn (mState != SessionState_Closed, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mState != SessionState_Closed, VBOX_E_INVALID_VM_STATE);
 
     AssertMsgReturn (mType == SessionType_Direct && !!mConsole,
       ("This is not a direct session!\n"), VBOX_E_INVALID_OBJECT_STATE);
@@ -272,7 +274,7 @@ STDMETHODIMP Session::GetRemoteConsole (IConsole **aConsole)
     if (mState != SessionState_Open)
         return VBOX_E_INVALID_VM_STATE;
 
-    mConsole.queryInterfaceTo (aConsole);
+    mConsole.queryInterfaceTo(aConsole);
 
     LogFlowThisFuncLeave();
 
@@ -282,14 +284,14 @@ STDMETHODIMP Session::GetRemoteConsole (IConsole **aConsole)
 STDMETHODIMP Session::AssignMachine (IMachine *aMachine)
 {
     LogFlowThisFuncEnter();
-    LogFlowThisFunc (("aMachine=%p\n", aMachine));
+    LogFlowThisFunc(("aMachine=%p\n", aMachine));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
-    AssertReturn (mState == SessionState_Closed, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mState == SessionState_Closed, VBOX_E_INVALID_VM_STATE);
 
     if (!aMachine)
     {
@@ -300,7 +302,7 @@ STDMETHODIMP Session::AssignMachine (IMachine *aMachine)
          *  called.
          */
 
-        AssertReturn (mType == SessionType_Null, VBOX_E_INVALID_OBJECT_STATE);
+        AssertReturn(mType == SessionType_Null, VBOX_E_INVALID_OBJECT_STATE);
         mType = SessionType_Remote;
         mState = SessionState_Spawning;
 
@@ -312,7 +314,7 @@ STDMETHODIMP Session::AssignMachine (IMachine *aMachine)
 
     /* query IInternalMachineControl interface */
     mControl = aMachine;
-    AssertReturn (!!mControl, E_FAIL);
+    AssertReturn(!!mControl, E_FAIL);
 
     rc = mConsole.createObject();
     AssertComRCReturn (rc, rc);
@@ -326,11 +328,11 @@ STDMETHODIMP Session::AssignMachine (IMachine *aMachine)
      *  Reference the VirtualBox object to ensure the server is up
      *  until the session is closed
      */
-    if (SUCCEEDED (rc))
+    if (SUCCEEDED(rc))
        rc = aMachine->COMGETTER(Parent) (mVirtualBox.asOutParam());
 
-    if (SUCCEEDED (rc))
-    {
+    if (SUCCEEDED(rc))
+    {        
         mType = SessionType_Direct;
         mState = SessionState_Open;
     }
@@ -342,7 +344,7 @@ STDMETHODIMP Session::AssignMachine (IMachine *aMachine)
         mConsole.setNull();
     }
 
-    LogFlowThisFunc (("rc=%08X\n", rc));
+    LogFlowThisFunc(("rc=%08X\n", rc));
     LogFlowThisFuncLeave();
 
     return rc;
@@ -351,23 +353,23 @@ STDMETHODIMP Session::AssignMachine (IMachine *aMachine)
 STDMETHODIMP Session::AssignRemoteMachine (IMachine *aMachine, IConsole *aConsole)
 {
     LogFlowThisFuncEnter();
-    LogFlowThisFunc (("aMachine=%p, aConsole=%p\n", aMachine, aConsole));
+    LogFlowThisFunc(("aMachine=%p, aConsole=%p\n", aMachine, aConsole));
 
-    AssertReturn (aMachine && aConsole, E_INVALIDARG);
+    AssertReturn(aMachine && aConsole, E_INVALIDARG);
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
-    AssertReturn (mState == SessionState_Closed ||
+    AssertReturn(mState == SessionState_Closed ||
                   mState == SessionState_Spawning, VBOX_E_INVALID_VM_STATE);
 
     HRESULT rc = E_FAIL;
 
     /* query IInternalMachineControl interface */
     mControl = aMachine;
-    AssertReturn (!!mControl, E_FAIL); // This test appears to be redundant --JS
+    AssertReturn(!!mControl, E_FAIL); // This test appears to be redundant --JS
 
     /// @todo (dmik)
     //      currently, the remote session returns the same machine and
@@ -394,7 +396,7 @@ STDMETHODIMP Session::AssignRemoteMachine (IMachine *aMachine, IConsole *aConsol
      */
     rc = aMachine->COMGETTER(Parent) (mVirtualBox.asOutParam());
 
-    if (SUCCEEDED (rc))
+    if (SUCCEEDED(rc))
     {
         /*
          *  RemoteSession type can be already set by AssignMachine() when its
@@ -415,7 +417,7 @@ STDMETHODIMP Session::AssignRemoteMachine (IMachine *aMachine, IConsole *aConsol
         mRemoteConsole.setNull();
     }
 
-    LogFlowThisFunc (("rc=%08X\n", rc));
+    LogFlowThisFunc(("rc=%08X\n", rc));
     LogFlowThisFuncLeave();
 
     return rc;
@@ -423,7 +425,7 @@ STDMETHODIMP Session::AssignRemoteMachine (IMachine *aMachine, IConsole *aConsol
 
 STDMETHODIMP Session::UpdateMachineState (MachineState_T aMachineState)
 {
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
 
     if (autoCaller.state() != Ready)
     {
@@ -431,23 +433,23 @@ STDMETHODIMP Session::UpdateMachineState (MachineState_T aMachineState)
          *  We might have already entered Session::uninit() at this point, so
          *  return silently (not interested in the state change during uninit)
          */
-        LogFlowThisFunc (("Already uninitialized.\n"));
+        LogFlowThisFunc(("Already uninitialized.\n"));
         return S_OK;
     }
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
     if (mState == SessionState_Closing)
     {
-        LogFlowThisFunc (("Already being closed.\n"));
+        LogFlowThisFunc(("Already being closed.\n"));
         return S_OK;
     }
 
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
-    AssertReturn (!mControl.isNull(), E_FAIL);
-    AssertReturn (!mConsole.isNull(), E_FAIL);
+    AssertReturn(!mControl.isNull(), E_FAIL);
+    AssertReturn(!mConsole.isNull(), E_FAIL);
 
     return mConsole->updateMachineState (aMachineState);
 }
@@ -456,24 +458,24 @@ STDMETHODIMP Session::Uninitialize()
 {
     LogFlowThisFuncEnter();
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
 
     HRESULT rc = S_OK;
 
     if (autoCaller.state() == Ready)
     {
         /* close() needs write lock */
-        AutoWriteLock alock (this);
+        AutoWriteLock alock(this);
 
-        LogFlowThisFunc (("mState=%d, mType=%d\n", mState, mType));
+        LogFlowThisFunc(("mState=%s, mType=%d\n", Global::stringifySessionState(mState), mType));
 
         if (mState == SessionState_Closing)
         {
-            LogFlowThisFunc (("Already being closed.\n"));
+            LogFlowThisFunc(("Already being closed.\n"));
             return S_OK;
         }
 
-        AssertReturn (mState == SessionState_Open ||
+        AssertReturn(mState == SessionState_Open ||
                       mState == SessionState_Spawning, VBOX_E_INVALID_VM_STATE);
 
         /* close ourselves */
@@ -485,142 +487,128 @@ STDMETHODIMP Session::Uninitialize()
          *  We might have already entered Session::uninit() at this point,
          *  return silently
          */
-        LogFlowThisFunc (("Already uninitialized.\n"));
+        LogFlowThisFunc(("Already uninitialized.\n"));
     }
     else
     {
-        LogWarningThisFunc (("UNEXPECTED uninitialization!\n"));
+        LogWarningThisFunc(("UNEXPECTED uninitialization!\n"));
         rc = autoCaller.rc();
     }
 
-    LogFlowThisFunc (("rc=%08X\n", rc));
+    LogFlowThisFunc(("rc=%08X\n", rc));
     LogFlowThisFuncLeave();
 
     return rc;
 }
 
-STDMETHODIMP Session::OnDVDDriveChange()
+STDMETHODIMP Session::OnNetworkAdapterChange(INetworkAdapter *networkAdapter, BOOL changeAdapter)
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
-    return mConsole->onDVDDriveChange();
-}
-
-STDMETHODIMP Session::OnFloppyDriveChange()
-{
-    LogFlowThisFunc (("\n"));
-
-    AutoCaller autoCaller (this);
-    AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
-
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
-
-    return mConsole->onFloppyDriveChange();
-}
-
-STDMETHODIMP Session::OnNetworkAdapterChange(INetworkAdapter *networkAdapter)
-{
-    LogFlowThisFunc (("\n"));
-
-    AutoCaller autoCaller (this);
-    AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
-
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
-
-    return mConsole->onNetworkAdapterChange(networkAdapter);
+    return mConsole->onNetworkAdapterChange(networkAdapter, changeAdapter);
 }
 
 STDMETHODIMP Session::OnSerialPortChange(ISerialPort *serialPort)
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onSerialPortChange(serialPort);
 }
 
 STDMETHODIMP Session::OnParallelPortChange(IParallelPort *parallelPort)
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onParallelPortChange(parallelPort);
 }
 
 STDMETHODIMP Session::OnStorageControllerChange()
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onStorageControllerChange();
 }
 
-STDMETHODIMP Session::OnVRDPServerChange()
+STDMETHODIMP Session::OnMediumChange(IMediumAttachment *aMediumAttachment, BOOL aForce)
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+
+    return mConsole->onMediumChange(aMediumAttachment, aForce);
+}
+
+STDMETHODIMP Session::OnVRDPServerChange()
+{
+    LogFlowThisFunc(("\n"));
+
+    AutoCaller autoCaller(this);
+    AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
+
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onVRDPServerChange();
 }
 
 STDMETHODIMP Session::OnUSBControllerChange()
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onUSBControllerChange();
 }
 
 STDMETHODIMP Session::OnSharedFolderChange (BOOL aGlobal)
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onSharedFolderChange (aGlobal);
 }
@@ -629,14 +617,14 @@ STDMETHODIMP Session::OnUSBDeviceAttach (IUSBDevice *aDevice,
                                          IVirtualBoxErrorInfo *aError,
                                          ULONG aMaskedIfs)
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onUSBDeviceAttach (aDevice, aError, aMaskedIfs);
 }
@@ -644,26 +632,26 @@ STDMETHODIMP Session::OnUSBDeviceAttach (IUSBDevice *aDevice,
 STDMETHODIMP Session::OnUSBDeviceDetach (IN_BSTR aId,
                                          IVirtualBoxErrorInfo *aError)
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
-    AssertReturn (mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AutoReadLock alock(this);
+    AssertReturn(mState == SessionState_Open, VBOX_E_INVALID_VM_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     return mConsole->onUSBDeviceDetach (aId, aError);
 }
 
 STDMETHODIMP Session::OnShowWindow (BOOL aCheck, BOOL *aCanShow, ULONG64 *aWinId)
 {
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this);
 
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
 
     if (mState != SessionState_Open)
     {
@@ -683,14 +671,14 @@ STDMETHODIMP Session::AccessGuestProperty (IN_BSTR aName, IN_BSTR aValue, IN_BST
                                            BOOL aIsSetter, BSTR *aRetValue, ULONG64 *aRetTimestamp, BSTR *aRetFlags)
 {
 #ifdef VBOX_WITH_GUEST_PROPS
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
     if (mState != SessionState_Open)
         return setError (VBOX_E_INVALID_VM_STATE,
-            tr ("Machine session is not open (session state: %d)."),
-            mState);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+            tr ("Machine session is not open (session state: %s)."),
+            Global::stringifySessionState(mState));
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
     CheckComArgNotNull(aName);
     if (!aIsSetter && !VALID_PTR (aRetValue))
         return E_POINTER;
@@ -720,23 +708,23 @@ STDMETHODIMP Session::EnumerateGuestProperties (IN_BSTR aPatterns,
                                                 ComSafeArrayOut(BSTR, aFlags))
 {
 #ifdef VBOX_WITH_GUEST_PROPS
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
     AssertComRCReturn (autoCaller.rc(), autoCaller.rc());
 
     if (mState != SessionState_Open)
         return setError (VBOX_E_INVALID_VM_STATE,
-            tr ("Machine session is not open (session state: %d)."),
-            mState);
-    AssertReturn (mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
+            tr ("Machine session is not open (session state: %s)."),
+            Global::stringifySessionState(mState));
+    AssertReturn(mType == SessionType_Direct, VBOX_E_INVALID_OBJECT_STATE);
     if (!VALID_PTR (aPatterns) && (aPatterns != NULL))
         return E_POINTER;
-    if (ComSafeArrayOutIsNull (aNames))
+    if (ComSafeArrayOutIsNull(aNames))
         return E_POINTER;
-    if (ComSafeArrayOutIsNull (aValues))
+    if (ComSafeArrayOutIsNull(aValues))
         return E_POINTER;
-    if (ComSafeArrayOutIsNull (aTimestamps))
+    if (ComSafeArrayOutIsNull(aTimestamps))
         return E_POINTER;
-    if (ComSafeArrayOutIsNull (aFlags))
+    if (ComSafeArrayOutIsNull(aFlags))
         return E_POINTER;
     return mConsole->enumerateGuestProperties(aPatterns,
                                               ComSafeArrayOutArg(aNames),
@@ -763,15 +751,15 @@ STDMETHODIMP Session::EnumerateGuestProperties (IN_BSTR aPatterns,
 HRESULT Session::close (bool aFinalRelease, bool aFromServer)
 {
     LogFlowThisFuncEnter();
-    LogFlowThisFunc (("aFinalRelease=%d, isFromServer=%d\n",
+    LogFlowThisFunc(("aFinalRelease=%d, isFromServer=%d\n",
                       aFinalRelease, aFromServer));
 
-    AutoCaller autoCaller (this);
-    AssertComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    AssertComRCReturnRC(autoCaller.rc());
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this);
 
-    LogFlowThisFunc (("mState=%d, mType=%d\n", mState, mType));
+    LogFlowThisFunc(("mState=%s, mType=%d\n", Global::stringifySessionState(mState), mType));
 
     if (mState != SessionState_Open)
     {
@@ -804,7 +792,7 @@ HRESULT Session::close (bool aFinalRelease, bool aFromServer)
     }
 
     /* go to the closing state */
-    mState =  SessionState_Closing;
+    mState = SessionState_Closing;
 
     if (mType == SessionType_Direct)
     {
@@ -817,7 +805,7 @@ HRESULT Session::close (bool aFinalRelease, bool aFromServer)
         mRemoteConsole.setNull();
     }
 
-    ComPtr <IProgress> progress;
+    ComPtr<IProgress> progress;
 
     if (!aFinalRelease && !aFromServer)
     {
@@ -840,9 +828,9 @@ HRESULT Session::close (bool aFinalRelease, bool aFromServer)
          */
         alock.leave();
 
-        LogFlowThisFunc (("Calling mControl->OnSessionEnd()...\n"));
+        LogFlowThisFunc(("Calling mControl->OnSessionEnd()...\n"));
         HRESULT rc = mControl->OnSessionEnd (this, progress.asOutParam());
-        LogFlowThisFunc (("mControl->OnSessionEnd()=%08X\n", rc));
+        LogFlowThisFunc(("mControl->OnSessionEnd()=%08X\n", rc));
 
         alock.enter();
 
@@ -892,9 +880,9 @@ HRESULT Session::grabIPCSemaphore()
     /* open the IPC semaphore based on the sessionId and try to grab it */
     Bstr ipcId;
     rc = mControl->GetIPCId (ipcId.asOutParam());
-    AssertComRCReturnRC (rc);
+    AssertComRCReturnRC(rc);
 
-    LogFlowThisFunc (("ipcId='%ls'\n", ipcId.raw()));
+    LogFlowThisFunc(("ipcId='%ls'\n", ipcId.raw()));
 
 #if defined(RT_OS_WINDOWS)
 
@@ -961,10 +949,10 @@ HRESULT Session::grabIPCSemaphore()
 
     /* wait until thread init is completed */
     vrc = RTThreadUserWait (mIPCThread, RT_INDEFINITE_WAIT);
-    AssertReturn (RT_SUCCESS (vrc) || vrc == VERR_INTERRUPTED, E_FAIL);
+    AssertReturn(RT_SUCCESS(vrc) || vrc == VERR_INTERRUPTED, E_FAIL);
 
     /* the thread must succeed */
-    AssertReturn ((bool) data [2], E_FAIL);
+    AssertReturn((bool) data [2], E_FAIL);
 
 #elif defined(VBOX_WITH_SYS_V_IPC_SESSION_WATCHER)
 
@@ -1034,7 +1022,7 @@ void Session::releaseIPCSemaphore()
 
         /* wait for the thread to finish */
         vrc = RTThreadUserWait (mIPCThread, RT_INDEFINITE_WAIT);
-        Assert (RT_SUCCESS (vrc) || vrc == VERR_INTERRUPTED);
+        Assert (RT_SUCCESS(vrc) || vrc == VERR_INTERRUPTED);
 
         mIPCThread = NIL_RTTHREAD;
     }
