@@ -25,6 +25,8 @@
 #include <iprt/file.h>
 #include <iprt/s3.h>
 
+#include <VBox/com/array.h>
+
 #include <VBox/param.h>
 #include <VBox/version.h>
 
@@ -242,7 +244,7 @@ int VFSExplorer::TaskVFSExplorer::uploadProgress(unsigned uPercent, void *pvUser
         pTask->progress->COMGETTER(Canceled)(&fCanceled);
         if (fCanceled)
             return -1;
-        pTask->progress->setCurrentOperationProgress(uPercent);
+        pTask->progress->SetCurrentOperationProgress(uPercent);
     }
     return VINF_SUCCESS;
 }
@@ -289,7 +291,7 @@ HRESULT VFSExplorer::updateFS(TaskVFSExplorer *aTask)
             throw setError(VBOX_E_FILE_ERROR, tr ("Can't open directory '%s' (%Rrc)"), pszPath, vrc);
 
         if(aTask->progress)
-            aTask->progress->setCurrentOperationProgress(33);
+            aTask->progress->SetCurrentOperationProgress(33);
         RTDIRENTRY entry;
         while (RT_SUCCESS(vrc))
         {
@@ -303,7 +305,7 @@ HRESULT VFSExplorer::updateFS(TaskVFSExplorer *aTask)
             }
         }
         if(aTask->progress)
-            aTask->progress->setCurrentOperationProgress(66);
+            aTask->progress->SetCurrentOperationProgress(66);
     }
     catch(HRESULT aRC)
     {
@@ -317,7 +319,7 @@ HRESULT VFSExplorer::updateFS(TaskVFSExplorer *aTask)
         RTDirClose(pDir);
 
     if(aTask->progress)
-        aTask->progress->setCurrentOperationProgress(99);
+        aTask->progress->SetCurrentOperationProgress(99);
 
     /* Assign the result on success (this clears the old list) */
     if (rc == S_OK)
@@ -362,7 +364,7 @@ HRESULT VFSExplorer::deleteFS(TaskVFSExplorer *aTask)
             if (RT_FAILURE(vrc))
                 throw setError(VBOX_E_FILE_ERROR, tr ("Can't delete file '%s' (%Rrc)"), szPath, vrc);
             if(aTask->progress)
-                aTask->progress->setCurrentOperationProgress((ULONG)(fPercentStep * i));
+                aTask->progress->SetCurrentOperationProgress((ULONG)(fPercentStep * i));
         }
     }
     catch(HRESULT aRC)
@@ -420,7 +422,7 @@ HRESULT VFSExplorer::updateS3(TaskVFSExplorer *aTask)
         else
         {
             PCRTS3KEYENTRY pKeys = NULL;
-            vrc = RTS3GetBucketKeys(hS3, m->strBucket, &pKeys);
+            vrc = RTS3GetBucketKeys(hS3, m->strBucket.c_str(), &pKeys);
             if (RT_FAILURE(vrc))
                 throw setError(E_FAIL, tr ("Can't get keys for bucket (%Rrc)"), vrc);
 
@@ -488,7 +490,7 @@ HRESULT VFSExplorer::deleteS3(TaskVFSExplorer *aTask)
             if (RT_FAILURE(vrc))
                 throw setError(VBOX_E_FILE_ERROR, tr ("Can't delete file '%s' (%Rrc)"), (*it).c_str(), vrc);
             if(aTask->progress)
-                aTask->progress->setCurrentOperationProgress((ULONG)(fPercentStep * i));
+                aTask->progress->SetCurrentOperationProgress((ULONG)(fPercentStep * i));
         }
     }
     catch(HRESULT aRC)
@@ -622,7 +624,7 @@ STDMETHODIMP VFSExplorer::Exists(ComSafeArrayIn(IN_BSTR, aNames), ComSafeArrayOu
         const VFSExplorer::Data::DirEntry &entry = (*it);
         for (size_t a=0; a < sfaNames.size(); ++a)
         {
-            if (entry.name == RTPathFilename(Utf8Str(sfaNames[a])))
+            if (entry.name == RTPathFilename(Utf8Str(sfaNames[a]).c_str()))
             {
                 BSTR name;
                 Bstr tmp(sfaNames[a]); /* gcc-3.3 cruft */
