@@ -1,6 +1,5 @@
 /** @file
- *
- * VirtualBox Driver Interface to VMM device
+ * VirtualBox Driver Interface to VMM device.
  */
 
 /*
@@ -27,8 +26,7 @@
 #include "Logging.h"
 
 #include <VBox/pdmdrv.h>
-#include <VBox/VBoxDev.h>
-#include <VBox/VBoxGuest.h>
+#include <VBox/VMMDev.h>
 #include <VBox/shflsvc.h>
 #include <iprt/asm.h>
 
@@ -103,6 +101,13 @@ VMMDev::VMMDev(Console *console) : mpDrv(NULL)
 
 VMMDev::~VMMDev()
 {
+#ifdef VBOX_WITH_HGCM
+    if (hgcmIsActive())
+    {
+        ASMAtomicWriteBool(&m_fHGCMActive, false);
+        HGCMHostShutdown();
+    }
+#endif /* VBOX_WITH_HGCM */
     RTSemEventDestroy (mCredentialsEvent);
     if (mpDrv)
         mpDrv->pVMMDev = NULL;
@@ -133,7 +138,7 @@ int VMMDev::WaitCredentialsJudgement (uint32_t u32Timeout, uint32_t *pu32Credent
 
     int rc = RTSemEventWait (mCredentialsEvent, u32Timeout);
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         *pu32CredentialsFlags = mu32CredentialsFlags;
     }
@@ -246,8 +251,8 @@ DECLCALLBACK(void) vmmdevUpdateMouseCapabilities(PPDMIVMMDEVCONNECTOR pInterface
      * Tell the console interface about the event
      * so that it can notify its consumers.
      */
-    pDrv->pVMMDev->getParent()->onMouseCapabilityChange(BOOL (newCapabilities & VMMDEV_MOUSEGUESTWANTSABS),
-                                                        BOOL (newCapabilities & VMMDEV_MOUSEGUESTNEEDSHOSTCUR));
+    pDrv->pVMMDev->getParent()->onMouseCapabilityChange(BOOL (newCapabilities & VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE),
+                                                        BOOL (newCapabilities & VMMDEV_MOUSE_GUEST_NEEDS_HOST_CURSOR));
 }
 
 
@@ -602,14 +607,16 @@ static DECLCALLBACK(int) iface_hgcmSave(PPDMDRVINS pDrvIns, PSSMHANDLE pSSM)
  * @returns VBox status code.
  * @param   pDrvIns         Driver instance of the driver which registered the data unit.
  * @param   pSSM            SSM operation handle.
- * @param   u32Version      Data layout version.
+ * @param   uVersion        Data layout version.
+ * @param   uPass           The data pass.
  */
-static DECLCALLBACK(int) iface_hgcmLoad(PPDMDRVINS pDrvIns, PSSMHANDLE pSSM, uint32_t u32Version)
+static DECLCALLBACK(int) iface_hgcmLoad(PPDMDRVINS pDrvIns, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
 {
     LogFlowFunc(("Enter\n"));
 
-    if (u32Version != HGCM_SSM_VERSION)
+    if (uVersion != HGCM_SSM_VERSION)
         return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
+    Assert(uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     return HGCMHostLoadState (pSSM);
 }
@@ -705,14 +712,9 @@ DECLCALLBACK(void) VMMDev::drvReset(PPDMDRVINS pDrvIns)
 /**
  * Construct a VMMDev driver instance.
  *
- * @returns VBox status.
- * @param   pDrvIns     The driver instance data.
- *                      If the registration structure is needed, pDrvIns->pDrvReg points to it.
- * @param   pCfgHandle  Configuration node handle for the driver. Use this to obtain the configuration
- *                      of the driver instance. It's also found in pDrvIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @copydoc FNPDMDRVCONSTRUCT
  */
-DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
+DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags)
 {
     PDRVMAINVMMDEV pData = PDMINS_2_DATA(pDrvIns, PDRVMAINVMMDEV);
     LogFlow(("Keyboard::drvConstruct: iInstance=%d\n", pDrvIns->iInstance));
@@ -722,14 +724,9 @@ DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
      */
     if (!CFGMR3AreValuesValid(pCfgHandle, "Object\0"))
         return VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
-
-    PPDMIBASE pBaseIgnore;
-    int rc = pDrvIns->pDrvHlp->pfnAttach(pDrvIns, &pBaseIgnore);
-    if (rc != VERR_PDM_NO_ATTACHED_DRIVER)
-    {
-        AssertMsgFailed(("Configuration error: Not possible to attach anything to this driver!\n"));
-        return VERR_PDM_DRVINS_NO_ATTACH;
-    }
+    AssertMsgReturn(PDMDrvHlpNoAttach(pDrvIns) == VERR_PDM_NO_ATTACHED_DRIVER,
+                    ("Configuration error: Not possible to attach anything to this driver!\n"),
+                    VERR_PDM_DRVINS_NO_ATTACH);
 
     /*
      * IBase.
@@ -780,7 +777,7 @@ DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
      * Get the Console object pointer and update the mpDrv member.
      */
     void *pv;
-    rc = CFGMR3QueryPtr(pCfgHandle, "Object", &pv);
+    int rc = CFGMR3QueryPtr(pCfgHandle, "Object", &pv);
     if (RT_FAILURE(rc))
     {
         AssertMsgFailed(("Configuration error: No/bad \"Object\" value! rc=%Rrc\n", rc));
@@ -823,7 +820,13 @@ DECLCALLBACK(int) VMMDev::drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
     else
         LogRel(("Failed to load Shared Folders service %Rrc\n", rc));
 
-    pDrvIns->pDrvHlp->pfnSSMRegister(pDrvIns, "HGCM", 0, HGCM_SSM_VERSION, 4096/* bad guess */, NULL, iface_hgcmSave, NULL, NULL, iface_hgcmLoad, NULL);
+    rc = PDMDrvHlpSSMRegisterEx(pDrvIns, HGCM_SSM_VERSION, 4096 /* bad guess */,
+                                NULL, NULL, NULL,
+                                NULL, iface_hgcmSave, NULL,
+                                NULL, iface_hgcmLoad, NULL);
+    if (RT_FAILURE(rc))
+        return rc;
+
 #endif /* VBOX_WITH_HGCM */
 
     return VINF_SUCCESS;
@@ -838,7 +841,7 @@ const PDMDRVREG VMMDev::DrvReg =
     /* u32Version */
     PDM_DRVREG_VERSION,
     /* szDriverName */
-    "MainVMMDev",
+    "HGCM",
     /* pszDescription */
     "Main VMMDev driver (Main as in the API).",
     /* fFlags */
@@ -863,7 +866,15 @@ const PDMDRVREG VMMDev::DrvReg =
     NULL,
     /* pfnResume */
     NULL,
+    /* pfnAttach */
+    NULL,
     /* pfnDetach */
-    NULL
+    NULL,
+    /* pfnPowerOff */
+    NULL,
+    /* pfnSoftReset */
+    NULL,
+    /* u32EndVersion */
+    PDM_DRVREG_VERSION
 };
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
