@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -126,12 +126,6 @@ RWLockHandle *VirtualBoxBaseProto::lockHandle() const
  * until the AutoInitSpan destructor signals that it has finished
  * initialization.
  *
- * Also, addCaller() will block if the object is probing uninitialization on
- * another thread with AutoMayUninitSpan (i.e. the object state is MayUninit).
- * And again, the block will last until the AutoMayUninitSpan destructor signals
- * that it has finished probing and the object is either ready again or will
- * uninitialize shortly (so that addCaller() will fail).
- *
  * When this method returns a failure, the caller must not use the object
  * and should return the failed result code to its own caller.
  *
@@ -163,31 +157,25 @@ HRESULT VirtualBoxBaseProto::addCaller (State *aState /* = NULL */,
         rc = S_OK;
     }
     else
-    if (mState == InInit || mState == MayUninit || mState == InUninit)
+    if (mState == InInit || mState == InUninit)
     {
         if (mStateChangeThread == RTThreadSelf())
         {
             /* Called from the same thread that is doing AutoInitSpan or
-             * AutoUninitSpan or AutoMayUninitSpan, just succeed */
+             * AutoUninitSpan, just succeed */
             rc = S_OK;
         }
-        else if (mState == InInit || mState == MayUninit)
+        else if (mState == InInit)
         {
-            /* One of the two:
+            /* addCaller() is called by a "child" thread while the "parent"
+             * thread is still doing AutoInitSpan/AutoReinitSpan, so wait for
+             * the state to become either Ready/Limited or InitFailed (in
+             * case of init failure).
              *
-             * 1) addCaller() is called by a "child" thread while the "parent"
-             *    thread is still doing AutoInitSpan/AutoReinitSpan, so wait for
-             *    the state to become either Ready/Limited or InitFailed (in
-             *    case of init failure).
-             *
-             * 2) addCaller() is called while another thread is in
-             *    AutoMayUninitSpan, so wait for the state to become either
-             *    Ready or WillUninit.
-             *
-             * Note that in either case we increase the number of callers anyway
-             * -- to prevent AutoUninitSpan from early completion if we are
-             * still not scheduled to pick up the posted semaphore when uninit()
-             * is called.
+             * Note that we increase the number of callers anyway -- to
+             * prevent AutoUninitSpan from early completion if we are
+             * still not scheduled to pick up the posted semaphore when
+             * uninit() is called.
              */
             ++ mCallers;
 
@@ -200,10 +188,7 @@ HRESULT VirtualBoxBaseProto::addCaller (State *aState /* = NULL */,
 
             ++ mInitUninitWaiters;
 
-            LogFlowThisFunc ((mState == InInit ?
-                              "Waiting for AutoInitSpan/AutoReinitSpan to "
-                              "finish...\n" :
-                              "Waiting for AutoMayUninitSpan to finish...\n"));
+            LogFlowThisFunc(("Waiting for AutoInitSpan/AutoReinitSpan to finish...\n"));
 
             stateLock.leave();
             RTSemEventMultiWait (mInitUninitSem, RT_INDEFINITE_WAIT);
@@ -256,19 +241,18 @@ void VirtualBoxBaseProto::releaseCaller()
         return;
     }
 
-    if (mState == InInit || mState == MayUninit || mState == InUninit)
+    if (mState == InInit || mState == InUninit)
     {
         if (mStateChangeThread == RTThreadSelf())
         {
-            /* Called from the same thread that is doing AutoInitSpan,
-             * AutoMayUninitSpan or AutoUninitSpan: just succeed */
+            /* Called from the same thread that is doing AutoInitSpan or
+             * AutoUninitSpan: just succeed */
             return;
         }
 
-        if (mState == MayUninit || mState == InUninit)
+        if (mState == InUninit)
         {
-            /* the caller is being released after AutoUninitSpan or
-             * AutoMayUninitSpan has begun */
+            /* the caller is being released after AutoUninitSpan has begun */
             AssertMsgReturn (mCallers != 0, ("mCallers is ZERO!"), (void) 0);
             -- mCallers;
 
@@ -529,113 +513,7 @@ VirtualBoxBaseProto::AutoUninitSpan::~AutoUninitSpan()
     mObj->setState (NotReady);
 }
 
-// VirtualBoxBaseProto::AutoMayUninitSpan methods
 ////////////////////////////////////////////////////////////////////////////////
-
-/**
- * Creates a smart initialization span object that places the object to
- * MayUninit state.
- *
- * Please see the AutoMayUninitSpan class description for more info.
- *
- * @param aObj      |this| pointer of the managed VirtualBoxBase object whose
- *                  uninit() method to be probably called.
- */
-VirtualBoxBaseProto::AutoMayUninitSpan::
-AutoMayUninitSpan (VirtualBoxBaseProto *aObj)
-    : mObj (aObj), mRC (E_FAIL), mAlreadyInProgress (false)
-    , mAcceptUninit (false)
-{
-    Assert (aObj);
-
-    AutoWriteLock stateLock (mObj->mStateLock);
-
-    AssertReturnVoid (mObj->mState != InInit &&
-                      mObj->mState != InUninit);
-
-    switch (mObj->mState)
-    {
-        case Ready:
-            break;
-        case MayUninit:
-            /* Nothing to be done if already in MayUninit. */
-            mAlreadyInProgress = true;
-            mRC = S_OK;
-            return;
-        default:
-            /* Abuse mObj->addCaller() to get the extended error info possibly
-             * set by reimplementations of addCaller() and return it to the
-             * caller. Note that this abuse is supposed to be safe because we
-             * should've filtered out all states where addCaller() would do
-             * something else but set error info. */
-            mRC = mObj->addCaller();
-            Assert (FAILED (mRC));
-            return;
-    }
-
-    /* go to MayUninit to cause new callers to wait until we finish */
-    mObj->setState (MayUninit);
-    mRC = S_OK;
-
-    /* wait for already existing callers to drop to zero */
-    if (mObj->mCallers > 0)
-    {
-        /* lazy creation */
-        Assert (mObj->mZeroCallersSem == NIL_RTSEMEVENT);
-        RTSemEventCreate (&mObj->mZeroCallersSem);
-
-        /* wait until remaining callers release the object */
-        LogFlowFunc (("{%p}: Waiting for callers (%d) to drop to zero...\n",
-                      mObj, mObj->mCallers));
-
-        stateLock.leave();
-        RTSemEventWait (mObj->mZeroCallersSem, RT_INDEFINITE_WAIT);
-    }
-}
-
-/**
- * Places the managed VirtualBoxBase object back to Ready state if
- * #acceptUninit() was not called, or places it to WillUninit state and calls
- * the object's uninit() method.
- *
- * Please see the AutoMayUninitSpan class description for more info.
- */
-VirtualBoxBaseProto::AutoMayUninitSpan::~AutoMayUninitSpan()
-{
-    /* if we did nothing in the constructor, do nothing here */
-    if (mAlreadyInProgress || FAILED (mRC))
-        return;
-
-    AutoWriteLock stateLock (mObj->mStateLock);
-
-    Assert (mObj->mState == MayUninit);
-
-    if (mObj->mCallers > 0)
-    {
-        Assert (mObj->mInitUninitWaiters > 0);
-
-        /* We have some pending addCaller() calls on other threads made after
-         * going to during MayUnit, signal that MayUnit is finished and they may
-         * go on. */
-        RTSemEventMultiSignal (mObj->mInitUninitSem);
-    }
-
-    if (!mAcceptUninit)
-    {
-        mObj->setState (Ready);
-    }
-    else
-    {
-        mObj->setState (WillUninit);
-        /* leave the lock to prevent nesting when uninit() is called */
-        stateLock.leave();
-        /* call uninit() to let the object uninit itself */
-        mObj->uninit();
-        /* Note: the object may no longer exist here (for example, it can call
-         * the destructor in uninit()) */
-    }
-}
-
 // VirtualBoxBase methods
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -787,10 +665,10 @@ HRESULT VirtualBoxSupportErrorInfoImplBase::setErrorInternal (
                  preserve));
 
     /* these are mandatory, others -- not */
-    AssertReturn ((!aWarning && FAILED (aResultCode)) ||
+    AssertReturn((!aWarning && FAILED (aResultCode)) ||
                   (aWarning && aResultCode != S_OK),
                   E_FAIL);
-    AssertReturn (!aText.isEmpty(), E_FAIL);
+    AssertReturn(!aText.isEmpty(), E_FAIL);
 
     /* reset the error severity bit if it's a warning */
     if (aWarning)
@@ -800,88 +678,88 @@ HRESULT VirtualBoxSupportErrorInfoImplBase::setErrorInternal (
 
     do
     {
-        ComObjPtr <VirtualBoxErrorInfo> info;
+        ComObjPtr<VirtualBoxErrorInfo> info;
         rc = info.createObject();
         CheckComRCBreakRC (rc);
 
 #if !defined (VBOX_WITH_XPCOM)
 
-        ComPtr <IVirtualBoxErrorInfo> curInfo;
+        ComPtr<IVirtualBoxErrorInfo> curInfo;
         if (preserve)
         {
             /* get the current error info if any */
-            ComPtr <IErrorInfo> err;
+            ComPtr<IErrorInfo> err;
             rc = ::GetErrorInfo (0, err.asOutParam());
             CheckComRCBreakRC (rc);
-            rc = err.queryInterfaceTo (curInfo.asOutParam());
+            rc = err.queryInterfaceTo(curInfo.asOutParam());
             if (FAILED (rc))
             {
                 /* create a IVirtualBoxErrorInfo wrapper for the native
                  * IErrorInfo object */
-                ComObjPtr <VirtualBoxErrorInfo> wrapper;
+                ComObjPtr<VirtualBoxErrorInfo> wrapper;
                 rc = wrapper.createObject();
-                if (SUCCEEDED (rc))
+                if (SUCCEEDED(rc))
                 {
                     rc = wrapper->init (err);
-                    if (SUCCEEDED (rc))
+                    if (SUCCEEDED(rc))
                         curInfo = wrapper;
                 }
             }
         }
         /* On failure, curInfo will stay null */
-        Assert (SUCCEEDED (rc) || curInfo.isNull());
+        Assert (SUCCEEDED(rc) || curInfo.isNull());
 
         /* set the current error info and preserve the previous one if any */
         rc = info->init (aResultCode, aIID, aComponent, aText, curInfo);
         CheckComRCBreakRC (rc);
 
-        ComPtr <IErrorInfo> err;
-        rc = info.queryInterfaceTo (err.asOutParam());
-        if (SUCCEEDED (rc))
+        ComPtr<IErrorInfo> err;
+        rc = info.queryInterfaceTo(err.asOutParam());
+        if (SUCCEEDED(rc))
             rc = ::SetErrorInfo (0, err);
 
 #else // !defined (VBOX_WITH_XPCOM)
 
         nsCOMPtr <nsIExceptionService> es;
         es = do_GetService (NS_EXCEPTIONSERVICE_CONTRACTID, &rc);
-        if (NS_SUCCEEDED (rc))
+        if (NS_SUCCEEDED(rc))
         {
             nsCOMPtr <nsIExceptionManager> em;
             rc = es->GetCurrentExceptionManager (getter_AddRefs (em));
             CheckComRCBreakRC (rc);
 
-            ComPtr <IVirtualBoxErrorInfo> curInfo;
+            ComPtr<IVirtualBoxErrorInfo> curInfo;
             if (preserve)
             {
                 /* get the current error info if any */
-                ComPtr <nsIException> ex;
+                ComPtr<nsIException> ex;
                 rc = em->GetCurrentException (ex.asOutParam());
                 CheckComRCBreakRC (rc);
-                rc = ex.queryInterfaceTo (curInfo.asOutParam());
+                rc = ex.queryInterfaceTo(curInfo.asOutParam());
                 if (FAILED (rc))
                 {
                     /* create a IVirtualBoxErrorInfo wrapper for the native
                      * nsIException object */
-                    ComObjPtr <VirtualBoxErrorInfo> wrapper;
+                    ComObjPtr<VirtualBoxErrorInfo> wrapper;
                     rc = wrapper.createObject();
-                    if (SUCCEEDED (rc))
+                    if (SUCCEEDED(rc))
                     {
                         rc = wrapper->init (ex);
-                        if (SUCCEEDED (rc))
+                        if (SUCCEEDED(rc))
                             curInfo = wrapper;
                     }
                 }
             }
             /* On failure, curInfo will stay null */
-            Assert (SUCCEEDED (rc) || curInfo.isNull());
+            Assert (SUCCEEDED(rc) || curInfo.isNull());
 
             /* set the current error info and preserve the previous one if any */
             rc = info->init (aResultCode, aIID, aComponent, aText, curInfo);
             CheckComRCBreakRC (rc);
 
-            ComPtr <nsIException> ex;
-            rc = info.queryInterfaceTo (ex.asOutParam());
-            if (SUCCEEDED (rc))
+            ComPtr<nsIException> ex;
+            rc = info.queryInterfaceTo(ex.asOutParam());
+            if (SUCCEEDED(rc))
                 rc = em->SetCurrentException (ex);
         }
         else if (rc == NS_ERROR_UNEXPECTED)
@@ -908,159 +786,7 @@ HRESULT VirtualBoxSupportErrorInfoImplBase::setErrorInternal (
 
     AssertComRC (rc);
 
-    return SUCCEEDED (rc) ? aResultCode : rc;
-}
-
-// VirtualBoxBaseWithChildren methods
-////////////////////////////////////////////////////////////////////////////////
-
-/**
- *  Uninitializes all dependent children registered with #addDependentChild().
- *
- *  @note
- *      This method will call uninit() methods of children. If these methods
- *      access the parent object, uninitDependentChildren() must be called
- *      either at the beginning of the parent uninitialization sequence (when
- *      it is still operational) or after setReady(false) is called to
- *      indicate the parent is out of action.
- */
-void VirtualBoxBaseWithChildren::uninitDependentChildren()
-{
-    /// @todo (r=dmik) see todo in VirtualBoxBase.h, in
-    //  template <class C> void removeDependentChild (C *child)
-
-    LogFlowThisFuncEnter();
-
-    AutoWriteLock alock (this);
-    AutoWriteLock mapLock (mMapLock);
-
-    LogFlowThisFunc (("count=%d...\n", mDependentChildren.size()));
-
-    if (mDependentChildren.size())
-    {
-        /* We keep the lock until we have enumerated all children.
-         * Those ones that will try to call #removeDependentChild() from
-         * a different thread will have to wait */
-
-        Assert (mUninitDoneSem == NIL_RTSEMEVENT);
-        int vrc = RTSemEventCreate (&mUninitDoneSem);
-        AssertRC (vrc);
-
-        Assert (mChildrenLeft == 0);
-        mChildrenLeft = (unsigned)mDependentChildren.size();
-
-        for (DependentChildren::iterator it = mDependentChildren.begin();
-            it != mDependentChildren.end(); ++ it)
-        {
-            VirtualBoxBase *child = (*it).second;
-            Assert (child);
-            if (child)
-                child->uninit();
-        }
-
-        mDependentChildren.clear();
-    }
-
-    /* Wait until all children started uninitializing on their own
-     * (and therefore are waiting for some parent's method or for
-     * #removeDependentChild() to return) are finished uninitialization */
-
-    if (mUninitDoneSem != NIL_RTSEMEVENT)
-    {
-        /* let stuck children run */
-        mapLock.leave();
-        alock.leave();
-
-        LogFlowThisFunc (("Waiting for uninitialization of all children...\n"));
-
-        RTSemEventWait (mUninitDoneSem, RT_INDEFINITE_WAIT);
-
-        alock.enter();
-        mapLock.enter();
-
-        RTSemEventDestroy (mUninitDoneSem);
-        mUninitDoneSem = NIL_RTSEMEVENT;
-        Assert (mChildrenLeft == 0);
-    }
-
-    LogFlowThisFuncLeave();
-}
-
-/**
- *  Returns a pointer to the dependent child corresponding to the given
- *  interface pointer (used as a key in the map) or NULL if the interface
- *  pointer doesn't correspond to any child registered using
- *  #addDependentChild().
- *
- *  @param  unk
- *      Pointer to map to the dependent child object (it is ComPtr <IUnknown>
- *      rather than IUnknown *, to guarantee IUnknown * identity)
- *  @return
- *      Pointer to the dependent child object
- */
-VirtualBoxBase *VirtualBoxBaseWithChildren::getDependentChild (
-    const ComPtr <IUnknown> &unk)
-{
-    AssertReturn (!!unk, NULL);
-
-    AutoWriteLock alock (mMapLock);
-    if (mUninitDoneSem != NIL_RTSEMEVENT)
-        return NULL;
-
-    DependentChildren::const_iterator it = mDependentChildren.find (unk);
-    if (it == mDependentChildren.end())
-        return NULL;
-    return (*it).second;
-}
-
-/** Helper for addDependentChild() template method */
-void VirtualBoxBaseWithChildren::addDependentChild (
-    const ComPtr <IUnknown> &unk, VirtualBoxBase *child)
-{
-    AssertReturn (!!unk && child, (void) 0);
-
-    AutoWriteLock alock (mMapLock);
-
-    if (mUninitDoneSem != NIL_RTSEMEVENT)
-    {
-        // for this very unlikely case, we have to increase the number of
-        // children left, for symmetry with #removeDependentChild()
-        ++ mChildrenLeft;
-        return;
-    }
-
-    std::pair <DependentChildren::iterator, bool> result =
-        mDependentChildren.insert (DependentChildren::value_type (unk, child));
-    AssertMsg (result.second, ("Failed to insert a child to the map\n"));
-}
-
-/** Helper for removeDependentChild() template method */
-void VirtualBoxBaseWithChildren::removeDependentChild (const ComPtr <IUnknown> &unk)
-{
-    /// @todo (r=dmik) see todo in VirtualBoxBase.h, in
-    //  template <class C> void removeDependentChild (C *child)
-
-    AssertReturn (!!unk, (void) 0);
-
-    AutoWriteLock alock (mMapLock);
-
-    if (mUninitDoneSem != NIL_RTSEMEVENT)
-    {
-        // uninitDependentChildren() is in action, just increase the number
-        // of children left and signal a semaphore when it reaches zero
-        Assert (mChildrenLeft != 0);
-        -- mChildrenLeft;
-        if (mChildrenLeft == 0)
-        {
-            int vrc = RTSemEventSignal (mUninitDoneSem);
-            AssertRC (vrc);
-        }
-        return;
-    }
-
-    DependentChildren::size_type result = mDependentChildren.erase (unk);
-    AssertMsg (result == 1, ("Failed to remove a child from the map\n"));
-    NOREF (result);
+    return SUCCEEDED(rc) ? aResultCode : rc;
 }
 
 // VirtualBoxBaseWithChildrenNEXT methods
@@ -1089,7 +815,7 @@ void VirtualBoxBaseWithChildren::removeDependentChild (const ComPtr <IUnknown> &
  */
 void VirtualBoxBaseWithChildrenNEXT::uninitDependentChildren()
 {
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
 
     /* sanity */
     AssertReturnVoid (autoCaller.state() == InUninit ||
@@ -1104,7 +830,7 @@ void VirtualBoxBaseWithChildrenNEXT::uninitDependentChildren()
         /* strongly reference the weak child from the map to make sure it won't
          * be deleted while we've released the lock */
         DependentChildren::iterator it = mDependentChildren.begin();
-        ComPtr <IUnknown> unk = it->first;
+        ComPtr<IUnknown> unk = it->first;
         Assert (!unk.isNull());
 
         VirtualBoxBase *child = it->second;
@@ -1143,7 +869,7 @@ void VirtualBoxBaseWithChildrenNEXT::uninitDependentChildren()
  * The relation is checked by using the given interface pointer as a key in the
  * map of dependent children.
  *
- * Note that ComPtr <IUnknown> is used as an argument instead of IUnknown * in
+ * Note that ComPtr<IUnknown> is used as an argument instead of IUnknown * in
  * order to guarantee IUnknown identity and disambiguation by doing
  * QueryInterface (IUnknown) rather than a regular C cast.
  *
@@ -1152,18 +878,17 @@ void VirtualBoxBaseWithChildrenNEXT::uninitDependentChildren()
  *
  * @note Locks #childrenLock() for reading.
  */
-VirtualBoxBaseNEXT *
-VirtualBoxBaseWithChildrenNEXT::getDependentChild (const ComPtr <IUnknown> &aUnk)
+VirtualBoxBase* VirtualBoxBaseWithChildrenNEXT::getDependentChild(const ComPtr<IUnknown> &aUnk)
 {
-    AssertReturn (!aUnk.isNull(), NULL);
+    AssertReturn(!aUnk.isNull(), NULL);
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
 
     /* return NULL if uninitDependentChildren() is in action */
     if (autoCaller.state() == InUninit)
         return NULL;
 
-    AutoReadLock alock (childrenLock());
+    AutoReadLock alock(childrenLock());
 
     DependentChildren::const_iterator it = mDependentChildren.find (aUnk);
     if (it == mDependentChildren.end())
@@ -1173,20 +898,20 @@ VirtualBoxBaseWithChildrenNEXT::getDependentChild (const ComPtr <IUnknown> &aUnk
 }
 
 /** Helper for addDependentChild(). */
-void VirtualBoxBaseWithChildrenNEXT::doAddDependentChild (
-    IUnknown *aUnk, VirtualBoxBaseNEXT *aChild)
+void VirtualBoxBaseWithChildrenNEXT::doAddDependentChild(IUnknown *aUnk,
+                                                         VirtualBoxBase *aChild)
 {
     AssertReturnVoid (aUnk != NULL);
     AssertReturnVoid (aChild != NULL);
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
 
     /* sanity */
     AssertReturnVoid (autoCaller.state() == InInit ||
                       autoCaller.state() == Ready ||
                       autoCaller.state() == Limited);
 
-    AutoWriteLock alock (childrenLock());
+    AutoWriteLock alock(childrenLock());
 
     std::pair <DependentChildren::iterator, bool> result =
         mDependentChildren.insert (DependentChildren::value_type (aUnk, aChild));
@@ -1198,7 +923,7 @@ void VirtualBoxBaseWithChildrenNEXT::doRemoveDependentChild (IUnknown *aUnk)
 {
     AssertReturnVoid (aUnk);
 
-    AutoCaller autoCaller (this);
+    AutoCaller autoCaller(this);
 
     /* sanity */
     AssertReturnVoid (autoCaller.state() == InUninit ||
@@ -1206,7 +931,7 @@ void VirtualBoxBaseWithChildrenNEXT::doRemoveDependentChild (IUnknown *aUnk)
                       autoCaller.state() == Ready ||
                       autoCaller.state() == Limited);
 
-    AutoWriteLock alock (childrenLock());
+    AutoWriteLock alock(childrenLock());
 
     DependentChildren::size_type result = mDependentChildren.erase (aUnk);
     AssertMsg (result == 1, ("Failed to remove child %p from the map\n", aUnk));
