@@ -36,7 +36,7 @@
 #include <Carbon/Carbon.h>
 #include <IOKit/IOKitLib.h>
 #include <IOKit/storage/IOStorageDeviceCharacteristics.h>
-#include <IOKit/scsi-commands/SCSITaskLib.h>
+#include <IOKit/scsi/SCSITaskLib.h>
 #include <SystemConfiguration/SystemConfiguration.h>
 #include <mach/mach_error.h>
 #ifdef VBOX_WITH_USB
@@ -762,7 +762,7 @@ static bool darwinIsMassStorageInterfaceInUse(io_object_t MSDObj, io_name_t pszN
  * @param   USBDevice   The USB device object.
  * @param   PropsRef    The USB device properties.
  */
-static void darwinDeterminUSBDeviceState(PUSBDEVICE pCur, io_object_t USBDevice, CFMutableDictionaryRef PropsRef)
+static void darwinDeterminUSBDeviceState(PUSBDEVICE pCur, io_object_t USBDevice, CFMutableDictionaryRef /* PropsRef */)
 {
     /*
      * Iterate the interfaces (among the children of the IOUSBDevice object).
@@ -1780,7 +1780,7 @@ int main(int argc, char **argv)
         for (PDARWINETHERNIC pCur = pEtherNICs; pCur; pCur = pCur->pNext)
         {
             RTPrintf("%s\n", pCur->szName);
-            RTPrintf("    szBSDName=%d\n", pCur->szBSDName);
+            RTPrintf("    szBSDName=%s\n", pCur->szBSDName);
             RTPrintf("         UUID=%RTuuid\n", &pCur->Uuid);
             RTPrintf("          Mac=%.6Rhxs\n", &pCur->Mac);
             RTPrintf("    fWireless=%RTbool\n", pCur->fWireless);
