@@ -25,6 +25,7 @@
 #include "VBox/com/string.h"
 
 #include <iprt/err.h>
+#include <iprt/path.h>
 
 namespace com
 {
@@ -35,77 +36,36 @@ const Bstr Bstr::Null; /* default ctor is OK */
 /* static */
 const Utf8Str Utf8Str::Null; /* default ctor is OK */
 
-const size_t Utf8Str::npos = (size_t)-1;
-
-size_t Utf8Str::find(const char *pcszFind,
-                     size_t pos /*= 0*/)
-    const
+Utf8Str& Utf8Str::toLower()
 {
-    const char *pszThis, *p;
-
-    if (    ((pszThis = c_str()))
-         && (pos < length())
-         && ((p = strstr(pszThis + pos, pcszFind)))
-       )
-        return p - pszThis;
-
-    return npos;
+    if (length())
+        ::RTStrToLower(m_psz);
+    return *this;
 }
 
-Utf8Str Utf8Str::substr(size_t pos /*= 0*/, size_t n /*= npos*/)
-    const
+Utf8Str& Utf8Str::toUpper()
 {
-    Utf8Str ret;
-
-    if (n)
-    {
-        const char *psz;
-
-        if ((psz = c_str()))
-        {
-            RTUNICP cp;
-
-            // walk the UTF-8 characters until where the caller wants to start
-            size_t i = pos;
-            while (*psz && i--)
-                if (RT_FAILURE(RTStrGetCpEx(&psz, &cp)))
-                    return ret;     // return empty string on bad encoding
-
-            const char *pFirst = psz;
-
-            if (n == npos)
-                // all the rest:
-                ret = pFirst;
-            else
-            {
-                i = n;
-                while (*psz && i--)
-                    if (RT_FAILURE(RTStrGetCpEx(&psz, &cp)))
-                        return ret;     // return empty string on bad encoding
-
-                size_t cbCopy = psz - pFirst;
-                ret.alloc(cbCopy + 1);
-                memcpy(ret.str, pFirst, cbCopy);
-                ret.str[cbCopy] = '\0';
-            }
-        }
-    }
-
-    return ret;
+    if (length())
+        ::RTStrToUpper(m_psz);
+    return *this;
 }
 
-int Utf8Str::toInt(uint64_t &i) const
+void Utf8Str::stripTrailingSlash()
 {
-    if (!str)
-        return VERR_NO_DIGITS;
-    return RTStrToUInt64Ex(str, NULL, 0, &i);
+    RTPathStripTrailingSlash(m_psz);
+    jolt();
 }
 
-int Utf8Str::toInt(uint32_t &i) const
+void Utf8Str::stripFilename()
 {
-    if (!str)
-        return VERR_NO_DIGITS;
-    return RTStrToUInt32Ex(str, NULL, 0, &i);
+    RTPathStripFilename(m_psz);
+    jolt();
+}
+
+void Utf8Str::stripExt()
+{
+    RTPathStripExt(m_psz);
+    jolt();
 }
 
 struct FormatData
