@@ -40,7 +40,7 @@
 #include <iprt/semaphore.h>
 #include <iprt/thread.h>
 
-#include <VBox/VBoxGuest.h>
+#include <VBox/VMMDev.h>
 
 /**
  * A service gets one thread, which synchronously delivers messages to
@@ -262,7 +262,7 @@ int HGCMService::loadServiceDLL (void)
 
         rc = RTLdrGetSymbol (m_hLdrMod, VBOX_HGCM_SVCLOAD_NAME, (void**)&m_pfnLoad);
 
-        if (RT_FAILURE (rc) || !m_pfnLoad)
+        if (RT_FAILURE(rc) || !m_pfnLoad)
         {
             Log(("HGCMService::loadServiceDLL: Error resolving the service entry point %s, rc = %d, m_pfnLoad = %p\n", VBOX_HGCM_SVCLOAD_NAME, rc, m_pfnLoad));
 
@@ -285,7 +285,7 @@ int HGCMService::loadServiceDLL (void)
 
             LogFlowFunc(("m_pfnLoad rc = %Rrc\n", rc));
 
-            if (RT_SUCCESS (rc))
+            if (RT_SUCCESS(rc))
             {
                 if (   m_fntable.pfnUnload == NULL
                     || m_fntable.pfnConnect == NULL
@@ -474,7 +474,7 @@ DECLCALLBACK(void) hgcmServiceThread (HGCMTHREADHANDLE ThreadHandle, void *pvUse
         HGCMMsgCore *pMsgCore;
         int rc = hgcmMsgGet (ThreadHandle, &pMsgCore);
 
-        if (RT_FAILURE (rc))
+        if (RT_FAILURE(rc))
         {
             /* The error means some serious unrecoverable problem in the hgcmMsg/hgcmThread layer. */
             AssertMsgFailed (("%Rrc\n", rc));
@@ -645,7 +645,7 @@ DECLCALLBACK(void) hgcmServiceThread (HGCMTHREADHANDLE ThreadHandle, void *pvUse
                         rc = VERR_NOT_SUPPORTED;
                     }
 
-                    if (RT_SUCCESS (rc))
+                    if (RT_SUCCESS(rc))
                     {
                         pSvc->m_hExtension = pMsg->handle;
                     }
@@ -804,7 +804,7 @@ void HGCMService::instanceDestroy (void)
     {
         rc = hgcmMsgSend (hMsg);
 
-        if (RT_SUCCESS (rc))
+        if (RT_SUCCESS(rc))
         {
             hgcmThreadWait (m_thread);
         }
@@ -883,7 +883,7 @@ int HGCMService::loadClientState (uint32_t u32ClientId, PSSMHANDLE pSSM)
     HGCMService *pSvc;
     int rc = HGCMService::ResolveService (&pSvc, pszServiceName);
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         /* The service is already loaded. */
         pSvc->ReleaseService ();
@@ -1190,7 +1190,7 @@ void HGCMService::ReleaseService (void)
         /* Resolve the service instance. */
         HGCMService *pSvc;
         rc = ResolveService (&pSvc, pszServiceName);
-        AssertReturn(pSvc, VERR_SSM_UNEXPECTED_DATA);
+        AssertLogRelMsgReturn(pSvc, ("rc=%Rrc, %s\n", rc, pszServiceName), VERR_SSM_UNEXPECTED_DATA);
 
         /* Get the number of clients. */
         uint32_t cClients;
@@ -1219,7 +1219,7 @@ void HGCMService::ReleaseService (void)
             if (RT_FAILURE(rc))
             {
                 pSvc->ReleaseService ();
-                AssertFailed();
+                AssertLogRelMsgFailed(("rc=%Rrc %s\n", rc, pszServiceName));
                 return rc;
             }
 
@@ -1228,7 +1228,7 @@ void HGCMService::ReleaseService (void)
             if (RT_FAILURE(rc))
             {
                 pSvc->ReleaseService ();
-                AssertFailed();
+                AssertLogRelMsgFailed(("rc=%Rrc %s\n", rc, pszServiceName));
                 return rc;
             }
         }
@@ -1295,7 +1295,7 @@ int HGCMService::CreateAndConnectClient (uint32_t *pu32ClientIdOut, uint32_t u32
 
             rc = hgcmMsgSend (hMsg);
 
-            if (RT_SUCCESS (rc))
+            if (RT_SUCCESS(rc))
             {
                 /* Add the client Id to the array. */
                 if (m_cClients == m_cClientsAllocated)
@@ -1647,7 +1647,7 @@ static DECLCALLBACK(void) hgcmThread (HGCMTHREADHANDLE ThreadHandle, void *pvUse
         HGCMMsgCore *pMsgCore;
         int rc = hgcmMsgGet (ThreadHandle, &pMsgCore);
 
-        if (RT_FAILURE (rc))
+        if (RT_FAILURE(rc))
         {
             /* The error means some serious unrecoverable problem in the hgcmMsg/hgcmThread layer. */
             AssertMsgFailed (("%Rrc\n", rc));
@@ -1670,7 +1670,7 @@ static DECLCALLBACK(void) hgcmThread (HGCMTHREADHANDLE ThreadHandle, void *pvUse
                 HGCMService *pService;
                 rc = HGCMService::ResolveService (&pService, pMsg->pszServiceName);
 
-                if (RT_SUCCESS (rc))
+                if (RT_SUCCESS(rc))
                 {
                     /* Call the service instance method. */
                     rc = pService->CreateAndConnectClient (pMsg->pu32ClientId, 0);
@@ -1725,7 +1725,7 @@ static DECLCALLBACK(void) hgcmThread (HGCMTHREADHANDLE ThreadHandle, void *pvUse
                 HGCMService *pService;
                 rc = HGCMService::ResolveService (&pService, pMsg->pszServiceName);
 
-                if (RT_SUCCESS (rc))
+                if (RT_SUCCESS(rc))
                 {
                     rc = pService->HostCall (pMsg->u32Function, pMsg->cParms, pMsg->paParms);
 
@@ -1790,14 +1790,14 @@ static DECLCALLBACK(void) hgcmThread (HGCMTHREADHANDLE ThreadHandle, void *pvUse
                     HGCMService *pService;
                     rc = HGCMService::ResolveService (&pService, handle->pszServiceName);
 
-                    if (RT_SUCCESS (rc))
+                    if (RT_SUCCESS(rc))
                     {
                         pService->RegisterExtension (handle, pMsg->pfnExtension, pMsg->pvExtension);
 
                         pService->ReleaseService ();
                     }
 
-                    if (RT_FAILURE (rc))
+                    if (RT_FAILURE(rc))
                     {
                         RTMemFree (handle);
                     }
@@ -1817,7 +1817,7 @@ static DECLCALLBACK(void) hgcmThread (HGCMTHREADHANDLE ThreadHandle, void *pvUse
                 HGCMService *pService;
                 rc = HGCMService::ResolveService (&pService, pMsg->handle->pszServiceName);
 
-                if (RT_SUCCESS (rc))
+                if (RT_SUCCESS(rc))
                 {
                     pService->UnregisterExtension (pMsg->handle);
 
@@ -2237,7 +2237,7 @@ int HGCMHostInit (void)
 
         rc = hgcmThreadCreate (&g_hgcmThread, "MainHGCMthread", hgcmThread, NULL);
 
-        if (RT_FAILURE (rc))
+        if (RT_FAILURE(rc))
         {
             LogRel(("Failed to start HGCM thread. HGCM services will be unavailable!!! rc = %Rrc\n", rc));
         }
@@ -2257,7 +2257,7 @@ int HGCMHostShutdown (void)
 
     int rc = HGCMHostReset ();
 
-    if (RT_SUCCESS (rc))
+    if (RT_SUCCESS(rc))
     {
         /* Send the quit message to the main hgcmThread. */
         HGCMMSGHANDLE hMsg = 0;
@@ -2268,7 +2268,7 @@ int HGCMHostShutdown (void)
         {
             rc = hgcmMsgSend (hMsg);
 
-            if (RT_SUCCESS (rc))
+            if (RT_SUCCESS(rc))
             {
                 /* Wait for the thread termination. */
                 hgcmThreadWait (g_hgcmThread);
