@@ -44,10 +44,10 @@ public:
 
 private:
     static DECLCALLBACK(void *) drvQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface);
-    static DECLCALLBACK(int)    drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle);
+    static DECLCALLBACK(int)    drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags);
     static DECLCALLBACK(void)   drvDestruct(PPDMDRVINS pDrvIns);
 
-    ComObjPtr <Console, ComWeakRef> mParent;
+    ComObjPtr<Console, ComWeakRef> mParent;
 };
 
 #endif /* ____H_AUDIOSNIFFERINTERFACE */
