@@ -28,9 +28,9 @@
 #include <VBox/ostypes.h>
 
 class ATL_NO_VTABLE GuestOSType :
-    public VirtualBoxBaseNEXT,
-    public VirtualBoxSupportErrorInfoImpl <GuestOSType, IGuestOSType>,
-    public VirtualBoxSupportTranslation <GuestOSType>,
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<GuestOSType, IGuestOSType>,
+    public VirtualBoxSupportTranslation<GuestOSType>,
     VBOX_SCRIPTABLE_IMPL(IGuestOSType)
 {
 public:
@@ -47,8 +47,6 @@ public:
         COM_INTERFACE_ENTRY(IDispatch)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (GuestOSType)
 
     HRESULT FinalConstruct();
@@ -59,7 +57,7 @@ public:
                   const char *aId, const char *aDescription,
                   VBOXOSTYPE aOSType, uint32_t aOSHint,
                   uint32_t aRAMSize, uint32_t aVRAMSize, uint32_t aHDDSize,
-                  NetworkAdapterType_T aNetworkAdapterType);
+                  NetworkAdapterType_T aNetworkAdapterType, uint32_t numSerialEnabled);
     void uninit();
 
     // IGuestOSType properties
@@ -81,6 +79,7 @@ public:
     bool recommendedIOAPIC() const { return !!(mOSHint & VBOXOSHINT_IOAPIC); }
     bool recommendedVirtEx() const { return !!(mOSHint & VBOXOSHINT_HWVIRTEX); }
     NetworkAdapterType_T networkAdapterType() const { return mNetworkAdapterType; }
+    uint32_t numSerialEnabled() const { return mNumSerialEnabled; }
 
     // for VirtualBoxSupportErrorInfoImpl
     static const wchar_t *getComponentName() { return L"GuestOSType"; }
@@ -98,6 +97,7 @@ private:
     const uint32_t mHDDSize;
     const uint32_t mMonitorCount;
     const NetworkAdapterType_T mNetworkAdapterType;
+    const uint32_t mNumSerialEnabled;
 };
 
 #endif // ____H_GUESTOSTYPEIMPL
