@@ -115,7 +115,7 @@ void listVMs(IVirtualBox *virtualBox)
     IMachine **machines = NULL;
     PRUint32 machineCnt = 0;
 
-    rc = virtualBox->GetMachines2(&machineCnt, &machines);
+    rc = virtualBox->GetMachines(&machineCnt, &machines);
     if (NS_SUCCEEDED(rc))
     {
         /*
@@ -142,12 +142,11 @@ void listVMs(IVirtualBox *virtualBox)
                     printf("\tName:        <inaccessible>\n");
                 }
 
-                nsID *iid = nsnull;
-                machine->GetId(&iid);
-                const char *uuidString = nsIDToString(iid);
+                nsXPIDLString iid;
+                machine->GetId(getter_Copies(iid));
+                const char *uuidString = ToNewCString(iid);
                 printf("\tUUID:        %s\n", uuidString);
                 free((void*)uuidString);
-                nsMemory::Free(iid);
 
                 if (isAccessible)
                 {
@@ -193,10 +192,9 @@ void createVM(IVirtualBox *virtualBox)
      * First create a unnamed new VM. It will be unconfigured and not be saved
      * in the configuration until we explicitely choose to do so.
      */
-    nsID VMuuid = {0};
     nsCOMPtr <IMachine> machine;
     rc = virtualBox->CreateMachine(NS_LITERAL_STRING("A brand new name").get(),
-                                   nsnull, nsnull, VMuuid, getter_AddRefs(machine));
+                                   nsnull, nsnull, nsnull, getter_AddRefs(machine));
     if (NS_FAILED(rc))
     {
         printf("Error: could not create machine! rc=%08X\n", rc);
@@ -271,10 +269,9 @@ void createVM(IVirtualBox *virtualBox)
             return;
         }
 
-        nsID *machineUUID = nsnull;
-        machine->GetId(&machineUUID);
-        rc = virtualBox->OpenSession(session, *machineUUID);
-        nsMemory::Free(machineUUID);
+        nsXPIDLString machineUUID;
+        machine->GetId(getter_Copies(machineUUID));
+        rc = virtualBox->OpenSession(session, machineUUID);
         if (NS_FAILED(rc))
         {
             printf("Error, could not open session! rc=0x%x\n", rc);
@@ -297,10 +294,10 @@ void createVM(IVirtualBox *virtualBox)
     /*
      * Create a virtual harddisk
      */
-    nsCOMPtr<IHardDisk2> hardDisk = 0;
-    rc = virtualBox->CreateHardDisk2(NS_LITERAL_STRING("VDI").get(),
-                                     NS_LITERAL_STRING("TestHardDisk.vdi").get(),
-                                     getter_AddRefs(hardDisk));
+    nsCOMPtr<IMedium> hardDisk = 0;
+    rc = virtualBox->CreateHardDisk(NS_LITERAL_STRING("VDI").get(),
+                                    NS_LITERAL_STRING("TestHardDisk.vdi").get(),
+                                    getter_AddRefs(hardDisk));
     if (NS_FAILED(rc))
     {
         printf("Failed creating a hard disk object! rc=%08X\n", rc);
@@ -313,8 +310,9 @@ void createVM(IVirtualBox *virtualBox)
          * a dynamically expanding image.
          */
         nsCOMPtr <IProgress> progress;
-        rc = hardDisk->CreateDynamicStorage(100,                                // size in megabytes
-                                            getter_AddRefs(progress));          // optional progress object
+        rc = hardDisk->CreateBaseStorage(100,                                // size in megabytes
+                                         MediumVariant_Standard,
+                                         getter_AddRefs(progress));          // optional progress object
         if (NS_FAILED(rc))
         {
             printf("Failed creating hard disk image! rc=%08X\n", rc);
@@ -327,7 +325,7 @@ void createVM(IVirtualBox *virtualBox)
              * Here we wait forever (timeout -1)  which is potentially dangerous.
              */
             rc = progress->WaitForCompletion(-1);
-            nsresult resultCode;
+            PRInt32 resultCode;
             progress->GetResultCode(&resultCode);
             if (NS_FAILED(rc) || NS_FAILED(resultCode))
             {
@@ -341,13 +339,13 @@ void createVM(IVirtualBox *virtualBox)
                  * by UUID, so query that one fist. The UUID has been assigned automatically
                  * when we've created the image.
                  */
-                nsID *vdiUUID = nsnull;
-                hardDisk->GetId(&vdiUUID);
-                rc = machine->AttachHardDisk2(*vdiUUID,
-                                              StorageBus::IDE, // controler identifier
-                                              0,               // channel number on the controller
-                                              0);              // device number on the controller
-                nsMemory::Free(vdiUUID);
+                nsXPIDLString vdiUUID;
+                hardDisk->GetId(getter_Copies(vdiUUID));
+                rc = machine->AttachDevice(NS_LITERAL_STRING("IDE Controller").get(), // controller identifier
+                                           0,                              // channel number on the controller
+                                           0,                              // device number on the controller
+                                           DeviceType_HardDisk,
+                                           vdiUUID);
                 if (NS_FAILED(rc))
                 {
                     printf("Error: could not attach hard disk! rc=%08X\n", rc);
@@ -362,11 +360,10 @@ void createVM(IVirtualBox *virtualBox)
      * has to be registered and then mounted to the VM's DVD drive and selected
      * as the boot device.
      */
-    nsID uuid = {0};
-    nsCOMPtr<IDVDImage2> dvdImage;
+    nsCOMPtr<IMedium> dvdImage;
 
-    rc = virtualBox->OpenDVDImage(NS_LITERAL_STRING("/home/achimha/isoimages/winnt4ger.iso").get(),
-                                  uuid, /* NULL UUID, i.e. a new one will be created */
+    rc = virtualBox->OpenDVDImage(NS_LITERAL_STRING("/home/vbox/isos/winnt4ger.iso").get(),
+                                  nsnull, /* NULL UUID, i.e. a new one will be created */
                                   getter_AddRefs(dvdImage));
     if (NS_FAILED(rc))
     {
@@ -377,12 +374,13 @@ void createVM(IVirtualBox *virtualBox)
         /*
          * Now assign it to our VM
          */
-        nsID *isoUUID = nsnull;
-        dvdImage->GetId(&isoUUID);
-        nsCOMPtr<IDVDDrive> dvdDrive;
-        machine->GetDVDDrive(getter_AddRefs(dvdDrive));
-        rc = dvdDrive->MountImage(*isoUUID);
-        nsMemory::Free(isoUUID);
+        nsXPIDLString isoUUID;
+        dvdImage->GetId(getter_Copies(isoUUID));
+        rc = machine->MountMedium(NS_LITERAL_STRING("IDE Controller").get(), // controller identifier
+                                  2,                              // channel number on the controller
+                                  0,                              // device number on the controller
+                                  isoUUID,
+                                  PR_FALSE);                      // aForce
         if (NS_FAILED(rc))
         {
             printf("Error: could not mount ISO image! rc=%08X\n", rc);
@@ -593,15 +591,15 @@ void printErrorInfo()
 
     nsCOMPtr <nsIExceptionService> es;
     es = do_GetService (NS_EXCEPTIONSERVICE_CONTRACTID, &rc);
-    if (NS_SUCCEEDED (rc))
+    if (NS_SUCCEEDED(rc))
     {
         nsCOMPtr <nsIExceptionManager> em;
         rc = es->GetCurrentExceptionManager (getter_AddRefs (em));
-        if (NS_SUCCEEDED (rc))
+        if (NS_SUCCEEDED(rc))
         {
             nsCOMPtr<nsIException> ex;
             rc = em->GetCurrentException (getter_AddRefs (ex));
-            if (NS_SUCCEEDED (rc) && ex)
+            if (NS_SUCCEEDED(rc) && ex)
             {
                 nsCOMPtr <IVirtualBoxErrorInfo> info;
                 info = do_QueryInterface(ex, &rc);
@@ -609,7 +607,7 @@ void printErrorInfo()
                 {
                     /* got extended error info */
                     printf ("Extended error info (IVirtualBoxErrorInfo):\n");
-                    nsresult resultCode = NS_OK;
+                    PRInt32 resultCode = NS_OK;
                     info->GetResultCode (&resultCode);
                     printf ("  resultCode=%08X\n", resultCode);
                     nsXPIDLString component;
