@@ -21,11 +21,14 @@
 
 #include "svchlp.h"
 
-#include "HostImpl.h"
+//#include "HostImpl.h"
 #include "Logging.h"
 
 #include <VBox/err.h>
 
+int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
+                                        SVCHlpMsg::Code aMsgCode);
+
 using namespace com;
 
 enum { PipeBufSize = 1024 };
@@ -61,7 +64,7 @@ SVCHlpClient::~SVCHlpClient()
 
 int SVCHlpClient::create (const char *aName)
 {
-    AssertReturn (aName, VERR_INVALID_PARAMETER);
+    AssertReturn(aName, VERR_INVALID_PARAMETER);
 
     if (mIsOpen)
         return VERR_WRONG_ORDER;
@@ -90,7 +93,7 @@ int SVCHlpClient::create (const char *aName)
 
 int SVCHlpClient::open (const char *aName)
 {
-    AssertReturn (aName, VERR_INVALID_PARAMETER);
+    AssertReturn(aName, VERR_INVALID_PARAMETER);
 
     if (mIsOpen)
         return VERR_WRONG_ORDER;
@@ -157,15 +160,15 @@ int SVCHlpClient::close()
 
 int SVCHlpClient::write (const void *aVal, size_t aLen)
 {
-    AssertReturn (aVal != NULL, VERR_INVALID_PARAMETER);
-    AssertReturn (aLen != 0, VERR_INVALID_PARAMETER);
+    AssertReturn(aVal != NULL, VERR_INVALID_PARAMETER);
+    AssertReturn(aLen != 0, VERR_INVALID_PARAMETER);
 
     if (!mIsOpen)
         return VERR_WRONG_ORDER;
 
     DWORD written = 0;
-    BOOL ok = WriteFile (mWriteEnd, aVal, aLen, &written, NULL);
-    AssertReturn (!ok || written == aLen, VERR_GENERAL_FAILURE);
+    BOOL ok = WriteFile (mWriteEnd, aVal, (ULONG)aLen, &written, NULL);
+    AssertReturn(!ok || written == aLen, VERR_GENERAL_FAILURE);
     return ok ? VINF_SUCCESS : rtErrConvertFromWin32OnFailure();
 }
 
@@ -175,14 +178,14 @@ int SVCHlpClient::write (const Utf8Str &aVal)
         return VERR_WRONG_ORDER;
 
     /* write -1 for NULL strings */
-    if (aVal.isNull())
+    if (aVal.isEmpty())
         return write ((size_t) ~0);
 
     size_t len = aVal.length();
 
     /* write string length */
     int vrc = write (len);
-    if (RT_SUCCESS (vrc))
+    if (RT_SUCCESS(vrc))
     {
         /* write string data */
         vrc = write (aVal.raw(), len);
@@ -199,15 +202,15 @@ int SVCHlpClient::write (const Guid &aGuid)
 
 int SVCHlpClient::read (void *aVal, size_t aLen)
 {
-    AssertReturn (aVal != NULL, VERR_INVALID_PARAMETER);
-    AssertReturn (aLen != 0, VERR_INVALID_PARAMETER);
+    AssertReturn(aVal != NULL, VERR_INVALID_PARAMETER);
+    AssertReturn(aLen != 0, VERR_INVALID_PARAMETER);
 
     if (!mIsOpen)
         return VERR_WRONG_ORDER;
 
     DWORD read = 0;
-    BOOL ok = ReadFile (mReadEnd, aVal, aLen, &read, NULL);
-    AssertReturn (!ok || read == aLen, VERR_GENERAL_FAILURE);
+    BOOL ok = ReadFile (mReadEnd, aVal, (ULONG)aLen, &read, NULL);
+    AssertReturn(!ok || read == aLen, VERR_GENERAL_FAILURE);
     return ok ? VINF_SUCCESS : rtErrConvertFromWin32OnFailure();
 }
 
@@ -220,7 +223,7 @@ int SVCHlpClient::read (Utf8Str &aVal)
 
     /* read string length */
     int vrc = read (len);
-    if (RT_FAILURE (vrc))
+    if (RT_FAILURE(vrc))
         return vrc;
 
     /* length -1 means a NULL string */
@@ -230,7 +233,7 @@ int SVCHlpClient::read (Utf8Str &aVal)
         return VINF_SUCCESS;
     }
 
-    aVal.alloc (len + 1);
+    aVal.reserve(len + 1);
     aVal.mutableRaw() [len] = 0;
 
     /* read string data */
@@ -243,8 +246,8 @@ int SVCHlpClient::read (Guid &aGuid)
 {
     Utf8Str guidStr;
     int vrc = read (guidStr);
-    if (RT_SUCCESS (vrc))
-        aGuid = Guid (guidStr);
+    if (RT_SUCCESS(vrc))
+        aGuid = Guid (guidStr.c_str());
     return vrc;
 }
 
@@ -262,7 +265,7 @@ int SVCHlpServer::run()
     do
     {
         vrc = read (msgCode);
-        if (RT_FAILURE (vrc))
+        if (RT_FAILURE(vrc))
             return vrc;
 
         /* terminate request received */
@@ -271,10 +274,16 @@ int SVCHlpServer::run()
 
         switch (msgCode)
         {
-            case SVCHlpMsg::CreateHostNetworkInterface:
-            case SVCHlpMsg::RemoveHostNetworkInterface:
+            case SVCHlpMsg::CreateHostOnlyNetworkInterface:
+            case SVCHlpMsg::RemoveHostOnlyNetworkInterface:
+            case SVCHlpMsg::EnableDynamicIpConfig:
+            case SVCHlpMsg::EnableStaticIpConfig:
+            case SVCHlpMsg::EnableStaticIpConfigV6:
+            case SVCHlpMsg::DhcpRediscover:
             {
-                vrc = Host::networkInterfaceHelperServer (this, msgCode);
+#ifdef VBOX_WITH_NETFLT
+                vrc = netIfNetworkInterfaceHelperServer (this, msgCode);
+#endif
                 break;
             }
             default:
@@ -283,7 +292,7 @@ int SVCHlpServer::run()
                     VERR_GENERAL_FAILURE);
         }
 
-        if (RT_FAILURE (vrc))
+        if (RT_FAILURE(vrc))
             return vrc;
     }
     while (1);
