@@ -28,10 +28,12 @@
  * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/dbg.h>
+#include "internal/iprt.h"
 
 #include <iprt/asm.h>
 #include <iprt/assert.h>
@@ -304,18 +306,21 @@ RTDECL(int) RTDbgModCreate(PRTDBGMOD phDbgMod, const char *pszName, RTUINTPTR cb
     RTMemFree(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModCreate);
 
 
 RTDECL(int)         RTDbgModCreateDeferred(PRTDBGMOD phDbgMod, const char *pszFilename, const char *pszName, RTUINTPTR cb, uint32_t fFlags)
 {
     return VERR_NOT_IMPLEMENTED;
 }
+RT_EXPORT_SYMBOL(RTDbgModCreateDeferred);
 
 
 RTDECL(int)         RTDbgModCreateFromImage(PRTDBGMOD phDbgMod, const char *pszFilename, const char *pszName, uint32_t fFlags)
 {
     return VERR_NOT_IMPLEMENTED;
 }
+RT_EXPORT_SYMBOL(RTDbgModCreateFromImage);
 
 
 RTDECL(int) RTDbgModCreateFromMap(PRTDBGMOD phDbgMod, const char *pszFilename, const char *pszName, RTUINTPTR uSubtrahend, uint32_t fFlags)
@@ -392,6 +397,7 @@ RTDECL(int) RTDbgModCreateFromMap(PRTDBGMOD phDbgMod, const char *pszFilename, c
     RTMemFree(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModCreateFromMap);
 
 
 /**
@@ -448,6 +454,7 @@ RTDECL(uint32_t) RTDbgModRetain(RTDBGMOD hDbgMod)
     RTDBGMOD_VALID_RETURN_RC(pDbgMod, UINT32_MAX);
     return ASMAtomicIncU32(&pDbgMod->cRefs);
 }
+RT_EXPORT_SYMBOL(RTDbgModRetain);
 
 
 /**
@@ -474,6 +481,7 @@ RTDECL(uint32_t) RTDbgModRelease(RTDBGMOD hDbgMod)
         rtDbgModDestroy(pDbgMod);
     return cRefs;
 }
+RT_EXPORT_SYMBOL(RTDbgModRelease);
 
 
 /**
@@ -489,6 +497,7 @@ RTDECL(const char *) RTDbgModName(RTDBGMOD hDbgMod)
     RTDBGMOD_VALID_RETURN_RC(pDbgMod, NULL);
     return pDbgMod->pszName;
 }
+RT_EXPORT_SYMBOL(RTDbgModName);
 
 
 /**
@@ -513,6 +522,7 @@ RTDECL(RTDBGSEGIDX) RTDbgModRvaToSegOff(RTDBGMOD hDbgMod, RTUINTPTR uRva, PRTUIN
     RTDBGMOD_UNLOCK(pDbgMod);
     return iSeg;
 }
+RT_EXPORT_SYMBOL(RTDbgModRvaToSegOff);
 
 
 /**
@@ -537,6 +547,50 @@ RTDECL(RTUINTPTR) RTDbgModImageSize(RTDBGMOD hDbgMod)
     RTDBGMOD_UNLOCK(pDbgMod);
     return cbImage;
 }
+RT_EXPORT_SYMBOL(RTDbgModImageSize);
+
+
+/**
+ * Gets the module tag value if any.
+ *
+ * @returns The tag. 0 if hDbgMod is invalid.
+ *
+ * @param   hDbgMod         The module handle.
+ */
+RTDECL(uint64_t) RTDbgModGetTag(RTDBGMOD hDbgMod)
+{
+    PRTDBGMODINT pDbgMod = hDbgMod;
+    RTDBGMOD_VALID_RETURN_RC(pDbgMod, 0);
+    return pDbgMod->uTag;
+}
+RT_EXPORT_SYMBOL(RTDbgModGetTag);
+
+
+/**
+ * Tags or untags the module.
+ *
+ * @returns IPRT status code.
+ * @retval  VERR_INVALID_HANDLE if hDbgMod is invalid.
+ *
+ * @param   hDbgMod         The module handle.
+ * @param   uTag            The tag value.  The convention is that 0 is no tag
+ *                          and any other value means it's tagged.  It's adviced
+ *                          to use some kind of unique number like an address
+ *                          (global or string cache for instance) to avoid
+ *                          collisions with other users
+ */
+RTDECL(int) RTDbgModSetTag(RTDBGMOD hDbgMod, uint64_t uTag)
+{
+    PRTDBGMODINT pDbgMod = hDbgMod;
+    RTDBGMOD_VALID_RETURN_RC(pDbgMod, VERR_INVALID_HANDLE);
+    RTDBGMOD_LOCK(pDbgMod);
+
+    pDbgMod->uTag = uTag;
+
+    RTDBGMOD_UNLOCK(pDbgMod);
+    return VINF_SUCCESS;
+}
+RT_EXPORT_SYMBOL(RTDbgModSetTag);
 
 
 /**
@@ -595,6 +649,7 @@ RTDECL(int) RTDbgModSegmentAdd(RTDBGMOD hDbgMod, RTUINTPTR uRva, RTUINTPTR cb, c
     return rc;
 
 }
+RT_EXPORT_SYMBOL(RTDbgModSegmentAdd);
 
 
 /**
@@ -619,6 +674,7 @@ RTDECL(RTDBGSEGIDX) RTDbgModSegmentCount(RTDBGMOD hDbgMod)
     RTDBGMOD_UNLOCK(pDbgMod);
     return cSegs;
 }
+RT_EXPORT_SYMBOL(RTDbgModSegmentCount);
 
 
 /**
@@ -650,6 +706,7 @@ RTDECL(int) RTDbgModSegmentByIndex(RTDBGMOD hDbgMod, RTDBGSEGIDX iSeg, PRTDBGSEG
     RTDBGMOD_UNLOCK(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSegmentByIndex);
 
 
 /**
@@ -674,6 +731,7 @@ RTDECL(RTUINTPTR) RTDbgModSegmentSize(RTDBGMOD hDbgMod, RTDBGSEGIDX iSeg)
     int rc = RTDbgModSegmentByIndex(hDbgMod, iSeg, &SegInfo);
     return RT_SUCCESS(rc) ? SegInfo.cb : RTUINTPTR_MAX;
 }
+RT_EXPORT_SYMBOL(RTDbgModSegmentSize);
 
 
 /**
@@ -695,6 +753,7 @@ RTDECL(RTUINTPTR) RTDbgModSegmentRva(RTDBGMOD hDbgMod, RTDBGSEGIDX iSeg)
     int rc = RTDbgModSegmentByIndex(hDbgMod, iSeg, &SegInfo);
     return RT_SUCCESS(rc) ? SegInfo.uRva : RTUINTPTR_MAX;
 }
+RT_EXPORT_SYMBOL(RTDbgModSegmentRva);
 
 
 /**
@@ -768,6 +827,7 @@ RTDECL(int) RTDbgModSymbolAdd(RTDBGMOD hDbgMod, const char *pszSymbol, RTDBGSEGI
     RTDBGMOD_UNLOCK(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolAdd);
 
 
 /**
@@ -793,6 +853,7 @@ RTDECL(uint32_t) RTDbgModSymbolCount(RTDBGMOD hDbgMod)
     RTDBGMOD_UNLOCK(pDbgMod);
     return cSymbols;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolCount);
 
 
 /**
@@ -820,6 +881,7 @@ RTDECL(int) RTDbgModSymbolByOrdinal(RTDBGMOD hDbgMod, uint32_t iOrdinal, PRTDBGS
     RTDBGMOD_UNLOCK(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolByOrdinal);
 
 
 /**
@@ -855,6 +917,7 @@ RTDECL(int) RTDbgModSymbolByOrdinalA(RTDBGMOD hDbgMod, uint32_t iOrdinal, PRTDBG
         RTDbgSymbolFree(pSymInfo);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolByOrdinalA);
 
 
 /**
@@ -915,6 +978,7 @@ RTDECL(int) RTDbgModSymbolByAddr(RTDBGMOD hDbgMod, RTDBGSEGIDX iSeg, RTUINTPTR o
     RTDBGMOD_UNLOCK(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolByAddr);
 
 
 /**
@@ -961,6 +1025,7 @@ RTDECL(int) RTDbgModSymbolByAddrA(RTDBGMOD hDbgMod, RTDBGSEGIDX iSeg, RTUINTPTR
         RTDbgSymbolFree(pSymInfo);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolByAddrA);
 
 
 /**
@@ -998,6 +1063,7 @@ RTDECL(int) RTDbgModSymbolByName(RTDBGMOD hDbgMod, const char *pszSymbol, PRTDBG
 
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolByName);
 
 
 /**
@@ -1033,6 +1099,7 @@ RTDECL(int) RTDbgModSymbolByNameA(RTDBGMOD hDbgMod, const char *pszSymbol, PRTDB
         RTDbgSymbolFree(pSymInfo);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModSymbolByNameA);
 
 
 /**
@@ -1101,6 +1168,7 @@ RTDECL(int) RTDbgModLineAdd(RTDBGMOD hDbgMod, const char *pszFile, uint32_t uLin
     RTDBGMOD_UNLOCK(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModLineAdd);
 
 
 /**
@@ -1126,6 +1194,7 @@ RTDECL(uint32_t) RTDbgModLineCount(RTDBGMOD hDbgMod)
     RTDBGMOD_UNLOCK(pDbgMod);
     return cLineNumbers;
 }
+RT_EXPORT_SYMBOL(RTDbgModLineCount);
 
 
 /**
@@ -1156,6 +1225,7 @@ RTDECL(int) RTDbgModLineByOrdinal(RTDBGMOD hDbgMod, uint32_t iOrdinal, PRTDBGLIN
     RTDBGMOD_UNLOCK(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModLineByOrdinal);
 
 
 /**
@@ -1194,6 +1264,7 @@ RTDECL(int) RTDbgModLineByOrdinalA(RTDBGMOD hDbgMod, uint32_t iOrdinal, PRTDBGLI
         RTDbgLineFree(pLineInfo);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModLineByOrdinalA);
 
 
 /**
@@ -1252,6 +1323,7 @@ RTDECL(int) RTDbgModLineByAddr(RTDBGMOD hDbgMod, RTDBGSEGIDX iSeg, RTUINTPTR off
     RTDBGMOD_UNLOCK(pDbgMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModLineByAddr);
 
 
 /**
@@ -1299,4 +1371,5 @@ RTDECL(int) RTDbgModLineByAddrA(RTDBGMOD hDbgMod, RTDBGSEGIDX iSeg, RTUINTPTR of
         RTDbgLineFree(pLineInfo);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgModLineByAddrA);
 
