@@ -28,10 +28,13 @@
  * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/string.h>
+#include "internal/iprt.h"
+
 #include <iprt/ctype.h>
 #include <iprt/string.h>
 
@@ -45,16 +48,17 @@
 RTDECL(char *) RTStrStrip(char *psz)
 {
     /* left */
-    while (isspace(*psz))
+    while (RT_C_IS_SPACE(*psz))
         psz++;
 
     /* right */
     char *pszEnd = strchr(psz, '\0');
-    while (--pszEnd > psz && isspace(*pszEnd))
+    while (--pszEnd > psz && RT_C_IS_SPACE(*pszEnd))
         *pszEnd = '\0';
 
     return psz;
 }
+RT_EXPORT_SYMBOL(RTStrStrip);
 
 
 /**
@@ -66,11 +70,12 @@ RTDECL(char *) RTStrStrip(char *psz)
 RTDECL(char *) RTStrStripL(const char *psz)
 {
     /* left */
-    while (isspace(*psz))
+    while (RT_C_IS_SPACE(*psz))
         psz++;
 
     return (char *)psz;
 }
+RT_EXPORT_SYMBOL(RTStrStripL);
 
 
 /**
@@ -83,9 +88,10 @@ RTDECL(char *) RTStrStripR(char *psz)
 {
     /* right */
     char *pszEnd = strchr(psz, '\0');
-    while (--pszEnd > psz && isspace(*pszEnd))
+    while (--pszEnd > psz && RT_C_IS_SPACE(*pszEnd))
         *pszEnd = '\0';
 
     return psz;
 }
+RT_EXPORT_SYMBOL(RTStrStripR);
 
