@@ -33,6 +33,8 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/string.h>
+#include "internal/iprt.h"
+
 #include <iprt/uni.h>
 #include <iprt/alloc.h>
 #include <iprt/assert.h>
@@ -186,15 +188,12 @@ static int rtUtf8Length(const char *psz, size_t cch, size_t *pcuc, size_t *pcchA
  *                  The recoding will stop when cch or '\\0' is reached. Pass RTSTR_MAX to process up to '\\0'.
  * @param   paCps   Where to store the code points array.
  * @param   cCps    The number of RTUNICP items the paCps buffer can hold, excluding the terminator ('\\0').
- * @param   pcCps   Where to store the actual number of decoded code points. This excludes the terminator.
  */
-static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps, size_t *pcCps)
+static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps)
 {
-    int                     rc = VINF_SUCCESS;
+    int                     rc   = VINF_SUCCESS;
     const unsigned char    *puch = (const unsigned char *)psz;
-    const PRTUNICP          pCpEnd = paCps + cCps;
-    PRTUNICP                pCp = paCps;
-    Assert(pCpEnd >= pCp);
+    PRTUNICP                pCp  = paCps;
     while (cch > 0)
     {
         /* read the next char and check for terminator. */
@@ -203,11 +202,12 @@ static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps
             break;
 
         /* check for output overflow */
-        if (pCp >= pCpEnd)
+        if (RT_UNLIKELY(cCps < 1))
         {
             rc = VERR_BUFFER_OVERFLOW;
             break;
         }
+        cCps--;
 
         /* decode and recode the code point */
         if (!(uch & RT_BIT(7)))
@@ -270,7 +270,6 @@ static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps
 
     /* done */
     *pCp = 0;
-    *pcCps = pCp - paCps;
     return rc;
 }
 
@@ -281,6 +280,7 @@ RTDECL(size_t) RTStrUniLen(const char *psz)
     int rc = rtUtf8Length(psz, RTSTR_MAX, &cCodePoints, NULL);
     return RT_SUCCESS(rc) ? cCodePoints : 0;
 }
+RT_EXPORT_SYMBOL(RTStrUniLen);
 
 
 RTDECL(int) RTStrUniLenEx(const char *psz, size_t cch, size_t *pcCps)
@@ -291,12 +291,14 @@ RTDECL(int) RTStrUniLenEx(const char *psz, size_t cch, size_t *pcCps)
         *pcCps = RT_SUCCESS(rc) ? cCodePoints : 0;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrUniLenEx);
 
 
 RTDECL(int) RTStrValidateEncoding(const char *psz)
 {
     return RTStrValidateEncodingEx(psz, RTSTR_MAX, 0);
 }
+RT_EXPORT_SYMBOL(RTStrValidateEncoding);
 
 
 RTDECL(int) RTStrValidateEncodingEx(const char *psz, size_t cch, uint32_t fFlags)
@@ -321,6 +323,7 @@ RTDECL(int) RTStrValidateEncodingEx(const char *psz, size_t cch, uint32_t fFlags
 
     return RTStrUniLenEx(psz, cch, &cCpsIgnored);
 }
+RT_EXPORT_SYMBOL(RTStrValidateEncodingEx);
 
 
 RTDECL(bool) RTStrIsValidEncoding(const char *psz)
@@ -328,6 +331,7 @@ RTDECL(bool) RTStrIsValidEncoding(const char *psz)
     int rc = RTStrValidateEncodingEx(psz, RTSTR_MAX, 0);
     return RT_SUCCESS(rc);
 }
+RT_EXPORT_SYMBOL(RTStrIsValidEncoding);
 
 
 RTDECL(int) RTStrToUni(const char *pszString, PRTUNICP *ppaCps)
@@ -355,7 +359,7 @@ RTDECL(int) RTStrToUni(const char *pszString, PRTUNICP *ppaCps)
             /*
              * Decode the string.
              */
-            rc = rtUtf8Decode(pszString, RTSTR_MAX, paCps, cCps, &cCps);
+            rc = rtUtf8Decode(pszString, RTSTR_MAX, paCps, cCps);
             if (RT_SUCCESS(rc))
             {
                 *ppaCps = paCps;
@@ -368,6 +372,7 @@ RTDECL(int) RTStrToUni(const char *pszString, PRTUNICP *ppaCps)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUni);
 
 
 RTDECL(int)  RTStrToUniEx(const char *pszString, size_t cchString, PRTUNICP *ppaCps, size_t cCps, size_t *pcCps)
@@ -413,7 +418,7 @@ RTDECL(int)  RTStrToUniEx(const char *pszString, size_t cchString, PRTUNICP *ppa
             /*
              * Encode the UTF-16 string.
              */
-            rc = rtUtf8Decode(pszString, cchString, paCpsResult, cCps - 1, &cCpsResult);
+            rc = rtUtf8Decode(pszString, cchString, paCpsResult, cCps - 1);
             if (RT_SUCCESS(rc))
             {
                 *ppaCps = paCpsResult;
@@ -427,6 +432,7 @@ RTDECL(int)  RTStrToUniEx(const char *pszString, size_t cchString, PRTUNICP *ppa
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUniEx);
 
 
 /**
@@ -575,15 +581,12 @@ static int rtUtf8CalcUtf16Length(const char *psz, size_t cch, size_t *pcwc)
  *                  The recoding will stop when cch or '\\0' is reached. Pass RTSTR_MAX to process up to '\\0'.
  * @param   pwsz    Where to store the UTF-16 string.
  * @param   cwc     The number of RTUTF16 items the pwsz buffer can hold, excluding the terminator ('\\0').
- * @param   pcwc    Where to store the actual number of RTUTF16 items encoded into the UTF-16. This excludes the terminator.
  */
-static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_t cwc, size_t *pcwc)
+static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_t cwc)
 {
-    int                     rc = VINF_SUCCESS;
+    int                     rc   = VINF_SUCCESS;
     const unsigned char    *puch = (const unsigned char *)psz;
-    const PRTUTF16          pwszEnd = pwsz + cwc;
-    PRTUTF16                pwc = pwsz;
-    Assert(pwszEnd >= pwc);
+    PRTUTF16                pwc  = pwsz;
     while (cch > 0)
     {
         /* read the next char and check for terminator. */
@@ -592,11 +595,12 @@ static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_
             break;
 
         /* check for output overflow */
-        if (pwc >= pwszEnd)
+        if (RT_UNLIKELY(cwc < 1))
         {
             rc = VERR_BUFFER_OVERFLOW;
             break;
         }
+        cwc--;
 
         /* decode and recode the code point */
         if (!(uch & RT_BIT(7)))
@@ -630,11 +634,13 @@ static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_
                        | ((RTUNICP)(puch[2] & 0x3f) << 6)
                        | ((RTUNICP)(puch[1] & 0x3f) << 12)
                        | ((RTUNICP)(uch     & 0x07) << 18);
-            if (pwc + 1 >= pwszEnd)
+            if (RT_UNLIKELY(cwc < 1))
             {
                 rc = VERR_BUFFER_OVERFLOW;
                 break;
             }
+            cwc--;
+
             uc -= 0x10000;
             *pwc++ = 0xd800 | (uc >> 10);
             *pwc++ = 0xdc00 | (uc & 0x3ff);
@@ -645,7 +651,6 @@ static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_
 
     /* done */
     *pwc = '\0';
-    *pcwc = pwc - pwsz;
     return rc;
 }
 
@@ -675,7 +680,7 @@ RTDECL(int) RTStrToUtf16(const char *pszString, PRTUTF16 *ppwszString)
             /*
              * Encode the UTF-16 string.
              */
-            rc = rtUtf8RecodeAsUtf16(pszString, RTSTR_MAX, pwsz, cwc, &cwc);
+            rc = rtUtf8RecodeAsUtf16(pszString, RTSTR_MAX, pwsz, cwc);
             if (RT_SUCCESS(rc))
             {
                 *ppwszString = pwsz;
@@ -688,6 +693,7 @@ RTDECL(int) RTStrToUtf16(const char *pszString, PRTUTF16 *ppwszString)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUtf16);
 
 
 RTDECL(int)  RTStrToUtf16Ex(const char *pszString, size_t cchString, PRTUTF16 *ppwsz, size_t cwc, size_t *pcwc)
@@ -733,7 +739,7 @@ RTDECL(int)  RTStrToUtf16Ex(const char *pszString, size_t cchString, PRTUTF16 *p
             /*
              * Encode the UTF-16 string.
              */
-            rc = rtUtf8RecodeAsUtf16(pszString, cchString, pwszResult, cwc - 1, &cwcResult);
+            rc = rtUtf8RecodeAsUtf16(pszString, cchString, pwszResult, cwc - 1);
             if (RT_SUCCESS(rc))
             {
                 *ppwsz = pwszResult;
@@ -747,6 +753,7 @@ RTDECL(int)  RTStrToUtf16Ex(const char *pszString, size_t cchString, PRTUTF16 *p
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUtf16Ex);
 
 
 RTDECL(size_t) RTStrCalcUtf16Len(const char *psz)
@@ -755,6 +762,7 @@ RTDECL(size_t) RTStrCalcUtf16Len(const char *psz)
     int rc = rtUtf8CalcUtf16Length(psz, RTSTR_MAX, &cwc);
     return RT_SUCCESS(rc) ? cwc : 0;
 }
+RT_EXPORT_SYMBOL(RTStrCalcUtf16Len);
 
 
 RTDECL(int) RTStrCalcUtf16LenEx(const char *psz, size_t cch, size_t *pcwc)
@@ -765,6 +773,7 @@ RTDECL(int) RTStrCalcUtf16LenEx(const char *psz, size_t cch, size_t *pcwc)
         *pcwc = RT_SUCCESS(rc) ? cwc : ~(size_t)0;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrCalcUtf16LenEx);
 
 
 /**
@@ -791,6 +800,7 @@ RTDECL(RTUNICP) RTStrGetCpInternal(const char *psz)
     RTStrGetCpExInternal(&psz, &Cp);
     return Cp;
 }
+RT_EXPORT_SYMBOL(RTStrGetCpInternal);
 
 
 RTDECL(int) RTStrGetCpExInternal(const char **ppsz, PRTUNICP pCp)
@@ -913,6 +923,7 @@ RTDECL(int) RTStrGetCpExInternal(const char **ppsz, PRTUNICP pCp)
     *ppsz = (const char *)puch;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTStrGetCpExInternal);
 
 
 /**
@@ -1068,6 +1079,7 @@ RTDECL(int) RTStrGetCpNExInternal(const char **ppsz, size_t *pcch, PRTUNICP pCp)
     (*pcch) = cch;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTStrGetCpNExInternal);
 
 
 RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
@@ -1105,7 +1117,7 @@ RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
     }
     else if (uc < 0x04000000)
     {
-        *puch++ = 0xf1 | (uc >> 24);
+        *puch++ = 0xf8 | (uc >> 24);
         *puch++ = 0x80 | ((uc >> 18) & 0x3f);
         *puch++ = 0x80 | ((uc >> 12) & 0x3f);
         *puch++ = 0x80 | ((uc >> 6) & 0x3f);
@@ -1113,7 +1125,7 @@ RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
     }
     else if (uc <= 0x7fffffff)
     {
-        *puch++ = 0xf3 | (uc >> 30);
+        *puch++ = 0xfc | (uc >> 30);
         *puch++ = 0x80 | ((uc >> 24) & 0x3f);
         *puch++ = 0x80 | ((uc >> 18) & 0x3f);
         *puch++ = 0x80 | ((uc >> 12) & 0x3f);
@@ -1128,6 +1140,7 @@ RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
 
     return (char *)puch;
 }
+RT_EXPORT_SYMBOL(RTStrPutCpInternal);
 
 
 RTDECL(char *) RTStrPrevCp(const char *pszStart, const char *psz)
@@ -1146,7 +1159,7 @@ RTDECL(char *) RTStrPrevCp(const char *pszStart, const char *psz)
         while (     (const unsigned char *)pszStart < puch
                &&   !(uMask & 1))
         {
-            unsigned uch = *--puch;
+            uch = *--puch;
             if ((uch & 0xc0) != 0x80)
             {
                 RTStrAssertMsgReturn((uch & (uMask >> 1)) == (uMask & 0xff),
@@ -1160,6 +1173,7 @@ RTDECL(char *) RTStrPrevCp(const char *pszStart, const char *psz)
     }
     return (char *)pszStart;
 }
+RT_EXPORT_SYMBOL(RTStrPrevCp);
 
 
 /**
@@ -1186,6 +1200,7 @@ RTDECL(int) RTStrCmp(const char *psz1, const char *psz2)
 
     return strcmp(psz1, psz2);
 }
+RT_EXPORT_SYMBOL(RTStrCmp);
 
 
 /**
@@ -1214,6 +1229,7 @@ RTDECL(int) RTStrNCmp(const char *psz1, const char *psz2, size_t cchMax)
 
     return strncmp(psz1, psz2, cchMax);
 }
+RT_EXPORT_SYMBOL(RTStrNCmp);
 
 
 /**
@@ -1288,6 +1304,7 @@ RTDECL(int) RTStrICmp(const char *psz1, const char *psz2)
     /* Hit some bad encoding, continue in case insensitive mode. */
     return RTStrCmp(psz1, psz2);
 }
+RT_EXPORT_SYMBOL(RTStrICmp);
 
 
 /**
@@ -1322,7 +1339,6 @@ RTDECL(int) RTStrNICmp(const char *psz1, const char *psz2, size_t cchMax)
     if (!psz2)
         return 1;
 
-    const char *pszStart1 = psz1;
     for (;;)
     {
         /* Get the codepoints */
@@ -1369,4 +1385,139 @@ RTDECL(int) RTStrNICmp(const char *psz1, const char *psz2, size_t cchMax)
     /* Hit some bad encoding, continue in case insensitive mode. */
     return RTStrNCmp(psz1, psz2, cchMax);
 }
+RT_EXPORT_SYMBOL(RTStrNICmp);
+
+
+RTDECL(char *) RTStrStr(const char *pszHaystack, const char *pszNeedle)
+{
+    /* Any NULL strings means NULL return. (In the RTStrCmp tradition.) */
+    if (!pszHaystack)
+        return NULL;
+    if (!pszNeedle)
+        return NULL;
+
+    /* The rest is CRT. */
+    return (char *)strstr(pszHaystack, pszNeedle);
+}
+RT_EXPORT_SYMBOL(RTStrStr);
+
+
+RTDECL(char *) RTStrIStr(const char *pszHaystack, const char *pszNeedle)
+{
+    /* Any NULL strings means NULL return. (In the RTStrCmp tradition.) */
+    if (!pszHaystack)
+        return NULL;
+    if (!pszNeedle)
+        return NULL;
+
+    /* The empty string matches everything. */
+    if (!*pszNeedle)
+        return (char *)pszHaystack;
+
+    /*
+     * The search strategy is to pick out the first char of the needle, fold it,
+     * and match it against the haystack code point by code point. When encountering
+     * a matching code point we use RTStrNICmp for the remainder (if any) of the needle.
+     */
+    const char * const pszNeedleStart = pszNeedle;
+    RTUNICP Cp0;
+    RTStrGetCpEx(&pszNeedle, &Cp0);     /* pszNeedle is advanced one code point. */
+    size_t const    cchNeedle   = strlen(pszNeedle);
+    size_t const    cchNeedleCp0= pszNeedle - pszNeedleStart;
+    RTUNICP const   Cp0Lower    = RTUniCpToLower(Cp0);
+    RTUNICP const   Cp0Upper    = RTUniCpToUpper(Cp0);
+    if (    Cp0Lower == Cp0Upper
+        &&  Cp0Lower == Cp0)
+    {
+        /* Cp0 is not a case sensitive char. */
+        for (;;)
+        {
+            RTUNICP Cp;
+            RTStrGetCpEx(&pszHaystack, &Cp);
+            if (!Cp)
+                break;
+            if (    Cp == Cp0
+                &&  !RTStrNICmp(pszHaystack, pszNeedle, cchNeedle))
+                return (char *)pszHaystack - cchNeedleCp0;
+        }
+    }
+    else if (   Cp0Lower == Cp0
+             || Cp0Upper != Cp0)
+    {
+        /* Cp0 is case sensitive */
+        for (;;)
+        {
+            RTUNICP Cp;
+            RTStrGetCpEx(&pszHaystack, &Cp);
+            if (!Cp)
+                break;
+            if (    (   Cp == Cp0Upper
+                     || Cp == Cp0Lower)
+                &&  !RTStrNICmp(pszHaystack, pszNeedle, cchNeedle))
+                return (char *)pszHaystack - cchNeedleCp0;
+        }
+    }
+    else
+    {
+        /* Cp0 is case sensitive and folds to two difference chars. (paranoia) */
+        for (;;)
+        {
+            RTUNICP Cp;
+            RTStrGetCpEx(&pszHaystack, &Cp);
+            if (!Cp)
+                break;
+            if (    (   Cp == Cp0
+                     || Cp == Cp0Upper
+                     || Cp == Cp0Lower)
+                &&  !RTStrNICmp(pszHaystack, pszNeedle, cchNeedle))
+                return (char *)pszHaystack - cchNeedleCp0;
+        }
+    }
+
+
+    return NULL;
+}
+RT_EXPORT_SYMBOL(RTStrIStr);
+
+
+RTDECL(char *) RTStrToLower(char *psz)
+{
+    /*
+     * Loop the code points in the string, converting them one by one.
+     * ASSUMES that the code points for upper and lower case are encoded
+     *         with the exact same length.
+     */
+    /** @todo Handled bad encodings correctly+quietly, remove assumption,
+     *        optimize. */
+    char *pszCur = psz;
+    while (*pszCur)
+    {
+        RTUNICP cp = RTStrGetCp(pszCur);
+        cp = RTUniCpToLower(cp);
+        pszCur = RTStrPutCp(pszCur, cp);
+    }
+    return psz;
+}
+RT_EXPORT_SYMBOL(RTStrToLower);
+
+
+RTDECL(char *) RTStrToUpper(char *psz)
+{
+    /*
+     * Loop the code points in the string, converting them one by one.
+     * ASSUMES that the code points for upper and lower case are encoded
+     *         with the exact same length.
+     */
+    /** @todo Handled bad encodings correctly+quietly, remove assumption,
+     *        optimize. */
+    char *pszCur = psz;
+    while(*pszCur)
+    {
+        RTUNICP cp = RTStrGetCp(pszCur);
+        cp = RTUniCpToUpper(cp);
+        pszCur = RTStrPutCp(pszCur, cp);
+    }
+    return psz;
+}
+RT_EXPORT_SYMBOL(RTStrToUpper);
 
