@@ -1,4 +1,4 @@
-/* $Id: RTLogDefaultInit-generic.cpp 29978 2008-04-21 17:24:28Z umoeller $ */
+/* $Id: RTLogDefaultInit-generic.cpp $ */
 /** @file
  * IPRT - Default Log Init, Generic Dummy.
  */
@@ -28,11 +28,16 @@
  * additional information or have any questions.
  */
 
+
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
 #include <iprt/log.h>
 
+
 RTDECL(PRTLOGGER) RTLogDefaultInit(void)
 {
     return NULL;
 }
-
+RT_EXPORT_SYMBOL(RTLogDefaultInit);
 
