@@ -1,4 +1,4 @@
-/* $Id: RTLogWriteStdErr-generic.cpp 29978 2008-04-21 17:24:28Z umoeller $ */
+/* $Id: RTLogWriteStdErr-generic.cpp $ */
 /** @file
  * IPRT - Log To StdErr, Generic.
  */
@@ -28,11 +28,17 @@
  * additional information or have any questions.
  */
 
+
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
 #include <iprt/log.h>
+#include "internal/iprt.h"
 #include <stdio.h>
 
 RTDECL(void) RTLogWriteStdErr(const char *pch, size_t cb)
 {
     fwrite(pch, 1, cb, stderr);
 }
+RT_EXPORT_SYMBOL(RTLogWriteStdErr);
 
