@@ -28,12 +28,19 @@
  * additional information or have any questions.
  */
 
+
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
 #include <iprt/log.h>
+#include "internal/iprt.h"
 #include <stdio.h>
 
+
 RTDECL(void) RTLogWriteStdOut(const char *pch, size_t cb)
 {
     fwrite(pch, 1, cb, stdout);
     fflush(stdout);
 }
+RT_EXPORT_SYMBOL(RTLogWriteStdOut);
 
