@@ -33,10 +33,12 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/mp.h>
+#include "internal/iprt.h"
 
 
 RTDECL(int) RTMpCpuIdToSetIndex(RTCPUID idCpu)
 {
     return idCpu != NIL_RTCPUID ? (int) idCpu : -1;
 }
+RT_EXPORT_SYMBOL(RTMpCpuIdToSetIndex);
 
