@@ -28,10 +28,13 @@
  * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/critsect.h>
+#include "internal/iprt.h"
+
 #include <iprt/semaphore.h>
 #include <iprt/thread.h>
 #include <iprt/assert.h>
@@ -54,6 +57,7 @@ RTDECL(int) RTCritSectInit(PRTCRITSECT pCritSect)
 {
     return RTCritSectInitEx(pCritSect, 0);
 }
+RT_EXPORT_SYMBOL(RTCritSectInit);
 
 
 /**
@@ -86,6 +90,7 @@ RTDECL(int) RTCritSectInitEx(PRTCRITSECT pCritSect, uint32_t fFlags)
     pCritSect->u32Magic = (uint32_t)rc;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTCritSectInitEx);
 
 
 /**
@@ -197,6 +202,7 @@ RTDECL(int) RTCritSectEnterMultipleDebug(unsigned cCritSects, PRTCRITSECT *papCr
         i = j;
     }
 }
+RT_EXPORT_SYMBOL(RTCritSectEnterMultiple);
 
 
 /**
@@ -260,6 +266,7 @@ RTDECL(int) RTCritSectTryEnterDebug(PRTCRITSECT pCritSect, const char *pszFile,
 
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTCritSectTryEnter);
 
 
 /**
@@ -307,12 +314,10 @@ RTDECL(int) RTCritSectEnterDebug(PRTCRITSECT pCritSect, const char *pszFile, uns
                 pCritSect->cNestings++;
                 return VINF_SUCCESS;
             }
-            else
-            {
-                AssertMsgFailed(("Nested entry of critsect %p\n", pCritSect));
-                ASMAtomicDecS32(&pCritSect->cLockers);
-                return VERR_SEM_NESTED;
-            }
+
+            AssertBreakpoint(); /* don't do normal assertion here, the logger uses this code too. */
+            ASMAtomicDecS32(&pCritSect->cLockers);
+            return VERR_SEM_NESTED;
         }
 
         for (;;)
@@ -348,6 +353,7 @@ RTDECL(int) RTCritSectEnterDebug(PRTCRITSECT pCritSect, const char *pszFile, uns
 
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTCritSectEnter);
 
 
 /**
@@ -393,6 +399,7 @@ RTDECL(int) RTCritSectLeave(PRTCRITSECT pCritSect)
     }
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTCritSectLeave);
 
 
 /**
@@ -413,6 +420,7 @@ RTDECL(int) RTCritSectLeaveMultiple(unsigned cCritSects, PRTCRITSECT *papCritSec
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTCritSectLeaveMultiple);
 
 
 #ifndef RTCRITSECT_STRICT
@@ -431,6 +439,9 @@ RTDECL(int) RTCritSectEnterMultipleDebug(unsigned cCritSects, PRTCRITSECT *papCr
     return RTCritSectEnterMultiple(cCritSects, papCritSects);
 }
 #endif /* RT_STRICT */
+RT_EXPORT_SYMBOL(RTCritSectEnterDebug);
+RT_EXPORT_SYMBOL(RTCritSectTryEnterDebug);
+RT_EXPORT_SYMBOL(RTCritSectEnterMultipleDebug);
 
 
 /**
@@ -459,7 +470,7 @@ RTDECL(int) RTCritSectDelete(PRTCRITSECT pCritSect)
     pCritSect->cNestings        = 0;
     pCritSect->NativeThreadOwner= NIL_RTNATIVETHREAD;
     RTSEMEVENT EventSem = pCritSect->EventSem;
-    pCritSect->EventSem         = NULL;
+    pCritSect->EventSem         = NIL_RTSEMEVENT;
     while (pCritSect->cLockers-- >= 0)
         RTSemEventSignal(EventSem);
     ASMAtomicWriteS32(&pCritSect->cLockers, -1);
@@ -468,4 +479,5 @@ RTDECL(int) RTCritSectDelete(PRTCRITSECT pCritSect)
 
     return rc;
 }
+RT_EXPORT_SYMBOL(RTCritSectDelete);
 
