@@ -1,4 +1,4 @@
-/* $Id: alloc-r0drv.cpp 29978 2008-04-21 17:24:28Z umoeller $ */
+/* $Id: alloc-r0drv.cpp $ */
 /** @file
  * IPRT - Memory Allocation, Ring-0 Driver.
  */
@@ -32,10 +32,14 @@
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include <iprt/string.h>
-#include <iprt/alloc.h>
+#include <iprt/mem.h>
+#include "internal/iprt.h"
+
+#include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/param.h>
+#include <iprt/string.h>
+#include <iprt/thread.h>
 #include "r0drv/alloc-r0drv.h"
 
 
@@ -79,10 +83,11 @@ static uint8_t const g_abFence[RTR0MEM_FENCE_EXTRA] =
  * @returns NULL on failure.
  * @param   cb      Size in bytes of the memory block to allocated.
  */
-RTDECL(void *)  RTMemTmpAlloc(size_t cb)
+RTDECL(void *)  RTMemTmpAlloc(size_t cb) RT_NO_THROW
 {
     return RTMemAlloc(cb);
 }
+RT_EXPORT_SYMBOL(RTMemTmpAlloc);
 
 
 /**
@@ -94,10 +99,11 @@ RTDECL(void *)  RTMemTmpAlloc(size_t cb)
  * @returns NULL on failure.
  * @param   cb      Size in bytes of the memory block to allocated.
  */
-RTDECL(void *)  RTMemTmpAllocZ(size_t cb)
+RTDECL(void *)  RTMemTmpAllocZ(size_t cb) RT_NO_THROW
 {
     return RTMemAllocZ(cb);
 }
+RT_EXPORT_SYMBOL(RTMemTmpAllocZ);
 
 
 /**
@@ -105,10 +111,11 @@ RTDECL(void *)  RTMemTmpAllocZ(size_t cb)
  *
  * @param   pv      Pointer to memory block.
  */
-RTDECL(void)    RTMemTmpFree(void *pv)
+RTDECL(void)    RTMemTmpFree(void *pv) RT_NO_THROW
 {
     return RTMemFree(pv);
 }
+RT_EXPORT_SYMBOL(RTMemTmpFree);
 
 
 /**
@@ -118,19 +125,23 @@ RTDECL(void)    RTMemTmpFree(void *pv)
  * @returns NULL on failure.
  * @param   cb      Size in bytes of the memory block to allocated.
  */
-RTDECL(void *)  RTMemAlloc(size_t cb)
+RTDECL(void *)  RTMemAlloc(size_t cb) RT_NO_THROW
 {
-    PRTMEMHDR pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, 0);
+    PRTMEMHDR pHdr;
+    RT_ASSERT_INTS_ON();
+
+    pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, 0);
     if (pHdr)
     {
 #ifdef RTR0MEM_STRICT
-        pHdr->cbReq = cb;
+        pHdr->cbReq = (uint32_t)cb; Assert(pHdr->cbReq == cb);
         memcpy((uint8_t *)(pHdr + 1) + cb, &g_abFence[0], RTR0MEM_FENCE_EXTRA);
 #endif
         return pHdr + 1;
     }
     return NULL;
 }
+RT_EXPORT_SYMBOL(RTMemAlloc);
 
 
 /**
@@ -144,13 +155,16 @@ RTDECL(void *)  RTMemAlloc(size_t cb)
  * @returns NULL on failure.
  * @param   cb      Size in bytes of the memory block to allocated.
  */
-RTDECL(void *)  RTMemAllocZ(size_t cb)
+RTDECL(void *)  RTMemAllocZ(size_t cb) RT_NO_THROW
 {
-    PRTMEMHDR pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_ZEROED);
+    PRTMEMHDR pHdr;
+    RT_ASSERT_INTS_ON();
+
+    pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_ZEROED);
     if (pHdr)
     {
 #ifdef RTR0MEM_STRICT
-        pHdr->cbReq = cb;
+        pHdr->cbReq = (uint32_t)cb; Assert(pHdr->cbReq == cb);
         memcpy((uint8_t *)(pHdr + 1) + cb, &g_abFence[0], RTR0MEM_FENCE_EXTRA);
         return memset(pHdr + 1, 0, cb);
 #else
@@ -159,6 +173,7 @@ RTDECL(void *)  RTMemAllocZ(size_t cb)
     }
     return NULL;
 }
+RT_EXPORT_SYMBOL(RTMemAllocZ);
 
 
 /**
@@ -169,7 +184,7 @@ RTDECL(void *)  RTMemAllocZ(size_t cb)
  * @param   pvOld   The memory block to reallocate.
  * @param   cbNew   The new block size (in bytes).
  */
-RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew)
+RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew) RT_NO_THROW
 {
     if (!cbNew)
         RTMemFree(pvOld);
@@ -178,6 +193,8 @@ RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew)
     else
     {
         PRTMEMHDR pHdrOld = (PRTMEMHDR)pvOld - 1;
+        RT_ASSERT_PREEMPTIBLE();
+
         if (pHdrOld->u32Magic == RTMEMHDR_MAGIC)
         {
             PRTMEMHDR pHdrNew;
@@ -189,7 +206,7 @@ RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew)
                 size_t cbCopy = RT_MIN(pHdrOld->cb, pHdrNew->cb);
                 memcpy(pHdrNew + 1, pvOld, cbCopy);
 #ifdef RTR0MEM_STRICT
-                pHdrNew->cbReq = cbNew;
+                pHdrNew->cbReq = (uint32_t)cbNew; Assert(pHdrNew->cbReq == cbNew);
                 memcpy((uint8_t *)(pHdrNew + 1) + cbNew, &g_abFence[0], RTR0MEM_FENCE_EXTRA);
                 AssertReleaseMsg(!memcmp((uint8_t *)(pHdrOld + 1) + pHdrOld->cbReq, &g_abFence[0], RTR0MEM_FENCE_EXTRA),
                                  ("pHdr=%p pvOld=%p cb=%zu cbNew=%zu\n"
@@ -209,6 +226,7 @@ RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew)
 
     return NULL;
 }
+RT_EXPORT_SYMBOL(RTMemRealloc);
 
 
 /**
@@ -216,9 +234,11 @@ RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew)
  *
  * @param   pv      Pointer to memory block.
  */
-RTDECL(void) RTMemFree(void *pv)
+RTDECL(void) RTMemFree(void *pv) RT_NO_THROW
 {
     PRTMEMHDR pHdr;
+    RT_ASSERT_INTS_ON();
+
     if (!pv)
         return;
     pHdr = (PRTMEMHDR)pv - 1;
@@ -230,7 +250,7 @@ RTDECL(void) RTMemFree(void *pv)
                          ("pHdr=%p pv=%p cb=%zu\n"
                           "fence:    %.*Rhxs\n"
                           "expected: %.*Rhxs\n",
-                          pHdr, pv, pHdr->cb, pv,
+                          pHdr, pv, pHdr->cb,
                           RTR0MEM_FENCE_EXTRA, (uint8_t *)(pHdr + 1) + pHdr->cb,
                           RTR0MEM_FENCE_EXTRA, &g_abFence[0]));
 #endif
@@ -239,6 +259,7 @@ RTDECL(void) RTMemFree(void *pv)
     else
         AssertMsgFailed(("pHdr->u32Magic=%RX32 pv=%p\n", pHdr->u32Magic, pv));
 }
+RT_EXPORT_SYMBOL(RTMemFree);
 
 
 /**
@@ -248,19 +269,27 @@ RTDECL(void) RTMemFree(void *pv)
  * @returns NULL on failure.
  * @param   cb      Size in bytes of the memory block to allocate.
  */
-RTDECL(void *)    RTMemExecAlloc(size_t cb)
+RTDECL(void *)    RTMemExecAlloc(size_t cb) RT_NO_THROW
 {
-    PRTMEMHDR pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_EXEC);
+    PRTMEMHDR pHdr;
+#ifdef RT_OS_SOLARIS /** @todo figure out why */
+    RT_ASSERT_INTS_ON();
+#else
+    RT_ASSERT_PREEMPTIBLE();
+#endif
+
+    pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_EXEC);
     if (pHdr)
     {
 #ifdef RTR0MEM_STRICT
-        pHdr->cbReq = cb;
+        pHdr->cbReq = (uint32_t)cb; Assert(pHdr->cbReq == cb);
         memcpy((uint8_t *)(pHdr + 1) + cb, &g_abFence[0], RTR0MEM_FENCE_EXTRA);
 #endif
         return pHdr + 1;
     }
     return NULL;
 }
+RT_EXPORT_SYMBOL(RTMemExecAlloc);
 
 
 /**
@@ -268,9 +297,11 @@ RTDECL(void *)    RTMemExecAlloc(size_t cb)
  *
  * @param   pv      Pointer to memory block.
  */
-RTDECL(void)      RTMemExecFree(void *pv)
+RTDECL(void)      RTMemExecFree(void *pv) RT_NO_THROW
 {
     PRTMEMHDR pHdr;
+    RT_ASSERT_INTS_ON();
+
     if (!pv)
         return;
     pHdr = (PRTMEMHDR)pv - 1;
@@ -290,4 +321,5 @@ RTDECL(void)      RTMemExecFree(void *pv)
     else
         AssertMsgFailed(("pHdr->u32Magic=%RX32 pv=%p\n", pHdr->u32Magic, pv));
 }
+RT_EXPORT_SYMBOL(RTMemExecFree);
 
