@@ -58,8 +58,11 @@
 #include <kern/clock.h>
 #include <kern/sched_prim.h>
 #include <kern/locks.h>
-#include <i386/mp_events.h>
+#if defined(RT_ARCH_X86) && MAC_OS_X_VERSION_MIN_REQUIRED < 1060
+# include <i386/mp_events.h>
+#endif
 #include <libkern/libkern.h>
+#include <libkern/sysctl.h>
 #include <mach/thread_act.h>
 #include <mach/vm_map.h>
 #include <pexpert/pexpert.h>
@@ -71,10 +74,20 @@
 #include <IOKit/IOTypes.h>
 #include <IOKit/IOLib.h>
 #include <IOKit/IOMemoryDescriptor.h>
+#include <IOKit/IOBufferMemoryDescriptor.h>
 #include <IOKit/IOMapper.h>
 
 
-__BEGIN_DECLS
+/* See osfmk/kern/ast.h. */
+#ifndef AST_PREEMPT
+# define AST_PREEMPT    UINT32_C(1)
+# define AST_QUANTUM    UINT32_C(2)
+# define AST_URGENT     UINT32_C(4)
+#endif
+
+
+RT_C_DECLS_BEGIN
+
 /* mach/vm_types.h */
 typedef struct pmap *pmap_t;
 
@@ -104,19 +117,24 @@ extern void mp_rendezvous_no_intrs(void (*)(void *), void *);
 /* osfmk/i386/cpu_number.h */
 extern int cpu_number(void);
 
+/* osfmk/vm/vm_user.c */
+extern kern_return_t vm_protect(vm_map_t, vm_offset_t, vm_size_t, boolean_t, vm_prot_t);
+
 /* i386/machine_routines.h */
 extern int ml_get_max_cpus(void);
 
-__END_DECLS
+RT_C_DECLS_END
 
 
 /*
  * Internals of the Darwin Ring-0 IPRT.
  */
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 extern lck_grp_t *g_pDarwinLockGroup;
-__END_DECLS
+int  rtThreadPreemptDarwinInit(void);
+void rtThreadPreemptDarwinTerm(void);
+RT_C_DECLS_END
 
 
 /**
