@@ -33,9 +33,13 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/initterm.h>
+#include "internal/iprt.h"
+
 #include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/err.h>
+#include <iprt/mp.h>
+#include <iprt/thread.h>
 #ifndef IN_GUEST /* play safe for now */
 # include "r0drv/mp-r0drv.h"
 # include "r0drv/power-r0drv.h"
@@ -65,6 +69,7 @@ RTR0DECL(int) RTR0Init(unsigned fReserved)
 {
     int rc;
     Assert(fReserved == 0);
+    RT_ASSERT_PREEMPTIBLE();
 
     /*
      * The first user initializes it.
@@ -77,7 +82,7 @@ RTR0DECL(int) RTR0Init(unsigned fReserved)
     rc = rtR0InitNative();
     if (RT_SUCCESS(rc))
     {
-#if !defined(RT_OS_LINUX) && !defined(RT_OS_WINDOWS)
+#if !defined(RT_OS_LINUX) /** @todo implement thread2-r0drv-linux.c */
         rc = rtThreadInit();
 #endif
         if (RT_SUCCESS(rc))
@@ -85,32 +90,30 @@ RTR0DECL(int) RTR0Init(unsigned fReserved)
 #ifndef IN_GUEST /* play safe for now */
             rc = rtR0MpNotificationInit();
             if (RT_SUCCESS(rc))
+            {
                 rc = rtR0PowerNotificationInit();
-#endif
+                if (RT_SUCCESS(rc))
+                    return rc;
+                rtR0MpNotificationTerm();
+            }
+#else
             if (RT_SUCCESS(rc))
                 return rc;
+#endif
+#if !defined(RT_OS_LINUX) /** @todo implement thread2-r0drv-linux.c */
+            rtThreadTerm();
+#endif
         }
-
         rtR0TermNative();
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTR0Init);
 
 
-/**
- * Terminates the ring-0 driver runtime library.
- */
-RTR0DECL(void) RTR0Term(void)
+static void rtR0Term(void)
 {
-    /*
-     * Last user does the cleanup.
-     */
-    int32_t cNewUsers = ASMAtomicDecS32(&g_crtR0Users);
-    Assert(cNewUsers >= 0);
-    if (cNewUsers != 0)
-        return;
-
-#if !defined(RT_OS_LINUX) && !defined(RT_OS_WINDOWS)
+#if !defined(RT_OS_LINUX) /** @todo implement thread2-r0drv-linux.c */
     rtThreadTerm();
 #endif
 #ifndef IN_GUEST /* play safe for now */
@@ -120,3 +123,29 @@ RTR0DECL(void) RTR0Term(void)
     rtR0TermNative();
 }
 
+
+/**
+ * Terminates the ring-0 driver runtime library.
+ */
+RTR0DECL(void) RTR0Term(void)
+{
+    int32_t cNewUsers;
+    RT_ASSERT_PREEMPTIBLE();
+
+    cNewUsers = ASMAtomicDecS32(&g_crtR0Users);
+    Assert(cNewUsers >= 0);
+    if (cNewUsers == 0)
+        rtR0Term();
+}
+RT_EXPORT_SYMBOL(RTR0Term);
+
+
+/* Note! Should *not* be exported since it's only for static linking. */
+RTR0DECL(void) RTR0TermForced(void)
+{
+    RT_ASSERT_PREEMPTIBLE();
+    AssertMsg(g_crtR0Users == 1, ("%d\n", g_crtR0Users));
+
+    rtR0Term();
+}
+
