@@ -31,8 +31,6 @@
 #ifndef ___the_solaris_kernel_h
 #define ___the_solaris_kernel_h
 
-#define _MACHDEP /* needed for cpuset_t and sys/x_call.h */
-
 #include <sys/kmem.h>
 #include <sys/types.h>
 #include <sys/mman.h>
@@ -48,6 +46,7 @@
 #include <sys/cyclic.h>
 #include <sys/class.h>
 #include <sys/cpuvar.h>
+#include <sys/archsystm.h>
 #include <sys/x_call.h> /* in platform dir */
 #include <sys/x86_archext.h>
 #include <vm/hat.h>
@@ -55,13 +54,20 @@
 #include <vm/seg_kmem.h>
 #include <sys/ddi.h>
 #include <sys/sunddi.h>
+#include <sys/spl.h>
+#include <sys/archsystm.h>
+#include "vbi.h"
 
 #undef u /* /usr/include/sys/user.h:249:1 is where this is defined to (curproc->p_user). very cool. */
 
 #include <iprt/cdefs.h>
+#include <iprt/types.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
+extern bool g_frtSolarisSplSetsEIF;
 extern struct ddi_dma_attr g_SolarisX86PhysMemLimits;
-__END_DECLS
+extern uintptr_t kernelbase;
+extern RTCPUSET g_rtMpSolarisCpuSet;
+RT_C_DECLS_END
 
 #endif
