@@ -32,13 +32,14 @@
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include "the-solaris-kernel.h"
-
+#include "../the-solaris-kernel.h"
+#include "internal/iprt.h"
 #include <iprt/memobj.h>
-#include <iprt/mem.h>
-#include <iprt/err.h>
+
 #include <iprt/assert.h>
+#include <iprt/err.h>
 #include <iprt/log.h>
+#include <iprt/mem.h>
 #include <iprt/param.h>
 #include <iprt/process.h>
 #include "internal/memobj.h"
@@ -57,7 +58,9 @@ typedef struct RTR0MEMOBJSOLARIS
     /** Pointer to kernel memory cookie. */
     ddi_umem_cookie_t   Cookie;
     /** Shadow locked pages. */
-    void              *handle;
+    void               *pvHandle;
+    /** Access during locking. */
+    int                 fAccess;
 } RTR0MEMOBJSOLARIS, *PRTR0MEMOBJSOLARIS;
 
 
@@ -68,6 +71,10 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
 
     switch (pMemSolaris->Core.enmType)
     {
+        case RTR0MEMOBJTYPE_LOW:
+            vbi_lowmem_free(pMemSolaris->Core.pv, pMemSolaris->Core.cb);
+            break;
+
         case RTR0MEMOBJTYPE_CONT:
             vbi_contig_free(pMemSolaris->Core.pv, pMemSolaris->Core.cb);
             break;
@@ -77,17 +84,24 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
             break;
 
         case RTR0MEMOBJTYPE_LOCK:
-            vbi_unlock_va(pMemSolaris->Core.pv, pMemSolaris->Core.cb, pMemSolaris->handle);
+            vbi_unlock_va(pMemSolaris->Core.pv, pMemSolaris->Core.cb, pMemSolaris->fAccess, pMemSolaris->pvHandle);
             break;
 
         case RTR0MEMOBJTYPE_MAPPING:
             vbi_unmap(pMemSolaris->Core.pv, pMemSolaris->Core.cb);
             break;
 
-        /* unused */
-        case RTR0MEMOBJTYPE_LOW:
-        case RTR0MEMOBJTYPE_PHYS:
         case RTR0MEMOBJTYPE_RES_VIRT:
+        {
+            if (pMemSolaris->Core.u.ResVirt.R0Process == NIL_RTR0PROCESS)
+                vmem_xfree(heap_arena, pMemSolaris->Core.pv, pMemSolaris->Core.cb);
+            else
+                AssertFailed();
+            break;
+        }
+
+        /* unused */
+        case RTR0MEMOBJTYPE_PHYS:
         default:
             AssertMsgFailed(("enmType=%d\n", pMemSolaris->Core.enmType));
             return VERR_INTERNAL_ERROR;
@@ -112,7 +126,7 @@ int rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecu
     }
 
     pMemSolaris->Core.pv = virtAddr;
-    pMemSolaris->handle = NULL;
+    pMemSolaris->pvHandle = NULL;
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
 }
@@ -120,21 +134,26 @@ int rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecu
 
 int rtR0MemObjNativeAllocLow(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
 {
-    /* Try page alloc first */
-    int rc = rtR0MemObjNativeAllocPage(ppMem, cb, fExecutable);
-    if (RT_SUCCESS(rc))
+    NOREF(fExecutable);
+
+    /* Create the object */
+    PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_LOW, NULL, cb);
+    if (!pMemSolaris)
+        return VERR_NO_MEMORY;
+
+    /* Allocate physically low page-aligned memory. */
+    caddr_t virtAddr;
+    uint64_t phys = (unsigned)0xffffffff;
+    virtAddr = vbi_lowmem_alloc(phys, cb);
+    if (virtAddr == NULL)
     {
-        size_t iPage = cb >> PAGE_SHIFT;
-        while (iPage-- > 0)
-            if (rtR0MemObjNativeGetPagePhysAddr(*ppMem, iPage) > (_4G - PAGE_SIZE))
-            {
-                /* Failed! Fall back to physical contiguous alloc */
-                RTR0MemObjFree(*ppMem, false);
-                rc = rtR0MemObjNativeAllocCont(ppMem, cb, fExecutable);
-                break;
-            }
+        rtR0MemObjDelete(&pMemSolaris->Core);
+        return VERR_NO_LOW_MEMORY;
     }
-    return rc;
+    pMemSolaris->Core.pv = virtAddr;
+    pMemSolaris->pvHandle = NULL;
+    *ppMem = &pMemSolaris->Core;
+    return VINF_SUCCESS;
 }
 
 
@@ -159,7 +178,7 @@ int rtR0MemObjNativeAllocCont(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecu
     Assert(phys < (uint64_t)1 << 32);
     pMemSolaris->Core.pv = virtAddr;
     pMemSolaris->Core.u.Cont.Phys = phys;
-    pMemSolaris->handle = NULL;
+    pMemSolaris->pvHandle = NULL;
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
 }
@@ -195,48 +214,65 @@ int rtR0MemObjNativeEnterPhys(PPRTR0MEMOBJINTERNAL ppMem, RTHCPHYS Phys, size_t
 }
 
 
-int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, RTR0PROCESS R0Process)
+int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, uint32_t fAccess, RTR0PROCESS R0Process)
 {
     AssertReturn(R0Process == RTR0ProcHandleSelf(), VERR_INVALID_PARAMETER);
+    NOREF(fAccess);
 
     /* Create the locking object */
     PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_LOCK, (void *)R3Ptr, cb);
     if (!pMemSolaris)
         return VERR_NO_MEMORY;
 
-    void *ppl;
+    int fPageAccess = S_READ;
+    if (fAccess & RTMEM_PROT_WRITE)
+        fPageAccess = S_WRITE;
+    if (fAccess & RTMEM_PROT_EXEC)
+        fPageAccess = S_EXEC;
+    void *pvPageList = NULL;
 
     /* Lock down user pages */
-    int rc = vbi_lock_va((caddr_t)R3Ptr, cb, &ppl);
+    int rc = vbi_lock_va((caddr_t)R3Ptr, cb, fPageAccess, &pvPageList);
     if (rc != 0)
     {
         cmn_err(CE_NOTE,"rtR0MemObjNativeLockUser: vbi_lock_va failed rc=%d\n", rc);
+        rtR0MemObjDelete(&pMemSolaris->Core);
         return VERR_LOCK_FAILED;
     }
 
     pMemSolaris->Core.u.Lock.R0Process = (RTR0PROCESS)vbi_proc();
-    pMemSolaris->handle = ppl;
+    pMemSolaris->pvHandle = pvPageList;
+    pMemSolaris->fAccess = fPageAccess;
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
 }
 
 
-int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb)
+int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb, uint32_t fAccess)
 {
+    NOREF(fAccess);
+
     PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_LOCK, pv, cb);
     if (!pMemSolaris)
         return VERR_NO_MEMORY;
 
-    void *ppl;
-    int rc = vbi_lock_va((caddr_t)pv, cb, &ppl);
+    int fPageAccess = S_READ;
+    if (fAccess & RTMEM_PROT_WRITE)
+        fPageAccess = S_WRITE;
+    if (fAccess & RTMEM_PROT_EXEC)
+        fPageAccess = S_EXEC;
+    void *pvPageList = NULL;
+    int rc = vbi_lock_va((caddr_t)pv, cb, fPageAccess, &pvPageList);
     if (rc != 0)
     {
         cmn_err(CE_NOTE,"rtR0MemObjNativeLockKernel: vbi_lock_va failed rc=%d\n", rc);
+        rtR0MemObjDelete(&pMemSolaris->Core);
         return VERR_LOCK_FAILED;
     }
 
     pMemSolaris->Core.u.Lock.R0Process = NIL_RTR0PROCESS;
-    pMemSolaris->handle = ppl;
+    pMemSolaris->pvHandle = pvPageList;
+    pMemSolaris->fAccess = fPageAccess;
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
 }
@@ -244,7 +280,26 @@ int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb)
 
 int rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixed, size_t cb, size_t uAlignment)
 {
-    return VERR_NOT_IMPLEMENTED;
+    PRTR0MEMOBJSOLARIS  pMemSolaris;
+    void               *pv;
+
+    /*
+     * Use xalloc.
+     */
+    pv = vmem_xalloc(heap_arena, cb, uAlignment, 0 /*phase*/, 0 /*nocross*/,
+                     NULL /*minaddr*/, NULL /*maxaddr*/, VM_SLEEP);
+    if (!pv)
+        return VERR_NO_MEMORY;
+    pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_RES_VIRT, pv, cb);
+    if (!pMemSolaris)
+    {
+        vmem_xfree(heap_arena, pv, cb);
+        return VERR_NO_MEMORY;
+    }
+
+    pMemSolaris->Core.u.ResVirt.R0Process = NIL_RTR0PROCESS;
+    *ppMem = &pMemSolaris->Core;
+    return VINF_SUCCESS;
 }
 
 
@@ -256,15 +311,17 @@ int rtR0MemObjNativeReserveUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3PtrFixed,
 int rtR0MemObjNativeMapKernel(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ pMemToMap, void *pvFixed, size_t uAlignment,
                               unsigned fProt, size_t offSub, size_t cbSub)
 {
-    /* @todo rtR0MemObjNativeMapKernel / Solaris - Should be fairly simple alloc kernel memory and memload it. */
+    /** @todo rtR0MemObjNativeMapKernel / Solaris - Should be fairly simple alloc kernel memory and memload it. */
     return VERR_NOT_IMPLEMENTED;
 }
 
+
 int rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, PRTR0MEMOBJINTERNAL pMemToMap, RTR3PTR R3PtrFixed, size_t uAlignment, unsigned fProt, RTR0PROCESS R0Process)
 {
     AssertMsgReturn(R3PtrFixed == (RTR3PTR)-1, ("%p\n", R3PtrFixed), VERR_NOT_SUPPORTED);
     AssertMsgReturn(R0Process == RTR0ProcHandleSelf(), ("%p != %p\n", R0Process, RTR0ProcHandleSelf()), VERR_NOT_SUPPORTED);
-    AssertMsgReturn(uAlignment == 0 || uAlignment == PAGE_SIZE, ("%d\n", uAlignment), VERR_NOT_SUPPORTED);
+    if (uAlignment > PAGE_SIZE)
+        return VERR_NOT_SUPPORTED;
 
     PRTR0MEMOBJSOLARIS pMemToMapSolaris = (PRTR0MEMOBJSOLARIS)pMemToMap;
     size_t size = pMemToMapSolaris->Core.cb;
@@ -288,7 +345,7 @@ int rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, PRTR0MEMOBJINTERNAL pMem
         {
             cmn_err(CE_NOTE, "rtR0MemObjNativeMapUser: no page to map.\n");
             rc = VERR_MAP_FAILED;
-            goto done;
+            goto l_done;
         }
         pv = (void *)((uintptr_t)pv + PAGE_SIZE);
     }
@@ -299,7 +356,7 @@ int rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, PRTR0MEMOBJINTERNAL pMem
         cmn_err(CE_NOTE, "rtR0MemObjNativeMapUser: vbi failure.\n");
         rc = VERR_MAP_FAILED;
         rtR0MemObjDelete(&pMemSolaris->Core);
-        goto done;
+        goto l_done;
     }
     else
         rc = VINF_SUCCESS;
@@ -307,12 +364,22 @@ int rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, PRTR0MEMOBJINTERNAL pMem
     pMemSolaris->Core.u.Mapping.R0Process = (RTR0PROCESS)vbi_proc();
     pMemSolaris->Core.pv = addr;
     *ppMem = &pMemSolaris->Core;
-done:
+l_done:
     kmem_free(paddrs, sizeof(uint64_t) * cPages);
     return rc;
 }
 
 
+int rtR0MemObjNativeProtect(PRTR0MEMOBJINTERNAL pMem, size_t offSub, size_t cbSub, uint32_t fProt)
+{
+    NOREF(pMem);
+    NOREF(offSub);
+    NOREF(cbSub);
+    NOREF(fProt);
+    return VERR_NOT_SUPPORTED;
+}
+
+
 RTHCPHYS rtR0MemObjNativeGetPagePhysAddr(PRTR0MEMOBJINTERNAL pMem, size_t iPage)
 {
     PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)pMem;
