@@ -101,7 +101,7 @@ RTDECL(void *)  RTMemAlloc(size_t cb) RT_NO_THROW
 
     AssertMsg(cb, ("Allocating ZERO bytes is really not a good idea! Good luck with the next assertion!\n"));
     void *pv = malloc(cb);
-    AssertMsg(pv, ("malloc(%d) failed!!!\n", cb));
+    AssertMsg(pv, ("malloc(%#zx) failed!!!\n", cb));
 #ifdef RT_OS_OS2 /* temporary workaround until libc062. */
     AssertMsg(   cb < 32
               || !((uintptr_t)pv & (RTMEM_ALIGNMENT - 1)), ("pv=%p RTMEM_ALIGNMENT=%#x\n", pv, RTMEM_ALIGNMENT));
@@ -135,7 +135,7 @@ RTDECL(void *)  RTMemAllocZ(size_t cb) RT_NO_THROW
     AssertMsg(cb, ("Allocating ZERO bytes is really not a good idea! Good luck with the next assertion!\n"));
 
     void *pv = calloc(1, cb);
-    AssertMsg(pv, ("calloc(1,%d) failed!!!\n", cb));
+    AssertMsg(pv, ("calloc(1,%#zx) failed!!!\n", cb));
 #ifdef RT_OS_OS2 /* temporary workaround until libc062. */
     AssertMsg(   cb < 32
               || !((uintptr_t)pv & (RTMEM_ALIGNMENT - 1)), ("pv=%p RTMEM_ALIGNMENT=%#x\n", pv, RTMEM_ALIGNMENT));
@@ -164,7 +164,7 @@ RTDECL(void *)  RTMemRealloc(void *pvOld, size_t cbNew) RT_NO_THROW
 #else /* !RTALLOC_USE_EFENCE */
 
     void *pv = realloc(pvOld, cbNew);
-    AssertMsg(pv && cbNew, ("realloc(%p, %d) failed!!!\n", pvOld, cbNew));
+    AssertMsg(pv && cbNew, ("realloc(%p, %#zx) failed!!!\n", pvOld, cbNew));
 #ifdef RT_OS_OS2 /* temporary workaround until libc062. */
     AssertMsg(   cbNew < 32
               || !((uintptr_t)pv & (RTMEM_ALIGNMENT - 1)), ("pv=%p RTMEM_ALIGNMENT=%#x\n", pv, RTMEM_ALIGNMENT));
