@@ -1,4 +1,4 @@
-/* $Id: path-posix.cpp 29999 2008-04-21 20:53:28Z frank $ */
+/* $Id: path-posix.cpp $ */
 /** @file
  * IPRT - Path Manipulation, POSIX.
  */
@@ -42,16 +42,15 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <pwd.h>
-#ifdef RT_OS_DARWIN
-# include <mach-o/dyld.h>
-#endif
 
 #include <iprt/path.h>
+#include <iprt/env.h>
 #include <iprt/assert.h>
 #include <iprt/string.h>
 #include <iprt/err.h>
 #include <iprt/log.h>
 #include "internal/path.h"
+#include "internal/process.h"
 #include "internal/fs.h"
 
 #ifdef RT_OS_L4
@@ -61,7 +60,7 @@
 
 
 
-RTDECL(int) RTPathReal(const char *pszPath, char *pszRealPath, unsigned cchRealPath)
+RTDECL(int) RTPathReal(const char *pszPath, char *pszRealPath, size_t cchRealPath)
 {
     /*
      * Convert input.
@@ -168,330 +167,160 @@ static int fsCleanPath(char *pszPath)
 }
 
 
-RTDECL(int) RTPathAbs(const char *pszPath, char *pszAbsPath, unsigned cchAbsPath)
+RTDECL(int) RTPathAbs(const char *pszPath, char *pszAbsPath, size_t cchAbsPath)
 {
-    /*
-     * Convert input.
-     */
-    char *pszNativePath;
-    int rc = rtPathToNative(&pszNativePath, pszPath);
-    if (RT_FAILURE(rc))
-    {
-        LogFlow(("RTPathAbs(%p:{%s}, %p, %d): returns %Rrc\n", pszPath,
-                 pszPath, pszAbsPath, cchAbsPath, rc));
-        return rc;
-    }
+    int rc;
 
     /*
-     * On POSIX platforms the API doesn't take a length parameter, which makes it
-     * a little bit more work.
+     * Validation.
      */
+    AssertPtr(pszAbsPath);
+    AssertPtr(pszPath);
+    if (RT_UNLIKELY(!*pszPath))
+        return VERR_INVALID_PARAMETER;
+
+    /*
+     * Make a clean working copy of the input.
+     */
+    size_t cchPath = strlen(pszPath);
+    if (cchPath > PATH_MAX)
+    {
+        LogFlow(("RTPathAbs(%p:{%s}, %p, %d): returns %Rrc\n", pszPath, pszPath, pszAbsPath, cchAbsPath, VERR_FILENAME_TOO_LONG));
+        return VERR_FILENAME_TOO_LONG;
+    }
+
     char szTmpPath[PATH_MAX + 1];
-    char *psz = realpath(pszNativePath, szTmpPath);
-    if (!psz)
-    {
-        if (errno == ENOENT || errno == ENOTDIR
-#ifdef RT_OS_OS2
-            /// @todo realpath() returns EIO for non-existent UNC paths like
-            //  //server/share/subdir (i.e. when a subdir is specified within
-            //  a share). We should either fix realpath() in libc or remove
-            //  this todo.
-            || errno == EIO
-#endif
-            )
-        {
-            if (strlen(pszNativePath) <= PATH_MAX)
-            {
-                /*
-                 * Iterate the path bit by bit an apply realpath to it.
-                 */
+    memcpy(szTmpPath, pszPath, cchPath + 1);
+    size_t cchTmpPath = fsCleanPath(szTmpPath);
 
-                char szTmpSrc[PATH_MAX + 1];
-                strcpy(szTmpSrc, pszNativePath);
-                fsCleanPath(szTmpSrc);
-
-                size_t cch = 0; // current resolved path length
-                char *pszCur = szTmpSrc;
+    /*
+     * Handle "." specially (fsCleanPath does).
+     */
+    if (szTmpPath[0] == '.' && !szTmpPath[1])
+        return RTPathGetCurrent(pszAbsPath, cchAbsPath);
 
+    /*
+     * Do we have a root slash?
+     */
+    char *pszCur = szTmpPath;
 #ifdef HAVE_DRIVE
-                if (pszCur[0] && RTPATH_IS_VOLSEP(pszCur[1]) && pszCur[2] == '/')
-                {
-                    psz = szTmpPath;
-                    cch = 2;
-                    pszCur += 3;
-                }
-#ifdef HAVE_UNC
-                else
-                if (pszCur[0] == '/' && pszCur[1] == '/')
-                {
-                    pszCur += 2;
-                    char *pszSlash = strchr(pszCur, '/');
-                    size_t cchElement = pszSlash ? pszSlash - pszCur : strlen(pszCur);
-                    if (cchElement && pszCur[cchElement])
-                    {
-                        psz = szTmpPath;
-                        cch = cchElement + 2;
-                        pszCur += cchElement + 1;
-                    }
-                    else
-                        /* we've got just "//server" or "//" */
-                        /// @todo (r=dmik) not 100% sure we should fail, but the
-                        //  above cases are just invalid (incomplete) paths,
-                        //  no matter that Win32 returns these paths as is.
-                        rc = VERR_INVALID_NAME;
-                }
-#endif
-#else
-                if (*pszCur == '/')
-                {
-                    psz = szTmpPath;
-                    pszCur++;
-                }
-#endif
-                else
-                {
-                    /* get the cwd */
-                    psz = getcwd(szTmpPath,  sizeof(szTmpPath));
-                    AssertMsg(psz, ("Couldn't get cwd!\n"));
-                    if (psz)
-                    {
-#ifdef HAVE_DRIVE
-                        if (*pszCur == '/')
-                        {
-                            cch = 2;
-                            pszCur++;
-                        }
-                        else
-#endif
-                            cch = strlen(psz);
-                    }
-                    else
-                        rc = RTErrConvertFromErrno(errno);
-                }
-
-                if (psz)
-                {
-                    bool fResolveSymlinks = true;
-                    char szTmpPath2[PATH_MAX + 1];
-
-                    /* make sure strrchr() will work correctly */
-                    psz[cch] = '\0';
-
-                    while (*pszCur)
-                    {
-                        char *pszSlash = strchr(pszCur, '/');
-                        size_t cchElement = pszSlash ? pszSlash - pszCur : strlen(pszCur);
-                        if (cch + cchElement + 1 > PATH_MAX)
-                        {
-                            rc = VERR_FILENAME_TOO_LONG;
-                            break;
-                        }
-
-                        if (!strncmp(pszCur, "..", cchElement))
-                        {
-                            char *pszLastSlash = strrchr(psz, '/');
-#ifdef HAVE_UNC
-                            if (pszLastSlash && pszLastSlash > psz &&
-                                pszLastSlash[-1] != '/')
-#else
-                            if (pszLastSlash)
-#endif
-                            {
-                                cch = pszLastSlash - psz;
-                                psz[cch] = '\0';
-                            }
-                            /* else: We've reached the root and the parent of
-                             * the root is the root. */
-                        }
-                        else
-                        {
-                            psz[cch++] = '/';
-                            memcpy(psz + cch, pszCur, cchElement);
-                            cch += cchElement;
-                            psz[cch] = '\0';
-
-                            if (fResolveSymlinks)
-                            {
-                                /* resolve possible symlinks */
-                                char *psz2 = realpath(psz, psz == szTmpPath
-                                                           ? szTmpPath2
-                                                           : szTmpPath);
-                                if (psz2)
-                                {
-                                    psz = psz2;
-                                    cch = strlen(psz);
-                                }
-                                else
-                                {
-                                    if (errno != ENOENT && errno != ENOTDIR
-#ifdef RT_OS_OS2
-                                        /// @todo see above
-                                        && errno != EIO
-#endif
-                                        )
-                                    {
-                                        rc = RTErrConvertFromErrno(errno);
-                                        break;
-                                    }
-
-                                    /* no more need to resolve symlinks */
-                                    fResolveSymlinks = false;
-                                }
-                            }
-                        }
-
-                        pszCur += cchElement;
-                        /* skip the slash */
-                        if (*pszCur)
-                            ++pszCur;
-                    }
-
-#ifdef HAVE_DRIVE
-                    /* check if we're at the root */
-                    if (cch == 2 && RTPATH_IS_VOLSEP(psz[1]))
-#else
-                    /* if the length is zero here, then we're at the root */
-                    if (!cch)
-#endif
-                    {
-                        psz[cch++] = '/';
-                        psz[cch] = '\0';
-                    }
-                }
-            }
-            else
-                rc = VERR_FILENAME_TOO_LONG;
-        }
-        else
-            rc = RTErrConvertFromErrno(errno);
-    }
-
-    RTStrFree(pszNativePath);
-
-    if (psz && RT_SUCCESS(rc))
+    if (pszCur[0] && RTPATH_IS_VOLSEP(pszCur[1]) && pszCur[2] == '/')
+        pszCur += 3;
+# ifdef HAVE_UNC
+    else if (pszCur[0] == '/' && pszCur[1] == '/')
+        pszCur += 2;
+# endif
+#else  /* !HAVE_DRIVE */
+    if (pszCur[0] == '/')
+        pszCur += 1;
+#endif /* !HAVE_DRIVE */
+    else
     {
         /*
-         * Convert result and copy it to the return buffer.
+         * No, prepend the current directory to the relative path.
          */
-        char *pszUtf8AbsPath;
-        rc = rtPathFromNative(&pszUtf8AbsPath, psz);
-        if (RT_FAILURE(rc))
+        char szCurDir[RTPATH_MAX];
+        rc = RTPathGetCurrent(szCurDir, sizeof(szCurDir));
+        AssertRCReturn(rc, rc);
+
+        size_t cchCurDir = fsCleanPath(szCurDir); /* paranoia */
+        if (cchCurDir + cchTmpPath + 1 > PATH_MAX)
         {
-            LogFlow(("RTPathAbs(%p:{%s}, %p, %d): returns %Rrc\n", pszPath,
-                     pszPath, pszAbsPath, cchAbsPath, rc));
-            return rc;
+            LogFlow(("RTPathAbs(%p:{%s}, %p, %d): returns %Rrc\n", pszPath, pszPath, pszAbsPath, cchAbsPath, VERR_FILENAME_TOO_LONG));
+            return VERR_FILENAME_TOO_LONG;
         }
 
-        /* replace '/' back with native RTPATH_SLASH */
-        psz = pszUtf8AbsPath;
-        for (; *psz; psz++)
-            if (*psz == '/')
-                *psz = RTPATH_SLASH;
+        memmove(szTmpPath + cchCurDir + 1, szTmpPath, cchTmpPath + 1);
+        memcpy(szTmpPath, szCurDir, cchCurDir);
+        szTmpPath[cchCurDir] = '/';
 
-        unsigned cch = strlen(pszUtf8AbsPath) + 1;
-        if (cch <= cchAbsPath)
-            memcpy(pszAbsPath, pszUtf8AbsPath, cch);
+
+#ifdef HAVE_DRIVE
+        if (pszCur[0] && RTPATH_IS_VOLSEP(pszCur[1]) && pszCur[2] == '/')
+            pszCur += 3;
+# ifdef HAVE_UNC
+        else if (pszCur[0] == '/' && pszCur[1] == '/')
+            pszCur += 2;
+# endif
+#else
+        if (pszCur[0] == '/')
+            pszCur += 1;
+#endif
         else
-            rc = VERR_BUFFER_OVERFLOW;
-        RTStrFree(pszUtf8AbsPath);
+            AssertMsgFailedReturn(("pszCur=%s\n", pszCur), VERR_INTERNAL_ERROR);
     }
 
-    LogFlow(("RTPathAbs(%p:{%s}, %p:{%s}, %d): returns %Rrc\n", pszPath,
-             pszPath, pszAbsPath, RT_SUCCESS(rc) ? pszAbsPath : "<failed>",
-             cchAbsPath, rc));
+    char *pszTop = pszCur;
+
+    /*
+     * Get rid of double dot path components by evaluating them.
+     */
+    for (;;)
+    {
+        if (   pszCur[0] == '.'
+            && pszCur[1] == '.'
+            && (!pszCur[2] || pszCur[2] == '/'))
+        {
+            /* rewind to the previous component if any */
+            char *pszPrev = pszCur - 1;
+            if (pszPrev > pszTop)
+                while (*--pszPrev != '/')
+                    ;
+
+            AssertMsg(*pszPrev == '/', ("szTmpPath={%s}, pszPrev=+%u\n", szTmpPath, pszPrev - szTmpPath));
+            memmove(pszPrev, pszCur + 2, strlen(pszCur + 2) + 1);
+
+            pszCur = pszPrev;
+        }
+        else
+        {
+            /* advance to end of component. */
+            while (*pszCur && *pszCur != '/')
+                pszCur++;
+        }
+
+        if (!*pszCur)
+            break;
+
+        /* skip the slash */
+        ++pszCur;
+    }
+
+    if (pszCur < pszTop)
+    {
+        /*
+         * We overwrote the root slash with '\0', restore it.
+         */
+        *pszCur++ = '/';
+        *pszCur = '\0';
+    }
+    else if (pszCur > pszTop && pszCur[-1] == '/')
+    {
+        /*
+         * Extra trailing slash in a non-root path, remove it.
+         * (A bit questionable...)
+         */
+        *--pszCur = '\0';
+    }
+
+    /*
+     * Copy the result to the user buffer.
+     */
+    cchTmpPath = pszCur - szTmpPath;
+    if (cchTmpPath < cchAbsPath)
+    {
+        memcpy(pszAbsPath, szTmpPath, cchTmpPath + 1);
+        rc = VINF_SUCCESS;
+    }
+    else
+        rc = VERR_BUFFER_OVERFLOW;
+
+    LogFlow(("RTPathAbs(%p:{%s}, %p:{%s}, %d): returns %Rrc\n", pszPath, pszPath, pszAbsPath,
+             RT_SUCCESS(rc) ? pszAbsPath : "<failed>", cchAbsPath, rc));
     return rc;
 }
 
 
-#ifndef RT_MINI
-RTDECL(int) RTPathProgram(char *pszPath, unsigned cchPath)
-{
-    /*
-     * First time only.
-     */
-    if (!g_szrtProgramPath[0])
-    {
-        /*
-         * Linux have no API for obtaining the executable path, but provides a symbolic link
-         * in the proc file system. Note that readlink is one of the weirdest Unix apis around.
-         *
-         * OS/2 have an api for getting the program file name.
-         */
-/** @todo use RTProcGetExecutableName() */
-#if defined(RT_OS_LINUX) || defined(RT_OS_FREEBSD) || defined(RT_OS_SOLARIS)
-# ifdef RT_OS_LINUX
-        int cchLink = readlink("/proc/self/exe", &g_szrtProgramPath[0], sizeof(g_szrtProgramPath) - 1);
-# elif defined(RT_OS_SOLARIS)
-        char szFileBuf[PATH_MAX + 1];
-        sprintf(szFileBuf, "/proc/%ld/path/a.out", (long)getpid());
-        int cchLink = readlink(szFileBuf, &g_szrtProgramPath[0], sizeof(g_szrtProgramPath) - 1);
-# else /* RT_OS_FREEBSD: */
-        int cchLink = readlink("/proc/curproc/file", &g_szrtProgramPath[0], sizeof(g_szrtProgramPath) - 1);
-# endif
-        if (cchLink < 0 || cchLink == sizeof(g_szrtProgramPath) - 1)
-        {
-            int rc = RTErrConvertFromErrno(errno);
-            AssertMsgFailed(("couldn't read /proc/self/exe. errno=%d cchLink=%d\n", errno, cchLink));
-            LogFlow(("RTPathProgram(%p, %u): returns %Rrc\n", pszPath, cchPath, rc));
-            return rc;
-        }
-        g_szrtProgramPath[cchLink] = '\0';
-
-#elif defined(RT_OS_OS2) || defined(RT_OS_L4)
-        _execname(g_szrtProgramPath, sizeof(g_szrtProgramPath));
-
-#elif defined(RT_OS_DARWIN)
-        const char *pszImageName = _dyld_get_image_name(0);
-        AssertReturn(pszImageName, VERR_INTERNAL_ERROR);
-        size_t cchImageName = strlen(pszImageName);
-        if (cchImageName >= sizeof(g_szrtProgramPath))
-            AssertReturn(pszImageName, VERR_INTERNAL_ERROR);
-        memcpy(g_szrtProgramPath, pszImageName, cchImageName + 1);
-
-#else
-# error needs porting.
-#endif
-
-        /*
-         * Convert to UTF-8 and strip of the filename.
-         */
-        char *pszTmp = NULL;
-        int rc = rtPathFromNative(&pszTmp, &g_szrtProgramPath[0]);
-        if (RT_FAILURE(rc))
-        {
-            LogFlow(("RTPathProgram(%p, %u): returns %Rrc\n", pszPath, cchPath, rc));
-            return rc;
-        }
-        size_t cch = strlen(pszTmp);
-        if (cch >= sizeof(g_szrtProgramPath))
-        {
-            RTStrFree(pszTmp);
-            LogFlow(("RTPathProgram(%p, %u): returns %Rrc\n", pszPath, cchPath, VERR_BUFFER_OVERFLOW));
-            return VERR_BUFFER_OVERFLOW;
-        }
-        memcpy(g_szrtProgramPath, pszTmp, cch + 1);
-        RTPathStripFilename(g_szrtProgramPath);
-        RTStrFree(pszTmp);
-    }
-
-    /*
-     * Calc the length and check if there is space before copying.
-     */
-    unsigned cch = strlen(g_szrtProgramPath) + 1;
-    if (cch <= cchPath)
-    {
-        memcpy(pszPath, g_szrtProgramPath, cch + 1);
-        LogFlow(("RTPathProgram(%p:{%s}, %u): returns %Rrc\n", pszPath, pszPath, cchPath, VINF_SUCCESS));
-        return VINF_SUCCESS;
-    }
-
-    AssertMsgFailed(("Buffer too small (%d < %d)\n", cchPath, cch));
-    LogFlow(("RTPathProgram(%p, %u): returns %Rrc\n", pszPath, cchPath, VERR_BUFFER_OVERFLOW));
-    return VERR_BUFFER_OVERFLOW;
-}
-#endif /* !RT_MINI */
-
-
 #ifndef RT_OS_L4
 /**
  * Worker for RTPathUserHome that looks up the home directory
@@ -564,7 +393,7 @@ static int rtPathUserHomeByEnv(char *pszPath, size_t cchPath)
      * Get HOME env. var it and validate it's existance.
      */
     int rc = VERR_PATH_NOT_FOUND;
-    const char *pszHome = getenv("HOME");
+    const char *pszHome = RTEnvGet("HOME");
     if (pszHome)
 
     {
@@ -592,7 +421,7 @@ static int rtPathUserHomeByEnv(char *pszPath, size_t cchPath)
 }
 
 
-RTDECL(int) RTPathUserHome(char *pszPath, unsigned cchPath)
+RTDECL(int) RTPathUserHome(char *pszPath, size_t cchPath)
 {
     int rc;
 #ifndef RT_OS_L4
@@ -632,17 +461,24 @@ RTDECL(int) RTPathUserHome(char *pszPath, unsigned cchPath)
 
 
 RTR3DECL(int) RTPathQueryInfo(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAdditionalAttribs)
+{
+    return RTPathQueryInfoEx(pszPath, pObjInfo, enmAdditionalAttribs, RTPATH_F_ON_LINK);
+}
+
+
+RTR3DECL(int) RTPathQueryInfoEx(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAdditionalAttribs, uint32_t fFlags)
 {
     /*
      * Validate input.
      */
-    AssertMsgReturn(VALID_PTR(pszPath), ("%p\n", pszPath), VERR_INVALID_POINTER);
+    AssertPtrReturn(pszPath, VERR_INVALID_POINTER);
     AssertReturn(*pszPath, VERR_INVALID_PARAMETER);
-    AssertMsgReturn(VALID_PTR(pObjInfo), ("%p\n", pszPath), VERR_INVALID_POINTER);
+    AssertPtrReturn(pObjInfo, VERR_INVALID_POINTER);
     AssertMsgReturn(    enmAdditionalAttribs >= RTFSOBJATTRADD_NOTHING
                     &&  enmAdditionalAttribs <= RTFSOBJATTRADD_LAST,
                     ("Invalid enmAdditionalAttribs=%p\n", enmAdditionalAttribs),
                     VERR_INVALID_PARAMETER);
+    AssertMsgReturn(RTPATH_F_IS_VALID(fFlags, 0), ("%#x\n", fFlags), VERR_INVALID_PARAMETER);
 
     /*
      * Convert the filename.
@@ -652,7 +488,11 @@ RTR3DECL(int) RTPathQueryInfo(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOB
     if (RT_SUCCESS(rc))
     {
         struct stat Stat;
-        if (!stat(pszNativePath, &Stat))
+        if (fFlags & RTPATH_F_FOLLOW_LINK)
+            rc = stat(pszNativePath, &Stat);
+        else
+            rc = lstat(pszNativePath, &Stat); /** @todo how doesn't have lstat again? */
+        if (!rc)
         {
             rtFsConvertStatToObjInfo(pObjInfo, &Stat, pszPath, 0);
             switch (enmAdditionalAttribs)
@@ -686,16 +526,24 @@ RTR3DECL(int) RTPathQueryInfo(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOB
 
 RTR3DECL(int) RTPathSetTimes(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
                              PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime)
+{
+    return RTPathSetTimesEx(pszPath, pAccessTime, pModificationTime, pChangeTime, pBirthTime, RTPATH_F_ON_LINK);
+}
+
+
+RTR3DECL(int) RTPathSetTimesEx(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
+                               PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime, uint32_t fFlags)
 {
     /*
      * Validate input.
      */
-    AssertMsgReturn(VALID_PTR(pszPath), ("%p\n", pszPath), VERR_INVALID_POINTER);
-    AssertMsgReturn(*pszPath, ("%p\n", pszPath), VERR_INVALID_PARAMETER);
-    AssertMsgReturn(!pAccessTime || VALID_PTR(pAccessTime), ("%p\n", pAccessTime), VERR_INVALID_POINTER);
-    AssertMsgReturn(!pModificationTime || VALID_PTR(pModificationTime), ("%p\n", pModificationTime), VERR_INVALID_POINTER);
-    AssertMsgReturn(!pChangeTime || VALID_PTR(pChangeTime), ("%p\n", pChangeTime), VERR_INVALID_POINTER);
-    AssertMsgReturn(!pBirthTime || VALID_PTR(pBirthTime), ("%p\n", pBirthTime), VERR_INVALID_POINTER);
+    AssertPtrReturn(pszPath, VERR_INVALID_POINTER);
+    AssertReturn(*pszPath, VERR_INVALID_PARAMETER);
+    AssertPtrNullReturn(pAccessTime, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pModificationTime, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pChangeTime, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pBirthTime, VERR_INVALID_POINTER);
+    AssertMsgReturn(RTPATH_F_IS_VALID(fFlags, 0), ("%#x\n", fFlags), VERR_INVALID_PARAMETER);
 
     /*
      * Convert the paths.
@@ -704,20 +552,13 @@ RTR3DECL(int) RTPathSetTimes(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRT
     int rc = rtPathToNative(&pszNativePath, pszPath);
     if (RT_SUCCESS(rc))
     {
+        RTFSOBJINFO ObjInfo;
+
         /*
          * If it's a no-op, we'll only verify the existance of the file.
          */
         if (!pAccessTime && !pModificationTime)
-        {
-            struct stat Stat;
-            if (!stat(pszNativePath, &Stat))
-                rc = VINF_SUCCESS;
-            else
-            {
-                rc = RTErrConvertFromErrno(errno);
-                Log(("RTPathSetTimes('%s',,,,): failed with %Rrc and errno=%d\n", pszPath, rc, errno));
-            }
-        }
+            rc = RTPathQueryInfoEx(pszPath, &ObjInfo, RTFSOBJATTRADD_NOTHING, fFlags);
         else
         {
             /*
@@ -732,8 +573,7 @@ RTR3DECL(int) RTPathSetTimes(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRT
             }
             else
             {
-                RTFSOBJINFO ObjInfo;
-                int rc = RTPathQueryInfo(pszPath, &ObjInfo, RTFSOBJATTRADD_UNIX);
+                rc = RTPathQueryInfoEx(pszPath, &ObjInfo, RTFSOBJATTRADD_UNIX, fFlags);
                 if (RT_SUCCESS(rc))
                 {
                     RTTimeSpecGetTimeval(pAccessTime        ? pAccessTime       : &ObjInfo.AccessTime,       &aTimevals[0]);
@@ -745,12 +585,37 @@ RTR3DECL(int) RTPathSetTimes(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRT
             }
             if (RT_SUCCESS(rc))
             {
-                if (utimes(pszNativePath, aTimevals))
+                if (fFlags & RTPATH_F_FOLLOW_LINK)
                 {
-                    rc = RTErrConvertFromErrno(errno);
+                    if (utimes(pszNativePath, aTimevals))
+                        rc = RTErrConvertFromErrno(errno);
+                }
+#if (defined(RT_OS_DARWIN) && MAC_OS_X_VERSION_MIN_REQUIRED >= 1050) \
+ || defined(RT_OS_FREEBSD) \
+ || defined(RT_OS_LINUX) \
+ || defined(RT_OS_OS2) /** @todo who really has lutimes? */
+                else
+                {
+                    if (lutimes(pszNativePath, aTimevals))
+                        rc = RTErrConvertFromErrno(errno);
+                }
+#else
+                else
+                {
+                    if (pAccessTime && pModificationTime)
+                        rc = RTPathQueryInfoEx(pszPath, &ObjInfo, RTFSOBJATTRADD_UNIX, fFlags);
+                    if (RT_SUCCESS(rc) && RTFS_IS_SYMLINK(ObjInfo.Attr.fMode))
+                        rc = VERR_NS_SYMLINK_SET_TIME;
+                    else if (RT_SUCCESS(rc))
+                    {
+                        if (utimes(pszNativePath, aTimevals))
+                            rc = RTErrConvertFromErrno(errno);
+                    }
+                }
+#endif
+                if (RT_FAILURE(rc))
                     Log(("RTPathSetTimes('%s',%p,%p,,): failed with %Rrc and errno=%d\n",
                          pszPath, pAccessTime, pModificationTime, rc, errno));
-                }
             }
         }
         rtPathFreeNative(pszNativePath);
@@ -957,12 +822,19 @@ RTR3DECL(int) RTPathRename(const char *pszSrc, const char *pszDst, unsigned fRen
 
 
 RTDECL(bool) RTPathExists(const char *pszPath)
+{
+    return RTPathExistsEx(pszPath, RTPATH_F_FOLLOW_LINK);
+}
+
+
+RTDECL(bool) RTPathExistsEx(const char *pszPath, uint32_t fFlags)
 {
     /*
      * Validate input.
      */
     AssertPtrReturn(pszPath, false);
     AssertReturn(*pszPath, false);
+    Assert(RTPATH_F_IS_VALID(fFlags, 0));
 
     /*
      * Convert the path and check if it exists using stat().
@@ -972,7 +844,11 @@ RTDECL(bool) RTPathExists(const char *pszPath)
     if (RT_SUCCESS(rc))
     {
         struct stat Stat;
-        if (!stat(pszNativePath, &Stat))
+        if (fFlags & RTPATH_F_FOLLOW_LINK)
+            rc = stat(pszNativePath, &Stat);
+        else
+            rc = lstat(pszNativePath, &Stat);
+        if (!rc)
             rc = VINF_SUCCESS;
         else
             rc = VERR_GENERAL_FAILURE;
@@ -981,3 +857,54 @@ RTDECL(bool) RTPathExists(const char *pszPath)
     return RT_SUCCESS(rc);
 }
 
+
+RTDECL(int)  RTPathGetCurrent(char *pszPath, size_t cchPath)
+{
+    int rc;
+    char szNativeCurDir[RTPATH_MAX];
+    if (getcwd(szNativeCurDir, sizeof(szNativeCurDir)) != NULL)
+    {
+        char *pszCurDir;
+        rc = rtPathFromNative(&pszCurDir, szNativeCurDir);
+        if (RT_SUCCESS(rc))
+        {
+            size_t cchCurDir = strlen(pszCurDir);
+            if (cchCurDir < cchPath)
+            {
+                memcpy(pszPath, pszCurDir, cchCurDir + 1);
+                RTStrFree(pszCurDir);
+                return VINF_SUCCESS;
+            }
+
+            rc = VERR_BUFFER_OVERFLOW;
+            RTStrFree(pszCurDir);
+        }
+    }
+    else
+        rc = RTErrConvertFromErrno(errno);
+    return rc;
+}
+
+
+RTDECL(int) RTPathSetCurrent(const char *pszPath)
+{
+    /*
+     * Validate input.
+     */
+    AssertPtrReturn(pszPath, VERR_INVALID_POINTER);
+    AssertReturn(*pszPath, VERR_INVALID_PARAMETER);
+
+    /*
+     * Change the directory.
+     */
+    char *pszNativePath;
+    int rc = rtPathToNative(&pszNativePath, pszPath);
+    if (RT_SUCCESS(rc))
+    {
+        if (chdir(pszNativePath))
+            rc = RTErrConvertFromErrno(errno);
+        RTStrFree(pszNativePath);
+    }
+    return rc;
+}
+
