@@ -51,7 +51,7 @@ RTR3DECL(RTTLS) RTTlsAlloc(void)
     int rc = pthread_key_create(&iTls, NULL);
     if (!rc)
     {
-        Assert((RTTLS)iTls != NIL_RTTLS);
+        Assert(iTls != (pthread_key_t)NIL_RTTLS);
         return iTls;
     }
     return NIL_RTTLS;
