@@ -1,4 +1,4 @@
-/* $Id: process.cpp 29978 2008-04-21 17:24:28Z umoeller $ */
+/* $Id: process.cpp $ */
 /** @file
  * IPRT - Process, Common.
  */
@@ -36,6 +36,7 @@
 #include <iprt/process.h>
 #include <iprt/assert.h>
 #include <iprt/err.h>
+#include <iprt/string.h>
 #include "internal/process.h"
 #include "internal/thread.h"
 
@@ -93,3 +94,23 @@ RTR3DECL(RTPROCPRIORITY) RTProcGetPriority(void)
     return g_enmProcessPriority;
 }
 
+
+RTR3DECL(char *) RTProcGetExecutableName(char *pszExecName, size_t cchExecName)
+{
+    AssertReturn(g_szrtProcExePath[0] != '\0', NULL);
+
+    /*
+     * Calc the length and check if there is space before copying.
+     */
+    size_t cch = g_cchrtProcExePath;
+    if (cch <= cchExecName)
+    {
+        memcpy(pszExecName, g_szrtProcExePath, cch);
+        pszExecName[cch] = '\0';
+        return pszExecName;
+    }
+
+    AssertMsgFailed(("Buffer too small (%zu <= %zu)\n", cchExecName, cch));
+    return NULL;
+}
+
