@@ -41,6 +41,8 @@
 #include <iprt/err.h>
 #include <iprt/param.h>
 #include <iprt/string.h>
+
+#include "internal/alignmentchecks.h"
 #include "internal/magics.h"
 
 #include <stdio.h>
@@ -199,11 +201,11 @@ RTR3DECL(int) RTStrmOpen(const char *pszFilename, const char *pszMode, PRTSTREAM
  * @param   pszFilenameFmt  Filename path format string.
  * @param   args            Arguments to the format string.
  */
-RTR3DECL(int) RTStrmOpenfV(const char *pszMode, PRTSTREAM *ppStream, const char *pszFilenameFmt, va_list args)
+RTR3DECL(int) RTStrmOpenFV(const char *pszMode, PRTSTREAM *ppStream, const char *pszFilenameFmt, va_list args)
 {
     int     rc;
     char    szFilename[RTPATH_MAX];
-    size_t  cch = RTStrPrintf(szFilename, sizeof(szFilename), pszFilenameFmt, args);
+    size_t  cch = RTStrPrintfV(szFilename, sizeof(szFilename), pszFilenameFmt, args);
     if (cch < sizeof(szFilename))
         rc = RTStrmOpen(szFilename, pszMode, ppStream);
     else
@@ -225,11 +227,11 @@ RTR3DECL(int) RTStrmOpenfV(const char *pszMode, PRTSTREAM *ppStream, const char
  * @param   pszFilenameFmt  Filename path format string.
  * @param   ...             Arguments to the format string.
  */
-RTR3DECL(int) RTStrmOpenf(const char *pszMode, PRTSTREAM *ppStream, const char *pszFilenameFmt, ...)
+RTR3DECL(int) RTStrmOpenF(const char *pszMode, PRTSTREAM *ppStream, const char *pszFilenameFmt, ...)
 {
     va_list args;
     va_start(args, pszFilenameFmt);
-    int rc = RTStrmOpenfV(pszMode, ppStream, pszFilenameFmt, args);
+    int rc = RTStrmOpenFV(pszMode, ppStream, pszFilenameFmt, args);
     va_end(args);
     return rc;
 }
@@ -296,6 +298,7 @@ RTR3DECL(int) RTStrmClearError(PRTSTREAM pStream)
     int rc;
     if (pStream && pStream->u32Magic == RTSTREAM_MAGIC)
     {
+        clearerr(pStream->pFile);
         ASMAtomicXchgS32(&pStream->i32Error, VINF_SUCCESS);
         rc = VINF_SUCCESS;
     }
@@ -308,6 +311,41 @@ RTR3DECL(int) RTStrmClearError(PRTSTREAM pStream)
 }
 
 
+/**
+ * Rewinds the stream.
+ *
+ * Stream errors will be reset on success.
+ *
+ * @returns IPRT status code.
+ *
+ * @param   pStream         The stream.
+ *
+ * @remarks Not all streams are rewindable and that behavior is currently
+ *          undefined for those.
+ */
+RTR3DECL(int) RTStrmRewind(PRTSTREAM pStream)
+{
+    AssertPtrReturn(pStream, VERR_INVALID_HANDLE);
+    AssertReturn(pStream->u32Magic == RTSTREAM_MAGIC, VERR_INVALID_HANDLE);
+
+    int rc;
+    clearerr(pStream->pFile);
+    errno = 0;
+    if (!fseek(pStream->pFile, 0, SEEK_SET))
+    {
+        ASMAtomicXchgS32(&pStream->i32Error, VINF_SUCCESS);
+        rc = VINF_SUCCESS;
+    }
+    else
+    {
+        rc = RTErrConvertFromErrno(errno);
+        ASMAtomicXchgS32(&pStream->i32Error, rc);
+    }
+
+    return rc;
+}
+
+
 /**
  * Reads from a file stream.
  *
@@ -401,7 +439,9 @@ RTR3DECL(int) RTStrmWriteEx(PRTSTREAM pStream, const void *pvBuf, size_t cbWrite
         {
             if (pcbWritten)
             {
+                IPRT_ALIGNMENT_CHECKS_DISABLE(); /* glibc / mempcpy again */
                 *pcbWritten = fwrite(pvBuf, 1, cbWrite, pStream->pFile);
+                IPRT_ALIGNMENT_CHECKS_ENABLE();
                 if (    *pcbWritten == cbWrite
                     ||  !ferror(pStream->pFile))
                     return VINF_SUCCESS;
@@ -410,9 +450,12 @@ RTR3DECL(int) RTStrmWriteEx(PRTSTREAM pStream, const void *pvBuf, size_t cbWrite
             else
             {
                 /*
-                 * Must read it all!
+                 * Must write it all!
                  */
-                if (fwrite(pvBuf, cbWrite, 1, pStream->pFile) == 1)
+                IPRT_ALIGNMENT_CHECKS_DISABLE(); /* glibc / mempcpy again */
+                size_t cbWritten = fwrite(pvBuf, cbWrite, 1, pStream->pFile);
+                IPRT_ALIGNMENT_CHECKS_ENABLE();
+                if (cbWritten == 1)
                     return VINF_SUCCESS;
                 if (!ferror(pStream->pFile))
                     return VINF_SUCCESS; /* WEIRD! But anyway... */
@@ -499,29 +542,32 @@ RTR3DECL(int) RTStrmGetLine(PRTSTREAM pStream, char *pszString, size_t cchString
             if (RT_SUCCESS(rc))
             {
                 cchString--;            /* save space for the terminator. */
-                #ifdef HAVE_FWRITE_UNLOCKED
+#ifdef HAVE_FWRITE_UNLOCKED
                 flockfile(pStream->pFile);
-                #endif
+#endif
                 for (;;)
                 {
-                    #ifdef HAVE_FWRITE_UNLOCKED
+#ifdef HAVE_FWRITE_UNLOCKED
                     int ch = fgetc_unlocked(pStream->pFile);
-                    #else
+#else
                     int ch = fgetc(pStream->pFile);
-                    #endif
+#endif
                     if (ch == EOF)
                     {
-                        #ifdef HAVE_FWRITE_UNLOCKED
+#ifdef HAVE_FWRITE_UNLOCKED
                         if (feof_unlocked(pStream->pFile))
-                        #else
+#else
                         if (feof(pStream->pFile))
-                        #endif
+#endif
+                        {
+                            rc = VERR_EOF;
                             break;
-                        #ifdef HAVE_FWRITE_UNLOCKED
+                        }
+#ifdef HAVE_FWRITE_UNLOCKED
                         if (ferror_unlocked(pStream->pFile))
-                        #else
+#else
                         if (ferror(pStream->pFile))
-                        #endif
+#endif
                             rc = VERR_READ_ERROR;
                         else
                         {
@@ -539,9 +585,9 @@ RTR3DECL(int) RTStrmGetLine(PRTSTREAM pStream, char *pszString, size_t cchString
                         break;
                     }
                 }
-                #ifdef HAVE_FWRITE_UNLOCKED
+#ifdef HAVE_FWRITE_UNLOCKED
                 funlockfile(pStream->pFile);
-                #endif
+#endif
 
                 *pszString = '\0';
                 if (RT_FAILURE(rc))
@@ -593,12 +639,14 @@ static DECLCALLBACK(size_t) rtstrmOutput(void *pvArg, const char *pachChars, siz
         int rc = pStream->i32Error;
         if (RT_SUCCESS(rc))
         {
-            #ifdef HAVE_FWRITE_UNLOCKED
+            IPRT_ALIGNMENT_CHECKS_DISABLE(); /* glibc / mempcpy again */
+#ifdef HAVE_FWRITE_UNLOCKED
             if (fwrite_unlocked(pachChars, cchChars, 1, pStream->pFile) != 1)
-            #else
+#else
             if (fwrite(pachChars, cchChars, 1, pStream->pFile) != 1)
-            #endif
+#endif
                 ASMAtomicXchgS32(&pStream->i32Error, VERR_WRITE_ERROR);
+            IPRT_ALIGNMENT_CHECKS_ENABLE();
         }
     }
     /* else: ignore termination call. */
@@ -623,13 +671,13 @@ RTR3DECL(int) RTStrmPrintfV(PRTSTREAM pStream, const char *pszFormat, va_list ar
         if (RT_SUCCESS(rc))
         {
             /** @todo consider making this thread safe... */
-            #ifdef HAVE_FWRITE_UNLOCKED
+#ifdef HAVE_FWRITE_UNLOCKED
             flockfile(pStream->pFile);
             rc = (int)RTStrFormatV(rtstrmOutput, pStream, NULL, NULL, pszFormat, args);
             funlockfile(pStream->pFile);
-            #else
+#else
             rc = (int)RTStrFormatV(rtstrmOutput, pStream, NULL, NULL, pszFormat, args);
-            #endif
+#endif
             Assert(rc >= 0);
         }
         else
