@@ -33,50 +33,66 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #ifdef RT_OS_WINDOWS
-#include <winsock.h>
+# include <winsock.h>
+# include <limits.h>
 #else /* !RT_OS_WINDOWS */
-#include <errno.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <arpa/inet.h>
-#include <sys/un.h>
-#include <netdb.h>
-#include <unistd.h>
+# include <errno.h>
+# include <sys/stat.h>
+# include <sys/socket.h>
+# include <netinet/in.h>
+# include <netinet/tcp.h>
+# include <arpa/inet.h>
+# ifdef IPRT_WITH_TCPIP_V6
+#  include <netinet6/in6.h>
+# endif
+# include <sys/un.h>
+# include <netdb.h>
+# include <unistd.h>
 #endif /* !RT_OS_WINDOWS */
 
+#include "internal/iprt.h"
 #include <iprt/tcp.h>
-#include <iprt/thread.h>
-#include <iprt/alloc.h>
-#include <iprt/assert.h>
+
 #include <iprt/asm.h>
+#include <iprt/assert.h>
 #include <iprt/err.h>
+#include <iprt/mempool.h>
 #include <iprt/string.h>
+#include <iprt/thread.h>
+#include <iprt/time.h>
 
-
-/* non-standard linux stuff (it seems). */
-#ifndef MSG_NOSIGNAL
-# define MSG_NOSIGNAL 0
-#endif
-#ifndef SHUT_RDWR
-# ifdef SD_BOTH
-#  define SHUT_RDWR SD_BOTH
-# else
-#  define SHUT_RDWR 2
-# endif
-#endif
-
-/* fixup backlevel OSes. */
-#if defined(RT_OS_OS2) || defined(RT_OS_WINDOWS)
-# define socklen_t  int
-#endif
+#include "internal/magics.h"
 
 
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
-#define BACKLOG         10   /* how many pending connections queue will hold */
+/* non-standard linux stuff (it seems). */
+#ifndef MSG_NOSIGNAL
+# define MSG_NOSIGNAL           0
+#endif
+#ifndef SHUT_RDWR
+# ifdef SD_BOTH
+#  define SHUT_RDWR             SD_BOTH
+# else
+#  define SHUT_RDWR             2
+# endif
+#endif
+#ifndef SHUT_WR
+# ifdef SD_SEND
+#  define SHUT_WR               SD_SEND
+# else
+#  define SHUT_WR               1
+# endif
+#endif
+
+/* fixup backlevel OSes. */
+#if defined(RT_OS_OS2) || defined(RT_OS_WINDOWS)
+# define socklen_t              int
+#endif
+
+/** How many pending connection. */
+#define RTTCP_SERVER_BACKLOG    10
 
 
 /*******************************************************************************
@@ -101,12 +117,8 @@ typedef enum RTTCPSERVERSTATE
     RTTCPSERVERSTATE_STOPPING,
     /** Listener terminated. */
     RTTCPSERVERSTATE_STOPPED,
-    /** Destroying signaling to the listener, listener will wait. */
-    RTTCPSERVERSTATE_SIGNALING,
     /** Listener cleans up. */
-    RTTCPSERVERSTATE_DESTROYING,
-    /** Freed. */
-    RTTCPSERVERSTATE_FREED
+    RTTCPSERVERSTATE_DESTROYING
 } RTTCPSERVERSTATE;
 
 /*
@@ -114,6 +126,8 @@ typedef enum RTTCPSERVERSTATE
  */
 typedef struct RTTCPSERVER
 {
+    /** The magic value (RTTCPSERVER_MAGIC). */
+    uint32_t volatile           u32Magic;
     /** The server state. */
     RTTCPSERVERSTATE volatile   enmState;
     /** The server thread. */
@@ -129,15 +143,24 @@ typedef struct RTTCPSERVER
     void                       *pvUser;
 } RTTCPSERVER;
 
+typedef union RTSOCKADDRUNION
+{
+    struct sockaddr     Addr;
+    struct sockaddr_in  Ipv4;
+#ifdef IPRT_WITH_TCPIP_V6
+    struct sockaddr_in6 Ipv6;
+#endif
+} RTSOCKADDRUNION;
 
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
 static DECLCALLBACK(int)  rtTcpServerThread(RTTHREAD ThreadSelf, void *pvServer);
-static int rtTcpServerListen(PRTTCPSERVER pServer);
-static void rcTcpServerListenCleanup(PRTTCPSERVER pServer);
-static void rtTcpServerDestroyServerSock(RTSOCKET SockServer, const char *pszMsg);
-static int rtTcpClose(RTSOCKET Sock, const char *pszMsg);
+static int  rtTcpServerListen(PRTTCPSERVER pServer);
+static int  rcTcpServerListenCleanup(PRTTCPSERVER pServer);
+static int  rtTcpServerDestroySocket(RTSOCKET volatile *pSockClient, const char *pszMsg);
+static int  rtTcpClose(RTSOCKET Sock, const char *pszMsg, bool fTryGracefulShutdown);
+static int  rtTcpConvertAddress(RTSOCKADDRUNION *pSrc, size_t cbSrc, PRTNETADDR pAddr);
 
 
 
@@ -216,15 +239,42 @@ DECLINLINE(RTSOCKET) rtTcpAtomicXchgSock(RTSOCKET volatile *pSock, const RTSOCKE
 
 
 /**
- * Changes the TCP server state.
+ * Tries to change the TCP server state.
  */
-DECLINLINE(bool) rtTcpServerSetState(PRTTCPSERVER pServer, RTTCPSERVERSTATE enmStateNew, RTTCPSERVERSTATE enmStateOld)
+DECLINLINE(bool) rtTcpServerTrySetState(PRTTCPSERVER pServer, RTTCPSERVERSTATE enmStateNew, RTTCPSERVERSTATE enmStateOld)
 {
     bool fRc;
     ASMAtomicCmpXchgSize(&pServer->enmState, enmStateNew, enmStateOld, fRc);
     return fRc;
 }
 
+/**
+ * Changes the TCP server state.
+ */
+DECLINLINE(void) rtTcpServerSetState(PRTTCPSERVER pServer, RTTCPSERVERSTATE enmStateNew, RTTCPSERVERSTATE enmStateOld)
+{
+    bool fRc;
+    ASMAtomicCmpXchgSize(&pServer->enmState, enmStateNew, enmStateOld, fRc);
+    Assert(fRc); NOREF(fRc);
+}
+
+/**
+ * Closes the a socket (client or server).
+ *
+ * @returns IPRT status code.
+ */
+static int rtTcpServerDestroySocket(RTSOCKET volatile *pSock, const char *pszMsg, bool fTryGracefulShutdown)
+{
+    RTSOCKET Sock = rtTcpAtomicXchgSock(pSock, NIL_RTSOCKET);
+    if (Sock != NIL_RTSOCKET)
+    {
+        if (!fTryGracefulShutdown)
+            shutdown(Sock, SHUT_RDWR);
+        return rtTcpClose(Sock, pszMsg, fTryGracefulShutdown);
+    }
+    return VINF_TCP_SERVER_NO_CLIENT;
+}
+
 
 /**
  * Create single connection at a time TCP Server in a separate thread.
@@ -248,24 +298,24 @@ RTR3DECL(int)  RTTcpServerCreate(const char *pszAddress, unsigned uPort, RTTHREA
                                  PFNRTTCPSERVE pfnServe, void *pvUser, PPRTTCPSERVER ppServer)
 {
     /*
-     * Do params checking
+     * Validate input.
      */
-    if (!uPort || !pfnServe || !pszThrdName || !ppServer)
-    {
-        AssertMsgFailed(("Invalid params\n"));
-        return VERR_INVALID_PARAMETER;
-    }
+    AssertReturn(uPort > 0, VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pfnServe, VERR_INVALID_POINTER);
+    AssertPtrReturn(pszThrdName, VERR_INVALID_POINTER);
+    AssertPtrReturn(ppServer, VERR_INVALID_POINTER);
 
     /*
      * Create the server.
      */
-    PRTTCPSERVER    pServer;
+    PRTTCPSERVER pServer;
     int rc = RTTcpServerCreateEx(pszAddress, uPort, &pServer);
     if (RT_SUCCESS(rc))
     {
         /*
          * Create the listener thread.
          */
+        RTMemPoolRetain(pServer);
         pServer->enmState   = RTTCPSERVERSTATE_STARTING;
         pServer->pvUser     = pvUser;
         pServer->pfnServe   = pfnServe;
@@ -275,8 +325,11 @@ RTR3DECL(int)  RTTcpServerCreate(const char *pszAddress, unsigned uPort, RTTHREA
             /* done */
             if (ppServer)
                 *ppServer = pServer;
+            else
+                RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
             return rc;
         }
+        RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
 
         /*
          * Destroy the server.
@@ -299,9 +352,12 @@ RTR3DECL(int)  RTTcpServerCreate(const char *pszAddress, unsigned uPort, RTTHREA
 static DECLCALLBACK(int)  rtTcpServerThread(RTTHREAD ThreadSelf, void *pvServer)
 {
     PRTTCPSERVER    pServer = (PRTTCPSERVER)pvServer;
-    if (rtTcpServerSetState(pServer, RTTCPSERVERSTATE_ACCEPTING, RTTCPSERVERSTATE_STARTING))
-        return rtTcpServerListen(pServer);
-    rcTcpServerListenCleanup(pServer);
+    int             rc;
+    if (rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_ACCEPTING, RTTCPSERVERSTATE_STARTING))
+        rc = rtTcpServerListen(pServer);
+    else
+        rc = rcTcpServerListenCleanup(pServer);
+    RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
     NOREF(ThreadSelf);
     return VINF_SUCCESS;
 }
@@ -322,13 +378,10 @@ RTR3DECL(int) RTTcpServerCreateEx(const char *pszAddress, uint32_t uPort, PPRTTC
     int rc;
 
     /*
-     * Do params checking
+     * Validate input.
      */
-    if (!uPort || !ppServer)
-    {
-        AssertMsgFailed(("Invalid params\n"));
-        return VERR_INVALID_PARAMETER;
-    }
+    AssertReturn(uPort > 0, VERR_INVALID_PARAMETER);
+    AssertPtrReturn(ppServer, VERR_INVALID_PARAMETER);
 
 #ifdef RT_OS_WINDOWS
     /*
@@ -397,23 +450,27 @@ RTR3DECL(int) RTTcpServerCreateEx(const char *pszAddress, uint32_t uPort, PPRTTC
                 /*
                  * Listen for connections on a socket.
                  */
-                if (listen(WaitSock, BACKLOG) != -1)
+                if (listen(WaitSock, RTTCP_SERVER_BACKLOG) != -1)
                 {
                     /*
                      * Create the server handle.
                      */
-                    PRTTCPSERVER    pServer = (PRTTCPSERVER)RTMemAllocZ(sizeof(*pServer));
+                    PRTTCPSERVER pServer = (PRTTCPSERVER)RTMemPoolAlloc(RTMEMPOOL_DEFAULT, sizeof(*pServer));
                     if (pServer)
                     {
+                        pServer->u32Magic   = RTTCPSERVER_MAGIC;
+                        pServer->enmState   = RTTCPSERVERSTATE_CREATED;
+                        pServer->Thread     = NIL_RTTHREAD;
                         pServer->SockServer = WaitSock;
                         pServer->SockClient = NIL_RTSOCKET;
-                        pServer->Thread     = NIL_RTTHREAD;
-                        pServer->enmState   = RTTCPSERVERSTATE_CREATED;
+                        pServer->pfnServe   = NULL;
+                        pServer->pvUser     = NULL;
                         *ppServer = pServer;
                         return VINF_SUCCESS;
                     }
-                    else
-                        rc = VERR_NO_MEMORY;
+
+                    /* bail out */
+                    rc = VERR_NO_MEMORY;
                 }
                 else
                 {
@@ -431,7 +488,7 @@ RTR3DECL(int) RTTcpServerCreateEx(const char *pszAddress, uint32_t uPort, PPRTTC
             rc = rtTcpError();
             AssertMsgFailed(("setsockopt() %Rrc\n", rc));
         }
-        rtTcpClose(WaitSock, "RTServerCreateEx");
+        rtTcpClose(WaitSock, "RTServerCreateEx", false /*fTryGracefulShutdown*/);
     }
     else
     {
@@ -451,7 +508,10 @@ RTR3DECL(int) RTTcpServerCreateEx(const char *pszAddress, uint32_t uPort, PPRTTC
  * return VERR_TCP_SERVER_STOP too terminate this loop. A stopped server
  * can only be destroyed.
  *
- * @returns iprt status code.
+ * @returns IPRT status code.
+ * @retval  VERR_TCP_SERVER_STOP if stopped by pfnServe.
+ * @retval  VERR_TCP_SERVER_SHUTDOWN if shut down by RTTcpServerShutdown.
+ *
  * @param   pServer         The server handle as returned from RTTcpServerCreateEx().
  * @param   pfnServe        The function which will serve a new client connection.
  * @param   pvUser          User argument passed to pfnServe.
@@ -459,14 +519,15 @@ RTR3DECL(int) RTTcpServerCreateEx(const char *pszAddress, uint32_t uPort, PPRTTC
 RTR3DECL(int) RTTcpServerListen(PRTTCPSERVER pServer, PFNRTTCPSERVE pfnServe, void *pvUser)
 {
     /*
-     * Validate input.
+     * Validate input and retain the instance.
      */
-    if (!pfnServe || !pServer)
-    {
-        AssertMsgFailed(("pfnServer=%p pServer=%p\n", pfnServe, pServer));
-        return VERR_INVALID_PARAMETER;
-    }
-    if (rtTcpServerSetState(pServer, RTTCPSERVERSTATE_ACCEPTING, RTTCPSERVERSTATE_CREATED))
+    AssertPtrReturn(pfnServe, VERR_INVALID_POINTER);
+    AssertPtrReturn(pServer, VERR_INVALID_HANDLE);
+    AssertReturn(pServer->u32Magic == RTTCPSERVER_MAGIC, VERR_INVALID_HANDLE);
+    AssertReturn(RTMemPoolRetain(pServer) != UINT32_MAX, VERR_INVALID_HANDLE);
+
+    int rc = VERR_INVALID_STATE;
+    if (rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_ACCEPTING, RTTCPSERVERSTATE_CREATED))
     {
         Assert(!pServer->pfnServe);
         Assert(!pServer->pvUser);
@@ -477,58 +538,55 @@ RTR3DECL(int) RTTcpServerListen(PRTTCPSERVER pServer, PFNRTTCPSERVE pfnServe, vo
         pServer->pvUser   = pvUser;
         pServer->Thread   = RTThreadSelf();
         Assert(pServer->Thread != NIL_RTTHREAD);
-        return rtTcpServerListen(pServer);
+        rc = rtTcpServerListen(pServer);
     }
-    AssertMsgFailed(("pServer->enmState=%d\n", pServer->enmState));
-    return VERR_INVALID_PARAMETER;
+    else
+    {
+        AssertMsgFailed(("enmState=%d\n", pServer->enmState));
+        rc = VERR_INVALID_STATE;
+    }
+    RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
+    return rc;
 }
 
 
 /**
- * Closes the client socket.
- */
-static int rtTcpServerDestroyClientSock(RTSOCKET volatile *pSock, const char *pszMsg)
-{
-    RTSOCKET Sock = rtTcpAtomicXchgSock(pSock, NIL_RTSOCKET);
-    if (Sock != NIL_RTSOCKET)
-        shutdown(Sock, SHUT_RDWR);
-    return rtTcpClose(Sock, pszMsg);
-}
-
-
-/**
- * Internal worker common for RTTcpServerListen and the thread created by RTTcpServerCreate().
+ * Internal worker common for RTTcpServerListen and the thread created by
+ * RTTcpServerCreate().
+ *
+ * The caller makes sure it has its own memory reference and releases it upon
+ * return.
  */
 static int rtTcpServerListen(PRTTCPSERVER pServer)
 {
     /*
      * Accept connection loop.
      */
-    int rc = VINF_SUCCESS;
     for (;;)
     {
         /*
          * Change state.
          */
-        RTTCPSERVERSTATE enmState = pServer->enmState;
+        RTTCPSERVERSTATE    enmState   = pServer->enmState;
+        RTSOCKET            SockServer = pServer->SockServer;
         if (    enmState != RTTCPSERVERSTATE_ACCEPTING
             &&  enmState != RTTCPSERVERSTATE_SERVING)
-            break;
-        if (!rtTcpServerSetState(pServer, RTTCPSERVERSTATE_ACCEPTING, enmState))
+            return rcTcpServerListenCleanup(pServer);
+        if (!rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_ACCEPTING, enmState))
             continue;
 
         /*
          * Accept connection.
          */
-        struct sockaddr_in RemoteAddr = {0};
-        socklen_t Len = sizeof(RemoteAddr);
-        RTSOCKET Socket = accept(pServer->SockServer, (struct sockaddr *)&RemoteAddr, &Len);
+        struct sockaddr_in  RemoteAddr   = {0};
+        socklen_t           cbRemoteAddr = sizeof(RemoteAddr);
+        RTSOCKET Socket = accept(SockServer, (struct sockaddr *)&RemoteAddr, &cbRemoteAddr);
         if (Socket == -1)
         {
 #ifndef RT_OS_WINDOWS
             /* These are typical for what can happen during destruction. */
             if (errno == EBADF || errno == EINVAL || errno == ENOTSOCK)
-                break;
+                return rcTcpServerListenCleanup(pServer);
 #endif
             continue;
         }
@@ -536,81 +594,71 @@ static int rtTcpServerListen(PRTTCPSERVER pServer)
         /*
          * Run a pfnServe callback.
          */
-        if (!rtTcpServerSetState(pServer, RTTCPSERVERSTATE_SERVING, RTTCPSERVERSTATE_ACCEPTING))
-            break;
+        if (!rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_SERVING, RTTCPSERVERSTATE_ACCEPTING))
+        {
+            rtTcpClose(Socket, "rtTcpServerListen", true /*fTryGracefulShutdown*/);
+            return rcTcpServerListenCleanup(pServer);
+        }
         rtTcpAtomicXchgSock(&pServer->SockClient, Socket);
-        rc = pServer->pfnServe(Socket, pServer->pvUser);
-        rtTcpServerDestroyClientSock(&pServer->SockClient, "Listener: client");
+        int rc = pServer->pfnServe(Socket, pServer->pvUser);
+        rtTcpServerDestroySocket(&pServer->SockClient, "Listener: client", true /*fTryGracefulShutdown*/);
 
         /*
          * Stop the server?
          */
         if (rc == VERR_TCP_SERVER_STOP)
         {
-            if (rtTcpServerSetState(pServer, RTTCPSERVERSTATE_STOPPING, RTTCPSERVERSTATE_SERVING))
+            if (rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_STOPPING, RTTCPSERVERSTATE_SERVING))
             {
                 /*
                  * Reset the server socket and change the state to stopped. After that state change
                  * we cannot safely access the handle so we'll have to return here.
                  */
-                RTSOCKET SockServer = rtTcpAtomicXchgSock(&pServer->SockServer, NIL_RTSOCKET);
+                SockServer = rtTcpAtomicXchgSock(&pServer->SockServer, NIL_RTSOCKET);
                 rtTcpServerSetState(pServer, RTTCPSERVERSTATE_STOPPED, RTTCPSERVERSTATE_STOPPING);
-                rtTcpClose(SockServer, "Listener: server stopped");
-                return rc;
+                rtTcpClose(SockServer, "Listener: server stopped", false /*fTryGracefulShutdown*/);
             }
-            break;
+            else
+                rcTcpServerListenCleanup(pServer); /* ignore rc */
+            return rc;
         }
     }
-
-    /*
-     * Perform any pending clean and be gone.
-     */
-    rcTcpServerListenCleanup(pServer);
-    return rc;
 }
 
 
 /**
  * Clean up after listener.
  */
-static void rcTcpServerListenCleanup(PRTTCPSERVER pServer)
+static int rcTcpServerListenCleanup(PRTTCPSERVER pServer)
 {
     /*
-     * Wait for any destroyers to finish signaling us.
+     * Close the server socket, the client one shouldn't be set.
      */
-    for (unsigned cTries = 99; cTries > 0; cTries--)
+    rtTcpServerDestroySocket(&pServer->SockServer, "ListenCleanup", false /*fTryGracefulShutdown*/);
+    Assert(pServer->SockClient == NIL_RTSOCKET);
+
+    /*
+     * Figure the return code and make sure the state is OK.
+     */
+    RTTCPSERVERSTATE enmState = pServer->enmState;
+    switch (enmState)
     {
-        RTTCPSERVERSTATE enmState = pServer->enmState;
-        switch (enmState)
-        {
-            /*
-             * Intermediate state while the destroyer closes the client socket.
-             */
-            case RTTCPSERVERSTATE_SIGNALING:
-                if (!RTThreadYield())
-                    RTThreadSleep(1);
-                break;
+        case RTTCPSERVERSTATE_STOPPING:
+        case RTTCPSERVERSTATE_STOPPED:
+            return VERR_TCP_SERVER_SHUTDOWN;
 
-            /*
-             * Free the handle.
-             */
-            case RTTCPSERVERSTATE_DESTROYING:
-            {
-                rtTcpClose(rtTcpAtomicXchgSock(&pServer->SockServer, NIL_RTSOCKET), "Listener-cleanup: server");
-                rtTcpServerSetState(pServer, RTTCPSERVERSTATE_FREED, RTTCPSERVERSTATE_DESTROYING);
-                RTMemFree(pServer);
-                return;
-            }
+        case RTTCPSERVERSTATE_ACCEPTING:
+            rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_STOPPED, enmState);
+            return VERR_TCP_SERVER_DESTROYED;
 
-            /*
-             * Everything else means failure.
-             */
-            default:
-                AssertMsgFailed(("pServer=%p enmState=%d\n", pServer, enmState));
-                return;
-        }
+        case RTTCPSERVERSTATE_DESTROYING:
+            return VERR_TCP_SERVER_DESTROYED;
+
+        case RTTCPSERVERSTATE_STARTING:
+        case RTTCPSERVERSTATE_SERVING:
+        default:
+            AssertMsgFailedReturn(("pServer=%p enmState=%d\n", pServer, enmState), VERR_INTERNAL_ERROR_4);
     }
-    AssertMsgFailed(("Timed out when trying to clean up after listener. pServer=%p enmState=%d\n", pServer, pServer->enmState));
 }
 
 
@@ -623,17 +671,69 @@ static void rcTcpServerListenCleanup(PRTTCPSERVER pServer)
 RTR3DECL(int) RTTcpServerDisconnectClient(PRTTCPSERVER pServer)
 {
     /*
-     * Validate input.
+     * Validate input and retain the instance.
      */
-    if (    !pServer
-        ||  pServer->enmState <= RTTCPSERVERSTATE_INVALID
-        ||  pServer->enmState >= RTTCPSERVERSTATE_FREED)
-    {
-        AssertMsgFailed(("Invalid parameter!\n"));
-        return VERR_INVALID_PARAMETER;
-    }
+    AssertPtrReturn(pServer, VERR_INVALID_HANDLE);
+    AssertReturn(pServer->u32Magic == RTTCPSERVER_MAGIC, VERR_INVALID_HANDLE);
+    AssertReturn(RTMemPoolRetain(pServer) != UINT32_MAX, VERR_INVALID_HANDLE);
 
-    return rtTcpServerDestroyClientSock(&pServer->SockClient, "DisconnectClient: client");
+    int rc = rtTcpServerDestroySocket(&pServer->SockClient, "DisconnectClient: client", true /*fTryGracefulShutdown*/);
+
+    RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
+    return rc;
+}
+
+
+/**
+ * Shuts down the server, leaving client connections open.
+ *
+ * @returns IPRT status code.
+ * @param   pServer         Handle to the server.
+ */
+RTR3DECL(int) RTTcpServerShutdown(PRTTCPSERVER pServer)
+{
+    /*
+     * Validate input and retain the instance.
+     */
+    AssertPtrReturn(pServer, VERR_INVALID_HANDLE);
+    AssertReturn(pServer->u32Magic == RTTCPSERVER_MAGIC, VERR_INVALID_HANDLE);
+    AssertReturn(RTMemPoolRetain(pServer) != UINT32_MAX, VERR_INVALID_HANDLE);
+
+    /*
+     * Try change the state to stopping, then replace and destroy the server socket.
+     */
+    for (;;)
+    {
+        RTTCPSERVERSTATE enmState = pServer->enmState;
+        if (    enmState != RTTCPSERVERSTATE_ACCEPTING
+            &&  enmState != RTTCPSERVERSTATE_SERVING)
+        {
+            RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
+            switch (enmState)
+            {
+                case RTTCPSERVERSTATE_CREATED:
+                case RTTCPSERVERSTATE_STARTING:
+                default:
+                    AssertMsgFailed(("%d\n", enmState));
+                    return VERR_INVALID_STATE;
+
+                case RTTCPSERVERSTATE_STOPPING:
+                case RTTCPSERVERSTATE_STOPPED:
+                    return VINF_SUCCESS;
+
+                case RTTCPSERVERSTATE_DESTROYING:
+                    return VERR_TCP_SERVER_DESTROYED;
+            }
+        }
+        if (rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_STOPPING, enmState))
+        {
+            rtTcpServerDestroySocket(&pServer->SockServer, "RTTcpServerShutdown", false /*fTryGracefulShutdown*/);
+            rtTcpServerSetState(pServer, RTTCPSERVERSTATE_STOPPED, RTTCPSERVERSTATE_STOPPING);
+
+            RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
+            return VINF_SUCCESS;
+        }
+    }
 }
 
 
@@ -647,111 +747,69 @@ RTR3DECL(int) RTTcpServerDisconnectClient(PRTTCPSERVER pServer)
 RTR3DECL(int) RTTcpServerDestroy(PRTTCPSERVER pServer)
 {
     /*
-     * Validate input.
+     * Validate input and retain the instance.
      */
-    if (    !pServer
-        ||  pServer->enmState <= RTTCPSERVERSTATE_INVALID
-        ||  pServer->enmState >= RTTCPSERVERSTATE_FREED)
-    {
-        AssertMsgFailed(("Invalid parameter!\n"));
-        return VERR_INVALID_PARAMETER;
-    }
-
-/** @todo r=bird: Some of this horrible code can probably be exchanged with a RTThreadWait(). (It didn't exist when this code was written.) */
+    AssertPtrReturn(pServer, VERR_INVALID_HANDLE);
+    AssertReturn(pServer->u32Magic == RTTCPSERVER_MAGIC, VERR_INVALID_HANDLE);
+    AssertReturn(RTMemPoolRetain(pServer) != UINT32_MAX, VERR_INVALID_HANDLE); /* paranoia */
 
     /*
-     * Move it to the destroying state.
+     * Move the state along so the listener can figure out what's going on.
      */
-    RTSOCKET    SockServer = rtTcpAtomicXchgSock(&pServer->SockServer, NIL_RTSOCKET);
-    for (unsigned cTries = 99; cTries > 0; cTries--)
+    for (;;)
     {
+        bool             fDestroyable;
         RTTCPSERVERSTATE enmState = pServer->enmState;
         switch (enmState)
         {
-            /*
-             * Try move it to the destroying state.
-             */
             case RTTCPSERVERSTATE_STARTING:
             case RTTCPSERVERSTATE_ACCEPTING:
             case RTTCPSERVERSTATE_SERVING:
-            {
-                if (rtTcpServerSetState(pServer, RTTCPSERVERSTATE_SIGNALING, enmState))
-                {
-                    /* client */
-                    rtTcpServerDestroyClientSock(&pServer->SockClient, "Destroyer: client");
-
-                    bool fRc = rtTcpServerSetState(pServer, RTTCPSERVERSTATE_DESTROYING, RTTCPSERVERSTATE_SIGNALING);
-                    Assert(fRc); NOREF(fRc);
-
-                    /* server */
-                    rtTcpServerDestroyServerSock(SockServer, "Destroyer: server destroying");
-                    RTThreadYield();
-
-                    return VINF_SUCCESS;
-                }
-                break;
-            }
-
-
-            /*
-             * Intermediate state.
-             */
-            case RTTCPSERVERSTATE_STOPPING:
-                if (!RTThreadYield())
-                    RTThreadSleep(1);
-                break;
-
-            /*
-             * Just release the handle.
-             */
             case RTTCPSERVERSTATE_CREATED:
             case RTTCPSERVERSTATE_STOPPED:
-                if (rtTcpServerSetState(pServer, RTTCPSERVERSTATE_FREED, enmState))
-                {
-                    rtTcpServerDestroyServerSock(SockServer, "Destroyer: server freeing");
-                    RTMemFree(pServer);
-                    return VINF_TCP_SERVER_STOP;
-                }
+                fDestroyable = rtTcpServerTrySetState(pServer, RTTCPSERVERSTATE_DESTROYING, enmState);
+                break;
+
+            /* destroyable states */
+            case RTTCPSERVERSTATE_STOPPING:
+                fDestroyable = true;
                 break;
 
             /*
-             * Everything else means failure.
+             * Everything else means user or internal misbehavior.
              */
             default:
                 AssertMsgFailed(("pServer=%p enmState=%d\n", pServer, enmState));
+                RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
                 return VERR_INTERNAL_ERROR;
         }
+        if (fDestroyable)
+            break;
     }
 
-    AssertMsgFailed(("Giving up! pServer=%p enmState=%d\n", pServer, pServer->enmState));
-    rtTcpServerDestroyServerSock(SockServer, "Destroyer: server timeout");
-    return VERR_INTERNAL_ERROR;
+    /*
+     * Destroy it.
+     */
+    ASMAtomicWriteU32(&pServer->u32Magic, ~RTTCPSERVER_MAGIC);
+    rtTcpServerDestroySocket(&pServer->SockServer, "Destroyer: server", false /*fTryGracefulShutdown*/);
+    rtTcpServerDestroySocket(&pServer->SockClient, "Destroyer: client", true  /*fTryGracefulShutdown*/);
+
+    /*
+     * Release it.
+     */
+    RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
+    RTMemPoolRelease(RTMEMPOOL_DEFAULT, pServer);
+    return VINF_SUCCESS;
 }
 
 
-/**
- * Shutdowns the server socket.
- */
-static void rtTcpServerDestroyServerSock(RTSOCKET SockServer, const char *pszMsg)
-{
-    if (SockServer == NIL_RTSOCKET)
-        return;
-    shutdown(SockServer, SHUT_RDWR);
-    rtTcpClose(SockServer, "Destroyer: server destroying");
-}
-
-
-
-RTR3DECL(int)  RTTcpRead(RTSOCKET Sock, void *pvBuffer, size_t cbBuffer, size_t *pcbRead)
+RTR3DECL(int) RTTcpRead(RTSOCKET Sock, void *pvBuffer, size_t cbBuffer, size_t *pcbRead)
 {
     /*
-     * Do params checking
+     * Validate input.
      */
-    if (!pvBuffer || !cbBuffer)
-    {
-        AssertMsgFailed(("Invalid params\n"));
-        return VERR_INVALID_PARAMETER;
-    }
+    AssertReturn(cbBuffer > 0, VERR_INVALID_PARAMETER);
+    AssertPtr(pvBuffer);
 
     /*
      * Read loop.
@@ -762,11 +820,25 @@ RTR3DECL(int)  RTTcpRead(RTSOCKET Sock, void *pvBuffer, size_t cbBuffer, size_t
     for (;;)
     {
         rtTcpErrorReset();
-        ssize_t cbBytesRead = recv(Sock, (char *)pvBuffer + cbRead, cbToRead, MSG_NOSIGNAL);
-        if (cbBytesRead < 0)
-            return rtTcpError();
-        if (cbBytesRead == 0 && rtTcpError())
-            return rtTcpError();
+#ifdef RT_OS_WINDOWS
+        int    cbNow = cbToRead >= INT_MAX/2 ? INT_MAX/2 : (int)cbToRead;
+#else
+        size_t cbNow = cbToRead;
+#endif
+        ssize_t cbBytesRead = recv(Sock, (char *)pvBuffer + cbRead, cbNow, MSG_NOSIGNAL);
+        if (cbBytesRead <= 0)
+        {
+            int rc = rtTcpError();
+            Assert(RT_FAILURE_NP(rc) || cbBytesRead == 0);
+            if (RT_FAILURE_NP(rc))
+                return rc;
+            if (pcbRead)
+            {
+                *pcbRead = 0;
+                return VINF_SUCCESS;
+            }
+            return VERR_NET_SHUTDOWN;
+        }
         if (pcbRead)
         {
             /* return partial data */
@@ -789,9 +861,15 @@ RTR3DECL(int)  RTTcpRead(RTSOCKET Sock, void *pvBuffer, size_t cbBuffer, size_t
 
 RTR3DECL(int)  RTTcpWrite(RTSOCKET Sock, const void *pvBuffer, size_t cbBuffer)
 {
+
     do
     {
-        ssize_t cbWritten = send(Sock, (const char *)pvBuffer, cbBuffer, MSG_NOSIGNAL);
+#ifdef RT_OS_WINDOWS
+        int    cbNow = cbBuffer >= INT_MAX/2 ? INT_MAX/2 : (int)cbBuffer;
+#else
+        size_t cbNow = cbBuffer;
+#endif
+        ssize_t cbWritten = send(Sock, (const char *)pvBuffer, cbNow, MSG_NOSIGNAL);
         if (cbWritten < 0)
             return rtTcpError();
         AssertMsg(cbBuffer >= (size_t)cbWritten, ("Wrote more than we requested!!! cbWritten=%d cbBuffer=%d rtTcpError()=%d\n",
@@ -841,15 +919,77 @@ RTR3DECL(int)  RTTcpSelectOne(RTSOCKET Sock, unsigned cMillies)
 }
 
 
+static int rtTcpConvertAddress(RTSOCKADDRUNION *pSrc, size_t cbSrc, PRTNETADDR pAddr)
+{
+    /*
+     * Convert the address.
+     */
+    if (   cbSrc == sizeof(struct sockaddr_in)
+        && pSrc->Addr.sa_family == AF_INET)
+    {
+        RT_ZERO(*pAddr);
+        pAddr->enmType      = RTNETADDRTYPE_IPV4;
+        pAddr->uPort        = RT_N2H_U16(pSrc->Ipv4.sin_port);
+        pAddr->uAddr.IPv4.u = pSrc->Ipv4.sin_addr.s_addr;
+    }
+#ifdef IPRT_WITH_TCPIP_V6
+    else if (   cbSrc == sizeof(struct sockaddr_in6)
+             && pSrc->Addr.sa_family == AF_INET6)
+    {
+        RT_ZERO(*pAddr);
+        pAddr->enmType            = RTNETADDRTYPE_IPV6;
+        pAddr->uPort              = RT_N2H_U16(pSrc->Ipv6.sin6_port);
+        pAddr->uAddr.IPv6.au32[0] = pSrc->Ipv6.sin6_addr.s6_addr32[0];
+        pAddr->uAddr.IPv6.au32[1] = pSrc->Ipv6.sin6_addr.s6_addr32[1];
+        pAddr->uAddr.IPv6.au32[2] = pSrc->Ipv6.sin6_addr.s6_addr32[2];
+        pAddr->uAddr.IPv6.au32[3] = pSrc->Ipv6.sin6_addr.s6_addr32[3];
+    }
+#endif
+    else
+        return VERR_NET_ADDRESS_FAMILY_NOT_SUPPORTED;
+    return VINF_SUCCESS;
+}
+
+
+RTR3DECL(int) RTTcpGetLocalAddress(RTSOCKET Sock, PRTNETADDR pAddr)
+{
+    RTSOCKADDRUNION u;
+#ifdef RT_OS_WINDOWS
+    int             cbAddr = sizeof(u);
+#else
+    socklen_t       cbAddr = sizeof(u);
+#endif
+    RT_ZERO(u);
+    if (!getsockname(Sock, &u.Addr, &cbAddr))
+        return rtTcpConvertAddress(&u, cbAddr, pAddr);
+    return rtTcpError();
+}
+
+
+RTR3DECL(int) RTTcpGetPeerAddress(RTSOCKET Sock, PRTNETADDR pAddr)
+{
+    RTSOCKADDRUNION u;
+#ifdef RT_OS_WINDOWS
+    int             cbAddr = sizeof(u);
+#else
+    socklen_t       cbAddr = sizeof(u);
+#endif
+    RT_ZERO(u);
+    if (!getpeername(Sock, &u.Addr, &cbAddr))
+        return rtTcpConvertAddress(&u, cbAddr, pAddr);
+    return rtTcpError();
+}
+
+
 RTR3DECL(int) RTTcpClientConnect(const char *pszAddress, uint32_t uPort, PRTSOCKET pSock)
 {
     int rc;
 
     /*
-     * Do params checking
+     * Validate input.
      */
-    AssertReturn(uPort, VERR_INVALID_PARAMETER);
-    AssertReturn(VALID_PTR(pszAddress), VERR_INVALID_PARAMETER);
+    AssertReturn(uPort > 0, VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pszAddress, VERR_INVALID_POINTER);
 
 #ifdef RT_OS_WINDOWS
     /*
@@ -899,7 +1039,7 @@ RTR3DECL(int) RTTcpClientConnect(const char *pszAddress, uint32_t uPort, PRTSOCK
             return VINF_SUCCESS;
         }
         rc = rtTcpError();
-        rtTcpClose(Sock, "RTTcpClientConnect");
+        rtTcpClose(Sock, "RTTcpClientConnect", false /*fTryGracefulShutdown*/);
     }
     else
         rc = rtTcpError();
@@ -909,26 +1049,59 @@ RTR3DECL(int) RTTcpClientConnect(const char *pszAddress, uint32_t uPort, PRTSOCK
 
 RTR3DECL(int) RTTcpClientClose(RTSOCKET Sock)
 {
-    return rtTcpClose(Sock, "RTTcpClientClose");
+    return rtTcpClose(Sock, "RTTcpClientClose", true /*fTryGracefulShutdown*/);
 }
 
 
 /**
  * Internal close function which does all the proper bitching.
  */
-static int rtTcpClose(RTSOCKET Sock, const char *pszMsg)
+static int rtTcpClose(RTSOCKET Sock, const char *pszMsg, bool fTryGracefulShutdown)
 {
+    int rc;
+
     /* ignore nil handles. */
     if (Sock == NIL_RTSOCKET)
         return VINF_SUCCESS;
 
+    /*
+     * Try to gracefully shut it down.
+     */
+    if (fTryGracefulShutdown)
+    {
+        rc = shutdown(Sock, SHUT_WR);
+        if (!rc)
+        {
+            uint64_t u64Start = RTTimeMilliTS();
+            for (;;)
+            {
+                rc = RTTcpSelectOne(Sock, 1000);
+                if (rc == VERR_TIMEOUT)
+                {
+                    if (RTTimeMilliTS() - u64Start > 30000)
+                        break;
+                }
+                else if (rc != VINF_SUCCESS)
+                    break;
+                {
+                    char abBitBucket[16*_1K];
+                    ssize_t cbBytesRead = recv(Sock, &abBitBucket[0], sizeof(abBitBucket), MSG_NOSIGNAL);
+                    if (cbBytesRead == 0)
+                        break; /* orderly shutdown in progress */
+                    if (cbBytesRead < 0)
+                        break; /* some kind of error, never mind which... */
+                }
+            }  /* forever */
+        }
+    }
+
     /*
      * Attempt to close it.
      */
 #ifdef RT_OS_WINDOWS
-    int rc = closesocket(Sock);
+    rc = closesocket(Sock);
 #else
-    int rc = close(Sock);
+    rc = close(Sock);
 #endif
     if (!rc)
         return VINF_SUCCESS;
