@@ -285,8 +285,10 @@ typedef struct CPUM
         uint32_t            ecx;
     } CPUFeaturesExt;
 
-    /* CPU manufacturer. */
-    CPUMCPUVENDOR           enmCPUVendor;
+    /* Host CPU manufacturer. */
+    CPUMCPUVENDOR           enmHostCpuVendor;
+    /* Guest CPU manufacturer. */
+    CPUMCPUVENDOR           enmGuestCpuVendor;
 
     /** CR4 mask */
     struct
@@ -297,7 +299,12 @@ typedef struct CPUM
 
     /** Have we entered rawmode? */
     bool                    fRawEntered;
-    uint8_t                 abPadding[3 + (HC_ARCH_BITS == 64) * 4];
+    /** Synthetic CPU type? */
+    bool                    fSyntheticCpu;
+    /** Indiciates that a state restore is pending.
+     * This is used to verify load order dependencies (PGM). */
+    bool                    fPendingRestore;
+    uint8_t                 abPadding[1 + (HC_ARCH_BITS == 64) * 4];
 
     /** The standard set of CpuId leafs. */
     CPUMCPUID               aGuestCpuIdStd[6];
@@ -308,8 +315,10 @@ typedef struct CPUM
     /** The default set of CpuId leafs. */
     CPUMCPUID               GuestCpuIdDef;
 
+#if HC_ARCH_BITS == 32
     /** Align the next member, and thereby the structure, on a 64-byte boundrary. */
-    uint8_t                 abPadding2[HC_ARCH_BITS == 32 ? 8 : 4];
+    uint8_t                 abPadding2[4];
+#endif
 
     /**
      * Guest context on raw mode entry.
