@@ -294,29 +294,29 @@ DECLINLINE(int) emR3ExecuteInstruction(PVM pVM, PVMCPU pVCpu, const char *pszPre
  */
 static int emR3ExecuteIOInstruction(PVM pVM, PVMCPU pVCpu)
 {
-    int         rc;
-    PCPUMCTX    pCtx = pVCpu->em.s.pCtx;
+    PCPUMCTX pCtx = pVCpu->em.s.pCtx;
 
     STAM_PROFILE_START(&pVCpu->em.s.StatIOEmu, a);
 
     /* Try to restart the io instruction that was refused in ring-0. */
-    rc = HWACCMR3RestartPendingIOInstr(pVM, pVCpu, pCtx);
-    if (IOM_SUCCESS(rc))
+    VBOXSTRICTRC rcStrict = HWACCMR3RestartPendingIOInstr(pVM, pVCpu, pCtx);
+    if (IOM_SUCCESS(rcStrict))
     {
         STAM_COUNTER_INC(&pVCpu->em.s.CTX_SUFF(pStats)->StatIoRestarted);
         STAM_PROFILE_STOP(&pVCpu->em.s.StatIOEmu, a);
-        return rc;     /* rip already updated. */
+        return VBOXSTRICTRC_TODO(rcStrict);     /* rip already updated. */
     }
-    AssertMsgReturn(rc == VERR_NOT_FOUND, ("%Rrc\n", rc), RT_SUCCESS_NP(rc) ? VERR_INTERNAL_ERROR_5 : rc);
+    AssertMsgReturn(rcStrict == VERR_NOT_FOUND, ("%Rrc\n", VBOXSTRICTRC_VAL(rcStrict)),
+                    RT_SUCCESS_NP(rcStrict) ? VERR_INTERNAL_ERROR_5 : VBOXSTRICTRC_TODO(rcStrict));
 
     /** @todo probably we should fall back to the recompiler; otherwise we'll go back and forth between HC & GC
      *   as io instructions tend to come in packages of more than one
      */
     DISCPUSTATE Cpu;
-    rc = CPUMR3DisasmInstrCPU(pVM, pVCpu, pCtx, pCtx->rip, &Cpu, "IO EMU");
-    if (RT_SUCCESS(rc))
+    int rc2 = CPUMR3DisasmInstrCPU(pVM, pVCpu, pCtx, pCtx->rip, &Cpu, "IO EMU");
+    if (RT_SUCCESS(rc2))
     {
-        rc = VINF_EM_RAW_EMULATE_INSTR;
+        rcStrict = VINF_EM_RAW_EMULATE_INSTR;
 
         if (!(Cpu.prefix & (PREFIX_REP | PREFIX_REPNE)))
         {
@@ -325,14 +325,14 @@ static int emR3ExecuteIOInstruction(PVM pVM, PVMCPU pVCpu)
                 case OP_IN:
                 {
                     STAM_COUNTER_INC(&pVCpu->em.s.CTX_SUFF(pStats)->StatIn);
-                    rc = IOMInterpretIN(pVM, CPUMCTX2CORE(pCtx), &Cpu);
+                    rcStrict = IOMInterpretIN(pVM, CPUMCTX2CORE(pCtx), &Cpu);
                     break;
                 }
 
                 case OP_OUT:
                 {
                     STAM_COUNTER_INC(&pVCpu->em.s.CTX_SUFF(pStats)->StatOut);
-                    rc = IOMInterpretOUT(pVM, CPUMCTX2CORE(pCtx), &Cpu);
+                    rcStrict = IOMInterpretOUT(pVM, CPUMCTX2CORE(pCtx), &Cpu);
                     break;
                 }
             }
@@ -345,7 +345,7 @@ static int emR3ExecuteIOInstruction(PVM pVM, PVMCPU pVCpu)
                 case OP_INSWD:
                 {
                     STAM_COUNTER_INC(&pVCpu->em.s.CTX_SUFF(pStats)->StatIn);
-                    rc = IOMInterpretINS(pVM, CPUMCTX2CORE(pCtx), &Cpu);
+                    rcStrict = IOMInterpretINS(pVM, CPUMCTX2CORE(pCtx), &Cpu);
                     break;
                 }
 
@@ -353,7 +353,7 @@ static int emR3ExecuteIOInstruction(PVM pVM, PVMCPU pVCpu)
                 case OP_OUTSWD:
                 {
                     STAM_COUNTER_INC(&pVCpu->em.s.CTX_SUFF(pStats)->StatOut);
-                    rc = IOMInterpretOUTS(pVM, CPUMCTX2CORE(pCtx), &Cpu);
+                    rcStrict = IOMInterpretOUTS(pVM, CPUMCTX2CORE(pCtx), &Cpu);
                     break;
                 }
             }
@@ -361,31 +361,32 @@ static int emR3ExecuteIOInstruction(PVM pVM, PVMCPU pVCpu)
 
         /*
          * Handled the I/O return codes.
-         * (The unhandled cases end up with rc == VINF_EM_RAW_EMULATE_INSTR.)
+         * (The unhandled cases end up with rcStrict == VINF_EM_RAW_EMULATE_INSTR.)
          */
-        if (IOM_SUCCESS(rc))
+        if (IOM_SUCCESS(rcStrict))
         {
             pCtx->rip += Cpu.opsize;
             STAM_PROFILE_STOP(&pVCpu->em.s.StatIOEmu, a);
-            return rc;
+            return VBOXSTRICTRC_TODO(rcStrict);
         }
 
-        if (rc == VINF_EM_RAW_GUEST_TRAP)
+        if (rcStrict == VINF_EM_RAW_GUEST_TRAP)
         {
             /* The active trap will be dispatched. */
             Assert(TRPMHasTrap(pVCpu));
             STAM_PROFILE_STOP(&pVCpu->em.s.StatIOEmu, a);
             return VINF_SUCCESS;
         }
-        AssertMsg(rc != VINF_TRPM_XCPT_DISPATCHED, ("Handle VINF_TRPM_XCPT_DISPATCHED\n"));
+        AssertMsg(rcStrict != VINF_TRPM_XCPT_DISPATCHED, ("Handle VINF_TRPM_XCPT_DISPATCHED\n"));
 
-        if (RT_FAILURE(rc))
+        if (RT_FAILURE(rcStrict))
         {
             STAM_PROFILE_STOP(&pVCpu->em.s.StatIOEmu, a);
-            return rc;
+            return VBOXSTRICTRC_TODO(rcStrict);
         }
-        AssertMsg(rc == VINF_EM_RAW_EMULATE_INSTR || rc == VINF_EM_RESCHEDULE_REM, ("rc=%Rrc\n", rc));
+        AssertMsg(rcStrict == VINF_EM_RAW_EMULATE_INSTR || rcStrict == VINF_EM_RESCHEDULE_REM, ("rcStrict=%Rrc\n", VBOXSTRICTRC_VAL(rcStrict)));
     }
+
     STAM_PROFILE_STOP(&pVCpu->em.s.StatIOEmu, a);
     return emR3ExecuteInstruction(pVM, pVCpu, "IO: ");
 }
@@ -516,7 +517,7 @@ int emR3HwAccExecute(PVM pVM, PVMCPU pVCpu, bool *pfFFDone)
 
         uint32_t cpl = CPUMGetGuestCPL(pVCpu, CPUMCTX2CORE(pCtx));
 
-        if (pVM->cCPUs == 1)
+        if (pVM->cCpus == 1)
         {
             if (pCtx->eflags.Bits.u1VM)
                 Log(("HWV86: %08X IF=%d\n", pCtx->eip, pCtx->eflags.Bits.u1IF));
