@@ -1,4 +1,4 @@
-/* $Id: IOMInternal.h 29963 2008-04-21 14:29:54Z sandervl $ */
+/* $Id: IOMInternal.h $ */
 /** @file
  * IOM - Internal header file.
  */
@@ -27,12 +27,11 @@
 #include <VBox/iom.h>
 #include <VBox/stam.h>
 #include <VBox/pgm.h>
+#include <VBox/pdmcritsect.h>
 #include <VBox/param.h>
+#include <iprt/assert.h>
 #include <iprt/avl.h>
 
-#if !defined(IN_IOM_R3) && !defined(IN_IOM_R0) && !defined(IN_IOM_GC)
-# error "Not in IOM! This is an internal header!"
-#endif
 
 
 /** @defgroup grp_iom_int   Internals
@@ -54,39 +53,40 @@ typedef struct IOMMMIORANGE
     uint32_t                    cb;
     uint32_t                    u32Alignment; /**< Alignment padding. */
 
-    /** Pointer to user argument. */
+    /** Pointer to user argument - R3. */
     RTR3PTR                     pvUserR3;
-    /** Pointer to device instance. */
+    /** Pointer to device instance - R3. */
     PPDMDEVINSR3                pDevInsR3;
-    /** Pointer to write callback function. */
+    /** Pointer to write callback function - R3. */
     R3PTRTYPE(PFNIOMMMIOWRITE)  pfnWriteCallbackR3;
-    /** Pointer to read callback function. */
+    /** Pointer to read callback function - R3. */
     R3PTRTYPE(PFNIOMMMIOREAD)   pfnReadCallbackR3;
-    /** Pointer to fill (memset) callback function. */
+    /** Pointer to fill (memset) callback function - R3. */
     R3PTRTYPE(PFNIOMMMIOFILL)   pfnFillCallbackR3;
 
-    /** Pointer to user argument. */
+    /** Pointer to user argument - R0. */
     RTR0PTR                     pvUserR0;
-    /** Pointer to device instance. */
+    /** Pointer to device instance - R0. */
     PPDMDEVINSR0                pDevInsR0;
-    /** Pointer to write callback function. */
+    /** Pointer to write callback function - R0. */
     R0PTRTYPE(PFNIOMMMIOWRITE)  pfnWriteCallbackR0;
-    /** Pointer to read callback function. */
+    /** Pointer to read callback function - R0. */
     R0PTRTYPE(PFNIOMMMIOREAD)   pfnReadCallbackR0;
-    /** Pointer to fill (memset) callback function. */
+    /** Pointer to fill (memset) callback function - R0. */
     R0PTRTYPE(PFNIOMMMIOFILL)   pfnFillCallbackR0;
 
-    /** Pointer to user argument. */
-    RTGCPTR                     pvUserGC;
-    /** Pointer to device instance. */
-    PPDMDEVINSGC                pDevInsGC;
-    /** Pointer to write callback function. */
-    GCPTRTYPE(PFNIOMMMIOWRITE)  pfnWriteCallbackGC;
-    /** Pointer to read callback function. */
-    GCPTRTYPE(PFNIOMMMIOREAD)   pfnReadCallbackGC;
-    /** Pointer to fill (memset) callback function. */
-    GCPTRTYPE(PFNIOMMMIOFILL)   pfnFillCallbackGC;
-    RTGCPTR                     GCPtrAlignment; /**< Alignment padding */
+    /** Pointer to user argument - RC. */
+    RTRCPTR                     pvUserRC;
+    /** Pointer to device instance - RC. */
+    PPDMDEVINSRC                pDevInsRC;
+    /** Pointer to write callback function - RC. */
+    RCPTRTYPE(PFNIOMMMIOWRITE)  pfnWriteCallbackRC;
+    /** Pointer to read callback function - RC. */
+    RCPTRTYPE(PFNIOMMMIOREAD)   pfnReadCallbackRC;
+    /** Pointer to fill (memset) callback function - RC. */
+    RCPTRTYPE(PFNIOMMMIOFILL)   pfnFillCallbackRC;
+    /** Alignment padding. */
+    RTRCPTR                     RCPtrAlignment;
 
     /** Description / Name. For easing debugging. */
     R3PTRTYPE(const char *)     pszDesc;
@@ -99,45 +99,38 @@ typedef struct IOMMMIORANGE *PIOMMMIORANGE;
  * MMIO address statistics. (one address)
  *
  * This is a simple way of making on demand statistics, however it's a
- * bit free with the hypervisor heap memory..
+ * bit free with the hypervisor heap memory.
  */
 typedef struct IOMMMIOSTATS
 {
     /** Avl node core with the address as Key. */
     AVLOGCPHYSNODECORE          Core;
+
     /** Number of reads to this address from R3. */
     STAMCOUNTER                 ReadR3;
-    /** Number of writes to this address from R3. */
-    STAMCOUNTER                 WriteR3;
-    /** Number of reads to this address from R0. */
-    STAMCOUNTER                 ReadR0;
-    /** Number of writes to this address from R0. */
-    STAMCOUNTER                 WriteR0;
-    /** Number of reads to this address from GC. */
-    STAMCOUNTER                 ReadGC;
-    /** Number of writes to this address from GC. */
-    STAMCOUNTER                 WriteGC;
     /** Profiling read handler overhead in R3. */
     STAMPROFILEADV              ProfReadR3;
+
+    /** Number of writes to this address from R3. */
+    STAMCOUNTER                 WriteR3;
     /** Profiling write handler overhead in R3. */
     STAMPROFILEADV              ProfWriteR3;
-    /** Profiling read handler overhead in R0. */
-    STAMPROFILEADV              ProfReadR0;
-    /** Profiling write handler overhead in R0. */
-    STAMPROFILEADV              ProfWriteR0;
-    /** Profiling read handler overhead in GC. */
-    STAMPROFILEADV              ProfReadGC;
-    /** Profiling write handler overhead in GC. */
-    STAMPROFILEADV              ProfWriteGC;
-    /** Number of reads to this address from R0 which was serviced in R3. */
-    STAMCOUNTER                 ReadR0ToR3;
-    /** Number of writes to this address from R0 which was serviced in R3. */
-    STAMCOUNTER                 WriteR0ToR3;
-    /** Number of reads to this address from GC which was serviced in R3. */
-    STAMCOUNTER                 ReadGCToR3;
-    /** Number of writes to this address from GC which was serviced in R3. */
-    STAMCOUNTER                 WriteGCToR3;
+
+    /** Number of reads to this address from R0/RC. */
+    STAMCOUNTER                 ReadRZ;
+    /** Profiling read handler overhead in R0/RC. */
+    STAMPROFILEADV              ProfReadRZ;
+    /** Number of reads to this address from R0/RC which was serviced in R3. */
+    STAMCOUNTER                 ReadRZToR3;
+
+    /** Number of writes to this address from R0/RC. */
+    STAMCOUNTER                 WriteRZ;
+    /** Profiling write handler overhead in R0/RC. */
+    STAMPROFILEADV              ProfWriteRZ;
+    /** Number of writes to this address from R0/RC which was serviced in R3. */
+    STAMCOUNTER                 WriteRZToR3;
 } IOMMMIOSTATS;
+AssertCompileMemberAlignment(IOMMMIOSTATS, ReadR3, 8);
 /** Pointer to I/O port statistics. */
 typedef IOMMMIOSTATS *PIOMMMIOSTATS;
 
@@ -149,7 +142,7 @@ typedef struct IOMIOPORTRANGER3
 {
     /** Avl node core with Port as Key and Port + cPorts - 1 as KeyLast. */
     AVLROIOPORTNODECORE         Core;
-#if HC_ARCH_BITS == 64 && GC_ARCH_BITS == 32 && !defined(RT_OS_WINDOWS)
+#if HC_ARCH_BITS == 64 && !defined(RT_OS_WINDOWS)
     uint32_t                    u32Alignment; /**< The sizeof(Core) differs. */
 #endif
     /** Start I/O port address. */
@@ -181,7 +174,7 @@ typedef struct IOMIOPORTRANGER0
 {
     /** Avl node core with Port as Key and Port + cPorts - 1 as KeyLast. */
     AVLROIOPORTNODECORE         Core;
-#if HC_ARCH_BITS == 64 && GC_ARCH_BITS == 32 && !defined(RT_OS_WINDOWS)
+#if HC_ARCH_BITS == 64 && !defined(RT_OS_WINDOWS)
     uint32_t                    u32Alignment; /**< The sizeof(Core) differs. */
 #endif
     /** Start I/O port address. */
@@ -207,9 +200,9 @@ typedef struct IOMIOPORTRANGER0
 typedef IOMIOPORTRANGER0 *PIOMIOPORTRANGER0;
 
 /**
- * I/O port range descriptor.
+ * I/O port range descriptor, RC version.
  */
-typedef struct IOMIOPORTRANGEGC
+typedef struct IOMIOPORTRANGERC
 {
     /** Avl node core with Port as Key and Port + cPorts - 1 as KeyLast. */
     AVLROIOPORTNODECORE         Core;
@@ -218,73 +211,64 @@ typedef struct IOMIOPORTRANGEGC
     /** Size of the range. */
     uint16_t                    cPorts;
     /** Pointer to user argument. */
-    RTGCPTR                     pvUser;
+    RTRCPTR                     pvUser;
     /** Pointer to the associated device instance. */
-    GCPTRTYPE(PPDMDEVINS)       pDevIns;
+    RCPTRTYPE(PPDMDEVINS)       pDevIns;
     /** Pointer to OUT callback function. */
-    GCPTRTYPE(PFNIOMIOPORTOUT)  pfnOutCallback;
+    RCPTRTYPE(PFNIOMIOPORTOUT)  pfnOutCallback;
     /** Pointer to IN callback function. */
-    GCPTRTYPE(PFNIOMIOPORTIN)   pfnInCallback;
+    RCPTRTYPE(PFNIOMIOPORTIN)   pfnInCallback;
     /** Pointer to string OUT callback function. */
-    GCPTRTYPE(PFNIOMIOPORTOUTSTRING) pfnOutStrCallback;
+    RCPTRTYPE(PFNIOMIOPORTOUTSTRING) pfnOutStrCallback;
     /** Pointer to string IN callback function. */
-    GCPTRTYPE(PFNIOMIOPORTINSTRING) pfnInStrCallback;
-#if HC_ARCH_BITS == 64 && GC_ARCH_BITS == 32
-    RTGCPTR                     GCPtrAlignment; /**< pszDesc is 8 byte aligned. */
+    RCPTRTYPE(PFNIOMIOPORTINSTRING) pfnInStrCallback;
+#if HC_ARCH_BITS == 64
+    RTRCPTR                     RCPtrAlignment; /**< pszDesc is 8 byte aligned. */
 #endif
     /** Description / Name. For easing debugging. */
     R3PTRTYPE(const char *)     pszDesc;
-} IOMIOPORTRANGEGC;
-/** Pointer to I/O port range descriptor, GC version. */
-typedef IOMIOPORTRANGEGC *PIOMIOPORTRANGEGC;
+} IOMIOPORTRANGERC;
+/** Pointer to I/O port range descriptor, RC version. */
+typedef IOMIOPORTRANGERC *PIOMIOPORTRANGERC;
 
 
 /**
  * I/O port statistics. (one I/O port)
  *
  * This is a simple way of making on demand statistics, however it's a
- * bit free with the hypervisor heap memory..
+ * bit free with the hypervisor heap memory.
  */
 typedef struct IOMIOPORTSTATS
 {
     /** Avl node core with the port as Key. */
     AVLOIOPORTNODECORE          Core;
-#if HC_ARCH_BITS == 64 && GC_ARCH_BITS == 32 && !defined(RT_OS_WINDOWS)
+#if HC_ARCH_BITS != 64 || !defined(RT_OS_WINDOWS)
     uint32_t                    u32Alignment; /**< The sizeof(Core) differs. */
 #endif
     /** Number of INs to this port from R3. */
     STAMCOUNTER                 InR3;
-    /** Number of OUTs to this port from R3. */
-    STAMCOUNTER                 OutR3;
-    /** Number of INs to this port from R0. */
-    STAMCOUNTER                 InR0;
-    /** Number of OUTs to this port from R0. */
-    STAMCOUNTER                 OutR0;
-    /** Number of INs to this port from GC. */
-    STAMCOUNTER                 InGC;
-    /** Number of OUTs to this port from GC. */
-    STAMCOUNTER                 OutGC;
     /** Profiling IN handler overhead in R3. */
     STAMPROFILEADV              ProfInR3;
+    /** Number of OUTs to this port from R3. */
+    STAMCOUNTER                 OutR3;
     /** Profiling OUT handler overhead in R3. */
     STAMPROFILEADV              ProfOutR3;
-    /** Profiling IN handler overhead in R0. */
-    STAMPROFILEADV              ProfInR0;
-    /** Profiling OUT handler overhead in R0. */
-    STAMPROFILEADV              ProfOutR0;
-    /** Profiling IN handler overhead in GC. */
-    STAMPROFILEADV              ProfInGC;
-    /** Profiling OUT handler overhead in GC. */
-    STAMPROFILEADV              ProfOutGC;
-    /** Number of INs to this port from R0 which was serviced in R3. */
-    STAMCOUNTER                 InR0ToR3;
-    /** Number of OUTs to this port from R0 which was serviced in R3. */
-    STAMCOUNTER                 OutR0ToR3;
-    /** Number of INs to this port from GC which was serviced in R3. */
-    STAMCOUNTER                 InGCToR3;
-    /** Number of OUTs to this port from GC which was serviced in R3. */
-    STAMCOUNTER                 OutGCToR3;
+
+    /** Number of INs to this port from R0/RC. */
+    STAMCOUNTER                 InRZ;
+    /** Profiling IN handler overhead in R0/RC. */
+    STAMPROFILEADV              ProfInRZ;
+    /** Number of INs to this port from R0/RC which was serviced in R3. */
+    STAMCOUNTER                 InRZToR3;
+
+    /** Number of OUTs to this port from R0/RC. */
+    STAMCOUNTER                 OutRZ;
+    /** Profiling OUT handler overhead in R0/RC. */
+    STAMPROFILEADV              ProfOutRZ;
+    /** Number of OUTs to this port from R0/RC which was serviced in R3. */
+    STAMCOUNTER                 OutRZToR3;
 } IOMIOPORTSTATS;
+AssertCompileMemberAlignment(IOMIOPORTSTATS, InR3, 8);
 /** Pointer to I/O port statistics. */
 typedef IOMIOPORTSTATS *PIOMIOPORTSTATS;
 
@@ -293,7 +277,7 @@ typedef IOMIOPORTSTATS *PIOMIOPORTSTATS;
  * The IOM trees.
  * These are offset based the nodes and root must be in the same
  * memory block in HC. The locations of IOM structure and the hypervisor heap
- * are quite different in HC and GC.
+ * are quite different in R3, R0 and RC.
  */
 typedef struct IOMTREES
 {
@@ -301,8 +285,8 @@ typedef struct IOMTREES
     AVLROIOPORTTREE         IOPortTreeR3;
     /** Tree containing I/O port range descriptors registered for R0 (IOMIOPORTRANGER0). */
     AVLROIOPORTTREE         IOPortTreeR0;
-    /** Tree containing I/O port range descriptors registered for GC (IOMIOPORTRANGEGC). */
-    AVLROIOPORTTREE         IOPortTreeGC;
+    /** Tree containing I/O port range descriptors registered for RC (IOMIOPORTRANGERC). */
+    AVLROIOPORTTREE         IOPortTreeRC;
 
     /** Tree containing the MMIO range descriptors (IOMMMIORANGE). */
     AVLROGCPHYSTREE         MMIOTree;
@@ -331,16 +315,23 @@ typedef struct IOM
     /** Offset to the VM structure. */
     RTINT                           offVM;
 
-    /** Pointer to the trees - GC ptr. */
-    GCPTRTYPE(PIOMTREES)            pTreesGC;
-    /** Pointer to the trees - HC ptr. */
-    R3R0PTRTYPE(PIOMTREES)          pTreesHC;
+    /** Pointer to the trees - RC ptr. */
+    RCPTRTYPE(PIOMTREES)            pTreesRC;
+    /** Pointer to the trees - R3 ptr. */
+    R3PTRTYPE(PIOMTREES)            pTreesR3;
+    /** Pointer to the trees - R0 ptr. */
+    R0PTRTYPE(PIOMTREES)            pTreesR0;
 
     /** The ring-0 address of IOMMMIOHandler. */
     R0PTRTYPE(PFNPGMR0PHYSHANDLER)  pfnMMIOHandlerR0;
-    /** The GC address of IOMMMIOHandler. */
-    GCPTRTYPE(PFNPGMGCPHYSHANDLER)  pfnMMIOHandlerGC;
-    RTGCPTR                         Alignment;
+    /** The RC address of IOMMMIOHandler. */
+    RCPTRTYPE(PFNPGMRCPHYSHANDLER)  pfnMMIOHandlerRC;
+#if HC_ARCH_BITS == 64
+    RTRCPTR                         padding;
+#endif
+
+    /** Lock serializing EMT access to IOM. */
+    PDMCRITSECT                     EmtLock;
 
     /** @name Caching of I/O Port and MMIO ranges and statistics.
      * (Saves quite some time in rep outs/ins instruction emulation.)
@@ -359,75 +350,93 @@ typedef struct IOM
     R0PTRTYPE(PIOMMMIORANGE)        pMMIORangeLastR0;
     R0PTRTYPE(PIOMMMIOSTATS)        pMMIOStatsLastR0;
 
-    GCPTRTYPE(PIOMIOPORTRANGEGC)    pRangeLastReadGC;
-    GCPTRTYPE(PIOMIOPORTRANGEGC)    pRangeLastWriteGC;
-    GCPTRTYPE(PIOMIOPORTSTATS)      pStatsLastReadGC;
-    GCPTRTYPE(PIOMIOPORTSTATS)      pStatsLastWriteGC;
-    GCPTRTYPE(PIOMMMIORANGE)        pMMIORangeLastGC;
-    GCPTRTYPE(PIOMMMIOSTATS)        pMMIOStatsLastGC;
+    RCPTRTYPE(PIOMIOPORTRANGERC)    pRangeLastReadRC;
+    RCPTRTYPE(PIOMIOPORTRANGERC)    pRangeLastWriteRC;
+    RCPTRTYPE(PIOMIOPORTSTATS)      pStatsLastReadRC;
+    RCPTRTYPE(PIOMIOPORTSTATS)      pStatsLastWriteRC;
+    RCPTRTYPE(PIOMMMIORANGE)        pMMIORangeLastRC;
+    RCPTRTYPE(PIOMMMIOSTATS)        pMMIOStatsLastRC;
     /** @} */
 
     /** @name I/O Port statistics.
      * @{ */
-    STAMPROFILE             StatGCIOPortHandler;
-
-    STAMCOUNTER             StatGCInstIn;
-    STAMCOUNTER             StatGCInstOut;
-    STAMCOUNTER             StatGCInstIns;
-    STAMCOUNTER             StatGCInstOuts;
+    STAMCOUNTER                     StatInstIn;
+    STAMCOUNTER                     StatInstOut;
+    STAMCOUNTER                     StatInstIns;
+    STAMCOUNTER                     StatInstOuts;
     /** @} */
 
     /** @name MMIO statistics.
      * @{ */
-    STAMPROFILE             StatGCMMIOHandler;
-    STAMCOUNTER             StatGCMMIOFailures;
+    STAMPROFILE                     StatRZMMIOHandler;
+    STAMCOUNTER                     StatRZMMIOFailures;
 
-    STAMPROFILE             StatGCInstMov;
-    STAMPROFILE             StatGCInstCmp;
-    STAMPROFILE             StatGCInstAnd;
-    STAMPROFILE             StatGCInstTest;
-    STAMPROFILE             StatGCInstXchg;
-    STAMPROFILE             StatGCInstStos;
-    STAMPROFILE             StatGCInstLods;
-    STAMPROFILE             StatGCInstMovs;
-    STAMPROFILE             StatGCInstMovsToMMIO;
-    STAMPROFILE             StatGCInstMovsFromMMIO;
-    STAMPROFILE             StatGCInstMovsMMIO;
-    STAMCOUNTER             StatGCInstOther;
+    STAMPROFILE                     StatRZInstMov;
+    STAMPROFILE                     StatRZInstCmp;
+    STAMPROFILE                     StatRZInstAnd;
+    STAMPROFILE                     StatRZInstOr;
+    STAMPROFILE                     StatRZInstXor;
+    STAMPROFILE                     StatRZInstBt;
+    STAMPROFILE                     StatRZInstTest;
+    STAMPROFILE                     StatRZInstXchg;
+    STAMPROFILE                     StatRZInstStos;
+    STAMPROFILE                     StatRZInstLods;
+#ifdef IOM_WITH_MOVS_SUPPORT
+    STAMPROFILEADV                  StatRZInstMovs;
+    STAMPROFILE                     StatRZInstMovsToMMIO;
+    STAMPROFILE                     StatRZInstMovsFromMMIO;
+    STAMPROFILE                     StatRZInstMovsMMIO;
+#endif
+    STAMCOUNTER                     StatRZInstOther;
 
-    STAMCOUNTER             StatGCMMIO1Byte;
-    STAMCOUNTER             StatGCMMIO2Bytes;
-    STAMCOUNTER             StatGCMMIO4Bytes;
+    STAMCOUNTER                     StatRZMMIO1Byte;
+    STAMCOUNTER                     StatRZMMIO2Bytes;
+    STAMCOUNTER                     StatRZMMIO4Bytes;
+    STAMCOUNTER                     StatRZMMIO8Bytes;
 
-    RTUINT                  cMovsMaxBytes;
-    RTUINT                  cStosMaxBytes;
+    STAMCOUNTER                     StatR3MMIOHandler;
+
+    RTUINT                          cMovsMaxBytes;
+    RTUINT                          cStosMaxBytes;
     /** @} */
-
 } IOM;
 /** Pointer to IOM instance data. */
 typedef IOM *PIOM;
 
 
-__BEGIN_DECLS
-
-#ifdef IN_IOM_R3
-PIOMIOPORTSTATS iomr3IOPortStatsCreate(PVM pVM, RTIOPORT Port, const char *pszDesc);
-PIOMMMIOSTATS iomR3MMIOStatsCreate(PVM pVM, RTGCPHYS GCPhys, const char *pszDesc);
-#endif /* IN_IOM_R3 */
-
 /**
- * \#PF Handler callback for MMIO ranges.
- *
- * @returns VBox status code (appropriate for GC return).
- *
- * @param   pVM         VM Handle.
- * @param   uErrorCode  CPU Error code.
- * @param   pRegFrame   Trap register frame.
- * @param   pvFault     The fault address (cr2).
- * @param   GCPhysFault The GC physical address corresponding to pvFault.
- * @param   pvUser      Pointer to the MMIO range entry.
+ * IOM per virtual CPU instance data.
  */
-IOMDECL(int) IOMMMIOHandler(PVM pVM, RTGCUINT uErrorCode, PCPUMCTXCORE pRegFrame, void *pvFault, RTGCPHYS GCPhysFault, void *pvUser);
+typedef struct IOMCPU
+{
+    /** For saving stack space, the disassembler state is allocated here instead of
+     * on the stack.
+     * @note The DISCPUSTATE structure is not R3/R0/RZ clean!  */
+    union
+    {
+        /** The disassembler scratch space. */
+        DISCPUSTATE                 DisState;
+        /** Padding. */
+        uint8_t                     abDisStatePadding[DISCPUSTATE_PADDING_SIZE];
+    };
+    uint8_t                         Dummy[16];
+} IOMCPU;
+/** Pointer to IOM per virtual CPU instance data. */
+typedef IOMCPU *PIOMCPU;
+
+
+RT_C_DECLS_BEGIN
+
+#ifdef IN_RING3
+PIOMIOPORTSTATS iomR3IOPortStatsCreate(PVM pVM, RTIOPORT Port, const char *pszDesc);
+PIOMMMIOSTATS   iomR3MMIOStatsCreate(PVM pVM, RTGCPHYS GCPhys, const char *pszDesc);
+#endif /* IN_RING3 */
+
+VMMDECL(int)        IOMMMIOHandler(PVM pVM, RTGCUINT uErrorCode, PCPUMCTXCORE pRegFrame, RTGCPTR pvFault, RTGCPHYS GCPhysFault, void *pvUser);
+#ifdef IN_RING3
+DECLCALLBACK(int)   IOMR3MMIOHandler(PVM pVM, RTGCPHYS GCPhys, void *pvPhys, void *pvBuf, size_t cbBuf, PGMACCESSTYPE enmAccessType, void *pvUser);
+#endif
+
 
 /**
  * Gets the I/O port range for the specified I/O port in the current context.
@@ -438,12 +447,17 @@ IOMDECL(int) IOMMMIOHandler(PVM pVM, RTGCUINT uErrorCode, PCPUMCTXCORE pRegFrame
  * @param   pIOM    IOM instance data.
  * @param   Port    Port to lookup.
  */
-DECLINLINE(CTXALLSUFF(PIOMIOPORTRANGE)) iomIOPortGetRange(PIOM pIOM, RTIOPORT Port)
+DECLINLINE(CTX_SUFF(PIOMIOPORTRANGE)) iomIOPortGetRange(PIOM pIOM, RTIOPORT Port)
 {
-    CTXALLSUFF(PIOMIOPORTRANGE) pRange = (CTXALLSUFF(PIOMIOPORTRANGE))RTAvlroIOPortRangeGet(&pIOM->CTXSUFF(pTrees)->CTXALLSUFF(IOPortTree), Port);
+#ifdef IN_RING3
+    if (PDMCritSectIsInitialized(&pIOM->EmtLock))
+#endif
+        Assert(IOMIsLockOwner(IOM2VM(pIOM)));
+    CTX_SUFF(PIOMIOPORTRANGE) pRange = (CTX_SUFF(PIOMIOPORTRANGE))RTAvlroIOPortRangeGet(&pIOM->CTX_SUFF(pTrees)->CTX_SUFF(IOPortTree), Port);
     return pRange;
 }
 
+
 /**
  * Gets the I/O port range for the specified I/O port in the HC.
  *
@@ -453,9 +467,13 @@ DECLINLINE(CTXALLSUFF(PIOMIOPORTRANGE)) iomIOPortGetRange(PIOM pIOM, RTIOPORT Po
  * @param   pIOM    IOM instance data.
  * @param   Port    Port to lookup.
  */
-DECLINLINE(PIOMIOPORTRANGER3) iomIOPortGetRangeHC(PIOM pIOM, RTIOPORT Port)
+DECLINLINE(PIOMIOPORTRANGER3) iomIOPortGetRangeR3(PIOM pIOM, RTIOPORT Port)
 {
-    PIOMIOPORTRANGER3 pRange = (PIOMIOPORTRANGER3)RTAvlroIOPortRangeGet(&pIOM->CTXSUFF(pTrees)->IOPortTreeR3, Port);
+#ifdef IN_RING3
+    if (PDMCritSectIsInitialized(&pIOM->EmtLock))
+#endif
+        Assert(IOMIsLockOwner(IOM2VM(pIOM)));
+    PIOMIOPORTRANGER3 pRange = (PIOMIOPORTRANGER3)RTAvlroIOPortRangeGet(&pIOM->CTX_SUFF(pTrees)->IOPortTreeR3, Port);
     return pRange;
 }
 
@@ -471,13 +489,37 @@ DECLINLINE(PIOMIOPORTRANGER3) iomIOPortGetRangeHC(PIOM pIOM, RTIOPORT Port)
  */
 DECLINLINE(PIOMMMIORANGE) iomMMIOGetRange(PIOM pIOM, RTGCPHYS GCPhys)
 {
-    PIOMMMIORANGE pRange = CTXALLSUFF(pIOM->pMMIORangeLast);
+#ifdef IN_RING3
+    if (PDMCritSectIsInitialized(&pIOM->EmtLock))
+#endif
+        Assert(IOMIsLockOwner(IOM2VM(pIOM)));
+    PIOMMMIORANGE pRange = pIOM->CTX_SUFF(pMMIORangeLast);
     if (    !pRange
         ||  GCPhys - pRange->GCPhys >= pRange->cb)
-        CTXALLSUFF(pIOM->pMMIORangeLast) = pRange = (PIOMMMIORANGE)RTAvlroGCPhysRangeGet(&pIOM->CTXSUFF(pTrees)->MMIOTree, GCPhys);
+        pIOM->CTX_SUFF(pMMIORangeLast) = pRange = (PIOMMMIORANGE)RTAvlroGCPhysRangeGet(&pIOM->CTX_SUFF(pTrees)->MMIOTree, GCPhys);
     return pRange;
 }
 
+#ifdef VBOX_STRICT
+/**
+ * Gets the MMIO range for the specified physical address in the current context.
+ *
+ * @returns Pointer to MMIO range.
+ * @returns NULL if address not in a MMIO range.
+ *
+ * @param   pIOM    IOM instance data.
+ * @param   GCPhys  Physical address to lookup.
+ */
+DECLINLINE(PIOMMMIORANGE) iomMMIOGetRangeUnsafe(PIOM pIOM, RTGCPHYS GCPhys)
+{
+    PIOMMMIORANGE pRange = pIOM->CTX_SUFF(pMMIORangeLast);
+    if (    !pRange
+        ||  GCPhys - pRange->GCPhys >= pRange->cb)
+        pIOM->CTX_SUFF(pMMIORangeLast) = pRange = (PIOMMMIORANGE)RTAvlroGCPhysRangeGet(&pIOM->CTX_SUFF(pTrees)->MMIOTree, GCPhys);
+    return pRange;
+}
+#endif
+
 
 #ifdef VBOX_WITH_STATISTICS
 /**
@@ -495,15 +537,16 @@ DECLINLINE(PIOMMMIORANGE) iomMMIOGetRange(PIOM pIOM, RTGCPHYS GCPhys)
  */
 DECLINLINE(PIOMMMIOSTATS) iomMMIOGetStats(PIOM pIOM, RTGCPHYS GCPhys, PIOMMMIORANGE pRange)
 {
+    Assert(IOMIsLockOwner(IOM2VM(pIOM)));
     /* For large ranges, we'll put everything on the first byte. */
     if (pRange->cb > PAGE_SIZE)
         GCPhys = pRange->GCPhys;
 
-    PIOMMMIOSTATS pStats = CTXALLSUFF(pIOM->pMMIOStatsLast);
+    PIOMMMIOSTATS pStats = pIOM->CTX_SUFF(pMMIOStatsLast);
     if (    !pStats
         ||  pStats->Core.Key != GCPhys)
     {
-        pStats = (PIOMMMIOSTATS)RTAvloGCPhysGet(&pIOM->CTXSUFF(pTrees)->MMIOStatTree, GCPhys);
+        pStats = (PIOMMMIOSTATS)RTAvloGCPhysGet(&pIOM->CTX_SUFF(pTrees)->MMIOStatTree, GCPhys);
 # ifdef IN_RING3
         if (!pStats)
             pStats = iomR3MMIOStatsCreate(IOM2VM(pIOM), GCPhys, pRange->pszDesc);
@@ -513,11 +556,16 @@ DECLINLINE(PIOMMMIOSTATS) iomMMIOGetStats(PIOM pIOM, RTGCPHYS GCPhys, PIOMMMIORA
 }
 #endif
 
-/* Disassembly helpers used in IOMAll.cpp & IOMAllMMIO.cpp */
-bool iomGetRegImmData(PDISCPUSTATE pCpu, PCOP_PARAMETER pParam, PCPUMCTXCORE pRegFrame, uint32_t *pu32Data, unsigned *pcbSize);
-bool iomSaveDataToReg(PDISCPUSTATE pCpu, PCOP_PARAMETER pParam, PCPUMCTXCORE pRegFrame, unsigned u32Data);
+/* IOM locking helpers. */
+int     iomLock(PVM pVM);
+int     iomTryLock(PVM pVM);
+void    iomUnlock(PVM pVM);
 
-__END_DECLS
+/* Disassembly helpers used in IOMAll.cpp & IOMAllMMIO.cpp */
+bool    iomGetRegImmData(PDISCPUSTATE pCpu, PCOP_PARAMETER pParam, PCPUMCTXCORE pRegFrame, uint64_t *pu64Data, unsigned *pcbSize);
+bool    iomSaveDataToReg(PDISCPUSTATE pCpu, PCOP_PARAMETER pParam, PCPUMCTXCORE pRegFrame, uint64_t u32Data);
+
+RT_C_DECLS_END
 
 
 #ifdef IN_RING3
