@@ -176,7 +176,7 @@ Hypervisor Memory Area (HMA) Layout: Base 00000000a0000000, 0x00800000 bytes
 *   Internal Functions                                                         *
 *******************************************************************************/
 static DECLCALLBACK(int) mmR3Save(PVM pVM, PSSMHANDLE pSSM);
-static DECLCALLBACK(int) mmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version);
+static DECLCALLBACK(int) mmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass);
 
 
 
@@ -201,7 +201,16 @@ VMMR3DECL(int) MMR3InitUVM(PUVM pUVM)
     /*
      * Init the heap.
      */
-    return mmR3HeapCreateU(pUVM, &pUVM->mm.s.pHeap);
+    int rc = mmR3HeapCreateU(pUVM, &pUVM->mm.s.pHeap);
+    if (RT_SUCCESS(rc))
+    {
+        rc = mmR3UkHeapCreateU(pUVM, &pUVM->mm.s.pUkHeap);
+        if (RT_SUCCESS(rc))
+            return VINF_SUCCESS;
+        mmR3HeapDestroy(pUVM->mm.s.pHeap);
+        pUVM->mm.s.pHeap = NULL;
+    }
+    return rc;
 }
 
 
@@ -256,6 +265,7 @@ VMMR3DECL(int) MMR3Init(PVM pVM)
              * Register the saved state data unit.
              */
             rc = SSMR3RegisterInternal(pVM, "mm", 1, MM_SAVED_STATE_VERSION, sizeof(uint32_t) * 2,
+                                       NULL, NULL, NULL,
                                        NULL, mmR3Save, NULL,
                                        NULL, mmR3Load, NULL);
             if (RT_SUCCESS(rc))
@@ -291,18 +301,7 @@ VMMR3DECL(int) MMR3InitPaging(PVM pVM)
         AssertRCReturn(rc, rc);
     }
 
-    /** @cfgm{RamPreAlloc, boolean, false}
-     * Indicates whether the base RAM should all be allocated before starting
-     * the VM (default), or if it should be allocated when first written to.
-     */
-    bool fPreAlloc;
-    rc = CFGMR3QueryBool(CFGMR3GetRoot(pVM), "RamPreAlloc", &fPreAlloc);
-    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
-        fPreAlloc = false;
-    else
-        AssertMsgRCReturn(rc, ("Configuration error: Failed to query integer \"RamPreAlloc\", rc=%Rrc.\n", rc), rc);
-
-    /** @cfgm{RamSize, uint64_t, 0, 0, UINT64_MAX}
+    /** @cfgm{RamSize, uint64_t, 0, 16TB, 0}
      * Specifies the size of the base RAM that is to be set up during
      * VM initialization.
      */
@@ -312,30 +311,49 @@ VMMR3DECL(int) MMR3InitPaging(PVM pVM)
         cbRam = 0;
     else
         AssertMsgRCReturn(rc, ("Configuration error: Failed to query integer \"RamSize\", rc=%Rrc.\n", rc), rc);
-
+    AssertLogRelMsg(!(cbRam & ~X86_PTE_PAE_PG_MASK), ("%RGp X86_PTE_PAE_PG_MASK=%RX64\n", cbRam, X86_PTE_PAE_PG_MASK));
+    AssertLogRelMsgReturn(cbRam <= GMM_GCPHYS_LAST, ("cbRam=%RGp GMM_GCPHYS_LAST=%RX64\n", cbRam, GMM_GCPHYS_LAST), VERR_OUT_OF_RANGE);
     cbRam &= X86_PTE_PAE_PG_MASK;
-    pVM->mm.s.cbRamBase = cbRam;            /* Warning: don't move this code to MMR3Init without fixing REMR3Init.  */
-    Log(("MM: %RU64 bytes of RAM%s\n", cbRam, fPreAlloc ? " (PreAlloc)" : ""));
+    pVM->mm.s.cbRamBase = cbRam;
+
+    /** @cfgm{RamHoleSize, uint32_t, 0, 4032MB, 512MB}
+     * Specifies the size of the memory hole. The memory hole is used
+     * to avoid mapping RAM to the range normally used for PCI memory regions.
+     * Must be aligned on a 4MB boundrary. */
+    uint32_t cbRamHole;
+    rc = CFGMR3QueryU32Def(CFGMR3GetRoot(pVM), "RamHoleSize", &cbRamHole, MM_RAM_HOLE_SIZE_DEFAULT);
+    AssertLogRelMsgRCReturn(rc, ("Configuration error: Failed to query integer \"RamHoleSize\", rc=%Rrc.\n", rc), rc);
+    AssertLogRelMsgReturn(cbRamHole <= 4032U * _1M,
+                          ("Configuration error: \"RamHoleSize\"=%#RX32 is too large.\n", cbRamHole), VERR_OUT_OF_RANGE);
+    AssertLogRelMsgReturn(cbRamHole > 16 * _1M,
+                          ("Configuration error: \"RamHoleSize\"=%#RX32 is too large.\n", cbRamHole), VERR_OUT_OF_RANGE);
+    AssertLogRelMsgReturn(!(cbRamHole & (_4M - 1)),
+                          ("Configuration error: \"RamHoleSize\"=%#RX32 is misaligned.\n", cbRamHole), VERR_OUT_OF_RANGE);
+    uint64_t const offRamHole = _4G - cbRamHole;
+    if (cbRam < offRamHole)
+        Log(("MM: %RU64 bytes of RAM\n", cbRam));
+    else
+        Log(("MM: %RU64 bytes of RAM with a hole at %RU64 up to 4GB.\n", cbRam, offRamHole));
 
     /** @cfgm{MM/Policy, string, no overcommitment}
      * Specifies the policy to use when reserving memory for this VM. The recognized
      * value is 'no overcommitment' (default). See GMMPOLICY.
      */
-    GMMOCPOLICY enmPolicy;
+    GMMOCPOLICY enmOcPolicy;
     char sz[64];
     rc = CFGMR3QueryString(CFGMR3GetRoot(pVM), "Policy", sz, sizeof(sz));
     if (RT_SUCCESS(rc))
     {
         if (    !RTStrICmp(sz, "no_oc")
             ||  !RTStrICmp(sz, "no overcommitment"))
-            enmPolicy = GMMOCPOLICY_NO_OC;
+            enmOcPolicy = GMMOCPOLICY_NO_OC;
         else
             return VMSetError(pVM, VERR_INVALID_PARAMETER, RT_SRC_POS, "Unknown \"MM/Policy\" value \"%s\"", sz);
     }
     else if (rc == VERR_CFGM_VALUE_NOT_FOUND)
-        enmPolicy = GMMOCPOLICY_NO_OC;
+        enmOcPolicy = GMMOCPOLICY_NO_OC;
     else
-        AssertMsgRCReturn(rc, ("Configuration error: Failed to query string \"MM/Policy\", rc=%Rrc.\n", rc), rc);
+        AssertMsgFailedReturn(("Configuration error: Failed to query string \"MM/Policy\", rc=%Rrc.\n", rc), rc);
 
     /** @cfgm{MM/Priority, string, normal}
      * Specifies the memory priority of this VM. The priority comes into play when the
@@ -358,20 +376,26 @@ VMMR3DECL(int) MMR3InitPaging(PVM pVM)
     else if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         enmPriority = GMMPRIORITY_NORMAL;
     else
-        AssertMsgRCReturn(rc, ("Configuration error: Failed to query string \"MM/Priority\", rc=%Rrc.\n", rc), rc);
+        AssertMsgFailedReturn(("Configuration error: Failed to query string \"MM/Priority\", rc=%Rrc.\n", rc), rc);
 
     /*
      * Make the initial memory reservation with GMM.
      */
-    rc = GMMR3InitialReservation(pVM, cbRam >> PAGE_SHIFT, 1, 1, enmPolicy, enmPriority);
+    uint64_t cBasePages = (cbRam >> PAGE_SHIFT) + pVM->mm.s.cBasePages;
+    rc = GMMR3InitialReservation(pVM,
+                                 RT_MAX(cBasePages + pVM->mm.s.cHandyPages, 1),
+                                 RT_MAX(pVM->mm.s.cShadowPages, 1),
+                                 RT_MAX(pVM->mm.s.cFixedPages, 1),
+                                 enmOcPolicy,
+                                 enmPriority);
     if (RT_FAILURE(rc))
     {
         if (rc == VERR_GMM_MEMORY_RESERVATION_DECLINED)
             return VMSetError(pVM, rc, RT_SRC_POS,
-                              N_("Insufficient free memory to start the VM (cbRam=%#RX64 enmPolicy=%d enmPriority=%d)"),
-                              cbRam, enmPolicy, enmPriority);
+                              N_("Insufficient free memory to start the VM (cbRam=%#RX64 enmOcPolicy=%d enmPriority=%d)"),
+                              cbRam, enmOcPolicy, enmPriority);
         return VMSetError(pVM, rc, RT_SRC_POS, "GMMR3InitialReservation(,%#RX64,0,0,%d,%d)",
-                          cbRam >> PAGE_SHIFT, enmPolicy, enmPriority);
+                          cbRam >> PAGE_SHIFT, enmOcPolicy, enmPriority);
     }
 
     /*
@@ -386,29 +410,21 @@ VMMR3DECL(int) MMR3InitPaging(PVM pVM)
     /*
      * Setup the base ram (PGM).
      */
-    rc = PGMR3PhysRegisterRam(pVM, 0, cbRam, "Base RAM");
-#ifdef VBOX_WITH_NEW_PHYS_CODE
-    if (RT_SUCCESS(rc) && fPreAlloc)
+    if (cbRam > offRamHole)
     {
-        /** @todo RamPreAlloc should be handled at the very end of the VM creation. (lazy bird) */
-        return VM_SET_ERROR(pVM, VERR_NOT_IMPLEMENTED, "TODO: RamPreAlloc");
+        rc = PGMR3PhysRegisterRam(pVM, 0, offRamHole, "Base RAM");
+        if (RT_SUCCESS(rc))
+            rc = PGMR3PhysRegisterRam(pVM, _4G, cbRam - offRamHole, "Above 4GB Base RAM");
     }
-#else
-    if (RT_SUCCESS(rc))
-    {
-        /*
-         * Allocate the first chunk, as we'll map ROM ranges there.
-         * If requested, allocated the rest too.
-         */
-        RTGCPHYS GCPhys = (RTGCPHYS)0;
-        rc = PGM3PhysGrowRange(pVM, &GCPhys);
-        if (RT_SUCCESS(rc) && fPreAlloc)
-            for (GCPhys = PGM_DYNAMIC_CHUNK_SIZE;
-                 GCPhys < cbRam && RT_SUCCESS(rc);
-                 GCPhys += PGM_DYNAMIC_CHUNK_SIZE)
-                rc = PGM3PhysGrowRange(pVM, &GCPhys);
-    }
-#endif
+    else
+        rc = PGMR3PhysRegisterRam(pVM, 0, RT_MIN(cbRam, offRamHole), "Base RAM");
+
+    /*
+     * Enabled mmR3UpdateReservation here since we don't want the
+     * PGMR3PhysRegisterRam calls above mess things up.
+     */
+    pVM->mm.s.fDoneMMR3InitPaging = true;
+    AssertMsg(pVM->mm.s.cBasePages == cBasePages || RT_FAILURE(rc), ("%RX64 != %RX64\n", pVM->mm.s.cBasePages, cBasePages));
 
     LogFlow(("MMR3InitPaging: returns %Rrc\n", rc));
     return rc;
@@ -431,41 +447,25 @@ VMMR3DECL(int) MMR3Term(PVM pVM)
      */
     mmR3PagePoolTerm(pVM);
 
-    /*
-     * Release locked memory.
-     * (Associated record are released by the heap.)
-     */
-    PMMLOCKEDMEM pLockedMem = pVM->mm.s.pLockedMem;
-    while (pLockedMem)
-    {
-        int rc = SUPPageUnlock(pLockedMem->pv);
-        AssertMsgRC(rc, ("SUPPageUnlock(%p) -> rc=%d\n", pLockedMem->pv, rc));
-        switch (pLockedMem->eType)
-        {
-            case MM_LOCKED_TYPE_HYPER:
-                rc = SUPPageFree(pLockedMem->pv, pLockedMem->cb >> PAGE_SHIFT);
-                AssertMsgRC(rc, ("SUPPageFree(%p) -> rc=%d\n", pLockedMem->pv, rc));
-                break;
-            case MM_LOCKED_TYPE_HYPER_NOFREE:
-            case MM_LOCKED_TYPE_HYPER_PAGES:
-            case MM_LOCKED_TYPE_PHYS:
-                /* nothing to do. */
-                break;
-        }
-        /* next */
-        pLockedMem = pLockedMem->pNext;
-    }
+    /* Clean up the hypervisor heap. */
+    mmR3HyperTerm(pVM);
 
     /*
      * Zero stuff to detect after termination use of the MM interface
      */
     pVM->mm.s.offLookupHyper = NIL_OFFSET;
-    pVM->mm.s.pLockedMem     = NULL;
     pVM->mm.s.pHyperHeapR3   = NULL;        /* freed above. */
     pVM->mm.s.pHyperHeapR0   = NIL_RTR0PTR; /* freed above. */
     pVM->mm.s.pHyperHeapRC   = NIL_RTRCPTR; /* freed above. */
     pVM->mm.s.offVM          = 0;           /* init assertion on this */
 
+    /*
+     * Destroy the User-kernel heap here since the support driver session
+     * may have been terminated by the time we get to MMR3TermUVM.
+     */
+    mmR3UkHeapDestroy(pVM->pUVM->mm.s.pUkHeap);
+    pVM->pUVM->mm.s.pUkHeap = NULL;
+
     return VINF_SUCCESS;
 }
 
@@ -482,8 +482,13 @@ VMMR3DECL(int) MMR3Term(PVM pVM)
 VMMR3DECL(void) MMR3TermUVM(PUVM pUVM)
 {
     /*
-     * Destroy the heap.
+     * Destroy the heaps.
      */
+    if (pUVM->mm.s.pUkHeap)
+    {
+        mmR3UkHeapDestroy(pUVM->mm.s.pUkHeap);
+        pUVM->mm.s.pUkHeap = NULL;
+    }
     mmR3HeapDestroy(pUVM->mm.s.pHeap);
     pUVM->mm.s.pHeap = NULL;
 }
@@ -492,14 +497,11 @@ VMMR3DECL(void) MMR3TermUVM(PUVM pUVM)
 /**
  * Reset notification.
  *
- * MM will reload shadow ROMs into RAM at this point and make
- * the ROM writable.
- *
  * @param   pVM             The VM handle.
  */
 VMMR3DECL(void) MMR3Reset(PVM pVM)
 {
-    mmR3PhysRomReset(pVM);
+    /* nothing to do anylonger. */
 }
 
 
@@ -526,19 +528,21 @@ static DECLCALLBACK(int) mmR3Save(PVM pVM, PSSMHANDLE pSSM)
  * @returns VBox status code.
  * @param   pVM             VM Handle.
  * @param   pSSM            SSM operation handle.
- * @param   u32Version      Data layout version.
+ * @param   uVersion       Data layout version.
+ * @param   uPass           The data pass.
  */
-static DECLCALLBACK(int) mmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
+static DECLCALLBACK(int) mmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
 {
     LogFlow(("mmR3Load:\n"));
+    Assert(uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     /*
      * Validate version.
      */
-    if (    SSM_VERSION_MAJOR_CHANGED(u32Version, MM_SAVED_STATE_VERSION)
-        ||  !u32Version)
+    if (    SSM_VERSION_MAJOR_CHANGED(uVersion, MM_SAVED_STATE_VERSION)
+        ||  !uVersion)
     {
-        AssertMsgFailed(("mmR3Load: Invalid version u32Version=%d!\n", u32Version));
+        AssertMsgFailed(("mmR3Load: Invalid version uVersion=%d!\n", uVersion));
         return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
     }
 
@@ -548,9 +552,9 @@ static DECLCALLBACK(int) mmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
     int rc;
     RTUINT cb1;
 
-    /* cBasePages */
+    /* cBasePages (ignored) */
     uint64_t cPages;
-    if (u32Version != 1)
+    if (uVersion >= 2)
         rc = SSMR3GetU64(pSSM, &cPages);
     else
     {
@@ -559,15 +563,10 @@ static DECLCALLBACK(int) mmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
     }
     if (RT_FAILURE(rc))
         return rc;
-    if (cPages != pVM->mm.s.cBasePages)
-    {
-        LogRel(("mmR3Load: Memory configuration has changed. cPages=%#RX64 saved=%#RX64\n", pVM->mm.s.cBasePages, cPages));
-        return VERR_SSM_LOAD_MEMORY_SIZE_MISMATCH;
-    }
 
     /* cbRamBase */
     uint64_t cb;
-    if (u32Version != 1)
+    if (uVersion != 1)
         rc = SSMR3GetU64(pSSM, &cb);
     else
     {
@@ -576,11 +575,9 @@ static DECLCALLBACK(int) mmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
     }
     if (RT_FAILURE(rc))
         return rc;
-    if (cb != pVM->mm.s.cbRamBase)
-    {
-        LogRel(("mmR3Load: Memory configuration has changed. cbRamBase=%#RX64 save=%#RX64\n", pVM->mm.s.cbRamBase, cb));
-        return VERR_SSM_LOAD_MEMORY_SIZE_MISMATCH;
-    }
+    AssertLogRelMsgReturn(cb == pVM->mm.s.cbRamBase,
+                          ("Memory configuration has changed. cbRamBase=%#RX64 save=%#RX64\n", pVM->mm.s.cbRamBase, cb),
+                          VERR_SSM_LOAD_MEMORY_SIZE_MISMATCH);
 
     /* (PGM restores the physical memory.) */
     return rc;
@@ -600,7 +597,7 @@ int mmR3UpdateReservation(PVM pVM)
     VM_ASSERT_EMT(pVM);
     if (pVM->mm.s.fDoneMMR3InitPaging)
         return GMMR3UpdateReservation(pVM,
-                                      RT_MAX(pVM->mm.s.cBasePages, 1),
+                                      RT_MAX(pVM->mm.s.cBasePages + pVM->mm.s.cHandyPages, 1),
                                       RT_MAX(pVM->mm.s.cShadowPages, 1),
                                       RT_MAX(pVM->mm.s.cFixedPages, 1));
     return VINF_SUCCESS;
@@ -624,13 +621,41 @@ VMMR3DECL(int) MMR3IncreaseBaseReservation(PVM pVM, uint64_t cAddBasePages)
     int rc = mmR3UpdateReservation(pVM);
     if (RT_FAILURE(rc))
     {
-        VMSetError(pVM, rc, RT_SRC_POS, N_("Failed to reserved physical memory for the RAM (%#RX64 -> %#RX64)"), cOld, pVM->mm.s.cBasePages);
+        VMSetError(pVM, rc, RT_SRC_POS, N_("Failed to reserved physical memory for the RAM (%#RX64 -> %#RX64 + %#RX32)"),
+                   cOld, pVM->mm.s.cBasePages, pVM->mm.s.cHandyPages);
         pVM->mm.s.cBasePages = cOld;
     }
     return rc;
 }
 
 
+/**
+ * Interface for PGM to make reservations for handy pages in addition to the
+ * base memory.
+ *
+ * This can be called before MMR3InitPaging.
+ *
+ * @returns VBox status code. Will set VM error on failure.
+ * @param   pVM             The shared VM structure.
+ * @param   cHandyPages     The number of handy pages.
+ */
+VMMR3DECL(int) MMR3ReserveHandyPages(PVM pVM, uint32_t cHandyPages)
+{
+    AssertReturn(!pVM->mm.s.cHandyPages, VERR_WRONG_ORDER);
+
+    pVM->mm.s.cHandyPages = cHandyPages;
+    LogFlow(("MMR3ReserveHandyPages: %RU32 (base %RU64)\n", pVM->mm.s.cHandyPages, pVM->mm.s.cBasePages));
+    int rc = mmR3UpdateReservation(pVM);
+    if (RT_FAILURE(rc))
+    {
+        VMSetError(pVM, rc, RT_SRC_POS, N_("Failed to reserved physical memory for the RAM (%#RX64 + %#RX32)"),
+                   pVM->mm.s.cBasePages, pVM->mm.s.cHandyPages);
+        pVM->mm.s.cHandyPages = 0;
+    }
+    return rc;
+}
+
+
 /**
  * Interface for PGM to adjust the reservation of fixed pages.
  *
@@ -681,128 +706,6 @@ VMMR3DECL(int) MMR3UpdateShadowReservation(PVM pVM, uint32_t cShadowPages)
 }
 
 
-/**
- * Locks physical memory which backs a virtual memory range (HC) adding
- * the required records to the pLockedMem list.
- *
- * @returns VBox status code.
- * @param   pVM             The VM handle.
- * @param   pv              Pointer to memory range which shall be locked down.
- *                          This pointer is page aligned.
- * @param   cb              Size of memory range (in bytes). This size is page aligned.
- * @param   eType           Memory type.
- * @param   ppLockedMem     Where to store the pointer to the created locked memory record.
- *                          This is optional, pass NULL if not used.
- * @param   fSilentFailure  Don't raise an error when unsuccessful. Upper layer with deal with it.
- */
-int mmR3LockMem(PVM pVM, void *pv, size_t cb, MMLOCKEDTYPE eType, PMMLOCKEDMEM *ppLockedMem, bool fSilentFailure)
-{
-    Assert(RT_ALIGN_P(pv, PAGE_SIZE) == pv);
-    Assert(RT_ALIGN_Z(cb, PAGE_SIZE) == cb);
-
-    if (ppLockedMem)
-        *ppLockedMem = NULL;
-
-    /*
-     * Allocate locked mem structure.
-     */
-    unsigned        cPages = (unsigned)(cb >> PAGE_SHIFT);
-    AssertReturn(cPages == (cb >> PAGE_SHIFT), VERR_OUT_OF_RANGE);
-    PMMLOCKEDMEM    pLockedMem = (PMMLOCKEDMEM)MMR3HeapAlloc(pVM, MM_TAG_MM, RT_OFFSETOF(MMLOCKEDMEM, aPhysPages[cPages]));
-    if (!pLockedMem)
-        return VERR_NO_MEMORY;
-    pLockedMem->pv      = pv;
-    pLockedMem->cb      = cb;
-    pLockedMem->eType   = eType;
-    memset(&pLockedMem->u, 0, sizeof(pLockedMem->u));
-
-    /*
-     * Lock the memory.
-     */
-    int rc = SUPPageLock(pv, cPages, &pLockedMem->aPhysPages[0]);
-    if (RT_SUCCESS(rc))
-    {
-        /*
-         * Setup the reserved field.
-         */
-        PSUPPAGE    pPhysPage = &pLockedMem->aPhysPages[0];
-        for (unsigned c = cPages; c > 0; c--, pPhysPage++)
-            pPhysPage->uReserved = (RTHCUINTPTR)pLockedMem;
-
-        /*
-         * Insert into the list.
-         *
-         * ASSUME no protected needed here as only one thread in the system can possibly
-         * be doing this. No other threads will walk this list either we assume.
-         */
-        pLockedMem->pNext = pVM->mm.s.pLockedMem;
-        pVM->mm.s.pLockedMem = pLockedMem;
-        /* Set return value. */
-        if (ppLockedMem)
-            *ppLockedMem = pLockedMem;
-    }
-    else
-    {
-        AssertMsgFailed(("SUPPageLock failed with rc=%d\n", rc));
-        MMR3HeapFree(pLockedMem);
-        if (!fSilentFailure)
-            rc = VMSetError(pVM, rc, RT_SRC_POS, N_("Failed to lock %d bytes of host memory (out of memory)"), cb);
-    }
-
-    return rc;
-}
-
-
-/**
- * Maps a part of or an entire locked memory region into the guest context.
- *
- * @returns VBox status.
- *          God knows what happens if we fail...
- * @param   pVM         VM handle.
- * @param   pLockedMem  Locked memory structure.
- * @param   Addr        GC Address where to start the mapping.
- * @param   iPage       Page number in the locked memory region.
- * @param   cPages      Number of pages to map.
- * @param   fFlags      See the fFlags argument of PGR3Map().
- */
-int mmR3MapLocked(PVM pVM, PMMLOCKEDMEM pLockedMem, RTGCPTR Addr, unsigned iPage, size_t cPages, unsigned fFlags)
-{
-    /*
-     * Adjust ~0 argument
-     */
-    if (cPages == ~(size_t)0)
-        cPages = (pLockedMem->cb >> PAGE_SHIFT) - iPage;
-    Assert(cPages != ~0U);
-    /* no incorrect arguments are accepted */
-    Assert(RT_ALIGN_GCPT(Addr, PAGE_SIZE, RTGCPTR) == Addr);
-    AssertMsg(iPage < (pLockedMem->cb >> PAGE_SHIFT), ("never even think about giving me a bad iPage(=%d)\n", iPage));
-    AssertMsg(iPage + cPages <= (pLockedMem->cb >> PAGE_SHIFT), ("never even think about giving me a bad cPages(=%d)\n", cPages));
-
-    /*
-     * Map the pages.
-     */
-    PSUPPAGE    pPhysPage = &pLockedMem->aPhysPages[iPage];
-    while (cPages)
-    {
-        RTHCPHYS HCPhys = pPhysPage->Phys;
-        int rc = PGMMap(pVM, Addr, HCPhys, PAGE_SIZE, fFlags);
-        if (RT_FAILURE(rc))
-        {
-            /** @todo how the hell can we do a proper bailout here. */
-            return rc;
-        }
-
-        /* next */
-        cPages--;
-        iPage++;
-        pPhysPage++;
-        Addr += PAGE_SIZE;
-    }
-
-    return VINF_SUCCESS;
-}
-
-
 /**
  * Convert HC Physical address to HC Virtual address.
  *
@@ -826,61 +729,69 @@ VMMR3DECL(int) MMR3HCPhys2HCVirt(PVM pVM, RTHCPHYS HCPhys, void **ppv)
         return rc;
 
     /*
-     * Iterate the locked memory - very slow.
+     * Iterate thru the lookup records for HMA.
      */
     uint32_t off = HCPhys & PAGE_OFFSET_MASK;
     HCPhys &= X86_PTE_PAE_PG_MASK;
-    for (PMMLOCKEDMEM pCur = pVM->mm.s.pLockedMem; pCur; pCur = pCur->pNext)
+    PMMLOOKUPHYPER pCur = (PMMLOOKUPHYPER)((uint8_t *)pVM->mm.s.CTX_SUFF(pHyperHeap) + pVM->mm.s.offLookupHyper);
+    for (;;)
     {
-        size_t iPage = pCur->cb >> PAGE_SHIFT;
-        while (iPage-- > 0)
-            if ((pCur->aPhysPages[iPage].Phys & X86_PTE_PAE_PG_MASK) == HCPhys)
+        switch (pCur->enmType)
+        {
+            case MMLOOKUPHYPERTYPE_LOCKED:
             {
-                *ppv = (char *)pCur->pv + (iPage << PAGE_SHIFT) + off;
-                return VINF_SUCCESS;
+                PCRTHCPHYS  paHCPhysPages = pCur->u.Locked.paHCPhysPages;
+                size_t      iPage         = pCur->cb >> PAGE_SHIFT;
+                while (iPage-- > 0)
+                    if (paHCPhysPages[iPage] == HCPhys)
+                    {
+                        *ppv = (char *)pCur->u.Locked.pvR3 + (iPage << PAGE_SHIFT) + off;
+                        return VINF_SUCCESS;
+                    }
+                break;
             }
+
+            case MMLOOKUPHYPERTYPE_HCPHYS:
+                if (pCur->u.HCPhys.HCPhys - HCPhys < pCur->cb)
+                {
+                    *ppv = (uint8_t *)pCur->u.HCPhys.pvR3 + pCur->u.HCPhys.HCPhys - HCPhys + off;
+                    return VINF_SUCCESS;
+                }
+                break;
+
+            case MMLOOKUPHYPERTYPE_GCPHYS:  /* (for now we'll not allow these kind of conversions) */
+            case MMLOOKUPHYPERTYPE_MMIO2:
+            case MMLOOKUPHYPERTYPE_DYNAMIC:
+                break;
+
+            default:
+                AssertMsgFailed(("enmType=%d\n", pCur->enmType));
+                break;
+        }
+
+        /* next */
+        if (pCur->offNext ==  (int32_t)NIL_OFFSET)
+            break;
+        pCur = (PMMLOOKUPHYPER)((uint8_t *)pCur + pCur->offNext);
     }
     /* give up */
     return VERR_INVALID_POINTER;
 }
 
 
-/**
- * Read memory from GC virtual address using the current guest CR3.
- *
- * @returns VBox status.
- * @param   pVM         VM handle.
- * @param   pvDst       Destination address (HC of course).
- * @param   GCPtr       GC virtual address.
- * @param   cb          Number of bytes to read.
- *
- * @remarks Intended for the debugger facility only.
- * @todo    Move to DBGF, it's only selecting which functions to use!
- */
-VMMR3DECL(int) MMR3ReadGCVirt(PVM pVM, void *pvDst, RTGCPTR GCPtr, size_t cb)
-{
-    if (GCPtr - pVM->mm.s.pvHyperAreaGC < pVM->mm.s.cbHyperArea)
-        return MMR3HyperReadGCVirt(pVM, pvDst, GCPtr, cb);
-    return PGMPhysSimpleReadGCPtr(pVM, pvDst, GCPtr, cb);
-}
-
 
 /**
- * Write to memory at GC virtual address translated using the current guest CR3.
+ * Get the size of the base RAM.
+ * This usually means the size of the first contigous block of physical memory.
  *
- * @returns VBox status.
- * @param   pVM         VM handle.
- * @param   GCPtrDst    GC virtual address.
- * @param   pvSrc       The source address (HC of course).
- * @param   cb          Number of bytes to read.
+ * @returns The guest base RAM size.
+ * @param   pVM         The VM handle.
+ * @thread  Any.
  *
- * @remarks Intended for the debugger facility only.
- * @todo    Move to DBGF, it's only selecting which functions to use!
+ * @deprecated
  */
-VMMR3DECL(int) MMR3WriteGCVirt(PVM pVM, RTGCPTR GCPtrDst, const void *pvSrc, size_t cb)
+VMMR3DECL(uint64_t) MMR3PhysGetRamSize(PVM pVM)
 {
-    if (GCPtrDst - pVM->mm.s.pvHyperAreaGC < pVM->mm.s.cbHyperArea)
-        return VERR_ACCESS_DENIED;
-    return PGMPhysSimpleWriteGCPtr(pVM, GCPtrDst, pvSrc, cb);
+    return pVM->mm.s.cbRamBase;
 }
 
