@@ -339,28 +339,14 @@ void *mmR3HeapAlloc(PMMHEAP pHeap, MMTAG enmTag, size_t cbSize, bool fZero)
 
         /* register the statistics */
         PUVM pUVM = pHeap->pUVM;
-        char szName[80];
-        const char *pszTag = mmR3GetTagName(enmTag);
-        RTStrPrintf(szName, sizeof(szName), "/MM/R3Heap/%s/cAllocations", pszTag);
-        STAMR3RegisterU(pUVM, &pStat->cAllocations,   STAMTYPE_U64, STAMVISIBILITY_ALWAYS, szName,  STAMUNIT_CALLS, "Number or MMR3HeapAlloc() calls.");
-
-        RTStrPrintf(szName, sizeof(szName), "/MM/R3Heap/%s/cReallocations", pszTag);
-        STAMR3RegisterU(pUVM, &pStat->cReallocations, STAMTYPE_U64, STAMVISIBILITY_ALWAYS, szName,  STAMUNIT_CALLS, "Number of MMR3HeapRealloc() calls.");
-
-        RTStrPrintf(szName, sizeof(szName), "/MM/R3Heap/%s/cFrees", pszTag);
-        STAMR3RegisterU(pUVM, &pStat->cFrees,         STAMTYPE_U64, STAMVISIBILITY_ALWAYS, szName,  STAMUNIT_CALLS, "Number of MMR3HeapFree() calls.");
-
-        RTStrPrintf(szName, sizeof(szName), "/MM/R3Heap/%s/cFailures", pszTag);
-        STAMR3RegisterU(pUVM, &pStat->cFailures,      STAMTYPE_U64, STAMVISIBILITY_ALWAYS, szName,  STAMUNIT_COUNT, "Number of failures.");
-
-        RTStrPrintf(szName, sizeof(szName), "/MM/R3Heap/%s/cbCurAllocated", pszTag);
-        STAMR3RegisterU(pUVM, &pStat->cbCurAllocated, STAMTYPE_U32, STAMVISIBILITY_ALWAYS, szName,  STAMUNIT_BYTES, "Number of bytes currently allocated.");
-
-        RTStrPrintf(szName, sizeof(szName), "/MM/R3Heap/%s/cbAllocated", pszTag);
-        STAMR3RegisterU(pUVM, &pStat->cbAllocated,    STAMTYPE_U64, STAMVISIBILITY_ALWAYS, szName,  STAMUNIT_BYTES, "Total number of bytes allocated.");
-
-        RTStrPrintf(szName, sizeof(szName), "/MM/R3Heap/%s/cbFreed", pszTag);
-        STAMR3RegisterU(pUVM, &pStat->cbFreed,        STAMTYPE_U64, STAMVISIBILITY_ALWAYS, szName,  STAMUNIT_BYTES, "Total number of bytes freed.");
+        const char *pszTag = mmGetTagName(enmTag);
+        STAMR3RegisterFU(pUVM, &pStat->cbCurAllocated, STAMTYPE_U32, STAMVISIBILITY_ALWAYS,  STAMUNIT_BYTES, "Number of bytes currently allocated.",    "/MM/R3Heap/%s", pszTag);
+        STAMR3RegisterFU(pUVM, &pStat->cAllocations,   STAMTYPE_U64, STAMVISIBILITY_ALWAYS,  STAMUNIT_CALLS, "Number or MMR3HeapAlloc() calls.",        "/MM/R3Heap/%s/cAllocations", pszTag);
+        STAMR3RegisterFU(pUVM, &pStat->cReallocations, STAMTYPE_U64, STAMVISIBILITY_ALWAYS,  STAMUNIT_CALLS, "Number of MMR3HeapRealloc() calls.",      "/MM/R3Heap/%s/cReallocations", pszTag);
+        STAMR3RegisterFU(pUVM, &pStat->cFrees,         STAMTYPE_U64, STAMVISIBILITY_ALWAYS,  STAMUNIT_CALLS, "Number of MMR3HeapFree() calls.",         "/MM/R3Heap/%s/cFrees", pszTag);
+        STAMR3RegisterFU(pUVM, &pStat->cFailures,      STAMTYPE_U64, STAMVISIBILITY_ALWAYS,  STAMUNIT_COUNT, "Number of failures.",                     "/MM/R3Heap/%s/cFailures", pszTag);
+        STAMR3RegisterFU(pUVM, &pStat->cbAllocated,    STAMTYPE_U64, STAMVISIBILITY_ALWAYS,  STAMUNIT_BYTES, "Total number of bytes allocated.",        "/MM/R3Heap/%s/cbAllocated", pszTag);
+        STAMR3RegisterFU(pUVM, &pStat->cbFreed,        STAMTYPE_U64, STAMVISIBILITY_ALWAYS,  STAMUNIT_BYTES, "Total number of bytes freed.",            "/MM/R3Heap/%s/cbFreed", pszTag);
     }
 #endif
 
@@ -711,98 +697,3 @@ VMMR3DECL(void) MMR3HeapFree(void *pv)
 }
 
 
-/**
- * Gets the string name of a memory tag.
- *
- * @returns name of enmTag.
- * @param   enmTag      The tag.
- */
-const char *mmR3GetTagName(MMTAG enmTag)
-{
-    switch (enmTag)
-    {
-        #define TAG2STR(tag) case MM_TAG_##tag: return #tag
-
-        TAG2STR(CFGM);
-        TAG2STR(CFGM_BYTES);
-        TAG2STR(CFGM_STRING);
-        TAG2STR(CFGM_USER);
-
-        TAG2STR(CSAM);
-        TAG2STR(CSAM_PATCH);
-
-        TAG2STR(DBGF);
-        TAG2STR(DBGF_INFO);
-        TAG2STR(DBGF_LINE);
-        TAG2STR(DBGF_LINE_DUP);
-        TAG2STR(DBGF_MODULE);
-        TAG2STR(DBGF_OS);
-        TAG2STR(DBGF_STACK);
-        TAG2STR(DBGF_SYMBOL);
-        TAG2STR(DBGF_SYMBOL_DUP);
-
-        TAG2STR(EM);
-
-        TAG2STR(IOM);
-        TAG2STR(IOM_STATS);
-
-        TAG2STR(MM);
-        TAG2STR(MM_LOOKUP_GUEST);
-        TAG2STR(MM_LOOKUP_PHYS);
-        TAG2STR(MM_LOOKUP_VIRT);
-        TAG2STR(MM_PAGE);
-
-        TAG2STR(PARAV);
-
-        TAG2STR(PATM);
-        TAG2STR(PATM_PATCH);
-
-        TAG2STR(PDM);
-        TAG2STR(PDM_DEVICE);
-        TAG2STR(PDM_DEVICE_USER);
-        TAG2STR(PDM_DRIVER);
-        TAG2STR(PDM_DRIVER_USER);
-        TAG2STR(PDM_USB);
-        TAG2STR(PDM_USB_USER);
-        TAG2STR(PDM_LUN);
-        TAG2STR(PDM_QUEUE);
-        TAG2STR(PDM_THREAD);
-        TAG2STR(PDM_ASYNC_COMPLETION);
-
-        TAG2STR(PGM);
-        TAG2STR(PGM_CHUNK_MAPPING);
-        TAG2STR(PGM_HANDLERS);
-        TAG2STR(PGM_PHYS);
-        TAG2STR(PGM_POOL);
-
-        TAG2STR(REM);
-
-        TAG2STR(SELM);
-
-        TAG2STR(SSM);
-
-        TAG2STR(STAM);
-
-        TAG2STR(TM);
-
-        TAG2STR(TRPM);
-
-        TAG2STR(VM);
-        TAG2STR(VM_REQ);
-
-        TAG2STR(VMM);
-
-        TAG2STR(HWACCM);
-
-        #undef TAG2STR
-
-        default:
-        {
-            AssertMsgFailed(("Unknown tag %d! forgot to add it to the switch?\n", enmTag));
-            static char sz[48];
-            RTStrPrintf(sz, sizeof(sz), "%d", enmTag);
-            return sz;
-        }
-    }
-}
-
