@@ -73,19 +73,23 @@ int mmR3HyperInit(PVM pVM)
 
     /** @todo @bugref{1865}, @bugref{3202}: Change the cbHyperHeap default
      *        depending on whether VT-x/AMD-V is enabled or not! Don't waste
-     *        precious kernel space on heap for the PATM. */
+     *        precious kernel space on heap for the PATM.
+     */
     uint32_t cbHyperHeap;
     int rc = CFGMR3QueryU32(CFGMR3GetChild(CFGMR3GetRoot(pVM), "MM"), "cbHyperHeap", &cbHyperHeap);
     if (rc == VERR_CFGM_NO_PARENT || rc == VERR_CFGM_VALUE_NOT_FOUND)
-        cbHyperHeap = VMMIsHwVirtExtForced(pVM)
-                    ? 640*_1K
-                    : 1280*_1K;
-    else if (RT_FAILURE(rc))
     {
-        LogRel(("MM/cbHyperHeap query -> %Rrc\n", rc));
-        AssertRCReturn(rc, rc);
+        if (pVM->cCpus > 1)
+            cbHyperHeap = _2M + pVM->cCpus * _64K;
+        else
+            cbHyperHeap = VMMIsHwVirtExtForced(pVM)
+                        ? 640*_1K
+                        : 1280*_1K;
     }
+    else
+        AssertLogRelRCReturn(rc, rc);
     cbHyperHeap = RT_ALIGN_32(cbHyperHeap, PAGE_SIZE);
+    LogRel(("MM: cbHyperHeap=%#x (%u)\n", cbHyperHeap, cbHyperHeap));
 
     /*
      * Allocate the hypervisor heap.
@@ -104,13 +108,13 @@ int mmR3HyperInit(PVM pVM)
         /*
          * Map the VM structure into the hypervisor space.
          */
-        AssertRelease(pVM->cbSelf == RT_UOFFSETOF(VM, aCpus[pVM->cCPUs]));
+        AssertRelease(pVM->cbSelf == RT_UOFFSETOF(VM, aCpus[pVM->cCpus]));
         RTGCPTR GCPtr;
         rc = MMR3HyperMapPages(pVM, pVM, pVM->pVMR0, RT_ALIGN_Z(pVM->cbSelf, PAGE_SIZE) >> PAGE_SHIFT, pVM->paVMPagesR3, "VM", &GCPtr);
         if (RT_SUCCESS(rc))
         {
             pVM->pVMRC = (RTRCPTR)GCPtr;
-            for (uint32_t i = 0; i < pVM->cCPUs; i++)
+            for (VMCPUID i = 0; i < pVM->cCpus; i++)
                 pVM->aCpus[i].pVMRC = pVM->pVMRC;
 
             /* Reserve a page for fencing. */
@@ -142,6 +146,20 @@ int mmR3HyperInit(PVM pVM)
 }
 
 
+/**
+ * Cleans up the hypervisor heap.
+ *
+ * @returns VBox status.
+ */
+int mmR3HyperTerm(PVM pVM)
+{
+    if (pVM->mm.s.pHyperHeapR3)
+        PDMR3CritSectDelete(&pVM->mm.s.pHyperHeapR3->Lock);
+
+    return VINF_SUCCESS;
+}
+
+
 /**
  * Finalizes the HMA mapping.
  *
@@ -154,13 +172,19 @@ VMMR3DECL(int) MMR3HyperInitFinalize(PVM pVM)
 {
     LogFlow(("MMR3HyperInitFinalize:\n"));
 
+    /*
+     * Initialize the hyper heap critical section.
+     */
+    int rc = PDMR3CritSectInit(pVM, &pVM->mm.s.pHyperHeapR3->Lock, "MM-HYPER");
+    AssertRC(rc);
+
     /*
      * Adjust and create the HMA mapping.
      */
     while ((RTINT)pVM->mm.s.offHyperNextStatic + 64*_1K < (RTINT)pVM->mm.s.cbHyperArea - _4M)
         pVM->mm.s.cbHyperArea -= _4M;
-    int rc = PGMR3MapPT(pVM, pVM->mm.s.pvHyperAreaGC, pVM->mm.s.cbHyperArea,
-                        mmR3HyperRelocateCallback, NULL, "Hypervisor Memory Area");
+    rc = PGMR3MapPT(pVM, pVM->mm.s.pvHyperAreaGC, pVM->mm.s.cbHyperArea, 0 /*fFlags*/,
+                    mmR3HyperRelocateCallback, NULL, "Hypervisor Memory Area");
     if (RT_FAILURE(rc))
         return rc;
     pVM->mm.s.fPGMInitialized = true;
@@ -172,12 +196,19 @@ VMMR3DECL(int) MMR3HyperInitFinalize(PVM pVM)
     for (;;)
     {
         RTGCPTR     GCPtr = pVM->mm.s.pvHyperAreaGC + pLookup->off;
-        unsigned    cPages = pLookup->cb >> PAGE_SHIFT;
+        uint32_t    cPages = pLookup->cb >> PAGE_SHIFT;
         switch (pLookup->enmType)
         {
             case MMLOOKUPHYPERTYPE_LOCKED:
-                rc = mmR3MapLocked(pVM, pLookup->u.Locked.pLockedMem, GCPtr, 0, cPages, 0);
+            {
+                PCRTHCPHYS paHCPhysPages = pLookup->u.Locked.paHCPhysPages;
+                for (uint32_t i = 0; i < cPages; i++)
+                {
+                    rc = PGMMap(pVM, GCPtr + (i << PAGE_SHIFT), paHCPhysPages[i], PAGE_SIZE, 0);
+                    AssertRCReturn(rc, rc);
+                }
                 break;
+            }
 
             case MMLOOKUPHYPERTYPE_HCPHYS:
                 rc = PGMMap(pVM, GCPtr, pLookup->u.HCPhys.HCPhys, pLookup->cb, 0);
@@ -186,8 +217,8 @@ VMMR3DECL(int) MMR3HyperInitFinalize(PVM pVM)
             case MMLOOKUPHYPERTYPE_GCPHYS:
             {
                 const RTGCPHYS  GCPhys = pLookup->u.GCPhys.GCPhys;
-                const size_t    cb = pLookup->cb;
-                for (unsigned off = 0; off < cb; off += PAGE_SIZE)
+                const uint32_t  cb = pLookup->cb;
+                for (uint32_t off = 0; off < cb; off += PAGE_SIZE)
                 {
                     RTHCPHYS HCPhys;
                     rc = PGMPhysGCPhys2HCPhys(pVM, GCPhys + off, &HCPhys);
@@ -288,7 +319,7 @@ static DECLCALLBACK(bool) mmR3HyperRelocateCallback(PVM pVM, RTGCPTR GCPtrOld, R
              */
             RTGCINTPTR offDelta = GCPtrNew - GCPtrOld;
             pVM->pVMRC                          += offDelta;
-            for (uint32_t i = 0; i < pVM->cCPUs; i++)
+            for (VMCPUID i = 0; i < pVM->cCpus; i++)
                 pVM->aCpus[i].pVMRC              = pVM->pVMRC;
 
             pVM->mm.s.pvHyperAreaGC             += offDelta;
@@ -311,6 +342,20 @@ static DECLCALLBACK(bool) mmR3HyperRelocateCallback(PVM pVM, RTGCPTR GCPtrOld, R
     return false;
 }
 
+/**
+ * Service a VMMCALLRING3_MMHYPER_LOCK call.
+ *
+ * @returns VBox status code.
+ * @param   pVM     The VM handle.
+ */
+VMMR3DECL(int) MMR3LockCall(PVM pVM)
+{
+    PMMHYPERHEAP pHeap = pVM->mm.s.CTX_SUFF(pHyperHeap);
+
+    int rc = PDMR3CritSectEnterEx(&pHeap->Lock, true /* fHostCall */);
+    AssertRC(rc);
+    return rc;
+}
 
 /**
  * Maps contiguous HC physical memory into the hypervisor region in the GC.
@@ -553,31 +598,33 @@ VMMR3DECL(int) MMR3HyperMapPages(PVM pVM, void *pvR3, RTR0PTR pvR0, size_t cPage
     if (RT_SUCCESS(rc))
     {
         /*
-         * Create a locked memory record and tell PGM about this.
+         * Copy the physical page addresses and tell PGM about them.
          */
-        PMMLOCKEDMEM pLockedMem = (PMMLOCKEDMEM)MMR3HeapAlloc(pVM, MM_TAG_MM, RT_OFFSETOF(MMLOCKEDMEM, aPhysPages[cPages]));
-        if (pLockedMem)
+        PRTHCPHYS paHCPhysPages = (PRTHCPHYS)MMR3HeapAlloc(pVM, MM_TAG_MM, sizeof(RTHCPHYS) * cPages);
+        if (paHCPhysPages)
         {
-            pLockedMem->pv      = pvR3;
-            pLockedMem->cb      = cPages << PAGE_SHIFT;
-            pLockedMem->eType   = MM_LOCKED_TYPE_HYPER_PAGES;
-            memset(&pLockedMem->u, 0, sizeof(pLockedMem->u));
             for (size_t i = 0; i < cPages; i++)
             {
-                AssertReleaseReturn(paPages[i].Phys != 0 && paPages[i].Phys != NIL_RTHCPHYS && !(paPages[i].Phys & PAGE_OFFSET_MASK), VERR_INTERNAL_ERROR);
-                pLockedMem->aPhysPages[i].Phys = paPages[i].Phys;
-                pLockedMem->aPhysPages[i].uReserved = (RTHCUINTPTR)pLockedMem;
+                AssertReleaseMsgReturn(paPages[i].Phys != 0 && paPages[i].Phys != NIL_RTHCPHYS && !(paPages[i].Phys & PAGE_OFFSET_MASK),
+                                       ("i=%#zx Phys=%RHp %s\n", i, paPages[i].Phys, pszDesc),
+                                       VERR_INTERNAL_ERROR);
+                paHCPhysPages[i] = paPages[i].Phys;
             }
 
-            /* map the stuff into guest address space. */
             if (pVM->mm.s.fPGMInitialized)
-                rc = mmR3MapLocked(pVM, pLockedMem, GCPtr, 0, ~(size_t)0, 0);
+            {
+                for (size_t i = 0; i < cPages; i++)
+                {
+                    rc = PGMMap(pVM, GCPtr + (i << PAGE_SHIFT), paHCPhysPages[i], PAGE_SIZE, 0);
+                    AssertRCBreak(rc);
+                }
+            }
             if (RT_SUCCESS(rc))
             {
                 pLookup->enmType = MMLOOKUPHYPERTYPE_LOCKED;
-                pLookup->u.Locked.pvR3       = pvR3;
-                pLookup->u.Locked.pvR0       = pvR0;
-                pLookup->u.Locked.pLockedMem = pLockedMem;
+                pLookup->u.Locked.pvR3          = pvR3;
+                pLookup->u.Locked.pvR0          = pvR0;
+                pLookup->u.Locked.paHCPhysPages = paHCPhysPages;
 
                 /* done. */
                 *pGCPtr   = GCPtr;
@@ -655,8 +702,8 @@ static int mmR3HyperMap(PVM pVM, const size_t cb, const char *pszDesc, PRTGCPTR
     AssertReturn(cbAligned >= cb, VERR_INVALID_PARAMETER);
     if (pVM->mm.s.offHyperNextStatic + cbAligned >= pVM->mm.s.cbHyperArea) /* don't use the last page, it's a fence. */
     {
-        AssertMsgFailed(("Out of static mapping space in the HMA! offHyperAreaGC=%x cbAligned=%x\n",
-                         pVM->mm.s.offHyperNextStatic, cbAligned));
+        AssertMsgFailed(("Out of static mapping space in the HMA! offHyperAreaGC=%x cbAligned=%x cbHyperArea=%x\n",
+                         pVM->mm.s.offHyperNextStatic, cbAligned, pVM->mm.s.cbHyperArea));
         return VERR_NO_MEMORY;
     }
 
@@ -711,7 +758,7 @@ static int mmR3HyperHeapCreate(PVM pVM, const size_t cb, PMMHYPERHEAP *ppHeap, P
      */
     const uint32_t  cbAligned = RT_ALIGN_32(cb, PAGE_SIZE);
     AssertReturn(cbAligned >= cb, VERR_INVALID_PARAMETER);
-    uint32_t const  cPages = cb >> PAGE_SHIFT;
+    uint32_t const  cPages = cbAligned >> PAGE_SHIFT;
     PSUPPAGE        paPages = (PSUPPAGE)MMR3HeapAlloc(pVM, MM_TAG_MM, cPages * sizeof(paPages[0]));
     if (!paPages)
         return VERR_NO_MEMORY;
@@ -720,15 +767,19 @@ static int mmR3HyperHeapCreate(PVM pVM, const size_t cb, PMMHYPERHEAP *ppHeap, P
     int rc = SUPR3PageAllocEx(cPages,
                               0 /*fFlags*/,
                               &pv,
+#ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
                               VMMIsHwVirtExtForced(pVM) ? &pvR0 : NULL,
+#else
+                              NULL,
+#endif
                               paPages);
     if (RT_SUCCESS(rc))
     {
-        if (!VMMIsHwVirtExtForced(pVM))
 #ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
+        if (!VMMIsHwVirtExtForced(pVM))
             pvR0 = NIL_RTR0PTR;
 #else
-            pvR0 = (uintptr_t)pv;
+        pvR0 = (uintptr_t)pv;
 #endif
         memset(pv, 0, cbAligned);
 
@@ -772,7 +823,6 @@ static int mmR3HyperHeapCreate(PVM pVM, const size_t cb, PMMHYPERHEAP *ppHeap, P
     return rc;
 }
 
-
 /**
  * Allocates a new heap.
  */
@@ -801,17 +851,6 @@ static int mmR3HyperHeapMap(PVM pVM, PMMHYPERHEAP pHeap, PRTGCPTR ppHeapGC)
 }
 
 
-#if 0
-/**
- * Destroys a heap.
- */
-static int mmR3HyperHeapDestroy(PVM pVM, PMMHYPERHEAP pHeap)
-{
-    /* all this is dealt with when unlocking and freeing locked memory. */
-}
-#endif
-
-
 /**
  * Allocates memory in the Hypervisor (GC VMM) area which never will
  * be freed and doesn't have any offset based relation to other heap blocks.
@@ -833,9 +872,37 @@ static int mmR3HyperHeapDestroy(PVM pVM, PMMHYPERHEAP pHeap)
  *                      memory.
  * @remark  This is assumed not to be used at times when serialization is required.
  */
-VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag, void **ppv)
+VMMR3DECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag, void **ppv)
+{
+    return MMR3HyperAllocOnceNoRelEx(pVM, cb, uAlignment, enmTag, 0/*fFlags*/, ppv);
+}
+
+
+/**
+ * Allocates memory in the Hypervisor (GC VMM) area which never will
+ * be freed and doesn't have any offset based relation to other heap blocks.
+ *
+ * The latter means that two blocks allocated by this API will not have the
+ * same relative position to each other in GC and HC. In short, never use
+ * this API for allocating nodes for an offset based AVL tree!
+ *
+ * The returned memory is of course zeroed.
+ *
+ * @returns VBox status code.
+ * @param   pVM         The VM to operate on.
+ * @param   cb          Number of bytes to allocate.
+ * @param   uAlignment  Required memory alignment in bytes.
+ *                      Values are 0,8,16,32 and PAGE_SIZE.
+ *                      0 -> default alignment, i.e. 8 bytes.
+ * @param   enmTag      The statistics tag.
+ * @param   fFlags      Flags, see MMHYPER_AONR_FLAGS_KERNEL_MAPPING.
+ * @param   ppv         Where to store the address to the allocated memory.
+ * @remark  This is assumed not to be used at times when serialization is required.
+ */
+VMMR3DECL(int) MMR3HyperAllocOnceNoRelEx(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag, uint32_t fFlags, void **ppv)
 {
     AssertMsg(cb >= 8, ("Hey! Do you really mean to allocate less than 8 bytes?! cb=%d\n", cb));
+    Assert(!(fFlags & ~(MMHYPER_AONR_FLAGS_KERNEL_MAPPING)));
 
     /*
      * Choose between allocating a new chunk of HMA memory
@@ -844,9 +911,13 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
      */
     if (   (   cb < _64K
             && (   uAlignment != PAGE_SIZE
-               || cb < 48*_1K))
-        ||  VMR3GetState(pVM) != VMSTATE_CREATING)
+                || cb < 48*_1K)
+            && !(fFlags & MMHYPER_AONR_FLAGS_KERNEL_MAPPING)
+           )
+        ||  VMR3GetState(pVM) != VMSTATE_CREATING
+       )
     {
+        Assert(!(fFlags & MMHYPER_AONR_FLAGS_KERNEL_MAPPING));
         int rc = MMHyperAlloc(pVM, cb, uAlignment, enmTag, ppv);
         if (    rc != VERR_MM_HYPER_NO_MEMORY
             ||  cb <= 8*_1K)
@@ -857,6 +928,14 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
         }
     }
 
+#ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
+    /*
+     * Set MMHYPER_AONR_FLAGS_KERNEL_MAPPING if we're in going to execute in ring-0.
+     */
+    if (VMMIsHwVirtExtForced(pVM))
+        fFlags |= MMHYPER_AONR_FLAGS_KERNEL_MAPPING;
+#endif
+
     /*
      * Validate alignment.
      */
@@ -876,9 +955,10 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
     /*
      * Allocate the pages and map them into HMA space.
      */
-    cb = RT_ALIGN(cb, PAGE_SIZE);
-    uint32_t const  cPages = cb >> PAGE_SHIFT;
-    PSUPPAGE        paPages = (PSUPPAGE)RTMemTmpAlloc(cPages * sizeof(paPages[0]));
+    uint32_t const  cbAligned = RT_ALIGN_32(cb, PAGE_SIZE);
+    AssertReturn(cbAligned >= cb, VERR_INVALID_PARAMETER);
+    uint32_t const  cPages    = cbAligned >> PAGE_SHIFT;
+    PSUPPAGE        paPages   = (PSUPPAGE)RTMemTmpAlloc(cPages * sizeof(paPages[0]));
     if (!paPages)
         return VERR_NO_TMP_MEMORY;
     void           *pvPages;
@@ -886,17 +966,18 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
     int rc = SUPR3PageAllocEx(cPages,
                               0 /*fFlags*/,
                               &pvPages,
-                              VMMIsHwVirtExtForced(pVM) ? &pvR0 : NULL,
+                              fFlags & MMHYPER_AONR_FLAGS_KERNEL_MAPPING ? &pvR0 : NULL,
                               paPages);
     if (RT_SUCCESS(rc))
     {
-        if (!VMMIsHwVirtExtForced(pVM))
+        if (!(fFlags & MMHYPER_AONR_FLAGS_KERNEL_MAPPING))
 #ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
             pvR0 = NIL_RTR0PTR;
 #else
-            pvR0 = (uintptr_t)pvPages;
+            pvR0 = (RTR0PTR)pvPages;
 #endif
-        memset(pvPages, 0, cb);
+
+        memset(pvPages, 0, cbAligned);
 
         RTGCPTR GCPtr;
         rc = MMR3HyperMapPages(pVM,
@@ -904,17 +985,17 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
                                pvR0,
                                cPages,
                                paPages,
-                               MMR3HeapAPrintf(pVM, MM_TAG_MM, "alloc once (%s)", mmR3GetTagName(enmTag)),
+                               MMR3HeapAPrintf(pVM, MM_TAG_MM, "alloc once (%s)", mmGetTagName(enmTag)),
                                &GCPtr);
         if (RT_SUCCESS(rc))
         {
             *ppv = pvPages;
-            Log2(("MMR3HyperAllocOnceNoRel: cb=%#x uAlignment=%#x returns VINF_SUCCESS and *ppv=%p\n",
-                  cb, uAlignment, *ppv));
+            Log2(("MMR3HyperAllocOnceNoRel: cbAligned=%#x uAlignment=%#x returns VINF_SUCCESS and *ppv=%p\n",
+                  cbAligned, uAlignment, *ppv));
             MMR3HyperReserve(pVM, PAGE_SIZE, "fence", NULL);
             return rc;
         }
-        AssertMsgFailed(("Failed to allocate %zd bytes! %Rrc\n", cb, rc));
+        AssertMsgFailed(("Failed to allocate %zd bytes! %Rrc\n", cbAligned, rc));
         SUPR3PageFreeEx(pvPages, cPages);
 
 
@@ -923,7 +1004,7 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
          * out during vga/vmmdev mmio2 allocation with certain ram sizes.
          */
         /** @todo make a proper fix for this so we will never end up in this kind of situation! */
-        Log(("MMR3HyperAllocOnceNoRel: MMR3HyperMapHCRam failed with rc=%Rrc, try MMHyperAlloc(,%#d,,) instead\n",  rc, cb));
+        Log(("MMR3HyperAllocOnceNoRel: MMR3HyperMapHCRam failed with rc=%Rrc, try MMHyperAlloc(,%#x,,) instead\n",  rc, cb));
         int rc2 = MMHyperAlloc(pVM, cb, uAlignment, enmTag, ppv);
         if (RT_SUCCESS(rc2))
         {
@@ -933,7 +1014,7 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
         }
     }
     else
-        AssertMsgFailed(("Failed to allocate %zd bytes! %Rrc\n", cb, rc));
+        AssertMsgFailed(("Failed to allocate %zd bytes! %Rrc\n", cbAligned, rc));
 
     if (rc == VERR_NO_MEMORY)
         rc = VERR_MM_HYPER_NO_MEMORY;
@@ -942,6 +1023,101 @@ VMMDECL(int) MMR3HyperAllocOnceNoRel(PVM pVM, size_t cb, unsigned uAlignment, MM
 }
 
 
+/**
+ * Lookus up a ring-3 pointer to HMA.
+ *
+ * @returns The lookup record on success, NULL on failure.
+ * @param   pVM                 The VM handle.
+ * @param   pvR3                The ring-3 address to look up.
+ */
+DECLINLINE(PMMLOOKUPHYPER) mmR3HyperLookupR3(PVM pVM, void *pvR3)
+{
+    PMMLOOKUPHYPER  pLookup = (PMMLOOKUPHYPER)((uint8_t *)pVM->mm.s.pHyperHeapR3 + pVM->mm.s.offLookupHyper);
+    for (;;)
+    {
+        switch (pLookup->enmType)
+        {
+            case MMLOOKUPHYPERTYPE_LOCKED:
+            {
+                unsigned off = (uint8_t *)pvR3 - (uint8_t *)pLookup->u.Locked.pvR3;
+                if (off < pLookup->cb)
+                    return pLookup;
+                break;
+            }
+
+            case MMLOOKUPHYPERTYPE_HCPHYS:
+            {
+                unsigned off = (uint8_t *)pvR3 - (uint8_t *)pLookup->u.HCPhys.pvR3;
+                if (off < pLookup->cb)
+                    return pLookup;
+                break;
+            }
+
+            case MMLOOKUPHYPERTYPE_GCPHYS:
+            case MMLOOKUPHYPERTYPE_MMIO2:
+            case MMLOOKUPHYPERTYPE_DYNAMIC:
+                /** @todo ?    */
+                break;
+
+            default:
+                AssertMsgFailed(("enmType=%d\n", pLookup->enmType));
+                return NULL;
+        }
+
+        /* next */
+        if ((unsigned)pLookup->offNext == NIL_OFFSET)
+            return NULL;
+        pLookup = (PMMLOOKUPHYPER)((uint8_t *)pLookup + pLookup->offNext);
+    }
+}
+
+
+/**
+ * Set / unset guard status on one or more hyper heap pages.
+ *
+ * @returns VBox status code (first failure).
+ * @param   pVM                 The VM handle.
+ * @param   pvStart             The hyper heap page address. Must be page
+ *                              aligned.
+ * @param   cb                  The number of bytes. Must be page aligned.
+ * @param   fSet                Wheter to set or unset guard page status.
+ */
+VMMR3DECL(int) MMR3HyperSetGuard(PVM pVM, void *pvStart, size_t cb, bool fSet)
+{
+    /*
+     * Validate input.
+     */
+    AssertReturn(!((uintptr_t)pvStart & PAGE_OFFSET_MASK), VERR_INVALID_POINTER);
+    AssertReturn(!(cb & PAGE_OFFSET_MASK), VERR_INVALID_PARAMETER);
+    PMMLOOKUPHYPER pLookup = mmR3HyperLookupR3(pVM, pvStart);
+    AssertReturn(pLookup, VERR_INVALID_PARAMETER);
+    AssertReturn(pLookup->enmType == MMLOOKUPHYPERTYPE_LOCKED, VERR_INVALID_PARAMETER);
+
+    /*
+     * Get down to business.
+     * Note! We quietly ignore errors from the support library since the
+     *       protection stuff isn't possible to implement on all platforms.
+     */
+    uint8_t    *pbR3  = (uint8_t *)pLookup->u.Locked.pvR3;
+    RTR0PTR     R0Ptr = pLookup->u.Locked.pvR0 != (uintptr_t)pLookup->u.Locked.pvR3
+                      ? pLookup->u.Locked.pvR0
+                      : NIL_RTR0PTR;
+    uint32_t    off   = (uint32_t)((uint8_t *)pvStart - pbR3);
+    int         rc;
+    if (fSet)
+    {
+        rc = PGMMapSetPage(pVM, MMHyperR3ToRC(pVM, pvStart), cb, 0);
+        SUPR3PageProtect(pbR3, R0Ptr, off, cb, RTMEM_PROT_NONE);
+    }
+    else
+    {
+        rc = PGMMapSetPage(pVM, MMHyperR3ToRC(pVM, pvStart), cb, X86_PTE_P | X86_PTE_A | X86_PTE_D | X86_PTE_RW);
+        SUPR3PageProtect(pbR3, R0Ptr, off, cb, RTMEM_PROT_READ | RTMEM_PROT_WRITE);
+    }
+    return rc;
+}
+
+
 /**
  * Convert hypervisor HC virtual address to HC physical address.
  *
@@ -960,7 +1136,7 @@ VMMR3DECL(RTHCPHYS) MMR3HyperHCVirt2HCPhys(PVM pVM, void *pvR3)
             {
                 unsigned off = (uint8_t *)pvR3 - (uint8_t *)pLookup->u.Locked.pvR3;
                 if (off < pLookup->cb)
-                    return (pLookup->u.Locked.pLockedMem->aPhysPages[off >> PAGE_SHIFT].Phys & X86_PTE_PAE_PG_MASK) | (off & PAGE_OFFSET_MASK);
+                    return pLookup->u.Locked.paHCPhysPages[off >> PAGE_SHIFT] | (off & PAGE_OFFSET_MASK);
                 break;
             }
 
@@ -1046,10 +1222,23 @@ VMMR3DECL(int)   MMR3HyperHCPhys2HCVirtEx(PVM pVM, RTHCPHYS HCPhys, void **ppv)
 VMMR3DECL(int) MMR3HyperReadGCVirt(PVM pVM, void *pvDst, RTGCPTR GCPtr, size_t cb)
 {
     if (GCPtr - pVM->mm.s.pvHyperAreaGC >= pVM->mm.s.cbHyperArea)
-        return VERR_INVALID_PARAMETER;
+        return VERR_INVALID_POINTER;
     return PGMR3MapRead(pVM, pvDst, GCPtr, cb);
 }
 
+/**
+ * Release the MM hypervisor heap lock if owned by the current VCPU
+ *
+ * @param   pVM         The VM to operate on.
+ */
+VMMR3DECL(void) MMR3ReleaseOwnedLocks(PVM pVM)
+{
+    PMMHYPERHEAP pHeap = pVM->mm.s.CTX_SUFF(pHyperHeap);
+
+    while (pHeap && PDMCritSectIsOwner(&pHeap->Lock))
+        PDMCritSectLeave(&pHeap->Lock);
+}
+
 
 /**
  * Info handler for 'hma', it dumps the list of lookup records for the hypervisor memory area.
@@ -1074,12 +1263,7 @@ static DECLCALLBACK(void) mmR3HyperInfoHma(PVM pVM, PCDBGFINFOHLP pHlp, const ch
                                 pLookup->off + pVM->mm.s.pvHyperAreaGC + pLookup->cb,
                                 pLookup->u.Locked.pvR3,
                                 pLookup->u.Locked.pvR0,
-                                sizeof(RTHCPTR) * 2,
-                                pLookup->u.Locked.pLockedMem->eType == MM_LOCKED_TYPE_HYPER_NOFREE  ? "nofree"
-                                : pLookup->u.Locked.pLockedMem->eType == MM_LOCKED_TYPE_HYPER       ? "autofree"
-                                : pLookup->u.Locked.pLockedMem->eType == MM_LOCKED_TYPE_HYPER_PAGES ? "pages"
-                                : pLookup->u.Locked.pLockedMem->eType == MM_LOCKED_TYPE_PHYS        ? "gstphys"
-                                : "??",
+                                sizeof(RTHCPTR) * 2, "",
                                 pLookup->pszDesc);
                 break;
 
