@@ -96,7 +96,7 @@ VMMR3DECL(int) PDMR3LdrLoadVMMR0U(PUVM pUVM)
  */
 int pdmR3LdrInitU(PUVM pUVM)
 {
-#ifdef PDMLDR_FAKE_MODE
+#if defined(PDMLDR_FAKE_MODE) || !defined(VBOX_WITH_RAW_MODE)
     return VINF_SUCCESS;
 
 #else
@@ -147,7 +147,9 @@ void pdmR3LdrTermU(PUVM pUVM)
                 break;
             }
 
+#ifdef VBOX_WITH_RAW_MODE
             case PDMMOD_TYPE_RC:
+#endif
             case PDMMOD_TYPE_R3:
                 /* MM will free this memory for us - it's alloc only memory. :-) */
                 break;
@@ -176,6 +178,7 @@ void pdmR3LdrTermU(PUVM pUVM)
  */
 VMMR3DECL(void) PDMR3LdrRelocateU(PUVM pUVM, RTGCINTPTR offDelta)
 {
+#ifdef VBOX_WITH_RAW_MODE
     LogFlow(("PDMR3LdrRelocate: offDelta=%RGv\n", offDelta));
 
     /*
@@ -216,6 +219,7 @@ VMMR3DECL(void) PDMR3LdrRelocateU(PUVM pUVM, RTGCINTPTR offDelta)
             }
         }
     }
+#endif
 }
 
 
@@ -301,6 +305,7 @@ int pdmR3LoadR3U(PUVM pUVM, const char *pszFilename, const char *pszName)
 }
 
 
+#ifdef VBOX_WITH_RAW_MODE
 /**
  * Resolve an external symbol during RTLdrGetBits() of a RC module.
  *
@@ -497,7 +502,7 @@ VMMR3DECL(int) PDMR3LdrLoadRC(PVM pVM, const char *pszFilename, const char *pszN
                             if (pUVM->pdm.s.pModules)
                             {
                                 /* we don't expect this list to be very long, so rather save the tail pointer. */
-                                PPDMMOD pCur = pUVM->pdm.s.pModules;
+                                pCur = pUVM->pdm.s.pModules;
                                 while (pCur->pNext)
                                     pCur = pCur->pNext;
                                 pCur->pNext = pModule;
@@ -528,14 +533,16 @@ VMMR3DECL(int) PDMR3LdrLoadRC(PVM pVM, const char *pszFilename, const char *pszN
         int rc2 = RTLdrClose(pModule->hLdrMod);
         AssertRC(rc2);
     }
-    RTMemFree(pModule);
-    RTMemTmpFree(pszFile);
 
     /* Don't consider VERR_PDM_MODULE_NAME_CLASH and VERR_NO_MEMORY above as these are very unlikely. */
     if (RT_FAILURE(rc))
-        return VMSetError(pVM, rc, RT_SRC_POS, N_("Cannot load GC module %s"), pszFilename);
+        rc = VMSetError(pVM, rc, RT_SRC_POS, N_("Cannot load GC module %s"), pszFilename);
+
+    RTMemFree(pModule);
+    RTMemTmpFree(pszFile);
     return rc;
 }
+#endif /* VBOX_WITH_RAW_MODE */
 
 
 /**
@@ -601,7 +608,7 @@ static int pdmR3LoadR0U(PUVM pUVM, const char *pszFilename, const char *pszName)
         if (pUVM->pdm.s.pModules)
         {
             /* we don't expect this list to be very long, so rather save the tail pointer. */
-            PPDMMOD pCur = pUVM->pdm.s.pModules;
+            pCur = pUVM->pdm.s.pModules;
             while (pCur->pNext)
                 pCur = pCur->pNext;
             pCur->pNext = pModule;
@@ -774,7 +781,7 @@ VMMR3DECL(int) PDMR3LdrGetSymbolR0Lazy(PVM pVM, const char *pszModule, const cha
  */
 VMMR3DECL(int) PDMR3LdrGetSymbolRC(PVM pVM, const char *pszModule, const char *pszSymbol, PRTRCPTR pRCPtrValue)
 {
-#ifdef PDMLDR_FAKE_MODE
+#if defined(PDMLDR_FAKE_MODE) || !defined(VBOX_WITH_RAW_MODE)
     *pRCPtrValue = 0xfeedf00d;
     return VINF_SUCCESS;
 
@@ -830,7 +837,7 @@ VMMR3DECL(int) PDMR3LdrGetSymbolRC(PVM pVM, const char *pszModule, const char *p
  */
 VMMR3DECL(int) PDMR3LdrGetSymbolRCLazy(PVM pVM, const char *pszModule, const char *pszSymbol, PRTRCPTR pRCPtrValue)
 {
-#ifdef PDMLDR_FAKE_MODE
+#if defined(PDMLDR_FAKE_MODE) || !defined(VBOX_WITH_RAW_MODE)
     *pRCPtrValue = 0xfeedf00d;
     return VINF_SUCCESS;
 
