@@ -895,6 +895,12 @@ VMMR3DECL(int) PDMR3ThreadSuspend(PPDMTHREAD pThread)
     AssertReturn(pThread->u32Version == PDMTHREAD_VERSION, VERR_INVALID_MAGIC);
     Assert(pThread->Thread != RTThreadSelf());
 
+    /*
+     * This is a noop if the thread is already suspended.
+     */
+    if (pThread->enmState == PDMTHREADSTATE_SUSPENDED)
+        return VINF_SUCCESS;
+
     /*
      * Change the state to resuming and kick the thread.
      */
