@@ -438,7 +438,7 @@ static VBOXIDTE_GENERIC     g_aIdt[256] =
 *   Internal Functions                                                         *
 *******************************************************************************/
 static DECLCALLBACK(int) trpmR3Save(PVM pVM, PSSMHANDLE pSSM);
-static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version);
+static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass);
 static DECLCALLBACK(int) trpmR3GuestIDTWriteHandler(PVM pVM, RTGCPTR GCPtr, void *pvPtr, void *pvBuf, size_t cbBuf, PGMACCESSTYPE enmAccessType, void *pvUser);
 
 
@@ -466,7 +466,7 @@ VMMR3DECL(int) TRPMR3Init(PVM pVM)
     pVM->trpm.s.offVM              = RT_OFFSETOF(VM, trpm);
     pVM->trpm.s.offTRPMCPU         = RT_OFFSETOF(VM, aCpus[0].trpm) - RT_OFFSETOF(VM, trpm);
 
-    for (VMCPUID i = 0; i < pVM->cCPUs; i++)
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         PVMCPU pVCpu = &pVM->aCpus[i];
 
@@ -507,6 +507,7 @@ VMMR3DECL(int) TRPMR3Init(PVM pVM)
      * Register the saved state data unit.
      */
     int rc = SSMR3RegisterInternal(pVM, "trpm", 1, TRPM_SAVED_STATE_VERSION, sizeof(TRPM),
+                                   NULL, NULL, NULL,
                                    NULL, trpmR3Save, NULL,
                                    NULL, trpmR3Load, NULL);
     if (RT_FAILURE(rc))
@@ -742,7 +743,7 @@ VMMR3DECL(void) TRPMR3Reset(PVM pVM)
     /*
      * Reinitialize other members calling the relocator to get things right.
      */
-    for (unsigned i=0;i<pVM->cCPUs;i++)
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         PVMCPU pVCpu = &pVM->aCpus[i];
         pVCpu->trpm.s.uActiveVector = ~0;
@@ -774,10 +775,9 @@ static DECLCALLBACK(int) trpmR3Save(PVM pVM, PSSMHANDLE pSSM)
     /*
      * Active and saved traps.
      */
-    for (unsigned i=0;i<pVM->cCPUs;i++)
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         PTRPMCPU pTrpmCpu = &pVM->aCpus[i].trpm.s;
-
         SSMR3PutUInt(pSSM,      pTrpmCpu->uActiveVector);
         SSMR3PutUInt(pSSM,      pTrpmCpu->enmActiveType);
         SSMR3PutGCUInt(pSSM,    pTrpmCpu->uActiveErrorCode);
@@ -817,19 +817,21 @@ static DECLCALLBACK(int) trpmR3Save(PVM pVM, PSSMHANDLE pSSM)
  * @returns VBox status code.
  * @param   pVM             VM Handle.
  * @param   pSSM            SSM operation handle.
- * @param   u32Version      Data layout version.
+ * @param   uVersion        Data layout version.
+ * @param   uPass           The data pass.
  */
-static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
+static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
 {
     LogFlow(("trpmR3Load:\n"));
+    Assert(uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     /*
      * Validate version.
      */
-    if (    u32Version != TRPM_SAVED_STATE_VERSION
-        &&  u32Version != TRPM_SAVED_STATE_VERSION_UNI)
+    if (    uVersion != TRPM_SAVED_STATE_VERSION
+        &&  uVersion != TRPM_SAVED_STATE_VERSION_UNI)
     {
-        AssertMsgFailed(("trpmR3Load: Invalid version u32Version=%d!\n", u32Version));
+        AssertMsgFailed(("trpmR3Load: Invalid version uVersion=%d!\n", uVersion));
         return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
     }
 
@@ -843,9 +845,9 @@ static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Versio
      */
     PTRPM pTrpm = &pVM->trpm.s;
 
-    if (u32Version == TRPM_SAVED_STATE_VERSION)
+    if (uVersion == TRPM_SAVED_STATE_VERSION)
     {
-        for (unsigned i=0;i<pVM->cCPUs;i++)
+        for (VMCPUID i = 0; i < pVM->cCpus; i++)
         {
             PTRPMCPU pTrpmCpu = &pVM->aCpus[i].trpm.s;
             SSMR3GetUInt(pSSM,      &pTrpmCpu->uActiveVector);
@@ -1233,7 +1235,7 @@ VMMR3DECL(RTRCPTR) TRPMR3GetGuestTrapHandler(PVM pVM, unsigned iTrap)
 VMMR3DECL(int) TRPMR3SetGuestTrapHandler(PVM pVM, unsigned iTrap, RTRCPTR pHandler)
 {
     /* Only valid in raw mode which implies 1 VCPU */
-    Assert(PATMIsEnabled(pVM) && pVM->cCPUs == 1);
+    Assert(PATMIsEnabled(pVM) && pVM->cCpus == 1);
     PVMCPU pVCpu = &pVM->aCpus[0];
 
     /*
@@ -1356,7 +1358,7 @@ VMMR3DECL(int) TRPMR3SetGuestTrapHandler(PVM pVM, unsigned iTrap, RTRCPTR pHandl
 VMMR3DECL(bool) TRPMR3IsGateHandler(PVM pVM, RTRCPTR GCPtr)
 {
     /* Only valid in raw mode which implies 1 VCPU */
-    Assert(PATMIsEnabled(pVM) && pVM->cCPUs == 1);
+    Assert(PATMIsEnabled(pVM) && pVM->cCpus == 1);
     PVMCPU pVCpu = &pVM->aCpus[0];
 
     /*
