@@ -152,6 +152,50 @@ static void mmHyperHeapCheck(PMMHYPERHEAP pHeap);
 #endif
 
 
+
+/**
+ * Locks the hypervisor heap.
+ * This might call back to Ring-3 in order to deal with lock contention in GC and R3.
+ *
+ * @param   pVM     The VM handle.
+ */
+static int mmHyperLock(PVM pVM)
+{
+    PMMHYPERHEAP pHeap = pVM->mm.s.CTX_SUFF(pHyperHeap);
+
+#ifdef IN_RING3
+    if (!PDMCritSectIsInitialized(&pHeap->Lock))
+        return VINF_SUCCESS;     /* early init */
+#else
+    Assert(PDMCritSectIsInitialized(&pHeap->Lock));
+#endif
+    int rc = PDMCritSectEnter(&pHeap->Lock, VERR_SEM_BUSY);
+#if defined(IN_RC) || defined(IN_RING0)
+    if (rc == VERR_SEM_BUSY)
+        rc = VMMRZCallRing3NoCpu(pVM, VMMCALLRING3_MMHYPER_LOCK, 0);
+#endif
+    AssertRC(rc);
+    return rc;
+}
+
+
+/**
+ * Unlocks the hypervisor heap.
+ *
+ * @param   pVM     The VM handle.
+ */
+static void mmHyperUnlock(PVM pVM)
+{
+    PMMHYPERHEAP pHeap = pVM->mm.s.CTX_SUFF(pHyperHeap);
+
+#ifdef IN_RING3
+    if (!PDMCritSectIsInitialized(&pHeap->Lock))
+        return;     /* early init */
+#endif
+    Assert(PDMCritSectIsInitialized(&pHeap->Lock));
+    PDMCritSectLeave(&pHeap->Lock);
+}
+
 /**
  * Allocates memory in the Hypervisor (RC VMM) area.
  * The returned memory is of course zeroed.
@@ -165,9 +209,8 @@ static void mmHyperHeapCheck(PMMHYPERHEAP pHeap);
  * @param   enmTag      The statistics tag.
  * @param   ppv         Where to store the address to the allocated
  *                      memory.
- * @remark  This is assumed not to be used at times when serialization is required.
  */
-VMMDECL(int) MMHyperAlloc(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag, void **ppv)
+static int mmHyperAllocInternal(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag, void **ppv)
 {
     AssertMsg(cb >= 8, ("Hey! Do you really mean to allocate less than 8 bytes?! cb=%d\n", cb));
 
@@ -266,7 +309,7 @@ VMMDECL(int) MMHyperAlloc(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag,
             pStat->cAllocations++;
 #endif
             *ppv = pv;
-            /* ASMMemZero32(pv, cbAligned); - not required since memory is alloc-only and SUPPageAlloc zeros it. */
+            /* ASMMemZero32(pv, cbAligned); - not required since memory is alloc-only and SUPR3PageAlloc zeros it. */
             Log2(("MMHyperAlloc: cb=%#x uAlignment=%#x returns VINF_SUCCESS and *ppv=%p\n", cb, uAlignment, ppv));
             return VINF_SUCCESS;
         }
@@ -281,7 +324,23 @@ VMMDECL(int) MMHyperAlloc(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag,
     return VERR_MM_HYPER_NO_MEMORY;
 }
 
+/**
+ * Wrapper for mmHyperAllocInternal
+ */
+VMMDECL(int) MMHyperAlloc(PVM pVM, size_t cb, unsigned uAlignment, MMTAG enmTag, void **ppv)
+{
+    int rc;
 
+    rc = mmHyperLock(pVM);
+    AssertRCReturn(rc, rc);
+
+    LogFlow(("MMHyperAlloc %x align=%x tag=%s\n", cb, uAlignment, mmGetTagName(enmTag)));
+
+    rc = mmHyperAllocInternal(pVM, cb, uAlignment, enmTag, ppv);
+
+    mmHyperUnlock(pVM);
+    return rc;
+}
 
 /**
  * Allocates a chunk of memory from the specified heap.
@@ -680,13 +739,13 @@ static void mmR3HyperStatRegisterOne(PVM pVM, PMMHYPERSTAT pStat)
 {
     if (pStat->fRegistered)
         return;
-    const char *pszTag = mmR3GetTagName((MMTAG)pStat->Core.Key);
+    const char *pszTag = mmGetTagName((MMTAG)pStat->Core.Key);
+    STAMR3RegisterF(pVM, &pStat->cbCurAllocated, STAMTYPE_U32, STAMVISIBILITY_ALWAYS, STAMUNIT_BYTES, "Number of bytes currently allocated.",           "/MM/HyperHeap/%s", pszTag);
     STAMR3RegisterF(pVM, &pStat->cAllocations,   STAMTYPE_U64, STAMVISIBILITY_ALWAYS, STAMUNIT_COUNT, "Number of alloc calls.",                         "/MM/HyperHeap/%s/cAllocations", pszTag);
     STAMR3RegisterF(pVM, &pStat->cFrees,         STAMTYPE_U64, STAMVISIBILITY_ALWAYS, STAMUNIT_COUNT, "Number of free calls.",                          "/MM/HyperHeap/%s/cFrees", pszTag);
     STAMR3RegisterF(pVM, &pStat->cFailures,      STAMTYPE_U64, STAMVISIBILITY_ALWAYS, STAMUNIT_COUNT, "Number of failures.",                            "/MM/HyperHeap/%s/cFailures", pszTag);
     STAMR3RegisterF(pVM, &pStat->cbAllocated,    STAMTYPE_U64, STAMVISIBILITY_ALWAYS, STAMUNIT_BYTES, "Total number of allocated bytes.",               "/MM/HyperHeap/%s/cbAllocated", pszTag);
     STAMR3RegisterF(pVM, &pStat->cbFreed,        STAMTYPE_U64, STAMVISIBILITY_ALWAYS, STAMUNIT_BYTES, "Total number of freed bytes.",                   "/MM/HyperHeap/%s/cbFreed", pszTag);
-    STAMR3RegisterF(pVM, &pStat->cbCurAllocated, STAMTYPE_U32, STAMVISIBILITY_ALWAYS, STAMUNIT_BYTES, "Number of bytes currently allocated.",           "/MM/HyperHeap/%s/cbCurAllocated", pszTag);
     STAMR3RegisterF(pVM, &pStat->cbMaxAllocated, STAMTYPE_U32, STAMVISIBILITY_ALWAYS, STAMUNIT_BYTES, "Max number of bytes allocated at the same time.","/MM/HyperHeap/%s/cbMaxAllocated", pszTag);
     pStat->fRegistered = true;
 }
@@ -704,7 +763,7 @@ static void mmR3HyperStatRegisterOne(PVM pVM, PMMHYPERSTAT pStat)
  * @param   pv          The memory to free.
  * @remark  Try avoid free hyper memory.
  */
-VMMDECL(int) MMHyperFree(PVM pVM, void *pv)
+static int mmHyperFreeInternal(PVM pVM, void *pv)
 {
     Log2(("MMHyperFree: pv=%p\n", pv));
     if (!pv)
@@ -757,7 +816,7 @@ VMMDECL(int) MMHyperFree(PVM pVM, void *pv)
     AssertMsgReturn(pHeap->u32Magic == MMHYPERHEAP_MAGIC,
                     ("%p: u32Magic=%#x\n", pv, pHeap->u32Magic),
                     VERR_INVALID_POINTER);
-Assert(pHeap == pVM->mm.s.CTX_SUFF(pHyperHeap));
+    Assert(pHeap == pVM->mm.s.CTX_SUFF(pHyperHeap));
 
     /* Some more verifications using additional info from pHeap. */
     AssertMsgReturn((uintptr_t)pChunk + offPrev >= (uintptr_t)pHeap->CTX_SUFF(pbHeap),
@@ -847,6 +906,25 @@ Assert(pHeap == pVM->mm.s.CTX_SUFF(pHyperHeap));
 }
 
 
+/**
+ * Wrapper for mmHyperFreeInternal
+ */
+VMMDECL(int) MMHyperFree(PVM pVM, void *pv)
+{
+    int rc;
+
+    rc = mmHyperLock(pVM);
+    AssertRCReturn(rc, rc);
+
+    LogFlow(("MMHyperFree %p\n", pv));
+
+    rc = mmHyperFreeInternal(pVM, pv);
+
+    mmHyperUnlock(pVM);
+    return rc;
+}
+
+
 /**
  * Free memory a memory chunk.
  *
@@ -1031,7 +1109,7 @@ static void mmHyperHeapDumpOne(PMMHYPERHEAP pHeap, PMMHYPERCHUNKFREE pCur)
             Log(("%p  %06x USED offNext=%06x offPrev=-%06x %s%s\n",
                  pCur, (uintptr_t)pCur - (uintptr_t)pHeap->CTX_SUFF(pbHeap),
                  pCur->core.offNext, -MMHYPERCHUNK_GET_OFFPREV(&pCur->core),
-                 mmR3GetTagName((MMTAG)pStat->Core.Key), pszSelf));
+                 mmGetTagName((MMTAG)pStat->Core.Key), pszSelf));
 #else
             Log(("%p  %06x USED offNext=%06x offPrev=-%06x %d%s\n",
                  pCur, (uintptr_t)pCur - (uintptr_t)pHeap->CTX_SUFF(pbHeap),
@@ -1116,7 +1194,12 @@ static void mmHyperHeapCheck(PMMHYPERHEAP pHeap)
 VMMDECL(void) MMHyperHeapCheck(PVM pVM)
 {
 #ifdef MMHYPER_HEAP_STRICT
+    int rc;
+
+    rc = mmHyperLock(pVM);
+    AssertRC(rc);
     mmHyperHeapCheck(pVM->mm.s.CTX_SUFF(pHyperHeap));
+    mmHyperUnlock(pVM);
 #endif
 }
 
