@@ -57,7 +57,10 @@ VMMDECL(PPDMQUEUEITEMCORE) PDMQueueAlloc(PPDMQUEUE pQueue)
     {
         i = pQueue->iFreeTail;
         if (i == pQueue->iFreeHead)
+        {
+            STAM_REL_COUNTER_INC(&pQueue->StatAllocFailures);
             return NULL;
+        }
         pNew = pQueue->aFreeItems[i].CTX_SUFF(pItem);
         iNext = (i + 1) % (pQueue->cItems + PDMQUEUE_FREE_SLACK);
     } while (!ASMAtomicCmpXchgU32(&pQueue->iFreeTail, iNext, i));
@@ -91,11 +94,14 @@ VMMDECL(void) PDMQueueInsert(PPDMQUEUE pQueue, PPDMQUEUEITEMCORE pItem)
         PVM pVM = pQueue->CTX_SUFF(pVM);
         Log2(("PDMQueueInsert: VM_FF_PDM_QUEUES %d -> 1\n", VM_FF_ISSET(pVM, VM_FF_PDM_QUEUES)));
         VM_FF_SET(pVM, VM_FF_PDM_QUEUES);
+        ASMAtomicBitSet(&pVM->pdm.s.fQueueFlushing, PDM_QUEUE_FLUSH_FLAG_PENDING_BIT);
 #ifdef IN_RING3
         REMR3NotifyQueuePending(pVM); /** @todo r=bird: we can remove REMR3NotifyQueuePending and let VMR3NotifyFF do the work. */
-        VMR3NotifyFF(pVM, true);
+        VMR3NotifyGlobalFFU(pVM->pUVM, VMNOTIFYFF_FLAGS_DONE_REM);
 #endif
     }
+    STAM_REL_COUNTER_INC(&pQueue->StatInsert);
+    STAM_STATS({ ASMAtomicIncU32(&pQueue->cStatPending); });
 }
 
 
@@ -122,8 +128,8 @@ VMMDECL(void) PDMQueueInsertEx(PPDMQUEUE pQueue, PPDMQUEUEITEMCORE pItem, uint64
     }
     else */
     {
-        VM_FF_SET(pVM, VM_FF_TO_R3);
-        Log2(("PDMQueueInsertEx: Setting VM_FF_TO_R3\n"));
+        VMCPU_FF_SET(VMMGetCpu0(pVM), VMCPU_FF_TO_R3);
+        Log2(("PDMQueueInsertEx: Setting VMCPU_FF_TO_R3\n"));
     }
 #endif
 }
@@ -179,19 +185,12 @@ VMMDECL(void) PDMQueueFlush(PPDMQUEUE pQueue)
     Assert(pQueue->pVMR3);
     PVM pVM = pQueue->CTX_SUFF(pVM);
 
-#ifdef IN_RC
-    Assert(pQueue->pVMRC);
+#if defined(IN_RC) || defined(IN_RING0)
+    Assert(pQueue->CTX_SUFF(pVM));
     pVM->pdm.s.CTX_SUFF(pQueueFlush) = pQueue;
-    VMMGCCallHost(pVM, VMMCALLHOST_PDM_QUEUE_FLUSH, (uintptr_t)pQueue);
-
-#elif defined(IN_RING0)
-    Assert(pQueue->pVMR0);
-    pVM->pdm.s.CTX_SUFF(pQueueFlush) = pQueue;
-    VMMR0CallHost(pVM, VMMCALLHOST_PDM_QUEUE_FLUSH, (uintptr_t)pQueue);
+    VMMRZCallRing3NoCpu(pVM, VMMCALLRING3_PDM_QUEUE_FLUSH, (uintptr_t)pQueue);
 
 #else /* IN_RING3: */
-    PVMREQ pReq;
-    VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)PDMR3QueueFlushWorker, 2, pVM, pQueue);
-    VMR3ReqFree(pReq);
+    VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)PDMR3QueueFlushWorker, 2, pVM, pQueue);
 #endif
 }
