@@ -55,7 +55,7 @@
  */
 VMMDECL(RTGCPTR) SELMToFlatBySel(PVM pVM, RTSEL Sel, RTGCPTR Addr)
 {
-    Assert(pVM->cCPUs == 1 && !CPUMIsGuestInLongMode(VMMGetCpu(pVM)));    /* DON'T USE! */
+    Assert(pVM->cCpus == 1 && !CPUMIsGuestInLongMode(VMMGetCpu(pVM)));    /* DON'T USE! */
 
     /** @todo check the limit. */
     X86DESC    Desc;
@@ -68,7 +68,7 @@ VMMDECL(RTGCPTR) SELMToFlatBySel(PVM pVM, RTSEL Sel, RTGCPTR Addr)
         Desc = paLDT[Sel >> X86_SEL_SHIFT];
     }
 
-    return (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
+    return (RTGCPTR)(((RTGCUINTPTR)Addr + X86DESC_BASE(Desc)) & 0xffffffff);
 }
 #endif /* !IN_RING0 */
 
@@ -331,6 +331,10 @@ VMMDECL(int) SELMToFlatEx(PVM pVM, DIS_SELREG SelReg, PCCPUMCTXCORE pCtxCore, RT
         /* calc address assuming straight stuff. */
         pvFlat = (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
 
+        /* Cut the address to 32 bits. */
+        Assert(!CPUMIsGuestInLongMode(pVCpu));
+        pvFlat &= 0xffffffff;
+
         u1Present     = Desc.Gen.u1Present;
         u1Granularity = Desc.Gen.u1Granularity;
         u1DescType    = Desc.Gen.u1DescType;
@@ -538,6 +542,10 @@ VMMDECL(int) SELMToFlatBySelEx(PVM pVM, X86EFLAGS eflags, RTSEL Sel, RTGCPTR Add
         /* calc address assuming straight stuff. */
         pvFlat = (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
 
+        /* Cut the address to 32 bits. */
+        Assert(!CPUMIsGuestInLongMode(pVCpu));
+        pvFlat &= 0xffffffff;
+
         u1Present     = Desc.Gen.u1Present;
         u1Granularity = Desc.Gen.u1Granularity;
         u1DescType    = Desc.Gen.u1DescType;
@@ -736,6 +744,9 @@ DECLINLINE(int) selmValidateAndConvertCSAddrStd(PVM pVM, RTSEL SelCPL, RTSEL Sel
                 if ((RTGCUINTPTR)Addr <= u32Limit)
                 {
                     *ppvFlat = (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
+                    /* Cut the address to 32 bits. */
+                    *ppvFlat &= 0xffffffff;
+
                     if (pcBits)
                         *pcBits = Desc.Gen.u1DefBig ? 32 : 16; /** @todo GUEST64 */
                     return VINF_SUCCESS;
@@ -834,7 +845,7 @@ DECLINLINE(int) selmValidateAndConvertCSAddrHidden(PVMCPU pVCpu, RTSEL SelCPL, R
  */
 VMMDECL(int) SELMValidateAndConvertCSAddrGCTrap(PVM pVM, X86EFLAGS eflags, RTSEL SelCPL, RTSEL SelCS, RTGCPTR Addr, PRTGCPTR ppvFlat, uint32_t *pcBits)
 {
-    Assert(pVM->cCPUs == 1);
+    Assert(pVM->cCpus == 1);
     PVMCPU pVCpu = &pVM->aCpus[0];
 
     if (    CPUMIsGuestInRealMode(pVCpu)
@@ -995,7 +1006,7 @@ void selmSetRing1Stack(PVM pVM, uint32_t ss, RTGCPTR32 esp)
  */
 VMMDECL(int) SELMGetRing1Stack(PVM pVM, uint32_t *pSS, PRTGCPTR32 pEsp)
 {
-    Assert(pVM->cCPUs == 1);
+    Assert(pVM->cCpus == 1);
     PVMCPU pVCpu = &pVM->aCpus[0];
 
     if (pVM->selm.s.fSyncTSSRing0Stack)
