@@ -877,7 +877,8 @@ static int trpmGCTrap0dHandler(PVM pVM, PTRPMCPU pTrpmCpu, PCPUMCTXCORE pRegFram
     if (    pVCpu->trpm.s.uActiveErrorCode == 0
         &&  (Cpu.pCurInstr->optype & OPTYPE_PORTIO))
     {
-        rc = EMInterpretPortIO(pVM, pVCpu, pRegFrame, &Cpu, cbOp);
+        VBOXSTRICTRC rcStrict = EMInterpretPortIO(pVM, pVCpu, pRegFrame, &Cpu, cbOp);
+        rc = VBOXSTRICTRC_TODO(rcStrict);
         return trpmGCExitTrap(pVM, pVCpu, rc, pRegFrame);
     }
 
