@@ -24,6 +24,21 @@
 
 #include <iprt/mem.h>
 
+/**
+ * The GVMM per VM data.
+ */
+typedef struct GVMMPERVCPU
+{
+    /** The time the halted EMT thread expires.
+     * 0 if the EMT thread is blocked here. */
+    uint64_t volatile   u64HaltExpire;
+    /** The event semaphore the EMT thread is blocking on. */
+    RTSEMEVENTMULTI     HaltEventMulti;
+    /** The APIC ID of the CPU that EMT was scheduled on the last time we checked. */
+    uint8_t             iCpuEmt;
+} GVMMPERVCPU;
+/** Pointer to the GVMM per VCPU data. */
+typedef GVMMPERVCPU *PGVMMPERVCPU;
 
 /**
  * The GVMM per VM data.
@@ -39,14 +54,6 @@ typedef struct GVMMPERVM
     /** The ring-3 mapping of the VM pages. */
     RTR0MEMOBJ          VMPagesMapObj;
 
-    /** The time the halted EMT thread expires.
-     * 0 if the EMT thread is blocked here. */
-    uint64_t volatile   u64HaltExpire;
-    /** The event semaphore the EMT thread is blocking on. */
-    RTSEMEVENTMULTI     HaltEventMulti;
-    /** The APIC ID of the CPU that EMT was scheduled on the last time we checked. */
-    uint8_t             iCpuEmt;
-
     /** The scheduler statistics. */
     GVMMSTATSSCHED      StatsSched;
 
