@@ -50,8 +50,8 @@
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2);
-static DECLCALLBACK(void) hwaccmR0DisableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2);
+static DECLCALLBACK(void) hwaccmR0EnableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2);
+static DECLCALLBACK(void) hwaccmR0DisableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2);
 static DECLCALLBACK(void) HWACCMR0InitCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2);
 static              int   hwaccmR0CheckCpuRcArray(int *paRc, unsigned cErrorCodes, RTCPUID *pidCpu);
 static DECLCALLBACK(void) hwaccmR0PowerCallback(RTPOWEREVENT enmEvent, void *pvUser);
@@ -89,6 +89,9 @@ static struct
         /** Host CR4 value (set by ring-0 VMX init) */
         uint64_t                    hostCR4;
 
+        /** Host EFER value (set by ring-0 VMX init) */
+        uint64_t                    hostEFER;
+
         /** VMX MSR values */
         struct
         {
@@ -112,14 +115,17 @@ static struct
     } vmx;
     struct
     {
-        /** Set by the ring-0 driver to indicate SVM is supported by the CPU. */
-        bool                        fSupported;
+        /* HWCR msr (for diagnostics) */
+        uint64_t                    msrHWCR;
 
         /** SVM revision. */
         uint32_t                    u32Rev;
 
         /** SVM feature bits from cpuid 0x8000000a */
         uint32_t                    u32Features;
+
+        /** Set by the ring-0 driver to indicate SVM is supported by the CPU. */
+        bool                        fSupported;
     } svm;
     /** Saved error from detection */
     int32_t         lLastError;
@@ -132,6 +138,7 @@ static struct
 
     HWACCMSTATE                     enmHwAccmState;
 
+    bool                            fGlobalInit;
     volatile        bool            fSuspended;
 } HWACCMR0Globals;
 
@@ -144,7 +151,8 @@ static struct
  */
 VMMR0DECL(int) HWACCMR0Init(void)
 {
-    int        rc;
+    int     rc;
+    bool    fAMDVPresent = false;
 
     memset(&HWACCMR0Globals, 0, sizeof(HWACCMR0Globals));
     HWACCMR0Globals.enmHwAccmState = HWACCMSTATE_UNINITIALIZED;
@@ -163,6 +171,9 @@ VMMR0DECL(int) HWACCMR0Init(void)
     HWACCMR0Globals.pfnTermVM           = HWACCMR0DummyTermVM;
     HWACCMR0Globals.pfnSetupVM          = HWACCMR0DummySetupVM;
 
+    /* Default is global VT-x/AMD-V init */
+    HWACCMR0Globals.fGlobalInit         = true;
+
     /*
      * Check for VT-x and AMD-V capabilities
      */
@@ -261,6 +272,7 @@ VMMR0DECL(int) HWACCMR0Init(void)
                         if (!HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
                         {
                             HWACCMR0Globals.vmx.hostCR4             = ASMGetCR4();
+                            HWACCMR0Globals.vmx.hostEFER            = ASMRdMsr(MSR_K6_EFER);
 
                             rc = RTR0MemObjAllocCont(&pScatchMemObj, 1 << PAGE_SHIFT, true /* executable R0 mapping */);
                             if (RT_FAILURE(rc))
@@ -346,6 +358,11 @@ VMMR0DECL(int) HWACCMR0Init(void)
                 int     aRc[RTCPUSET_MAX_CPUS];
                 RTCPUID idCpu = 0;
 
+                fAMDVPresent = true;
+
+                /* Query AMD features. */
+                ASMCpuId(0x8000000A, &HWACCMR0Globals.svm.u32Rev, &HWACCMR0Globals.uMaxASID, &u32Dummy, &HWACCMR0Globals.svm.u32Features);
+
                 /* We need to check if AMD-V has been properly initialized on all CPUs. Some BIOSes might do a poor job. */
                 memset(aRc, 0, sizeof(aRc));
                 rc = RTMpOnAll(HWACCMR0InitCPU, (void *)u32VendorEBX, aRc);
@@ -355,13 +372,13 @@ VMMR0DECL(int) HWACCMR0Init(void)
                 if (RT_SUCCESS(rc))
                     rc = hwaccmR0CheckCpuRcArray(aRc, RT_ELEMENTS(aRc), &idCpu);
 
-                AssertMsgRC(rc, ("HWACCMR0InitCPU failed for cpu %d with rc=%d\n", idCpu, rc));
-
+#ifndef DEBUG_bird
+                AssertMsg(rc == VINF_SUCCESS || rc == VERR_SVM_IN_USE, ("HWACCMR0InitCPU failed for cpu %d with rc=%d\n", idCpu, rc));
+#endif
                 if (RT_SUCCESS(rc))
                 {
-                    /* Query AMD features. */
-                    ASMCpuId(0x8000000A, &HWACCMR0Globals.svm.u32Rev, &HWACCMR0Globals.uMaxASID, &u32Dummy, &HWACCMR0Globals.svm.u32Features);
-
+                    /* Read the HWCR msr for diagnostics. */
+                    HWACCMR0Globals.svm.msrHWCR    = ASMRdMsr(MSR_K8_HWCR);
                     HWACCMR0Globals.svm.fSupported = true;
                 }
                 else
@@ -390,7 +407,7 @@ VMMR0DECL(int) HWACCMR0Init(void)
         HWACCMR0Globals.pfnSetupVM          = VMXR0SetupVM;
     }
     else
-    if (HWACCMR0Globals.svm.fSupported)
+    if (fAMDVPresent)
     {
         HWACCMR0Globals.pfnEnterSession     = SVMR0Enter;
         HWACCMR0Globals.pfnLeaveSession     = SVMR0Leave;
@@ -454,6 +471,7 @@ VMMR0DECL(int) HWACCMR0Term(void)
     if (   HWACCMR0Globals.vmx.fSupported
         && HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
     {
+        Assert(HWACCMR0Globals.fGlobalInit);
         rc = SUPR0EnableVTx(false /* fEnable */);
         for (unsigned iCpu = 0; iCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo); iCpu++)
         {
@@ -463,23 +481,32 @@ VMMR0DECL(int) HWACCMR0Term(void)
     }
     else
     {
-        int aRc[RTCPUSET_MAX_CPUS];
-
         Assert(!HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx);
         if (!HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
         {
             rc = RTPowerNotificationDeregister(hwaccmR0PowerCallback, 0);
-            Assert(RT_SUCCESS(rc));
+            AssertRC(rc);
         }
+        else
+            rc = VINF_SUCCESS;
 
-        memset(aRc, 0, sizeof(aRc));
-        rc = RTMpOnAll(hwaccmR0DisableCPU, aRc, NULL);
-        Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+        /* Only disable VT-x/AMD-V on all CPUs if we enabled it before. */
+        if (HWACCMR0Globals.fGlobalInit)
+        {
+            int aRc[RTCPUSET_MAX_CPUS];
+
+            memset(aRc, 0, sizeof(aRc));
+            rc = RTMpOnAll(hwaccmR0DisableCpuCallback, aRc, NULL);
+            Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+#ifdef VBOX_STRICT
+            for (unsigned i=0;i<RT_ELEMENTS(HWACCMR0Globals.aCpuInfo);i++)
+                AssertMsgRC(aRc[i], ("hwaccmR0DisableCpuCallback failed for cpu %d with rc=%d\n", i, aRc[i]));
+#endif
+        }
 
         /* Free the per-cpu pages used for VT-x and AMD-V */
         for (unsigned i=0;i<RT_ELEMENTS(HWACCMR0Globals.aCpuInfo);i++)
         {
-            AssertMsgRC(aRc[i], ("hwaccmR0DisableCPU failed for cpu %d with rc=%d\n", i, aRc[i]));
             if (HWACCMR0Globals.aCpuInfo[i].pMemObj != NIL_RTR0MEMOBJ)
             {
                 RTR0MemObjFree(HWACCMR0Globals.aCpuInfo[i].pMemObj, false);
@@ -541,22 +568,28 @@ static DECLCALLBACK(void) HWACCMR0InitCPU(RTCPUID idCpu, void *pvUser1, void *pv
         {
             /* Turn on SVM in the EFER MSR. */
             val = ASMRdMsr(MSR_K6_EFER);
-            if (!(val & MSR_K6_EFER_SVME))
-                ASMWrMsr(MSR_K6_EFER, val | MSR_K6_EFER_SVME);
-
-            /* Paranoia. */
-            val = ASMRdMsr(MSR_K6_EFER);
             if (val & MSR_K6_EFER_SVME)
             {
-                /* Restore previous value. */
-                ASMWrMsr(MSR_K6_EFER, val & ~MSR_K6_EFER_SVME);
-                paRc[idCpu] = VINF_SUCCESS;
+                paRc[idCpu] = VERR_SVM_IN_USE;
             }
             else
-                paRc[idCpu] = VERR_SVM_ILLEGAL_EFER_MSR;
+            {
+                ASMWrMsr(MSR_K6_EFER, val | MSR_K6_EFER_SVME);
+
+                /* Paranoia. */
+                val = ASMRdMsr(MSR_K6_EFER);
+                if (val & MSR_K6_EFER_SVME)
+                {
+                    /* Restore previous value. */
+                    ASMWrMsr(MSR_K6_EFER, val & ~MSR_K6_EFER_SVME);
+                    paRc[idCpu] = VINF_SUCCESS;
+                }
+                else
+                    paRc[idCpu] = VERR_SVM_ILLEGAL_EFER_MSR;
+            }
         }
         else
-            paRc[idCpu] = HWACCMR0Globals.lLastError = VERR_SVM_DISABLED;
+            paRc[idCpu] = VERR_SVM_DISABLED;
     }
     else
         AssertFailed(); /* can't happen */
@@ -583,6 +616,8 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
     {
         int rc;
 
+        HWACCMR0Globals.fGlobalInit = pVM->hwaccm.s.fGlobalInit;
+
         if (   HWACCMR0Globals.vmx.fSupported
             && HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
         {
@@ -594,6 +629,8 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
                     HWACCMR0Globals.aCpuInfo[iCpu].fConfigured = true;
                     Assert(HWACCMR0Globals.aCpuInfo[iCpu].pMemObj == NIL_RTR0MEMOBJ);
                 }
+                /* If the host provides a VT-x init API, then we'll rely on that for global init. */
+                HWACCMR0Globals.fGlobalInit = pVM->hwaccm.s.fGlobalInit = true;
             }
             else
                 AssertMsgFailed(("HWACCMR0EnableAllCpus/SUPR0EnableVTx: rc=%Rrc\n", rc));
@@ -627,13 +664,18 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
 #endif
                 }
             }
-            /* First time, so initialize each cpu/core */
-            rc = RTMpOnAll(hwaccmR0EnableCPU, (void *)pVM, aRc);
+            if (HWACCMR0Globals.fGlobalInit)
+            {
+                /* First time, so initialize each cpu/core */
+                rc = RTMpOnAll(hwaccmR0EnableCpuCallback, (void *)pVM, aRc);
 
-            /* Check the return code of all invocations. */
-            if (RT_SUCCESS(rc))
-                rc = hwaccmR0CheckCpuRcArray(aRc, RT_ELEMENTS(aRc), &idCpu);
-            AssertMsgRC(rc, ("HWACCMR0EnableAllCpus failed for cpu %d with rc=%d\n", idCpu, rc));
+                /* Check the return code of all invocations. */
+                if (RT_SUCCESS(rc))
+                    rc = hwaccmR0CheckCpuRcArray(aRc, RT_ELEMENTS(aRc), &idCpu);
+                AssertMsgRC(rc, ("HWACCMR0EnableAllCpus failed for cpu %d with rc=%d\n", idCpu, rc));
+            }
+            else
+                rc = VINF_SUCCESS;
         }
 
         return rc;
@@ -642,17 +684,14 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
 }
 
 /**
- * Worker function passed to RTMpOnAll, RTMpOnOthers and RTMpOnSpecific that
- * is to be called on the target cpus.
+ * Disable VT-x or AMD-V on the current CPU
  *
+ * @returns VBox status code.
+ * @param   pVM         VM handle (can be 0!)
  * @param   idCpu       The identifier for the CPU the function is called on.
- * @param   pvUser1     The 1st user argument.
- * @param   pvUser2     The 2nd user argument.
  */
-static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+static int hwaccmR0EnableCpu(PVM pVM, RTCPUID idCpu)
 {
-    PVM             pVM = (PVM)pvUser1;     /* can be NULL! */
-    int            *paRc = (int *)pvUser2;
     void           *pvPageCpu;
     RTHCPHYS        pPageCpuPhys;
     PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
@@ -661,7 +700,7 @@ static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *
     Assert(idCpu == (RTCPUID)RTMpCpuIdToSetIndex(idCpu)); /// @todo fix idCpu == index assumption (rainy day)
     Assert(idCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo));
     Assert(!pCpu->fConfigured);
-    Assert(ASMAtomicReadBool(&pCpu->fInUse) == false);
+    Assert(!HWACCMR0Globals.fGlobalInit || ASMAtomicReadBool(&pCpu->fInUse) == false);
 
     pCpu->idCpu     = idCpu;
 
@@ -675,19 +714,81 @@ static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *
     if (!pCpu->pMemObj)
     {
         AssertFailed();
-        paRc[idCpu] = VERR_INTERNAL_ERROR;
-        return;
+        return VERR_INTERNAL_ERROR;
     }
 
     pvPageCpu    = RTR0MemObjAddress(pCpu->pMemObj);
     pPageCpuPhys = RTR0MemObjGetPagePhysAddr(pCpu->pMemObj, 0);
 
-    paRc[idCpu]  = HWACCMR0Globals.pfnEnableCpu(pCpu, pVM, pvPageCpu, pPageCpuPhys);
-    AssertRC(paRc[idCpu]);
-    if (RT_SUCCESS(paRc[idCpu]))
+    int rc  = HWACCMR0Globals.pfnEnableCpu(pCpu, pVM, pvPageCpu, pPageCpuPhys);
+    AssertRC(rc);
+    if (RT_SUCCESS(rc))
         pCpu->fConfigured = true;
 
-    return;
+    return rc;
+}
+
+
+/**
+ * Worker function passed to RTMpOnAll, RTMpOnOthers and RTMpOnSpecific that
+ * is to be called on the target cpus.
+ *
+ * @param   idCpu       The identifier for the CPU the function is called on.
+ * @param   pvUser1     The 1st user argument.
+ * @param   pvUser2     The 2nd user argument.
+ */
+static DECLCALLBACK(void) hwaccmR0EnableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+{
+    PVM             pVM = (PVM)pvUser1;     /* can be NULL! */
+    int            *paRc = (int *)pvUser2;
+
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        paRc[idCpu] = VINF_SUCCESS;
+        AssertFailed();
+        return;
+    }
+
+    paRc[idCpu] = hwaccmR0EnableCpu(pVM, idCpu);
+}
+
+
+/**
+ * Disable VT-x or AMD-V on the current CPU
+ *
+ * @returns VBox status code.
+ * @param   idCpu       The identifier for the CPU the function is called on.
+ */
+static int hwaccmR0DisableCpu(RTCPUID idCpu)
+{
+    void           *pvPageCpu;
+    RTHCPHYS        pPageCpuPhys;
+    int             rc;
+    PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
+
+    Assert(!HWACCMR0Globals.vmx.fSupported || !HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx);
+    Assert(idCpu == (RTCPUID)RTMpCpuIdToSetIndex(idCpu)); /// @todo fix idCpu == index assumption (rainy day)
+    Assert(idCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo));
+    Assert(!HWACCMR0Globals.fGlobalInit || ASMAtomicReadBool(&pCpu->fInUse) == false);
+    Assert(!pCpu->fConfigured || pCpu->pMemObj);
+
+    if (!pCpu->pMemObj)
+        return (pCpu->fConfigured) ? VERR_NO_MEMORY : VINF_SUCCESS /* not initialized. */;
+
+    pvPageCpu    = RTR0MemObjAddress(pCpu->pMemObj);
+    pPageCpuPhys = RTR0MemObjGetPagePhysAddr(pCpu->pMemObj, 0);
+
+    if (pCpu->fConfigured)
+    {
+        rc = HWACCMR0Globals.pfnDisableCpu(pCpu, pvPageCpu, pPageCpuPhys);
+        AssertRC(rc);
+        pCpu->fConfigured = false;
+    }
+    else
+        rc = VINF_SUCCESS; /* nothing to do */
+
+    pCpu->uCurrentASID = 0;
+    return rc;
 }
 
 /**
@@ -698,36 +799,18 @@ static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *
  * @param   pvUser1     The 1st user argument.
  * @param   pvUser2     The 2nd user argument.
  */
-static DECLCALLBACK(void) hwaccmR0DisableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+static DECLCALLBACK(void) hwaccmR0DisableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2)
 {
-    void           *pvPageCpu;
-    RTHCPHYS        pPageCpuPhys;
     int            *paRc = (int *)pvUser1;
-    PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
 
-    Assert(!HWACCMR0Globals.vmx.fSupported || !HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx);
-    Assert(idCpu == (RTCPUID)RTMpCpuIdToSetIndex(idCpu)); /// @todo fix idCpu == index assumption (rainy day)
-    Assert(idCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo));
-    Assert(ASMAtomicReadBool(&pCpu->fInUse) == false);
-    Assert(!pCpu->fConfigured || pCpu->pMemObj);
-
-    if (!pCpu->pMemObj)
-        return;
-
-    pvPageCpu    = RTR0MemObjAddress(pCpu->pMemObj);
-    pPageCpuPhys = RTR0MemObjGetPagePhysAddr(pCpu->pMemObj, 0);
-
-    if (pCpu->fConfigured)
+    if (!HWACCMR0Globals.fGlobalInit)
     {
-        paRc[idCpu] = HWACCMR0Globals.pfnDisableCpu(pCpu, pvPageCpu, pPageCpuPhys);
-        AssertRC(paRc[idCpu]);
-        pCpu->fConfigured = false;
+        paRc[idCpu] = VINF_SUCCESS;
+        AssertFailed();
+        return;
     }
-    else
-        paRc[idCpu] = VINF_SUCCESS; /* nothing to do */
 
-    pCpu->uCurrentASID = 0;
-    return;
+    paRc[idCpu] = hwaccmR0DisableCpu(idCpu);
 }
 
 /**
@@ -760,13 +843,17 @@ static DECLCALLBACK(void) hwaccmR0PowerCallback(RTPOWEREVENT enmEvent, void *pvU
         memset(aRc, 0, sizeof(aRc));
         if (enmEvent == RTPOWEREVENT_SUSPEND)
         {
-            /* Turn off VT-x or AMD-V on all CPUs. */
-            rc = RTMpOnAll(hwaccmR0DisableCPU, aRc, NULL);
-            Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            if (HWACCMR0Globals.fGlobalInit)
+            {
+                /* Turn off VT-x or AMD-V on all CPUs. */
+                rc = RTMpOnAll(hwaccmR0DisableCpuCallback, aRc, NULL);
+                Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            }
+            /* else nothing to do here for the local init case */
         }
         else
         {
-            /* Reinit the CPUs from scratch as the suspend state has messed with the MSRs. */
+            /* Reinit the CPUs from scratch as the suspend state might have messed with the MSRs. (lousy BIOSes as usual) */
             rc = RTMpOnAll(HWACCMR0InitCPU, (void *)((HWACCMR0Globals.vmx.fSupported) ? X86_CPUID_VENDOR_INTEL_EBX : X86_CPUID_VENDOR_AMD_EBX), aRc);
             Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
 
@@ -777,9 +864,13 @@ static DECLCALLBACK(void) hwaccmR0PowerCallback(RTPOWEREVENT enmEvent, void *pvU
                 SUPR0Printf("hwaccmR0PowerCallback HWACCMR0InitCPU failed with %d\n", rc);
 #endif
 
-            /* Turn VT-x or AMD-V back on on all CPUs. */
-            rc = RTMpOnAll(hwaccmR0EnableCPU, NULL, aRc);
-            Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            if (HWACCMR0Globals.fGlobalInit)
+            {
+                /* Turn VT-x or AMD-V back on on all CPUs. */
+                rc = RTMpOnAll(hwaccmR0EnableCpuCallback, NULL, aRc);
+                Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            }
+            /* else nothing to do here for the local init case */
         }
     }
     if (enmEvent == RTPOWEREVENT_RESUME)
@@ -815,6 +906,7 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
 
     pVM->hwaccm.s.vmx.msr.feature_ctrl      = HWACCMR0Globals.vmx.msr.feature_ctrl;
     pVM->hwaccm.s.vmx.hostCR4               = HWACCMR0Globals.vmx.hostCR4;
+    pVM->hwaccm.s.vmx.hostEFER              = HWACCMR0Globals.vmx.hostEFER;
     pVM->hwaccm.s.vmx.msr.vmx_basic_info    = HWACCMR0Globals.vmx.msr.vmx_basic_info;
     pVM->hwaccm.s.vmx.msr.vmx_pin_ctls      = HWACCMR0Globals.vmx.msr.vmx_pin_ctls;
     pVM->hwaccm.s.vmx.msr.vmx_proc_ctls     = HWACCMR0Globals.vmx.msr.vmx_proc_ctls;
@@ -828,6 +920,7 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
     pVM->hwaccm.s.vmx.msr.vmx_cr4_fixed1    = HWACCMR0Globals.vmx.msr.vmx_cr4_fixed1;
     pVM->hwaccm.s.vmx.msr.vmx_vmcs_enum     = HWACCMR0Globals.vmx.msr.vmx_vmcs_enum;
     pVM->hwaccm.s.vmx.msr.vmx_eptcaps       = HWACCMR0Globals.vmx.msr.vmx_eptcaps;
+    pVM->hwaccm.s.svm.msrHWCR               = HWACCMR0Globals.svm.msrHWCR;
     pVM->hwaccm.s.svm.u32Rev                = HWACCMR0Globals.svm.u32Rev;
     pVM->hwaccm.s.svm.u32Features           = HWACCMR0Globals.svm.u32Features;
     pVM->hwaccm.s.cpuid.u32AMDFeatureECX    = HWACCMR0Globals.cpuid.u32AMDFeatureECX;
@@ -846,7 +939,7 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
 #endif
     }
 
-    for (unsigned i=0;i<pVM->cCPUs;i++)
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         PVMCPU pVCpu = &pVM->aCpus[i];
 
@@ -931,15 +1024,29 @@ VMMR0DECL(int) HWACCMR0SetupVM(PVM pVM)
 
     ASMAtomicWriteBool(&pCpu->fInUse, true);
 
-    for (unsigned i=0;i<pVM->cCPUs;i++)
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         /* On first entry we'll sync everything. */
         pVM->aCpus[i].hwaccm.s.fContextUseFlags = HWACCM_CHANGED_ALL;
     }
 
+    /* Enable VT-x or AMD-V if local init is required. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0EnableCpu(pVM, idCpu);
+        AssertRCReturn(rc, rc);
+    }
+
     /* Setup VT-x or AMD-V. */
     rc = HWACCMR0Globals.pfnSetupVM(pVM);
 
+    /* Disable VT-x or AMD-V if local init was done before. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0DisableCpu(idCpu);
+        AssertRC(rc);
+    }
+
     ASMAtomicWriteBool(&pCpu->fInUse, false);
 
     return rc;
@@ -984,6 +1091,17 @@ VMMR0DECL(int) HWACCMR0Enter(PVM pVM, PVMCPU pVCpu)
     else
         pVM->hwaccm.s.u64RegisterMask = UINT64_C(0xFFFFFFFF);
 
+    /* Enable VT-x or AMD-V if local init is required. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0EnableCpu(pVM, idCpu);
+        AssertRCReturn(rc, rc);
+    }
+
+#ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
+    bool fStartedSet = PGMDynMapStartOrMigrateAutoSet(pVCpu);
+#endif
+
     rc  = HWACCMR0Globals.pfnEnterSession(pVM, pVCpu, pCpu);
     AssertRC(rc);
     /* We must save the host context here (VT-x) as we might be rescheduled on a different cpu after a long jump back to ring 3. */
@@ -992,14 +1110,13 @@ VMMR0DECL(int) HWACCMR0Enter(PVM pVM, PVMCPU pVCpu)
     rc |= HWACCMR0Globals.pfnLoadGuestState(pVM, pVCpu, pCtx);
     AssertRC(rc);
 
-    /* keep track of the CPU owning the VMCS for debugging scheduling weirdness and ring-3 calls. */
-    if (RT_SUCCESS(rc))
-    {
 #ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
-        PGMDynMapMigrateAutoSet(pVCpu);
+    if (fStartedSet)
+        PGMDynMapReleaseAutoSet(pVCpu);
 #endif
-    }
-    else
+
+    /* keep track of the CPU owning the VMCS for debugging scheduling weirdness and ring-3 calls. */
+    if (RT_FAILURE(rc))
         pVCpu->hwaccm.s.idEnteredCpu = NIL_RTCPUID;
     return rc;
 }
@@ -1050,6 +1167,18 @@ VMMR0DECL(int) HWACCMR0Leave(PVM pVM, PVMCPU pVCpu)
 #endif
     pVCpu->hwaccm.s.idEnteredCpu = NIL_RTCPUID;
 
+    /* Disable VT-x or AMD-V if local init was done before. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0DisableCpu(idCpu);
+        AssertRC(rc);
+
+        /* Reset these to force a TLB flush for the next entry. (-> EXPENSIVE) */
+        pVCpu->hwaccm.s.idLastCpu    = NIL_RTCPUID;
+        pVCpu->hwaccm.s.uCurrentASID = 0;
+        VMCPU_FF_SET(pVCpu, VMCPU_FF_TLB_FLUSH);
+    }
+
     ASMAtomicWriteBool(&pCpu->fInUse, false);
     return rc;
 }
@@ -1064,9 +1193,9 @@ VMMR0DECL(int) HWACCMR0Leave(PVM pVM, PVMCPU pVCpu)
 VMMR0DECL(int) HWACCMR0RunGuestCode(PVM pVM, PVMCPU pVCpu)
 {
     CPUMCTX *pCtx;
-    RTCPUID  idCpu = RTMpCpuId(); NOREF(idCpu);
     int      rc;
 #ifdef VBOX_STRICT
+    RTCPUID  idCpu = RTMpCpuId(); NOREF(idCpu);
     PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
 #endif
 
@@ -1195,7 +1324,7 @@ VMMR0DECL(PVMCPU)  HWACCMR0GetVMCPU(PVM pVM)
     RTCPUID idHostCpu = RTMpCpuId();
 
     /** @todo optimize for large number of VCPUs when that becomes more common. */
-    for (unsigned idCpu=0;idCpu<pVM->cCPUs;idCpu++)
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
     {
         PVMCPU pVCpu = &pVM->aCpus[idCpu];
 
@@ -1274,7 +1403,8 @@ VMMR0DECL(int) HWACCMR0EnterSwitcher(PVM pVM, bool *pfVTxDisabled)
     *pfVTxDisabled = false;
 
     if (    HWACCMR0Globals.enmHwAccmState != HWACCMSTATE_ENABLED
-        ||  !HWACCMR0Globals.vmx.fSupported /* no such issues with AMD-V */)
+        ||  !HWACCMR0Globals.vmx.fSupported /* no such issues with AMD-V */
+        ||  !HWACCMR0Globals.fGlobalInit    /* Local init implies the CPU is currently not in VMX root mode. */)
         return VINF_SUCCESS;    /* nothing to do */
 
     switch(VMMGetSwitcher(pVM))
@@ -1321,7 +1451,8 @@ VMMR0DECL(int) HWACCMR0LeaveSwitcher(PVM pVM, bool fVTxDisabled)
         return VINF_SUCCESS;    /* nothing to do */
 
     Assert(   HWACCMR0Globals.enmHwAccmState == HWACCMSTATE_ENABLED
-           && HWACCMR0Globals.vmx.fSupported);
+           && HWACCMR0Globals.vmx.fSupported
+           && HWACCMR0Globals.fGlobalInit);
 
     PHWACCM_CPUINFO pCpu = HWACCMR0GetCurrentCpu();
     void           *pvPageCpu;
@@ -1577,11 +1708,11 @@ VMMR0DECL(void) HWACCMDumpRegs(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx)
 
     Log(("FPU:\n"
         "FCW=%04x FSW=%04x FTW=%02x\n"
-        "res1=%02x FOP=%04x FPUIP=%08x CS=%04x Rsvrd1=%04x\n"
+        "FOP=%04x FPUIP=%08x CS=%04x Rsvrd1=%04x\n"
         "FPUDP=%04x DS=%04x Rsvrd2=%04x MXCSR=%08x MXCSR_MASK=%08x\n"
         ,
         pCtx->fpu.FCW, pCtx->fpu.FSW, pCtx->fpu.FTW,
-        pCtx->fpu.huh1, pCtx->fpu.FOP, pCtx->fpu.FPUIP, pCtx->fpu.CS, pCtx->fpu.Rsvrd1,
+        pCtx->fpu.FOP, pCtx->fpu.FPUIP, pCtx->fpu.CS, pCtx->fpu.Rsvrd1,
         pCtx->fpu.FPUDP, pCtx->fpu.DS, pCtx->fpu.Rsrvd2,
         pCtx->fpu.MXCSR, pCtx->fpu.MXCSR_MASK));
 
