@@ -464,6 +464,9 @@ static void vmmR0RecordRC(PVM pVM, PVMCPU pVCpu, int rc)
         case VINF_PGM_CHANGE_MODE:
             STAM_COUNTER_INC(&pVM->vmm.s.StatRZRetPGMChangeMode);
             break;
+        case VINF_PGM_POOL_FLUSH_PENDING:
+            STAM_COUNTER_INC(&pVM->vmm.s.StatRZRetPGMFlushPending);
+            break;
         case VINF_EM_PENDING_REQUEST:
             STAM_COUNTER_INC(&pVM->vmm.s.StatRZRetPendingRequest);
             break;
@@ -511,7 +514,7 @@ VMMR0DECL(int) VMMR0EntryInt(PVM pVM, VMMR0OPERATION enmOperation, void *pvArg)
  */
 VMMR0DECL(void) VMMR0EntryFast(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperation)
 {
-    if (RT_UNLIKELY(idCpu >= pVM->cCPUs))
+    if (RT_UNLIKELY(idCpu >= pVM->cCpus))
         return;
     PVMCPU pVCpu = &pVM->aCpus[idCpu];
 
@@ -530,9 +533,10 @@ VMMR0DECL(void) VMMR0EntryFast(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperati
                 int         rc;
                 bool        fVTxDisabled;
 
-                if (RT_UNLIKELY(pVM->cCPUs > 1))
+                if (RT_UNLIKELY(pVM->cCpus > 1))
                 {
                     pVCpu->vmm.s.iLastGZRc = VERR_RAW_MODE_INVALID_SMP;
+                    ASMSetFlags(uFlags);
                     return;
                 }
 
@@ -540,6 +544,7 @@ VMMR0DECL(void) VMMR0EntryFast(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperati
                 if (RT_UNLIKELY(!PGMGetHyperCR3(pVCpu)))
                 {
                     pVCpu->vmm.s.iLastGZRc = VERR_PGM_NO_CR3_SHADOW_ROOT;
+                    ASMSetFlags(uFlags);
                     return;
                 }
 #endif
@@ -549,6 +554,7 @@ VMMR0DECL(void) VMMR0EntryFast(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperati
                 if (RT_FAILURE(rc))
                 {
                     pVCpu->vmm.s.iLastGZRc = rc;
+                    ASMSetFlags(uFlags);
                     return;
                 }
 
@@ -727,9 +733,9 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
             return VERR_INVALID_POINTER;
         }
 
-        if (RT_UNLIKELY(idCpu >= pVM->cCPUs && idCpu != NIL_VMCPUID))
+        if (RT_UNLIKELY(idCpu >= pVM->cCpus && idCpu != NIL_VMCPUID))
         {
-            SUPR0Printf("vmmR0EntryExWorker: Invalid idCpu (%u vs cCPUs=%u)\n", idCpu, pVM->cCPUs);
+            SUPR0Printf("vmmR0EntryExWorker: Invalid idCpu (%u vs cCpus=%u)\n", idCpu, pVM->cCpus);
             return VERR_INVALID_PARAMETER;
         }
     }
@@ -817,7 +823,7 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
             return HWACCMR0EnableAllCpus(pVM);
 
         /*
-         * Setup the hardware accelerated raw-mode session.
+         * Setup the hardware accelerated session.
          */
         case VMMR0_DO_HWACC_SETUP_VM:
         {
@@ -1089,7 +1095,7 @@ VMMR0DECL(int) VMMR0EntryEx(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperation,
      */
     if (    VALID_PTR(pVM)
         &&  pVM->pVMR0
-        &&  idCpu < pVM->cCPUs)
+        &&  idCpu < pVM->cCpus)
     {
         switch (enmOperation)
         {
@@ -1260,16 +1266,19 @@ DECLEXPORT(bool) RTCALL RTAssertShouldPanic(void)
     {
         PVMCPU pVCpu = VMMGetCpu(pVM);
 
-#ifdef RT_ARCH_X86
-        if (    pVCpu->vmm.s.CallRing3JmpBufR0.eip
-            &&  !pVCpu->vmm.s.CallRing3JmpBufR0.fInRing3Call)
-#else
-        if (    pVCpu->vmm.s.CallRing3JmpBufR0.rip
-            &&  !pVCpu->vmm.s.CallRing3JmpBufR0.fInRing3Call)
-#endif
+        if (pVCpu)
         {
-            int rc = VMMRZCallRing3(pVM, pVCpu, VMMCALLRING3_VM_R0_ASSERTION, 0);
-            return RT_FAILURE_NP(rc);
+#ifdef RT_ARCH_X86
+            if (    pVCpu->vmm.s.CallRing3JmpBufR0.eip
+                &&  !pVCpu->vmm.s.CallRing3JmpBufR0.fInRing3Call)
+#else
+            if (    pVCpu->vmm.s.CallRing3JmpBufR0.rip
+                &&  !pVCpu->vmm.s.CallRing3JmpBufR0.fInRing3Call)
+#endif
+            {
+                int rc = VMMRZCallRing3(pVM, pVCpu, VMMCALLRING3_VM_R0_ASSERTION, 0);
+                return RT_FAILURE_NP(rc);
+            }
         }
     }
 #ifdef RT_OS_LINUX
