@@ -51,7 +51,8 @@
 static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
 {
     NULL, /* invalid entry */
-#ifndef RT_ARCH_AMD64
+#ifdef VBOX_WITH_RAW_MODE
+# ifndef RT_ARCH_AMD64
     &vmmR3Switcher32BitTo32Bit_Def,
     &vmmR3Switcher32BitToPAE_Def,
     &vmmR3Switcher32BitToAMD64_Def,
@@ -59,13 +60,13 @@ static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
     &vmmR3SwitcherPAEToPAE_Def,
     &vmmR3SwitcherPAEToAMD64_Def,
     NULL,   //&vmmR3SwitcherPAETo32Bit_Def,
-# ifdef VBOX_WITH_HYBRID_32BIT_KERNEL
+#  ifdef VBOX_WITH_HYBRID_32BIT_KERNEL
     &vmmR3SwitcherAMD64ToPAE_Def,
-# else
+#  else
     NULL,   //&vmmR3SwitcherAMD64ToPAE_Def,
-# endif
+#  endif
     NULL    //&vmmR3SwitcherAMD64ToAMD64_Def,
-#else  /* RT_ARCH_AMD64 */
+# else  /* RT_ARCH_AMD64 */
     NULL,   //&vmmR3Switcher32BitTo32Bit_Def,
     NULL,   //&vmmR3Switcher32BitToPAE_Def,
     NULL,   //&vmmR3Switcher32BitToAMD64_Def,
@@ -75,7 +76,18 @@ static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
     &vmmR3SwitcherAMD64To32Bit_Def,
     &vmmR3SwitcherAMD64ToPAE_Def,
     NULL    //&vmmR3SwitcherAMD64ToAMD64_Def,
-#endif /* RT_ARCH_AMD64 */
+# endif /* RT_ARCH_AMD64 */
+#else  /* !VBOX_WITH_RAW_MODE */
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+#endif /* !VBOX_WITH_RAW_MODE */
 };
 
 
@@ -90,6 +102,9 @@ static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
  */
 int vmmR3SwitcherInit(PVM pVM)
 {
+#ifndef VBOX_WITH_RAW_MODE
+    return VINF_SUCCESS;
+#else
     /*
      * Calc the size.
      */
@@ -210,6 +225,7 @@ int vmmR3SwitcherInit(PVM pVM)
     pVM->vmm.s.pvCoreCodeR0 = NIL_RTR0PTR;
     pVM->vmm.s.pvCoreCodeRC = 0;
     return rc;
+#endif
 }
 
 /**
@@ -220,6 +236,7 @@ int vmmR3SwitcherInit(PVM pVM)
  */
 void vmmR3SwitcherRelocate(PVM pVM, RTGCINTPTR offDelta)
 {
+#ifdef VBOX_WITH_RAW_MODE
     /*
      * Relocate all the switchers.
      */
@@ -250,6 +267,7 @@ void vmmR3SwitcherRelocate(PVM pVM, RTGCINTPTR offDelta)
     pVM->pfnVMMGCGuestToHostAsmGuestCtx = RCPtr + pSwitcher->offGCGuestToHostAsmGuestCtx;
 
 //    AssertFailed();
+#endif
 }
 
 
@@ -776,7 +794,7 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
             else
             {
                 RTLogPrintf("  %04x: %02x '%c' (nowhere)\n",
-                            offCode, pu8CodeR3[offCode], isprint(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
+                            offCode, pu8CodeR3[offCode], RT_C_IS_PRINT(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
                 offCode++;
                 continue;
             }
@@ -813,7 +831,7 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
                 else
                 {
                     RTLogPrintf("  %04x: %02x '%c'\n",
-                                offCode, pu8CodeR3[offCode], isprint(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
+                                offCode, pu8CodeR3[offCode], RT_C_IS_PRINT(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
                     cbInstr = 1;
                 }
                 offCode += cbInstr;
