@@ -37,6 +37,9 @@
 #include <iprt/stream.h>
 #include <iprt/string.h>
 
+/* does not work for more CPUs as SUPR3LowAlloc() would refuse to allocate more pages */
+#define NUM_CPUS  16
+
 
 int main(int argc, char **argv)
 {
@@ -51,10 +54,10 @@ int main(int argc, char **argv)
      */
     PVM         pVM;
     RTR0PTR     pvR0;
-    SUPPAGE     aPages[RT_ALIGN_Z(sizeof(*pVM), PAGE_SIZE) >> PAGE_SHIFT];
+    SUPPAGE     aPages[RT_ALIGN_Z(sizeof(*pVM) + NUM_CPUS * sizeof(VMCPU), PAGE_SIZE) >> PAGE_SHIFT];
     int rc = SUPR3Init(NULL);
     if (RT_SUCCESS(rc))
-        rc = SUPLowAlloc(RT_ELEMENTS(aPages), (void **)&pVM, &pvR0, &aPages[0]);
+        rc = SUPR3LowAlloc(RT_ELEMENTS(aPages), (void **)&pVM, &pvR0, &aPages[0]);
     if (RT_FAILURE(rc))
     {
         RTPrintf("Fatal error: SUP Failure! rc=%Rrc\n", rc);
@@ -69,6 +72,9 @@ int main(int argc, char **argv)
     pUVM->pVM = pVM;
     pVM->pUVM = pUVM;
 
+    pVM->cCpus = NUM_CPUS;
+    pVM->cbSelf = RT_UOFFSETOF(VM, aCpus[pVM->cCpus]);
+
     rc = STAMR3InitUVM(pUVM);
     if (RT_FAILURE(rc))
     {
