@@ -31,9 +31,9 @@
 
 #ifdef VBOX
 # ifndef LOG_GROUP
-#  include <VBox/log.h>
 #  define LOG_GROUP LOG_GROUP_REM
 # endif
+# include <VBox/log.h>
 # include <VBox/pgm.h> /* PGM_DYNAMIC_RAM_ALLOC */
 #endif
 
@@ -94,30 +94,54 @@ static inline void tswap64s(uint64_t *s)
 
 #else
 
+#ifndef VBOX
 static inline uint16_t tswap16(uint16_t s)
+#else
+DECLINLINE(uint16_t) tswap16(uint16_t s)
+#endif
 {
     return s;
 }
 
+#ifndef VBOX
 static inline uint32_t tswap32(uint32_t s)
+#else
+DECLINLINE(uint32_t) tswap32(uint32_t s)
+#endif
 {
     return s;
 }
 
+#ifndef VBOX
 static inline uint64_t tswap64(uint64_t s)
+#else
+DECLINLINE(uint64_t) tswap64(uint64_t s)
+#endif
 {
     return s;
 }
 
+#ifndef VBOX
 static inline void tswap16s(uint16_t *s)
+#else
+DECLINLINE(void) tswap16s(uint16_t *s)
+#endif
 {
 }
 
+#ifndef VBOX
 static inline void tswap32s(uint32_t *s)
+#else
+DECLINLINE(void) tswap32s(uint32_t *s)
+#endif
 {
 }
 
+#ifndef VBOX
 static inline void tswap64s(uint64_t *s)
+#else
+DECLINLINE(void) tswap64s(uint64_t *s)
+#endif
 {
 }
 
@@ -133,6 +157,11 @@ static inline void tswap64s(uint64_t *s)
 #define bswaptls(s) bswap64s(s)
 #endif
 
+typedef union {
+    float32 f;
+    uint32_t l;
+} CPU_FloatU;
+
 /* NOTE: arm FPA is horrible as double 32 bit words are stored in big
    endian ! */
 typedef union {
@@ -152,6 +181,36 @@ typedef union {
     uint64_t ll;
 } CPU_DoubleU;
 
+#ifdef TARGET_SPARC
+typedef union {
+    float128 q;
+#if defined(WORDS_BIGENDIAN) \
+    || (defined(__arm__) && !defined(__VFP_FP__) && !defined(CONFIG_SOFTFLOAT))
+    struct {
+        uint32_t upmost;
+        uint32_t upper;
+        uint32_t lower;
+        uint32_t lowest;
+    } l;
+    struct {
+        uint64_t upper;
+        uint64_t lower;
+    } ll;
+#else
+    struct {
+        uint32_t lowest;
+        uint32_t lower;
+        uint32_t upper;
+        uint32_t upmost;
+    } l;
+    struct {
+        uint64_t lower;
+        uint64_t upper;
+    } ll;
+#endif
+} CPU_QuadU;
+#endif
+
 /* CPU memory access without any memory or io remapping */
 
 /*
@@ -187,87 +246,88 @@ typedef union {
  *   user   : user mode access using soft MMU
  *   kernel : kernel mode access using soft MMU
  */
+
 #ifdef VBOX
+void        remAbort(int rc, const char *pszTip) __attribute__((__noreturn__));
 
-void     remR3PhysRead(RTGCPHYS SrcGCPhys, void *pvDst, unsigned cb);
-uint8_t  remR3PhysReadU8(RTGCPHYS SrcGCPhys);
-int8_t   remR3PhysReadS8(RTGCPHYS SrcGCPhys);
-uint16_t remR3PhysReadU16(RTGCPHYS SrcGCPhys);
-int16_t  remR3PhysReadS16(RTGCPHYS SrcGCPhys);
-uint32_t remR3PhysReadU32(RTGCPHYS SrcGCPhys);
-int32_t  remR3PhysReadS32(RTGCPHYS SrcGCPhys);
-uint64_t remR3PhysReadU64(RTGCPHYS SrcGCPhys);
-int64_t  remR3PhysReadS64(RTGCPHYS SrcGCPhys);
-void     remR3PhysWrite(RTGCPHYS DstGCPhys, const void *pvSrc, unsigned cb);
-void     remR3PhysWriteU8(RTGCPHYS DstGCPhys, uint8_t val);
-void     remR3PhysWriteU16(RTGCPHYS DstGCPhys, uint16_t val);
-void     remR3PhysWriteU32(RTGCPHYS DstGCPhys, uint32_t val);
-void     remR3PhysWriteU64(RTGCPHYS DstGCPhys, uint64_t val);
+void        remR3PhysRead(RTGCPHYS SrcGCPhys, void *pvDst, unsigned cb);
+RTCCUINTREG remR3PhysReadU8(RTGCPHYS SrcGCPhys);
+RTCCINTREG  remR3PhysReadS8(RTGCPHYS SrcGCPhys);
+RTCCUINTREG remR3PhysReadU16(RTGCPHYS SrcGCPhys);
+RTCCINTREG  remR3PhysReadS16(RTGCPHYS SrcGCPhys);
+RTCCUINTREG remR3PhysReadU32(RTGCPHYS SrcGCPhys);
+RTCCINTREG  remR3PhysReadS32(RTGCPHYS SrcGCPhys);
+uint64_t    remR3PhysReadU64(RTGCPHYS SrcGCPhys);
+int64_t     remR3PhysReadS64(RTGCPHYS SrcGCPhys);
+void        remR3PhysWrite(RTGCPHYS DstGCPhys, const void *pvSrc, unsigned cb);
+void        remR3PhysWriteU8(RTGCPHYS DstGCPhys, uint8_t val);
+void        remR3PhysWriteU16(RTGCPHYS DstGCPhys, uint16_t val);
+void        remR3PhysWriteU32(RTGCPHYS DstGCPhys, uint32_t val);
+void        remR3PhysWriteU64(RTGCPHYS DstGCPhys, uint64_t val);
 
-#ifndef VBOX_WITH_NEW_PHYS_CODE
-void     remR3GrowDynRange(unsigned long physaddr);
-#endif
-#if 0 /*defined(RT_ARCH_AMD64) && defined(VBOX_STRICT)*/
-# define VBOX_CHECK_ADDR(ptr) do { if ((uintptr_t)(ptr) >= _4G) __asm__("int3"); } while (0)
-#else
-# define VBOX_CHECK_ADDR(ptr) do { } while (0)
+#ifndef REM_PHYS_ADDR_IN_TLB
+void       *remR3TlbGCPhys2Ptr(CPUState *env1, target_ulong physAddr, int fWritable);
 #endif
 
-static inline int ldub_p(void *ptr)
+#endif /* VBOX */
+
+#if defined(VBOX) && defined(REM_PHYS_ADDR_IN_TLB)
+
+DECLINLINE(uint8_t) ldub_p(void *ptr)
 {
     VBOX_CHECK_ADDR(ptr);
     return remR3PhysReadU8((uintptr_t)ptr);
 }
 
-static inline int ldsb_p(void *ptr)
+DECLINLINE(int8_t) ldsb_p(void *ptr)
 {
     VBOX_CHECK_ADDR(ptr);
     return remR3PhysReadS8((uintptr_t)ptr);
 }
 
-static inline void stb_p(void *ptr, int v)
+DECLINLINE(void) stb_p(void *ptr, int v)
 {
     VBOX_CHECK_ADDR(ptr);
     remR3PhysWriteU8((uintptr_t)ptr, v);
 }
 
-static inline int lduw_le_p(void *ptr)
+DECLINLINE(uint32_t) lduw_le_p(void *ptr)
 {
     VBOX_CHECK_ADDR(ptr);
     return remR3PhysReadU16((uintptr_t)ptr);
 }
 
-static inline int ldsw_le_p(void *ptr)
+DECLINLINE(int32_t) ldsw_le_p(void *ptr)
 {
     VBOX_CHECK_ADDR(ptr);
     return remR3PhysReadS16((uintptr_t)ptr);
 }
 
-static inline void stw_le_p(void *ptr, int v)
+DECLINLINE(void) stw_le_p(void *ptr, int v)
 {
     VBOX_CHECK_ADDR(ptr);
     remR3PhysWriteU16((uintptr_t)ptr, v);
 }
 
-static inline int ldl_le_p(void *ptr)
+DECLINLINE(uint32_t) ldl_le_p(void *ptr)
 {
     VBOX_CHECK_ADDR(ptr);
     return remR3PhysReadU32((uintptr_t)ptr);
 }
 
-static inline void stl_le_p(void *ptr, int v)
+DECLINLINE(void) stl_le_p(void *ptr, int v)
 {
     VBOX_CHECK_ADDR(ptr);
     remR3PhysWriteU32((uintptr_t)ptr, v);
 }
 
-static inline void stq_le_p(void *ptr, uint64_t v)
+DECLINLINE(void) stq_le_p(void *ptr, uint64_t v)
 {
     VBOX_CHECK_ADDR(ptr);
     remR3PhysWriteU64((uintptr_t)ptr, v);
 }
 
-static inline uint64_t ldq_le_p(void *ptr)
+DECLINLINE(uint64_t) ldq_le_p(void *ptr)
 {
     VBOX_CHECK_ADDR(ptr);
     return remR3PhysReadU64((uintptr_t)ptr);
@@ -277,7 +337,7 @@ static inline uint64_t ldq_le_p(void *ptr)
 
 /* float access */
 
-static inline float32 ldfl_le_p(void *ptr)
+DECLINLINE(float32) ldfl_le_p(void *ptr)
 {
     union {
         float32 f;
@@ -287,7 +347,7 @@ static inline float32 ldfl_le_p(void *ptr)
     return u.f;
 }
 
-static inline void stfl_le_p(void *ptr, float32 v)
+DECLINLINE(void) stfl_le_p(void *ptr, float32 v)
 {
     union {
         float32 f;
@@ -297,23 +357,23 @@ static inline void stfl_le_p(void *ptr, float32 v)
     stl_le_p(ptr, u.i);
 }
 
-static inline float64 ldfq_le_p(void *ptr)
+DECLINLINE(float64) ldfq_le_p(void *ptr)
 {
     CPU_DoubleU u;
     u.l.lower = ldl_le_p(ptr);
-    u.l.upper = ldl_le_p(ptr + 4);
+    u.l.upper = ldl_le_p((uint8_t*)ptr + 4);
     return u.d;
 }
 
-static inline void stfq_le_p(void *ptr, float64 v)
+DECLINLINE(void) stfq_le_p(void *ptr, float64 v)
 {
     CPU_DoubleU u;
     u.d = v;
     stl_le_p(ptr, u.l.lower);
-    stl_le_p(ptr + 4, u.l.upper);
+    stl_le_p((uint8_t*)ptr + 4, u.l.upper);
 }
 
-#else  /* !VBOX */
+#else  /* !(VBOX && REM_PHYS_ADDR_IN_TLB) */
 
 static inline int ldub_p(void *ptr)
 {
@@ -513,6 +573,7 @@ static inline void stfq_le_p(void *ptr, float64 v)
 
 #if !defined(WORDS_BIGENDIAN) || defined(WORDS_ALIGNED)
 
+#ifndef VBOX
 static inline int lduw_be_p(void *ptr)
 {
 #if defined(__i386__)
@@ -527,7 +588,24 @@ static inline int lduw_be_p(void *ptr)
     return ((b[0] << 8) | b[1]);
 #endif
 }
+#else /* VBOX */
+DECLINLINE(int) lduw_be_p(void *ptr)
+{
+#if defined(__i386__) && !defined(_MSC_VER)
+    int val;
+    asm volatile ("movzwl %1, %0\n"
+                  "xchgb %b0, %h0\n"
+                  : "=q" (val)
+                  : "m" (*(uint16_t *)ptr));
+    return val;
+#else
+    uint8_t *b = (uint8_t *) ptr;
+    return ((b[0] << 8) | b[1]);
+#endif
+}
+#endif
 
+#ifndef VBOX
 static inline int ldsw_be_p(void *ptr)
 {
 #if defined(__i386__)
@@ -542,7 +620,24 @@ static inline int ldsw_be_p(void *ptr)
     return (int16_t)((b[0] << 8) | b[1]);
 #endif
 }
+#else
+DECLINLINE(int) ldsw_be_p(void *ptr)
+{
+#if defined(__i386__) && !defined(_MSC_VER)
+    int val;
+    asm volatile ("movzwl %1, %0\n"
+                  "xchgb %b0, %h0\n"
+                  : "=q" (val)
+                  : "m" (*(uint16_t *)ptr));
+    return (int16_t)val;
+#else
+    uint8_t *b = (uint8_t *) ptr;
+    return (int16_t)((b[0] << 8) | b[1]);
+#endif
+}
+#endif
 
+#ifndef VBOX
 static inline int ldl_be_p(void *ptr)
 {
 #if defined(__i386__) || defined(__x86_64__)
@@ -557,15 +652,36 @@ static inline int ldl_be_p(void *ptr)
     return (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];
 #endif
 }
+#else
+DECLINLINE(int) ldl_be_p(void *ptr)
+{
+#if (defined(__i386__) || defined(__x86_64__)) && !defined(_MSC_VER)
+    int val;
+    asm volatile ("movl %1, %0\n"
+                  "bswap %0\n"
+                  : "=r" (val)
+                  : "m" (*(uint32_t *)ptr));
+    return val;
+#else
+    uint8_t *b = (uint8_t *) ptr;
+    return (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];
+#endif
+}
+#endif
 
+#ifndef VBOX
 static inline uint64_t ldq_be_p(void *ptr)
+#else
+DECLINLINE(uint64_t) ldq_be_p(void *ptr)
+#endif
 {
     uint32_t a,b;
     a = ldl_be_p(ptr);
-    b = ldl_be_p(ptr+4);
+    b = ldl_be_p((uint8_t*)ptr+4);
     return (((uint64_t)a<<32)|b);
 }
 
+#ifndef VBOX
 static inline void stw_be_p(void *ptr, int v)
 {
 #if defined(__i386__)
@@ -579,7 +695,24 @@ static inline void stw_be_p(void *ptr, int v)
     d[1] = v;
 #endif
 }
+#else
+DECLINLINE(void) stw_be_p(void *ptr, int v)
+{
+#if defined(__i386__) && !defined(_MSC_VER)
+    asm volatile ("xchgb %b0, %h0\n"
+                  "movw %w0, %1\n"
+                  : "=q" (v)
+                  : "m" (*(uint16_t *)ptr), "0" (v));
+#else
+    uint8_t *d = (uint8_t *) ptr;
+    d[0] = v >> 8;
+    d[1] = v;
+#endif
+}
 
+#endif /* VBOX */
+
+#ifndef VBOX
 static inline void stl_be_p(void *ptr, int v)
 {
 #if defined(__i386__) || defined(__x86_64__)
@@ -595,16 +728,40 @@ static inline void stl_be_p(void *ptr, int v)
     d[3] = v;
 #endif
 }
+#else
+DECLINLINE(void) stl_be_p(void *ptr, int v)
+{
+#if !defined(_MSC_VER) && (defined(__i386__) || defined(__x86_64__))
+    asm volatile ("bswap %0\n"
+                  "movl %0, %1\n"
+                  : "=r" (v)
+                  : "m" (*(uint32_t *)ptr), "0" (v));
+#else
+    uint8_t *d = (uint8_t *) ptr;
+    d[0] = v >> 24;
+    d[1] = v >> 16;
+    d[2] = v >> 8;
+    d[3] = v;
+#endif
+}
+#endif /* VBOX */
 
+#ifndef VBOX
 static inline void stq_be_p(void *ptr, uint64_t v)
+#else
+DECLINLINE(void) stq_be_p(void *ptr, uint64_t v)
+#endif
 {
     stl_be_p(ptr, v >> 32);
-    stl_be_p(ptr + 4, v);
+    stl_be_p((uint8_t*)ptr + 4, v);
 }
 
 /* float access */
-
+#ifndef VBOX
 static inline float32 ldfl_be_p(void *ptr)
+#else
+DECLINLINE(float32) ldfl_be_p(void *ptr)
+#endif
 {
     union {
         float32 f;
@@ -614,7 +771,11 @@ static inline float32 ldfl_be_p(void *ptr)
     return u.f;
 }
 
+#ifndef VBOX
 static inline void stfl_be_p(void *ptr, float32 v)
+#else
+DECLINLINE(void) stfl_be_p(void *ptr, float32 v)
+#endif
 {
     union {
         float32 f;
@@ -624,20 +785,28 @@ static inline void stfl_be_p(void *ptr, float32 v)
     stl_be_p(ptr, u.i);
 }
 
+#ifndef VBOX
 static inline float64 ldfq_be_p(void *ptr)
+#else
+DECLINLINE(float64) ldfq_be_p(void *ptr)
+#endif
 {
     CPU_DoubleU u;
     u.l.upper = ldl_be_p(ptr);
-    u.l.lower = ldl_be_p(ptr + 4);
+    u.l.lower = ldl_be_p((uint8_t*)ptr + 4);
     return u.d;
 }
 
+#ifndef VBOX
 static inline void stfq_be_p(void *ptr, float64 v)
+#else
+DECLINLINE(void) stfq_be_p(void *ptr, float64 v)
+#endif
 {
     CPU_DoubleU u;
     u.d = v;
     stl_be_p(ptr, u.l.upper);
-    stl_be_p(ptr + 4, u.l.lower);
+    stl_be_p((uint8_t*)ptr + 4, u.l.lower);
 }
 
 #else
@@ -740,7 +909,6 @@ static inline void stfq_be_p(void *ptr, float64 v)
 /* All direct uses of g2h and h2g need to go away for usermode softmmu.  */
 #define g2h(x) ((void *)((unsigned long)(x) + GUEST_BASE))
 #define h2g(x) ((target_ulong)(x - GUEST_BASE))
-
 #define saddr(x) g2h(x)
 #define laddr(x) g2h(x)
 
@@ -830,10 +998,12 @@ extern unsigned long qemu_host_page_mask;
 /* original state of the write flag (used when tracking self-modifying
    code */
 #define PAGE_WRITE_ORG 0x0010
+#define PAGE_RESERVED  0x0020
 
 void page_dump(FILE *f);
 int page_get_flags(target_ulong address);
 void page_set_flags(target_ulong start, target_ulong end, int flags);
+int page_check_range(target_ulong start, target_ulong len, int flags);
 void page_unprotect_range(target_ulong data, target_ulong data_size);
 
 #define SINGLE_CPU_DEFINES
@@ -907,7 +1077,8 @@ void cpu_dump_state(CPUState *env, FILE *f,
 DECLNORETURN(void) cpu_abort(CPUState *env, const char *fmt, ...);
 extern CPUState *first_cpu;
 extern CPUState *cpu_single_env;
-extern int code_copy_enabled;
+extern int64_t qemu_icount;
+extern int use_icount;
 
 #define CPU_INTERRUPT_EXIT   0x01 /* wants exit from main loop */
 #define CPU_INTERRUPT_HARD   0x02 /* hardware interrupt pending */
@@ -916,35 +1087,47 @@ extern int code_copy_enabled;
 #define CPU_INTERRUPT_FIQ    0x10 /* Fast interrupt pending.  */
 #define CPU_INTERRUPT_HALT   0x20 /* CPU halt wanted */
 #define CPU_INTERRUPT_SMI    0x40 /* (x86 only) SMI interrupt pending */
+#define CPU_INTERRUPT_DEBUG  0x80 /* Debug event occured.  */
+#define CPU_INTERRUPT_VIRQ   0x100 /* virtual interrupt pending.  */
+#define CPU_INTERRUPT_NMI    0x200 /* NMI pending. */
 
 #ifdef VBOX
 /** Executes a single instruction. cpu_exec() will normally return EXCP_SINGLE_INSTR. */
-#define CPU_INTERRUPT_SINGLE_INSTR              0x0200
+#define CPU_INTERRUPT_SINGLE_INSTR              0x0400
 /** Executing a CPU_INTERRUPT_SINGLE_INSTR request, quit the cpu_loop. (for exceptions and suchlike) */
-#define CPU_INTERRUPT_SINGLE_INSTR_IN_FLIGHT    0x0400
+#define CPU_INTERRUPT_SINGLE_INSTR_IN_FLIGHT    0x0800
 /** VM execution was interrupted by VMR3Reset, VMR3Suspend or VMR3PowerOff. */
-#define CPU_INTERRUPT_RC                        0x0800
+#define CPU_INTERRUPT_RC                        0x1000
 /** Exit current TB to process an external interrupt request (also in op.c!!) */
-#define CPU_INTERRUPT_EXTERNAL_EXIT             0x1000
+#define CPU_INTERRUPT_EXTERNAL_EXIT             0x2000
 /** Exit current TB to process an external interrupt request (also in op.c!!) */
-#define CPU_INTERRUPT_EXTERNAL_HARD             0x2000
+#define CPU_INTERRUPT_EXTERNAL_HARD             0x4000
 /** Exit current TB to process an external interrupt request (also in op.c!!) */
-#define CPU_INTERRUPT_EXTERNAL_TIMER            0x4000
+#define CPU_INTERRUPT_EXTERNAL_TIMER            0x8000
 /** Exit current TB to process an external interrupt request (also in op.c!!) */
-#define CPU_INTERRUPT_EXTERNAL_DMA              0x8000
+#define CPU_INTERRUPT_EXTERNAL_DMA              0x10000
 #endif /* VBOX */
 void cpu_interrupt(CPUState *s, int mask);
 void cpu_reset_interrupt(CPUState *env, int mask);
 
+int cpu_watchpoint_insert(CPUState *env, target_ulong addr, int type);
+int cpu_watchpoint_remove(CPUState *env, target_ulong addr);
+void cpu_watchpoint_remove_all(CPUState *env);
 int cpu_breakpoint_insert(CPUState *env, target_ulong pc);
 int cpu_breakpoint_remove(CPUState *env, target_ulong pc);
+void cpu_breakpoint_remove_all(CPUState *env);
+
+#define SSTEP_ENABLE  0x1  /* Enable simulated HW single stepping */
+#define SSTEP_NOIRQ   0x2  /* Do not use IRQ while single stepping */
+#define SSTEP_NOTIMER 0x4  /* Do not Timers while single stepping */
+
 void cpu_single_step(CPUState *env, int enabled);
 void cpu_reset(CPUState *s);
 
 /* Return the physical page corresponding to a virtual one. Use it
    only for debugging because no protection checks are done. Return -1
    if no page found. */
-target_ulong cpu_get_phys_page_debug(CPUState *env, target_ulong addr);
+target_phys_addr_t cpu_get_phys_page_debug(CPUState *env, target_ulong addr);
 
 #define CPU_LOG_TB_OUT_ASM (1 << 0)
 #define CPU_LOG_TB_IN_ASM  (1 << 1)
@@ -982,6 +1165,13 @@ int cpu_inw(CPUState *env, int addr);
 int cpu_inl(CPUState *env, int addr);
 #endif
 
+/* address in the RAM (different from a physical address) */
+#ifdef USE_KQEMU
+typedef uint32_t ram_addr_t;
+#else
+typedef unsigned long ram_addr_t;
+#endif
+
 /* memory API */
 
 #ifndef VBOX
@@ -991,7 +1181,7 @@ extern int phys_ram_size;
 #else /* VBOX */
 extern RTGCPHYS phys_ram_size;
 /** This is required for bounds checking the phys_ram_dirty accesses. */
-extern uint32_t phys_ram_dirty_size;
+extern RTGCPHYS phys_ram_dirty_size;
 #endif /* VBOX */
 #if !defined(VBOX)
 extern uint8_t *phys_ram_base;
@@ -999,29 +1189,43 @@ extern uint8_t *phys_ram_base;
 extern uint8_t *phys_ram_dirty;
 
 /* physical memory access */
-#define TLB_INVALID_MASK   (1 << 3)
-#define IO_MEM_SHIFT       4
+
+/* MMIO pages are identified by a combination of an IO device index and
+   3 flags.  The ROMD code stores the page ram offset in iotlb entry,
+   so only a limited number of ids are avaiable.  */
+
+#define IO_MEM_SHIFT       3
 #define IO_MEM_NB_ENTRIES  (1 << (TARGET_PAGE_BITS  - IO_MEM_SHIFT))
 
 #define IO_MEM_RAM         (0 << IO_MEM_SHIFT) /* hardcoded offset */
 #define IO_MEM_ROM         (1 << IO_MEM_SHIFT) /* hardcoded offset */
 #define IO_MEM_UNASSIGNED  (2 << IO_MEM_SHIFT)
-#define IO_MEM_NOTDIRTY    (4 << IO_MEM_SHIFT) /* used internally, never use directly */
-#if defined(VBOX) && !defined(VBOX_WITH_NEW_PHYS_CODE)
-#define IO_MEM_RAM_MISSING (5 << IO_MEM_SHIFT) /* used internally, never use directly */
-#endif
-/* acts like a ROM when read and like a device when written. As an
-   exception, the write memory callback gets the ram offset instead of
-   the physical address */
+#define IO_MEM_NOTDIRTY    (3 << IO_MEM_SHIFT)
+
+/* Acts like a ROM when read and like a device when written.  */
 #define IO_MEM_ROMD        (1)
+#define IO_MEM_SUBPAGE     (2)
+#define IO_MEM_SUBWIDTH    (4)
+
+/* Flags stored in the low bits of the TLB virtual address.  These are
+   defined so that fast path ram access is all zeros.  */
+/* Zero if TLB entry is valid.  */
+#define TLB_INVALID_MASK   (1 << 3)
+/* Set if TLB entry references a clean RAM page.  The iotlb entry will
+   contain the page physical address.  */
+#define TLB_NOTDIRTY    (1 << 4)
+/* Set if TLB entry is an IO callback.  */
+#define TLB_MMIO        (1 << 5)
 
 typedef void CPUWriteMemoryFunc(void *opaque, target_phys_addr_t addr, uint32_t value);
 typedef uint32_t CPUReadMemoryFunc(void *opaque, target_phys_addr_t addr);
 
 void cpu_register_physical_memory(target_phys_addr_t start_addr,
-                                  unsigned long size,
-                                  unsigned long phys_offset);
+                                  ram_addr_t size,
+                                  ram_addr_t phys_offset);
 uint32_t cpu_get_physical_page_desc(target_phys_addr_t addr);
+ram_addr_t qemu_ram_alloc(ram_addr_t);
+void qemu_ram_free(ram_addr_t addr);
 int cpu_register_io_memory(int io_index,
                            CPUReadMemoryFunc **mem_read,
                            CPUWriteMemoryFunc **mem_write,
@@ -1031,13 +1235,23 @@ CPUReadMemoryFunc **cpu_get_io_memory_read(int io_index);
 
 void cpu_physical_memory_rw(target_phys_addr_t addr, uint8_t *buf,
                             int len, int is_write);
+#ifndef VBOX
 static inline void cpu_physical_memory_read(target_phys_addr_t addr,
                                             uint8_t *buf, int len)
+#else
+DECLINLINE(void) cpu_physical_memory_read(target_phys_addr_t addr,
+                                          uint8_t *buf, int len)
+#endif
 {
     cpu_physical_memory_rw(addr, buf, len, 0);
 }
+#ifndef VBOX
 static inline void cpu_physical_memory_write(target_phys_addr_t addr,
                                              const uint8_t *buf, int len)
+#else
+DECLINLINE(void) cpu_physical_memory_write(target_phys_addr_t addr,
+                                           const uint8_t *buf, int len)
+#endif
 {
     cpu_physical_memory_rw(addr, (uint8_t *)buf, len, 1);
 }
@@ -1046,6 +1260,7 @@ uint32_t lduw_phys(target_phys_addr_t addr);
 uint32_t ldl_phys(target_phys_addr_t addr);
 uint64_t ldq_phys(target_phys_addr_t addr);
 void stl_phys_notdirty(target_phys_addr_t addr, uint32_t val);
+void stq_phys_notdirty(target_phys_addr_t addr, uint64_t val);
 void stb_phys(target_phys_addr_t addr, uint32_t val);
 void stw_phys(target_phys_addr_t addr, uint32_t val);
 void stl_phys(target_phys_addr_t addr, uint32_t val);
@@ -1058,59 +1273,88 @@ int cpu_memory_rw_debug(CPUState *env, target_ulong addr,
 
 #define VGA_DIRTY_FLAG  0x01
 #define CODE_DIRTY_FLAG 0x02
+#define KQEMU_DIRTY_FLAG     0x04
+#define MIGRATION_DIRTY_FLAG 0x08
 
 /* read dirty bit (return 0 or 1) */
+#ifndef VBOX
 static inline int cpu_physical_memory_is_dirty(ram_addr_t addr)
 {
-#ifdef VBOX
+    return phys_ram_dirty[addr >> TARGET_PAGE_BITS] == 0xff;
+}
+#else
+DECLINLINE(int) cpu_physical_memory_is_dirty(ram_addr_t addr)
+{
     if (RT_UNLIKELY((addr >> TARGET_PAGE_BITS) >= phys_ram_dirty_size))
     {
         Log(("cpu_physical_memory_is_dirty: %RGp\n", (RTGCPHYS)addr));
         /*AssertMsgFailed(("cpu_physical_memory_is_dirty: %RGp\n", (RTGCPHYS)addr));*/
         return 0;
     }
-#endif
     return phys_ram_dirty[addr >> TARGET_PAGE_BITS] == 0xff;
 }
+#endif
 
+#ifndef VBOX
 static inline int cpu_physical_memory_get_dirty(ram_addr_t addr,
                                                 int dirty_flags)
 {
-#ifdef VBOX
+    return phys_ram_dirty[addr >> TARGET_PAGE_BITS] & dirty_flags;
+}
+#else
+DECLINLINE(int) cpu_physical_memory_get_dirty(ram_addr_t addr,
+                                              int dirty_flags)
+{
     if (RT_UNLIKELY((addr >> TARGET_PAGE_BITS) >= phys_ram_dirty_size))
     {
         Log(("cpu_physical_memory_is_dirty: %RGp\n", (RTGCPHYS)addr));
         /*AssertMsgFailed(("cpu_physical_memory_is_dirty: %RGp\n", (RTGCPHYS)addr));*/
         return 0xff & dirty_flags; /** @todo I don't think this is the right thing to return, fix! */
     }
-#endif
     return phys_ram_dirty[addr >> TARGET_PAGE_BITS] & dirty_flags;
 }
+#endif
 
+#ifndef VBOX
 static inline void cpu_physical_memory_set_dirty(ram_addr_t addr)
 {
-#ifdef VBOX
+    phys_ram_dirty[addr >> TARGET_PAGE_BITS] = 0xff;
+}
+#else
+DECLINLINE(void) cpu_physical_memory_set_dirty(ram_addr_t addr)
+{
     if (RT_UNLIKELY((addr >> TARGET_PAGE_BITS) >= phys_ram_dirty_size))
     {
         Log(("cpu_physical_memory_is_dirty: %RGp\n", (RTGCPHYS)addr));
         /*AssertMsgFailed(("cpu_physical_memory_is_dirty: %RGp\n", (RTGCPHYS)addr));*/
         return;
     }
-#endif
     phys_ram_dirty[addr >> TARGET_PAGE_BITS] = 0xff;
 }
+#endif
 
 void cpu_physical_memory_reset_dirty(ram_addr_t start, ram_addr_t end,
                                      int dirty_flags);
 void cpu_tlb_update_dirty(CPUState *env);
 
+int cpu_physical_memory_set_dirty_tracking(int enable);
+
+int cpu_physical_memory_get_dirty_tracking(void);
+
 void dump_exec_info(FILE *f,
                     int (*cpu_fprintf)(FILE *f, const char *fmt, ...));
 
 /*******************************************/
 /* host CPU ticks (if available) */
 
-#if defined(__powerpc__)
+#ifdef VBOX
+
+DECLINLINE(int64_t) cpu_get_real_ticks(void)
+{
+    return  ASMReadTSC();
+}
+
+#elif defined(__powerpc__)
 
 static inline uint32_t get_tbl(void)
 {
